// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.4.1.
//
// Warning: This code was generated with the option '-l dolfin'
// and contains DOLFIN-specific wrappers that depend on DOLFIN.

#ifndef __POISSON3D_5_H
#define __POISSON3D_5_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5BilinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[3][3] ={{2*d00 / detJ, 2*d10 / detJ, 2*d20 / detJ}, {2*d01 / detJ, 2*d11 / detJ, 2*d21 / detJ}, {2*d02 / detJ, 2*d12 / detJ, 2*d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.61248608016091, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 0, 4.18330013267038, -0.591607978309963, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05987111581756e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 0, -1.67332005306816, -1.18321595661993, 7.74596669241484, 0, 0.346410161513773, 0, 0.282842712474621, 0.199999999999999, 0, 1.01607205572352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.44948974278317, 0, 0, 0, 7.09929573971954, 0, -0.414039335605422, 0, 0, 0, 0, 0, 0, 0, 1.31603755427645e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0, 4.38178046004132, -0.774596669241483, 0, 0, 4.76235235991626, 0, -0.740656079818042, 0.130930734141596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.12132034355964, 0, 0, 0, 0, 0, 7.17137165600635, 0, -1.20615246152436e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.35849019742426e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 0, 1.58113883008419, 2.45967477524977, 0, 0, 0, 0, 5.34522483824849, -1.20948631362952, 0, 0, 0, 0, 0, -1.32866943359025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, 0, 0, 3.83405790253617, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 1.14970665364125e-14, 0, 0, 1.00926211832331e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.80288457473997, 0, 0, 0, -2.10229466468021, 0, -1.71651640581399, 0, 0, 9.81070843517429, 0, 0.305838870920591, 0, 0, 0.264864231681987, 0, 0.216260739578393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 0, 1.77281052085584, -1.25356634105602, 2.39356776939084, 0, -2.78313372537744, 0, -0.524404424085074, 0.370809924354783, 0, 9.40174755792015, -1.14927687399652e-14, 0.840917865872085, -0.301561193523231, 0, 0.350641890608828, 0, 0.0660687472643403, -0.046717659215121, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.3433848874678e-14, 1.29756443747035, 0, 0, 0, 5.01426536422406, 0, -0.383824736789373, 0, 0, 0, 0, 8.20651806648289, 0, 0, -0.631738055305786, 0, 0.0483573714561588, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0, 4.06201920231798, -0.820651806648292, 0, 0, 5.88642506110457, 0, -1.20156148406979, 0.242752078113805, 0, 0, 0, 5.13809303146605, 0, 0, -0.74161984870956, 0, 0.151382517704866, -0.0305838870920676, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 0, -0.792824967172097, 2.24244764232554, 2.14087209644419, 0, 0.095742710775635, -1.39713895489839e-14, -1.87616630392937, -2.15580611373102, 0, 0, 0, 0, 9.44035859331469, 1.15714811919607e-14, 0.422185670932519, 1.95127251818837e-14, 0.590936840285278, 0.522319308773324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.16057691494799, 0, 0, 0, 1.96214168703485, 0, 2.74642624930237, 0, 0, 0, 0, 0, 0, 0, 8.6522315682783, 0, -0.865042958313572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04119910644746e-14, 0, 0, 0, 0, 0, 0, 1.5543528562087e-14, 0, 0, 0, 0},
    {1.32664991614216, 0, 2.07610309995255, 1.46802658041887, 0, 0, 1.31624465810881, 0, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 0, 5.80409338312195, 0, -1.54743588659734, 0.34193825277012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.00508909139073, 0, 0, 0, 0, 0, 4.45964044034302, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0, 0.749149177264393, 2.75458800949268, 0, 0, 0, 0, 3.3240197266047, 2.06838446274243, 0, 0, 0, 0, 0, 0, 0, 0, 6.28180682119209, -1.77676328054752, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964681, 0, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.80288457473997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 0, -2.20543546226846, -1.55947837083931, 8.33749895885385, 0, 1.06532653910462, 0, 0.869835476750494, 0.615066564126913, 0, -2.18326971917505, 0, -0.278967971730994, -1.89076704011891, 0, -0.241593350361257, 0, -0.197260144544842, -0.139483985865489, 11.8490505948789, 0, 0.245326690731332, 0, 0.0313466946954478, 0, 0.219426862868129, 0, 0.0280373360835799, 0.190029237516524, 0, 0.0242810453028227, 0, 0.0198253904711049, 0.0140186680417927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.45525332069938, 0, 0, 0, 5.28531208250978, 0, -1.36277028773849, 0, 0, 2.33666428910958, -1.04179543359539e-14, -3.93352933905728, 0, 0, -1.19859611757307, 0, 0.30904725218263, 0, 0, 0, 11.55277744383, 0, 0.810328731479531, 0, -0.234843597212094, 0, 0.395334573318417, 0, 0, 0.120463442338382, 0, -0.0310604175145514, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331886, 0, 2.99085908983323, -1.2440333788203, 0.95014618758261, 0, -0.169967317119764, 0, -1.1796107330308, 0.490653381462657, 0, 4.9761335152812, 0, -3.2268221135323, -0.21547290184284, 0, 0.0385449644663875, 0, 0.267510569439613, -0.11126972805284, 0, 0, 10.7636648722209, 0, 1.37533064807499, 0, -0.500120235663638, 0, 0.324307824721468, 0.0216558414515664, 0, -0.00387391468765637, 0, -0.0268858238268342, 0.0111830284386732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.772618130456579, 0, 0, 0, 3.3588901228148, 0, -0.304724700110023, 0, 0, 0, 0, 7.32969916135162, 0, 0, -0.761724681099362, 0, 0.0691050664139862, 0, 0, 0, 0, 0, 9.05975063854702, 0, 0, 0, -0.736662483163211, 0, 0, 0.0765562109321137, 0, -0.00694532049720535, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376113, 0, 3.68560354207934, -0.814411017927954, 0, 0, 6.00856531485322, 0, -1.45362164025439, 0.321208037219811, 0, 0, 0, 6.99294104721297, 0, 0, -1.362614533662, 0, 0.329650402328134, -0.0728431359084717, 0, 0, 0, 0, 5.40219154319687, 0, 0, 0, -0.702817019232737, 0, 0, 0.136947913395212, 0, -0.0331311120155176, 0.00732101061700613, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.1542003094028, 0, 0, 0, -0.761811750275056, 0, 5.59815020469135, 0, 0, 2.13307290077016, 0, 0.0664963811608019, 0, 0, -2.36108976914458, 0, -3.33842298506509, 0, 0, 0, 0, 0, 0, 0, 11.5766222110848, 0, 0.360889439280294, 0, 0, 0.555625639776453, 0, 0.595437196138644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 0, 0.963624111659426, 1.36277028773849, 0.867360833110884, 0, -1.00852844760553, -1.31315568597201e-14, 1.71026313764871, -2.01556443707465, 0, 2.04415543160775, -1.06351258615242e-14, 0.182834820066006, 2.68821895760394, 2.34958042241799e-14, -3.12574097035512, 0, -1.01990506872275, 0.904016610326379, 0, 0, 0, 0, 0, 0, 11.0940489487798, 0, 0.992281903807273, -0.632607619454903, 0, 0.735567892896446, 0, 0.181909068193175, -0.147004726489975, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24564920565478e-14, 0, 0, 1.009146310714e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.705300630647536, 0, 0, 0, 1.81702705031798, 0, 1.25178444059442, 0, 0, 0, 0, 1.78428514239953, 0, 0, 5.631527706439, 0, -0.746494073225355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.68367982311023, 0, 0, -1.32524448062076, 0, 0.133143804689787, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29216299118995e-14, 0, 0},
    {1.1517511068998, 0, 2.20794021658195, 0.892142571199758, 0, 0, 2.13307290077015, 0, 3.91870514328394, -1.31949665727936, 0, 0, 0, 1.11713920350151, 0, 0, 6.6110513536048, 0, -2.33689608954558, 0.591817792331142, 0, 0, 0, 0, 0, 0, 0, 0, 6.06294257990893, 0, 0, -1.55575179137407, 0, 0.416806037297055, -0.0962371838080836, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493888, 0, -0.430945803685668, 3.04724700110021, 0.775791113542722, 0, 0.0346944333244404, -1.49568650774796e-14, -1.52970585407784, 0.901387818865989, 0, 0, 0, 0, 4.51561078268939, 0, 0.20194425340049, 0, -1.83730997289732, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, 0, 0.487107866980278, 0, 0.894874292847885, 0.903959543974648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.630840061880567, 0, 0, 0, 0.711024300256716, 0, 2.23926008187654, 0, 0, 0, 0, 0, 0, 0, 4.13862564412659, 0, 2.68954640486934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.98274066131697, 0, -1.30996183150502, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727542, 0, 1.12848100903608, 1.99489143482414, 0, 0, 0.47696960070847, 0, 4.00571020990215, 0.59009684435208, 0, 0, 0, 0, 0, 0, 2.77627448210728, 0, 4.8112068719424, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.69662601613678, 0, -2.34333096254029, 0.591780433634516, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.546323519313506, 0, 0, 0, 0, 0, 2.69341127303542, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.71889089790563, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059451, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199766, 0, 0.40720550896397, 2.53385243517954, 0, 0, 0, 0, 2.00755023262382, 3.29336152870136, 0, 0, 0, 0, 0, 0, 1.24513151875208e-14, 0, 5.00796559370719, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084685, 0, 0, 2.53908227033118, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.73861278752583, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867318, -0.381881307912986, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 3.62284418654736, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.3228756555323, 0, 4.83045891539648, 0.341565025531989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693486, 0, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62018517460197, 0, 0, 3.76497011940334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923788, -0.836660026534075, -0.591607978309962, 3.87298334620742, -0.599999999999998, 0.173205080756888, -0.489897948556638, 0.141421356237313, 0.0999999999999996, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386223, 1.22474487139159, 1.41421356237309, -0.500000000000003, 4.58257569495584, 3.54964786985977, -1.02469507659596, -0.207019667802713, -0.239045721866875, 0.0845154254728531, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000003, -2.84604989415154, 2.19089023002066, -0.387298334620745, 0, 5.49909083394701, 2.38117617995813, 0.481070235442362, -0.370328039909021, 0.0654653670707988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.59807621135333, 0, -1.5811388300842, -1.1180339887499, 0, 0, 6.87386354243376, 0, 0.267261241912429, 0.188982236504611, 0, 0, 0, -1.56646390951001e-14, 0, 0, 1.01970842391297e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.0062305898749, 1.06066017177982, -0.204124145231929, 1.58771324027146, 0, 0, 0, 3.58568582800318, -0.690065559342348, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.77942286340599, 1.36930639376292, 0.790569415042096, 1.22983738762489, 0, 0, 0, 4.62910049886275, 2.67261241912425, -0.604743156814767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.900000000000007, 0, 1.82574185835055, -1.42009389360937, 0, 0, 0, 0, 6.17213399848367, 0.698297248755177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067893, 0, 0, 1.91702895126809, 0, 0, 0, 0, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.10227038425243, 0, 0, 3.3203915431768, 0, 0, 0, 0, 0, 4.48998886412873, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219015, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849178, 0.152919435460295, -0.0529728463363973, -0.512907379155188, 0.132432115840996, -0.0458758306380885, 0.1081303697892, -0.0374574588632162, -0.0264864231681936, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114625, -0.262202212042537, 0.185404962177395, 5.56214886532174, 4.70087377896007, -1.21376039056904, 0.420458932936042, -0.150780596761618, -0.397100103328477, 0.175320945304415, 0.0572172135271331, 0.0330343736321715, -0.0233588296075578, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735171, 1.87287294316099, -0.529728463363976, -4.04586796856344, 2.50713268211204, 0.904683580569682, -0.191912368394685, -0.554003287767448, 0.156695792632001, 0, 6.35674156036771, 4.10325903324145, -1.42141062443803, 0.509731451534329, -0.315869027652891, -0.113979417590034, 0.0241786857280778, 0.0697978535688028, -0.0197418142283115, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709575, 3.51781181986757, 2.03100960115899, -0.410325903324144, 0, -3.39852909359329, 2.94321253055229, -1.04058276941337, -0.600780742034897, 0.121376039056905, 0, 0, 7.41619848709566, 2.56904651573303, 0, 0.428174419288832, -0.370809924354779, 0.131101106021265, 0.0756912588524378, -0.0152919435460309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.71269767715534, 0, 4.69041575982343, 0.94760708295868, 0, 0, -1.69926454679664, 0, -1.38744369255116, -0.280305955290692, 0, 0, 0, 8.8994381845148, 0, 0, 0.214087209644411, 0, 0.17480147469502, 0.0353152308909268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586045, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878174, 3.24961536185438, -0.938083151964683, -1.07790305686552, 0, 0, 0, 0, 4.72017929665735, -0.731247032282672, 0.211092835466259, -1.02353263143832, 0.295468420142642, 0.26115965438666, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577669, 0.580288457473998, 0.670059394260483, 0.947607082958683, 1.26655701279755, 0.981070843517428, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694758, 0, 0, 0, 0, 5.58499146191198, 4.32611578413915, -1.24884205625912, -0.432521479156774, -0.499432784842937, 0.220720193068322, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071078, -1.34846851907591, 1.03805154997627, 0.73401329020944, 0, 1.51986841535706, 0.658122329054406, -3.19105893754051, 2.45647831545196, -0.705653192844341, 0, 0, 0, 0, 0, 6.70198975429437, 2.90204669156098, 1.00508909139074, -0.773717943298673, 0.170969126385064, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 0, -0.749149177264396, 2.11891385345588, 0, 0, 1.89983551919633, 0, -1.77281052085583, -2.03704530421603, 0, 0, 0, 0, 0, 0, 8.37748719286796, 0, 0.558382828550407, 0.493545355707621, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22447624956168e-14},
    {0.741619848709566, 0.502544545695365, -0.096714742912331, 1.77807891440463, 0, 0, 0, 2.22982022017151, -0.429129101453498, 1.33513642962593, 0, 0, 0, 0, 0, 0, 0, 4.21396412211221, -0.81097777341897, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5744562646538, 0.648782218735174, 0.374574588632197, 1.37729400474634, 0, 0, 0, 2.87868552592027, 1.66200986330235, 1.03419223137121, 0, 0, 0, 0, 0, 0, 0, 5.44020428881872, 3.14090341059604, -0.888381640273766, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.663324958071084, 0, 0.865042958313566, -1.59036212878712, 0, 0, 0, 0, 3.83824736789369, -1.19418232635198, 0, 0, 0, 0, 0, 0, 0, 0, 7.25360571842496, 1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.46904157598234, 0, 0, 1.55707732496443, 0, 0, 0, 0, 0, 2.76353810488345, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.90144228736999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.812403840463591, 0, 0, 2.69693703815183, 0, 0, 0, 0, 0, 4.78658840631075, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.02544545695368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552312, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541881, -0.139483985865496, -0.945383520059461, 0.313838968197358, -0.120796675180629, 0.25624844449503, -0.0986300722724195, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963984, 0.0156733473477253, -0.519258730913241, 0.109713431434065, -0.0364215679542342, 0.01401866804179, 0.0950146187582634, -0.0315420030940289, 0.0121405226514124, -0.0257539376818875, 0.00991269523555186, 0.00700933402089779, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706032, -0.705300630647551, 1.61604676382125, 2.64265604125489, -1.44543616748915, -0.681385143869243, -0.196698948117362, 0.278174320132098, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444731, -0.366484958067585, -0.599298058786534, 0.327794111588103, 0.15452362609131, 0.0446071285599885, -0.0630840061880518, 6.54980915752513, 5.77638872191499, -1.22048487632307, 0.405164365739766, -0.155947837083929, -0.117421798606046, -0.453666435153256, 0.197667286659209, -0.0824223552841552, 0.0368331241581584, 0.0602317211691907, -0.0329445477765348, -0.0155302087572761, -0.00448318510329359, 0.00634018117570608, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659432, 0.914174100330065, 1.49542954491661, -0.622016689410148, 0.475073093791304, 2.87032228155655, -0.0849836585598799, -0.360555127546397, -0.589805366515403, 0.245326690731328, -5.15186050535791, 2.48806675764059, 2.8908723349783, -1.61341105676615, -0.107736450921414, -0.650927909109099, 0.019272482233193, 0.0817662172643042, 0.133755284719809, -0.0556348640264175, 0, 7.27756573058348, 5.38183243611043, -1.7866069197544, 0.687665324037496, 0.517781462682516, -0.250060117831819, -0.290543601573986, 0.162153912360733, 0.0108279207257825, 0.0654207163662268, -0.00193695734382766, -0.00821781403613334, -0.0134429119134184, 0.00559151421933731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927964, 0.386309065228293, 1.93297557093283, -0.525700051567136, 4.5772145303823, 1.6794450614074, 1.93925611658549, -0.152362350055012, -0.762375846428075, 0.207338896470047, 0, -4.7313004641042, 3.66484958067581, 0.282120252259006, -1.03801468669568, -0.38086234054968, -0.439781949681085, 0.0345525332069918, 0.172890591018093, -0.0470200420431731, 0, 0, 8.18726144690641, 4.52987531927351, -1.74354982331868, 0, 0.475513588177819, -0.368331241581605, -0.0283541521970749, 0.10432440129236, 0.0382781054660568, 0.0441997489897906, -0.00347266024860305, -0.0173761581875701, 0.00472569203284699, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880556, -3.39131543920102, 1.84280177103967, -0.407205508963983, 0, 5.01084538062905, 3.00428265742661, 1.33755284719805, -0.726810820127196, 0.160604018609904, 0, 0, -3.78504037128335, 3.49647052360648, 0, -1.13635292017211, -0.681307266831003, -0.303328474207914, 0.164825201164063, -0.0364215679542416, 0, 0, 0, 9.3568702250359, 2.70109577159844, 0, 0, 0.380410870542252, -0.351408509616368, 0, 0.11420776562532, 0.0684739566976048, 0.0304856587023835, -0.0165655560077574, 0.00366050530850137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54950975679639, 0, -2.32737334062814, -1.64570147151095, 0, 0, 6.74536878161603, 0, 0.917928424547679, 0.649073413641553, 0, 0, 0, -1.76635217326556, 0, 0, -1.52970585407782, 0, -0.208166599946601, -0.147196014438789, 0, 0, 0, 0, 10.9163485958752, 0, 0, 0, 0.177525072919717, 0, 0, 0.153741222957161, 0, 0.020921530482028, 0.0147937560766434, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412066, 1.5771001547014, -0.546323519313517, 0.772618130456561, 1.47524211088021, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292753, -1.14271762541258, 1.06653645038508, -0.12876968840943, 0.0332481905804026, -0.011517511068999, 4.57223067739883, -1.18054488457228, 0.408952744138947, -1.66921149253253, 0.578231822728847, 0.512529242570405, 0, 0, 0, 0, 0, 5.78831110554239, -0.698859394077356, 0.180444719640146, -0.062507884474852, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069323, -0.10313274764285, -0.0833438459664692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186249, 0.834522960396274, 0.481812055829709, 0.681385143869246, 0.433680416555438, 1.14215318100896, -0.504264223802767, 1.48113132435986, 0.855131568824349, -1.00778221853732, 1.20933866224478, 1.02207771580387, -0.263899331455877, 0.0914174100330005, 1.34410947880197, 3.53988526627811, -1.56287048517756, -0.88326369896241, -0.509952534361379, 0.452008305163194, 0, 0, 0, 0, 0, 6.56332786985937, 5.54702447438989, -1.4322355606878, 0.496140951903638, -0.316303809727451, -0.833026783443143, 0.367783946448223, 0.15753787423404, 0.0909545340965868, -0.0735023632449856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14091633431099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.743452142666481, 0.35265031532376, 1.01801377240995, 0.575875553449897, -1.46610739697708, 0.908513525158993, 0.327831580195603, 0.62589222029721, 1.80679520936143, -0.851731429836555, 0, 1.38210132827975, 0.892142571199764, -0.309047252182628, -4.54391937932164, 2.8157638532195, 1.01605126164418, -0.373247036612687, -1.07747138531276, 0.382016742281294, 0, 0, 0, 0, 0, 0, 7.50094613698214, 4.84183991155512, -1.67726254578565, 1.06930204231198, -0.662622240310384, -0.239103205508895, 0.066571902344891, 0.192176528696449, -0.0621208350291014, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10027752117729e-14, 0, 0},
    {0.575875553449902, 1.91213231759728, 1.10397010829098, 0.44607128559988, 0, -1.23153021346074, 1.06653645038507, 3.39369820402464, 1.95935257164197, -0.659748328639681, 0, 0, 1.6124515496597, 0.558569601750758, 0, -3.81689227863019, 3.3055256768024, -2.02381137955099, -1.16844804477279, 0.295908896165575, 0, 0, 0, 0, 0, 0, 0, 8.75110382647916, 3.03147128995446, 0, 0.898213715542065, -0.777875895687036, 0.360964616749972, 0.208403018648526, -0.0481185919040409, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.32945523855878e-14, 0, 0},
    {-1.32992762321609, 0, 2.54950975679639, -1.03015750727543, 0, 0, -0.615765106730374, 0, 4.52493093869951, 1.52362350055012, 0, 0, 0, 1.93494185959165, 0, 0, -1.9084461393151, 0, -2.69841517273464, -0.683372323427214, 0, 0, 0, 0, 0, 0, 0, 0, 10.501324591775, 0, 0, 0.449106857771034, 0, 0.481286155666633, 0.111125127955289, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.786795792469438, 0.746420027292178, -0.215472901842835, 1.5236235005501, 0.387895556771362, -0.0600925212577327, 0.0173472166622199, 2.64952825989834, -0.764852927038915, 0.45069390943299, 0, 0, 0, 0, 2.25780539134469, -0.349777707186207, 0.100972126700243, 3.18231422231115, -0.918654986448657, -1.37412670680299, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.44603151471411, -0.843695574376338, 0.243553933490139, -1.54996774159981, 0.447437146423943, 0.451979771987323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608038, 0.315420030940283, 0.364215679542343, 1.28769688409429, 0.458964212273842, 0.355512150128361, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137522, 0, 0, 0, 0, 2.67147136598154, 2.06931282206329, -0.597359157427889, 1.34477320243467, 1.5528103408493, -1.16134903279062, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.4438313884647, 4.99137033065849, -1.44088450201539, -0.654980915752514, -0.756306816047561, 0.381992627346313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.51507875363771, -0.732969916135158, 0.56424050451804, 0.997445717412072, 0, 0.550757054728608, 0.238484800354237, -2.60178510148047, 2.00285510495107, 0.295048422176044, 0, 0, 0, 0, 0, 3.20576563917784, 1.38813724105364, -3.12497053047329, 2.4056034359712, -0.899577092626427, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.73259766615765, 3.34831300806839, 1.5220381072759, -1.17166548127015, 0.295890216817259, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48690428533295, 0, -0.40720550896398, 2.87937776724947, 0, 0, 0.688446318410768, 0, -1.44543616748914, 0.851731429836557, 0, 0, 0, 0, 0, 0, 4.00720704897231, 0, -1.73609473915183, -2.59685538292347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.66574708269706, 0, 0.845576726264391, 0.854161481650102, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449896, 0.273161759656754, -0.0525700051567131, 1.63559471386625, 0, 0, 0, 1.34670563651772, -0.259173620587566, 2.12585572561676, 0, 0, 0, 0, 0, 0, 0, 3.35944544895281, -0.646525578093589, 0.914325240961068, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4.72691760029726, -0.909695716322936, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599885, 0.35265031532377, 0.203602754481986, 1.26692621758977, 0, 0, 0, 1.73858950082559, 1.0037751163119, 1.64668076435068, 0, 0, 0, 0, 0, 0, 0, 4.33702542542885, 2.50398279685359, 0.708233286251854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.10242438161534, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.515078753637711, 0, 0.470200420431694, -1.46292038553772, 0, 0, 0, 0, 2.31811933443412, -1.90142316513449, 0, 0, 0, 0, 0, 0, 0, 0, 5.78270056723845, -0.817797356933148, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.13656584215379, 1.31506763029891, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12763531151527e-14},
    {0.364215679542343, 0, 0, 1.26954113516559, 0, 0, 0, 0, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.46962034034308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880561, 0, 0, 2.19890974840547, 0, 0, 0, 0, 0, 4.33630850246743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.00955871244863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.4358503831282, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.912870929175276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.58198889747161, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867317, -0.381881307912987, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 0.724568837309471, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.661437827766144, 0, 1.93218356615859, -0.170782512765995, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693487, 3.54964786985977, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.540061724867317, 0, 3.54964786985977, 1.25499003980111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.90940653956493, 0, 0, 4.43705983732471, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923789, -0.836660026534075, -0.591607978309964, 3.87298334620742, -0.600000000000001, 0.173205080756889, -0.48989794855664, 0.141421356237313, 0.0999999999999997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386222, 1.22474487139158, 1.41421356237309, -0.500000000000002, 0.654653670707977, 3.54964786985977, -1.02469507659596, -0.20701966780271, -0.239045721866875, 0.0845154254728546, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000001, 0.316227766016839, 2.19089023002066, -0.38729833462074, 0, 1.57116880969914, 2.38117617995813, -0.0534522483824853, -0.37032803990902, 0.0654653670707977, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.519615242270665, 0, 1.58113883008419, -0.223606797749979, 0, 0, 2.94594151818589, 0, -0.267261241912424, 0.0377964473009217, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.00623058987491, 1.06066017177982, -0.204124145231932, 1.58771324027146, 4.53557367611073, 0, 0, 3.58568582800318, -0.690065559342349, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.779422863405996, 0.273861278752581, 0.790569415042093, 1.22983738762489, 0, 4.53557367611073, 0, 0.925820099772549, 2.67261241912425, -0.604743156814761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.450000000000004, 0, 0.730296743340221, 0.710046946804697, 0, 0, 4.53557367611072, 0, 2.46885359939347, -0.349148624377589, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067889, -3.1304951684997, 0, 1.91702895126808, 0, 0, 0, 5.29150262212918, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.367423461417479, 0, -3.1304951684997, 1.10679718105894, 0, 0, 0, 0, 5.29150262212918, 1.49666295470958, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.85788383248864, 0, 0, -2.34787137637477, 0, 0, 0, 0, 0, 6.34980314655501, 0, 0, 0, 0, 0, 0, 1.11852851939587e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219016, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849176, 0.152919435460295, -0.0529728463363977, -0.512907379155187, 0.132432115840995, -0.0458758306380897, 0.1081303697892, -0.0374574588632173, -0.0264864231681947, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114629, -0.262202212042535, 0.185404962177393, 0.618016540591305, 4.70087377896007, -1.21376039056904, 0.42045893293604, -0.150780596761618, -0.397100103328475, 0.175320945304413, 0.057217213527133, 0.0330343736321713, -0.0233588296075595, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735172, 1.87287294316099, -0.529728463363977, 0.20229339842817, 2.50713268211204, 0.904683580569681, -0.191912368394685, -0.554003287767448, 0.156695792632003, 0, 1.41260923563727, 4.10325903324145, -1.42141062443803, -0.0254865725767146, -0.315869027652895, -0.113979417590039, 0.0241786857280816, 0.0697978535688034, -0.0197418142283076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709574, 0.167514848565124, 2.03100960115899, -0.410325903324145, 0, 0.849632273398319, 2.94321253055228, -0.0495515604482551, -0.600780742034895, 0.121376039056903, 0, 0, 2.47206616236523, 2.56904651573303, 0, -0.10704360482221, -0.370809924354777, 0.00624290981053448, 0.0756912588524339, -0.015291943546034, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.428174419288845, 0, 1.34011878852098, -0.236901770739674, 0, 0, 2.54889682019496, 0, -0.396412483586046, 0.0700764888226733, 0, 0, 0, 3.95530585978436, 0, 0, -0.321130814466627, 0, 0.0499432784842904, -0.00882880772273737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586047, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878164, 3.24961536185438, -0.938083151964685, -1.07790305686551, 5.52770798392566, 0, 0, 0, 4.72017929665734, -0.731247032282673, 0.211092835466259, -1.02353263143831, 0.295468420142642, 0.261159654386662, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577672, 0.580288457473996, 0.670059394260487, 0.947607082958685, 0.180936716113935, 0.981070843517426, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694756, 0, 5.52770798392567, 0, 0, 0.797855923130281, 4.32611578413915, -1.24884205625911, -0.432521479156781, -0.499432784842937, 0.220720193068326, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071081, 0.149829835452877, 1.03805154997628, 0.734013290209435, 0, 0.434248118673443, 0.658122329054405, 0.354562104171167, 2.45647831545196, -0.705653192844349, 0, 0, 5.52770798392567, 0, 0, 1.91485421551268, 2.90204669156097, -0.111676565710086, -0.773717943298669, 0.170969126385058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.382970843102543, 0, 0.749149177264393, 0.423782770691177, 0, 0, 0.814215222512714, 0, 1.77281052085584, -0.407409060843206, 0, 0, 0, 5.52770798392568, 0, 0, 3.59035165408627, 0, -0.558382828550415, 0.0987090711415228, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709561, 0.502544545695366, -0.096714742912333, 1.77807891440463, -4.23078640106406, 0, 0, 2.2298202201715, -0.429129101453499, 1.33513642962593, 0, 0, 0, 0, 6.21867148191638, 0, 0, 4.21396412211221, -0.810977773418968, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.574456264653794, 0.129756443747034, 0.374574588632199, 1.37729400474634, 0, -4.23078640106406, 0, 0.575737105184058, 1.66200986330234, 1.03419223137121, 0, 0, 0, 0, 0, 6.21867148191637, 0, 1.08804085776374, 3.14090341059604, -0.88838164027376, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.331662479035532, 0, 0.346017183325429, 0.795181064393553, 0, 0, -4.23078640106406, 0, 1.53529894715748, 0.597091163175992, 0, 0, 0, 0, 0, 0, 6.21867148191637, 0, 2.90144228736999, -0.512907379155182, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.469041575982343, 3.81404493622062, 0, 1.55707732496444, 0, 0, 0, -3.76069902316805, 0, 2.76353810488346, 0, 0, 0, 0, 0, 0, 1.02342965294379e-14, 7.10705312219014, 0, 2.90144228737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.270801280154526, 0, 3.81404493622062, 0.89897901271727, 0, 0, 0, 0, -3.76069902316805, 1.59552946877025, 0, 0, 0, 0, 0, 0, 0, 0, 7.10705312219014, 1.67514848565122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.68079590171775, 0, 0, 4.4497190922574, 0, 0, 0, 0, 0, -2.63248931621764, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.2915619758885, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552311, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541882, -0.139483985865495, -0.945383520059461, 0.313838968197358, -0.120796675180628, 0.256248444495029, -0.0986300722724191, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963982, 0.0156733473477249, -0.519258730913241, 0.109713431434065, -0.0364215679542341, 0.0140186680417898, 0.0950146187582634, -0.0315420030940288, 0.0121405226514124, -0.0257539376818875, 0.00991269523555182, 0.00700933402089769, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706025, -0.705300630647557, 1.61604676382126, 2.64265604125489, -1.44543616748915, -0.681385143869247, -0.196698948117359, 0.278174320132096, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444727, -0.366484958067587, -0.599298058786529, 0.327794111588101, 0.154523626091317, 0.0446071285599885, -0.0630840061880538, 0.595437196138649, 5.77638872191499, -1.22048487632307, 0.405164365739765, -0.155947837083928, -0.117421798606047, -0.453666435153255, 0.197667286659209, -0.0824223552841549, 0.0368331241581608, 0.0602317211691907, -0.0329445477765342, -0.0155302087572759, -0.00448318510329279, 0.00634018117570581, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659437, 0.914174100330057, 1.49542954491662, -0.622016689410149, 0.475073093791305, 2.87032228155655, -0.084983658559881, -0.3605551275464, -0.589805366515401, 0.24532669073133, 0.147196014438797, 2.4880667576406, 2.89087233497828, -1.61341105676615, -0.107736450921419, -0.650927909109092, 0.0192724822331939, 0.0817662172643072, 0.133755284719809, -0.0556348640264189, 0, 1.32319376919699, 5.38183243611043, -1.7866069197544, 0.687665324037495, -0.0147937560766431, -0.25006011783182, -0.290543601573986, 0.162153912360733, 0.0108279207257832, 0.0654207163662248, -0.00193695734382736, -0.00821781403613383, -0.0134429119134179, 0.00559151421933732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927959, 0.386309065228289, 1.93297557093283, -0.525700051567137, 0.0803020093049556, 1.6794450614074, 1.9392561165855, -0.152362350055011, -0.762375846428077, 0.207338896470047, 0, 0.567756055692504, 3.66484958067581, 0.282120252259013, -0.0182107839771216, -0.38086234054968, -0.439781949681088, 0.0345525332069925, 0.172890591018093, -0.0470200420431761, 0, 0, 2.23288948551993, 4.52987531927351, -1.74354982331867, 0, -0.0570616305813383, -0.368331241581605, -0.0283541521970753, 0.0018302526542526, 0.0382781054660563, 0.0441997489897914, -0.00347266024860275, -0.0173761581875703, 0.00472569203284603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880563, 0.09974457174121, 1.84280177103967, -0.407205508963979, 0, 0.513932859551699, 3.00428265742661, -0.03933978962347, -0.726810820127196, 0.160604018609906, 0, 0, 1.51401614851335, 3.49647052360648, 0, -0.116549017453554, -0.681307266831002, 0.00892142571199782, 0.164825201164067, -0.036421567954237, 0, 0, 0, 3.40249826364942, 2.70109577159844, 0, 0, -0.152164348216903, -0.351408509616369, 0, 0.0117136169872135, 0.0684739566976059, -0.000896637020657868, -0.0165655560077585, 0.00366050530850301, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, 0, 1.16368667031409, -0.23510021021585, 0, 0, 2.24845626053868, 0, -0.458964212273845, 0.0927247733773657, 0, 0, 0, 3.53270434653115, 0, 0, -0.509901951359277, 0, 0.104083299973309, -0.0210280020626858, 0, 0, 0, 0, 4.96197663448874, 0, 0, 0, -0.355050145839439, 0, 0, 0.0512470743190546, 0, -0.010460765241014, 0.00211339372523455, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412067, 1.5771001547014, -0.546323519313514, 0.772618130456565, 1.4752421108802, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292756, -1.14271762541258, 1.06653645038508, -0.128769688409428, 0.0332481905804048, -0.0115175110689984, 4.57223067739883, -1.18054488457228, 0.408952744138948, -1.66921149253254, 0.578231822728845, 0.512529242570406, 6.52268767805531, 0, 0, 0, 0, 5.7883111055424, -0.698859394077357, 0.180444719640146, -0.0625078844748521, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069321, -0.103132747642851, -0.0833438459664682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186242, 0.834522960396282, 0.481812055829709, 0.681385143869242, 0.433680416555441, 1.14215318100896, -0.504264223802765, 1.48113132435985, 0.855131568824352, -1.00778221853732, 0.134370962471638, 1.02207771580387, -0.263899331455871, 0.0914174100330022, 1.34410947880196, 3.5398852662781, -1.56287048517756, -0.883263698962406, -0.509952534361376, 0.452008305163191, 0, 6.52268767805531, 0, 0, 0, 0.729258652206596, 5.5470244743899, -1.4322355606878, 0.496140951903638, -0.316303809727452, -0.833026783443137, 0.367783946448223, 0.157537874234044, 0.0909545340965877, -0.0735023632449854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01901886515007e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.74345214266648, 0.352650315323769, 1.01801377240995, 0.5758755534499, 0.0733053698488504, 0.908513525158993, 0.327831580195604, 0.62589222029721, 1.80679520936143, -0.851731429836554, 0, 0.307133628506613, 0.892142571199772, -0.309047252182627, 0.227195968966083, 2.8157638532195, 1.01605126164419, -0.37324703661268, -1.07747138531275, 0.382016742281307, 0, 0, 6.52268767805531, 0, 0, 0, 1.66687691932936, 4.84183991155511, -1.67726254578565, -0.0534651021155991, -0.662622240310383, -0.239103205508893, 0.0665719023448925, 0.192176528696449, -0.0621208350290961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449893, 0.0910539198855843, 1.10397010829098, 0.446071285599878, 0, 0.307882553365187, 1.06653645038508, 0.161604676382129, 1.95935257164197, -0.65974832863968, 0, 0, 0.537483849886575, 0.558569601750753, 0, 0.954223069657542, 3.3055256768024, -0.0963719704548154, -1.16844804477279, 0.295908896165572, 0, 0, 0, 6.52268767805531, 0, 0, 0, 2.91703460882639, 3.03147128995447, 0, -0.224553428885516, -0.777875895687036, 0.0171887912738079, 0.208403018648526, -0.0481185919040412, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.332481905804022, 0, 0.728431359084682, 0.257539376818849, 0, 0, 0.923647660095564, 0, 1.292837411057, -0.380905875137523, 0, 0, 0, 0.859974159818509, 0, 0, 2.86266920897263, 0, -0.770975763638467, 0.170843080856795, 0, 0, 0, 0, 6.52268767805531, 0, 0, 0, 4.66725537412222, 0, 0, -0.673660286656546, 0, 0.137510330190468, -0.0277812819888241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.78679579246944, 0.746420027292177, -0.215472901842835, 1.52362350055011, 0.387895556771359, -0.0600925212577337, 0.0173472166622209, 2.64952825989835, -0.764852927038921, 0.450693909432998, -5.28814187068052, 0, 0, 0, 2.2578053913447, -0.349777707186206, 0.100972126700245, 3.18231422231115, -0.91865498644866, -1.37412670680298, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0, 5.44603151471411, -0.843695574376342, 0.243553933490139, -1.54996774159981, 0.447437146423942, 0.451979771987325, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608043, 0.315420030940282, 0.364215679542341, 1.28769688409429, 0.0655663160391197, 0.355512150128356, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137518, 0, -5.28814187068053, 0, 0, 0.381638766568791, 2.0693128220633, -0.597359157427881, 1.34477320243467, 1.5528103408493, -1.16134903279063, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0.920547341209241, 4.99137033065848, -1.4408845020154, -0.654980915752512, -0.756306816047561, 0.381992627346309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637708, 0.0814411017927942, 0.564240504518039, 0.997445717412071, 0, 0.157359158493888, 0.238484800354236, 0.289087233497833, 2.00285510495107, 0.295048422176043, 0, 0, -5.28814187068052, 0, 0, 0.915933039765097, 1.38813724105364, 0.347218947830363, 2.4056034359712, -0.89957709262643, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 2.20931361890218, 3.34831300806839, -0.16911534525288, -1.17166548127015, 0.295890216817257, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.297380857066588, 0, 0.407205508963972, 0.575875553449888, 0, 0, 0.295048422176039, 0, 1.44543616748915, 0.17034628596731, 0, 0, 0, -5.28814187068053, 0, 0, 1.71737444955956, 0, 1.73609473915183, -0.519371076584704, 0, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 4.14246303544159, 0, -0.845576726264386, 0.170832296330017, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449895, 0.27316175965675, -0.0525700051567132, 1.63559471386626, 4.82647810240717, 0, 0, 1.34670563651771, -0.259173620587563, 2.12585572561677, 0, 0, 0, 0, -4.95763300376299, 0, 0, 3.35944544895281, -0.646525578093588, 0.914325240961076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 0, 4.72691760029725, -0.909695716322934, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599889, 0.0705300630647548, 0.203602754481986, 1.26692621758977, 0, 4.82647810240717, 0, 0.34771790016512, 1.00377511631191, 1.64668076435067, 0, 0, 0, 0, 0, -4.95763300376299, 1.29915143264338e-14, 0.867405085085771, 2.50398279685359, 0.708233286251852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873427, 0, 1.22048487632307, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.257539376818862, 0, 0.188080168172671, 0.731460192768861, 0, 0, 4.82647810240716, 0, 0.927247733773645, 0.950711582567244, 0, 0, 0, -1.22220110126485e-14, 0, 0, -4.95763300376297, 0, 2.31308022689538, 0.408898678466573, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 3.25462633686152, -0.657533815149454, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04894165366026e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, -3.97354131880428, 0, 1.26954113516558, 0, 0, 0, 5.11038857901935, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, -4.24939971751113, 0, 3.46962034034307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.210280020626859, 0, -3.97354131880429, 0.732969916135155, 0, 0, 0, 0, 5.11038857901935, 1.44543616748915, 0, 0, 0, 0, 0, 0, 0, 0, -4.24939971751114, 2.00318623748287, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 1.8119501277094, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.2711894277325, 0, 0, -3.4552533206994, 0, 0, 0, 0, 0, 6.13246629482322, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.2499377798012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[56][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0.8, 0.2}, {0, 0.6, 0.4}, {0, 0.4, 0.6}, {0, 0.2, 0.8}, {0.8, 0, 0.2}, {0.6, 0, 0.4}, {0.4, 0, 0.6}, {0.2, 0, 0.8}, {0.8, 0.2, 0}, {0.6, 0.4, 0}, {0.4, 0.6, 0}, {0.2, 0.8, 0}, {0, 0, 0.2}, {0, 0, 0.4}, {0, 0, 0.6}, {0, 0, 0.8}, {0, 0.2, 0}, {0, 0.4, 0}, {0, 0.6, 0}, {0, 0.8, 0}, {0.2, 0, 0}, {0.4, 0, 0}, {0.6, 0, 0}, {0.8, 0, 0}, {0.6, 0.2, 0.2}, {0.4, 0.4, 0.2}, {0.2, 0.6, 0.2}, {0.4, 0.2, 0.4}, {0.2, 0.4, 0.4}, {0.2, 0.2, 0.6}, {0, 0.2, 0.2}, {0, 0.4, 0.2}, {0, 0.6, 0.2}, {0, 0.2, 0.4}, {0, 0.4, 0.4}, {0, 0.2, 0.6}, {0.2, 0, 0.2}, {0.4, 0, 0.2}, {0.6, 0, 0.2}, {0.2, 0, 0.4}, {0.4, 0, 0.4}, {0.2, 0, 0.6}, {0.2, 0.2, 0}, {0.4, 0.2, 0}, {0.6, 0.2, 0}, {0.2, 0.4, 0}, {0.4, 0.4, 0}, {0.2, 0.6, 0}, {0.2, 0.2, 0.2}, {0.4, 0.2, 0.2}, {0.2, 0.4, 0.2}, {0.2, 0.2, 0.4}};
    
    // Components for each dof
    static unsigned int components[56] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5BilinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[3][3] ={{2*d00 / detJ, 2*d10 / detJ, 2*d20 / detJ}, {2*d01 / detJ, 2*d11 / detJ, 2*d21 / detJ}, {2*d02 / detJ, 2*d12 / detJ, 2*d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.61248608016091, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 0, 4.18330013267038, -0.591607978309963, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05987111581756e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 0, -1.67332005306816, -1.18321595661993, 7.74596669241484, 0, 0.346410161513773, 0, 0.282842712474621, 0.199999999999999, 0, 1.01607205572352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.44948974278317, 0, 0, 0, 7.09929573971954, 0, -0.414039335605422, 0, 0, 0, 0, 0, 0, 0, 1.31603755427645e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0, 4.38178046004132, -0.774596669241483, 0, 0, 4.76235235991626, 0, -0.740656079818042, 0.130930734141596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.12132034355964, 0, 0, 0, 0, 0, 7.17137165600635, 0, -1.20615246152436e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.35849019742426e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 0, 1.58113883008419, 2.45967477524977, 0, 0, 0, 0, 5.34522483824849, -1.20948631362952, 0, 0, 0, 0, 0, -1.32866943359025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, 0, 0, 3.83405790253617, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 1.14970665364125e-14, 0, 0, 1.00926211832331e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.80288457473997, 0, 0, 0, -2.10229466468021, 0, -1.71651640581399, 0, 0, 9.81070843517429, 0, 0.305838870920591, 0, 0, 0.264864231681987, 0, 0.216260739578393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 0, 1.77281052085584, -1.25356634105602, 2.39356776939084, 0, -2.78313372537744, 0, -0.524404424085074, 0.370809924354783, 0, 9.40174755792015, -1.14927687399652e-14, 0.840917865872085, -0.301561193523231, 0, 0.350641890608828, 0, 0.0660687472643403, -0.046717659215121, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.3433848874678e-14, 1.29756443747035, 0, 0, 0, 5.01426536422406, 0, -0.383824736789373, 0, 0, 0, 0, 8.20651806648289, 0, 0, -0.631738055305786, 0, 0.0483573714561588, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0, 4.06201920231798, -0.820651806648292, 0, 0, 5.88642506110457, 0, -1.20156148406979, 0.242752078113805, 0, 0, 0, 5.13809303146605, 0, 0, -0.74161984870956, 0, 0.151382517704866, -0.0305838870920676, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 0, -0.792824967172097, 2.24244764232554, 2.14087209644419, 0, 0.095742710775635, -1.39713895489839e-14, -1.87616630392937, -2.15580611373102, 0, 0, 0, 0, 9.44035859331469, 1.15714811919607e-14, 0.422185670932519, 1.95127251818837e-14, 0.590936840285278, 0.522319308773324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.16057691494799, 0, 0, 0, 1.96214168703485, 0, 2.74642624930237, 0, 0, 0, 0, 0, 0, 0, 8.6522315682783, 0, -0.865042958313572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04119910644746e-14, 0, 0, 0, 0, 0, 0, 1.5543528562087e-14, 0, 0, 0, 0},
    {1.32664991614216, 0, 2.07610309995255, 1.46802658041887, 0, 0, 1.31624465810881, 0, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 0, 5.80409338312195, 0, -1.54743588659734, 0.34193825277012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.00508909139073, 0, 0, 0, 0, 0, 4.45964044034302, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0, 0.749149177264393, 2.75458800949268, 0, 0, 0, 0, 3.3240197266047, 2.06838446274243, 0, 0, 0, 0, 0, 0, 0, 0, 6.28180682119209, -1.77676328054752, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964681, 0, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.80288457473997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 0, -2.20543546226846, -1.55947837083931, 8.33749895885385, 0, 1.06532653910462, 0, 0.869835476750494, 0.615066564126913, 0, -2.18326971917505, 0, -0.278967971730994, -1.89076704011891, 0, -0.241593350361257, 0, -0.197260144544842, -0.139483985865489, 11.8490505948789, 0, 0.245326690731332, 0, 0.0313466946954478, 0, 0.219426862868129, 0, 0.0280373360835799, 0.190029237516524, 0, 0.0242810453028227, 0, 0.0198253904711049, 0.0140186680417927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.45525332069938, 0, 0, 0, 5.28531208250978, 0, -1.36277028773849, 0, 0, 2.33666428910958, -1.04179543359539e-14, -3.93352933905728, 0, 0, -1.19859611757307, 0, 0.30904725218263, 0, 0, 0, 11.55277744383, 0, 0.810328731479531, 0, -0.234843597212094, 0, 0.395334573318417, 0, 0, 0.120463442338382, 0, -0.0310604175145514, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331886, 0, 2.99085908983323, -1.2440333788203, 0.95014618758261, 0, -0.169967317119764, 0, -1.1796107330308, 0.490653381462657, 0, 4.9761335152812, 0, -3.2268221135323, -0.21547290184284, 0, 0.0385449644663875, 0, 0.267510569439613, -0.11126972805284, 0, 0, 10.7636648722209, 0, 1.37533064807499, 0, -0.500120235663638, 0, 0.324307824721468, 0.0216558414515664, 0, -0.00387391468765637, 0, -0.0268858238268342, 0.0111830284386732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.772618130456579, 0, 0, 0, 3.3588901228148, 0, -0.304724700110023, 0, 0, 0, 0, 7.32969916135162, 0, 0, -0.761724681099362, 0, 0.0691050664139862, 0, 0, 0, 0, 0, 9.05975063854702, 0, 0, 0, -0.736662483163211, 0, 0, 0.0765562109321137, 0, -0.00694532049720535, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376113, 0, 3.68560354207934, -0.814411017927954, 0, 0, 6.00856531485322, 0, -1.45362164025439, 0.321208037219811, 0, 0, 0, 6.99294104721297, 0, 0, -1.362614533662, 0, 0.329650402328134, -0.0728431359084717, 0, 0, 0, 0, 5.40219154319687, 0, 0, 0, -0.702817019232737, 0, 0, 0.136947913395212, 0, -0.0331311120155176, 0.00732101061700613, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.1542003094028, 0, 0, 0, -0.761811750275056, 0, 5.59815020469135, 0, 0, 2.13307290077016, 0, 0.0664963811608019, 0, 0, -2.36108976914458, 0, -3.33842298506509, 0, 0, 0, 0, 0, 0, 0, 11.5766222110848, 0, 0.360889439280294, 0, 0, 0.555625639776453, 0, 0.595437196138644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 0, 0.963624111659426, 1.36277028773849, 0.867360833110884, 0, -1.00852844760553, -1.31315568597201e-14, 1.71026313764871, -2.01556443707465, 0, 2.04415543160775, -1.06351258615242e-14, 0.182834820066006, 2.68821895760394, 2.34958042241799e-14, -3.12574097035512, 0, -1.01990506872275, 0.904016610326379, 0, 0, 0, 0, 0, 0, 11.0940489487798, 0, 0.992281903807273, -0.632607619454903, 0, 0.735567892896446, 0, 0.181909068193175, -0.147004726489975, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24564920565478e-14, 0, 0, 1.009146310714e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.705300630647536, 0, 0, 0, 1.81702705031798, 0, 1.25178444059442, 0, 0, 0, 0, 1.78428514239953, 0, 0, 5.631527706439, 0, -0.746494073225355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.68367982311023, 0, 0, -1.32524448062076, 0, 0.133143804689787, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29216299118995e-14, 0, 0},
    {1.1517511068998, 0, 2.20794021658195, 0.892142571199758, 0, 0, 2.13307290077015, 0, 3.91870514328394, -1.31949665727936, 0, 0, 0, 1.11713920350151, 0, 0, 6.6110513536048, 0, -2.33689608954558, 0.591817792331142, 0, 0, 0, 0, 0, 0, 0, 0, 6.06294257990893, 0, 0, -1.55575179137407, 0, 0.416806037297055, -0.0962371838080836, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493888, 0, -0.430945803685668, 3.04724700110021, 0.775791113542722, 0, 0.0346944333244404, -1.49568650774796e-14, -1.52970585407784, 0.901387818865989, 0, 0, 0, 0, 4.51561078268939, 0, 0.20194425340049, 0, -1.83730997289732, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, 0, 0.487107866980278, 0, 0.894874292847885, 0.903959543974648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.630840061880567, 0, 0, 0, 0.711024300256716, 0, 2.23926008187654, 0, 0, 0, 0, 0, 0, 0, 4.13862564412659, 0, 2.68954640486934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.98274066131697, 0, -1.30996183150502, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727542, 0, 1.12848100903608, 1.99489143482414, 0, 0, 0.47696960070847, 0, 4.00571020990215, 0.59009684435208, 0, 0, 0, 0, 0, 0, 2.77627448210728, 0, 4.8112068719424, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.69662601613678, 0, -2.34333096254029, 0.591780433634516, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.546323519313506, 0, 0, 0, 0, 0, 2.69341127303542, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.71889089790563, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059451, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199766, 0, 0.40720550896397, 2.53385243517954, 0, 0, 0, 0, 2.00755023262382, 3.29336152870136, 0, 0, 0, 0, 0, 0, 1.24513151875208e-14, 0, 5.00796559370719, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084685, 0, 0, 2.53908227033118, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.73861278752583, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867318, -0.381881307912986, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 3.62284418654736, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.3228756555323, 0, 4.83045891539648, 0.341565025531989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693486, 0, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62018517460197, 0, 0, 3.76497011940334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923788, -0.836660026534075, -0.591607978309962, 3.87298334620742, -0.599999999999998, 0.173205080756888, -0.489897948556638, 0.141421356237313, 0.0999999999999996, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386223, 1.22474487139159, 1.41421356237309, -0.500000000000003, 4.58257569495584, 3.54964786985977, -1.02469507659596, -0.207019667802713, -0.239045721866875, 0.0845154254728531, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000003, -2.84604989415154, 2.19089023002066, -0.387298334620745, 0, 5.49909083394701, 2.38117617995813, 0.481070235442362, -0.370328039909021, 0.0654653670707988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.59807621135333, 0, -1.5811388300842, -1.1180339887499, 0, 0, 6.87386354243376, 0, 0.267261241912429, 0.188982236504611, 0, 0, 0, -1.56646390951001e-14, 0, 0, 1.01970842391297e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.0062305898749, 1.06066017177982, -0.204124145231929, 1.58771324027146, 0, 0, 0, 3.58568582800318, -0.690065559342348, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.77942286340599, 1.36930639376292, 0.790569415042096, 1.22983738762489, 0, 0, 0, 4.62910049886275, 2.67261241912425, -0.604743156814767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.900000000000007, 0, 1.82574185835055, -1.42009389360937, 0, 0, 0, 0, 6.17213399848367, 0.698297248755177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067893, 0, 0, 1.91702895126809, 0, 0, 0, 0, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.10227038425243, 0, 0, 3.3203915431768, 0, 0, 0, 0, 0, 4.48998886412873, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219015, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849178, 0.152919435460295, -0.0529728463363973, -0.512907379155188, 0.132432115840996, -0.0458758306380885, 0.1081303697892, -0.0374574588632162, -0.0264864231681936, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114625, -0.262202212042537, 0.185404962177395, 5.56214886532174, 4.70087377896007, -1.21376039056904, 0.420458932936042, -0.150780596761618, -0.397100103328477, 0.175320945304415, 0.0572172135271331, 0.0330343736321715, -0.0233588296075578, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735171, 1.87287294316099, -0.529728463363976, -4.04586796856344, 2.50713268211204, 0.904683580569682, -0.191912368394685, -0.554003287767448, 0.156695792632001, 0, 6.35674156036771, 4.10325903324145, -1.42141062443803, 0.509731451534329, -0.315869027652891, -0.113979417590034, 0.0241786857280778, 0.0697978535688028, -0.0197418142283115, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709575, 3.51781181986757, 2.03100960115899, -0.410325903324144, 0, -3.39852909359329, 2.94321253055229, -1.04058276941337, -0.600780742034897, 0.121376039056905, 0, 0, 7.41619848709566, 2.56904651573303, 0, 0.428174419288832, -0.370809924354779, 0.131101106021265, 0.0756912588524378, -0.0152919435460309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.71269767715534, 0, 4.69041575982343, 0.94760708295868, 0, 0, -1.69926454679664, 0, -1.38744369255116, -0.280305955290692, 0, 0, 0, 8.8994381845148, 0, 0, 0.214087209644411, 0, 0.17480147469502, 0.0353152308909268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586045, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878174, 3.24961536185438, -0.938083151964683, -1.07790305686552, 0, 0, 0, 0, 4.72017929665735, -0.731247032282672, 0.211092835466259, -1.02353263143832, 0.295468420142642, 0.26115965438666, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577669, 0.580288457473998, 0.670059394260483, 0.947607082958683, 1.26655701279755, 0.981070843517428, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694758, 0, 0, 0, 0, 5.58499146191198, 4.32611578413915, -1.24884205625912, -0.432521479156774, -0.499432784842937, 0.220720193068322, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071078, -1.34846851907591, 1.03805154997627, 0.73401329020944, 0, 1.51986841535706, 0.658122329054406, -3.19105893754051, 2.45647831545196, -0.705653192844341, 0, 0, 0, 0, 0, 6.70198975429437, 2.90204669156098, 1.00508909139074, -0.773717943298673, 0.170969126385064, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 0, -0.749149177264396, 2.11891385345588, 0, 0, 1.89983551919633, 0, -1.77281052085583, -2.03704530421603, 0, 0, 0, 0, 0, 0, 8.37748719286796, 0, 0.558382828550407, 0.493545355707621, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22447624956168e-14},
    {0.741619848709566, 0.502544545695365, -0.096714742912331, 1.77807891440463, 0, 0, 0, 2.22982022017151, -0.429129101453498, 1.33513642962593, 0, 0, 0, 0, 0, 0, 0, 4.21396412211221, -0.81097777341897, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5744562646538, 0.648782218735174, 0.374574588632197, 1.37729400474634, 0, 0, 0, 2.87868552592027, 1.66200986330235, 1.03419223137121, 0, 0, 0, 0, 0, 0, 0, 5.44020428881872, 3.14090341059604, -0.888381640273766, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.663324958071084, 0, 0.865042958313566, -1.59036212878712, 0, 0, 0, 0, 3.83824736789369, -1.19418232635198, 0, 0, 0, 0, 0, 0, 0, 0, 7.25360571842496, 1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.46904157598234, 0, 0, 1.55707732496443, 0, 0, 0, 0, 0, 2.76353810488345, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.90144228736999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.812403840463591, 0, 0, 2.69693703815183, 0, 0, 0, 0, 0, 4.78658840631075, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.02544545695368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552312, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541881, -0.139483985865496, -0.945383520059461, 0.313838968197358, -0.120796675180629, 0.25624844449503, -0.0986300722724195, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963984, 0.0156733473477253, -0.519258730913241, 0.109713431434065, -0.0364215679542342, 0.01401866804179, 0.0950146187582634, -0.0315420030940289, 0.0121405226514124, -0.0257539376818875, 0.00991269523555186, 0.00700933402089779, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706032, -0.705300630647551, 1.61604676382125, 2.64265604125489, -1.44543616748915, -0.681385143869243, -0.196698948117362, 0.278174320132098, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444731, -0.366484958067585, -0.599298058786534, 0.327794111588103, 0.15452362609131, 0.0446071285599885, -0.0630840061880518, 6.54980915752513, 5.77638872191499, -1.22048487632307, 0.405164365739766, -0.155947837083929, -0.117421798606046, -0.453666435153256, 0.197667286659209, -0.0824223552841552, 0.0368331241581584, 0.0602317211691907, -0.0329445477765348, -0.0155302087572761, -0.00448318510329359, 0.00634018117570608, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659432, 0.914174100330065, 1.49542954491661, -0.622016689410148, 0.475073093791304, 2.87032228155655, -0.0849836585598799, -0.360555127546397, -0.589805366515403, 0.245326690731328, -5.15186050535791, 2.48806675764059, 2.8908723349783, -1.61341105676615, -0.107736450921414, -0.650927909109099, 0.019272482233193, 0.0817662172643042, 0.133755284719809, -0.0556348640264175, 0, 7.27756573058348, 5.38183243611043, -1.7866069197544, 0.687665324037496, 0.517781462682516, -0.250060117831819, -0.290543601573986, 0.162153912360733, 0.0108279207257825, 0.0654207163662268, -0.00193695734382766, -0.00821781403613334, -0.0134429119134184, 0.00559151421933731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927964, 0.386309065228293, 1.93297557093283, -0.525700051567136, 4.5772145303823, 1.6794450614074, 1.93925611658549, -0.152362350055012, -0.762375846428075, 0.207338896470047, 0, -4.7313004641042, 3.66484958067581, 0.282120252259006, -1.03801468669568, -0.38086234054968, -0.439781949681085, 0.0345525332069918, 0.172890591018093, -0.0470200420431731, 0, 0, 8.18726144690641, 4.52987531927351, -1.74354982331868, 0, 0.475513588177819, -0.368331241581605, -0.0283541521970749, 0.10432440129236, 0.0382781054660568, 0.0441997489897906, -0.00347266024860305, -0.0173761581875701, 0.00472569203284699, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880556, -3.39131543920102, 1.84280177103967, -0.407205508963983, 0, 5.01084538062905, 3.00428265742661, 1.33755284719805, -0.726810820127196, 0.160604018609904, 0, 0, -3.78504037128335, 3.49647052360648, 0, -1.13635292017211, -0.681307266831003, -0.303328474207914, 0.164825201164063, -0.0364215679542416, 0, 0, 0, 9.3568702250359, 2.70109577159844, 0, 0, 0.380410870542252, -0.351408509616368, 0, 0.11420776562532, 0.0684739566976048, 0.0304856587023835, -0.0165655560077574, 0.00366050530850137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54950975679639, 0, -2.32737334062814, -1.64570147151095, 0, 0, 6.74536878161603, 0, 0.917928424547679, 0.649073413641553, 0, 0, 0, -1.76635217326556, 0, 0, -1.52970585407782, 0, -0.208166599946601, -0.147196014438789, 0, 0, 0, 0, 10.9163485958752, 0, 0, 0, 0.177525072919717, 0, 0, 0.153741222957161, 0, 0.020921530482028, 0.0147937560766434, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412066, 1.5771001547014, -0.546323519313517, 0.772618130456561, 1.47524211088021, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292753, -1.14271762541258, 1.06653645038508, -0.12876968840943, 0.0332481905804026, -0.011517511068999, 4.57223067739883, -1.18054488457228, 0.408952744138947, -1.66921149253253, 0.578231822728847, 0.512529242570405, 0, 0, 0, 0, 0, 5.78831110554239, -0.698859394077356, 0.180444719640146, -0.062507884474852, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069323, -0.10313274764285, -0.0833438459664692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186249, 0.834522960396274, 0.481812055829709, 0.681385143869246, 0.433680416555438, 1.14215318100896, -0.504264223802767, 1.48113132435986, 0.855131568824349, -1.00778221853732, 1.20933866224478, 1.02207771580387, -0.263899331455877, 0.0914174100330005, 1.34410947880197, 3.53988526627811, -1.56287048517756, -0.88326369896241, -0.509952534361379, 0.452008305163194, 0, 0, 0, 0, 0, 6.56332786985937, 5.54702447438989, -1.4322355606878, 0.496140951903638, -0.316303809727451, -0.833026783443143, 0.367783946448223, 0.15753787423404, 0.0909545340965868, -0.0735023632449856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14091633431099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.743452142666481, 0.35265031532376, 1.01801377240995, 0.575875553449897, -1.46610739697708, 0.908513525158993, 0.327831580195603, 0.62589222029721, 1.80679520936143, -0.851731429836555, 0, 1.38210132827975, 0.892142571199764, -0.309047252182628, -4.54391937932164, 2.8157638532195, 1.01605126164418, -0.373247036612687, -1.07747138531276, 0.382016742281294, 0, 0, 0, 0, 0, 0, 7.50094613698214, 4.84183991155512, -1.67726254578565, 1.06930204231198, -0.662622240310384, -0.239103205508895, 0.066571902344891, 0.192176528696449, -0.0621208350291014, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10027752117729e-14, 0, 0},
    {0.575875553449902, 1.91213231759728, 1.10397010829098, 0.44607128559988, 0, -1.23153021346074, 1.06653645038507, 3.39369820402464, 1.95935257164197, -0.659748328639681, 0, 0, 1.6124515496597, 0.558569601750758, 0, -3.81689227863019, 3.3055256768024, -2.02381137955099, -1.16844804477279, 0.295908896165575, 0, 0, 0, 0, 0, 0, 0, 8.75110382647916, 3.03147128995446, 0, 0.898213715542065, -0.777875895687036, 0.360964616749972, 0.208403018648526, -0.0481185919040409, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.32945523855878e-14, 0, 0},
    {-1.32992762321609, 0, 2.54950975679639, -1.03015750727543, 0, 0, -0.615765106730374, 0, 4.52493093869951, 1.52362350055012, 0, 0, 0, 1.93494185959165, 0, 0, -1.9084461393151, 0, -2.69841517273464, -0.683372323427214, 0, 0, 0, 0, 0, 0, 0, 0, 10.501324591775, 0, 0, 0.449106857771034, 0, 0.481286155666633, 0.111125127955289, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.786795792469438, 0.746420027292178, -0.215472901842835, 1.5236235005501, 0.387895556771362, -0.0600925212577327, 0.0173472166622199, 2.64952825989834, -0.764852927038915, 0.45069390943299, 0, 0, 0, 0, 2.25780539134469, -0.349777707186207, 0.100972126700243, 3.18231422231115, -0.918654986448657, -1.37412670680299, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.44603151471411, -0.843695574376338, 0.243553933490139, -1.54996774159981, 0.447437146423943, 0.451979771987323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608038, 0.315420030940283, 0.364215679542343, 1.28769688409429, 0.458964212273842, 0.355512150128361, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137522, 0, 0, 0, 0, 2.67147136598154, 2.06931282206329, -0.597359157427889, 1.34477320243467, 1.5528103408493, -1.16134903279062, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.4438313884647, 4.99137033065849, -1.44088450201539, -0.654980915752514, -0.756306816047561, 0.381992627346313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.51507875363771, -0.732969916135158, 0.56424050451804, 0.997445717412072, 0, 0.550757054728608, 0.238484800354237, -2.60178510148047, 2.00285510495107, 0.295048422176044, 0, 0, 0, 0, 0, 3.20576563917784, 1.38813724105364, -3.12497053047329, 2.4056034359712, -0.899577092626427, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.73259766615765, 3.34831300806839, 1.5220381072759, -1.17166548127015, 0.295890216817259, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48690428533295, 0, -0.40720550896398, 2.87937776724947, 0, 0, 0.688446318410768, 0, -1.44543616748914, 0.851731429836557, 0, 0, 0, 0, 0, 0, 4.00720704897231, 0, -1.73609473915183, -2.59685538292347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.66574708269706, 0, 0.845576726264391, 0.854161481650102, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449896, 0.273161759656754, -0.0525700051567131, 1.63559471386625, 0, 0, 0, 1.34670563651772, -0.259173620587566, 2.12585572561676, 0, 0, 0, 0, 0, 0, 0, 3.35944544895281, -0.646525578093589, 0.914325240961068, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4.72691760029726, -0.909695716322936, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599885, 0.35265031532377, 0.203602754481986, 1.26692621758977, 0, 0, 0, 1.73858950082559, 1.0037751163119, 1.64668076435068, 0, 0, 0, 0, 0, 0, 0, 4.33702542542885, 2.50398279685359, 0.708233286251854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.10242438161534, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.515078753637711, 0, 0.470200420431694, -1.46292038553772, 0, 0, 0, 0, 2.31811933443412, -1.90142316513449, 0, 0, 0, 0, 0, 0, 0, 0, 5.78270056723845, -0.817797356933148, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.13656584215379, 1.31506763029891, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12763531151527e-14},
    {0.364215679542343, 0, 0, 1.26954113516559, 0, 0, 0, 0, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.46962034034308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880561, 0, 0, 2.19890974840547, 0, 0, 0, 0, 0, 4.33630850246743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.00955871244863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.4358503831282, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.912870929175276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.58198889747161, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867317, -0.381881307912987, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 0.724568837309471, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.661437827766144, 0, 1.93218356615859, -0.170782512765995, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693487, 3.54964786985977, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.540061724867317, 0, 3.54964786985977, 1.25499003980111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.90940653956493, 0, 0, 4.43705983732471, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923789, -0.836660026534075, -0.591607978309964, 3.87298334620742, -0.600000000000001, 0.173205080756889, -0.48989794855664, 0.141421356237313, 0.0999999999999997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386222, 1.22474487139158, 1.41421356237309, -0.500000000000002, 0.654653670707977, 3.54964786985977, -1.02469507659596, -0.20701966780271, -0.239045721866875, 0.0845154254728546, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000001, 0.316227766016839, 2.19089023002066, -0.38729833462074, 0, 1.57116880969914, 2.38117617995813, -0.0534522483824853, -0.37032803990902, 0.0654653670707977, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.519615242270665, 0, 1.58113883008419, -0.223606797749979, 0, 0, 2.94594151818589, 0, -0.267261241912424, 0.0377964473009217, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.00623058987491, 1.06066017177982, -0.204124145231932, 1.58771324027146, 4.53557367611073, 0, 0, 3.58568582800318, -0.690065559342349, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.779422863405996, 0.273861278752581, 0.790569415042093, 1.22983738762489, 0, 4.53557367611073, 0, 0.925820099772549, 2.67261241912425, -0.604743156814761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.450000000000004, 0, 0.730296743340221, 0.710046946804697, 0, 0, 4.53557367611072, 0, 2.46885359939347, -0.349148624377589, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067889, -3.1304951684997, 0, 1.91702895126808, 0, 0, 0, 5.29150262212918, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.367423461417479, 0, -3.1304951684997, 1.10679718105894, 0, 0, 0, 0, 5.29150262212918, 1.49666295470958, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.85788383248864, 0, 0, -2.34787137637477, 0, 0, 0, 0, 0, 6.34980314655501, 0, 0, 0, 0, 0, 0, 1.11852851939587e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219016, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849176, 0.152919435460295, -0.0529728463363977, -0.512907379155187, 0.132432115840995, -0.0458758306380897, 0.1081303697892, -0.0374574588632173, -0.0264864231681947, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114629, -0.262202212042535, 0.185404962177393, 0.618016540591305, 4.70087377896007, -1.21376039056904, 0.42045893293604, -0.150780596761618, -0.397100103328475, 0.175320945304413, 0.057217213527133, 0.0330343736321713, -0.0233588296075595, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735172, 1.87287294316099, -0.529728463363977, 0.20229339842817, 2.50713268211204, 0.904683580569681, -0.191912368394685, -0.554003287767448, 0.156695792632003, 0, 1.41260923563727, 4.10325903324145, -1.42141062443803, -0.0254865725767146, -0.315869027652895, -0.113979417590039, 0.0241786857280816, 0.0697978535688034, -0.0197418142283076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709574, 0.167514848565124, 2.03100960115899, -0.410325903324145, 0, 0.849632273398319, 2.94321253055228, -0.0495515604482551, -0.600780742034895, 0.121376039056903, 0, 0, 2.47206616236523, 2.56904651573303, 0, -0.10704360482221, -0.370809924354777, 0.00624290981053448, 0.0756912588524339, -0.015291943546034, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.428174419288845, 0, 1.34011878852098, -0.236901770739674, 0, 0, 2.54889682019496, 0, -0.396412483586046, 0.0700764888226733, 0, 0, 0, 3.95530585978436, 0, 0, -0.321130814466627, 0, 0.0499432784842904, -0.00882880772273737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586047, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878164, 3.24961536185438, -0.938083151964685, -1.07790305686551, 5.52770798392566, 0, 0, 0, 4.72017929665734, -0.731247032282673, 0.211092835466259, -1.02353263143831, 0.295468420142642, 0.261159654386662, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577672, 0.580288457473996, 0.670059394260487, 0.947607082958685, 0.180936716113935, 0.981070843517426, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694756, 0, 5.52770798392567, 0, 0, 0.797855923130281, 4.32611578413915, -1.24884205625911, -0.432521479156781, -0.499432784842937, 0.220720193068326, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071081, 0.149829835452877, 1.03805154997628, 0.734013290209435, 0, 0.434248118673443, 0.658122329054405, 0.354562104171167, 2.45647831545196, -0.705653192844349, 0, 0, 5.52770798392567, 0, 0, 1.91485421551268, 2.90204669156097, -0.111676565710086, -0.773717943298669, 0.170969126385058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.382970843102543, 0, 0.749149177264393, 0.423782770691177, 0, 0, 0.814215222512714, 0, 1.77281052085584, -0.407409060843206, 0, 0, 0, 5.52770798392568, 0, 0, 3.59035165408627, 0, -0.558382828550415, 0.0987090711415228, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709561, 0.502544545695366, -0.096714742912333, 1.77807891440463, -4.23078640106406, 0, 0, 2.2298202201715, -0.429129101453499, 1.33513642962593, 0, 0, 0, 0, 6.21867148191638, 0, 0, 4.21396412211221, -0.810977773418968, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.574456264653794, 0.129756443747034, 0.374574588632199, 1.37729400474634, 0, -4.23078640106406, 0, 0.575737105184058, 1.66200986330234, 1.03419223137121, 0, 0, 0, 0, 0, 6.21867148191637, 0, 1.08804085776374, 3.14090341059604, -0.88838164027376, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.331662479035532, 0, 0.346017183325429, 0.795181064393553, 0, 0, -4.23078640106406, 0, 1.53529894715748, 0.597091163175992, 0, 0, 0, 0, 0, 0, 6.21867148191637, 0, 2.90144228736999, -0.512907379155182, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.469041575982343, 3.81404493622062, 0, 1.55707732496444, 0, 0, 0, -3.76069902316805, 0, 2.76353810488346, 0, 0, 0, 0, 0, 0, 1.02342965294379e-14, 7.10705312219014, 0, 2.90144228737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.270801280154526, 0, 3.81404493622062, 0.89897901271727, 0, 0, 0, 0, -3.76069902316805, 1.59552946877025, 0, 0, 0, 0, 0, 0, 0, 0, 7.10705312219014, 1.67514848565122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.68079590171775, 0, 0, 4.4497190922574, 0, 0, 0, 0, 0, -2.63248931621764, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.2915619758885, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552311, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541882, -0.139483985865495, -0.945383520059461, 0.313838968197358, -0.120796675180628, 0.256248444495029, -0.0986300722724191, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963982, 0.0156733473477249, -0.519258730913241, 0.109713431434065, -0.0364215679542341, 0.0140186680417898, 0.0950146187582634, -0.0315420030940288, 0.0121405226514124, -0.0257539376818875, 0.00991269523555182, 0.00700933402089769, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706025, -0.705300630647557, 1.61604676382126, 2.64265604125489, -1.44543616748915, -0.681385143869247, -0.196698948117359, 0.278174320132096, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444727, -0.366484958067587, -0.599298058786529, 0.327794111588101, 0.154523626091317, 0.0446071285599885, -0.0630840061880538, 0.595437196138649, 5.77638872191499, -1.22048487632307, 0.405164365739765, -0.155947837083928, -0.117421798606047, -0.453666435153255, 0.197667286659209, -0.0824223552841549, 0.0368331241581608, 0.0602317211691907, -0.0329445477765342, -0.0155302087572759, -0.00448318510329279, 0.00634018117570581, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659437, 0.914174100330057, 1.49542954491662, -0.622016689410149, 0.475073093791305, 2.87032228155655, -0.084983658559881, -0.3605551275464, -0.589805366515401, 0.24532669073133, 0.147196014438797, 2.4880667576406, 2.89087233497828, -1.61341105676615, -0.107736450921419, -0.650927909109092, 0.0192724822331939, 0.0817662172643072, 0.133755284719809, -0.0556348640264189, 0, 1.32319376919699, 5.38183243611043, -1.7866069197544, 0.687665324037495, -0.0147937560766431, -0.25006011783182, -0.290543601573986, 0.162153912360733, 0.0108279207257832, 0.0654207163662248, -0.00193695734382736, -0.00821781403613383, -0.0134429119134179, 0.00559151421933732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927959, 0.386309065228289, 1.93297557093283, -0.525700051567137, 0.0803020093049556, 1.6794450614074, 1.9392561165855, -0.152362350055011, -0.762375846428077, 0.207338896470047, 0, 0.567756055692504, 3.66484958067581, 0.282120252259013, -0.0182107839771216, -0.38086234054968, -0.439781949681088, 0.0345525332069925, 0.172890591018093, -0.0470200420431761, 0, 0, 2.23288948551993, 4.52987531927351, -1.74354982331867, 0, -0.0570616305813383, -0.368331241581605, -0.0283541521970753, 0.0018302526542526, 0.0382781054660563, 0.0441997489897914, -0.00347266024860275, -0.0173761581875703, 0.00472569203284603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880563, 0.09974457174121, 1.84280177103967, -0.407205508963979, 0, 0.513932859551699, 3.00428265742661, -0.03933978962347, -0.726810820127196, 0.160604018609906, 0, 0, 1.51401614851335, 3.49647052360648, 0, -0.116549017453554, -0.681307266831002, 0.00892142571199782, 0.164825201164067, -0.036421567954237, 0, 0, 0, 3.40249826364942, 2.70109577159844, 0, 0, -0.152164348216903, -0.351408509616369, 0, 0.0117136169872135, 0.0684739566976059, -0.000896637020657868, -0.0165655560077585, 0.00366050530850301, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, 0, 1.16368667031409, -0.23510021021585, 0, 0, 2.24845626053868, 0, -0.458964212273845, 0.0927247733773657, 0, 0, 0, 3.53270434653115, 0, 0, -0.509901951359277, 0, 0.104083299973309, -0.0210280020626858, 0, 0, 0, 0, 4.96197663448874, 0, 0, 0, -0.355050145839439, 0, 0, 0.0512470743190546, 0, -0.010460765241014, 0.00211339372523455, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412067, 1.5771001547014, -0.546323519313514, 0.772618130456565, 1.4752421108802, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292756, -1.14271762541258, 1.06653645038508, -0.128769688409428, 0.0332481905804048, -0.0115175110689984, 4.57223067739883, -1.18054488457228, 0.408952744138948, -1.66921149253254, 0.578231822728845, 0.512529242570406, 6.52268767805531, 0, 0, 0, 0, 5.7883111055424, -0.698859394077357, 0.180444719640146, -0.0625078844748521, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069321, -0.103132747642851, -0.0833438459664682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186242, 0.834522960396282, 0.481812055829709, 0.681385143869242, 0.433680416555441, 1.14215318100896, -0.504264223802765, 1.48113132435985, 0.855131568824352, -1.00778221853732, 0.134370962471638, 1.02207771580387, -0.263899331455871, 0.0914174100330022, 1.34410947880196, 3.5398852662781, -1.56287048517756, -0.883263698962406, -0.509952534361376, 0.452008305163191, 0, 6.52268767805531, 0, 0, 0, 0.729258652206596, 5.5470244743899, -1.4322355606878, 0.496140951903638, -0.316303809727452, -0.833026783443137, 0.367783946448223, 0.157537874234044, 0.0909545340965877, -0.0735023632449854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01901886515007e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.74345214266648, 0.352650315323769, 1.01801377240995, 0.5758755534499, 0.0733053698488504, 0.908513525158993, 0.327831580195604, 0.62589222029721, 1.80679520936143, -0.851731429836554, 0, 0.307133628506613, 0.892142571199772, -0.309047252182627, 0.227195968966083, 2.8157638532195, 1.01605126164419, -0.37324703661268, -1.07747138531275, 0.382016742281307, 0, 0, 6.52268767805531, 0, 0, 0, 1.66687691932936, 4.84183991155511, -1.67726254578565, -0.0534651021155991, -0.662622240310383, -0.239103205508893, 0.0665719023448925, 0.192176528696449, -0.0621208350290961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449893, 0.0910539198855843, 1.10397010829098, 0.446071285599878, 0, 0.307882553365187, 1.06653645038508, 0.161604676382129, 1.95935257164197, -0.65974832863968, 0, 0, 0.537483849886575, 0.558569601750753, 0, 0.954223069657542, 3.3055256768024, -0.0963719704548154, -1.16844804477279, 0.295908896165572, 0, 0, 0, 6.52268767805531, 0, 0, 0, 2.91703460882639, 3.03147128995447, 0, -0.224553428885516, -0.777875895687036, 0.0171887912738079, 0.208403018648526, -0.0481185919040412, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.332481905804022, 0, 0.728431359084682, 0.257539376818849, 0, 0, 0.923647660095564, 0, 1.292837411057, -0.380905875137523, 0, 0, 0, 0.859974159818509, 0, 0, 2.86266920897263, 0, -0.770975763638467, 0.170843080856795, 0, 0, 0, 0, 6.52268767805531, 0, 0, 0, 4.66725537412222, 0, 0, -0.673660286656546, 0, 0.137510330190468, -0.0277812819888241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.78679579246944, 0.746420027292177, -0.215472901842835, 1.52362350055011, 0.387895556771359, -0.0600925212577337, 0.0173472166622209, 2.64952825989835, -0.764852927038921, 0.450693909432998, -5.28814187068052, 0, 0, 0, 2.2578053913447, -0.349777707186206, 0.100972126700245, 3.18231422231115, -0.91865498644866, -1.37412670680298, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0, 5.44603151471411, -0.843695574376342, 0.243553933490139, -1.54996774159981, 0.447437146423942, 0.451979771987325, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608043, 0.315420030940282, 0.364215679542341, 1.28769688409429, 0.0655663160391197, 0.355512150128356, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137518, 0, -5.28814187068053, 0, 0, 0.381638766568791, 2.0693128220633, -0.597359157427881, 1.34477320243467, 1.5528103408493, -1.16134903279063, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0.920547341209241, 4.99137033065848, -1.4408845020154, -0.654980915752512, -0.756306816047561, 0.381992627346309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637708, 0.0814411017927942, 0.564240504518039, 0.997445717412071, 0, 0.157359158493888, 0.238484800354236, 0.289087233497833, 2.00285510495107, 0.295048422176043, 0, 0, -5.28814187068052, 0, 0, 0.915933039765097, 1.38813724105364, 0.347218947830363, 2.4056034359712, -0.89957709262643, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 2.20931361890218, 3.34831300806839, -0.16911534525288, -1.17166548127015, 0.295890216817257, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.297380857066588, 0, 0.407205508963972, 0.575875553449888, 0, 0, 0.295048422176039, 0, 1.44543616748915, 0.17034628596731, 0, 0, 0, -5.28814187068053, 0, 0, 1.71737444955956, 0, 1.73609473915183, -0.519371076584704, 0, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 4.14246303544159, 0, -0.845576726264386, 0.170832296330017, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449895, 0.27316175965675, -0.0525700051567132, 1.63559471386626, 4.82647810240717, 0, 0, 1.34670563651771, -0.259173620587563, 2.12585572561677, 0, 0, 0, 0, -4.95763300376299, 0, 0, 3.35944544895281, -0.646525578093588, 0.914325240961076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 0, 4.72691760029725, -0.909695716322934, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599889, 0.0705300630647548, 0.203602754481986, 1.26692621758977, 0, 4.82647810240717, 0, 0.34771790016512, 1.00377511631191, 1.64668076435067, 0, 0, 0, 0, 0, -4.95763300376299, 1.29915143264338e-14, 0.867405085085771, 2.50398279685359, 0.708233286251852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873427, 0, 1.22048487632307, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.257539376818862, 0, 0.188080168172671, 0.731460192768861, 0, 0, 4.82647810240716, 0, 0.927247733773645, 0.950711582567244, 0, 0, 0, -1.22220110126485e-14, 0, 0, -4.95763300376297, 0, 2.31308022689538, 0.408898678466573, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 3.25462633686152, -0.657533815149454, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04894165366026e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, -3.97354131880428, 0, 1.26954113516558, 0, 0, 0, 5.11038857901935, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, -4.24939971751113, 0, 3.46962034034307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.210280020626859, 0, -3.97354131880429, 0.732969916135155, 0, 0, 0, 0, 5.11038857901935, 1.44543616748915, 0, 0, 0, 0, 0, 0, 0, 0, -4.24939971751114, 2.00318623748287, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 1.8119501277094, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.2711894277325, 0, 0, -3.4552533206994, 0, 0, 0, 0, 0, 6.13246629482322, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.2499377798012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[56][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0.8, 0.2}, {0, 0.6, 0.4}, {0, 0.4, 0.6}, {0, 0.2, 0.8}, {0.8, 0, 0.2}, {0.6, 0, 0.4}, {0.4, 0, 0.6}, {0.2, 0, 0.8}, {0.8, 0.2, 0}, {0.6, 0.4, 0}, {0.4, 0.6, 0}, {0.2, 0.8, 0}, {0, 0, 0.2}, {0, 0, 0.4}, {0, 0, 0.6}, {0, 0, 0.8}, {0, 0.2, 0}, {0, 0.4, 0}, {0, 0.6, 0}, {0, 0.8, 0}, {0.2, 0, 0}, {0.4, 0, 0}, {0.6, 0, 0}, {0.8, 0, 0}, {0.6, 0.2, 0.2}, {0.4, 0.4, 0.2}, {0.2, 0.6, 0.2}, {0.4, 0.2, 0.4}, {0.2, 0.4, 0.4}, {0.2, 0.2, 0.6}, {0, 0.2, 0.2}, {0, 0.4, 0.2}, {0, 0.6, 0.2}, {0, 0.2, 0.4}, {0, 0.4, 0.4}, {0, 0.2, 0.6}, {0.2, 0, 0.2}, {0.4, 0, 0.2}, {0.6, 0, 0.2}, {0.2, 0, 0.4}, {0.4, 0, 0.4}, {0.2, 0, 0.6}, {0.2, 0.2, 0}, {0.4, 0.2, 0}, {0.6, 0.2, 0}, {0.2, 0.4, 0}, {0.4, 0.4, 0}, {0.2, 0.6, 0}, {0.2, 0.2, 0.2}, {0.4, 0.2, 0.2}, {0.2, 0.4, 0.2}, {0.2, 0.2, 0.4}};
    
    // Components for each dof
    static unsigned int components[56] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_finite_element_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5BilinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5BilinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_dof_map_1();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class UFC_Poisson3D_5BilinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    // Take absolute value of determinant
    detJ = std::abs(detJ);
    
    // Set scale factor
    const double det = detJ;
    
    // Compute geometry tensors
    const double G0_0_0 = det*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01 + Jinv_02*Jinv_02);
    const double G0_0_1 = det*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11 + Jinv_02*Jinv_12);
    const double G0_0_2 = det*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21 + Jinv_02*Jinv_22);
    const double G0_1_0 = det*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01 + Jinv_12*Jinv_02);
    const double G0_1_1 = det*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11 + Jinv_12*Jinv_12);
    const double G0_1_2 = det*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21 + Jinv_12*Jinv_22);
    const double G0_2_0 = det*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01 + Jinv_22*Jinv_02);
    const double G0_2_1 = det*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11 + Jinv_22*Jinv_12);
    const double G0_2_2 = det*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21 + Jinv_22*Jinv_22);
    
    // Compute element tensor
    A[0] = 0.0262652567340065*G0_0_0 + 0.0262652567340065*G0_0_1 + 0.0262652567340065*G0_0_2 + 0.0262652567340065*G0_1_0 + 0.0262652567340065*G0_1_1 + 0.0262652567340065*G0_1_2 + 0.0262652567340065*G0_2_0 + 0.0262652567340065*G0_2_1 + 0.0262652567340065*G0_2_2;
    A[1] = -0.00277890512265508*G0_0_0 - 0.00277890512265508*G0_1_0 - 0.00277890512265509*G0_2_0;
    A[2] = -0.00277890512265508*G0_0_1 - 0.00277890512265508*G0_1_1 - 0.00277890512265509*G0_2_1;
    A[3] = -0.00277890512265508*G0_0_2 - 0.00277890512265509*G0_1_2 - 0.00277890512265509*G0_2_2;
    A[4] = -0.00480034930989785*G0_0_1 - 0.0048003493098978*G0_0_2 - 0.00480034930989787*G0_1_1 - 0.00480034930989782*G0_1_2 - 0.00480034930989786*G0_2_1 - 0.00480034930989781*G0_2_2;
    A[5] = -0.00336507226698197*G0_0_1 - 0.00336507226698204*G0_0_2 - 0.00336507226698194*G0_1_1 - 0.00336507226698202*G0_1_2 - 0.00336507226698197*G0_2_1 - 0.00336507226698203*G0_2_2;
    A[6] = -0.00336507226698198*G0_0_1 - 0.00336507226698193*G0_0_2 - 0.00336507226698198*G0_1_1 - 0.00336507226698194*G0_1_2 - 0.00336507226698195*G0_2_1 - 0.00336507226698192*G0_2_2;
    A[7] = -0.00480034930989786*G0_0_1 - 0.00480034930989788*G0_0_2 - 0.00480034930989786*G0_1_1 - 0.00480034930989786*G0_1_2 - 0.00480034930989789*G0_2_1 - 0.00480034930989789*G0_2_2;
    A[8] = -0.00480034930989792*G0_0_0 - 0.00480034930989788*G0_0_2 - 0.00480034930989794*G0_1_0 - 0.0048003493098979*G0_1_2 - 0.00480034930989795*G0_2_0 - 0.00480034930989793*G0_2_2;
    A[9] = -0.00336507226698193*G0_0_0 - 0.00336507226698196*G0_0_2 - 0.00336507226698192*G0_1_0 - 0.00336507226698195*G0_1_2 - 0.00336507226698191*G0_2_0 - 0.00336507226698192*G0_2_2;
    A[10] = -0.00336507226698196*G0_0_0 - 0.00336507226698196*G0_0_2 - 0.00336507226698198*G0_1_0 - 0.00336507226698198*G0_1_2 - 0.00336507226698196*G0_2_0 - 0.00336507226698199*G0_2_2;
    A[11] = -0.00480034930989786*G0_0_0 - 0.00480034930989785*G0_0_2 - 0.00480034930989785*G0_1_0 - 0.00480034930989783*G0_1_2 - 0.00480034930989789*G0_2_0 - 0.00480034930989785*G0_2_2;
    A[12] = -0.00480034930989791*G0_0_0 - 0.00480034930989788*G0_0_1 - 0.00480034930989792*G0_1_0 - 0.0048003493098979*G0_1_1 - 0.00480034930989794*G0_2_0 - 0.00480034930989793*G0_2_1;
    A[13] = -0.00336507226698187*G0_0_0 - 0.00336507226698191*G0_0_1 - 0.00336507226698187*G0_1_0 - 0.00336507226698189*G0_1_1 - 0.00336507226698186*G0_2_0 - 0.00336507226698187*G0_2_1;
    A[14] = -0.00336507226698201*G0_0_0 - 0.00336507226698199*G0_0_1 - 0.00336507226698201*G0_1_0 - 0.00336507226698202*G0_1_1 - 0.00336507226698201*G0_2_0 - 0.00336507226698202*G0_2_1;
    A[15] = -0.00480034930989779*G0_0_0 - 0.00480034930989781*G0_0_1 - 0.00480034930989781*G0_1_0 - 0.00480034930989782*G0_1_1 - 0.00480034930989781*G0_2_0 - 0.0048003493098978*G0_2_1;
    A[16] = -0.0178705039816149*G0_0_0 - 0.0178705039816149*G0_0_1 - 0.0551276864210886*G0_0_2 - 0.0178705039816149*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0551276864210886*G0_1_2 - 0.0178705039816149*G0_2_0 - 0.0178705039816149*G0_2_1 - 0.0551276864210886*G0_2_2;
    A[17] = 0.0214691349673986*G0_0_0 + 0.0214691349673986*G0_0_1 + 0.0551276864210887*G0_0_2 + 0.0214691349673986*G0_1_0 + 0.0214691349673986*G0_1_1 + 0.0551276864210887*G0_1_2 + 0.0214691349673987*G0_2_0 + 0.0214691349673986*G0_2_1 + 0.0551276864210887*G0_2_2;
    A[18] = -0.0155544887532733*G0_0_0 - 0.0155544887532733*G0_0_1 - 0.0388177200243169*G0_0_2 - 0.0155544887532733*G0_1_0 - 0.0155544887532733*G0_1_1 - 0.038817720024317*G0_1_2 - 0.0155544887532733*G0_2_0 - 0.0155544887532733*G0_2_1 - 0.0388177200243169*G0_2_2;
    A[19] = 0.00480034930989785*G0_0_0 + 0.00480034930989785*G0_0_1 + 0.0153313684129655*G0_0_2 + 0.00480034930989784*G0_1_0 + 0.00480034930989784*G0_1_1 + 0.0153313684129655*G0_1_2 + 0.00480034930989787*G0_2_0 + 0.00480034930989787*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[20] = -0.0178705039816149*G0_0_0 - 0.0551276864210887*G0_0_1 - 0.0178705039816149*G0_0_2 - 0.0178705039816149*G0_1_0 - 0.0551276864210887*G0_1_1 - 0.0178705039816149*G0_1_2 - 0.0178705039816149*G0_2_0 - 0.0551276864210887*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[21] = 0.0214691349673986*G0_0_0 + 0.0551276864210887*G0_0_1 + 0.0214691349673986*G0_0_2 + 0.0214691349673986*G0_1_0 + 0.0551276864210887*G0_1_1 + 0.0214691349673987*G0_1_2 + 0.0214691349673986*G0_2_0 + 0.0551276864210887*G0_2_1 + 0.0214691349673986*G0_2_2;
    A[22] = -0.0155544887532732*G0_0_0 - 0.0388177200243169*G0_0_1 - 0.0155544887532732*G0_0_2 - 0.0155544887532733*G0_1_0 - 0.0388177200243168*G0_1_1 - 0.0155544887532733*G0_1_2 - 0.0155544887532733*G0_2_0 - 0.0388177200243168*G0_2_1 - 0.0155544887532733*G0_2_2;
    A[23] = 0.00480034930989778*G0_0_0 + 0.0153313684129655*G0_0_1 + 0.00480034930989778*G0_0_2 + 0.0048003493098978*G0_1_0 + 0.0153313684129655*G0_1_1 + 0.0048003493098978*G0_1_2 + 0.0048003493098978*G0_2_0 + 0.0153313684129655*G0_2_1 + 0.0048003493098978*G0_2_2;
    A[24] = -0.0551276864210886*G0_0_0 - 0.0178705039816148*G0_0_1 - 0.0178705039816148*G0_0_2 - 0.0551276864210887*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0178705039816148*G0_1_2 - 0.0551276864210887*G0_2_0 - 0.0178705039816149*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[25] = 0.0551276864210887*G0_0_0 + 0.0214691349673986*G0_0_1 + 0.0214691349673986*G0_0_2 + 0.0551276864210887*G0_1_0 + 0.0214691349673986*G0_1_1 + 0.0214691349673986*G0_1_2 + 0.0551276864210887*G0_2_0 + 0.0214691349673986*G0_2_1 + 0.0214691349673986*G0_2_2;
    A[26] = -0.0388177200243169*G0_0_0 - 0.0155544887532733*G0_0_1 - 0.0155544887532733*G0_0_2 - 0.0388177200243169*G0_1_0 - 0.0155544887532733*G0_1_1 - 0.0155544887532733*G0_1_2 - 0.0388177200243169*G0_2_0 - 0.0155544887532733*G0_2_1 - 0.0155544887532733*G0_2_2;
    A[27] = 0.0153313684129655*G0_0_0 + 0.0048003493098979*G0_0_1 + 0.0048003493098979*G0_0_2 + 0.0153313684129655*G0_1_0 + 0.00480034930989792*G0_1_1 + 0.00480034930989792*G0_1_2 + 0.0153313684129655*G0_2_0 + 0.00480034930989795*G0_2_1 + 0.00480034930989795*G0_2_2;
    A[28] = -0.0137421252872639*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872639*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.013742125287264*G0_1_2 - 0.0137421252872639*G0_2_0 - 0.0137421252872641*G0_2_1 - 0.013742125287264*G0_2_2;
    A[29] = -0.00946499969937454*G0_0_0 - 0.00946499969937446*G0_0_1 - 0.00946499969937451*G0_0_2 - 0.00946499969937458*G0_1_0 - 0.00946499969937447*G0_1_1 - 0.00946499969937453*G0_1_2 - 0.00946499969937458*G0_2_0 - 0.00946499969937447*G0_2_1 - 0.00946499969937451*G0_2_2;
    A[30] = -0.013742125287264*G0_0_0 - 0.013742125287264*G0_0_1 - 0.0137421252872639*G0_0_2 - 0.013742125287264*G0_1_0 - 0.013742125287264*G0_1_1 - 0.013742125287264*G0_1_2 - 0.013742125287264*G0_2_0 - 0.013742125287264*G0_2_1 - 0.0137421252872639*G0_2_2;
    A[31] = -0.00946499969937465*G0_0_0 - 0.00946499969937462*G0_0_1 - 0.00946499969937463*G0_0_2 - 0.00946499969937469*G0_1_0 - 0.00946499969937464*G0_1_1 - 0.00946499969937465*G0_1_2 - 0.00946499969937469*G0_2_0 - 0.00946499969937462*G0_2_1 - 0.00946499969937464*G0_2_2;
    A[32] = -0.00946499969937457*G0_0_0 - 0.00946499969937457*G0_0_1 - 0.00946499969937458*G0_0_2 - 0.00946499969937453*G0_1_0 - 0.00946499969937457*G0_1_1 - 0.00946499969937456*G0_1_2 - 0.00946499969937453*G0_2_0 - 0.00946499969937457*G0_2_1 - 0.00946499969937459*G0_2_2;
    A[33] = -0.0137421252872641*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.0137421252872641*G0_0_2 - 0.013742125287264*G0_1_0 - 0.013742125287264*G0_1_1 - 0.0137421252872641*G0_1_2 - 0.013742125287264*G0_2_0 - 0.0137421252872641*G0_2_1 - 0.013742125287264*G0_2_2;
    A[34] = 0.0479930547204847*G0_0_0 + 0.0335985307773499*G0_0_1 + 0.0335985307773498*G0_0_2 + 0.0479930547204847*G0_1_0 + 0.0335985307773499*G0_1_1 + 0.0335985307773499*G0_1_2 + 0.0479930547204847*G0_2_0 + 0.0335985307773499*G0_2_1 + 0.0335985307773499*G0_2_2;
    A[35] = -0.0345719095719093*G0_0_0 - 0.0298472385060924*G0_0_1 - 0.0298472385060925*G0_0_2 - 0.0345719095719093*G0_1_0 - 0.0298472385060925*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0345719095719093*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0298472385060925*G0_2_2;
    A[36] = 0.013742125287264*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.013742125287264*G0_1_0 + 0.0189195610202553*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.013742125287264*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[37] = -0.0345719095719092*G0_0_0 - 0.0298472385060924*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0345719095719092*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.0345719095719092*G0_2_0 - 0.0298472385060924*G0_2_1 - 0.0298472385060923*G0_2_2;
    A[38] = 0.00946499969937457*G0_0_0 + 0.0117431758056758*G0_0_1 + 0.0117431758056758*G0_0_2 + 0.00946499969937454*G0_1_0 + 0.0117431758056757*G0_1_1 + 0.0117431758056758*G0_1_2 + 0.00946499969937453*G0_2_0 + 0.0117431758056758*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[39] = 0.013742125287264*G0_0_0 + 0.0189195610202552*G0_0_1 + 0.0189195610202552*G0_0_2 + 0.013742125287264*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202551*G0_1_2 + 0.013742125287264*G0_2_0 + 0.0189195610202552*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[40] = 0.0335985307773498*G0_0_0 + 0.0479930547204846*G0_0_1 + 0.0335985307773497*G0_0_2 + 0.0335985307773498*G0_1_0 + 0.0479930547204846*G0_1_1 + 0.0335985307773497*G0_1_2 + 0.0335985307773498*G0_2_0 + 0.0479930547204846*G0_2_1 + 0.0335985307773497*G0_2_2;
    A[41] = -0.0298472385060923*G0_0_0 - 0.0345719095719091*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.0345719095719091*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.0298472385060923*G0_2_0 - 0.0345719095719091*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[42] = 0.0189195610202553*G0_0_0 + 0.0137421252872641*G0_0_1 + 0.0189195610202552*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0137421252872641*G0_1_1 + 0.0189195610202552*G0_1_2 + 0.0189195610202554*G0_2_0 + 0.0137421252872641*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[43] = -0.0298472385060922*G0_0_0 - 0.0345719095719089*G0_0_1 - 0.0298472385060921*G0_0_2 - 0.0298472385060922*G0_1_0 - 0.034571909571909*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.034571909571909*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[44] = 0.0117431758056756*G0_0_0 + 0.00946499969937466*G0_0_1 + 0.0117431758056756*G0_0_2 + 0.0117431758056756*G0_1_0 + 0.00946499969937467*G0_1_1 + 0.0117431758056757*G0_1_2 + 0.0117431758056756*G0_2_0 + 0.00946499969937465*G0_2_1 + 0.0117431758056756*G0_2_2;
    A[45] = 0.0189195610202552*G0_0_0 + 0.013742125287264*G0_0_1 + 0.0189195610202551*G0_0_2 + 0.0189195610202552*G0_1_0 + 0.013742125287264*G0_1_1 + 0.0189195610202551*G0_1_2 + 0.0189195610202552*G0_2_0 + 0.013742125287264*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[46] = 0.0335985307773497*G0_0_0 + 0.0335985307773497*G0_0_1 + 0.0479930547204846*G0_0_2 + 0.0335985307773497*G0_1_0 + 0.0335985307773497*G0_1_1 + 0.0479930547204847*G0_1_2 + 0.0335985307773497*G0_2_0 + 0.0335985307773497*G0_2_1 + 0.0479930547204846*G0_2_2;
    A[47] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0345719095719091*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.0298472385060923*G0_1_1 - 0.0345719095719091*G0_1_2 - 0.0298472385060923*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.0345719095719091*G0_2_2;
    A[48] = 0.0189195610202553*G0_0_0 + 0.0189195610202552*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.013742125287264*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202552*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[49] = -0.0298472385060923*G0_0_0 - 0.0298472385060922*G0_0_1 - 0.034571909571909*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.0298472385060923*G0_1_1 - 0.0345719095719091*G0_1_2 - 0.0298472385060923*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.0345719095719091*G0_2_2;
    A[50] = 0.0117431758056755*G0_0_0 + 0.0117431758056757*G0_0_1 + 0.0094649996993745*G0_0_2 + 0.0117431758056755*G0_1_0 + 0.0117431758056757*G0_1_1 + 0.00946499969937451*G0_1_2 + 0.0117431758056755*G0_2_0 + 0.0117431758056757*G0_2_1 + 0.00946499969937449*G0_2_2;
    A[51] = 0.0189195610202553*G0_0_0 + 0.0189195610202551*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.013742125287264*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202551*G0_2_1 + 0.013742125287264*G0_2_2;
    A[52] = -0.0782878387045041*G0_0_0 - 0.0782878387045042*G0_0_1 - 0.0782878387045043*G0_0_2 - 0.0782878387045041*G0_1_0 - 0.0782878387045042*G0_1_1 - 0.0782878387045043*G0_1_2 - 0.0782878387045041*G0_2_0 - 0.0782878387045043*G0_2_1 - 0.0782878387045042*G0_2_2;
    A[53] = 0.0440369092712833*G0_0_0 + 0.0440369092712835*G0_0_1 + 0.0440369092712837*G0_0_2 + 0.0440369092712834*G0_1_0 + 0.0440369092712835*G0_1_1 + 0.0440369092712837*G0_1_2 + 0.0440369092712833*G0_2_0 + 0.0440369092712835*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[54] = 0.0440369092712838*G0_0_0 + 0.0440369092712837*G0_0_1 + 0.0440369092712836*G0_0_2 + 0.0440369092712838*G0_1_0 + 0.0440369092712836*G0_1_1 + 0.0440369092712836*G0_1_2 + 0.0440369092712838*G0_2_0 + 0.0440369092712837*G0_2_1 + 0.0440369092712836*G0_2_2;
    A[55] = 0.0440369092712839*G0_0_0 + 0.0440369092712836*G0_0_1 + 0.0440369092712837*G0_0_2 + 0.0440369092712839*G0_1_0 + 0.0440369092712836*G0_1_1 + 0.0440369092712837*G0_1_2 + 0.0440369092712839*G0_2_0 + 0.0440369092712836*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[56] = -0.00277890512265508*G0_0_0 - 0.00277890512265508*G0_0_1 - 0.00277890512265509*G0_0_2;
    A[57] = 0.0262652567340065*G0_0_0;
    A[58] = 0.00277890512265508*G0_0_1;
    A[59] = 0.0027789051226551*G0_0_2;
    A[60] = 0.00480034930989784*G0_0_1 + 0.0048003493098978*G0_0_2;
    A[61] = 0.00336507226698203*G0_0_1 + 0.00336507226698205*G0_0_2;
    A[62] = 0.00336507226698198*G0_0_1 + 0.00336507226698199*G0_0_2;
    A[63] = 0.00480034930989783*G0_0_1 + 0.00480034930989783*G0_0_2;
    A[64] = -0.0178705039816151*G0_0_0 + 0.0372571824394736*G0_0_2;
    A[65] = 0.0214691349673988*G0_0_0 - 0.03365855145369*G0_0_2;
    A[66] = -0.0155544887532734*G0_0_0 + 0.0232632312710436*G0_0_2;
    A[67] = 0.00480034930989782*G0_0_0 - 0.0105310191030677*G0_0_2;
    A[68] = -0.017870503981615*G0_0_0 + 0.0372571824394737*G0_0_1;
    A[69] = 0.0214691349673987*G0_0_0 - 0.0336585514536899*G0_0_1;
    A[70] = -0.0155544887532733*G0_0_0 + 0.0232632312710435*G0_0_1;
    A[71] = 0.0048003493098978*G0_0_0 - 0.0105310191030676*G0_0_1;
    A[72] = -0.00480034930989792*G0_0_0 - 0.00480034930989793*G0_0_1;
    A[73] = -0.00336507226698196*G0_0_0 - 0.00336507226698196*G0_0_1;
    A[74] = -0.00336507226698205*G0_0_0 - 0.00336507226698206*G0_0_1;
    A[75] = -0.00480034930989783*G0_0_0 - 0.00480034930989783*G0_0_1;
    A[76] = -0.00480034930989792*G0_0_0 - 0.00480034930989793*G0_0_2;
    A[77] = -0.00336507226698191*G0_0_0 - 0.00336507226698189*G0_0_2;
    A[78] = -0.00336507226698209*G0_0_0 - 0.0033650722669821*G0_0_2;
    A[79] = -0.00480034930989781*G0_0_0 - 0.0048003493098978*G0_0_2;
    A[80] = 0.0153313684129654*G0_0_0 + 0.0105310191030675*G0_0_1 + 0.0105310191030675*G0_0_2;
    A[81] = -0.0388177200243168*G0_0_0 - 0.0232632312710434*G0_0_1 - 0.0232632312710434*G0_0_2;
    A[82] = 0.0551276864210885*G0_0_0 + 0.0336585514536899*G0_0_1 + 0.0336585514536899*G0_0_2;
    A[83] = -0.0551276864210886*G0_0_0 - 0.0372571824394737*G0_0_1 - 0.0372571824394737*G0_0_2;
    A[84] = 0.047993054720485*G0_0_0 + 0.0143945239431347*G0_0_1 + 0.0143945239431349*G0_0_2;
    A[85] = -0.0345719095719093*G0_0_0 - 0.00472467106581669*G0_0_1 - 0.0047246710658168*G0_0_2;
    A[86] = 0.0137421252872641*G0_0_0 - 0.00517743573299126*G0_0_1 - 0.00517743573299132*G0_0_2;
    A[87] = -0.0345719095719093*G0_0_0 - 0.00472467106581676*G0_0_1 - 0.00472467106581678*G0_0_2;
    A[88] = 0.00946499969937471*G0_0_0 - 0.00227817610630112*G0_0_1 - 0.00227817610630103*G0_0_2;
    A[89] = 0.0137421252872639*G0_0_0 - 0.00517743573299147*G0_0_1 - 0.00517743573299141*G0_0_2;
    A[90] = -0.0137421252872641*G0_0_0;
    A[91] = -0.00946499969937446*G0_0_0;
    A[92] = -0.0137421252872641*G0_0_0;
    A[93] = -0.00946499969937471*G0_0_0;
    A[94] = -0.00946499969937473*G0_0_0;
    A[95] = -0.0137421252872639*G0_0_0;
    A[96] = 0.0189195610202554*G0_0_0 + 0.00517743573299117*G0_0_1;
    A[97] = -0.0298472385060925*G0_0_0 + 0.00472467106581687*G0_0_1;
    A[98] = 0.0335985307773502*G0_0_0 - 0.0143945239431349*G0_0_1;
    A[99] = 0.0117431758056757*G0_0_0 + 0.00227817610630087*G0_0_1;
    A[100] = -0.0298472385060925*G0_0_0 + 0.00472467106581678*G0_0_1;
    A[101] = 0.0189195610202554*G0_0_0 + 0.00517743573299146*G0_0_1;
    A[102] = 0.0189195610202555*G0_0_0 + 0.00517743573299125*G0_0_2;
    A[103] = -0.0298472385060926*G0_0_0 + 0.0047246710658169*G0_0_2;
    A[104] = 0.0335985307773501*G0_0_0 - 0.014394523943135*G0_0_2;
    A[105] = 0.0117431758056757*G0_0_0 + 0.00227817610630111*G0_0_2;
    A[106] = -0.0298472385060925*G0_0_0 + 0.00472467106581678*G0_0_2;
    A[107] = 0.0189195610202554*G0_0_0 + 0.00517743573299133*G0_0_2;
    A[108] = 0.0440369092712839*G0_0_0;
    A[109] = -0.0782878387045048*G0_0_0;
    A[110] = 0.0440369092712838*G0_0_0;
    A[111] = 0.0440369092712841*G0_0_0;
    A[112] = -0.00277890512265508*G0_1_0 - 0.00277890512265508*G0_1_1 - 0.00277890512265509*G0_1_2;
    A[113] = 0.00277890512265507*G0_1_0;
    A[114] = 0.0262652567340065*G0_1_1;
    A[115] = 0.0027789051226551*G0_1_2;
    A[116] = -0.017870503981615*G0_1_1 + 0.0372571824394738*G0_1_2;
    A[117] = 0.0214691349673988*G0_1_1 - 0.03365855145369*G0_1_2;
    A[118] = -0.0155544887532734*G0_1_1 + 0.0232632312710436*G0_1_2;
    A[119] = 0.00480034930989792*G0_1_1 - 0.0105310191030676*G0_1_2;
    A[120] = 0.00480034930989786*G0_1_0 + 0.00480034930989784*G0_1_2;
    A[121] = 0.00336507226698202*G0_1_0 + 0.00336507226698207*G0_1_2;
    A[122] = 0.00336507226698191*G0_1_0 + 0.00336507226698189*G0_1_2;
    A[123] = 0.00480034930989792*G0_1_0 + 0.00480034930989793*G0_1_2;
    A[124] = -0.0105310191030676*G0_1_0 + 0.00480034930989782*G0_1_1;
    A[125] = 0.0232632312710435*G0_1_0 - 0.0155544887532733*G0_1_1;
    A[126] = -0.03365855145369*G0_1_0 + 0.0214691349673986*G0_1_1;
    A[127] = 0.0372571824394738*G0_1_0 - 0.0178705039816149*G0_1_1;
    A[128] = -0.00480034930989784*G0_1_0 - 0.00480034930989784*G0_1_1;
    A[129] = -0.00336507226698207*G0_1_0 - 0.00336507226698206*G0_1_1;
    A[130] = -0.00336507226698189*G0_1_0 - 0.00336507226698189*G0_1_1;
    A[131] = -0.00480034930989792*G0_1_0 - 0.00480034930989792*G0_1_1;
    A[132] = 0.0105310191030677*G0_1_0 + 0.0153313684129655*G0_1_1 + 0.0105310191030677*G0_1_2;
    A[133] = -0.0232632312710436*G0_1_0 - 0.0388177200243169*G0_1_1 - 0.0232632312710436*G0_1_2;
    A[134] = 0.0336585514536901*G0_1_0 + 0.0551276864210887*G0_1_1 + 0.0336585514536901*G0_1_2;
    A[135] = -0.0372571824394738*G0_1_0 - 0.0551276864210887*G0_1_1 - 0.0372571824394738*G0_1_2;
    A[136] = -0.0048003493098978*G0_1_1 - 0.00480034930989779*G0_1_2;
    A[137] = -0.00336507226698194*G0_1_1 - 0.00336507226698194*G0_1_2;
    A[138] = -0.00336507226698191*G0_1_1 - 0.00336507226698191*G0_1_2;
    A[139] = -0.00480034930989783*G0_1_1 - 0.00480034930989783*G0_1_2;
    A[140] = -0.00517743573299126*G0_1_0 + 0.0137421252872641*G0_1_1 - 0.0051774357329913*G0_1_2;
    A[141] = -0.0047246710658168*G0_1_0 - 0.0345719095719093*G0_1_1 - 0.00472467106581676*G0_1_2;
    A[142] = 0.0143945239431349*G0_1_0 + 0.0479930547204849*G0_1_1 + 0.0143945239431348*G0_1_2;
    A[143] = -0.00227817610630113*G0_1_0 + 0.00946499969937467*G0_1_1 - 0.00227817610630111*G0_1_2;
    A[144] = -0.00472467106581671*G0_1_0 - 0.0345719095719093*G0_1_1 - 0.00472467106581673*G0_1_2;
    A[145] = -0.00517743573299126*G0_1_0 + 0.013742125287264*G0_1_1 - 0.0051774357329913*G0_1_2;
    A[146] = 0.00517743573299132*G0_1_0 + 0.0189195610202553*G0_1_1;
    A[147] = 0.00472467106581694*G0_1_0 - 0.0298472385060924*G0_1_1;
    A[148] = -0.0143945239431349*G0_1_0 + 0.03359853077735*G0_1_1;
    A[149] = 0.0022781761063011*G0_1_0 + 0.0117431758056758*G0_1_1;
    A[150] = 0.00472467106581671*G0_1_0 - 0.0298472385060925*G0_1_1;
    A[151] = 0.00517743573299127*G0_1_0 + 0.0189195610202553*G0_1_1;
    A[152] = -0.013742125287264*G0_1_1;
    A[153] = -0.00946499969937448*G0_1_1;
    A[154] = -0.0137421252872641*G0_1_1;
    A[155] = -0.00946499969937467*G0_1_1;
    A[156] = -0.00946499969937468*G0_1_1;
    A[157] = -0.013742125287264*G0_1_1;
    A[158] = 0.0189195610202553*G0_1_1 + 0.00517743573299141*G0_1_2;
    A[159] = 0.0117431758056756*G0_1_1 + 0.00227817610630113*G0_1_2;
    A[160] = 0.0189195610202552*G0_1_1 + 0.00517743573299129*G0_1_2;
    A[161] = -0.0298472385060925*G0_1_1 + 0.00472467106581678*G0_1_2;
    A[162] = -0.0298472385060924*G0_1_1 + 0.00472467106581678*G0_1_2;
    A[163] = 0.03359853077735*G0_1_1 - 0.0143945239431348*G0_1_2;
    A[164] = 0.0440369092712838*G0_1_1;
    A[165] = 0.0440369092712838*G0_1_1;
    A[166] = -0.0782878387045047*G0_1_1;
    A[167] = 0.044036909271284*G0_1_1;
    A[168] = -0.00277890512265508*G0_2_0 - 0.00277890512265508*G0_2_1 - 0.00277890512265509*G0_2_2;
    A[169] = 0.0027789051226551*G0_2_0;
    A[170] = 0.0027789051226551*G0_2_1;
    A[171] = 0.0262652567340065*G0_2_2;
    A[172] = -0.0105310191030676*G0_2_1 + 0.00480034930989794*G0_2_2;
    A[173] = 0.0232632312710435*G0_2_1 - 0.0155544887532734*G0_2_2;
    A[174] = -0.03365855145369*G0_2_1 + 0.0214691349673988*G0_2_2;
    A[175] = 0.0372571824394738*G0_2_1 - 0.0178705039816151*G0_2_2;
    A[176] = -0.0105310191030675*G0_2_0 + 0.00480034930989798*G0_2_2;
    A[177] = 0.0232632312710435*G0_2_0 - 0.0155544887532736*G0_2_2;
    A[178] = -0.03365855145369*G0_2_0 + 0.0214691349673988*G0_2_2;
    A[179] = 0.0372571824394738*G0_2_0 - 0.0178705039816151*G0_2_2;
    A[180] = 0.004800349309898*G0_2_0 + 0.00480034930989798*G0_2_1;
    A[181] = 0.00336507226698197*G0_2_0 + 0.00336507226698197*G0_2_1;
    A[182] = 0.00336507226698199*G0_2_0 + 0.003365072266982*G0_2_1;
    A[183] = 0.00480034930989792*G0_2_0 + 0.0048003493098979*G0_2_1;
    A[184] = 0.0105310191030676*G0_2_0 + 0.0105310191030676*G0_2_1 + 0.0153313684129654*G0_2_2;
    A[185] = -0.0232632312710434*G0_2_0 - 0.0232632312710434*G0_2_1 - 0.0388177200243168*G0_2_2;
    A[186] = 0.03365855145369*G0_2_0 + 0.03365855145369*G0_2_1 + 0.0551276864210887*G0_2_2;
    A[187] = -0.0372571824394738*G0_2_0 - 0.0372571824394738*G0_2_1 - 0.0551276864210888*G0_2_2;
    A[188] = -0.00480034930989788*G0_2_0 - 0.00480034930989789*G0_2_2;
    A[189] = -0.00336507226698204*G0_2_0 - 0.00336507226698204*G0_2_2;
    A[190] = -0.00336507226698192*G0_2_0 - 0.00336507226698191*G0_2_2;
    A[191] = -0.00480034930989793*G0_2_0 - 0.00480034930989792*G0_2_2;
    A[192] = -0.00480034930989786*G0_2_1 - 0.00480034930989787*G0_2_2;
    A[193] = -0.00336507226698201*G0_2_1 - 0.00336507226698199*G0_2_2;
    A[194] = -0.00336507226698194*G0_2_1 - 0.00336507226698195*G0_2_2;
    A[195] = -0.00480034930989797*G0_2_1 - 0.00480034930989798*G0_2_2;
    A[196] = -0.00517743573299137*G0_2_0 - 0.00517743573299122*G0_2_1 + 0.0137421252872643*G0_2_2;
    A[197] = -0.00227817610630093*G0_2_0 - 0.00227817610630105*G0_2_1 + 0.0094649996993747*G0_2_2;
    A[198] = -0.00517743573299124*G0_2_0 - 0.0051774357329912*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[199] = -0.0047246710658169*G0_2_0 - 0.00472467106581705*G0_2_1 - 0.0345719095719096*G0_2_2;
    A[200] = -0.00472467106581683*G0_2_0 - 0.00472467106581673*G0_2_1 - 0.0345719095719092*G0_2_2;
    A[201] = 0.0143945239431349*G0_2_0 + 0.0143945239431349*G0_2_1 + 0.0479930547204849*G0_2_2;
    A[202] = 0.00517743573299131*G0_2_0 + 0.0189195610202554*G0_2_2;
    A[203] = 0.00227817610630113*G0_2_0 + 0.0117431758056757*G0_2_2;
    A[204] = 0.00517743573299125*G0_2_0 + 0.0189195610202553*G0_2_2;
    A[205] = 0.00472467106581673*G0_2_0 - 0.0298472385060927*G0_2_2;
    A[206] = 0.00472467106581683*G0_2_0 - 0.0298472385060924*G0_2_2;
    A[207] = -0.0143945239431349*G0_2_0 + 0.0335985307773502*G0_2_2;
    A[208] = 0.0051774357329913*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[209] = 0.00227817610630117*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[210] = 0.00517743573299119*G0_2_1 + 0.0189195610202555*G0_2_2;
    A[211] = 0.00472467106581668*G0_2_1 - 0.0298472385060926*G0_2_2;
    A[212] = 0.00472467106581704*G0_2_1 - 0.0298472385060925*G0_2_2;
    A[213] = -0.0143945239431349*G0_2_1 + 0.0335985307773501*G0_2_2;
    A[214] = -0.0137421252872642*G0_2_2;
    A[215] = -0.00946499969937462*G0_2_2;
    A[216] = -0.0137421252872643*G0_2_2;
    A[217] = -0.00946499969937459*G0_2_2;
    A[218] = -0.00946499969937472*G0_2_2;
    A[219] = -0.0137421252872642*G0_2_2;
    A[220] = 0.0440369092712839*G0_2_2;
    A[221] = 0.0440369092712842*G0_2_2;
    A[222] = 0.0440369092712838*G0_2_2;
    A[223] = -0.0782878387045048*G0_2_2;
    A[224] = -0.00480034930989786*G0_1_0 - 0.00480034930989787*G0_1_1 - 0.00480034930989786*G0_1_2 - 0.0048003493098978*G0_2_0 - 0.00480034930989782*G0_2_1 - 0.00480034930989781*G0_2_2;
    A[225] = 0.00480034930989784*G0_1_0 + 0.0048003493098978*G0_2_0;
    A[226] = -0.017870503981615*G0_1_1 + 0.0372571824394738*G0_2_1;
    A[227] = -0.0105310191030676*G0_1_2 + 0.00480034930989794*G0_2_2;
    A[228] = 0.179096479009672*G0_1_1 + 0.0699110399631223*G0_1_2 + 0.0699110399631223*G0_2_1 + 0.139822079926245*G0_2_2;
    A[229] = -0.147063705006412*G0_1_1 + 0.0263830016434185*G0_1_2 - 0.0387263842124946*G0_2_1 - 0.108637424175617*G0_2_2;
    A[230] = 0.100717304493345*G0_1_1 - 0.0246345732456845*G0_1_2 + 0.0189782768992833*G0_2_1 + 0.0788945694544644*G0_2_2;
    A[231] = -0.0328352243499811*G0_1_1 + 0.02091590090722*G0_1_2 - 0.00537576492437587*G0_2_1 - 0.0328352243499812*G0_2_2;
    A[232] = -0.0107515298487518*G0_1_0 - 0.0053757649243759*G0_1_2 - 0.00537576492437585*G0_2_0 - 0.00537576492437594*G0_2_2;
    A[233] = -0.0139743792087543*G0_1_0 - 0.0116844599266474*G0_1_2 - 0.0022899192821068*G0_2_0 - 0.00228991928210663*G0_2_2;
    A[234] = -0.011169064988509*G0_1_0 - 0.00887914570640231*G0_1_2 - 0.00228991928210675*G0_2_0 - 0.00228991928210672*G0_2_2;
    A[235] = -0.0328352243499812*G0_1_0 - 0.0274594594256052*G0_1_2 - 0.00537576492437589*G0_2_0 - 0.005375764924376*G0_2_2;
    A[236] = -0.0274594594256049*G0_1_0 - 0.00537576492437588*G0_1_1 - 0.0328352243499807*G0_2_0 - 0.00537576492437596*G0_2_1;
    A[237] = 0.0599162925551808*G0_1_0 + 0.0283728175438241*G0_1_1 + 0.0788945694544643*G0_2_0 + 0.0189782768992836*G0_2_1;
    A[238] = -0.0699110399631223*G0_1_0 - 0.0574176057031951*G0_1_1 - 0.108637424175617*G0_2_0 - 0.0387263842124947*G0_2_1;
    A[239] = 0.0699110399631224*G0_1_0 + 0.0895482395048356*G0_1_1 + 0.139822079926245*G0_2_0 + 0.0699110399631224*G0_2_1;
    A[240] = 0.0107515298487519*G0_1_0 + 0.010751529848752*G0_1_1 + 0.00537576492437602*G0_1_2 + 0.00537576492437602*G0_2_0 + 0.00537576492437608*G0_2_1;
    A[241] = 0.0139743792087545*G0_1_0 + 0.0139743792087544*G0_1_1 + 0.00228991928210673*G0_1_2 + 0.00228991928210681*G0_2_0 + 0.00228991928210674*G0_2_1;
    A[242] = 0.0111690649885091*G0_1_0 + 0.0111690649885092*G0_1_1 + 0.00228991928210686*G0_1_2 + 0.0022899192821068*G0_2_0 + 0.00228991928210683*G0_2_1;
    A[243] = 0.0328352243499812*G0_1_0 + 0.0328352243499812*G0_1_1 + 0.00537576492437587*G0_1_2 + 0.00537576492437589*G0_2_0 + 0.0053757649243759*G0_2_1;
    A[244] = 0.0274594594256048*G0_1_0 + 0.022083694501229*G0_1_1 + 0.0274594594256048*G0_1_2 + 0.0328352243499809*G0_2_0 + 0.0274594594256048*G0_2_1 + 0.0328352243499809*G0_2_2;
    A[245] = -0.0599162925551808*G0_1_0 - 0.0315434750113567*G0_1_1 - 0.0599162925551809*G0_1_2 - 0.0788945694544645*G0_2_0 - 0.0599162925551808*G0_2_1 - 0.0788945694544646*G0_2_2;
    A[246] = 0.0699110399631225*G0_1_0 + 0.0124934342599273*G0_1_1 + 0.0699110399631225*G0_1_2 + 0.108637424175618*G0_2_0 + 0.0699110399631227*G0_2_1 + 0.108637424175618*G0_2_2;
    A[247] = -0.0699110399631224*G0_1_0 + 0.0196371995417132*G0_1_1 - 0.0699110399631224*G0_1_2 - 0.139822079926245*G0_2_0 - 0.0699110399631228*G0_2_1 - 0.139822079926245*G0_2_2;
    A[248] = 0.00537576492437577*G0_1_1 + 0.00537576492437575*G0_1_2 + 0.00537576492437607*G0_2_1 + 0.00537576492437604*G0_2_2;
    A[249] = 0.00228991928210674*G0_1_1 + 0.00228991928210677*G0_1_2 + 0.00228991928210673*G0_2_1 + 0.00228991928210677*G0_2_2;
    A[250] = 0.00228991928210667*G0_1_1 + 0.00228991928210667*G0_1_2 + 0.00228991928210687*G0_2_1 + 0.00228991928210687*G0_2_2;
    A[251] = 0.00537576492437596*G0_1_1 + 0.0053757649243759*G0_1_2 + 0.00537576492437602*G0_2_1 + 0.00537576492437596*G0_2_2;
    A[252] = 0.0425363923627808*G0_1_0 - 0.0187890812890812*G0_1_1 + 0.0212681961813905*G0_1_2 + 0.0118736555368496*G0_2_0 - 0.00939454064454054*G0_2_1 + 0.01187365553685*G0_2_2;
    A[253] = -0.0845508658008648*G0_1_0 + 0.043971669405697*G0_1_1 - 0.0422754329004322*G0_1_2 - 0.0429604514890965*G0_2_0 - 0.000685018588664194*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[254] = 0.192588083213081*G0_1_0 - 0.0644569872000425*G0_1_1 + 0.0962940416065402*G0_1_2 + 0.12473862300251*G0_2_0 + 0.0284445813959699*G0_2_1 + 0.12473862300251*G0_2_2;
    A[255] = 0.0142875305635729*G0_1_0 - 0.0292600797158081*G0_1_1 + 0.00844856259352797*G0_1_2 + 0.00254435475789696*G0_2_0 - 0.00329461321214759*G0_2_1 + 0.00254435475789638*G0_2_2;
    A[256] = -0.0493213383838383*G0_1_0 + 0.1273155976932*G0_1_1 - 0.00704590548340569*G0_1_2 - 0.0429604514890968*G0_2_0 - 0.000685018588664536*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[257] = 0.0477555816097474*G0_1_0 - 0.0641307878721068*G0_1_1 + 0.0264873854283577*G0_1_2 + 0.0118736555368495*G0_2_0 - 0.00939454064454059*G0_2_1 + 0.01187365553685*G0_2_2;
    A[258] = -0.0425363923627807*G0_1_0 - 0.0613254736518622*G0_1_1 - 0.02126819618139*G0_1_2 - 0.0118736555368496*G0_2_0 - 0.0212681961813906*G0_2_1;
    A[259] = 0.0845508658008647*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0429604514890967*G0_2_0 + 0.0422754329004327*G0_2_1;
    A[260] = -0.192588083213081*G0_1_0 - 0.257045070413124*G0_1_1 - 0.0962940416065408*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0962940416065406*G0_2_1;
    A[261] = -0.0142875305635726*G0_1_0 - 0.0435476102793813*G0_1_1 - 0.00583896797004512*G0_1_2 - 0.00254435475789679*G0_2_0 - 0.00583896797004446*G0_2_1;
    A[262] = 0.0493213383838383*G0_1_0 + 0.176636936077039*G0_1_1 + 0.0422754329004327*G0_1_2 + 0.0429604514890968*G0_2_0 + 0.0422754329004324*G0_2_1;
    A[263] = -0.0477555816097473*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0212681961813898*G0_1_2 - 0.0118736555368495*G0_2_0 - 0.0212681961813902*G0_2_1;
    A[264] = 0.0187890812890814*G0_1_1 + 0.00939454064454121*G0_1_2 + 0.00939454064454083*G0_2_1;
    A[265] = 0.00658922642429554*G0_1_1 + 0.00329461321214781*G0_1_2 + 0.00329461321214796*G0_2_1;
    A[266] = 0.0187890812890811*G0_1_1 + 0.00939454064454076*G0_1_2 + 0.00939454064454043*G0_2_1;
    A[267] = 0.0292600797158086*G0_1_1 + 0.00329461321214707*G0_1_2 + 0.00329461321214784*G0_2_1;
    A[268] = 0.0292600797158082*G0_1_1 + 0.00329461321214766*G0_1_2 + 0.00329461321214768*G0_2_1;
    A[269] = 0.0641307878721068*G0_1_1 + 0.00939454064454119*G0_1_2 + 0.00939454064454057*G0_2_1;
    A[270] = -0.0306627368259307*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.0118736555368496*G0_2_2;
    A[271] = -0.00913358118219214*G0_1_1 - 0.00583896797004455*G0_1_2 - 0.00583896797004428*G0_2_1 - 0.00254435475789646*G0_2_2;
    A[272] = -0.0306627368259309*G0_1_1 - 0.0212681961813906*G0_1_2 - 0.0212681961813906*G0_2_1 - 0.01187365553685*G0_2_2;
    A[273] = 0.0642612676032807*G0_1_1 + 0.0422754329004322*G0_1_2 + 0.0422754329004321*G0_2_1 + 0.0429604514890965*G0_2_2;
    A[274] = 0.0642612676032806*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[275] = -0.128522535206561*G0_1_1 - 0.0962940416065403*G0_1_2 - 0.09629404160654*G0_2_1 - 0.12473862300251*G0_2_2;
    A[276] = -0.0505608958299935*G0_1_1 - 0.00260959462348341*G0_1_2 - 0.00260959462348446*G0_2_1;
    A[277] = -0.0505608958299925*G0_1_1 - 0.00260959462348339*G0_1_2 - 0.00260959462348371*G0_2_1;
    A[278] = 0.0962288017409544*G0_1_1 - 0.0352295274170267*G0_1_2 - 0.0352295274170263*G0_2_1;
    A[279] = -0.156575677409009*G0_1_1 - 0.00260959462348363*G0_1_2 - 0.00260959462348347*G0_2_1;
    A[280] = -0.00336507226698197*G0_1_0 - 0.00336507226698194*G0_1_1 - 0.00336507226698197*G0_1_2 - 0.00336507226698205*G0_2_0 - 0.00336507226698202*G0_2_1 - 0.00336507226698203*G0_2_2;
    A[281] = 0.00336507226698203*G0_1_0 + 0.00336507226698205*G0_2_0;
    A[282] = 0.0214691349673988*G0_1_1 - 0.03365855145369*G0_2_1;
    A[283] = 0.0232632312710435*G0_1_2 - 0.0155544887532734*G0_2_2;
    A[284] = -0.147063705006412*G0_1_1 - 0.0387263842124946*G0_1_2 + 0.0263830016434185*G0_2_1 - 0.108637424175617*G0_2_2;
    A[285] = 0.304226541205705*G0_1_1 + 0.0781312630270952*G0_1_2 + 0.0781312630270952*G0_2_1 + 0.281001149056702*G0_2_2;
    A[286] = -0.204187731314465*G0_1_1 + 0.0153052724667312*G0_1_2 - 0.0364951808094161*G0_2_1 - 0.204187731314466*G0_2_2;
    A[287] = 0.0788945694544646*G0_1_1 - 0.0246345732456839*G0_1_2 + 0.0189782768992834*G0_2_1 + 0.100717304493345*G0_2_2;
    A[288] = -0.013974379208754*G0_1_0 - 0.0022899192821065*G0_1_2 - 0.0116844599266473*G0_2_0 - 0.00228991928210663*G0_2_2;
    A[289] = 0.0263047138047134*G0_1_0 + 0.0131523569023564*G0_1_2 + 0.0131523569023566*G0_2_0 + 0.0132175967679434*G0_2_2;
    A[290] = 0.00523223722008437*G0_1_0 + 0.00578025209101598*G0_1_2 - 0.000548014870931182*G0_2_0 - 0.000482775005344208*G0_2_2;
    A[291] = 0.0788945694544647*G0_1_0 + 0.059916292555181*G0_1_2 + 0.0189782768992834*G0_2_0 + 0.0283728175438242*G0_2_2;
    A[292] = -0.00887914570640257*G0_1_0 - 0.00228991928210658*G0_1_1 - 0.0111690649885096*G0_2_0 - 0.00228991928210669*G0_2_1;
    A[293] = 0.00578025209101613*G0_1_0 - 0.000482775005344528*G0_1_1 + 0.00523223722008479*G0_2_0 - 0.000548014870931567*G0_2_1;
    A[294] = 0.00480165410720946*G0_1_0 + 0.00506261356955802*G0_1_1 + 0.00960330821441898*G0_2_0 + 0.00480165410720958*G0_2_1;
    A[295] = -0.0387263842124946*G0_1_0 - 0.0574176057031953*G0_1_1 - 0.108637424175617*G0_2_0 - 0.0699110399631225*G0_2_1;
    A[296] = 0.0139743792087542*G0_1_0 + 0.0139743792087541*G0_1_1 + 0.0116844599266473*G0_1_2 + 0.0116844599266473*G0_2_0 + 0.0116844599266472*G0_2_1 + 0.00939454064454072*G0_2_2;
    A[297] = -0.0263047138047137*G0_1_0 - 0.0263047138047136*G0_1_1 - 0.0131523569023567*G0_1_2 - 0.0131523569023565*G0_2_0 - 0.0131523569023564*G0_2_1 + 6.52398655869178e-05*G0_2_2;
    A[298] = -0.00523223722008433*G0_1_0 - 0.00523223722008439*G0_1_1 + 0.000548014870931335*G0_1_2 + 0.000548014870931175*G0_2_0 + 0.000548014870931161*G0_2_1 + 6.52398655870438e-05*G0_2_2;
    A[299] = -0.0788945694544647*G0_1_0 - 0.0788945694544647*G0_1_1 - 0.0189782768992835*G0_1_2 - 0.0189782768992834*G0_2_0 - 0.0189782768992834*G0_2_1 + 0.00939454064454077*G0_2_2;
    A[300] = 0.00887914570640282*G0_1_0 + 0.00658922642429583*G0_1_1 + 0.00887914570640276*G0_1_2 + 0.0111690649885094*G0_2_0 + 0.00887914570640276*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[301] = -0.00578025209101617*G0_1_0 - 0.00626302709636039*G0_1_1 - 0.00578025209101608*G0_1_2 - 0.0052322372200846*G0_2_0 - 0.00578025209101616*G0_2_1 - 0.0052322372200845*G0_2_2;
    A[302] = -0.00480165410720937*G0_1_0 + 0.000260959462348365*G0_1_1 - 0.00480165410720934*G0_1_2 - 0.00960330821441905*G0_2_0 - 0.00480165410720946*G0_2_1 - 0.009603308214419*G0_2_2;
    A[303] = 0.0387263842124946*G0_1_0 - 0.0186912214907006*G0_1_1 + 0.0387263842124946*G0_1_2 + 0.108637424175617*G0_2_0 + 0.0387263842124949*G0_2_1 + 0.108637424175617*G0_2_2;
    A[304] = 0.00228991928210699*G0_1_1 + 0.002289919282107*G0_1_2 + 0.00228991928210656*G0_2_1 + 0.0022899192821066*G0_2_2;
    A[305] = 0.00178757231708607*G0_1_1 + 0.00178757231708607*G0_1_2 + 0.00178757231708616*G0_2_1 + 0.00178757231708614*G0_2_2;
    A[306] = 0.00178757231708628*G0_1_1 + 0.0017875723170863*G0_1_2 + 0.00178757231708595*G0_2_1 + 0.00178757231708597*G0_2_2;
    A[307] = 0.00228991928210653*G0_1_1 + 0.00228991928210657*G0_1_2 + 0.00228991928210665*G0_2_1 + 0.00228991928210669*G0_2_2;
    A[308] = 0.0403182369328198*G0_1_0 + 0.0148746893538562*G0_1_1 + 0.00123955744615479*G0_1_2 + 0.0540186087061085*G0_2_0 + 0.0149399292194431*G0_2_1 + 0.00117431758056753*G0_2_2;
    A[309] = -0.0808321934624013*G0_1_0 - 0.0436454700777617*G0_1_1 - 0.00649136662591533*G0_1_2 - 0.108232937008979*G0_2_0 - 0.0338921101724922*G0_2_1 - 0.00352295274170301*G0_2_2;
    A[310] = 0.0109602974186308*G0_1_0 + 0.132306447410613*G0_1_1 - 0.062369311501255*G0_1_2 + 0.146659217839773*G0_2_0 + 0.073329608919886*G0_2_1 - 0.12473862300251*G0_2_2;
    A[311] = -0.0377738821749242*G0_1_0 + 0.0189848008858413*G0_1_1 - 0.0188869410874623*G0_1_2 - 0.00939454064454088*G0_2_0 + 0.00949240044292073*G0_2_1 - 0.0155923278753139*G0_2_2;
    A[312] = 0.167731694424402*G0_1_0 - 0.166948816037356*G0_1_1 + 0.0838658472122008*G0_1_2 + 0.0777006799142205*G0_2_0 - 0.00616516729797981*G0_2_1 + 0.182410664181495*G0_2_2;
    A[313] = -0.0823327103709032*G0_1_0 + 0.127087258163647*G0_1_1 - 0.0463855444324189*G0_1_2 + 0.00182671623643896*G0_2_0 + 0.037773882174924*G0_2_1 - 0.0510175748891019*G0_2_2;
    A[314] = -0.0403182369328201*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.0390786794866655*G0_1_2 - 0.0540186087061084*G0_2_0 - 0.0390786794866652*G0_2_1 - 0.0528442911255405*G0_2_2;
    A[315] = 0.0808321934624015*G0_1_0 + 0.0371867233846394*G0_1_1 + 0.0743408268364857*G0_1_2 + 0.108232937008978*G0_2_0 + 0.0743408268364859*G0_2_1 + 0.104709984267275*G0_2_2;
    A[316] = -0.0109602974186309*G0_1_0 + 0.121346149991982*G0_1_1 - 0.0733296089198859*G0_1_2 - 0.146659217839773*G0_2_0 - 0.0733296089198864*G0_2_1 - 0.271397840842283*G0_2_2;
    A[317] = 0.0377738821749241*G0_1_0 + 0.0567586830607661*G0_1_1 + 0.0188869410874625*G0_1_2 + 0.00939454064454089*G0_2_0 + 0.0188869410874615*G0_2_1 - 0.00619778723077291*G0_2_2;
    A[318] = -0.167731694424402*G0_1_0 - 0.334680510461758*G0_1_1 - 0.0838658472122007*G0_1_2 - 0.0777006799142204*G0_2_0 - 0.0838658472122002*G0_2_1 + 0.104709984267275*G0_2_2;
    A[319] = 0.0823327103709033*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0359471659384844*G0_1_2 - 0.00182671623643887*G0_2_0 + 0.0359471659384851*G0_2_1 - 0.052844291125541*G0_2_2;
    A[320] = -0.014874689353856*G0_1_1 - 0.0149399292194436*G0_1_2 - 0.014939929219443*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[321] = 0.00450155072550904*G0_1_1 + 0.0019245760348191*G0_1_2 + 0.00192457603481882*G0_2_1 - 0.000652398655870661*G0_2_2;
    A[322] = -0.0148746893538561*G0_1_1 - 0.0149399292194429*G0_1_2 - 0.0149399292194429*G0_2_1 - 0.0150051690850296*G0_2_2;
    A[323] = -0.0189848008858422*G0_1_1 - 0.00949240044292042*G0_1_2 - 0.00949240044292104*G0_2_1 - 0.000652398655871181*G0_2_2;
    A[324] = -0.0189848008858416*G0_1_1 - 0.00949240044292096*G0_1_2 - 0.00949240044292085*G0_2_1 - 0.000652398655871195*G0_2_2;
    A[325] = -0.127087258163647*G0_1_1 - 0.0377738821749243*G0_1_2 - 0.0377738821749239*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[326] = -0.00130479731174201*G0_1_1 - 0.00123955744615466*G0_1_2 - 0.00123955744615453*G0_2_1 - 0.00117431758056786*G0_2_2;
    A[327] = -0.0156575677409009*G0_1_1 - 0.0130805930502111*G0_1_2 - 0.013080593050211*G0_2_1 - 0.0105036183595215*G0_2_2;
    A[328] = -0.00130479731174162*G0_1_1 - 0.00123955744615479*G0_1_2 - 0.00123955744615429*G0_2_1 - 0.0011743175805676*G0_2_2;
    A[329] = 0.00880738185425716*G0_1_1 + 0.00649136662591562*G0_1_2 + 0.00649136662591563*G0_2_1 + 0.00352295274170339*G0_2_2;
    A[330] = 0.00880738185425679*G0_1_1 + 0.0064913666259153*G0_1_2 + 0.00649136662591528*G0_2_1 + 0.00352295274170301*G0_2_2;
    A[331] = 0.0476251018785731*G0_1_1 + 0.062369311501255*G0_1_2 + 0.0623693115012548*G0_2_1 + 0.12473862300251*G0_2_2;
    A[332] = 0.0391439193522527*G0_1_1 + 0.0319675341376727*G0_1_2 + 0.0319675341376733*G0_2_1 + 0.0260959462348356*G0_2_2;
    A[333] = 0.0391439193522524*G0_1_1 + 0.0319675341376735*G0_1_2 + 0.0319675341376731*G0_2_1 + 0.0260959462348357*G0_2_2;
    A[334] = -0.15657567740901*G0_1_1 - 0.0903572138381162*G0_1_2 - 0.0903572138381162*G0_2_1 - 0.185933616923199*G0_2_2;
    A[335] = 0.185933616923198*G0_1_1 + 0.0156575677409007*G0_1_2 + 0.015657567740901*G0_2_1 + 0.0260959462348342*G0_2_2;
    A[336] = -0.00336507226698198*G0_1_0 - 0.00336507226698198*G0_1_1 - 0.00336507226698195*G0_1_2 - 0.00336507226698193*G0_2_0 - 0.00336507226698194*G0_2_1 - 0.00336507226698192*G0_2_2;
    A[337] = 0.00336507226698198*G0_1_0 + 0.00336507226698199*G0_2_0;
    A[338] = -0.0155544887532734*G0_1_1 + 0.0232632312710436*G0_2_1;
    A[339] = -0.03365855145369*G0_1_2 + 0.0214691349673988*G0_2_2;
    A[340] = 0.100717304493345*G0_1_1 + 0.0189782768992833*G0_1_2 - 0.0246345732456845*G0_2_1 + 0.0788945694544644*G0_2_2;
    A[341] = -0.204187731314465*G0_1_1 - 0.0364951808094161*G0_1_2 + 0.0153052724667312*G0_2_1 - 0.204187731314466*G0_2_2;
    A[342] = 0.281001149056702*G0_1_1 + 0.0781312630270955*G0_1_2 + 0.0781312630270955*G0_2_1 + 0.304226541205706*G0_2_2;
    A[343] = -0.108637424175618*G0_1_1 + 0.0263830016434179*G0_1_2 - 0.0387263842124948*G0_2_1 - 0.147063705006412*G0_2_2;
    A[344] = -0.0111690649885095*G0_1_0 - 0.00228991928210686*G0_1_2 - 0.00887914570640256*G0_2_0 - 0.00228991928210664*G0_2_2;
    A[345] = 0.00523223722008471*G0_1_0 - 0.000548014870931157*G0_1_2 + 0.00578025209101608*G0_2_0 - 0.000482775005344286*G0_2_2;
    A[346] = 0.00960330821441928*G0_1_0 + 0.0048016541072096*G0_1_2 + 0.00480165410720944*G0_2_0 + 0.0050626135695579*G0_2_2;
    A[347] = -0.108637424175618*G0_1_0 - 0.0699110399631228*G0_1_2 - 0.0387263842124948*G0_2_0 - 0.0574176057031956*G0_2_2;
    A[348] = -0.0116844599266475*G0_1_0 - 0.00228991928210683*G0_1_1 - 0.0139743792087539*G0_2_0 - 0.00228991928210661*G0_2_1;
    A[349] = 0.0131523569023567*G0_1_0 + 0.0132175967679438*G0_1_1 + 0.0263047138047134*G0_2_0 + 0.0131523569023567*G0_2_1;
    A[350] = -0.000548014870931345*G0_1_0 - 0.000482775005344278*G0_1_1 + 0.00523223722008463*G0_2_0 + 0.00578025209101622*G0_2_1;
    A[351] = 0.0189782768992835*G0_1_0 + 0.0283728175438241*G0_1_1 + 0.0788945694544646*G0_2_0 + 0.0599162925551808*G0_2_1;
    A[352] = 0.0111690649885093*G0_1_0 + 0.0111690649885094*G0_1_1 + 0.0088791457064026*G0_1_2 + 0.00887914570640267*G0_2_0 + 0.00887914570640273*G0_2_1 + 0.00658922642429571*G0_2_2;
    A[353] = -0.00523223722008428*G0_1_0 - 0.00523223722008442*G0_1_1 - 0.00578025209101592*G0_1_2 - 0.00578025209101601*G0_2_0 - 0.00578025209101612*G0_2_1 - 0.00626302709636017*G0_2_2;
    A[354] = -0.00960330821441932*G0_1_0 - 0.00960330821441927*G0_1_1 - 0.00480165410720955*G0_1_2 - 0.00480165410720937*G0_2_0 - 0.00480165410720935*G0_2_1 + 0.000260959462348298*G0_2_2;
    A[355] = 0.108637424175618*G0_1_0 + 0.108637424175618*G0_1_1 + 0.0387263842124949*G0_1_2 + 0.0387263842124948*G0_2_0 + 0.0387263842124948*G0_2_1 - 0.0186912214907007*G0_2_2;
    A[356] = 0.0116844599266472*G0_1_0 + 0.00939454064454057*G0_1_1 + 0.0116844599266472*G0_1_2 + 0.0139743792087542*G0_2_0 + 0.0116844599266473*G0_2_1 + 0.0139743792087542*G0_2_2;
    A[357] = -0.0131523569023567*G0_1_0 + 6.52398655869976e-05*G0_1_1 - 0.0131523569023567*G0_1_2 - 0.0263047138047137*G0_2_0 - 0.0131523569023568*G0_2_1 - 0.0263047138047137*G0_2_2;
    A[358] = 0.000548014870931405*G0_1_0 + 6.52398655871959e-05*G0_1_1 + 0.00054801487093143*G0_1_2 - 0.00523223722008444*G0_2_0 + 0.000548014870931577*G0_2_1 - 0.00523223722008443*G0_2_2;
    A[359] = -0.0189782768992835*G0_1_0 + 0.00939454064454053*G0_1_1 - 0.0189782768992834*G0_1_2 - 0.0788945694544646*G0_2_0 - 0.0189782768992838*G0_2_1 - 0.0788945694544645*G0_2_2;
    A[360] = 0.00228991928210648*G0_1_1 + 0.00228991928210649*G0_1_2 + 0.00228991928210689*G0_2_1 + 0.00228991928210687*G0_2_2;
    A[361] = 0.00178757231708635*G0_1_1 + 0.00178757231708629*G0_1_2 + 0.00178757231708618*G0_2_1 + 0.00178757231708617*G0_2_2;
    A[362] = 0.00178757231708619*G0_1_1 + 0.00178757231708615*G0_1_2 + 0.00178757231708654*G0_2_1 + 0.00178757231708651*G0_2_2;
    A[363] = 0.00228991928210684*G0_1_1 + 0.00228991928210681*G0_1_2 + 0.00228991928210664*G0_2_1 + 0.00228991928210659*G0_2_2;
    A[364] = 0.0540186087061087*G0_1_0 + 0.00117431758056746*G0_1_1 + 0.0149399292194428*G0_1_2 + 0.0403182369328201*G0_2_0 + 0.00123955744615476*G0_2_1 + 0.0148746893538562*G0_2_2;
    A[365] = -0.00939454064454073*G0_1_0 - 0.0155923278753138*G0_1_1 + 0.0094924004429213*G0_1_2 - 0.0377738821749231*G0_2_0 - 0.0188869410874615*G0_2_1 + 0.0189848008858427*G0_2_2;
    A[366] = 0.00182671623643825*G0_1_0 - 0.0510175748891022*G0_1_1 + 0.0377738821749234*G0_1_2 - 0.0823327103709047*G0_2_0 - 0.046385544432419*G0_2_1 + 0.127087258163646*G0_2_2;
    A[367] = -0.108232937008977*G0_1_0 - 0.00352295274170158*G0_1_1 - 0.0338921101724913*G0_1_2 - 0.0808321934624009*G0_2_0 - 0.00649136662591481*G0_2_1 - 0.0436454700777615*G0_2_2;
    A[368] = 0.0777006799142207*G0_1_0 + 0.182410664181495*G0_1_1 - 0.00616516729798061*G0_1_2 + 0.167731694424402*G0_2_0 + 0.0838658472122004*G0_2_1 - 0.166948816037357*G0_2_2;
    A[369] = 0.146659217839772*G0_1_0 - 0.124738623002511*G0_1_1 + 0.0733296089198861*G0_1_2 + 0.0109602974186304*G0_2_0 - 0.0623693115012556*G0_2_1 + 0.132306447410613*G0_2_2;
    A[370] = -0.0540186087061083*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0390786794866652*G0_1_2 - 0.0403182369328201*G0_2_0 - 0.0390786794866655*G0_2_1 - 0.0254435475789642*G0_2_2;
    A[371] = 0.00939454064454045*G0_1_0 - 0.00619778723077319*G0_1_1 + 0.018886941087462*G0_1_2 + 0.0377738821749235*G0_2_0 + 0.018886941087462*G0_2_1 + 0.0567586830607664*G0_2_2;
    A[372] = -0.0018267162364382*G0_1_0 - 0.0528442911255405*G0_1_1 + 0.0359471659384851*G0_1_2 + 0.0823327103709047*G0_2_0 + 0.0359471659384853*G0_2_1 + 0.20941996853455*G0_2_2;
    A[373] = 0.108232937008978*G0_1_0 + 0.104709984267275*G0_1_1 + 0.0743408268364854*G0_1_2 + 0.080832193462401*G0_2_0 + 0.0743408268364861*G0_2_1 + 0.0371867233846393*G0_2_2;
    A[374] = -0.0777006799142207*G0_1_0 + 0.104709984267275*G0_1_1 - 0.0838658472122011*G0_1_2 - 0.167731694424402*G0_2_0 - 0.0838658472122012*G0_2_1 - 0.334680510461758*G0_2_2;
    A[375] = -0.146659217839771*G0_1_0 - 0.271397840842283*G0_1_1 - 0.0733296089198854*G0_1_2 - 0.0109602974186303*G0_2_0 - 0.0733296089198861*G0_2_1 + 0.121346149991983*G0_2_2;
    A[376] = -0.00117431758056745*G0_1_1 - 0.00123955744615397*G0_1_2 - 0.00123955744615462*G0_2_1 - 0.00130479731174176*G0_2_2;
    A[377] = -0.0105036183595215*G0_1_1 - 0.0130805930502112*G0_1_2 - 0.0130805930502112*G0_2_1 - 0.0156575677409014*G0_2_2;
    A[378] = -0.00117431758056754*G0_1_1 - 0.00123955744615475*G0_1_2 - 0.00123955744615492*G0_2_1 - 0.00130479731174206*G0_2_2;
    A[379] = 0.00352295274170273*G0_1_1 + 0.00649136662591431*G0_1_2 + 0.00649136662591517*G0_2_1 + 0.00880738185425684*G0_2_2;
    A[380] = 0.00352295274170179*G0_1_1 + 0.00649136662591503*G0_1_2 + 0.00649136662591494*G0_2_1 + 0.00880738185425722*G0_2_2;
    A[381] = 0.124738623002511*G0_1_1 + 0.0623693115012562*G0_1_2 + 0.0623693115012556*G0_2_1 + 0.0476251018785737*G0_2_2;
    A[382] = -0.0150051690850298*G0_1_1 - 0.014939929219443*G0_1_2 - 0.0149399292194432*G0_2_1 - 0.0148746893538559*G0_2_2;
    A[383] = -0.000652398655871146*G0_1_1 + 0.00192457603481872*G0_1_2 + 0.00192457603481891*G0_2_1 + 0.00450155072550908*G0_2_2;
    A[384] = -0.0150051690850301*G0_1_1 - 0.0149399292194429*G0_1_2 - 0.0149399292194433*G0_2_1 - 0.0148746893538561*G0_2_2;
    A[385] = -0.000652398655871076*G0_1_1 - 0.00949240044292127*G0_1_2 - 0.00949240044292133*G0_2_1 - 0.0189848008858426*G0_2_2;
    A[386] = -0.000652398655870776*G0_1_1 - 0.00949240044292116*G0_1_2 - 0.00949240044292123*G0_2_1 - 0.0189848008858426*G0_2_2;
    A[387] = -0.0150051690850297*G0_1_1 - 0.0377738821749233*G0_1_2 - 0.0377738821749231*G0_2_1 - 0.127087258163646*G0_2_2;
    A[388] = 0.0260959462348351*G0_1_1 + 0.0319675341376733*G0_1_2 + 0.031967534137673*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[389] = 0.0260959462348354*G0_1_1 + 0.0319675341376725*G0_1_2 + 0.0319675341376729*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[390] = 0.0260959462348345*G0_1_1 + 0.0156575677409015*G0_1_2 + 0.0156575677409006*G0_2_1 + 0.185933616923199*G0_2_2;
    A[391] = -0.185933616923198*G0_1_1 - 0.0903572138381162*G0_1_2 - 0.0903572138381158*G0_2_1 - 0.15657567740901*G0_2_2;
    A[392] = -0.00480034930989786*G0_1_0 - 0.00480034930989786*G0_1_1 - 0.00480034930989789*G0_1_2 - 0.00480034930989788*G0_2_0 - 0.00480034930989786*G0_2_1 - 0.00480034930989789*G0_2_2;
    A[393] = 0.00480034930989783*G0_1_0 + 0.00480034930989783*G0_2_0;
    A[394] = 0.00480034930989792*G0_1_1 - 0.0105310191030676*G0_2_1;
    A[395] = 0.0372571824394738*G0_1_2 - 0.0178705039816151*G0_2_2;
    A[396] = -0.0328352243499811*G0_1_1 - 0.00537576492437587*G0_1_2 + 0.02091590090722*G0_2_1 - 0.0328352243499812*G0_2_2;
    A[397] = 0.0788945694544646*G0_1_1 + 0.0189782768992834*G0_1_2 - 0.0246345732456839*G0_2_1 + 0.100717304493345*G0_2_2;
    A[398] = -0.108637424175618*G0_1_1 - 0.0387263842124948*G0_1_2 + 0.0263830016434179*G0_2_1 - 0.147063705006412*G0_2_2;
    A[399] = 0.139822079926246*G0_1_1 + 0.0699110399631229*G0_1_2 + 0.0699110399631229*G0_2_1 + 0.179096479009672*G0_2_2;
    A[400] = -0.0328352243499806*G0_1_0 - 0.00537576492437569*G0_1_2 - 0.0274594594256049*G0_2_0 - 0.00537576492437591*G0_2_2;
    A[401] = 0.0788945694544641*G0_1_0 + 0.0189782768992833*G0_1_2 + 0.0599162925551807*G0_2_0 + 0.0283728175438241*G0_2_2;
    A[402] = -0.108637424175618*G0_1_0 - 0.0387263842124948*G0_1_2 - 0.0699110399631226*G0_2_0 - 0.0574176057031955*G0_2_2;
    A[403] = 0.139822079926246*G0_1_0 + 0.0699110399631228*G0_1_2 + 0.0699110399631228*G0_2_0 + 0.0895482395048364*G0_2_2;
    A[404] = -0.00537576492437565*G0_1_0 - 0.00537576492437576*G0_1_1 - 0.0107515298487519*G0_2_0 - 0.00537576492437598*G0_2_1;
    A[405] = -0.00228991928210672*G0_1_0 - 0.00228991928210681*G0_1_1 - 0.0139743792087541*G0_2_0 - 0.0116844599266473*G0_2_1;
    A[406] = -0.00228991928210676*G0_1_0 - 0.00228991928210652*G0_1_1 - 0.0111690649885093*G0_2_0 - 0.0088791457064025*G0_2_1;
    A[407] = -0.00537576492437596*G0_1_0 - 0.00537576492437621*G0_1_1 - 0.0328352243499812*G0_2_0 - 0.0274594594256053*G0_2_1;
    A[408] = 0.032835224349981*G0_1_0 + 0.0328352243499809*G0_1_1 + 0.027459459425605*G0_1_2 + 0.0274594594256049*G0_2_0 + 0.0274594594256049*G0_2_1 + 0.022083694501229*G0_2_2;
    A[409] = -0.0788945694544645*G0_1_0 - 0.0788945694544644*G0_1_1 - 0.0599162925551809*G0_1_2 - 0.0599162925551809*G0_2_0 - 0.0599162925551808*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[410] = 0.108637424175618*G0_1_0 + 0.108637424175618*G0_1_1 + 0.0699110399631226*G0_1_2 + 0.0699110399631226*G0_2_0 + 0.0699110399631225*G0_2_1 + 0.012493434259927*G0_2_2;
    A[411] = -0.139822079926246*G0_1_0 - 0.139822079926246*G0_1_1 - 0.0699110399631225*G0_1_2 - 0.0699110399631229*G0_2_0 - 0.0699110399631229*G0_2_1 + 0.0196371995417136*G0_2_2;
    A[412] = 0.00537576492437607*G0_1_0 + 0.00537576492437604*G0_1_2 + 0.010751529848752*G0_2_0 + 0.00537576492437606*G0_2_1 + 0.010751529848752*G0_2_2;
    A[413] = 0.00228991928210672*G0_1_0 + 0.00228991928210679*G0_1_2 + 0.0139743792087543*G0_2_0 + 0.00228991928210673*G0_2_1 + 0.0139743792087544*G0_2_2;
    A[414] = 0.00228991928210685*G0_1_0 + 0.00228991928210684*G0_1_2 + 0.0111690649885093*G0_2_0 + 0.00228991928210684*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[415] = 0.00537576492437595*G0_1_0 + 0.00537576492437596*G0_1_2 + 0.0328352243499812*G0_2_0 + 0.00537576492437594*G0_2_1 + 0.0328352243499812*G0_2_2;
    A[416] = 0.00537576492437617*G0_1_1 + 0.00537576492437614*G0_1_2 + 0.00537576492437597*G0_2_1 + 0.00537576492437597*G0_2_2;
    A[417] = 0.00228991928210654*G0_1_1 + 0.00228991928210659*G0_1_2 + 0.00228991928210672*G0_2_1 + 0.00228991928210674*G0_2_2;
    A[418] = 0.00228991928210683*G0_1_1 + 0.00228991928210687*G0_1_2 + 0.00228991928210668*G0_2_1 + 0.00228991928210672*G0_2_2;
    A[419] = 0.00537576492437584*G0_1_1 + 0.00537576492437582*G0_1_2 + 0.00537576492437603*G0_2_1 + 0.00537576492437603*G0_2_2;
    A[420] = 0.0118736555368493*G0_1_0 + 0.0118736555368503*G0_1_1 - 0.00939454064454026*G0_1_2 + 0.0425363923627805*G0_2_0 + 0.0212681961813906*G0_2_1 - 0.0187890812890813*G0_2_2;
    A[421] = 0.00254435475789689*G0_1_0 + 0.00254435475789624*G0_1_1 - 0.00329461321214769*G0_1_2 + 0.0142875305635722*G0_2_0 + 0.00844856259352763*G0_2_1 - 0.0292600797158085*G0_2_2;
    A[422] = 0.0118736555368497*G0_1_0 + 0.01187365553685*G0_1_1 - 0.00939454064454079*G0_1_2 + 0.0477555816097478*G0_2_0 + 0.0264873854283576*G0_2_1 - 0.0641307878721072*G0_2_2;
    A[423] = -0.0429604514890972*G0_1_0 - 0.0429604514890981*G0_1_1 - 0.000685018588665158*G0_1_2 - 0.0845508658008652*G0_2_0 - 0.0422754329004332*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[424] = -0.0429604514890968*G0_1_0 - 0.0429604514890964*G0_1_1 - 0.000685018588663635*G0_1_2 - 0.0493213383838381*G0_2_0 - 0.00704590548340546*G0_2_1 + 0.127315597693201*G0_2_2;
    A[425] = 0.124738623002511*G0_1_0 + 0.124738623002512*G0_1_1 + 0.02844458139597*G0_1_2 + 0.192588083213082*G0_2_0 + 0.0962940416065417*G0_2_1 - 0.0644569872000426*G0_2_2;
    A[426] = -0.0118736555368497*G0_1_0 - 0.0212681961813905*G0_1_2 - 0.0425363923627807*G0_2_0 - 0.0212681961813901*G0_2_1 - 0.0613254736518622*G0_2_2;
    A[427] = -0.00254435475789633*G0_1_0 - 0.00583896797004453*G0_1_2 - 0.0142875305635721*G0_2_0 - 0.00583896797004465*G0_2_1 - 0.0435476102793812*G0_2_2;
    A[428] = -0.0118736555368496*G0_1_0 - 0.0212681961813905*G0_1_2 - 0.0477555816097478*G0_2_0 - 0.0212681961813902*G0_2_1 - 0.111886369481855*G0_2_2;
    A[429] = 0.042960451489097*G0_1_0 + 0.0422754329004327*G0_1_2 + 0.0845508658008653*G0_2_0 + 0.0422754329004322*G0_2_1 + 0.128522535206562*G0_2_2;
    A[430] = 0.0429604514890968*G0_1_0 + 0.042275432900433*G0_1_2 + 0.049321338383838*G0_2_0 + 0.0422754329004325*G0_2_1 + 0.17663693607704*G0_2_2;
    A[431] = -0.124738623002511*G0_1_0 - 0.0962940416065413*G0_1_2 - 0.192588083213082*G0_2_0 - 0.0962940416065403*G0_2_1 - 0.257045070413125*G0_2_2;
    A[432] = -0.0118736555368498*G0_1_1 - 0.0212681961813908*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[433] = -0.00254435475789612*G0_1_1 - 0.0058389679700442*G0_1_2 - 0.00583896797004416*G0_2_1 - 0.00913358118219213*G0_2_2;
    A[434] = -0.0118736555368502*G0_1_1 - 0.0212681961813904*G0_1_2 - 0.0212681961813905*G0_2_1 - 0.030662736825931*G0_2_2;
    A[435] = 0.042960451489097*G0_1_1 + 0.0422754329004332*G0_1_2 + 0.0422754329004327*G0_2_1 + 0.0642612676032814*G0_2_2;
    A[436] = 0.042960451489098*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0422754329004331*G0_2_1 + 0.064261267603281*G0_2_2;
    A[437] = -0.124738623002512*G0_1_1 - 0.0962940416065416*G0_1_2 - 0.0962940416065415*G0_2_1 - 0.128522535206563*G0_2_2;
    A[438] = 0.0093945406445406*G0_1_2 + 0.0093945406445408*G0_2_1 + 0.0187890812890814*G0_2_2;
    A[439] = 0.00329461321214797*G0_1_2 + 0.00329461321214803*G0_2_1 + 0.00658922642429572*G0_2_2;
    A[440] = 0.00939454064454017*G0_1_2 + 0.00939454064454068*G0_2_1 + 0.0187890812890811*G0_2_2;
    A[441] = 0.00329461321214806*G0_1_2 + 0.00329461321214782*G0_2_1 + 0.0292600797158089*G0_2_2;
    A[442] = 0.00329461321214766*G0_1_2 + 0.00329461321214774*G0_2_1 + 0.0292600797158086*G0_2_2;
    A[443] = 0.00939454064454068*G0_1_2 + 0.00939454064454065*G0_2_1 + 0.0641307878721073*G0_2_2;
    A[444] = -0.00260959462348388*G0_1_2 - 0.00260959462348379*G0_2_1 - 0.0505608958299931*G0_2_2;
    A[445] = -0.00260959462348272*G0_1_2 - 0.00260959462348359*G0_2_1 - 0.0505608958299921*G0_2_2;
    A[446] = -0.00260959462348415*G0_1_2 - 0.00260959462348322*G0_2_1 - 0.15657567740901*G0_2_2;
    A[447] = -0.0352295274170269*G0_1_2 - 0.0352295274170272*G0_2_1 + 0.0962288017409542*G0_2_2;
    A[448] = -0.00480034930989792*G0_0_0 - 0.00480034930989794*G0_0_1 - 0.00480034930989795*G0_0_2 - 0.00480034930989788*G0_2_0 - 0.0048003493098979*G0_2_1 - 0.00480034930989793*G0_2_2;
    A[449] = -0.0178705039816151*G0_0_0 + 0.0372571824394736*G0_2_0;
    A[450] = 0.00480034930989787*G0_0_1 + 0.00480034930989784*G0_2_1;
    A[451] = -0.0105310191030675*G0_0_2 + 0.00480034930989798*G0_2_2;
    A[452] = -0.0107515298487518*G0_0_1 - 0.00537576492437585*G0_0_2 - 0.0053757649243759*G0_2_1 - 0.00537576492437594*G0_2_2;
    A[453] = -0.013974379208754*G0_0_1 - 0.0116844599266473*G0_0_2 - 0.0022899192821065*G0_2_1 - 0.00228991928210663*G0_2_2;
    A[454] = -0.0111690649885095*G0_0_1 - 0.00887914570640256*G0_0_2 - 0.00228991928210686*G0_2_1 - 0.00228991928210664*G0_2_2;
    A[455] = -0.0328352243499806*G0_0_1 - 0.0274594594256049*G0_0_2 - 0.00537576492437569*G0_2_1 - 0.00537576492437591*G0_2_2;
    A[456] = 0.179096479009671*G0_0_0 + 0.069911039963122*G0_0_2 + 0.069911039963122*G0_2_0 + 0.139822079926245*G0_2_2;
    A[457] = -0.147063705006411*G0_0_0 + 0.0263830016434189*G0_0_2 - 0.0387263842124942*G0_2_0 - 0.108637424175617*G0_2_2;
    A[458] = 0.100717304493345*G0_0_0 - 0.0246345732456847*G0_0_2 + 0.0189782768992831*G0_2_0 + 0.0788945694544642*G0_2_2;
    A[459] = -0.0328352243499806*G0_0_0 + 0.0209159009072206*G0_0_2 - 0.00537576492437569*G0_2_0 - 0.0328352243499809*G0_2_2;
    A[460] = 0.0895482395048355*G0_0_0 + 0.069911039963122*G0_0_1 + 0.0699110399631223*G0_2_0 + 0.139822079926245*G0_2_1;
    A[461] = -0.0574176057031955*G0_0_0 - 0.0699110399631224*G0_0_1 - 0.0387263842124949*G0_2_0 - 0.108637424175617*G0_2_1;
    A[462] = 0.0283728175438242*G0_0_0 + 0.0599162925551805*G0_0_1 + 0.0189782768992836*G0_2_0 + 0.0788945694544639*G0_2_1;
    A[463] = -0.00537576492437589*G0_0_0 - 0.0274594594256046*G0_0_1 - 0.005375764924376*G0_2_0 - 0.0328352243499806*G0_2_1;
    A[464] = 0.010751529848752*G0_0_0 + 0.010751529848752*G0_0_1 + 0.00537576492437615*G0_0_2 + 0.005375764924376*G0_2_0 + 0.00537576492437596*G0_2_1;
    A[465] = 0.0139743792087538*G0_0_0 + 0.0139743792087538*G0_0_1 + 0.00228991928210645*G0_0_2 + 0.00228991928210632*G0_2_0 + 0.00228991928210635*G0_2_1;
    A[466] = 0.0111690649885098*G0_0_0 + 0.0111690649885098*G0_0_1 + 0.00228991928210712*G0_0_2 + 0.00228991928210703*G0_2_0 + 0.00228991928210706*G0_2_1;
    A[467] = 0.0328352243499806*G0_0_0 + 0.0328352243499806*G0_0_1 + 0.00537576492437567*G0_0_2 + 0.00537576492437571*G0_2_0 + 0.0053757649243757*G0_2_1;
    A[468] = 0.00537576492437593*G0_0_0 + 0.00537576492437589*G0_0_2 + 0.00537576492437603*G0_2_0 + 0.00537576492437596*G0_2_2;
    A[469] = 0.00228991928210673*G0_0_0 + 0.00228991928210671*G0_0_2 + 0.00228991928210684*G0_2_0 + 0.00228991928210684*G0_2_2;
    A[470] = 0.00228991928210688*G0_0_0 + 0.00228991928210681*G0_0_2 + 0.00228991928210669*G0_2_0 + 0.0022899192821066*G0_2_2;
    A[471] = 0.0053757649243759*G0_0_0 + 0.00537576492437587*G0_0_2 + 0.005375764924376*G0_2_0 + 0.00537576492437598*G0_2_2;
    A[472] = 0.022083694501229*G0_0_0 + 0.0274594594256048*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.0274594594256049*G0_2_0 + 0.0328352243499808*G0_2_1 + 0.0328352243499808*G0_2_2;
    A[473] = -0.0315434750113566*G0_0_0 - 0.0599162925551806*G0_0_1 - 0.0599162925551806*G0_0_2 - 0.0599162925551805*G0_2_0 - 0.078894569454464*G0_2_1 - 0.078894569454464*G0_2_2;
    A[474] = 0.0124934342599271*G0_0_0 + 0.0699110399631224*G0_0_1 + 0.0699110399631224*G0_0_2 + 0.0699110399631222*G0_2_0 + 0.108637424175617*G0_2_1 + 0.108637424175617*G0_2_2;
    A[475] = 0.0196371995417135*G0_0_0 - 0.0699110399631221*G0_0_1 - 0.069911039963122*G0_0_2 - 0.0699110399631223*G0_2_0 - 0.139822079926245*G0_2_1 - 0.139822079926245*G0_2_2;
    A[476] = -0.064456987200042*G0_0_0 + 0.192588083213081*G0_0_1 + 0.0962940416065399*G0_0_2 + 0.0284445813959705*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[477] = 0.0439716694056961*G0_0_0 - 0.0845508658008653*G0_0_1 - 0.0422754329004328*G0_0_2 - 0.00068501858866494*G0_2_0 - 0.0429604514890966*G0_2_1 - 0.042960451489097*G0_2_2;
    A[478] = -0.0187890812890808*G0_0_0 + 0.0425363923627808*G0_0_1 + 0.0212681961813907*G0_0_2 - 0.00939454064454019*G0_2_0 + 0.0118736555368498*G0_2_1 + 0.01187365553685*G0_2_2;
    A[479] = 0.127315597693201*G0_0_0 - 0.049321338383837*G0_0_1 - 0.00704590548340457*G0_0_2 - 0.000685018588664526*G0_2_0 - 0.0429604514890961*G0_2_1 - 0.0429604514890961*G0_2_2;
    A[480] = -0.0292600797158087*G0_0_0 + 0.014287530563572*G0_0_1 + 0.00844856259352741*G0_0_2 - 0.00329461321214766*G0_2_0 + 0.00254435475789634*G0_2_1 + 0.00254435475789637*G0_2_2;
    A[481] = -0.0641307878721065*G0_0_0 + 0.047755581609748*G0_0_1 + 0.0264873854283575*G0_0_2 - 0.00939454064454063*G0_2_0 + 0.0118736555368494*G0_2_1 + 0.0118736555368495*G0_2_2;
    A[482] = 0.0187890812890811*G0_0_0 + 0.00939454064454058*G0_0_2 + 0.00939454064454043*G0_2_0;
    A[483] = 0.00658922642429571*G0_0_0 + 0.00329461321214784*G0_0_2 + 0.00329461321214823*G0_2_0;
    A[484] = 0.0187890812890808*G0_0_0 + 0.00939454064454048*G0_0_2 + 0.00939454064454019*G0_2_0;
    A[485] = 0.0292600797158089*G0_0_0 + 0.00329461321214805*G0_0_2 + 0.00329461321214814*G0_2_0;
    A[486] = 0.0292600797158088*G0_0_0 + 0.00329461321214791*G0_0_2 + 0.00329461321214768*G0_2_0;
    A[487] = 0.0641307878721065*G0_0_0 + 0.00939454064454042*G0_0_2 + 0.0093945406445406*G0_2_0;
    A[488] = -0.061325473651862*G0_0_0 - 0.0425363923627812*G0_0_1 - 0.0212681961813905*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.0118736555368504*G0_2_1;
    A[489] = 0.128522535206562*G0_0_0 + 0.0845508658008653*G0_0_1 + 0.0422754329004328*G0_0_2 + 0.0422754329004326*G0_2_0 + 0.0429604514890972*G0_2_1;
    A[490] = -0.257045070413124*G0_0_0 - 0.192588083213081*G0_0_1 - 0.0962940416065414*G0_0_2 - 0.0962940416065401*G0_2_0 - 0.12473862300251*G0_2_1;
    A[491] = -0.0435476102793806*G0_0_0 - 0.0142875305635719*G0_0_1 - 0.0058389679700444*G0_0_2 - 0.00583896797004407*G0_2_0 - 0.00254435475789666*G0_2_1;
    A[492] = 0.176636936077038*G0_0_0 + 0.0493213383838372*G0_0_1 + 0.0422754329004328*G0_0_2 + 0.0422754329004319*G0_2_0 + 0.0429604514890962*G0_2_1;
    A[493] = -0.111886369481855*G0_0_0 - 0.047755581609748*G0_0_1 - 0.0212681961813906*G0_0_2 - 0.0212681961813901*G0_2_0 - 0.0118736555368494*G0_2_1;
    A[494] = -0.030662736825931*G0_0_0 - 0.0212681961813906*G0_0_2 - 0.0212681961813905*G0_2_0 - 0.01187365553685*G0_2_2;
    A[495] = 0.064261267603281*G0_0_0 + 0.0422754329004328*G0_0_2 + 0.0422754329004326*G0_2_0 + 0.0429604514890973*G0_2_2;
    A[496] = -0.128522535206562*G0_0_0 - 0.0962940416065403*G0_0_2 - 0.0962940416065404*G0_2_0 - 0.124738623002511*G0_2_2;
    A[497] = -0.00913358118219218*G0_0_0 - 0.00583896797004425*G0_0_2 - 0.00583896797004436*G0_2_0 - 0.00254435475789621*G0_2_2;
    A[498] = 0.0642612676032809*G0_0_0 + 0.0422754329004328*G0_0_2 + 0.0422754329004323*G0_2_0 + 0.042960451489097*G0_2_2;
    A[499] = -0.0306627368259311*G0_0_0 - 0.0212681961813906*G0_0_2 - 0.0212681961813903*G0_2_0 - 0.0118736555368499*G0_2_2;
    A[500] = -0.050560895829993*G0_0_0 - 0.00260959462348334*G0_0_2 - 0.00260959462348371*G0_2_0;
    A[501] = 0.0962288017409539*G0_0_0 - 0.0352295274170277*G0_0_2 - 0.0352295274170271*G0_2_0;
    A[502] = -0.0505608958299919*G0_0_0 - 0.00260959462348306*G0_0_2 - 0.00260959462348331*G0_2_0;
    A[503] = -0.15657567740901*G0_0_0 - 0.00260959462348363*G0_0_2 - 0.00260959462348362*G0_2_0;
    A[504] = -0.00336507226698193*G0_0_0 - 0.00336507226698192*G0_0_1 - 0.00336507226698191*G0_0_2 - 0.00336507226698196*G0_2_0 - 0.00336507226698195*G0_2_1 - 0.00336507226698192*G0_2_2;
    A[505] = 0.0214691349673988*G0_0_0 - 0.03365855145369*G0_2_0;
    A[506] = 0.00336507226698202*G0_0_1 + 0.00336507226698207*G0_2_1;
    A[507] = 0.0232632312710435*G0_0_2 - 0.0155544887532736*G0_2_2;
    A[508] = -0.0139743792087543*G0_0_1 - 0.0022899192821068*G0_0_2 - 0.0116844599266473*G0_2_1 - 0.00228991928210663*G0_2_2;
    A[509] = 0.0263047138047134*G0_0_1 + 0.0131523569023566*G0_0_2 + 0.0131523569023564*G0_2_1 + 0.0132175967679434*G0_2_2;
    A[510] = 0.00523223722008471*G0_0_1 + 0.00578025209101608*G0_0_2 - 0.000548014870931157*G0_2_1 - 0.000482775005344286*G0_2_2;
    A[511] = 0.0788945694544641*G0_0_1 + 0.0599162925551807*G0_0_2 + 0.0189782768992833*G0_2_1 + 0.0283728175438241*G0_2_2;
    A[512] = -0.147063705006411*G0_0_0 - 0.0387263842124942*G0_0_2 + 0.0263830016434189*G0_2_0 - 0.108637424175617*G0_2_2;
    A[513] = 0.304226541205704*G0_0_0 + 0.0781312630270946*G0_0_2 + 0.0781312630270946*G0_2_0 + 0.281001149056702*G0_2_2;
    A[514] = -0.204187731314465*G0_0_0 + 0.0153052724667314*G0_0_2 - 0.0364951808094158*G0_2_0 - 0.204187731314466*G0_2_2;
    A[515] = 0.078894569454464*G0_0_0 - 0.0246345732456847*G0_0_2 + 0.0189782768992832*G0_2_0 + 0.100717304493345*G0_2_2;
    A[516] = -0.0574176057031948*G0_0_0 - 0.0387263842124943*G0_0_1 - 0.0699110399631223*G0_2_0 - 0.108637424175617*G0_2_1;
    A[517] = 0.00506261356955791*G0_0_0 + 0.00480165410720935*G0_0_1 + 0.00480165410720955*G0_2_0 + 0.00960330821441874*G0_2_1;
    A[518] = -0.000482775005344391*G0_0_0 + 0.00578025209101656*G0_0_1 - 0.000548014870931617*G0_2_0 + 0.00523223722008512*G0_2_1;
    A[519] = -0.00228991928210681*G0_0_0 - 0.00887914570640306*G0_0_1 - 0.00228991928210657*G0_2_0 - 0.0111690649885097*G0_2_1;
    A[520] = 0.0139743792087538*G0_0_0 + 0.0139743792087538*G0_0_1 + 0.0116844599266472*G0_0_2 + 0.0116844599266471*G0_2_0 + 0.0116844599266471*G0_2_1 + 0.00939454064454046*G0_2_2;
    A[521] = -0.0263047138047129*G0_0_0 - 0.0263047138047129*G0_0_1 - 0.0131523569023565*G0_0_2 - 0.013152356902356*G0_2_0 - 0.0131523569023561*G0_2_1 + 6.5239865587358e-05*G0_2_2;
    A[522] = -0.00523223722008506*G0_0_0 - 0.00523223722008517*G0_0_1 + 0.000548014870931148*G0_0_2 + 0.000548014870931074*G0_2_0 + 0.000548014870931018*G0_2_1 + 6.52398655867308e-05*G0_2_2;
    A[523] = -0.0788945694544641*G0_0_0 - 0.0788945694544641*G0_0_1 - 0.0189782768992833*G0_0_2 - 0.0189782768992832*G0_2_0 - 0.0189782768992833*G0_2_1 + 0.00939454064454095*G0_2_2;
    A[524] = 0.00228991928210681*G0_0_0 + 0.0022899192821068*G0_0_2 + 0.00228991928210667*G0_2_0 + 0.00228991928210671*G0_2_2;
    A[525] = 0.00178757231708626*G0_0_0 + 0.00178757231708635*G0_0_2 + 0.00178757231708612*G0_2_0 + 0.00178757231708617*G0_2_2;
    A[526] = 0.00178757231708624*G0_0_0 + 0.00178757231708624*G0_0_2 + 0.00178757231708634*G0_2_0 + 0.00178757231708636*G0_2_2;
    A[527] = 0.00228991928210681*G0_0_0 + 0.00228991928210684*G0_0_2 + 0.00228991928210659*G0_2_0 + 0.00228991928210661*G0_2_2;
    A[528] = 0.00658922642429593*G0_0_0 + 0.00887914570640261*G0_0_1 + 0.00887914570640261*G0_0_2 + 0.00887914570640269*G0_2_0 + 0.0111690649885093*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[529] = -0.00626302709636066*G0_0_0 - 0.0057802520910162*G0_0_1 - 0.00578025209101622*G0_0_2 - 0.00578025209101627*G0_2_0 - 0.00523223722008477*G0_2_1 - 0.00523223722008482*G0_2_2;
    A[530] = 0.000260959462348533*G0_0_0 - 0.00480165410720943*G0_0_1 - 0.00480165410720938*G0_0_2 - 0.0048016541072092*G0_2_0 - 0.00960330821441887*G0_2_1 - 0.00960330821441877*G0_2_2;
    A[531] = -0.0186912214907007*G0_0_0 + 0.0387263842124943*G0_0_1 + 0.0387263842124943*G0_0_2 + 0.0387263842124948*G0_2_0 + 0.108637424175617*G0_2_1 + 0.108637424175617*G0_2_2;
    A[532] = 0.132306447410612*G0_0_0 + 0.0109602974186311*G0_0_1 - 0.0623693115012545*G0_0_2 + 0.0733296089198852*G0_2_0 + 0.146659217839773*G0_2_1 - 0.12473862300251*G0_2_2;
    A[533] = -0.0436454700777607*G0_0_0 - 0.0808321934624015*G0_0_1 - 0.00649136662591492*G0_0_2 - 0.0338921101724915*G0_2_0 - 0.108232937008979*G0_2_1 - 0.00352295274170285*G0_2_2;
    A[534] = 0.0148746893538553*G0_0_0 + 0.0403182369328201*G0_0_1 + 0.00123955744615429*G0_0_2 + 0.0149399292194424*G0_2_0 + 0.0540186087061082*G0_2_1 + 0.00117431758056746*G0_2_2;
    A[535] = -0.166948816037357*G0_0_0 + 0.1677316944244*G0_0_1 + 0.0838658472121989*G0_0_2 - 0.0061651672979797*G0_2_0 + 0.0777006799142198*G0_2_1 + 0.182410664181495*G0_2_2;
    A[536] = 0.0189848008858427*G0_0_0 - 0.0377738821749233*G0_0_1 - 0.0188869410874611*G0_0_2 + 0.00949240044292111*G0_2_0 - 0.00939454064454027*G0_2_1 - 0.0155923278753138*G0_2_2;
    A[537] = 0.127087258163646*G0_0_0 - 0.0823327103709043*G0_0_1 - 0.0463855444324192*G0_0_2 + 0.0377738821749238*G0_2_0 + 0.00182671623643923*G0_2_1 - 0.0510175748891016*G0_2_2;
    A[538] = -0.0148746893538554*G0_0_0 - 0.0149399292194428*G0_0_2 - 0.0149399292194425*G0_2_0 - 0.0150051690850296*G0_2_2;
    A[539] = 0.00450155072550914*G0_0_0 + 0.0019245760348188*G0_0_2 + 0.0019245760348187*G0_2_0 - 0.000652398655871159*G0_2_2;
    A[540] = -0.0148746893538553*G0_0_0 - 0.014939929219443*G0_0_2 - 0.0149399292194425*G0_2_0 - 0.0150051690850299*G0_2_2;
    A[541] = -0.0189848008858428*G0_0_0 - 0.00949240044292151*G0_0_2 - 0.0094924004429216*G0_2_0 - 0.000652398655871687*G0_2_2;
    A[542] = -0.0189848008858426*G0_0_0 - 0.00949240044292089*G0_0_2 - 0.00949240044292107*G0_2_0 - 0.000652398655870531*G0_2_2;
    A[543] = -0.127087258163646*G0_0_0 - 0.0377738821749234*G0_0_2 - 0.0377738821749238*G0_2_0 - 0.0150051690850299*G0_2_2;
    A[544] = -0.0254435475789637*G0_0_0 - 0.0403182369328191*G0_0_1 - 0.0390786794866649*G0_0_2 - 0.0390786794866649*G0_2_0 - 0.0540186087061073*G0_2_1 - 0.05284429112554*G0_2_2;
    A[545] = 0.0371867233846395*G0_0_0 + 0.0808321934624006*G0_0_1 + 0.0743408268364857*G0_0_2 + 0.0743408268364857*G0_2_0 + 0.108232937008977*G0_2_1 + 0.104709984267275*G0_2_2;
    A[546] = 0.121346149991982*G0_0_0 - 0.0109602974186307*G0_0_1 - 0.0733296089198851*G0_0_2 - 0.0733296089198868*G0_2_0 - 0.146659217839773*G0_2_1 - 0.271397840842283*G0_2_2;
    A[547] = 0.0567586830607654*G0_0_0 + 0.037773882174923*G0_0_1 + 0.0188869410874616*G0_0_2 + 0.0188869410874612*G0_2_0 + 0.00939454064454081*G0_2_1 - 0.00619778723077357*G0_2_2;
    A[548] = -0.334680510461757*G0_0_0 - 0.1677316944244*G0_0_1 - 0.083865847212201*G0_0_2 - 0.0838658472121997*G0_2_0 - 0.0777006799142199*G0_2_1 + 0.104709984267275*G0_2_2;
    A[549] = 0.20941996853455*G0_0_0 + 0.0823327103709042*G0_0_1 + 0.0359471659384854*G0_0_2 + 0.0359471659384848*G0_2_0 - 0.00182671623643911*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[550] = -0.00130479731174172*G0_0_0 - 0.0012395574461543*G0_0_2 - 0.00123955744615449*G0_2_0 - 0.00117431758056729*G0_2_2;
    A[551] = 0.00880738185425671*G0_0_0 + 0.00649136662591478*G0_0_2 + 0.00649136662591508*G0_2_0 + 0.00352295274170221*G0_2_2;
    A[552] = 0.0476251018785731*G0_0_0 + 0.0623693115012545*G0_0_2 + 0.0623693115012551*G0_2_0 + 0.124738623002511*G0_2_2;
    A[553] = -0.015657567740901*G0_0_0 - 0.0130805930502108*G0_0_2 - 0.0130805930502109*G0_2_0 - 0.0105036183595212*G0_2_2;
    A[554] = 0.00880738185425684*G0_0_0 + 0.00649136662591491*G0_0_2 + 0.00649136662591529*G0_2_0 + 0.00352295274170288*G0_2_2;
    A[555] = -0.00130479731174185*G0_0_0 - 0.00123955744615431*G0_0_2 - 0.00123955744615474*G0_2_0 - 0.00117431758056746*G0_2_2;
    A[556] = 0.0391439193522518*G0_0_0 + 0.0319675341376721*G0_0_2 + 0.0319675341376724*G0_2_0 + 0.0260959462348347*G0_2_2;
    A[557] = -0.156575677409009*G0_0_0 - 0.0903572138381139*G0_0_2 - 0.0903572138381152*G0_2_0 - 0.185933616923198*G0_2_2;
    A[558] = 0.0391439193522515*G0_0_0 + 0.031967534137672*G0_0_2 + 0.0319675341376729*G0_2_0 + 0.0260959462348349*G0_2_2;
    A[559] = 0.1859336169232*G0_0_0 + 0.0156575677409014*G0_0_2 + 0.0156575677409015*G0_2_0 + 0.0260959462348344*G0_2_2;
    A[560] = -0.00336507226698196*G0_0_0 - 0.00336507226698198*G0_0_1 - 0.00336507226698196*G0_0_2 - 0.00336507226698196*G0_2_0 - 0.00336507226698198*G0_2_1 - 0.00336507226698199*G0_2_2;
    A[561] = -0.0155544887532734*G0_0_0 + 0.0232632312710436*G0_2_0;
    A[562] = 0.00336507226698191*G0_0_1 + 0.00336507226698189*G0_2_1;
    A[563] = -0.03365855145369*G0_0_2 + 0.0214691349673988*G0_2_2;
    A[564] = -0.011169064988509*G0_0_1 - 0.00228991928210675*G0_0_2 - 0.00887914570640231*G0_2_1 - 0.00228991928210672*G0_2_2;
    A[565] = 0.00523223722008437*G0_0_1 - 0.000548014870931181*G0_0_2 + 0.00578025209101598*G0_2_1 - 0.000482775005344207*G0_2_2;
    A[566] = 0.00960330821441928*G0_0_1 + 0.00480165410720943*G0_0_2 + 0.0048016541072096*G0_2_1 + 0.0050626135695579*G0_2_2;
    A[567] = -0.108637424175618*G0_0_1 - 0.0699110399631226*G0_0_2 - 0.0387263842124948*G0_2_1 - 0.0574176057031955*G0_2_2;
    A[568] = 0.100717304493345*G0_0_0 + 0.0189782768992831*G0_0_2 - 0.0246345732456847*G0_2_0 + 0.0788945694544642*G0_2_2;
    A[569] = -0.204187731314465*G0_0_0 - 0.0364951808094158*G0_0_2 + 0.0153052724667314*G0_2_0 - 0.204187731314466*G0_2_2;
    A[570] = 0.281001149056702*G0_0_0 + 0.0781312630270952*G0_0_2 + 0.0781312630270952*G0_2_0 + 0.304226541205706*G0_2_2;
    A[571] = -0.108637424175617*G0_0_0 + 0.0263830016434183*G0_0_2 - 0.0387263842124948*G0_2_0 - 0.147063705006412*G0_2_2;
    A[572] = 0.0283728175438237*G0_0_0 + 0.0189782768992831*G0_0_1 + 0.0599162925551807*G0_2_0 + 0.0788945694544643*G0_2_1;
    A[573] = -0.000482775005344379*G0_0_0 - 0.000548014870931331*G0_0_1 + 0.00578025209101602*G0_2_0 + 0.00523223722008486*G0_2_1;
    A[574] = 0.0132175967679437*G0_0_0 + 0.0131523569023562*G0_0_1 + 0.0131523569023568*G0_2_0 + 0.0263047138047129*G0_2_1;
    A[575] = -0.00228991928210671*G0_0_0 - 0.0116844599266469*G0_0_1 - 0.00228991928210675*G0_2_0 - 0.0139743792087536*G0_2_1;
    A[576] = 0.0111690649885093*G0_0_0 + 0.0111690649885093*G0_0_1 + 0.00887914570640253*G0_0_2 + 0.00887914570640263*G0_2_0 + 0.00887914570640262*G0_2_1 + 0.00658922642429581*G0_2_2;
    A[577] = -0.0052322372200848*G0_0_0 - 0.00523223722008477*G0_0_1 - 0.00578025209101588*G0_0_2 - 0.00578025209101639*G0_2_0 - 0.00578025209101639*G0_2_1 - 0.00626302709636044*G0_2_2;
    A[578] = -0.00960330821441899*G0_0_0 - 0.00960330821441889*G0_0_1 - 0.0048016541072096*G0_0_2 - 0.00480165410720937*G0_2_0 - 0.00480165410720927*G0_2_1 + 0.000260959462348474*G0_2_2;
    A[579] = 0.108637424175618*G0_0_0 + 0.108637424175618*G0_0_1 + 0.0387263842124948*G0_0_2 + 0.0387263842124948*G0_2_0 + 0.0387263842124948*G0_2_1 - 0.0186912214907007*G0_2_2;
    A[580] = 0.00228991928210663*G0_0_0 + 0.00228991928210662*G0_0_2 + 0.00228991928210677*G0_2_0 + 0.00228991928210671*G0_2_2;
    A[581] = 0.00178757231708628*G0_0_0 + 0.0017875723170862*G0_0_2 + 0.00178757231708623*G0_2_0 + 0.0017875723170862*G0_2_2;
    A[582] = 0.00178757231708613*G0_0_0 + 0.00178757231708612*G0_0_2 + 0.00178757231708627*G0_2_0 + 0.00178757231708624*G0_2_2;
    A[583] = 0.00228991928210672*G0_0_0 + 0.00228991928210667*G0_0_2 + 0.00228991928210674*G0_2_0 + 0.0022899192821067*G0_2_2;
    A[584] = 0.00939454064454041*G0_0_0 + 0.0116844599266471*G0_0_1 + 0.0116844599266471*G0_0_2 + 0.0116844599266472*G0_2_0 + 0.013974379208754*G0_2_1 + 0.013974379208754*G0_2_2;
    A[585] = 6.52398655873296e-05*G0_0_0 - 0.0131523569023565*G0_0_1 - 0.0131523569023565*G0_0_2 - 0.0131523569023566*G0_2_0 - 0.0263047138047132*G0_2_1 - 0.0263047138047132*G0_2_2;
    A[586] = 6.52398655869694e-05*G0_0_0 + 0.000548014870931326*G0_0_1 + 0.000548014870931267*G0_0_2 + 0.000548014870931313*G0_2_0 - 0.00523223722008477*G0_2_1 - 0.00523223722008487*G0_2_2;
    A[587] = 0.00939454064454066*G0_0_0 - 0.0189782768992833*G0_0_1 - 0.0189782768992832*G0_0_2 - 0.0189782768992835*G0_2_0 - 0.0788945694544644*G0_2_1 - 0.0788945694544643*G0_2_2;
    A[588] = -0.0510175748891015*G0_0_0 + 0.00182671623643824*G0_0_1 + 0.0377738821749229*G0_0_2 - 0.0463855444324184*G0_2_0 - 0.0823327103709048*G0_2_1 + 0.127087258163646*G0_2_2;
    A[589] = -0.015592327875315*G0_0_0 - 0.00939454064454066*G0_0_1 + 0.00949240044292063*G0_0_2 - 0.0188869410874624*G0_2_0 - 0.0377738821749231*G0_2_1 + 0.0189848008858423*G0_2_2;
    A[590] = 0.00117431758056825*G0_0_0 + 0.0540186087061081*G0_0_1 + 0.0149399292194433*G0_0_2 + 0.00123955744615525*G0_2_0 + 0.0403182369328198*G0_2_1 + 0.0148746893538562*G0_2_2;
    A[591] = 0.182410664181496*G0_0_0 + 0.0777006799142218*G0_0_1 - 0.0061651672979788*G0_0_2 + 0.0838658472122007*G0_2_0 + 0.167731694424402*G0_2_1 - 0.166948816037356*G0_2_2;
    A[592] = -0.0035229527417028*G0_0_0 - 0.108232937008978*G0_0_1 - 0.0338921101724925*G0_0_2 - 0.00649136662591513*G0_2_0 - 0.0808321934624011*G0_2_1 - 0.0436454700777616*G0_2_2;
    A[593] = -0.124738623002511*G0_0_0 + 0.146659217839772*G0_0_1 + 0.0733296089198866*G0_0_2 - 0.0623693115012557*G0_2_0 + 0.0109602974186298*G0_2_1 + 0.132306447410613*G0_2_2;
    A[594] = -0.00117431758056801*G0_0_0 - 0.00123955744615459*G0_0_2 - 0.00123955744615513*G0_2_0 - 0.00130479731174191*G0_2_2;
    A[595] = -0.0105036183595212*G0_0_0 - 0.013080593050211*G0_0_2 - 0.0130805930502109*G0_2_0 - 0.0156575677409011*G0_2_2;
    A[596] = -0.00117431758056827*G0_0_0 - 0.00123955744615468*G0_0_2 - 0.00123955744615525*G0_2_0 - 0.00130479731174192*G0_2_2;
    A[597] = 0.00352295274170265*G0_0_0 + 0.0064913666259151*G0_0_2 + 0.0064913666259153*G0_2_0 + 0.00880738185425733*G0_2_2;
    A[598] = 0.00352295274170272*G0_0_0 + 0.00649136662591487*G0_0_2 + 0.00649136662591511*G0_2_0 + 0.0088073818542567*G0_2_2;
    A[599] = 0.124738623002511*G0_0_0 + 0.0623693115012557*G0_0_2 + 0.0623693115012557*G0_2_0 + 0.0476251018785737*G0_2_2;
    A[600] = -0.0528442911255404*G0_0_0 - 0.0540186087061086*G0_0_1 - 0.0390786794866651*G0_0_2 - 0.0390786794866652*G0_2_0 - 0.0403182369328204*G0_2_1 - 0.0254435475789642*G0_2_2;
    A[601] = -0.00619778723077342*G0_0_0 + 0.00939454064454079*G0_0_1 + 0.0188869410874619*G0_0_2 + 0.0188869410874618*G0_2_0 + 0.0377738821749238*G0_2_1 + 0.0567586830607661*G0_2_2;
    A[602] = -0.0528442911255403*G0_0_0 - 0.00182671623643837*G0_0_1 + 0.0359471659384845*G0_0_2 + 0.0359471659384855*G0_2_0 + 0.0823327103709046*G0_2_1 + 0.20941996853455*G0_2_2;
    A[603] = 0.104709984267275*G0_0_0 + 0.108232937008977*G0_0_1 + 0.0743408268364853*G0_0_2 + 0.0743408268364862*G0_2_0 + 0.0808321934624004*G0_2_1 + 0.0371867233846393*G0_2_2;
    A[604] = 0.104709984267275*G0_0_0 - 0.0777006799142216*G0_0_1 - 0.0838658472122006*G0_0_2 - 0.0838658472122012*G0_2_0 - 0.167731694424402*G0_2_1 - 0.334680510461758*G0_2_2;
    A[605] = -0.271397840842283*G0_0_0 - 0.146659217839772*G0_0_1 - 0.0733296089198855*G0_0_2 - 0.0733296089198857*G0_2_0 - 0.0109602974186299*G0_2_1 + 0.121346149991983*G0_2_2;
    A[606] = -0.01500516908503*G0_0_0 - 0.0149399292194431*G0_0_2 - 0.014939929219443*G0_2_0 - 0.0148746893538561*G0_2_2;
    A[607] = -0.000652398655870794*G0_0_0 - 0.00949240044292095*G0_0_2 - 0.00949240044292118*G0_2_0 - 0.018984800885842*G0_2_2;
    A[608] = -0.0150051690850296*G0_0_0 - 0.0377738821749228*G0_0_2 - 0.0377738821749233*G0_2_0 - 0.127087258163646*G0_2_2;
    A[609] = -0.000652398655871054*G0_0_0 + 0.00192457603481887*G0_0_2 + 0.00192457603481894*G0_2_0 + 0.00450155072550913*G0_2_2;
    A[610] = -0.000652398655870885*G0_0_0 - 0.0094924004429206*G0_0_2 - 0.0094924004429212*G0_2_0 - 0.0189848008858423*G0_2_2;
    A[611] = -0.0150051690850299*G0_0_0 - 0.0149399292194432*G0_0_2 - 0.014939929219443*G0_2_0 - 0.0148746893538562*G0_2_2;
    A[612] = 0.0260959462348356*G0_0_0 + 0.031967534137673*G0_0_2 + 0.0319675341376734*G0_2_0 + 0.0391439193522524*G0_2_2;
    A[613] = 0.026095946234834*G0_0_0 + 0.0156575677408996*G0_0_2 + 0.0156575677409003*G0_2_0 + 0.185933616923198*G0_2_2;
    A[614] = 0.0260959462348364*G0_0_0 + 0.0319675341376734*G0_0_2 + 0.0319675341376734*G0_2_0 + 0.0391439193522525*G0_2_2;
    A[615] = -0.185933616923199*G0_0_0 - 0.0903572138381162*G0_0_2 - 0.0903572138381162*G0_2_0 - 0.156575677409009*G0_2_2;
    A[616] = -0.00480034930989786*G0_0_0 - 0.00480034930989785*G0_0_1 - 0.00480034930989789*G0_0_2 - 0.00480034930989784*G0_2_0 - 0.00480034930989783*G0_2_1 - 0.00480034930989785*G0_2_2;
    A[617] = 0.00480034930989782*G0_0_0 - 0.0105310191030677*G0_2_0;
    A[618] = 0.00480034930989792*G0_0_1 + 0.00480034930989793*G0_2_1;
    A[619] = 0.0372571824394738*G0_0_2 - 0.0178705039816151*G0_2_2;
    A[620] = -0.0328352243499812*G0_0_1 - 0.00537576492437589*G0_0_2 - 0.0274594594256052*G0_2_1 - 0.005375764924376*G0_2_2;
    A[621] = 0.0788945694544647*G0_0_1 + 0.0189782768992834*G0_0_2 + 0.059916292555181*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[622] = -0.108637424175618*G0_0_1 - 0.0387263842124948*G0_0_2 - 0.0699110399631228*G0_2_1 - 0.0574176057031956*G0_2_2;
    A[623] = 0.139822079926246*G0_0_1 + 0.0699110399631228*G0_0_2 + 0.0699110399631228*G0_2_1 + 0.0895482395048364*G0_2_2;
    A[624] = -0.0328352243499806*G0_0_0 - 0.00537576492437569*G0_0_2 + 0.0209159009072206*G0_2_0 - 0.0328352243499809*G0_2_2;
    A[625] = 0.078894569454464*G0_0_0 + 0.0189782768992832*G0_0_2 - 0.0246345732456847*G0_2_0 + 0.100717304493345*G0_2_2;
    A[626] = -0.108637424175618*G0_0_0 - 0.0387263842124948*G0_0_2 + 0.0263830016434183*G0_2_0 - 0.147063705006412*G0_2_2;
    A[627] = 0.139822079926246*G0_0_0 + 0.0699110399631229*G0_0_2 + 0.0699110399631229*G0_2_0 + 0.179096479009672*G0_2_2;
    A[628] = -0.00537576492437563*G0_0_0 - 0.00537576492437576*G0_0_1 - 0.0274594594256049*G0_2_0 - 0.032835224349981*G0_2_1;
    A[629] = -0.00228991928210673*G0_0_0 - 0.00228991928210681*G0_0_1 - 0.00887914570640257*G0_2_0 - 0.0111690649885095*G0_2_1;
    A[630] = -0.00228991928210674*G0_0_0 - 0.00228991928210653*G0_0_1 - 0.0116844599266474*G0_2_0 - 0.0139743792087539*G0_2_1;
    A[631] = -0.00537576492437598*G0_0_0 - 0.00537576492437623*G0_0_1 - 0.00537576492437602*G0_2_0 - 0.0107515298487523*G0_2_1;
    A[632] = 0.032835224349981*G0_0_0 + 0.0328352243499809*G0_0_1 + 0.027459459425605*G0_0_2 + 0.0274594594256049*G0_2_0 + 0.0274594594256049*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[633] = -0.0788945694544645*G0_0_0 - 0.0788945694544644*G0_0_1 - 0.0599162925551809*G0_0_2 - 0.0599162925551808*G0_2_0 - 0.0599162925551807*G0_2_1 - 0.0315434750113565*G0_2_2;
    A[634] = 0.108637424175618*G0_0_0 + 0.108637424175618*G0_0_1 + 0.0699110399631226*G0_0_2 + 0.0699110399631226*G0_2_0 + 0.0699110399631226*G0_2_1 + 0.0124934342599269*G0_2_2;
    A[635] = -0.139822079926246*G0_0_0 - 0.139822079926246*G0_0_1 - 0.0699110399631225*G0_0_2 - 0.0699110399631229*G0_2_0 - 0.0699110399631229*G0_2_1 + 0.0196371995417136*G0_2_2;
    A[636] = 0.00537576492437605*G0_0_0 + 0.00537576492437602*G0_0_2 + 0.00537576492437599*G0_2_0 + 0.00537576492437599*G0_2_2;
    A[637] = 0.00228991928210675*G0_0_0 + 0.00228991928210682*G0_0_2 + 0.00228991928210688*G0_2_0 + 0.00228991928210692*G0_2_2;
    A[638] = 0.00228991928210683*G0_0_0 + 0.00228991928210683*G0_0_2 + 0.00228991928210669*G0_2_0 + 0.0022899192821067*G0_2_2;
    A[639] = 0.00537576492437598*G0_0_0 + 0.00537576492437599*G0_0_2 + 0.00537576492437603*G0_2_0 + 0.00537576492437603*G0_2_2;
    A[640] = 0.00537576492437616*G0_0_1 + 0.00537576492437613*G0_0_2 + 0.00537576492437606*G0_2_0 + 0.0107515298487521*G0_2_1 + 0.0107515298487521*G0_2_2;
    A[641] = 0.00228991928210655*G0_0_1 + 0.0022899192821066*G0_0_2 + 0.00228991928210662*G0_2_0 + 0.013974379208754*G0_2_1 + 0.013974379208754*G0_2_2;
    A[642] = 0.00228991928210685*G0_0_1 + 0.00228991928210688*G0_0_2 + 0.00228991928210683*G0_2_0 + 0.0111690649885094*G0_2_1 + 0.0111690649885095*G0_2_2;
    A[643] = 0.00537576492437584*G0_0_1 + 0.00537576492437582*G0_0_2 + 0.00537576492437604*G0_2_0 + 0.032835224349981*G0_2_1 + 0.032835224349981*G0_2_2;
    A[644] = 0.0118736555368492*G0_0_0 + 0.0118736555368502*G0_0_1 - 0.00939454064454027*G0_0_2 + 0.0264873854283569*G0_2_0 + 0.0477555816097483*G0_2_1 - 0.0641307878721069*G0_2_2;
    A[645] = 0.00254435475789687*G0_0_0 + 0.00254435475789625*G0_0_1 - 0.00329461321214767*G0_0_2 + 0.00844856259352788*G0_2_0 + 0.0142875305635717*G0_2_1 - 0.0292600797158088*G0_2_2;
    A[646] = 0.0118736555368496*G0_0_0 + 0.01187365553685*G0_0_1 - 0.00939454064454084*G0_0_2 + 0.0212681961813902*G0_2_0 + 0.0425363923627809*G0_2_1 - 0.0187890812890815*G0_2_2;
    A[647] = -0.0429604514890972*G0_0_0 - 0.0429604514890981*G0_0_1 - 0.000685018588665163*G0_0_2 - 0.0070459054834056*G0_2_0 - 0.049321338383839*G0_2_1 + 0.1273155976932*G0_2_2;
    A[648] = -0.0429604514890969*G0_0_0 - 0.0429604514890965*G0_0_1 - 0.000685018588663688*G0_0_2 - 0.0422754329004325*G0_2_0 - 0.0845508658008648*G0_2_1 + 0.0439716694056974*G0_2_2;
    A[649] = 0.124738623002511*G0_0_0 + 0.124738623002512*G0_0_1 + 0.02844458139597*G0_0_2 + 0.0962940416065416*G0_2_0 + 0.192588083213083*G0_2_1 - 0.0644569872000423*G0_2_2;
    A[650] = -0.0118736555368496*G0_0_0 - 0.0212681961813904*G0_0_2 - 0.0212681961813902*G0_2_0 - 0.0306627368259311*G0_2_2;
    A[651] = -0.00254435475789632*G0_0_0 - 0.00583896797004453*G0_0_2 - 0.00583896797004434*G0_2_0 - 0.0091335811821924*G0_2_2;
    A[652] = -0.0118736555368496*G0_0_0 - 0.0212681961813905*G0_0_2 - 0.0212681961813902*G0_2_0 - 0.0306627368259311*G0_2_2;
    A[653] = 0.042960451489097*G0_0_0 + 0.0422754329004327*G0_0_2 + 0.0422754329004325*G0_2_0 + 0.0642612676032811*G0_2_2;
    A[654] = 0.0429604514890968*G0_0_0 + 0.042275432900433*G0_0_2 + 0.0422754329004325*G0_2_0 + 0.0642612676032813*G0_2_2;
    A[655] = -0.124738623002511*G0_0_0 - 0.0962940416065412*G0_0_2 - 0.0962940416065414*G0_2_0 - 0.128522535206563*G0_2_2;
    A[656] = -0.0118736555368498*G0_0_1 - 0.0212681961813908*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.0425363923627807*G0_2_1 - 0.0613254736518622*G0_2_2;
    A[657] = -0.00254435475789609*G0_0_1 - 0.0058389679700442*G0_0_2 - 0.00583896797004398*G0_2_0 - 0.014287530563572*G0_2_1 - 0.0435476102793807*G0_2_2;
    A[658] = -0.0118736555368502*G0_0_1 - 0.0212681961813904*G0_0_2 - 0.0212681961813911*G0_2_0 - 0.0477555816097482*G0_2_1 - 0.111886369481855*G0_2_2;
    A[659] = 0.0429604514890971*G0_0_1 + 0.0422754329004332*G0_0_2 + 0.0422754329004322*G0_2_0 + 0.0845508658008652*G0_2_1 + 0.128522535206563*G0_2_2;
    A[660] = 0.042960451489098*G0_0_1 + 0.0422754329004327*G0_0_2 + 0.0422754329004333*G0_2_0 + 0.0493213383838389*G0_2_1 + 0.176636936077039*G0_2_2;
    A[661] = -0.124738623002512*G0_0_1 - 0.0962940416065417*G0_0_2 - 0.0962940416065409*G0_2_0 - 0.192588083213083*G0_2_1 - 0.257045070413125*G0_2_2;
    A[662] = 0.00939454064454066*G0_0_2 + 0.00939454064454068*G0_2_0 + 0.0187890812890815*G0_2_2;
    A[663] = 0.00329461321214792*G0_0_2 + 0.00329461321214802*G0_2_0 + 0.0292600797158086*G0_2_2;
    A[664] = 0.00939454064454018*G0_0_2 + 0.00939454064454021*G0_2_0 + 0.0641307878721069*G0_2_2;
    A[665] = 0.00329461321214801*G0_0_2 + 0.00329461321214776*G0_2_0 + 0.00658922642429581*G0_2_2;
    A[666] = 0.00329461321214764*G0_0_2 + 0.00329461321214794*G0_2_0 + 0.0292600797158087*G0_2_2;
    A[667] = 0.00939454064454073*G0_0_2 + 0.00939454064454073*G0_2_0 + 0.0187890812890815*G0_2_2;
    A[668] = -0.00260959462348397*G0_0_2 - 0.002609594623484*G0_2_0 - 0.0505608958299932*G0_2_2;
    A[669] = -0.00260959462348267*G0_0_2 - 0.00260959462348273*G0_2_0 - 0.156575677409009*G0_2_2;
    A[670] = -0.00260959462348404*G0_0_2 - 0.0026095946234836*G0_2_0 - 0.050560895829993*G0_2_2;
    A[671] = -0.0352295274170269*G0_0_2 - 0.0352295274170268*G0_2_0 + 0.0962288017409539*G0_2_2;
    A[672] = -0.00480034930989791*G0_0_0 - 0.00480034930989792*G0_0_1 - 0.00480034930989794*G0_0_2 - 0.00480034930989788*G0_1_0 - 0.0048003493098979*G0_1_1 - 0.00480034930989793*G0_1_2;
    A[673] = -0.017870503981615*G0_0_0 + 0.0372571824394737*G0_1_0;
    A[674] = -0.0105310191030676*G0_0_1 + 0.00480034930989782*G0_1_1;
    A[675] = 0.004800349309898*G0_0_2 + 0.00480034930989798*G0_1_2;
    A[676] = -0.0274594594256049*G0_0_1 - 0.0328352243499807*G0_0_2 - 0.00537576492437588*G0_1_1 - 0.00537576492437596*G0_1_2;
    A[677] = -0.00887914570640257*G0_0_1 - 0.0111690649885096*G0_0_2 - 0.00228991928210658*G0_1_1 - 0.00228991928210669*G0_1_2;
    A[678] = -0.0116844599266475*G0_0_1 - 0.0139743792087539*G0_0_2 - 0.00228991928210683*G0_1_1 - 0.00228991928210661*G0_1_2;
    A[679] = -0.00537576492437565*G0_0_1 - 0.0107515298487518*G0_0_2 - 0.00537576492437576*G0_1_1 - 0.00537576492437598*G0_1_2;
    A[680] = 0.0895482395048355*G0_0_0 + 0.0699110399631223*G0_0_2 + 0.069911039963122*G0_1_0 + 0.139822079926245*G0_1_2;
    A[681] = -0.0574176057031948*G0_0_0 - 0.0699110399631223*G0_0_2 - 0.0387263842124943*G0_1_0 - 0.108637424175617*G0_1_2;
    A[682] = 0.0283728175438237*G0_0_0 + 0.0599162925551807*G0_0_2 + 0.0189782768992831*G0_1_0 + 0.0788945694544643*G0_1_2;
    A[683] = -0.00537576492437563*G0_0_0 - 0.0274594594256049*G0_0_2 - 0.00537576492437576*G0_1_0 - 0.032835224349981*G0_1_2;
    A[684] = 0.179096479009671*G0_0_0 + 0.0699110399631223*G0_0_1 + 0.0699110399631223*G0_1_0 + 0.139822079926245*G0_1_1;
    A[685] = -0.147063705006412*G0_0_0 + 0.0263830016434183*G0_0_1 - 0.0387263842124949*G0_1_0 - 0.108637424175617*G0_1_1;
    A[686] = 0.100717304493345*G0_0_0 - 0.0246345732456845*G0_0_1 + 0.0189782768992836*G0_1_0 + 0.078894569454464*G0_1_1;
    A[687] = -0.0328352243499808*G0_0_0 + 0.0209159009072206*G0_0_1 - 0.00537576492437601*G0_1_0 - 0.0328352243499806*G0_1_1;
    A[688] = 0.005375764924376*G0_0_0 + 0.00537576492437598*G0_0_1 + 0.00537576492437597*G0_1_0 + 0.00537576492437594*G0_1_1;
    A[689] = 0.00228991928210652*G0_0_0 + 0.00228991928210655*G0_0_1 + 0.00228991928210642*G0_1_0 + 0.00228991928210645*G0_1_1;
    A[690] = 0.00228991928210707*G0_0_0 + 0.0022899192821071*G0_0_1 + 0.00228991928210693*G0_1_0 + 0.00228991928210695*G0_1_1;
    A[691] = 0.00537576492437567*G0_0_0 + 0.00537576492437565*G0_0_1 + 0.00537576492437578*G0_1_0 + 0.00537576492437577*G0_1_1;
    A[692] = 0.010751529848752*G0_0_0 + 0.00537576492437603*G0_0_1 + 0.010751529848752*G0_0_2 + 0.00537576492437597*G0_1_0 + 0.0053757649243759*G0_1_2;
    A[693] = 0.013974379208754*G0_0_0 + 0.00228991928210661*G0_0_1 + 0.013974379208754*G0_0_2 + 0.00228991928210683*G0_1_0 + 0.00228991928210684*G0_1_2;
    A[694] = 0.0111690649885097*G0_0_0 + 0.00228991928210691*G0_0_1 + 0.0111690649885096*G0_0_2 + 0.00228991928210669*G0_1_0 + 0.0022899192821066*G0_1_2;
    A[695] = 0.0328352243499808*G0_0_0 + 0.00537576492437593*G0_0_1 + 0.0328352243499808*G0_0_2 + 0.00537576492437601*G0_1_0 + 0.00537576492437599*G0_1_2;
    A[696] = 0.022083694501229*G0_0_0 + 0.0274594594256049*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.0274594594256049*G0_1_0 + 0.0328352243499807*G0_1_1 + 0.0328352243499808*G0_1_2;
    A[697] = -0.0315434750113566*G0_0_0 - 0.0599162925551806*G0_0_1 - 0.0599162925551806*G0_0_2 - 0.0599162925551805*G0_1_0 - 0.078894569454464*G0_1_1 - 0.078894569454464*G0_1_2;
    A[698] = 0.0124934342599271*G0_0_0 + 0.0699110399631225*G0_0_1 + 0.0699110399631224*G0_0_2 + 0.0699110399631222*G0_1_0 + 0.108637424175617*G0_1_1 + 0.108637424175617*G0_1_2;
    A[699] = 0.0196371995417134*G0_0_0 - 0.0699110399631224*G0_0_1 - 0.0699110399631223*G0_0_2 - 0.0699110399631224*G0_1_0 - 0.139822079926245*G0_1_1 - 0.139822079926245*G0_1_2;
    A[700] = -0.0644569872000421*G0_0_0 + 0.0962940416065403*G0_0_1 + 0.192588083213081*G0_0_2 + 0.0284445813959704*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2;
    A[701] = 0.1273155976932*G0_0_0 - 0.00704590548340553*G0_0_1 - 0.0493213383838382*G0_0_2 - 0.000685018588664869*G0_1_0 - 0.0429604514890966*G0_1_1 - 0.0429604514890971*G0_1_2;
    A[702] = -0.0641307878721065*G0_0_0 + 0.0264873854283573*G0_0_1 + 0.0477555816097482*G0_0_2 - 0.00939454064454032*G0_1_0 + 0.0118736555368496*G0_1_1 + 0.0118736555368499*G0_1_2;
    A[703] = 0.0439716694056967*G0_0_0 - 0.0422754329004318*G0_0_1 - 0.0845508658008643*G0_0_2 - 0.000685018588664461*G0_1_0 - 0.0429604514890962*G0_1_1 - 0.0429604514890961*G0_1_2;
    A[704] = -0.0292600797158087*G0_0_0 + 0.00844856259352788*G0_0_1 + 0.0142875305635719*G0_0_2 - 0.00329461321214767*G0_1_0 + 0.00254435475789649*G0_1_1 + 0.00254435475789637*G0_1_2;
    A[705] = -0.0187890812890812*G0_0_0 + 0.0212681961813901*G0_0_1 + 0.0425363923627806*G0_0_2 - 0.00939454064454075*G0_1_0 + 0.0118736555368493*G0_1_1 + 0.0118736555368495*G0_1_2;
    A[706] = 0.0187890812890812*G0_0_0 + 0.00939454064454081*G0_0_1 + 0.00939454064454051*G0_1_0;
    A[707] = 0.0292600797158086*G0_0_0 + 0.00329461321214788*G0_0_1 + 0.00329461321214826*G0_1_0;
    A[708] = 0.0641307878721066*G0_0_0 + 0.00939454064454029*G0_0_1 + 0.00939454064454032*G0_1_0;
    A[709] = 0.006589226424296*G0_0_0 + 0.003294613212148*G0_0_1 + 0.00329461321214804*G0_1_0;
    A[710] = 0.0292600797158087*G0_0_0 + 0.00329461321214795*G0_0_1 + 0.00329461321214769*G0_1_0;
    A[711] = 0.0187890812890811*G0_0_0 + 0.00939454064454043*G0_0_1 + 0.00939454064454072*G0_1_0;
    A[712] = -0.030662736825931*G0_0_0 - 0.0212681961813906*G0_0_1 - 0.0212681961813904*G0_1_0 - 0.0118736555368503*G0_1_1;
    A[713] = 0.064261267603281*G0_0_0 + 0.0422754329004328*G0_0_1 + 0.0422754329004325*G0_1_0 + 0.0429604514890972*G0_1_1;
    A[714] = -0.128522535206562*G0_0_0 - 0.0962940416065406*G0_0_1 - 0.0962940416065401*G0_1_0 - 0.12473862300251*G0_1_1;
    A[715] = -0.00913358118219209*G0_0_0 - 0.00583896797004424*G0_0_1 - 0.00583896797004412*G0_1_0 - 0.00254435475789672*G0_1_1;
    A[716] = 0.0642612676032804*G0_0_0 + 0.042275432900432*G0_0_1 + 0.042275432900432*G0_1_0 + 0.0429604514890964*G0_1_1;
    A[717] = -0.0306627368259308*G0_0_0 - 0.0212681961813901*G0_0_1 - 0.02126819618139*G0_1_0 - 0.0118736555368492*G0_1_1;
    A[718] = -0.0613254736518622*G0_0_0 - 0.0212681961813905*G0_0_1 - 0.042536392362781*G0_0_2 - 0.0212681961813904*G0_1_0 - 0.01187365553685*G0_1_2;
    A[719] = 0.128522535206562*G0_0_0 + 0.0422754329004325*G0_0_1 + 0.0845508658008654*G0_0_2 + 0.0422754329004325*G0_1_0 + 0.0429604514890973*G0_1_2;
    A[720] = -0.257045070413124*G0_0_0 - 0.0962940416065407*G0_0_1 - 0.192588083213081*G0_0_2 - 0.0962940416065404*G0_1_0 - 0.124738623002511*G0_1_2;
    A[721] = -0.0435476102793807*G0_0_0 - 0.00583896797004385*G0_0_1 - 0.0142875305635718*G0_0_2 - 0.00583896797004431*G0_1_0 - 0.00254435475789617*G0_1_2;
    A[722] = 0.176636936077039*G0_0_0 + 0.0422754329004329*G0_0_1 + 0.0493213383838383*G0_0_2 + 0.0422754329004324*G0_1_0 + 0.042960451489097*G0_1_2;
    A[723] = -0.111886369481855*G0_0_0 - 0.0212681961813912*G0_0_1 - 0.0477555816097482*G0_0_2 - 0.0212681961813903*G0_1_0 - 0.0118736555368499*G0_1_2;
    A[724] = -0.0505608958299929*G0_0_0 - 0.00260959462348373*G0_0_1 - 0.00260959462348379*G0_1_0;
    A[725] = 0.096228801740954*G0_0_0 - 0.0352295274170267*G0_0_1 - 0.035229527417027*G0_1_0;
    A[726] = -0.156575677409009*G0_0_0 - 0.00260959462348323*G0_0_1 - 0.00260959462348339*G0_1_0;
    A[727] = -0.0505608958299929*G0_0_0 - 0.00260959462348366*G0_0_1 - 0.0026095946234836*G0_1_0;
    A[728] = -0.00336507226698187*G0_0_0 - 0.00336507226698187*G0_0_1 - 0.00336507226698186*G0_0_2 - 0.00336507226698191*G0_1_0 - 0.00336507226698189*G0_1_1 - 0.00336507226698187*G0_1_2;
    A[729] = 0.0214691349673987*G0_0_0 - 0.0336585514536899*G0_1_0;
    A[730] = 0.0232632312710435*G0_0_1 - 0.0155544887532733*G0_1_1;
    A[731] = 0.00336507226698197*G0_0_2 + 0.00336507226698197*G0_1_2;
    A[732] = 0.0599162925551808*G0_0_1 + 0.0788945694544643*G0_0_2 + 0.0283728175438241*G0_1_1 + 0.0189782768992836*G0_1_2;
    A[733] = 0.00578025209101613*G0_0_1 + 0.00523223722008478*G0_0_2 - 0.000482775005344528*G0_1_1 - 0.000548014870931567*G0_1_2;
    A[734] = 0.0131523569023567*G0_0_1 + 0.0263047138047134*G0_0_2 + 0.0132175967679438*G0_1_1 + 0.0131523569023567*G0_1_2;
    A[735] = -0.00228991928210672*G0_0_1 - 0.0139743792087541*G0_0_2 - 0.00228991928210681*G0_1_1 - 0.0116844599266473*G0_1_2;
    A[736] = -0.0574176057031955*G0_0_0 - 0.0387263842124949*G0_0_2 - 0.0699110399631224*G0_1_0 - 0.108637424175617*G0_1_2;
    A[737] = 0.00506261356955791*G0_0_0 + 0.00480165410720955*G0_0_2 + 0.00480165410720935*G0_1_0 + 0.00960330821441872*G0_1_2;
    A[738] = -0.000482775005344378*G0_0_0 + 0.00578025209101603*G0_0_2 - 0.000548014870931331*G0_1_0 + 0.00523223722008486*G0_1_2;
    A[739] = -0.00228991928210673*G0_0_0 - 0.00887914570640257*G0_0_2 - 0.00228991928210681*G0_1_0 - 0.0111690649885095*G0_1_2;
    A[740] = -0.147063705006412*G0_0_0 - 0.0387263842124949*G0_0_1 + 0.0263830016434183*G0_1_0 - 0.108637424175617*G0_1_1;
    A[741] = 0.304226541205705*G0_0_0 + 0.0781312630270956*G0_0_1 + 0.0781312630270956*G0_1_0 + 0.281001149056702*G0_1_1;
    A[742] = -0.204187731314466*G0_0_0 + 0.0153052724667308*G0_0_1 - 0.0364951808094165*G0_1_0 - 0.204187731314465*G0_1_1;
    A[743] = 0.0788945694544644*G0_0_0 - 0.0246345732456844*G0_0_1 + 0.0189782768992837*G0_1_0 + 0.100717304493345*G0_1_1;
    A[744] = 0.0022899192821068*G0_0_0 + 0.00228991928210682*G0_0_1 + 0.00228991928210677*G0_1_0 + 0.00228991928210678*G0_1_1;
    A[745] = 0.00178757231708629*G0_0_0 + 0.00178757231708626*G0_0_1 + 0.00178757231708655*G0_1_0 + 0.00178757231708653*G0_1_1;
    A[746] = 0.00178757231708621*G0_0_0 + 0.00178757231708623*G0_0_1 + 0.00178757231708616*G0_1_0 + 0.00178757231708617*G0_1_1;
    A[747] = 0.00228991928210669*G0_0_0 + 0.00228991928210671*G0_0_1 + 0.00228991928210679*G0_1_0 + 0.00228991928210681*G0_1_1;
    A[748] = 0.0139743792087539*G0_0_0 + 0.0116844599266471*G0_0_1 + 0.013974379208754*G0_0_2 + 0.0116844599266472*G0_1_0 + 0.00939454064454045*G0_1_1 + 0.0116844599266473*G0_1_2;
    A[749] = -0.0263047138047136*G0_0_0 - 0.0131523569023567*G0_0_1 - 0.0263047138047136*G0_0_2 - 0.013152356902357*G0_1_0 + 6.52398655870395e-05*G0_1_1 - 0.013152356902357*G0_1_2;
    A[750] = -0.00523223722008456*G0_0_0 + 0.000548014870931546*G0_0_1 - 0.00523223722008445*G0_0_2 + 0.000548014870931769*G0_1_0 + 6.52398655872977e-05*G0_1_1 + 0.000548014870931864*G0_1_2;
    A[751] = -0.0788945694544645*G0_0_0 - 0.0189782768992837*G0_0_1 - 0.0788945694544644*G0_0_2 - 0.0189782768992837*G0_1_0 + 0.00939454064454035*G0_1_1 - 0.0189782768992837*G0_1_2;
    A[752] = 0.0065892264242955*G0_0_0 + 0.00887914570640243*G0_0_1 + 0.00887914570640238*G0_0_2 + 0.00887914570640243*G0_1_0 + 0.0111690649885093*G0_1_1 + 0.0111690649885092*G0_1_2;
    A[753] = -0.00626302709636011*G0_0_0 - 0.00578025209101587*G0_0_1 - 0.00578025209101579*G0_0_2 - 0.00578025209101589*G0_1_0 - 0.00523223722008462*G0_1_1 - 0.00523223722008453*G0_1_2;
    A[754] = 0.000260959462348327*G0_0_0 - 0.00480165410720953*G0_0_1 - 0.00480165410720957*G0_0_2 - 0.00480165410720943*G0_1_0 - 0.00960330821441881*G0_1_1 - 0.00960330821441885*G0_1_2;
    A[755] = -0.0186912214907006*G0_0_0 + 0.0387263842124949*G0_0_1 + 0.0387263842124949*G0_0_2 + 0.0387263842124947*G0_1_0 + 0.108637424175617*G0_1_1 + 0.108637424175617*G0_1_2;
    A[756] = 0.132306447410613*G0_0_0 - 0.0623693115012553*G0_0_1 + 0.0109602974186309*G0_0_2 + 0.0733296089198858*G0_1_0 - 0.12473862300251*G0_1_1 + 0.146659217839773*G0_1_2;
    A[757] = -0.166948816037356*G0_0_0 + 0.0838658472122008*G0_0_1 + 0.167731694424402*G0_0_2 - 0.0061651672979795*G0_1_0 + 0.182410664181496*G0_1_1 + 0.077700679914221*G0_1_2;
    A[758] = 0.127087258163646*G0_0_0 - 0.0463855444324192*G0_0_1 - 0.0823327103709046*G0_0_2 + 0.0377738821749232*G0_1_0 - 0.0510175748891022*G0_1_1 + 0.00182671623643828*G0_1_2;
    A[759] = -0.0436454700777611*G0_0_0 - 0.0064913666259153*G0_0_1 - 0.0808321934624013*G0_0_2 - 0.0338921101724914*G0_1_0 - 0.00352295274170298*G0_1_1 - 0.108232937008978*G0_1_2;
    A[760] = 0.0189848008858424*G0_0_0 - 0.0188869410874619*G0_0_1 - 0.0377738821749235*G0_0_2 + 0.00949240044292091*G0_1_0 - 0.0155923278753139*G0_1_1 - 0.00939454064454069*G0_1_2;
    A[761] = 0.0148746893538561*G0_0_0 + 0.00123955744615497*G0_0_1 + 0.0403182369328202*G0_0_2 + 0.014939929219443*G0_1_0 + 0.00117431758056787*G0_1_1 + 0.0540186087061087*G0_1_2;
    A[762] = -0.0148746893538561*G0_0_0 - 0.0149399292194432*G0_0_1 - 0.0149399292194431*G0_1_0 - 0.0150051690850303*G0_1_1;
    A[763] = -0.0189848008858422*G0_0_0 - 0.00949240044292086*G0_0_1 - 0.00949240044292162*G0_1_0 - 0.000652398655870534*G0_1_1;
    A[764] = -0.127087258163646*G0_0_0 - 0.0377738821749235*G0_0_1 - 0.0377738821749232*G0_1_0 - 0.0150051690850301*G0_1_1;
    A[765] = 0.00450155072550902*G0_0_0 + 0.00192457603481896*G0_0_1 + 0.00192457603481895*G0_1_0 - 0.000652398655871434*G0_1_1;
    A[766] = -0.0189848008858424*G0_0_0 - 0.00949240044292126*G0_0_1 - 0.00949240044292092*G0_1_0 - 0.000652398655870431*G0_1_1;
    A[767] = -0.014874689353856*G0_0_0 - 0.0149399292194429*G0_0_1 - 0.0149399292194429*G0_1_0 - 0.01500516908503*G0_1_1;
    A[768] = -0.00130479731174161*G0_0_0 - 0.00123955744615445*G0_0_1 - 0.00123955744615443*G0_1_0 - 0.00117431758056696*G0_1_1;
    A[769] = 0.00880738185425657*G0_0_0 + 0.00649136662591476*G0_0_1 + 0.00649136662591476*G0_1_0 + 0.00352295274170217*G0_1_1;
    A[770] = 0.0476251018785738*G0_0_0 + 0.0623693115012555*G0_0_1 + 0.0623693115012553*G0_1_0 + 0.124738623002511*G0_1_1;
    A[771] = -0.0156575677409011*G0_0_0 - 0.0130805930502109*G0_0_1 - 0.0130805930502113*G0_1_0 - 0.0105036183595208*G0_1_1;
    A[772] = 0.00880738185425689*G0_0_0 + 0.0064913666259152*G0_0_1 + 0.0064913666259154*G0_1_0 + 0.00352295274170283*G0_1_1;
    A[773] = -0.0013047973117418*G0_0_0 - 0.00123955744615495*G0_0_1 - 0.0012395574461548*G0_1_0 - 0.00117431758056788*G0_1_1;
    A[774] = -0.0254435475789633*G0_0_0 - 0.0390786794866648*G0_0_1 - 0.0403182369328195*G0_0_2 - 0.0390786794866649*G0_1_0 - 0.0528442911255405*G0_1_1 - 0.0540186087061081*G0_1_2;
    A[775] = 0.0371867233846388*G0_0_0 + 0.0743408268364855*G0_0_1 + 0.0808321934624004*G0_0_2 + 0.0743408268364854*G0_1_0 + 0.104709984267275*G0_1_1 + 0.108232937008977*G0_1_2;
    A[776] = 0.121346149991983*G0_0_0 - 0.073329608919886*G0_0_1 - 0.0109602974186305*G0_0_2 - 0.073329608919886*G0_1_0 - 0.271397840842283*G0_1_1 - 0.146659217839772*G0_1_2;
    A[777] = 0.0567586830607657*G0_0_0 + 0.0188869410874612*G0_0_1 + 0.0377738821749232*G0_0_2 + 0.018886941087462*G0_1_0 - 0.00619778723077337*G0_1_1 + 0.00939454064454057*G0_1_2;
    A[778] = -0.334680510461757*G0_0_0 - 0.0838658472122006*G0_0_1 - 0.167731694424402*G0_0_2 - 0.0838658472122006*G0_1_0 + 0.104709984267275*G0_1_1 - 0.077700679914221*G0_1_2;
    A[779] = 0.20941996853455*G0_0_0 + 0.0359471659384858*G0_0_1 + 0.0823327103709046*G0_0_2 + 0.0359471659384848*G0_1_0 - 0.0528442911255403*G0_1_1 - 0.00182671623643833*G0_1_2;
    A[780] = 0.0391439193522525*G0_0_0 + 0.0319675341376727*G0_0_1 + 0.0319675341376733*G0_1_0 + 0.0260959462348343*G0_1_1;
    A[781] = -0.156575677409009*G0_0_0 - 0.0903572138381162*G0_0_1 - 0.090357213838116*G0_1_0 - 0.185933616923199*G0_1_1;
    A[782] = 0.185933616923199*G0_0_0 + 0.0156575677409014*G0_0_1 + 0.0156575677409012*G0_1_0 + 0.0260959462348355*G0_1_1;
    A[783] = 0.0391439193522523*G0_0_0 + 0.0319675341376728*G0_0_1 + 0.0319675341376726*G0_1_0 + 0.0260959462348347*G0_1_1;
    A[784] = -0.00336507226698201*G0_0_0 - 0.00336507226698201*G0_0_1 - 0.00336507226698201*G0_0_2 - 0.00336507226698199*G0_1_0 - 0.00336507226698202*G0_1_1 - 0.00336507226698202*G0_1_2;
    A[785] = -0.0155544887532733*G0_0_0 + 0.0232632312710435*G0_1_0;
    A[786] = -0.03365855145369*G0_0_1 + 0.0214691349673986*G0_1_1;
    A[787] = 0.00336507226698199*G0_0_2 + 0.003365072266982*G0_1_2;
    A[788] = -0.0699110399631223*G0_0_1 - 0.108637424175617*G0_0_2 - 0.0574176057031951*G0_1_1 - 0.0387263842124947*G0_1_2;
    A[789] = 0.00480165410720946*G0_0_1 + 0.009603308214419*G0_0_2 + 0.00506261356955801*G0_1_1 + 0.00480165410720958*G0_1_2;
    A[790] = -0.000548014870931348*G0_0_1 + 0.00523223722008463*G0_0_2 - 0.000482775005344278*G0_1_1 + 0.00578025209101623*G0_1_2;
    A[791] = -0.00228991928210676*G0_0_1 - 0.0111690649885093*G0_0_2 - 0.00228991928210652*G0_1_1 - 0.0088791457064025*G0_1_2;
    A[792] = 0.0283728175438242*G0_0_0 + 0.0189782768992836*G0_0_2 + 0.0599162925551805*G0_1_0 + 0.0788945694544639*G0_1_2;
    A[793] = -0.000482775005344392*G0_0_0 - 0.000548014870931617*G0_0_2 + 0.00578025209101656*G0_1_0 + 0.00523223722008512*G0_1_2;
    A[794] = 0.0132175967679437*G0_0_0 + 0.0131523569023568*G0_0_2 + 0.0131523569023562*G0_1_0 + 0.0263047138047129*G0_1_2;
    A[795] = -0.00228991928210674*G0_0_0 - 0.0116844599266474*G0_0_2 - 0.00228991928210653*G0_1_0 - 0.0139743792087539*G0_1_2;
    A[796] = 0.100717304493345*G0_0_0 + 0.0189782768992836*G0_0_1 - 0.0246345732456845*G0_1_0 + 0.078894569454464*G0_1_1;
    A[797] = -0.204187731314466*G0_0_0 - 0.0364951808094165*G0_0_1 + 0.0153052724667308*G0_1_0 - 0.204187731314465*G0_1_1;
    A[798] = 0.281001149056702*G0_0_0 + 0.0781312630270953*G0_0_1 + 0.0781312630270953*G0_1_0 + 0.304226541205704*G0_1_1;
    A[799] = -0.108637424175617*G0_0_0 + 0.0263830016434184*G0_0_1 - 0.0387263842124948*G0_1_0 - 0.147063705006411*G0_1_1;
    A[800] = 0.00228991928210657*G0_0_0 + 0.00228991928210658*G0_0_1 + 0.00228991928210663*G0_1_0 + 0.00228991928210664*G0_1_1;
    A[801] = 0.00178757231708616*G0_0_0 + 0.00178757231708617*G0_0_1 + 0.00178757231708586*G0_1_0 + 0.00178757231708588*G0_1_1;
    A[802] = 0.00178757231708623*G0_0_0 + 0.00178757231708621*G0_0_1 + 0.00178757231708637*G0_1_0 + 0.00178757231708636*G0_1_1;
    A[803] = 0.00228991928210678*G0_0_0 + 0.00228991928210677*G0_0_1 + 0.00228991928210656*G0_1_0 + 0.00228991928210655*G0_1_1;
    A[804] = 0.0111690649885093*G0_0_0 + 0.00887914570640263*G0_0_1 + 0.0111690649885092*G0_0_2 + 0.00887914570640254*G0_1_0 + 0.00658922642429587*G0_1_1 + 0.00887914570640251*G0_1_2;
    A[805] = -0.00523223722008423*G0_0_0 - 0.00578025209101593*G0_0_1 - 0.00523223722008419*G0_0_2 - 0.00578025209101574*G0_1_0 - 0.00626302709636035*G0_1_1 - 0.00578025209101574*G0_1_2;
    A[806] = -0.00960330821441936*G0_0_0 - 0.00480165410720966*G0_0_1 - 0.00960330821441947*G0_0_2 - 0.00480165410720971*G0_1_0 + 0.000260959462348246*G0_1_1 - 0.00480165410720981*G0_1_2;
    A[807] = 0.108637424175618*G0_0_0 + 0.0387263842124951*G0_0_1 + 0.108637424175617*G0_0_2 + 0.0387263842124948*G0_1_0 - 0.0186912214907003*G0_1_1 + 0.0387263842124948*G0_1_2;
    A[808] = 0.00939454064454066*G0_0_0 + 0.0116844599266472*G0_0_1 + 0.0116844599266472*G0_0_2 + 0.0116844599266473*G0_1_0 + 0.013974379208754*G0_1_1 + 0.013974379208754*G0_1_2;
    A[809] = 6.5239865587041e-05*G0_0_0 - 0.0131523569023567*G0_0_1 - 0.0131523569023568*G0_0_2 - 0.0131523569023567*G0_1_0 - 0.0263047138047132*G0_1_1 - 0.0263047138047133*G0_1_2;
    A[810] = 6.5239865587084e-05*G0_0_0 + 0.000548014870931444*G0_0_1 + 0.000548014870931459*G0_0_2 + 0.000548014870931434*G0_1_0 - 0.00523223722008469*G0_1_1 - 0.00523223722008466*G0_1_2;
    A[811] = 0.00939454064454058*G0_0_0 - 0.0189782768992837*G0_0_1 - 0.0189782768992836*G0_0_2 - 0.0189782768992835*G0_1_0 - 0.078894569454464*G0_1_1 - 0.0788945694544639*G0_1_2;
    A[812] = -0.0510175748891022*G0_0_0 + 0.0377738821749235*G0_0_1 + 0.0018267162364384*G0_0_2 - 0.0463855444324186*G0_1_0 + 0.127087258163646*G0_1_1 - 0.0823327103709047*G0_1_2;
    A[813] = 0.182410664181495*G0_0_0 - 0.00616516729798044*G0_0_1 + 0.07770067991422*G0_0_2 + 0.0838658472121996*G0_1_0 - 0.166948816037357*G0_1_1 + 0.1677316944244*G0_1_2;
    A[814] = -0.12473862300251*G0_0_0 + 0.0733296089198861*G0_0_1 + 0.146659217839773*G0_0_2 - 0.0623693115012549*G0_1_0 + 0.132306447410613*G0_1_1 + 0.0109602974186309*G0_1_2;
    A[815] = -0.0155923278753141*G0_0_0 + 0.00949240044292112*G0_0_1 - 0.00939454064454064*G0_0_2 - 0.0188869410874623*G0_1_0 + 0.0189848008858427*G0_1_1 - 0.0377738821749232*G0_1_2;
    A[816] = -0.00352295274170235*G0_0_0 - 0.0338921101724912*G0_0_1 - 0.108232937008978*G0_0_2 - 0.00649136662591467*G0_1_0 - 0.0436454700777609*G0_1_1 - 0.080832193462401*G0_1_2;
    A[817] = 0.00117431758056761*G0_0_0 + 0.0149399292194427*G0_0_1 + 0.0540186087061083*G0_0_2 + 0.00123955744615494*G0_1_0 + 0.0148746893538555*G0_1_1 + 0.0403182369328197*G0_1_2;
    A[818] = -0.00117431758056746*G0_0_0 - 0.00123955744615421*G0_0_1 - 0.00123955744615464*G0_1_0 - 0.00130479731174134*G0_1_1;
    A[819] = 0.0035229527417024*G0_0_0 + 0.00649136662591455*G0_0_1 + 0.00649136662591558*G0_1_0 + 0.0088073818542564*G0_1_1;
    A[820] = 0.12473862300251*G0_0_0 + 0.0623693115012555*G0_0_1 + 0.0623693115012549*G0_1_0 + 0.0476251018785735*G0_1_1;
    A[821] = -0.0105036183595211*G0_0_0 - 0.013080593050211*G0_0_1 - 0.0130805930502109*G0_1_0 - 0.0156575677409005*G0_1_1;
    A[822] = 0.00352295274170233*G0_0_0 + 0.00649136662591529*G0_0_1 + 0.00649136662591467*G0_1_0 + 0.00880738185425654*G0_1_1;
    A[823] = -0.00117431758056766*G0_0_0 - 0.00123955744615483*G0_0_1 - 0.00123955744615495*G0_1_0 - 0.00130479731174205*G0_1_1;
    A[824] = -0.0150051690850299*G0_0_0 - 0.0149399292194432*G0_0_1 - 0.0149399292194428*G0_1_0 - 0.0148746893538563*G0_1_1;
    A[825] = -0.000652398655870868*G0_0_0 - 0.00949240044292114*G0_0_1 - 0.00949240044292102*G0_1_0 - 0.018984800885842*G0_1_1;
    A[826] = -0.0150051690850301*G0_0_0 - 0.0377738821749235*G0_0_1 - 0.0377738821749233*G0_1_0 - 0.127087258163646*G0_1_1;
    A[827] = -0.000652398655871078*G0_0_0 + 0.00192457603481873*G0_0_1 + 0.00192457603481911*G0_1_0 + 0.0045015507255086*G0_1_1;
    A[828] = -0.000652398655870735*G0_0_0 - 0.00949240044292111*G0_0_1 - 0.00949240044292156*G0_1_0 - 0.0189848008858426*G0_1_1;
    A[829] = -0.01500516908503*G0_0_0 - 0.0149399292194427*G0_0_1 - 0.0149399292194426*G0_1_0 - 0.0148746893538554*G0_1_1;
    A[830] = -0.0528442911255407*G0_0_0 - 0.039078679486665*G0_0_1 - 0.0540186087061082*G0_0_2 - 0.039078679486665*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.0403182369328196*G0_1_2;
    A[831] = -0.00619778723077302*G0_0_0 + 0.0188869410874619*G0_0_1 + 0.00939454064454087*G0_0_2 + 0.0188869410874617*G0_1_0 + 0.0567586830607656*G0_1_1 + 0.0377738821749236*G0_1_2;
    A[832] = -0.0528442911255407*G0_0_0 + 0.0359471659384851*G0_0_1 - 0.00182671623643855*G0_0_2 + 0.0359471659384853*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0823327103709045*G0_1_2;
    A[833] = 0.104709984267275*G0_0_0 + 0.0743408268364858*G0_0_1 + 0.108232937008978*G0_0_2 + 0.0743408268364854*G0_1_0 + 0.0371867233846395*G0_1_1 + 0.0808321934624009*G0_1_2;
    A[834] = 0.104709984267275*G0_0_0 - 0.0838658472122003*G0_0_1 - 0.0777006799142199*G0_0_2 - 0.0838658472122005*G0_1_0 - 0.334680510461757*G0_1_1 - 0.1677316944244*G0_1_2;
    A[835] = -0.271397840842282*G0_0_0 - 0.0733296089198865*G0_0_1 - 0.146659217839772*G0_0_2 - 0.0733296089198856*G0_1_0 + 0.121346149991982*G0_1_1 - 0.0109602974186309*G0_1_2;
    A[836] = 0.0260959462348347*G0_0_0 + 0.0319675341376731*G0_0_1 + 0.0319675341376724*G0_1_0 + 0.039143919352253*G0_1_1;
    A[837] = 0.0260959462348351*G0_0_0 + 0.0156575677409017*G0_0_1 + 0.015657567740901*G0_1_0 + 0.185933616923199*G0_1_1;
    A[838] = -0.185933616923198*G0_0_0 - 0.0903572138381162*G0_0_1 - 0.0903572138381153*G0_1_0 - 0.15657567740901*G0_1_1;
    A[839] = 0.0260959462348351*G0_0_0 + 0.0319675341376725*G0_0_1 + 0.0319675341376731*G0_1_0 + 0.0391439193522522*G0_1_1;
    A[840] = -0.00480034930989779*G0_0_0 - 0.00480034930989781*G0_0_1 - 0.00480034930989781*G0_0_2 - 0.00480034930989781*G0_1_0 - 0.00480034930989782*G0_1_1 - 0.0048003493098978*G0_1_2;
    A[841] = 0.0048003493098978*G0_0_0 - 0.0105310191030676*G0_1_0;
    A[842] = 0.0372571824394738*G0_0_1 - 0.0178705039816149*G0_1_1;
    A[843] = 0.00480034930989792*G0_0_2 + 0.0048003493098979*G0_1_2;
    A[844] = 0.0699110399631224*G0_0_1 + 0.139822079926245*G0_0_2 + 0.0895482395048356*G0_1_1 + 0.0699110399631224*G0_1_2;
    A[845] = -0.0387263842124946*G0_0_1 - 0.108637424175617*G0_0_2 - 0.0574176057031953*G0_1_1 - 0.0699110399631225*G0_1_2;
    A[846] = 0.0189782768992835*G0_0_1 + 0.0788945694544645*G0_0_2 + 0.0283728175438241*G0_1_1 + 0.0599162925551808*G0_1_2;
    A[847] = -0.00537576492437596*G0_0_1 - 0.0328352243499812*G0_0_2 - 0.00537576492437621*G0_1_1 - 0.0274594594256053*G0_1_2;
    A[848] = -0.00537576492437589*G0_0_0 - 0.005375764924376*G0_0_2 - 0.0274594594256046*G0_1_0 - 0.0328352243499806*G0_1_2;
    A[849] = -0.00228991928210681*G0_0_0 - 0.00228991928210657*G0_0_2 - 0.00887914570640306*G0_1_0 - 0.0111690649885097*G0_1_2;
    A[850] = -0.00228991928210671*G0_0_0 - 0.00228991928210675*G0_0_2 - 0.0116844599266469*G0_1_0 - 0.0139743792087536*G0_1_2;
    A[851] = -0.00537576492437598*G0_0_0 - 0.00537576492437602*G0_0_2 - 0.00537576492437622*G0_1_0 - 0.0107515298487523*G0_1_2;
    A[852] = -0.0328352243499808*G0_0_0 - 0.00537576492437601*G0_0_1 + 0.0209159009072206*G0_1_0 - 0.0328352243499806*G0_1_1;
    A[853] = 0.0788945694544644*G0_0_0 + 0.0189782768992837*G0_0_1 - 0.0246345732456844*G0_1_0 + 0.100717304493345*G0_1_1;
    A[854] = -0.108637424175617*G0_0_0 - 0.0387263842124948*G0_0_1 + 0.0263830016434184*G0_1_0 - 0.147063705006411*G0_1_1;
    A[855] = 0.139822079926245*G0_0_0 + 0.0699110399631225*G0_0_1 + 0.0699110399631225*G0_1_0 + 0.179096479009671*G0_1_1;
    A[856] = 0.00537576492437604*G0_0_0 + 0.00537576492437609*G0_0_1 + 0.00537576492437595*G0_1_0 + 0.00537576492437602*G0_1_1;
    A[857] = 0.00228991928210682*G0_0_0 + 0.00228991928210675*G0_0_1 + 0.00228991928210711*G0_1_0 + 0.00228991928210702*G0_1_1;
    A[858] = 0.00228991928210675*G0_0_0 + 0.00228991928210678*G0_0_1 + 0.00228991928210654*G0_1_0 + 0.00228991928210657*G0_1_1;
    A[859] = 0.00537576492437598*G0_0_0 + 0.00537576492437599*G0_0_1 + 0.00537576492437622*G0_1_0 + 0.00537576492437622*G0_1_1;
    A[860] = 0.0328352243499809*G0_0_0 + 0.0274594594256048*G0_0_1 + 0.032835224349981*G0_0_2 + 0.0274594594256048*G0_1_0 + 0.0220836945012289*G0_1_1 + 0.0274594594256048*G0_1_2;
    A[861] = -0.0788945694544646*G0_0_0 - 0.0599162925551808*G0_0_1 - 0.0788945694544647*G0_0_2 - 0.0599162925551808*G0_1_0 - 0.0315434750113565*G0_1_1 - 0.0599162925551809*G0_1_2;
    A[862] = 0.108637424175618*G0_0_0 + 0.0699110399631227*G0_0_1 + 0.108637424175618*G0_0_2 + 0.0699110399631225*G0_1_0 + 0.0124934342599272*G0_1_1 + 0.0699110399631226*G0_1_2;
    A[863] = -0.139822079926245*G0_0_0 - 0.0699110399631228*G0_0_1 - 0.139822079926245*G0_0_2 - 0.0699110399631225*G0_1_0 + 0.0196371995417132*G0_1_1 - 0.0699110399631225*G0_1_2;
    A[864] = 0.00537576492437609*G0_0_1 + 0.00537576492437606*G0_0_2 + 0.00537576492437584*G0_1_0 + 0.010751529848752*G0_1_1 + 0.0107515298487519*G0_1_2;
    A[865] = 0.00228991928210677*G0_0_1 + 0.00228991928210681*G0_0_2 + 0.00228991928210678*G0_1_0 + 0.0139743792087539*G0_1_1 + 0.0139743792087539*G0_1_2;
    A[866] = 0.00228991928210687*G0_0_1 + 0.00228991928210687*G0_0_2 + 0.00228991928210683*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.0111690649885095*G0_1_2;
    A[867] = 0.00537576492437607*G0_0_1 + 0.00537576492437602*G0_0_2 + 0.00537576492437597*G0_1_0 + 0.0328352243499807*G0_1_1 + 0.0328352243499806*G0_1_2;
    A[868] = 0.0118736555368497*G0_0_0 - 0.00939454064454059*G0_0_1 + 0.0118736555368499*G0_0_2 + 0.026487385428357*G0_1_0 - 0.0641307878721065*G0_1_1 + 0.0477555816097483*G0_1_2;
    A[869] = -0.0429604514890966*G0_0_0 - 0.0006850185886642*G0_0_1 - 0.0429604514890966*G0_0_2 - 0.007045905483405*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0493213383838378*G0_1_2;
    A[870] = 0.12473862300251*G0_0_0 + 0.02844458139597*G0_0_1 + 0.12473862300251*G0_0_2 + 0.0962940416065402*G0_1_0 - 0.0644569872000423*G0_1_1 + 0.192588083213081*G0_1_2;
    A[871] = 0.00254435475789691*G0_0_0 - 0.00329461321214755*G0_0_1 + 0.00254435475789637*G0_0_2 + 0.00844856259352857*G0_1_0 - 0.0292600797158085*G0_1_1 + 0.0142875305635719*G0_1_2;
    A[872] = -0.0429604514890967*G0_0_0 - 0.000685018588664455*G0_0_1 - 0.0429604514890967*G0_0_2 - 0.0422754329004325*G0_1_0 + 0.0439716694056968*G0_1_1 - 0.0845508658008649*G0_1_2;
    A[873] = 0.0118736555368495*G0_0_0 - 0.00939454064454067*G0_0_1 + 0.01187365553685*G0_0_2 + 0.0212681961813898*G0_1_0 - 0.0187890812890812*G0_1_1 + 0.0425363923627812*G0_1_2;
    A[874] = -0.0118736555368497*G0_0_0 - 0.0212681961813907*G0_0_1 - 0.0212681961813901*G0_1_0 - 0.0306627368259312*G0_1_1;
    A[875] = 0.0429604514890968*G0_0_0 + 0.0422754329004329*G0_0_1 + 0.042275432900432*G0_1_0 + 0.0642612676032811*G0_1_1;
    A[876] = -0.124738623002511*G0_0_0 - 0.0962940416065407*G0_0_1 - 0.0962940416065402*G0_1_0 - 0.128522535206562*G0_1_1;
    A[877] = -0.00254435475789676*G0_0_0 - 0.00583896797004438*G0_0_1 - 0.00583896797004473*G0_1_0 - 0.00913358118219253*G0_1_1;
    A[878] = 0.0429604514890967*G0_0_0 + 0.0422754329004324*G0_0_1 + 0.0422754329004325*G0_1_0 + 0.0642612676032809*G0_1_1;
    A[879] = -0.0118736555368495*G0_0_0 - 0.0212681961813903*G0_0_1 - 0.0212681961813898*G0_1_0 - 0.0306627368259307*G0_1_1;
    A[880] = 0.00939454064454087*G0_0_1 + 0.00939454064454066*G0_1_0 + 0.0187890812890815*G0_1_1;
    A[881] = 0.00329461321214798*G0_0_1 + 0.00329461321214777*G0_1_0 + 0.0292600797158083*G0_1_1;
    A[882] = 0.00939454064454048*G0_0_1 + 0.00939454064454026*G0_1_0 + 0.0641307878721065*G0_1_1;
    A[883] = 0.00329461321214781*G0_0_1 + 0.00329461321214775*G0_1_0 + 0.00658922642429577*G0_1_1;
    A[884] = 0.00329461321214764*G0_0_1 + 0.00329461321214818*G0_1_0 + 0.0292600797158085*G0_1_1;
    A[885] = 0.00939454064454064*G0_0_1 + 0.00939454064454038*G0_1_0 + 0.0187890812890812*G0_1_1;
    A[886] = -0.0212681961813905*G0_0_1 - 0.0118736555368496*G0_0_2 - 0.0212681961813902*G0_1_0 - 0.0613254736518619*G0_1_1 - 0.0425363923627808*G0_1_2;
    A[887] = -0.00583896797004437*G0_0_1 - 0.00254435475789649*G0_0_2 - 0.00583896797004441*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.0142875305635722*G0_1_2;
    A[888] = -0.0212681961813907*G0_0_1 - 0.0118736555368499*G0_0_2 - 0.0212681961813908*G0_1_0 - 0.111886369481855*G0_1_1 - 0.0477555816097482*G0_1_2;
    A[889] = 0.0422754329004322*G0_0_1 + 0.0429604514890965*G0_0_2 + 0.0422754329004325*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0845508658008646*G0_1_2;
    A[890] = 0.0422754329004324*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.0422754329004329*G0_1_0 + 0.176636936077038*G0_1_1 + 0.0493213383838379*G0_1_2;
    A[891] = -0.0962940416065401*G0_0_1 - 0.12473862300251*G0_0_2 - 0.0962940416065409*G0_1_0 - 0.257045070413123*G0_1_1 - 0.192588083213081*G0_1_2;
    A[892] = -0.00260959462348453*G0_0_1 - 0.00260959462348416*G0_1_0 - 0.0505608958299936*G0_1_1;
    A[893] = -0.00260959462348373*G0_0_1 - 0.00260959462348273*G0_1_0 - 0.156575677409009*G0_1_1;
    A[894] = -0.0352295274170263*G0_0_1 - 0.0352295274170269*G0_1_0 + 0.0962288017409544*G0_1_1;
    A[895] = -0.00260959462348352*G0_0_1 - 0.00260959462348383*G0_1_0 - 0.0505608958299926*G0_1_1;
    A[896] = -0.0178705039816149*G0_0_0 - 0.0178705039816149*G0_0_1 - 0.0178705039816149*G0_0_2 - 0.0178705039816149*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0178705039816149*G0_1_2 - 0.0551276864210886*G0_2_0 - 0.0551276864210886*G0_2_1 - 0.0551276864210886*G0_2_2;
    A[897] = -0.00480034930989792*G0_0_0 - 0.00480034930989793*G0_1_0;
    A[898] = -0.00480034930989784*G0_0_1 - 0.00480034930989784*G0_1_1;
    A[899] = 0.0105310191030676*G0_0_2 + 0.0105310191030676*G0_1_2 + 0.0153313684129654*G0_2_2;
    A[900] = 0.0107515298487519*G0_0_1 + 0.00537576492437602*G0_0_2 + 0.010751529848752*G0_1_1 + 0.00537576492437608*G0_1_2 + 0.00537576492437602*G0_2_1;
    A[901] = 0.0139743792087542*G0_0_1 + 0.0116844599266473*G0_0_2 + 0.0139743792087541*G0_1_1 + 0.0116844599266472*G0_1_2 + 0.0116844599266473*G0_2_1 + 0.00939454064454072*G0_2_2;
    A[902] = 0.0111690649885093*G0_0_1 + 0.00887914570640267*G0_0_2 + 0.0111690649885094*G0_1_1 + 0.00887914570640273*G0_1_2 + 0.0088791457064026*G0_2_1 + 0.00658922642429571*G0_2_2;
    A[903] = 0.032835224349981*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.0328352243499809*G0_1_1 + 0.0274594594256049*G0_1_2 + 0.027459459425605*G0_2_1 + 0.022083694501229*G0_2_2;
    A[904] = 0.010751529848752*G0_0_0 + 0.00537576492437599*G0_0_2 + 0.010751529848752*G0_1_0 + 0.00537576492437596*G0_1_2 + 0.00537576492437615*G0_2_0;
    A[905] = 0.0139743792087538*G0_0_0 + 0.0116844599266471*G0_0_2 + 0.0139743792087538*G0_1_0 + 0.0116844599266471*G0_1_2 + 0.0116844599266472*G0_2_0 + 0.00939454064454046*G0_2_2;
    A[906] = 0.0111690649885093*G0_0_0 + 0.00887914570640263*G0_0_2 + 0.0111690649885093*G0_1_0 + 0.00887914570640262*G0_1_2 + 0.00887914570640253*G0_2_0 + 0.00658922642429581*G0_2_2;
    A[907] = 0.032835224349981*G0_0_0 + 0.0274594594256049*G0_0_2 + 0.0328352243499809*G0_1_0 + 0.0274594594256049*G0_1_2 + 0.027459459425605*G0_2_0 + 0.0220836945012289*G0_2_2;
    A[908] = 0.005375764924376*G0_0_0 + 0.00537576492437597*G0_0_1 + 0.00537576492437598*G0_1_0 + 0.00537576492437594*G0_1_1;
    A[909] = 0.0022899192821068*G0_0_0 + 0.00228991928210677*G0_0_1 + 0.00228991928210681*G0_1_0 + 0.00228991928210678*G0_1_1;
    A[910] = 0.00228991928210657*G0_0_0 + 0.00228991928210663*G0_0_1 + 0.00228991928210658*G0_1_0 + 0.00228991928210664*G0_1_1;
    A[911] = 0.00537576492437604*G0_0_0 + 0.00537576492437595*G0_0_1 + 0.00537576492437609*G0_1_0 + 0.00537576492437601*G0_1_1;
    A[912] = 0.179096479009671*G0_0_0 + 0.179096479009671*G0_0_1 + 0.109185439046549*G0_0_2 + 0.179096479009671*G0_1_0 + 0.179096479009671*G0_1_1 + 0.109185439046549*G0_1_2 + 0.109185439046549*G0_2_0 + 0.109185439046549*G0_2_1 + 0.179096479009672*G0_2_2;
    A[913] = -0.147063705006412*G0_0_0 - 0.147063705006412*G0_0_1 - 0.17344670664983*G0_0_2 - 0.147063705006412*G0_1_0 - 0.147063705006412*G0_1_1 - 0.17344670664983*G0_1_2 - 0.108337320793917*G0_2_0 - 0.108337320793917*G0_2_1 - 0.243357746612953*G0_2_2;
    A[914] = 0.100717304493345*G0_0_0 + 0.100717304493345*G0_0_1 + 0.125351877739029*G0_0_2 + 0.100717304493345*G0_1_0 + 0.100717304493345*G0_1_1 + 0.125351877739029*G0_1_2 + 0.0817390275940614*G0_2_0 + 0.0817390275940614*G0_2_1 + 0.185268170294211*G0_2_2;
    A[915] = -0.0328352243499809*G0_0_0 - 0.0328352243499809*G0_0_1 - 0.053751125257201*G0_0_2 - 0.0328352243499809*G0_1_0 - 0.0328352243499808*G0_1_1 - 0.053751125257201*G0_1_2 - 0.027459459425605*G0_2_0 - 0.027459459425605*G0_2_1 - 0.0812105846828062*G0_2_2;
    A[916] = 0.0895482395048357*G0_0_0 + 0.0196371995417133*G0_0_1 + 0.0895482395048356*G0_0_2 + 0.0895482395048357*G0_1_0 + 0.0196371995417133*G0_1_1 + 0.0895482395048356*G0_1_2 + 0.0196371995417132*G0_2_0 + 0.0895482395048358*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[917] = -0.0574176057031953*G0_0_0 + 0.0124934342599271*G0_0_1 - 0.0574176057031952*G0_0_2 - 0.0574176057031954*G0_1_0 + 0.0124934342599271*G0_1_1 - 0.0574176057031953*G0_1_2 - 0.0186912214907005*G0_2_0 - 0.0574176057031955*G0_2_1 - 0.0186912214907005*G0_2_2;
    A[918] = 0.0283728175438241*G0_0_0 - 0.0315434750113565*G0_0_1 + 0.028372817543824*G0_0_2 + 0.0283728175438241*G0_1_0 - 0.0315434750113565*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.00939454064454039*G0_2_0 + 0.0283728175438242*G0_2_1 + 0.0093945406445404*G0_2_2;
    A[919] = -0.00537576492437602*G0_0_0 + 0.0220836945012288*G0_0_1 - 0.00537576492437605*G0_0_2 - 0.00537576492437607*G0_1_0 + 0.0220836945012289*G0_1_1 - 0.0053757649243761*G0_1_2 - 0.00537576492437592*G0_2_1;
    A[920] = 0.0196371995417131*G0_0_0 + 0.0895482395048356*G0_0_1 + 0.0895482395048355*G0_0_2 + 0.0196371995417131*G0_1_0 + 0.0895482395048356*G0_1_1 + 0.0895482395048356*G0_1_2 + 0.0895482395048358*G0_2_0 + 0.0196371995417131*G0_2_1 + 0.0196371995417131*G0_2_2;
    A[921] = 0.0124934342599274*G0_0_0 - 0.0574176057031952*G0_0_1 - 0.0574176057031951*G0_0_2 + 0.0124934342599274*G0_1_0 - 0.0574176057031952*G0_1_1 - 0.0574176057031952*G0_1_2 - 0.0574176057031954*G0_2_0 - 0.0186912214907004*G0_2_1 - 0.0186912214907004*G0_2_2;
    A[922] = -0.0315434750113567*G0_0_0 + 0.0283728175438241*G0_0_1 + 0.028372817543824*G0_0_2 - 0.0315434750113567*G0_1_0 + 0.028372817543824*G0_1_1 + 0.028372817543824*G0_1_2 + 0.0283728175438243*G0_2_0 + 0.00939454064454056*G0_2_1 + 0.00939454064454056*G0_2_2;
    A[923] = 0.0220836945012291*G0_0_0 - 0.00537576492437593*G0_0_1 - 0.00537576492437593*G0_0_2 + 0.0220836945012291*G0_1_0 - 0.0053757649243759*G0_1_1 - 0.00537576492437589*G0_1_2 - 0.00537576492437595*G0_2_0;
    A[924] = 0.0187890812890807*G0_0_0 + 0.0187890812890808*G0_0_1 + 0.00939454064454035*G0_0_2 + 0.0187890812890806*G0_1_0 + 0.0187890812890808*G0_1_1 + 0.0093945406445403*G0_1_2 + 0.00939454064454017*G0_2_0 + 0.0093945406445406*G0_2_1;
    A[925] = 0.00658922642429541*G0_0_0 + 0.00658922642429554*G0_0_1 + 0.00329461321214756*G0_0_2 + 0.00658922642429556*G0_1_0 + 0.00658922642429567*G0_1_1 + 0.00329461321214765*G0_1_2 + 0.00329461321214785*G0_2_0 + 0.00329461321214772*G0_2_1;
    A[926] = 0.0187890812890811*G0_0_0 + 0.0187890812890809*G0_0_1 + 0.00939454064454027*G0_0_2 + 0.0187890812890811*G0_1_0 + 0.0187890812890808*G0_1_1 + 0.00939454064454035*G0_1_2 + 0.00939454064454063*G0_2_0 + 0.00939454064454058*G0_2_1;
    A[927] = 0.0292600797158087*G0_0_0 + 0.0292600797158083*G0_0_1 + 0.0259654665036603*G0_0_2 + 0.0292600797158087*G0_1_0 + 0.0292600797158083*G0_1_1 + 0.0259654665036604*G0_1_2 + 0.0259654665036608*G0_2_0 + 0.0259654665036605*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[928] = 0.0292600797158085*G0_0_0 + 0.0292600797158086*G0_0_1 + 0.0259654665036607*G0_0_2 + 0.0292600797158084*G0_1_0 + 0.0292600797158087*G0_1_1 + 0.0259654665036606*G0_1_2 + 0.0259654665036604*G0_2_0 + 0.0259654665036606*G0_2_1 + 0.0226708532915127*G0_2_2;
    A[929] = 0.0641307878721067*G0_0_0 + 0.0641307878721068*G0_0_1 + 0.0547362472275662*G0_0_2 + 0.0641307878721067*G0_1_0 + 0.0641307878721067*G0_1_1 + 0.0547362472275663*G0_1_2 + 0.0547362472275663*G0_2_0 + 0.0547362472275662*G0_2_1 + 0.0453417065830256*G0_2_2;
    A[930] = -0.064456987200042*G0_0_0 - 0.257045070413123*G0_0_1 - 0.160751028806582*G0_0_2 - 0.064456987200042*G0_1_0 - 0.257045070413123*G0_1_1 - 0.160751028806583*G0_1_2 - 0.0929015685960119*G0_2_0 - 0.160751028806583*G0_2_1 - 0.0644569872000419*G0_2_2;
    A[931] = 0.0439716694056966*G0_0_0 + 0.128522535206561*G0_0_1 + 0.0862471023061293*G0_0_2 + 0.0439716694056967*G0_1_0 + 0.128522535206561*G0_1_1 + 0.0862471023061293*G0_1_2 + 0.0446566879943613*G0_2_0 + 0.0862471023061294*G0_2_1 + 0.0439716694056972*G0_2_2;
    A[932] = -0.0187890812890812*G0_0_0 - 0.0613254736518616*G0_0_1 - 0.0400572774704713*G0_0_2 - 0.0187890812890811*G0_1_0 - 0.0613254736518617*G0_1_1 - 0.0400572774704713*G0_1_2 - 0.00939454064454065*G0_2_0 - 0.0400572774704716*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[933] = 0.1273155976932*G0_0_0 + 0.176636936077039*G0_0_1 + 0.134361503176606*G0_0_2 + 0.1273155976932*G0_1_0 + 0.176636936077038*G0_1_1 + 0.134361503176606*G0_1_2 + 0.128000616281865*G0_2_0 + 0.134361503176606*G0_2_1 + 0.092086070276173*G0_2_2;
    A[934] = -0.0292600797158085*G0_0_0 - 0.0435476102793811*G0_0_1 - 0.0377086423093365*G0_0_2 - 0.0292600797158084*G0_1_0 - 0.043547610279381*G0_1_1 - 0.0377086423093365*G0_1_2 - 0.0259654665036604*G0_2_0 - 0.0377086423093366*G0_2_1 - 0.0318696743392922*G0_2_2;
    A[935] = -0.0641307878721067*G0_0_0 - 0.111886369481854*G0_0_1 - 0.0906181733004637*G0_0_2 - 0.0641307878721066*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0906181733004637*G0_1_2 - 0.0547362472275661*G0_2_0 - 0.0906181733004638*G0_2_1 - 0.0693499771190733*G0_2_2;
    A[936] = -0.257045070413123*G0_0_0 - 0.0644569872000418*G0_0_1 - 0.160751028806582*G0_0_2 - 0.257045070413123*G0_1_0 - 0.0644569872000418*G0_1_1 - 0.160751028806582*G0_1_2 - 0.160751028806583*G0_2_0 - 0.0929015685960116*G0_2_1 - 0.0644569872000416*G0_2_2;
    A[937] = 0.128522535206562*G0_0_0 + 0.0439716694056968*G0_0_1 + 0.0862471023061291*G0_0_2 + 0.128522535206562*G0_1_0 + 0.0439716694056968*G0_1_1 + 0.0862471023061292*G0_1_2 + 0.0862471023061291*G0_2_0 + 0.0446566879943609*G0_2_1 + 0.0439716694056964*G0_2_2;
    A[938] = -0.0613254736518621*G0_0_0 - 0.018789081289081*G0_0_1 - 0.0400572774704714*G0_0_2 - 0.0613254736518621*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.0400572774704713*G0_1_2 - 0.0400572774704718*G0_2_0 - 0.00939454064454069*G0_2_1 - 0.0187890812890809*G0_2_2;
    A[939] = 0.176636936077038*G0_0_0 + 0.1273155976932*G0_0_1 + 0.134361503176605*G0_0_2 + 0.176636936077038*G0_1_0 + 0.1273155976932*G0_1_1 + 0.134361503176605*G0_1_2 + 0.134361503176605*G0_2_0 + 0.128000616281864*G0_2_1 + 0.0920860702761728*G0_2_2;
    A[940] = -0.0435476102793804*G0_0_0 - 0.0292600797158082*G0_0_1 - 0.0377086423093363*G0_0_2 - 0.0435476102793804*G0_1_0 - 0.0292600797158082*G0_1_1 - 0.0377086423093364*G0_1_2 - 0.0377086423093359*G0_2_0 - 0.0259654665036606*G0_2_1 - 0.0318696743392919*G0_2_2;
    A[941] = -0.111886369481854*G0_0_0 - 0.0641307878721066*G0_0_1 - 0.0906181733004636*G0_0_2 - 0.111886369481854*G0_1_0 - 0.0641307878721065*G0_1_1 - 0.0906181733004635*G0_1_2 - 0.0906181733004638*G0_2_0 - 0.0547362472275661*G0_2_1 - 0.0693499771190732*G0_2_2;
    A[942] = -0.128522535206561*G0_0_0 - 0.128522535206561*G0_0_1 - 0.0322284936000208*G0_0_2 - 0.128522535206562*G0_1_0 - 0.128522535206561*G0_1_1 - 0.0322284936000209*G0_1_2 - 0.0322284936000209*G0_2_0 - 0.0322284936000208*G0_2_1 - 0.0606730749959908*G0_2_2;
    A[943] = 0.0642612676032808*G0_0_0 + 0.0642612676032808*G0_0_1 + 0.0219858347028481*G0_0_2 + 0.0642612676032807*G0_1_0 + 0.0642612676032808*G0_1_1 + 0.0219858347028482*G0_1_2 + 0.0219858347028483*G0_2_0 + 0.0219858347028483*G0_2_1 + 0.0226708532915126*G0_2_2;
    A[944] = -0.030662736825931*G0_0_0 - 0.0306627368259309*G0_0_1 - 0.00939454064454054*G0_0_2 - 0.0306627368259309*G0_1_0 - 0.0306627368259308*G0_1_1 - 0.0093945406445405*G0_1_2 - 0.00939454064454068*G0_2_0 - 0.00939454064454035*G0_2_1;
    A[945] = 0.0642612676032807*G0_0_0 + 0.0642612676032805*G0_0_1 + 0.0219858347028481*G0_0_2 + 0.0642612676032808*G0_1_0 + 0.0642612676032807*G0_1_1 + 0.0219858347028482*G0_1_2 + 0.0219858347028484*G0_2_0 + 0.0219858347028482*G0_2_1 + 0.0226708532915125*G0_2_2;
    A[946] = -0.00913358118219214*G0_0_0 - 0.00913358118219208*G0_0_1 - 0.00329461321214744*G0_0_2 - 0.00913358118219217*G0_1_0 - 0.00913358118219212*G0_1_1 - 0.00329461321214753*G0_1_2 - 0.00329461321214775*G0_2_0 - 0.00329461321214801*G0_2_1;
    A[947] = -0.0306627368259306*G0_0_0 - 0.0306627368259307*G0_0_1 - 0.00939454064454031*G0_0_2 - 0.0306627368259307*G0_1_0 - 0.0306627368259308*G0_1_1 - 0.00939454064454039*G0_1_2 - 0.00939454064454049*G0_2_0 - 0.00939454064454034*G0_2_1;
    A[948] = 0.0962288017409532*G0_0_0 + 0.0962288017409533*G0_0_1 + 0.131458329157981*G0_0_2 + 0.0962288017409534*G0_1_0 + 0.0962288017409532*G0_1_1 + 0.131458329157981*G0_1_2 + 0.13145832915798*G0_2_0 + 0.13145832915798*G0_2_1 + 0.166687856575007*G0_2_2;
    A[949] = -0.0505608958299921*G0_0_0 - 0.0505608958299921*G0_0_1 - 0.0479513012065084*G0_0_2 - 0.0505608958299921*G0_1_0 - 0.0505608958299923*G0_1_1 - 0.0479513012065085*G0_1_2 - 0.0479513012065079*G0_2_0 - 0.0479513012065085*G0_2_1 - 0.0453417065830253*G0_2_2;
    A[950] = -0.050560895829992*G0_0_0 - 0.0505608958299922*G0_0_1 - 0.0479513012065088*G0_0_2 - 0.0505608958299922*G0_1_0 - 0.0505608958299921*G0_1_1 - 0.0479513012065089*G0_1_2 - 0.047951301206509*G0_2_0 - 0.0479513012065087*G0_2_1 - 0.0453417065830251*G0_2_2;
    A[951] = -0.156575677409009*G0_0_0 - 0.156575677409009*G0_0_1 - 0.153966082785526*G0_0_2 - 0.156575677409009*G0_1_0 - 0.156575677409009*G0_1_1 - 0.153966082785526*G0_1_2 - 0.153966082785526*G0_2_0 - 0.153966082785525*G0_2_1 - 0.151356488162042*G0_2_2;
    A[952] = 0.0214691349673986*G0_0_0 + 0.0214691349673986*G0_0_1 + 0.0214691349673987*G0_0_2 + 0.0214691349673986*G0_1_0 + 0.0214691349673986*G0_1_1 + 0.0214691349673986*G0_1_2 + 0.0551276864210887*G0_2_0 + 0.0551276864210887*G0_2_1 + 0.0551276864210887*G0_2_2;
    A[953] = -0.00336507226698196*G0_0_0 - 0.00336507226698196*G0_1_0;
    A[954] = -0.00336507226698207*G0_0_1 - 0.00336507226698206*G0_1_1;
    A[955] = -0.0232632312710434*G0_0_2 - 0.0232632312710434*G0_1_2 - 0.0388177200243168*G0_2_2;
    A[956] = 0.0139743792087545*G0_0_1 + 0.00228991928210681*G0_0_2 + 0.0139743792087544*G0_1_1 + 0.00228991928210674*G0_1_2 + 0.00228991928210673*G0_2_1;
    A[957] = -0.0263047138047137*G0_0_1 - 0.0131523569023565*G0_0_2 - 0.0263047138047136*G0_1_1 - 0.0131523569023564*G0_1_2 - 0.0131523569023567*G0_2_1 + 6.52398655869178e-05*G0_2_2;
    A[958] = -0.00523223722008427*G0_0_1 - 0.00578025209101601*G0_0_2 - 0.00523223722008441*G0_1_1 - 0.00578025209101612*G0_1_2 - 0.00578025209101593*G0_2_1 - 0.00626302709636017*G0_2_2;
    A[959] = -0.0788945694544645*G0_0_1 - 0.0599162925551809*G0_0_2 - 0.0788945694544644*G0_1_1 - 0.0599162925551808*G0_1_2 - 0.0599162925551809*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[960] = 0.0139743792087538*G0_0_0 + 0.00228991928210632*G0_0_2 + 0.0139743792087538*G0_1_0 + 0.00228991928210635*G0_1_2 + 0.00228991928210645*G0_2_0;
    A[961] = -0.0263047138047129*G0_0_0 - 0.013152356902356*G0_0_2 - 0.0263047138047129*G0_1_0 - 0.0131523569023561*G0_1_2 - 0.0131523569023565*G0_2_0 + 6.52398655873593e-05*G0_2_2;
    A[962] = -0.0052322372200848*G0_0_0 - 0.00578025209101639*G0_0_2 - 0.00523223722008477*G0_1_0 - 0.00578025209101639*G0_1_2 - 0.00578025209101588*G0_2_0 - 0.00626302709636043*G0_2_2;
    A[963] = -0.0788945694544645*G0_0_0 - 0.0599162925551808*G0_0_2 - 0.0788945694544644*G0_1_0 - 0.0599162925551807*G0_1_2 - 0.0599162925551809*G0_2_0 - 0.0315434750113565*G0_2_2;
    A[964] = 0.00228991928210652*G0_0_0 + 0.00228991928210642*G0_0_1 + 0.00228991928210655*G0_1_0 + 0.00228991928210645*G0_1_1;
    A[965] = 0.00178757231708629*G0_0_0 + 0.00178757231708655*G0_0_1 + 0.00178757231708626*G0_1_0 + 0.00178757231708653*G0_1_1;
    A[966] = 0.00178757231708616*G0_0_0 + 0.00178757231708586*G0_0_1 + 0.00178757231708617*G0_1_0 + 0.00178757231708588*G0_1_1;
    A[967] = 0.00228991928210682*G0_0_0 + 0.00228991928210711*G0_0_1 + 0.00228991928210675*G0_1_0 + 0.00228991928210702*G0_1_1;
    A[968] = -0.147063705006412*G0_0_0 - 0.147063705006412*G0_0_1 - 0.108337320793917*G0_0_2 - 0.147063705006412*G0_1_0 - 0.147063705006412*G0_1_1 - 0.108337320793917*G0_1_2 - 0.17344670664983*G0_2_0 - 0.17344670664983*G0_2_1 - 0.243357746612953*G0_2_2;
    A[969] = 0.304226541205705*G0_0_0 + 0.304226541205705*G0_0_1 + 0.226095278178609*G0_0_2 + 0.304226541205705*G0_1_0 + 0.304226541205705*G0_1_1 + 0.226095278178609*G0_1_2 + 0.226095278178609*G0_2_0 + 0.226095278178609*G0_2_1 + 0.428965164208216*G0_2_2;
    A[970] = -0.204187731314465*G0_0_0 - 0.204187731314465*G0_0_1 - 0.219493003781196*G0_0_2 - 0.204187731314465*G0_1_0 - 0.204187731314465*G0_1_1 - 0.219493003781196*G0_1_2 - 0.167692550505049*G0_2_0 - 0.167692550505049*G0_2_1 - 0.387185554286246*G0_2_2;
    A[971] = 0.0788945694544644*G0_0_0 + 0.0788945694544644*G0_0_1 + 0.103529142700148*G0_0_2 + 0.0788945694544643*G0_1_0 + 0.0788945694544643*G0_1_1 + 0.103529142700148*G0_1_2 + 0.0599162925551808*G0_2_0 + 0.0599162925551808*G0_2_1 + 0.185268170294211*G0_2_2;
    A[972] = -0.0574176057031953*G0_0_0 - 0.0186912214907005*G0_0_1 - 0.0574176057031953*G0_0_2 - 0.0574176057031954*G0_1_0 - 0.0186912214907004*G0_1_1 - 0.0574176057031953*G0_1_2 + 0.0124934342599273*G0_2_0 - 0.0574176057031954*G0_2_1 + 0.0124934342599273*G0_2_2;
    A[973] = 0.00506261356955771*G0_0_0 + 0.000260959462348403*G0_0_1 + 0.00506261356955768*G0_0_2 + 0.00506261356955777*G0_1_0 + 0.000260959462348372*G0_1_1 + 0.00506261356955774*G0_1_2 + 0.000260959462348323*G0_2_0 + 0.00506261356955793*G0_2_1 + 0.000260959462348367*G0_2_2;
    A[974] = -0.000482775005344333*G0_0_0 - 0.00626302709636067*G0_0_1 - 0.000482775005344301*G0_0_2 - 0.000482775005344363*G0_1_0 - 0.00626302709636064*G0_1_1 - 0.000482775005344331*G0_1_2 + 6.52398655873061e-05*G0_2_0 - 0.000482775005344394*G0_2_1 + 6.52398655872777e-05*G0_2_2;
    A[975] = -0.00228991928210683*G0_0_0 + 0.00658922642429614*G0_0_1 - 0.00228991928210683*G0_0_2 - 0.00228991928210676*G0_1_0 + 0.00658922642429612*G0_1_1 - 0.00228991928210676*G0_1_2 - 0.00228991928210681*G0_2_1;
    A[976] = -0.0186912214907004*G0_0_0 - 0.0574176057031953*G0_0_1 - 0.0574176057031953*G0_0_2 - 0.0186912214907004*G0_1_0 - 0.0574176057031954*G0_1_1 - 0.0574176057031953*G0_1_2 - 0.0574176057031955*G0_2_0 + 0.0124934342599273*G0_2_1 + 0.0124934342599273*G0_2_2;
    A[977] = 0.00026095946234824*G0_0_0 + 0.00506261356955799*G0_0_1 + 0.00506261356955794*G0_0_2 + 0.000260959462348205*G0_1_0 + 0.005062613569558*G0_1_1 + 0.00506261356955795*G0_1_2 + 0.00506261356955815*G0_2_0 + 0.000260959462348477*G0_2_1 + 0.00026095946234845*G0_2_2;
    A[978] = -0.00626302709636031*G0_0_0 - 0.000482775005344323*G0_0_1 - 0.000482775005344297*G0_0_2 - 0.00626302709636027*G0_1_0 - 0.000482775005344269*G0_1_1 - 0.000482775005344243*G0_1_2 - 0.000482775005344622*G0_2_0 + 6.52398655869511e-05*G0_2_1 + 6.52398655869689e-05*G0_2_2;
    A[979] = 0.00658922642429581*G0_0_0 - 0.0022899192821065*G0_0_1 - 0.0022899192821065*G0_0_2 + 0.00658922642429579*G0_1_0 - 0.00228991928210653*G0_1_1 - 0.00228991928210652*G0_1_2 - 0.00228991928210671*G0_2_0;
    A[980] = -0.0148746893538554*G0_0_0 - 0.0148746893538559*G0_0_1 + 6.5239865587007e-05*G0_0_2 - 0.0148746893538555*G0_1_0 - 0.0148746893538559*G0_1_1 + 6.52398655869967e-05*G0_1_2 + 6.52398655874721e-05*G0_2_0 + 6.52398655868679e-05*G0_2_1;
    A[981] = 0.00450155072550867*G0_0_0 + 0.00450155072550883*G0_0_1 + 0.00257697469068982*G0_0_2 + 0.00450155072550857*G0_1_0 + 0.00450155072550871*G0_1_1 + 0.00257697469068977*G0_1_2 + 0.00257697469068985*G0_2_0 + 0.00257697469068997*G0_2_1;
    A[982] = -0.0148746893538554*G0_0_0 - 0.0148746893538558*G0_0_1 + 6.52398655874961e-05*G0_0_2 - 0.0148746893538554*G0_1_0 - 0.0148746893538557*G0_1_1 + 6.52398655873764e-05*G0_1_2 + 6.52398655870289e-05*G0_2_0 + 6.52398655870824e-05*G0_2_1;
    A[983] = -0.0189848008858424*G0_0_0 - 0.0189848008858416*G0_0_1 - 0.00949240044292066*G0_0_2 - 0.0189848008858424*G0_1_0 - 0.0189848008858416*G0_1_1 - 0.00949240044292074*G0_1_2 - 0.00949240044292134*G0_2_0 - 0.00949240044292078*G0_2_1 - 0.000652398655870931*G0_2_2;
    A[984] = -0.018984800885842*G0_0_0 - 0.0189848008858422*G0_0_1 - 0.00949240044292125*G0_0_2 - 0.0189848008858421*G0_1_0 - 0.0189848008858424*G0_1_1 - 0.00949240044292119*G0_1_2 - 0.00949240044292081*G0_2_0 - 0.00949240044292118*G0_2_1 - 0.000652398655871127*G0_2_2;
    A[985] = -0.127087258163646*G0_0_0 - 0.127087258163646*G0_0_1 - 0.0893133759887223*G0_0_2 - 0.127087258163646*G0_1_0 - 0.127087258163646*G0_1_1 - 0.0893133759887223*G0_1_2 - 0.0893133759887224*G0_2_0 - 0.0893133759887223*G0_2_1 - 0.0665446628988286*G0_2_2;
    A[986] = 0.132306447410613*G0_0_0 + 0.121346149991982*G0_0_1 + 0.194675758911868*G0_0_2 + 0.132306447410613*G0_1_0 + 0.121346149991982*G0_1_1 + 0.194675758911868*G0_1_2 + 0.0589768384907264*G0_2_0 + 0.194675758911868*G0_2_1 - 0.00339247301052913*G0_2_2;
    A[987] = -0.0436454700777612*G0_0_0 + 0.0371867233846405*G0_0_1 - 0.0371541034518457*G0_0_2 - 0.0436454700777612*G0_1_0 + 0.0371867233846405*G0_1_1 - 0.0371541034518457*G0_1_2 - 0.00975335990526952*G0_2_0 - 0.037154103451846*G0_2_1 - 0.00678494602105738*G0_2_2;
    A[988] = 0.0148746893538554*G0_0_0 - 0.025443547578965*G0_0_1 + 0.0136351319077009*G0_0_2 + 0.0148746893538554*G0_1_0 - 0.0254435475789649*G0_1_1 + 0.0136351319077009*G0_1_2 - 6.52398655870364e-05*G0_2_0 + 0.0136351319077012*G0_2_1 - 0.000130479731174106*G0_2_2;
    A[989] = -0.166948816037356*G0_0_0 - 0.334680510461758*G0_0_1 - 0.250814663249557*G0_0_2 - 0.166948816037356*G0_1_0 - 0.334680510461758*G0_1_1 - 0.250814663249557*G0_1_2 - 0.160783648739376*G0_2_0 - 0.250814663249557*G0_2_1 - 0.0622388317700801*G0_2_2;
    A[990] = 0.0189848008858422*G0_0_0 + 0.0567586830607664*G0_0_1 + 0.0378717419733038*G0_0_2 + 0.0189848008858421*G0_1_0 + 0.0567586830607663*G0_1_1 + 0.0378717419733039*G0_1_2 + 0.00949240044292085*G0_2_0 + 0.0378717419733043*G0_2_1 + 0.012787013655069*G0_2_2;
    A[991] = 0.127087258163646*G0_0_0 + 0.209419968534549*G0_0_1 + 0.173472802596065*G0_0_2 + 0.127087258163646*G0_1_0 + 0.209419968534549*G0_1_1 + 0.173472802596065*G0_1_2 + 0.0893133759887222*G0_2_0 + 0.173472802596064*G0_2_1 + 0.0846813455320386*G0_2_2;
    A[992] = 0.121346149991982*G0_0_0 + 0.132306447410613*G0_0_1 + 0.194675758911868*G0_0_2 + 0.121346149991982*G0_1_0 + 0.132306447410613*G0_1_1 + 0.194675758911868*G0_1_2 + 0.194675758911868*G0_2_0 + 0.0589768384907261*G0_2_1 - 0.0033924730105294*G0_2_2;
    A[993] = 0.0371867233846398*G0_0_0 - 0.0436454700777618*G0_0_1 - 0.0371541034518463*G0_0_2 + 0.0371867233846398*G0_1_0 - 0.0436454700777619*G0_1_1 - 0.0371541034518465*G0_1_2 - 0.0371541034518462*G0_2_0 - 0.0097533599052694*G0_2_1 - 0.00678494602105682*G0_2_2;
    A[994] = -0.0254435475789638*G0_0_0 + 0.0148746893538561*G0_0_1 + 0.0136351319077007*G0_0_2 - 0.0254435475789638*G0_1_0 + 0.0148746893538561*G0_1_1 + 0.0136351319077007*G0_1_2 + 0.0136351319077017*G0_2_0 - 6.52398655867552e-05*G0_2_1 - 0.000130479731174362*G0_2_2;
    A[995] = -0.334680510461758*G0_0_0 - 0.166948816037356*G0_0_1 - 0.250814663249557*G0_0_2 - 0.334680510461758*G0_1_0 - 0.166948816037356*G0_1_1 - 0.250814663249557*G0_1_2 - 0.250814663249556*G0_2_0 - 0.160783648739375*G0_2_1 - 0.0622388317700799*G0_2_2;
    A[996] = 0.0567586830607656*G0_0_0 + 0.0189848008858416*G0_0_1 + 0.0378717419733045*G0_0_2 + 0.0567586830607657*G0_1_0 + 0.0189848008858416*G0_1_1 + 0.0378717419733046*G0_1_2 + 0.0378717419733034*G0_2_0 + 0.00949240044292085*G0_2_1 + 0.0127870136550689*G0_2_2;
    A[997] = 0.20941996853455*G0_0_0 + 0.127087258163646*G0_0_1 + 0.173472802596065*G0_0_2 + 0.20941996853455*G0_1_0 + 0.127087258163646*G0_1_1 + 0.173472802596065*G0_1_2 + 0.173472802596065*G0_2_0 + 0.0893133759887222*G0_2_1 + 0.0846813455320384*G0_2_2;
    A[998] = 0.0476251018785736*G0_0_0 + 0.0476251018785733*G0_0_1 - 0.0147442096226817*G0_0_2 + 0.0476251018785736*G0_1_0 + 0.0476251018785734*G0_1_1 - 0.0147442096226816*G0_1_2 - 0.0147442096226819*G0_2_0 - 0.0147442096226821*G0_2_1 + 0.0476251018785736*G0_2_2;
    A[999] = 0.00880738185425673*G0_0_0 + 0.00880738185425669*G0_0_1 + 0.00231601522834171*G0_0_2 + 0.00880738185425674*G0_1_0 + 0.00880738185425668*G0_1_1 + 0.00231601522834174*G0_1_2 + 0.00231601522834156*G0_2_0 + 0.00231601522834158*G0_2_1 - 0.000652398655870841*G0_2_2;
    A[1000] = -0.00130479731174154*G0_0_0 - 0.00130479731174221*G0_0_1 - 6.52398655868727e-05*G0_0_2 - 0.0013047973117416*G0_1_0 - 0.00130479731174225*G0_1_1 - 6.52398655868626e-05*G0_1_2 - 6.52398655867786e-05*G0_2_0 - 6.52398655873215e-05*G0_2_1;
    A[1001] = 0.00880738185425706*G0_0_0 + 0.00880738185425711*G0_0_1 + 0.00231601522834164*G0_0_2 + 0.00880738185425701*G0_1_0 + 0.00880738185425701*G0_1_1 + 0.00231601522834162*G0_1_2 + 0.0023160152283416*G0_2_0 + 0.00231601522834175*G0_2_1 - 0.000652398655870554*G0_2_2;
    A[1002] = -0.0156575677409011*G0_0_0 - 0.0156575677409005*G0_0_1 - 0.00257697469068991*G0_0_2 - 0.0156575677409011*G0_1_0 - 0.0156575677409005*G0_1_1 - 0.00257697469068987*G0_1_2 - 0.00257697469069011*G0_2_0 - 0.00257697469068978*G0_2_1;
    A[1003] = -0.00130479731174183*G0_0_0 - 0.00130479731174223*G0_0_1 - 6.52398655873975e-05*G0_0_2 - 0.00130479731174181*G0_1_0 - 0.00130479731174211*G0_1_1 - 6.52398655872768e-05*G0_1_2 - 6.52398655871915e-05*G0_2_0 - 6.52398655873001e-05*G0_2_1;
    A[1004] = -0.15657567740901*G0_0_0 - 0.15657567740901*G0_0_1 - 0.0662184635708941*G0_0_2 - 0.15657567740901*G0_1_0 - 0.15657567740901*G0_1_1 - 0.0662184635708943*G0_1_2 - 0.0662184635708924*G0_2_0 - 0.0662184635708926*G0_2_1 - 0.161794866655976*G0_2_2;
    A[1005] = 0.0391439193522526*G0_0_0 + 0.0391439193522526*G0_0_1 + 0.00717638521457891*G0_0_2 + 0.0391439193522527*G0_1_0 + 0.0391439193522528*G0_1_1 + 0.00717638521457894*G0_1_2 + 0.00717638521457841*G0_2_0 + 0.00717638521457926*G0_2_1 + 0.00130479731174168*G0_2_2;
    A[1006] = 0.0391439193522525*G0_0_0 + 0.0391439193522525*G0_0_1 + 0.00717638521457963*G0_0_2 + 0.0391439193522526*G0_1_0 + 0.0391439193522524*G0_1_1 + 0.00717638521457959*G0_1_2 + 0.00717638521457967*G0_2_0 + 0.00717638521457922*G0_2_1 + 0.00130479731174161*G0_2_2;
    A[1007] = 0.185933616923199*G0_0_0 + 0.185933616923199*G0_0_1 + 0.170276049182298*G0_0_2 + 0.185933616923199*G0_1_0 + 0.185933616923199*G0_1_1 + 0.170276049182298*G0_1_2 + 0.170276049182297*G0_2_0 + 0.170276049182296*G0_2_1 + 0.180714427676231*G0_2_2;
    A[1008] = -0.0155544887532733*G0_0_0 - 0.0155544887532733*G0_0_1 - 0.0155544887532733*G0_0_2 - 0.0155544887532733*G0_1_0 - 0.0155544887532733*G0_1_1 - 0.0155544887532733*G0_1_2 - 0.0388177200243169*G0_2_0 - 0.038817720024317*G0_2_1 - 0.0388177200243169*G0_2_2;
    A[1009] = -0.00336507226698205*G0_0_0 - 0.00336507226698206*G0_1_0;
    A[1010] = -0.00336507226698189*G0_0_1 - 0.00336507226698189*G0_1_1;
    A[1011] = 0.03365855145369*G0_0_2 + 0.03365855145369*G0_1_2 + 0.0551276864210887*G0_2_2;
    A[1012] = 0.0111690649885091*G0_0_1 + 0.0022899192821068*G0_0_2 + 0.0111690649885092*G0_1_1 + 0.00228991928210683*G0_1_2 + 0.00228991928210687*G0_2_1;
    A[1013] = -0.00523223722008433*G0_0_1 + 0.000548014870931175*G0_0_2 - 0.00523223722008439*G0_1_1 + 0.000548014870931161*G0_1_2 + 0.000548014870931335*G0_2_1 + 6.52398655870436e-05*G0_2_2;
    A[1014] = -0.00960330821441932*G0_0_1 - 0.00480165410720937*G0_0_2 - 0.00960330821441928*G0_1_1 - 0.00480165410720935*G0_1_2 - 0.00480165410720956*G0_2_1 + 0.000260959462348298*G0_2_2;
    A[1015] = 0.108637424175618*G0_0_1 + 0.0699110399631226*G0_0_2 + 0.108637424175618*G0_1_1 + 0.0699110399631225*G0_1_2 + 0.0699110399631226*G0_2_1 + 0.012493434259927*G0_2_2;
    A[1016] = 0.0111690649885098*G0_0_0 + 0.00228991928210703*G0_0_2 + 0.0111690649885098*G0_1_0 + 0.00228991928210706*G0_1_2 + 0.00228991928210711*G0_2_0;
    A[1017] = -0.00523223722008508*G0_0_0 + 0.000548014870931074*G0_0_2 - 0.00523223722008517*G0_1_0 + 0.000548014870931015*G0_1_2 + 0.000548014870931154*G0_2_0 + 6.52398655867346e-05*G0_2_2;
    A[1018] = -0.00960330821441899*G0_0_0 - 0.00480165410720937*G0_0_2 - 0.00960330821441889*G0_1_0 - 0.00480165410720927*G0_1_2 - 0.00480165410720961*G0_2_0 + 0.000260959462348478*G0_2_2;
    A[1019] = 0.108637424175618*G0_0_0 + 0.0699110399631226*G0_0_2 + 0.108637424175618*G0_1_0 + 0.0699110399631226*G0_1_2 + 0.0699110399631226*G0_2_0 + 0.0124934342599269*G0_2_2;
    A[1020] = 0.00228991928210707*G0_0_0 + 0.00228991928210693*G0_0_1 + 0.0022899192821071*G0_1_0 + 0.00228991928210695*G0_1_1;
    A[1021] = 0.00178757231708621*G0_0_0 + 0.00178757231708616*G0_0_1 + 0.00178757231708623*G0_1_0 + 0.00178757231708617*G0_1_1;
    A[1022] = 0.00178757231708623*G0_0_0 + 0.00178757231708637*G0_0_1 + 0.00178757231708621*G0_1_0 + 0.00178757231708636*G0_1_1;
    A[1023] = 0.00228991928210675*G0_0_0 + 0.00228991928210654*G0_0_1 + 0.00228991928210678*G0_1_0 + 0.00228991928210657*G0_1_1;
    A[1024] = 0.100717304493345*G0_0_0 + 0.100717304493345*G0_0_1 + 0.0817390275940614*G0_0_2 + 0.100717304493345*G0_1_0 + 0.100717304493345*G0_1_1 + 0.0817390275940614*G0_1_2 + 0.125351877739029*G0_2_0 + 0.125351877739029*G0_2_1 + 0.18526817029421*G0_2_2;
    A[1025] = -0.204187731314465*G0_0_0 - 0.204187731314465*G0_0_1 - 0.167692550505049*G0_0_2 - 0.204187731314465*G0_1_0 - 0.204187731314465*G0_1_1 - 0.167692550505049*G0_1_2 - 0.219493003781196*G0_2_0 - 0.219493003781196*G0_2_1 - 0.387185554286246*G0_2_2;
    A[1026] = 0.281001149056702*G0_0_0 + 0.281001149056702*G0_0_1 + 0.202869886029606*G0_0_2 + 0.281001149056702*G0_1_0 + 0.281001149056702*G0_1_1 + 0.202869886029606*G0_1_2 + 0.202869886029606*G0_2_0 + 0.202869886029606*G0_2_1 + 0.428965164208217*G0_2_2;
    A[1027] = -0.108637424175618*G0_0_0 - 0.108637424175618*G0_0_1 - 0.135020425819036*G0_0_2 - 0.108637424175618*G0_1_0 - 0.108637424175618*G0_1_1 - 0.135020425819036*G0_1_2 - 0.0699110399631226*G0_2_0 - 0.0699110399631225*G0_2_1 - 0.243357746612953*G0_2_2;
    A[1028] = 0.028372817543824*G0_0_0 + 0.00939454064454046*G0_0_1 + 0.028372817543824*G0_0_2 + 0.028372817543824*G0_1_0 + 0.00939454064454044*G0_1_1 + 0.028372817543824*G0_1_2 - 0.0315434750113569*G0_2_0 + 0.0283728175438242*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1029] = -0.000482775005344303*G0_0_0 + 6.5239865587011e-05*G0_0_1 - 0.000482775005344291*G0_0_2 - 0.000482775005344314*G0_1_0 + 6.5239865587025e-05*G0_1_1 - 0.000482775005344301*G0_1_2 - 0.00626302709636042*G0_2_0 - 0.000482775005344401*G0_2_1 - 0.00626302709636048*G0_2_2;
    A[1030] = 0.0132175967679439*G0_0_0 + 6.5239865587499e-05*G0_0_1 + 0.0132175967679439*G0_0_2 + 0.0132175967679439*G0_1_0 + 6.52398655874783e-05*G0_1_1 + 0.0132175967679439*G0_1_2 + 6.52398655869913e-05*G0_2_0 + 0.0132175967679439*G0_2_1 + 6.52398655870375e-05*G0_2_2;
    A[1031] = -0.00228991928210676*G0_0_0 + 0.00939454064454026*G0_0_1 - 0.00228991928210676*G0_0_2 - 0.00228991928210679*G0_1_0 + 0.00939454064454027*G0_1_1 - 0.00228991928210678*G0_1_2 - 0.00228991928210677*G0_2_1;
    A[1032] = 0.00939454064454053*G0_0_0 + 0.0283728175438241*G0_0_1 + 0.0283728175438241*G0_0_2 + 0.00939454064454051*G0_1_0 + 0.0283728175438241*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.0283728175438243*G0_2_0 - 0.0315434750113569*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1033] = 6.52398655870068e-05*G0_0_0 - 0.000482775005344542*G0_0_1 - 0.000482775005344491*G0_0_2 + 6.52398655870281e-05*G0_1_0 - 0.000482775005344554*G0_1_1 - 0.000482775005344504*G0_1_2 - 0.00048277500534464*G0_2_0 - 0.00626302709636057*G0_2_1 - 0.00626302709636054*G0_2_2;
    A[1034] = 6.52398655871209e-05*G0_0_0 + 0.0132175967679438*G0_0_1 + 0.0132175967679438*G0_0_2 + 6.52398655870975e-05*G0_1_0 + 0.0132175967679438*G0_1_1 + 0.0132175967679438*G0_1_2 + 0.0132175967679441*G0_2_0 + 6.52398655871871e-05*G0_2_1 + 6.5239865587157e-05*G0_2_2;
    A[1035] = 0.0093945406445407*G0_0_0 - 0.00228991928210683*G0_0_1 - 0.00228991928210683*G0_0_2 + 0.00939454064454074*G0_1_0 - 0.00228991928210685*G0_1_1 - 0.00228991928210685*G0_1_2 - 0.00228991928210682*G0_2_0;
    A[1036] = -0.00117431758056813*G0_0_0 - 0.00117431758056742*G0_0_1 + 6.52398655872853e-05*G0_0_2 - 0.00117431758056819*G0_1_0 - 0.00117431758056743*G0_1_1 + 6.52398655873515e-05*G0_1_2 + 6.5239865586884e-05*G0_2_0 + 6.52398655874341e-05*G0_2_1;
    A[1037] = -0.0105036183595208*G0_0_0 - 0.0105036183595211*G0_0_1 + 0.00257697469069014*G0_0_2 - 0.0105036183595208*G0_1_0 - 0.0105036183595211*G0_1_1 + 0.00257697469069013*G0_1_2 + 0.00257697469069023*G0_2_0 + 0.00257697469069004*G0_2_1;
    A[1038] = -0.00117431758056807*G0_0_0 - 0.00117431758056781*G0_0_1 + 6.52398655866855e-05*G0_0_2 - 0.00117431758056803*G0_1_0 - 0.00117431758056782*G0_1_1 + 6.52398655867253e-05*G0_1_2 + 6.52398655871671e-05*G0_2_0 + 6.52398655871838e-05*G0_2_1;
    A[1039] = 0.00352295274170285*G0_0_0 + 0.00352295274170191*G0_0_1 - 0.00296841388421299*G0_0_2 + 0.003522952741703*G0_1_0 + 0.00352295274170196*G0_1_1 - 0.00296841388421302*G0_1_2 - 0.00296841388421248*G0_2_0 - 0.00296841388421302*G0_2_1 - 0.000652398655870976*G0_2_2;
    A[1040] = 0.00352295274170281*G0_0_0 + 0.00352295274170329*G0_0_1 - 0.00296841388421202*G0_0_2 + 0.00352295274170279*G0_1_0 + 0.00352295274170331*G0_1_1 - 0.00296841388421207*G0_1_2 - 0.00296841388421278*G0_2_0 - 0.00296841388421238*G0_2_1 - 0.000652398655870552*G0_2_2;
    A[1041] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0623693115012556*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0623693115012556*G0_1_2 + 0.0623693115012553*G0_2_0 + 0.0623693115012553*G0_2_1 + 0.0476251018785734*G0_2_2;
    A[1042] = -0.0510175748891017*G0_0_0 - 0.0528442911255397*G0_0_1 - 0.0887914570640252*G0_0_2 - 0.0510175748891017*G0_1_0 - 0.0528442911255397*G0_1_1 - 0.0887914570640252*G0_1_2 - 0.00463203045668257*G0_2_0 - 0.0887914570640255*G0_2_1 + 0.0846813455320401*G0_2_2;
    A[1043] = -0.0155923278753145*G0_0_0 - 0.00619778723077418*G0_0_1 - 0.0250847283182357*G0_0_2 - 0.0155923278753145*G0_1_0 - 0.00619778723077416*G0_1_1 - 0.0250847283182357*G0_1_2 + 0.00329461321214773*G0_2_0 - 0.0250847283182353*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[1044] = 0.00117431758056813*G0_0_0 - 0.0528442911255401*G0_0_1 - 0.0137656116388752*G0_0_2 + 0.00117431758056808*G0_1_0 - 0.0528442911255401*G0_1_1 - 0.0137656116388752*G0_1_2 - 6.5239865587138e-05*G0_2_0 - 0.0137656116388754*G0_2_1 - 0.000130479731174183*G0_2_2;
    A[1045] = 0.182410664181495*G0_0_0 + 0.104709984267274*G0_0_1 + 0.188575831479475*G0_0_2 + 0.182410664181495*G0_1_0 + 0.104709984267274*G0_1_1 + 0.188575831479475*G0_1_2 + 0.0985448169692945*G0_2_0 + 0.188575831479475*G0_2_1 - 0.0622388317700828*G0_2_2;
    A[1046] = -0.00352295274170293*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0303691574307896*G0_0_2 - 0.0035229527417029*G0_1_0 + 0.104709984267275*G0_1_1 + 0.0303691574307896*G0_1_2 + 0.00296841388421272*G0_2_0 + 0.0303691574307893*G0_2_1 - 0.0067849460210568*G0_2_2;
    A[1047] = -0.124738623002511*G0_0_0 - 0.271397840842282*G0_0_1 - 0.198068231922397*G0_0_2 - 0.124738623002511*G0_1_0 - 0.271397840842282*G0_1_1 - 0.198068231922397*G0_1_2 - 0.0623693115012552*G0_2_0 - 0.198068231922396*G0_2_1 - 0.00339247301052765*G0_2_2;
    A[1048] = -0.0528442911255401*G0_0_0 - 0.051017574889102*G0_0_1 - 0.0887914570640254*G0_0_2 - 0.0528442911255402*G0_1_0 - 0.051017574889102*G0_1_1 - 0.0887914570640254*G0_1_2 - 0.0887914570640255*G0_2_0 - 0.00463203045668253*G0_2_1 + 0.0846813455320403*G0_2_2;
    A[1049] = -0.00619778723077293*G0_0_0 - 0.0155923278753133*G0_0_1 - 0.0250847283182348*G0_0_2 - 0.0061977872307729*G0_1_0 - 0.0155923278753133*G0_1_1 - 0.0250847283182348*G0_1_2 - 0.0250847283182351*G0_2_0 + 0.00329461321214789*G0_2_1 + 0.012787013655069*G0_2_2;
    A[1050] = -0.0528442911255415*G0_0_0 + 0.0011743175805672*G0_0_1 - 0.0137656116388753*G0_0_2 - 0.0528442911255415*G0_1_0 + 0.0011743175805672*G0_1_1 - 0.0137656116388753*G0_1_2 - 0.0137656116388759*G0_2_0 - 6.52398655875272e-05*G0_2_1 - 0.000130479731173911*G0_2_2;
    A[1051] = 0.104709984267274*G0_0_0 + 0.182410664181496*G0_0_1 + 0.188575831479476*G0_0_2 + 0.104709984267274*G0_1_0 + 0.182410664181496*G0_1_1 + 0.188575831479475*G0_1_2 + 0.188575831479475*G0_2_0 + 0.0985448169692942*G0_2_1 - 0.0622388317700826*G0_2_2;
    A[1052] = 0.104709984267276*G0_0_0 - 0.00352295274170185*G0_0_1 + 0.0303691574307891*G0_0_2 + 0.104709984267276*G0_1_0 - 0.0035229527417019*G0_1_1 + 0.030369157430789*G0_1_2 + 0.0303691574307898*G0_2_0 + 0.00296841388421298*G0_2_1 - 0.00678494602105702*G0_2_2;
    A[1053] = -0.271397840842283*G0_0_0 - 0.124738623002511*G0_0_1 - 0.198068231922397*G0_0_2 - 0.271397840842283*G0_1_0 - 0.124738623002511*G0_1_1 - 0.198068231922397*G0_1_2 - 0.198068231922397*G0_2_0 - 0.0623693115012553*G0_2_1 - 0.00339247301052768*G0_2_2;
    A[1054] = -0.0150051690850298*G0_0_0 - 0.0150051690850297*G0_0_1 + 0.0227687130898933*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0150051690850298*G0_1_1 + 0.0227687130898933*G0_1_2 + 0.0227687130898937*G0_2_0 + 0.0227687130898938*G0_2_1 - 0.0665446628988292*G0_2_2;
    A[1055] = -0.000652398655870698*G0_0_0 - 0.000652398655870477*G0_0_1 + 0.0088400017870506*G0_0_2 - 0.000652398655870678*G0_1_0 - 0.000652398655870472*G0_1_1 + 0.00884000178705058*G0_1_2 + 0.00884000178705029*G0_2_0 + 0.0088400017870504*G0_2_1 - 0.000652398655870883*G0_2_2;
    A[1056] = -0.0150051690850306*G0_0_0 - 0.01500516908503*G0_0_1 - 6.5239865587488e-05*G0_0_2 - 0.0150051690850306*G0_1_0 - 0.01500516908503*G0_1_1 - 6.52398655875526e-05*G0_1_2 - 6.52398655873312e-05*G0_2_0 - 6.52398655868531e-05*G0_2_1;
    A[1057] = -0.000652398655871077*G0_0_0 - 0.000652398655871142*G0_0_1 + 0.00884000178705014*G0_0_2 - 0.000652398655871091*G0_1_0 - 0.000652398655871127*G0_1_1 + 0.00884000178705014*G0_1_2 + 0.00884000178705037*G0_2_0 + 0.00884000178705021*G0_2_1 - 0.000652398655871096*G0_2_2;
    A[1058] = -0.000652398655870831*G0_0_0 - 0.00065239865587135*G0_0_1 - 0.00257697469069005*G0_0_2 - 0.000652398655870823*G0_1_0 - 0.000652398655871327*G0_1_1 - 0.00257697469069004*G0_1_2 - 0.00257697469068979*G0_2_0 - 0.00257697469069014*G0_2_1;
    A[1059] = -0.0150051690850301*G0_0_0 - 0.0150051690850298*G0_0_1 - 6.52398655868404e-05*G0_0_2 - 0.0150051690850301*G0_1_0 - 0.0150051690850299*G0_1_1 - 6.5239865586879e-05*G0_1_2 - 6.52398655871211e-05*G0_2_0 - 6.52398655869977e-05*G0_2_1;
    A[1060] = 0.0260959462348342*G0_0_0 + 0.0260959462348338*G0_0_1 + 0.0104383784939335*G0_0_2 + 0.0260959462348341*G0_1_0 + 0.0260959462348338*G0_1_1 + 0.0104383784939335*G0_1_2 + 0.0104383784939328*G0_2_0 + 0.010438378493933*G0_2_1 + 0.180714427676231*G0_2_2;
    A[1061] = 0.0260959462348357*G0_0_0 + 0.0260959462348346*G0_0_1 - 0.0058715879028375*G0_0_2 + 0.0260959462348359*G0_1_0 + 0.0260959462348345*G0_1_1 - 0.00587158790283745*G0_1_2 - 0.005871587902837*G0_2_0 - 0.00587158790283778*G0_2_1 + 0.00130479731174188*G0_2_2;
    A[1062] = 0.0260959462348353*G0_0_0 + 0.0260959462348362*G0_0_1 - 0.005871587902838*G0_0_2 + 0.0260959462348353*G0_1_0 + 0.0260959462348363*G0_1_1 - 0.00587158790283797*G0_1_2 - 0.00587158790283798*G0_2_0 - 0.0058715879028375*G0_2_1 + 0.00130479731174173*G0_2_2;
    A[1063] = -0.185933616923198*G0_0_0 - 0.185933616923199*G0_0_1 - 0.0955764030850825*G0_0_2 - 0.185933616923199*G0_1_0 - 0.185933616923199*G0_1_1 - 0.0955764030850825*G0_1_2 - 0.0955764030850822*G0_2_0 - 0.095576403085082*G0_2_1 - 0.161794866655976*G0_2_2;
    A[1064] = 0.00480034930989785*G0_0_0 + 0.00480034930989784*G0_0_1 + 0.00480034930989787*G0_0_2 + 0.00480034930989785*G0_1_0 + 0.00480034930989784*G0_1_1 + 0.00480034930989788*G0_1_2 + 0.0153313684129655*G0_2_0 + 0.0153313684129655*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[1065] = -0.00480034930989783*G0_0_0 - 0.00480034930989783*G0_1_0;
    A[1066] = -0.00480034930989792*G0_0_1 - 0.00480034930989792*G0_1_1;
    A[1067] = -0.0372571824394738*G0_0_2 - 0.0372571824394738*G0_1_2 - 0.0551276864210888*G0_2_2;
    A[1068] = 0.0328352243499812*G0_0_1 + 0.00537576492437589*G0_0_2 + 0.0328352243499812*G0_1_1 + 0.0053757649243759*G0_1_2 + 0.00537576492437587*G0_2_1;
    A[1069] = -0.0788945694544647*G0_0_1 - 0.0189782768992834*G0_0_2 - 0.0788945694544647*G0_1_1 - 0.0189782768992834*G0_1_2 - 0.0189782768992835*G0_2_1 + 0.00939454064454077*G0_2_2;
    A[1070] = 0.108637424175618*G0_0_1 + 0.0387263842124948*G0_0_2 + 0.108637424175618*G0_1_1 + 0.0387263842124948*G0_1_2 + 0.0387263842124949*G0_2_1 - 0.0186912214907007*G0_2_2;
    A[1071] = -0.139822079926246*G0_0_1 - 0.0699110399631228*G0_0_2 - 0.139822079926246*G0_1_1 - 0.0699110399631229*G0_1_2 - 0.0699110399631225*G0_2_1 + 0.0196371995417136*G0_2_2;
    A[1072] = 0.0328352243499806*G0_0_0 + 0.00537576492437571*G0_0_2 + 0.0328352243499806*G0_1_0 + 0.0053757649243757*G0_1_2 + 0.00537576492437567*G0_2_0;
    A[1073] = -0.0788945694544641*G0_0_0 - 0.0189782768992832*G0_0_2 - 0.0788945694544641*G0_1_0 - 0.0189782768992833*G0_1_2 - 0.0189782768992833*G0_2_0 + 0.00939454064454096*G0_2_2;
    A[1074] = 0.108637424175618*G0_0_0 + 0.0387263842124948*G0_0_2 + 0.108637424175618*G0_1_0 + 0.0387263842124948*G0_1_2 + 0.0387263842124948*G0_2_0 - 0.0186912214907007*G0_2_2;
    A[1075] = -0.139822079926246*G0_0_0 - 0.0699110399631229*G0_0_2 - 0.139822079926246*G0_1_0 - 0.0699110399631229*G0_1_2 - 0.0699110399631225*G0_2_0 + 0.0196371995417136*G0_2_2;
    A[1076] = 0.00537576492437567*G0_0_0 + 0.00537576492437578*G0_0_1 + 0.00537576492437565*G0_1_0 + 0.00537576492437577*G0_1_1;
    A[1077] = 0.00228991928210669*G0_0_0 + 0.00228991928210679*G0_0_1 + 0.00228991928210671*G0_1_0 + 0.00228991928210681*G0_1_1;
    A[1078] = 0.00228991928210678*G0_0_0 + 0.00228991928210656*G0_0_1 + 0.00228991928210678*G0_1_0 + 0.00228991928210655*G0_1_1;
    A[1079] = 0.00537576492437598*G0_0_0 + 0.00537576492437622*G0_0_1 + 0.00537576492437599*G0_1_0 + 0.00537576492437622*G0_1_1;
    A[1080] = -0.0328352243499809*G0_0_0 - 0.0328352243499809*G0_0_1 - 0.027459459425605*G0_0_2 - 0.0328352243499809*G0_1_0 - 0.0328352243499808*G0_1_1 - 0.027459459425605*G0_1_2 - 0.053751125257201*G0_2_0 - 0.053751125257201*G0_2_1 - 0.0812105846828062*G0_2_2;
    A[1081] = 0.0788945694544644*G0_0_0 + 0.0788945694544643*G0_0_1 + 0.0599162925551808*G0_0_2 + 0.0788945694544644*G0_1_0 + 0.0788945694544643*G0_1_1 + 0.0599162925551808*G0_1_2 + 0.103529142700148*G0_2_0 + 0.103529142700148*G0_2_1 + 0.18526817029421*G0_2_2;
    A[1082] = -0.108637424175618*G0_0_0 - 0.108637424175618*G0_0_1 - 0.0699110399631226*G0_0_2 - 0.108637424175618*G0_1_0 - 0.108637424175618*G0_1_1 - 0.0699110399631225*G0_1_2 - 0.135020425819036*G0_2_0 - 0.135020425819036*G0_2_1 - 0.243357746612953*G0_2_2;
    A[1083] = 0.139822079926246*G0_0_0 + 0.139822079926246*G0_0_1 + 0.0699110399631225*G0_0_2 + 0.139822079926246*G0_1_0 + 0.139822079926246*G0_1_1 + 0.0699110399631225*G0_1_2 + 0.0699110399631225*G0_2_0 + 0.0699110399631225*G0_2_1 + 0.179096479009672*G0_2_2;
    A[1084] = -0.00537576492437602*G0_0_0 - 0.00537576492437598*G0_0_2 - 0.00537576492437599*G0_1_0 - 0.00537576492437596*G0_1_2 + 0.0220836945012292*G0_2_0 - 0.00537576492437593*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1085] = -0.00228991928210678*G0_0_0 - 0.00228991928210685*G0_0_2 - 0.00228991928210679*G0_1_0 - 0.00228991928210686*G0_1_2 + 0.00658922642429596*G0_2_0 - 0.00228991928210682*G0_2_1 + 0.00658922642429598*G0_2_2;
    A[1086] = -0.00228991928210681*G0_0_0 - 0.00228991928210681*G0_0_2 - 0.00228991928210681*G0_1_0 - 0.00228991928210681*G0_1_2 + 0.00939454064454058*G0_2_0 - 0.00228991928210673*G0_2_1 + 0.00939454064454054*G0_2_2;
    A[1087] = -0.00537576492437598*G0_0_0 - 0.00537576492437598*G0_0_2 - 0.00537576492437598*G0_1_0 - 0.00537576492437599*G0_1_2 - 0.00537576492437602*G0_2_1;
    A[1088] = -0.00537576492437614*G0_0_1 - 0.0053757649243761*G0_0_2 - 0.00537576492437612*G0_1_1 - 0.00537576492437609*G0_1_2 - 0.00537576492437605*G0_2_0 + 0.0220836945012291*G0_2_1 + 0.0220836945012291*G0_2_2;
    A[1089] = -0.00228991928210655*G0_0_1 - 0.00228991928210661*G0_0_2 - 0.00228991928210657*G0_1_1 - 0.00228991928210662*G0_1_2 - 0.00228991928210659*G0_2_0 + 0.00658922642429598*G0_2_1 + 0.00658922642429595*G0_2_2;
    A[1090] = -0.00228991928210684*G0_0_1 - 0.00228991928210687*G0_0_2 - 0.00228991928210684*G0_1_1 - 0.00228991928210687*G0_1_2 - 0.00228991928210688*G0_2_0 + 0.00939454064454049*G0_2_1 + 0.00939454064454051*G0_2_2;
    A[1091] = -0.00537576492437586*G0_0_1 - 0.00537576492437584*G0_0_2 - 0.00537576492437585*G0_1_1 - 0.00537576492437583*G0_1_2 - 0.00537576492437596*G0_2_0;
    A[1092] = -0.0118736555368493*G0_0_0 - 0.0118736555368502*G0_0_1 + 0.00939454064454028*G0_0_2 - 0.0118736555368493*G0_1_0 - 0.0118736555368502*G0_1_1 + 0.00939454064454028*G0_1_2 + 0.00939454064454073*G0_2_0 + 0.00939454064454029*G0_2_1;
    A[1093] = -0.00254435475789674*G0_0_0 - 0.00254435475789623*G0_0_1 + 0.00329461321214771*G0_0_2 - 0.00254435475789679*G0_1_0 - 0.00254435475789625*G0_1_1 + 0.0032946132121477*G0_1_2 + 0.0032946132121476*G0_2_0 + 0.00329461321214784*G0_2_1;
    A[1094] = -0.0118736555368497*G0_0_0 - 0.0118736555368499*G0_0_1 + 0.00939454064454087*G0_0_2 - 0.0118736555368496*G0_1_0 - 0.0118736555368499*G0_1_1 + 0.00939454064454088*G0_1_2 + 0.00939454064454055*G0_2_0 + 0.00939454064454046*G0_2_1;
    A[1095] = 0.0429604514890973*G0_0_0 + 0.0429604514890982*G0_0_1 + 0.000685018588665199*G0_0_2 + 0.0429604514890972*G0_1_0 + 0.0429604514890982*G0_1_1 + 0.00068501858866518*G0_1_2 + 0.000685018588664474*G0_2_0 + 0.00068501858866488*G0_2_1 + 0.0226708532915133*G0_2_2;
    A[1096] = 0.0429604514890968*G0_0_0 + 0.0429604514890965*G0_0_1 + 0.000685018588663695*G0_0_2 + 0.0429604514890969*G0_1_0 + 0.0429604514890965*G0_1_1 + 0.000685018588663727*G0_1_2 + 0.000685018588664491*G0_2_0 + 0.000685018588664228*G0_2_1 + 0.0226708532915127*G0_2_2;
    A[1097] = -0.124738623002511*G0_0_0 - 0.124738623002512*G0_0_1 - 0.02844458139597*G0_0_2 - 0.124738623002511*G0_1_0 - 0.124738623002512*G0_1_1 - 0.02844458139597*G0_1_2 - 0.02844458139597*G0_2_0 - 0.02844458139597*G0_2_1 - 0.0606730749959912*G0_2_2;
    A[1098] = 0.0118736555368495*G0_0_0 + 0.0212681961813903*G0_0_2 + 0.0118736555368496*G0_1_0 + 0.0212681961813903*G0_1_2 - 0.0146137298915079*G0_2_0 + 0.0212681961813903*G0_2_1 - 0.0693499771190745*G0_2_2;
    A[1099] = 0.00254435475789643*G0_0_0 + 0.00583896797004461*G0_0_2 + 0.00254435475789641*G0_1_0 + 0.0058389679700446*G0_1_2 - 0.00590420783563132*G0_2_0 + 0.00583896797004432*G0_2_1 - 0.0318696743392922*G0_2_2;
    A[1100] = 0.0118736555368496*G0_0_0 + 0.0212681961813905*G0_0_2 + 0.0118736555368496*G0_1_0 + 0.0212681961813905*G0_1_2 - 0.00939454064454056*G0_2_0 + 0.0212681961813906*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[1101] = -0.0429604514890969*G0_0_0 - 0.0422754329004326*G0_0_2 - 0.0429604514890969*G0_1_0 - 0.0422754329004326*G0_1_2 - 0.035914546005691*G0_2_0 - 0.0422754329004321*G0_2_1 + 0.0920860702761748*G0_2_2;
    A[1102] = -0.0429604514890968*G0_0_0 - 0.042275432900433*G0_0_2 - 0.0429604514890968*G0_1_0 - 0.042275432900433*G0_1_2 - 0.000685018588664467*G0_2_0 - 0.0422754329004329*G0_2_1 + 0.0439716694056968*G0_2_2;
    A[1103] = 0.124738623002511*G0_0_0 + 0.0962940416065412*G0_0_2 + 0.124738623002511*G0_1_0 + 0.0962940416065412*G0_1_2 + 0.0284445813959699*G0_2_0 + 0.0962940416065406*G0_2_1 - 0.064456987200043*G0_2_2;
    A[1104] = 0.0118736555368497*G0_0_1 + 0.0212681961813907*G0_0_2 + 0.0118736555368498*G0_1_1 + 0.0212681961813907*G0_1_2 + 0.0212681961813903*G0_2_0 - 0.0146137298915079*G0_2_1 - 0.0693499771190743*G0_2_2;
    A[1105] = 0.00254435475789612*G0_0_1 + 0.0058389679700442*G0_0_2 + 0.00254435475789608*G0_1_1 + 0.0058389679700442*G0_1_2 + 0.00583896797004413*G0_2_0 - 0.00590420783563148*G0_2_1 - 0.0318696743392922*G0_2_2;
    A[1106] = 0.0118736555368502*G0_0_1 + 0.0212681961813904*G0_0_2 + 0.0118736555368502*G0_1_1 + 0.0212681961813904*G0_1_2 + 0.0212681961813909*G0_2_0 - 0.00939454064454024*G0_2_1 - 0.0187890812890814*G0_2_2;
    A[1107] = -0.0429604514890969*G0_0_1 - 0.0422754329004331*G0_0_2 - 0.042960451489097*G0_1_1 - 0.0422754329004331*G0_1_2 - 0.042275432900432*G0_2_0 - 0.0359145460056909*G0_2_1 + 0.0920860702761745*G0_2_2;
    A[1108] = -0.042960451489098*G0_0_1 - 0.0422754329004327*G0_0_2 - 0.042960451489098*G0_1_1 - 0.0422754329004328*G0_1_2 - 0.0422754329004332*G0_2_0 - 0.000685018588664876*G0_2_1 + 0.0439716694056969*G0_2_2;
    A[1109] = 0.124738623002512*G0_0_1 + 0.0962940416065416*G0_0_2 + 0.124738623002512*G0_1_1 + 0.0962940416065416*G0_1_2 + 0.0962940416065408*G0_2_0 + 0.02844458139597*G0_2_1 - 0.0644569872000428*G0_2_2;
    A[1110] = -0.0093945406445407*G0_0_2 - 0.00939454064454066*G0_1_2 - 0.00939454064454074*G0_2_0 - 0.00939454064454062*G0_2_1 + 0.0453417065830259*G0_2_2;
    A[1111] = -0.00329461321214791*G0_0_2 - 0.00329461321214791*G0_1_2 - 0.00329461321214784*G0_2_0 - 0.00329461321214795*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[1112] = -0.00939454064454019*G0_0_2 - 0.00939454064454018*G0_1_2 - 0.00939454064454036*G0_2_0 - 0.00939454064454065*G0_2_1;
    A[1113] = -0.00329461321214794*G0_0_2 - 0.00329461321214794*G0_1_2 - 0.00329461321214804*G0_2_0 - 0.00329461321214797*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[1114] = -0.00329461321214768*G0_0_2 - 0.00329461321214767*G0_1_2 - 0.00329461321214794*G0_2_0 - 0.00329461321214776*G0_2_1;
    A[1115] = -0.00939454064454076*G0_0_2 - 0.00939454064454077*G0_1_2 - 0.00939454064454046*G0_2_0 - 0.0093945406445405*G0_2_1;
    A[1116] = 0.0026095946234841*G0_0_2 + 0.00260959462348406*G0_1_2 + 0.00260959462348423*G0_2_0 + 0.00260959462348414*G0_2_1 - 0.151356488162042*G0_2_2;
    A[1117] = 0.00260959462348263*G0_0_2 + 0.00260959462348265*G0_1_2 + 0.00260959462348295*G0_2_0 + 0.00260959462348357*G0_2_1 - 0.045341706583026*G0_2_2;
    A[1118] = 0.00260959462348397*G0_0_2 + 0.00260959462348393*G0_1_2 + 0.00260959462348377*G0_2_0 + 0.00260959462348325*G0_2_1 - 0.0453417065830257*G0_2_2;
    A[1119] = 0.0352295274170268*G0_0_2 + 0.0352295274170268*G0_1_2 + 0.0352295274170265*G0_2_0 + 0.0352295274170267*G0_2_1 + 0.166687856575008*G0_2_2;
    A[1120] = -0.0178705039816149*G0_0_0 - 0.0178705039816149*G0_0_1 - 0.0178705039816149*G0_0_2 - 0.0551276864210887*G0_1_0 - 0.0551276864210887*G0_1_1 - 0.0551276864210887*G0_1_2 - 0.0178705039816149*G0_2_0 - 0.0178705039816149*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[1121] = -0.00480034930989792*G0_0_0 - 0.00480034930989793*G0_2_0;
    A[1122] = 0.0105310191030677*G0_0_1 + 0.0153313684129655*G0_1_1 + 0.0105310191030677*G0_2_1;
    A[1123] = -0.00480034930989788*G0_0_2 - 0.00480034930989789*G0_2_2;
    A[1124] = 0.0274594594256048*G0_0_1 + 0.0328352243499809*G0_0_2 + 0.022083694501229*G0_1_1 + 0.0274594594256048*G0_1_2 + 0.0274594594256048*G0_2_1 + 0.0328352243499809*G0_2_2;
    A[1125] = 0.00887914570640282*G0_0_1 + 0.0111690649885094*G0_0_2 + 0.00658922642429583*G0_1_1 + 0.00887914570640276*G0_1_2 + 0.00887914570640276*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[1126] = 0.0116844599266472*G0_0_1 + 0.0139743792087542*G0_0_2 + 0.00939454064454057*G0_1_1 + 0.0116844599266473*G0_1_2 + 0.0116844599266472*G0_2_1 + 0.0139743792087542*G0_2_2;
    A[1127] = 0.00537576492437607*G0_0_1 + 0.010751529848752*G0_0_2 + 0.00537576492437606*G0_1_2 + 0.00537576492437604*G0_2_1 + 0.010751529848752*G0_2_2;
    A[1128] = 0.00537576492437594*G0_0_0 + 0.00537576492437603*G0_0_2 + 0.00537576492437589*G0_2_0 + 0.00537576492437596*G0_2_2;
    A[1129] = 0.00228991928210681*G0_0_0 + 0.00228991928210667*G0_0_2 + 0.0022899192821068*G0_2_0 + 0.00228991928210671*G0_2_2;
    A[1130] = 0.00228991928210663*G0_0_0 + 0.00228991928210677*G0_0_2 + 0.00228991928210662*G0_2_0 + 0.00228991928210671*G0_2_2;
    A[1131] = 0.00537576492437605*G0_0_0 + 0.00537576492437599*G0_0_2 + 0.00537576492437602*G0_2_0 + 0.00537576492437599*G0_2_2;
    A[1132] = 0.010751529848752*G0_0_0 + 0.00537576492437597*G0_0_1 + 0.00537576492437603*G0_1_0 + 0.0107515298487519*G0_2_0 + 0.0053757649243759*G0_2_1;
    A[1133] = 0.0139743792087539*G0_0_0 + 0.0116844599266472*G0_0_1 + 0.0116844599266471*G0_1_0 + 0.00939454064454045*G0_1_1 + 0.013974379208754*G0_2_0 + 0.0116844599266473*G0_2_1;
    A[1134] = 0.0111690649885093*G0_0_0 + 0.00887914570640254*G0_0_1 + 0.00887914570640263*G0_1_0 + 0.00658922642429586*G0_1_1 + 0.0111690649885092*G0_2_0 + 0.00887914570640251*G0_2_1;
    A[1135] = 0.0328352243499809*G0_0_0 + 0.0274594594256048*G0_0_1 + 0.0274594594256048*G0_1_0 + 0.0220836945012289*G0_1_1 + 0.032835224349981*G0_2_0 + 0.0274594594256048*G0_2_1;
    A[1136] = 0.0895482395048357*G0_0_0 + 0.0895482395048357*G0_0_1 + 0.0196371995417132*G0_0_2 + 0.0196371995417133*G0_1_0 + 0.0196371995417133*G0_1_1 + 0.0895482395048358*G0_1_2 + 0.0895482395048356*G0_2_0 + 0.0895482395048356*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[1137] = -0.0574176057031954*G0_0_0 - 0.0574176057031954*G0_0_1 + 0.0124934342599273*G0_0_2 - 0.0186912214907005*G0_1_0 - 0.0186912214907004*G0_1_1 - 0.0574176057031954*G0_1_2 - 0.0574176057031953*G0_2_0 - 0.0574176057031953*G0_2_1 + 0.0124934342599273*G0_2_2;
    A[1138] = 0.028372817543824*G0_0_0 + 0.028372817543824*G0_0_1 - 0.0315434750113569*G0_0_2 + 0.00939454064454046*G0_1_0 + 0.00939454064454044*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.028372817543824*G0_2_0 + 0.028372817543824*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1139] = -0.00537576492437602*G0_0_0 - 0.00537576492437599*G0_0_1 + 0.0220836945012292*G0_0_2 - 0.00537576492437593*G0_1_2 - 0.00537576492437598*G0_2_0 - 0.00537576492437596*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1140] = 0.179096479009672*G0_0_0 + 0.109185439046549*G0_0_1 + 0.179096479009672*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009671*G0_1_1 + 0.109185439046549*G0_1_2 + 0.179096479009672*G0_2_0 + 0.109185439046549*G0_2_1 + 0.179096479009671*G0_2_2;
    A[1141] = -0.147063705006412*G0_0_0 - 0.17344670664983*G0_0_1 - 0.147063705006412*G0_0_2 - 0.108337320793917*G0_1_0 - 0.243357746612953*G0_1_1 - 0.108337320793917*G0_1_2 - 0.147063705006412*G0_2_0 - 0.17344670664983*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1142] = 0.100717304493345*G0_0_0 + 0.12535187773903*G0_0_1 + 0.100717304493345*G0_0_2 + 0.0817390275940613*G0_1_0 + 0.18526817029421*G0_1_1 + 0.0817390275940613*G0_1_2 + 0.100717304493345*G0_2_0 + 0.12535187773903*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1143] = -0.0328352243499809*G0_0_0 - 0.0537511252572013*G0_0_1 - 0.0328352243499809*G0_0_2 - 0.0274594594256048*G0_1_0 - 0.081210584682806*G0_1_1 - 0.0274594594256048*G0_1_2 - 0.0328352243499809*G0_2_0 - 0.0537511252572013*G0_2_1 - 0.032835224349981*G0_2_2;
    A[1144] = 0.0196371995417131*G0_0_0 + 0.0895482395048357*G0_0_1 + 0.0895482395048357*G0_0_2 + 0.0895482395048358*G0_1_0 + 0.0196371995417132*G0_1_1 + 0.0196371995417131*G0_1_2 + 0.0196371995417131*G0_2_0 + 0.0895482395048356*G0_2_1 + 0.0895482395048356*G0_2_2;
    A[1145] = 0.0124934342599274*G0_0_0 - 0.0574176057031953*G0_0_1 - 0.0574176057031952*G0_0_2 - 0.0574176057031954*G0_1_0 - 0.0186912214907004*G0_1_1 - 0.0186912214907005*G0_1_2 + 0.0124934342599274*G0_2_0 - 0.0574176057031952*G0_2_1 - 0.0574176057031951*G0_2_2;
    A[1146] = -0.0315434750113567*G0_0_0 + 0.0283728175438241*G0_0_1 + 0.0283728175438241*G0_0_2 + 0.0283728175438243*G0_1_0 + 0.00939454064454066*G0_1_1 + 0.00939454064454065*G0_1_2 - 0.0315434750113567*G0_2_0 + 0.0283728175438241*G0_2_1 + 0.028372817543824*G0_2_2;
    A[1147] = 0.0220836945012291*G0_0_0 - 0.00537576492437592*G0_0_1 - 0.00537576492437592*G0_0_2 - 0.00537576492437596*G0_1_0 + 0.0220836945012291*G0_2_0 - 0.00537576492437585*G0_2_1 - 0.00537576492437585*G0_2_2;
    A[1148] = 0.0187890812890808*G0_0_0 + 0.00939454064454041*G0_0_1 + 0.0187890812890808*G0_0_2 + 0.00939454064454042*G0_1_0 + 0.00939454064454036*G0_1_2 + 0.0187890812890808*G0_2_0 + 0.00939454064454028*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1149] = 0.0292600797158083*G0_0_0 + 0.0259654665036604*G0_0_1 + 0.0292600797158083*G0_0_2 + 0.0259654665036606*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0259654665036605*G0_1_2 + 0.0292600797158084*G0_2_0 + 0.0259654665036605*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[1150] = 0.0641307878721064*G0_0_0 + 0.0547362472275659*G0_0_1 + 0.0641307878721062*G0_0_2 + 0.054736247227566*G0_1_0 + 0.0453417065830254*G0_1_1 + 0.0547362472275658*G0_1_2 + 0.0641307878721064*G0_2_0 + 0.0547362472275658*G0_2_1 + 0.0641307878721061*G0_2_2;
    A[1151] = 0.00658922642429589*G0_0_0 + 0.00329461321214786*G0_0_1 + 0.00658922642429574*G0_0_2 + 0.00329461321214797*G0_1_0 + 0.00329461321214826*G0_1_2 + 0.00658922642429592*G0_2_0 + 0.00329461321214787*G0_2_1 + 0.00658922642429579*G0_2_2;
    A[1152] = 0.0292600797158087*G0_0_0 + 0.0259654665036607*G0_0_1 + 0.0292600797158086*G0_0_2 + 0.0259654665036606*G0_1_0 + 0.0226708532915128*G0_1_1 + 0.0259654665036606*G0_1_2 + 0.0292600797158087*G0_2_0 + 0.0259654665036608*G0_2_1 + 0.0292600797158086*G0_2_2;
    A[1153] = 0.0187890812890811*G0_0_0 + 0.00939454064454062*G0_0_1 + 0.0187890812890812*G0_0_2 + 0.00939454064454063*G0_1_0 + 0.0093945406445405*G0_1_2 + 0.0187890812890811*G0_2_0 + 0.00939454064454054*G0_2_1 + 0.0187890812890812*G0_2_2;
    A[1154] = -0.0644569872000419*G0_0_0 - 0.160751028806582*G0_0_1 - 0.257045070413124*G0_0_2 - 0.0929015685960118*G0_1_0 - 0.0644569872000423*G0_1_1 - 0.160751028806583*G0_1_2 - 0.0644569872000421*G0_2_0 - 0.160751028806582*G0_2_1 - 0.257045070413124*G0_2_2;
    A[1155] = 0.1273155976932*G0_0_0 + 0.134361503176606*G0_0_1 + 0.176636936077039*G0_0_2 + 0.128000616281865*G0_1_0 + 0.0920860702761737*G0_1_1 + 0.134361503176606*G0_1_2 + 0.1273155976932*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077039*G0_2_2;
    A[1156] = -0.0641307878721065*G0_0_0 - 0.0906181733004636*G0_0_1 - 0.111886369481854*G0_0_2 - 0.0547362472275661*G0_1_0 - 0.0693499771190737*G0_1_1 - 0.0906181733004642*G0_1_2 - 0.0641307878721065*G0_2_0 - 0.0906181733004636*G0_2_1 - 0.111886369481854*G0_2_2;
    A[1157] = 0.0439716694056969*G0_0_0 + 0.0862471023061294*G0_0_1 + 0.128522535206562*G0_0_2 + 0.0446566879943612*G0_1_0 + 0.0439716694056969*G0_1_1 + 0.0862471023061291*G0_1_2 + 0.043971669405697*G0_2_0 + 0.0862471023061294*G0_2_1 + 0.128522535206562*G0_2_2;
    A[1158] = -0.0292600797158086*G0_0_0 - 0.0377086423093369*G0_0_1 - 0.0435476102793809*G0_0_2 - 0.0259654665036606*G0_1_0 - 0.0318696743392923*G0_1_1 - 0.0377086423093366*G0_1_2 - 0.0292600797158086*G0_2_0 - 0.0377086423093369*G0_2_1 - 0.043547610279381*G0_2_2;
    A[1159] = -0.0187890812890811*G0_0_0 - 0.0400572774704711*G0_0_1 - 0.0613254736518619*G0_0_2 - 0.00939454064454058*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.0400572774704713*G0_1_2 - 0.0187890812890811*G0_2_0 - 0.0400572774704711*G0_2_1 - 0.0613254736518619*G0_2_2;
    A[1160] = -0.128522535206562*G0_0_0 - 0.032228493600021*G0_0_1 - 0.128522535206562*G0_0_2 - 0.032228493600021*G0_1_0 - 0.0606730749959907*G0_1_1 - 0.0322284936000208*G0_1_2 - 0.128522535206561*G0_2_0 - 0.032228493600021*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1161] = 0.0642612676032809*G0_0_0 + 0.0219858347028484*G0_0_1 + 0.0642612676032809*G0_0_2 + 0.0219858347028483*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0219858347028481*G0_1_2 + 0.0642612676032808*G0_2_0 + 0.0219858347028484*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[1162] = -0.030662736825931*G0_0_0 - 0.00939454064454055*G0_0_1 - 0.030662736825931*G0_0_2 - 0.0093945406445406*G0_1_0 - 0.00939454064454073*G0_1_2 - 0.0306627368259309*G0_2_0 - 0.00939454064454042*G0_2_1 - 0.030662736825931*G0_2_2;
    A[1163] = 0.0642612676032809*G0_0_0 + 0.0219858347028484*G0_0_1 + 0.0642612676032808*G0_0_2 + 0.0219858347028482*G0_1_0 + 0.0226708532915124*G0_1_1 + 0.021985834702848*G0_1_2 + 0.0642612676032808*G0_2_0 + 0.0219858347028484*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[1164] = -0.00913358118219245*G0_0_0 - 0.00329461321214783*G0_0_1 - 0.00913358118219215*G0_0_2 - 0.00329461321214802*G0_1_0 - 0.00329461321214774*G0_1_2 - 0.00913358118219242*G0_2_0 - 0.00329461321214783*G0_2_1 - 0.00913358118219212*G0_2_2;
    A[1165] = -0.0306627368259306*G0_0_0 - 0.0093945406445405*G0_0_1 - 0.0306627368259309*G0_0_2 - 0.00939454064454026*G0_1_0 - 0.00939454064454032*G0_1_2 - 0.0306627368259305*G0_2_0 - 0.00939454064454042*G0_2_1 - 0.0306627368259308*G0_2_2;
    A[1166] = -0.257045070413123*G0_0_0 - 0.160751028806582*G0_0_1 - 0.0644569872000415*G0_0_2 - 0.160751028806583*G0_1_0 - 0.0644569872000417*G0_1_1 - 0.0929015685960116*G0_1_2 - 0.257045070413123*G0_2_0 - 0.160751028806582*G0_2_1 - 0.0644569872000416*G0_2_2;
    A[1167] = 0.128522535206562*G0_0_0 + 0.0862471023061291*G0_0_1 + 0.0439716694056964*G0_0_2 + 0.0862471023061291*G0_1_0 + 0.0439716694056965*G0_1_1 + 0.0446566879943609*G0_1_2 + 0.128522535206562*G0_2_0 + 0.086247102306129*G0_2_1 + 0.0439716694056964*G0_2_2;
    A[1168] = -0.061325473651862*G0_0_0 - 0.0400572774704714*G0_0_1 - 0.018789081289081*G0_0_2 - 0.0400572774704716*G0_1_0 - 0.018789081289081*G0_1_1 - 0.00939454064454043*G0_1_2 - 0.0613254736518619*G0_2_0 - 0.0400572774704714*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[1169] = 0.176636936077038*G0_0_0 + 0.134361503176605*G0_0_1 + 0.1273155976932*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761731*G0_1_1 + 0.128000616281864*G0_1_2 + 0.176636936077038*G0_2_0 + 0.134361503176605*G0_2_1 + 0.1273155976932*G0_2_2;
    A[1170] = -0.0435476102793802*G0_0_0 - 0.0377086423093362*G0_0_1 - 0.0292600797158081*G0_0_2 - 0.0377086423093358*G0_1_0 - 0.0318696743392919*G0_1_1 - 0.0259654665036605*G0_1_2 - 0.0435476102793802*G0_2_0 - 0.0377086423093362*G0_2_1 - 0.0292600797158081*G0_2_2;
    A[1171] = -0.111886369481854*G0_0_0 - 0.0906181733004634*G0_0_1 - 0.0641307878721063*G0_0_2 - 0.090618173300464*G0_1_0 - 0.0693499771190732*G0_1_1 - 0.0547362472275659*G0_1_2 - 0.111886369481854*G0_2_0 - 0.0906181733004634*G0_2_1 - 0.0641307878721063*G0_2_2;
    A[1172] = 0.0962288017409534*G0_0_0 + 0.131458329157981*G0_0_1 + 0.0962288017409537*G0_0_2 + 0.13145832915798*G0_1_0 + 0.166687856575007*G0_1_1 + 0.13145832915798*G0_1_2 + 0.0962288017409535*G0_2_0 + 0.131458329157981*G0_2_1 + 0.0962288017409538*G0_2_2;
    A[1173] = -0.0505608958299924*G0_0_0 - 0.0479513012065087*G0_0_1 - 0.0505608958299921*G0_0_2 - 0.0479513012065083*G0_1_0 - 0.0453417065830252*G0_1_1 - 0.047951301206509*G0_1_2 - 0.0505608958299923*G0_2_0 - 0.0479513012065087*G0_2_1 - 0.0505608958299921*G0_2_2;
    A[1174] = -0.156575677409009*G0_0_0 - 0.153966082785525*G0_0_1 - 0.156575677409009*G0_0_2 - 0.153966082785526*G0_1_0 - 0.151356488162042*G0_1_1 - 0.153966082785525*G0_1_2 - 0.156575677409009*G0_2_0 - 0.153966082785525*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1175] = -0.0505608958299929*G0_0_0 - 0.0479513012065093*G0_0_1 - 0.0505608958299932*G0_0_2 - 0.0479513012065092*G0_1_0 - 0.0453417065830253*G0_1_1 - 0.0479513012065091*G0_1_2 - 0.050560895829993*G0_2_0 - 0.0479513012065094*G0_2_1 - 0.0505608958299932*G0_2_2;
    A[1176] = 0.0214691349673986*G0_0_0 + 0.0214691349673986*G0_0_1 + 0.0214691349673986*G0_0_2 + 0.0551276864210887*G0_1_0 + 0.0551276864210887*G0_1_1 + 0.0551276864210887*G0_1_2 + 0.0214691349673986*G0_2_0 + 0.0214691349673987*G0_2_1 + 0.0214691349673986*G0_2_2;
    A[1177] = -0.00336507226698191*G0_0_0 - 0.00336507226698189*G0_2_0;
    A[1178] = -0.0232632312710436*G0_0_1 - 0.0388177200243169*G0_1_1 - 0.0232632312710436*G0_2_1;
    A[1179] = -0.00336507226698204*G0_0_2 - 0.00336507226698204*G0_2_2;
    A[1180] = -0.0599162925551808*G0_0_1 - 0.0788945694544645*G0_0_2 - 0.0315434750113567*G0_1_1 - 0.0599162925551808*G0_1_2 - 0.0599162925551809*G0_2_1 - 0.0788945694544646*G0_2_2;
    A[1181] = -0.00578025209101617*G0_0_1 - 0.00523223722008459*G0_0_2 - 0.00626302709636039*G0_1_1 - 0.00578025209101616*G0_1_2 - 0.00578025209101608*G0_2_1 - 0.00523223722008449*G0_2_2;
    A[1182] = -0.0131523569023567*G0_0_1 - 0.0263047138047137*G0_0_2 + 6.52398655869983e-05*G0_1_1 - 0.0131523569023568*G0_1_2 - 0.0131523569023567*G0_2_1 - 0.0263047138047137*G0_2_2;
    A[1183] = 0.00228991928210672*G0_0_1 + 0.0139743792087543*G0_0_2 + 0.00228991928210673*G0_1_2 + 0.00228991928210679*G0_2_1 + 0.0139743792087544*G0_2_2;
    A[1184] = 0.00228991928210673*G0_0_0 + 0.00228991928210684*G0_0_2 + 0.0022899192821067*G0_2_0 + 0.00228991928210684*G0_2_2;
    A[1185] = 0.00178757231708626*G0_0_0 + 0.00178757231708612*G0_0_2 + 0.00178757231708635*G0_2_0 + 0.00178757231708617*G0_2_2;
    A[1186] = 0.00178757231708628*G0_0_0 + 0.00178757231708623*G0_0_2 + 0.0017875723170862*G0_2_0 + 0.0017875723170862*G0_2_2;
    A[1187] = 0.00228991928210675*G0_0_0 + 0.00228991928210688*G0_0_2 + 0.00228991928210682*G0_2_0 + 0.00228991928210692*G0_2_2;
    A[1188] = 0.013974379208754*G0_0_0 + 0.00228991928210683*G0_0_1 + 0.00228991928210661*G0_1_0 + 0.013974379208754*G0_2_0 + 0.00228991928210684*G0_2_1;
    A[1189] = -0.0263047138047136*G0_0_0 - 0.013152356902357*G0_0_1 - 0.0131523569023567*G0_1_0 + 6.52398655870391e-05*G0_1_1 - 0.0263047138047136*G0_2_0 - 0.013152356902357*G0_2_1;
    A[1190] = -0.00523223722008423*G0_0_0 - 0.00578025209101573*G0_0_1 - 0.00578025209101593*G0_1_0 - 0.00626302709636035*G0_1_1 - 0.00523223722008418*G0_2_0 - 0.00578025209101573*G0_2_1;
    A[1191] = -0.0788945694544646*G0_0_0 - 0.0599162925551808*G0_0_1 - 0.0599162925551808*G0_1_0 - 0.0315434750113565*G0_1_1 - 0.0788945694544647*G0_2_0 - 0.0599162925551809*G0_2_1;
    A[1192] = -0.0574176057031953*G0_0_0 - 0.0574176057031954*G0_0_1 - 0.0186912214907005*G0_0_2 + 0.0124934342599271*G0_1_0 + 0.0124934342599271*G0_1_1 - 0.0574176057031955*G0_1_2 - 0.0574176057031952*G0_2_0 - 0.0574176057031953*G0_2_1 - 0.0186912214907005*G0_2_2;
    A[1193] = 0.00506261356955771*G0_0_0 + 0.00506261356955777*G0_0_1 + 0.00026095946234832*G0_0_2 + 0.000260959462348404*G0_1_0 + 0.000260959462348369*G0_1_1 + 0.00506261356955795*G0_1_2 + 0.00506261356955768*G0_2_0 + 0.00506261356955774*G0_2_1 + 0.000260959462348367*G0_2_2;
    A[1194] = -0.000482775005344306*G0_0_0 - 0.000482775005344313*G0_0_1 - 0.00626302709636042*G0_0_2 + 6.52398655870088e-05*G0_1_0 + 6.52398655870266e-05*G0_1_1 - 0.000482775005344406*G0_1_2 - 0.000482775005344293*G0_2_0 - 0.000482775005344301*G0_2_1 - 0.00626302709636048*G0_2_2;
    A[1195] = -0.00228991928210678*G0_0_0 - 0.00228991928210679*G0_0_1 + 0.00658922642429596*G0_0_2 - 0.00228991928210682*G0_1_2 - 0.00228991928210685*G0_2_0 - 0.00228991928210686*G0_2_1 + 0.00658922642429598*G0_2_2;
    A[1196] = -0.147063705006412*G0_0_0 - 0.108337320793917*G0_0_1 - 0.147063705006412*G0_0_2 - 0.17344670664983*G0_1_0 - 0.243357746612953*G0_1_1 - 0.17344670664983*G0_1_2 - 0.147063705006412*G0_2_0 - 0.108337320793917*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1197] = 0.304226541205705*G0_0_0 + 0.226095278178609*G0_0_1 + 0.304226541205705*G0_0_2 + 0.226095278178609*G0_1_0 + 0.428965164208216*G0_1_1 + 0.226095278178609*G0_1_2 + 0.304226541205705*G0_2_0 + 0.226095278178609*G0_2_1 + 0.304226541205705*G0_2_2;
    A[1198] = -0.204187731314465*G0_0_0 - 0.219493003781196*G0_0_1 - 0.204187731314465*G0_0_2 - 0.167692550505049*G0_1_0 - 0.387185554286245*G0_1_1 - 0.167692550505049*G0_1_2 - 0.204187731314466*G0_2_0 - 0.219493003781196*G0_2_1 - 0.204187731314466*G0_2_2;
    A[1199] = 0.0788945694544646*G0_0_0 + 0.103529142700149*G0_0_1 + 0.0788945694544646*G0_0_2 + 0.0599162925551808*G0_1_0 + 0.18526817029421*G0_1_1 + 0.0599162925551808*G0_1_2 + 0.0788945694544646*G0_2_0 + 0.103529142700149*G0_2_1 + 0.0788945694544647*G0_2_2;
    A[1200] = -0.0186912214907003*G0_0_0 - 0.0574176057031954*G0_0_1 - 0.0574176057031954*G0_0_2 - 0.0574176057031955*G0_1_0 + 0.0124934342599271*G0_1_1 + 0.0124934342599271*G0_1_2 - 0.0186912214907004*G0_2_0 - 0.0574176057031953*G0_2_1 - 0.0574176057031953*G0_2_2;
    A[1201] = 0.000260959462348346*G0_0_0 + 0.00506261356955797*G0_0_1 + 0.00506261356955796*G0_0_2 + 0.00506261356955813*G0_1_0 + 0.000260959462348482*G0_1_1 + 0.000260959462348473*G0_1_2 + 0.000260959462348372*G0_2_0 + 0.00506261356955794*G0_2_1 + 0.00506261356955792*G0_2_2;
    A[1202] = -0.00626302709636051*G0_0_0 - 0.000482775005344457*G0_0_1 - 0.000482775005344466*G0_0_2 - 0.00048277500534467*G0_1_0 + 6.5239865586743e-05*G0_1_1 + 6.52398655867653e-05*G0_1_2 - 0.00626302709636053*G0_2_0 - 0.000482775005344463*G0_2_1 - 0.000482775005344474*G0_2_2;
    A[1203] = 0.00658922642429579*G0_0_0 - 0.0022899192821069*G0_0_1 - 0.00228991928210685*G0_0_2 - 0.00228991928210673*G0_1_0 + 0.00658922642429579*G0_2_0 - 0.00228991928210691*G0_2_1 - 0.00228991928210686*G0_2_2;
    A[1204] = -0.0148746893538558*G0_0_0 + 6.52398655871429e-05*G0_0_1 - 0.014874689353856*G0_0_2 + 6.52398655871237e-05*G0_1_0 + 6.52398655870483e-05*G0_1_2 - 0.0148746893538556*G0_2_0 + 6.523986558727e-05*G0_2_1 - 0.0148746893538559*G0_2_2;
    A[1205] = -0.0189848008858417*G0_0_0 - 0.00949240044292074*G0_0_1 - 0.0189848008858419*G0_0_2 - 0.00949240044292101*G0_1_0 - 0.000652398655870851*G0_1_1 - 0.00949240044292115*G0_1_2 - 0.0189848008858418*G0_2_0 - 0.00949240044292089*G0_2_1 - 0.018984800885842*G0_2_2;
    A[1206] = -0.127087258163646*G0_0_0 - 0.089313375988722*G0_0_1 - 0.127087258163645*G0_0_2 - 0.0893133759887223*G0_1_0 - 0.0665446628988287*G0_1_1 - 0.0893133759887218*G0_1_2 - 0.127087258163646*G0_2_0 - 0.089313375988722*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1207] = 0.00450155072550886*G0_0_0 + 0.00257697469068995*G0_0_1 + 0.00450155072550929*G0_0_2 + 0.0025769746906899*G0_1_0 + 0.0025769746906898*G0_1_2 + 0.00450155072550876*G0_2_0 + 0.00257697469068994*G0_2_1 + 0.0045015507255093*G0_2_2;
    A[1208] = -0.0189848008858426*G0_0_0 - 0.00949240044292125*G0_0_1 - 0.0189848008858424*G0_0_2 - 0.00949240044292131*G0_1_0 - 0.000652398655870922*G0_1_1 - 0.0094924004429212*G0_1_2 - 0.0189848008858426*G0_2_0 - 0.00949240044292135*G0_2_1 - 0.0189848008858424*G0_2_2;
    A[1209] = -0.0148746893538557*G0_0_0 + 6.52398655872481e-05*G0_0_1 - 0.0148746893538561*G0_0_2 + 6.52398655870162e-05*G0_1_0 + 6.52398655869991e-05*G0_1_2 - 0.0148746893538556*G0_2_0 + 6.52398655872703e-05*G0_2_1 - 0.0148746893538562*G0_2_2;
    A[1210] = 0.132306447410613*G0_0_0 + 0.194675758911868*G0_0_1 + 0.121346149991982*G0_0_2 + 0.0589768384907264*G0_1_0 - 0.00339247301052842*G0_1_1 + 0.194675758911868*G0_1_2 + 0.132306447410613*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1211] = -0.166948816037356*G0_0_0 - 0.250814663249557*G0_0_1 - 0.334680510461758*G0_0_2 - 0.160783648739376*G0_1_0 - 0.0622388317700813*G0_1_1 - 0.250814663249557*G0_1_2 - 0.166948816037356*G0_2_0 - 0.250814663249557*G0_2_1 - 0.334680510461758*G0_2_2;
    A[1212] = 0.127087258163646*G0_0_0 + 0.173472802596065*G0_0_1 + 0.20941996853455*G0_0_2 + 0.0893133759887224*G0_1_0 + 0.0846813455320393*G0_1_1 + 0.173472802596065*G0_1_2 + 0.127087258163646*G0_2_0 + 0.173472802596065*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1213] = -0.043645470077761*G0_0_0 - 0.0371541034518459*G0_0_1 + 0.0371867233846405*G0_0_2 - 0.00975335990526968*G0_1_0 - 0.00678494602105721*G0_1_1 - 0.0371541034518461*G0_1_2 - 0.0436454700777611*G0_2_0 - 0.0371541034518459*G0_2_1 + 0.0371867233846406*G0_2_2;
    A[1214] = 0.0189848008858426*G0_0_0 + 0.0378717419733043*G0_0_1 + 0.0567586830607661*G0_0_2 + 0.00949240044292127*G0_1_0 + 0.0127870136550692*G0_1_1 + 0.0378717419733044*G0_1_2 + 0.0189848008858426*G0_2_0 + 0.0378717419733042*G0_2_1 + 0.0567586830607662*G0_2_2;
    A[1215] = 0.0148746893538556*G0_0_0 + 0.0136351319077009*G0_0_1 - 0.0254435475789647*G0_0_2 - 6.52398655870358e-05*G0_1_0 - 0.000130479731174047*G0_1_1 + 0.0136351319077011*G0_1_2 + 0.0148746893538555*G0_2_0 + 0.013635131907701*G0_2_1 - 0.0254435475789647*G0_2_2;
    A[1216] = 0.0476251018785734*G0_0_0 - 0.0147442096226819*G0_0_1 + 0.0476251018785734*G0_0_2 - 0.0147442096226816*G0_1_0 + 0.0476251018785734*G0_1_1 - 0.0147442096226817*G0_1_2 + 0.0476251018785734*G0_2_0 - 0.0147442096226818*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[1217] = 0.00880738185425672*G0_0_0 + 0.00231601522834156*G0_0_1 + 0.00880738185425674*G0_0_2 + 0.00231601522834134*G0_1_0 - 0.000652398655871005*G0_1_1 + 0.00231601522834171*G0_1_2 + 0.00880738185425671*G0_2_0 + 0.00231601522834151*G0_2_1 + 0.00880738185425676*G0_2_2;
    A[1218] = -0.00130479731174164*G0_0_0 - 6.52398655870965e-05*G0_0_1 - 0.0013047973117416*G0_0_2 - 6.52398655867137e-05*G0_1_0 - 6.52398655866841e-05*G0_1_2 - 0.00130479731174159*G0_2_0 - 6.52398655872222e-05*G0_2_1 - 0.00130479731174165*G0_2_2;
    A[1219] = 0.00880738185425729*G0_0_0 + 0.00231601522834195*G0_0_1 + 0.00880738185425735*G0_0_2 + 0.00231601522834167*G0_1_0 - 0.000652398655870759*G0_1_1 + 0.00231601522834162*G0_1_2 + 0.00880738185425729*G0_2_0 + 0.00231601522834187*G0_2_1 + 0.00880738185425736*G0_2_2;
    A[1220] = -0.0156575677409011*G0_0_0 - 0.00257697469068996*G0_0_1 - 0.0156575677409011*G0_0_2 - 0.00257697469068994*G0_1_0 - 0.00257697469069012*G0_1_2 - 0.0156575677409011*G0_2_0 - 0.00257697469068995*G0_2_1 - 0.015657567740901*G0_2_2;
    A[1221] = -0.00130479731174203*G0_0_0 - 6.52398655873351e-05*G0_0_1 - 0.00130479731174219*G0_0_2 - 6.52398655870433e-05*G0_1_0 - 6.52398655870594e-05*G0_1_2 - 0.00130479731174197*G0_2_0 - 6.5239865587358e-05*G0_2_1 - 0.00130479731174222*G0_2_2;
    A[1222] = 0.121346149991981*G0_0_0 + 0.194675758911868*G0_0_1 + 0.132306447410612*G0_0_2 + 0.194675758911868*G0_1_0 - 0.00339247301052892*G0_1_1 + 0.0589768384907261*G0_1_2 + 0.121346149991981*G0_2_0 + 0.194675758911868*G0_2_1 + 0.132306447410613*G0_2_2;
    A[1223] = 0.0371867233846402*G0_0_0 - 0.0371541034518464*G0_0_1 - 0.0436454700777613*G0_0_2 - 0.0371541034518462*G0_1_0 - 0.00678494602105686*G0_1_1 - 0.0097533599052695*G0_1_2 + 0.0371867233846401*G0_2_0 - 0.0371541034518463*G0_2_1 - 0.0436454700777613*G0_2_2;
    A[1224] = -0.0254435475789641*G0_0_0 + 0.0136351319077015*G0_0_1 + 0.0148746893538561*G0_0_2 + 0.0136351319077016*G0_1_0 - 0.00013047973117383*G0_1_1 - 6.52398655869618e-05*G0_1_2 - 0.025443547578964*G0_2_0 + 0.0136351319077016*G0_2_1 + 0.014874689353856*G0_2_2;
    A[1225] = -0.334680510461757*G0_0_0 - 0.250814663249556*G0_0_1 - 0.166948816037356*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700805*G0_1_1 - 0.160783648739376*G0_1_2 - 0.334680510461757*G0_2_0 - 0.250814663249556*G0_2_1 - 0.166948816037356*G0_2_2;
    A[1226] = 0.0567586830607656*G0_0_0 + 0.0378717419733039*G0_0_1 + 0.0189848008858416*G0_0_2 + 0.0378717419733034*G0_1_0 + 0.0127870136550687*G0_1_1 + 0.00949240044292108*G0_1_2 + 0.0567586830607655*G0_2_0 + 0.0378717419733039*G0_2_1 + 0.0189848008858418*G0_2_2;
    A[1227] = 0.20941996853455*G0_0_0 + 0.173472802596064*G0_0_1 + 0.127087258163645*G0_0_2 + 0.173472802596065*G0_1_0 + 0.0846813455320386*G0_1_1 + 0.089313375988722*G0_1_2 + 0.20941996853455*G0_2_0 + 0.173472802596064*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1228] = -0.156575677409009*G0_0_0 - 0.0662184635708931*G0_0_1 - 0.156575677409009*G0_0_2 - 0.0662184635708924*G0_1_0 - 0.161794866655975*G0_1_1 - 0.0662184635708929*G0_1_2 - 0.156575677409009*G0_2_0 - 0.0662184635708932*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1229] = 0.0391439193522522*G0_0_0 + 0.00717638521457925*G0_0_1 + 0.039143919352252*G0_0_2 + 0.0071763852145789*G0_1_0 + 0.00130479731174175*G0_1_1 + 0.00717638521457957*G0_1_2 + 0.0391439193522519*G0_2_0 + 0.00717638521457946*G0_2_1 + 0.039143919352252*G0_2_2;
    A[1230] = 0.185933616923197*G0_0_0 + 0.170276049182297*G0_0_1 + 0.185933616923198*G0_0_2 + 0.170276049182297*G0_1_0 + 0.180714427676231*G0_1_1 + 0.170276049182297*G0_1_2 + 0.185933616923198*G0_2_0 + 0.170276049182297*G0_2_1 + 0.185933616923198*G0_2_2;
    A[1231] = 0.0391439193522522*G0_0_0 + 0.00717638521457953*G0_0_1 + 0.0391439193522526*G0_0_2 + 0.00717638521457983*G0_1_0 + 0.00130479731174176*G0_1_1 + 0.00717638521457985*G0_1_2 + 0.0391439193522524*G0_2_0 + 0.00717638521457971*G0_2_1 + 0.0391439193522527*G0_2_2;
    A[1232] = -0.0155544887532732*G0_0_0 - 0.0155544887532733*G0_0_1 - 0.0155544887532733*G0_0_2 - 0.0388177200243169*G0_1_0 - 0.0388177200243168*G0_1_1 - 0.0388177200243168*G0_1_2 - 0.0155544887532732*G0_2_0 - 0.0155544887532733*G0_2_1 - 0.0155544887532733*G0_2_2;
    A[1233] = -0.00336507226698209*G0_0_0 - 0.0033650722669821*G0_2_0;
    A[1234] = 0.0336585514536901*G0_0_1 + 0.0551276864210887*G0_1_1 + 0.0336585514536901*G0_2_1;
    A[1235] = -0.00336507226698192*G0_0_2 - 0.00336507226698191*G0_2_2;
    A[1236] = 0.0699110399631225*G0_0_1 + 0.108637424175618*G0_0_2 + 0.0124934342599273*G0_1_1 + 0.0699110399631227*G0_1_2 + 0.0699110399631225*G0_2_1 + 0.108637424175618*G0_2_2;
    A[1237] = -0.00480165410720937*G0_0_1 - 0.00960330821441906*G0_0_2 + 0.000260959462348364*G0_1_1 - 0.00480165410720946*G0_1_2 - 0.00480165410720934*G0_2_1 - 0.009603308214419*G0_2_2;
    A[1238] = 0.000548014870931404*G0_0_1 - 0.00523223722008444*G0_0_2 + 6.52398655871953e-05*G0_1_1 + 0.00054801487093158*G0_1_2 + 0.000548014870931427*G0_2_1 - 0.00523223722008443*G0_2_2;
    A[1239] = 0.00228991928210685*G0_0_1 + 0.0111690649885093*G0_0_2 + 0.00228991928210684*G0_1_2 + 0.00228991928210684*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[1240] = 0.00228991928210688*G0_0_0 + 0.00228991928210669*G0_0_2 + 0.00228991928210681*G0_2_0 + 0.0022899192821066*G0_2_2;
    A[1241] = 0.00178757231708624*G0_0_0 + 0.00178757231708634*G0_0_2 + 0.00178757231708624*G0_2_0 + 0.00178757231708636*G0_2_2;
    A[1242] = 0.00178757231708613*G0_0_0 + 0.00178757231708626*G0_0_2 + 0.00178757231708612*G0_2_0 + 0.00178757231708624*G0_2_2;
    A[1243] = 0.00228991928210683*G0_0_0 + 0.00228991928210669*G0_0_2 + 0.00228991928210683*G0_2_0 + 0.0022899192821067*G0_2_2;
    A[1244] = 0.0111690649885097*G0_0_0 + 0.00228991928210669*G0_0_1 + 0.00228991928210692*G0_1_0 + 0.0111690649885096*G0_2_0 + 0.0022899192821066*G0_2_1;
    A[1245] = -0.00523223722008456*G0_0_0 + 0.000548014870931769*G0_0_1 + 0.000548014870931556*G0_1_0 + 6.52398655873016e-05*G0_1_1 - 0.00523223722008445*G0_2_0 + 0.000548014870931864*G0_2_1;
    A[1246] = -0.00960330821441936*G0_0_0 - 0.00480165410720971*G0_0_1 - 0.00480165410720967*G0_1_0 + 0.000260959462348245*G0_1_1 - 0.00960330821441946*G0_2_0 - 0.00480165410720981*G0_2_1;
    A[1247] = 0.108637424175618*G0_0_0 + 0.0699110399631225*G0_0_1 + 0.0699110399631227*G0_1_0 + 0.0124934342599272*G0_1_1 + 0.108637424175618*G0_2_0 + 0.0699110399631226*G0_2_1;
    A[1248] = 0.0283728175438241*G0_0_0 + 0.0283728175438241*G0_0_1 + 0.00939454064454039*G0_0_2 - 0.0315434750113565*G0_1_0 - 0.0315434750113565*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.028372817543824*G0_2_0 + 0.0283728175438241*G0_2_1 + 0.0093945406445404*G0_2_2;
    A[1249] = -0.000482775005344333*G0_0_0 - 0.000482775005344363*G0_0_1 + 6.52398655873058e-05*G0_0_2 - 0.00626302709636067*G0_1_0 - 0.00626302709636064*G0_1_1 - 0.000482775005344395*G0_1_2 - 0.000482775005344301*G0_2_0 - 0.000482775005344332*G0_2_1 + 6.52398655872791e-05*G0_2_2;
    A[1250] = 0.0132175967679439*G0_0_0 + 0.0132175967679439*G0_0_1 + 6.52398655869902e-05*G0_0_2 + 6.52398655874986e-05*G0_1_0 + 6.52398655874779e-05*G0_1_1 + 0.0132175967679439*G0_1_2 + 0.0132175967679439*G0_2_0 + 0.0132175967679439*G0_2_1 + 6.52398655870383e-05*G0_2_2;
    A[1251] = -0.00228991928210681*G0_0_0 - 0.00228991928210681*G0_0_1 + 0.00939454064454058*G0_0_2 - 0.00228991928210673*G0_1_2 - 0.00228991928210681*G0_2_0 - 0.00228991928210681*G0_2_1 + 0.00939454064454054*G0_2_2;
    A[1252] = 0.100717304493345*G0_0_0 + 0.0817390275940613*G0_0_1 + 0.100717304493345*G0_0_2 + 0.12535187773903*G0_1_0 + 0.18526817029421*G0_1_1 + 0.12535187773903*G0_1_2 + 0.100717304493345*G0_2_0 + 0.0817390275940613*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1253] = -0.204187731314465*G0_0_0 - 0.167692550505049*G0_0_1 - 0.204187731314465*G0_0_2 - 0.219493003781196*G0_1_0 - 0.387185554286245*G0_1_1 - 0.219493003781196*G0_1_2 - 0.204187731314465*G0_2_0 - 0.167692550505049*G0_2_1 - 0.204187731314466*G0_2_2;
    A[1254] = 0.281001149056702*G0_0_0 + 0.202869886029607*G0_0_1 + 0.281001149056702*G0_0_2 + 0.202869886029607*G0_1_0 + 0.428965164208216*G0_1_1 + 0.202869886029607*G0_1_2 + 0.281001149056702*G0_2_0 + 0.202869886029607*G0_2_1 + 0.281001149056702*G0_2_2;
    A[1255] = -0.108637424175618*G0_0_0 - 0.135020425819036*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0699110399631226*G0_1_0 - 0.243357746612953*G0_1_1 - 0.0699110399631227*G0_1_2 - 0.108637424175618*G0_2_0 - 0.135020425819036*G0_2_1 - 0.108637424175618*G0_2_2;
    A[1256] = 0.00939454064454042*G0_0_0 + 0.0283728175438243*G0_0_1 + 0.0283728175438242*G0_0_2 + 0.0283728175438243*G0_1_0 - 0.0315434750113564*G0_1_1 - 0.0315434750113564*G0_1_2 + 0.00939454064454042*G0_2_0 + 0.0283728175438242*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1257] = 6.52398655869879e-05*G0_0_0 - 0.000482775005344476*G0_0_1 - 0.000482775005344468*G0_0_2 - 0.000482775005344677*G0_1_0 - 0.00626302709636048*G0_1_1 - 0.00626302709636046*G0_1_2 + 6.52398655869988e-05*G0_2_0 - 0.000482775005344384*G0_2_1 - 0.000482775005344373*G0_2_2;
    A[1258] = 6.52398655872328e-05*G0_0_0 + 0.0132175967679439*G0_0_1 + 0.0132175967679439*G0_0_2 + 0.0132175967679441*G0_1_0 + 6.52398655875107e-05*G0_1_1 + 6.52398655874892e-05*G0_1_2 + 6.52398655872214e-05*G0_2_0 + 0.0132175967679438*G0_2_1 + 0.0132175967679439*G0_2_2;
    A[1259] = 0.00939454064454071*G0_0_0 - 0.00228991928210662*G0_0_1 - 0.00228991928210664*G0_0_2 - 0.00228991928210681*G0_1_0 + 0.00939454064454072*G0_2_0 - 0.00228991928210653*G0_2_1 - 0.00228991928210655*G0_2_2;
    A[1260] = -0.00117431758056779*G0_0_0 + 6.5239865587218e-05*G0_0_1 - 0.00117431758056736*G0_0_2 + 6.52398655871218e-05*G0_1_0 + 6.5239865587198e-05*G0_1_2 - 0.00117431758056782*G0_2_0 + 6.52398655870565e-05*G0_2_1 - 0.00117431758056744*G0_2_2;
    A[1261] = 0.00352295274170223*G0_0_0 - 0.00296841388421298*G0_0_1 + 0.00352295274170244*G0_0_2 - 0.00296841388421271*G0_1_0 - 0.000652398655870829*G0_1_1 - 0.00296841388421228*G0_1_2 + 0.00352295274170218*G0_2_0 - 0.00296841388421286*G0_2_1 + 0.00352295274170251*G0_2_2;
    A[1262] = 0.124738623002511*G0_0_0 + 0.0623693115012552*G0_0_1 + 0.12473862300251*G0_0_2 + 0.0623693115012554*G0_1_0 + 0.0476251018785735*G0_1_1 + 0.062369311501255*G0_1_2 + 0.124738623002511*G0_2_0 + 0.0623693115012552*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1263] = -0.0105036183595206*G0_0_0 + 0.00257697469069019*G0_0_1 - 0.0105036183595213*G0_0_2 + 0.00257697469068985*G0_1_0 + 0.0025769746906898*G0_1_2 - 0.0105036183595206*G0_2_0 + 0.00257697469069019*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[1264] = 0.00352295274170325*G0_0_0 - 0.00296841388421194*G0_0_1 + 0.00352295274170331*G0_0_2 - 0.00296841388421217*G0_1_0 - 0.000652398655870752*G0_1_1 - 0.00296841388421232*G0_1_2 + 0.00352295274170333*G0_2_0 - 0.00296841388421187*G0_2_1 + 0.00352295274170344*G0_2_2;
    A[1265] = -0.00117431758056765*G0_0_0 + 6.52398655870681e-05*G0_0_1 - 0.00117431758056721*G0_0_2 + 6.52398655872e-05*G0_1_0 + 6.52398655872075e-05*G0_1_2 - 0.00117431758056763*G0_2_0 + 6.52398655870822e-05*G0_2_1 - 0.00117431758056717*G0_2_2;
    A[1266] = -0.0510175748891014*G0_0_0 - 0.0887914570640253*G0_0_1 - 0.0528442911255402*G0_0_2 - 0.00463203045668261*G0_1_0 + 0.0846813455320393*G0_1_1 - 0.088791457064026*G0_1_2 - 0.0510175748891015*G0_2_0 - 0.0887914570640253*G0_2_1 - 0.05284429112554*G0_2_2;
    A[1267] = 0.182410664181495*G0_0_0 + 0.188575831479475*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0985448169692951*G0_1_0 - 0.0622388317700814*G0_1_1 + 0.188575831479476*G0_1_2 + 0.182410664181495*G0_2_0 + 0.188575831479475*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1268] = -0.124738623002511*G0_0_0 - 0.198068231922397*G0_0_1 - 0.271397840842283*G0_0_2 - 0.0623693115012555*G0_1_0 - 0.00339247301052862*G0_1_1 - 0.198068231922397*G0_1_2 - 0.124738623002511*G0_2_0 - 0.198068231922397*G0_2_1 - 0.271397840842283*G0_2_2;
    A[1269] = -0.0155923278753143*G0_0_0 - 0.0250847283182354*G0_0_1 - 0.00619778723077391*G0_0_2 + 0.00329461321214811*G0_1_0 + 0.0127870136550693*G0_1_1 - 0.025084728318235*G0_1_2 - 0.0155923278753142*G0_2_0 - 0.0250847283182354*G0_2_1 - 0.006197787230774*G0_2_2;
    A[1270] = -0.0035229527417032*G0_0_0 + 0.0303691574307893*G0_0_1 + 0.104709984267276*G0_0_2 + 0.00296841388421218*G0_1_0 - 0.00678494602105698*G0_1_1 + 0.0303691574307892*G0_1_2 - 0.00352295274170326*G0_2_0 + 0.0303691574307892*G0_2_1 + 0.104709984267276*G0_2_2;
    A[1271] = 0.00117431758056765*G0_0_0 - 0.0137656116388753*G0_0_1 - 0.0528442911255405*G0_0_2 - 6.52398655872122e-05*G0_1_0 - 0.000130479731174682*G0_1_1 - 0.0137656116388756*G0_1_2 + 0.00117431758056762*G0_2_0 - 0.0137656116388753*G0_2_1 - 0.0528442911255405*G0_2_2;
    A[1272] = -0.0150051690850296*G0_0_0 + 0.0227687130898939*G0_0_1 - 0.01500516908503*G0_0_2 + 0.0227687130898934*G0_1_0 - 0.0665446628988285*G0_1_1 + 0.022768713089893*G0_1_2 - 0.0150051690850295*G0_2_0 + 0.0227687130898939*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1273] = -0.000652398655870951*G0_0_0 + 0.00884000178705049*G0_0_1 - 0.000652398655870859*G0_0_2 + 0.00884000178705051*G0_1_0 - 0.000652398655870507*G0_1_1 + 0.00884000178705013*G0_1_2 - 0.000652398655870997*G0_2_0 + 0.0088400017870505*G0_2_1 - 0.000652398655870895*G0_2_2;
    A[1274] = -0.0150051690850304*G0_0_0 - 6.52398655873127e-05*G0_0_1 - 0.0150051690850303*G0_0_2 - 6.52398655874981e-05*G0_1_0 - 6.52398655875626e-05*G0_1_2 - 0.0150051690850303*G0_2_0 - 6.52398655871484e-05*G0_2_1 - 0.0150051690850303*G0_2_2;
    A[1275] = -0.000652398655871091*G0_0_0 + 0.0088400017870502*G0_0_1 - 0.000652398655871118*G0_0_2 + 0.00884000178705045*G0_1_0 - 0.000652398655870841*G0_1_1 + 0.00884000178705074*G0_1_2 - 0.000652398655871144*G0_2_0 + 0.0088400017870502*G0_2_1 - 0.000652398655871197*G0_2_2;
    A[1276] = -0.000652398655870889*G0_0_0 - 0.00257697469069014*G0_0_1 - 0.000652398655870914*G0_0_2 - 0.00257697469068987*G0_1_0 - 0.00257697469068977*G0_1_2 - 0.000652398655870868*G0_2_0 - 0.00257697469069014*G0_2_1 - 0.000652398655870947*G0_2_2;
    A[1277] = -0.01500516908503*G0_0_0 - 6.52398655870218e-05*G0_0_1 - 0.01500516908503*G0_0_2 - 6.52398655874774e-05*G0_1_0 - 6.52398655874878e-05*G0_1_2 - 0.01500516908503*G0_2_0 - 6.52398655870355e-05*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1278] = -0.05284429112554*G0_0_0 - 0.0887914570640255*G0_0_1 - 0.0510175748891016*G0_0_2 - 0.0887914570640255*G0_1_0 + 0.0846813455320394*G0_1_1 - 0.0046320304566824*G0_1_2 - 0.0528442911255399*G0_2_0 - 0.0887914570640255*G0_2_1 - 0.0510175748891018*G0_2_2;
    A[1279] = -0.00619778723077325*G0_0_0 - 0.0250847283182347*G0_0_1 - 0.0155923278753138*G0_0_2 - 0.0250847283182349*G0_1_0 + 0.012787013655069*G0_1_1 + 0.00329461321214824*G0_1_2 - 0.0061977872307733*G0_2_0 - 0.0250847283182348*G0_2_1 - 0.0155923278753138*G0_2_2;
    A[1280] = -0.0528442911255414*G0_0_0 - 0.0137656116388758*G0_0_1 + 0.00117431758056714*G0_0_2 - 0.013765611638876*G0_1_0 - 0.000130479731174805*G0_1_1 - 6.52398655873099e-05*G0_1_2 - 0.0528442911255413*G0_2_0 - 0.0137656116388758*G0_2_1 + 0.00117431758056723*G0_2_2;
    A[1281] = 0.104709984267274*G0_0_0 + 0.188575831479475*G0_0_1 + 0.182410664181495*G0_0_2 + 0.188575831479475*G0_1_0 - 0.0622388317700817*G0_1_1 + 0.0985448169692946*G0_1_2 + 0.104709984267274*G0_2_0 + 0.188575831479475*G0_2_1 + 0.182410664181495*G0_2_2;
    A[1282] = 0.104709984267276*G0_0_0 + 0.0303691574307892*G0_0_1 - 0.00352295274170217*G0_0_2 + 0.0303691574307898*G0_1_0 - 0.00678494602105671*G0_1_1 + 0.00296841388421237*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0303691574307893*G0_2_1 - 0.00352295274170222*G0_2_2;
    A[1283] = -0.271397840842283*G0_0_0 - 0.198068231922396*G0_0_1 - 0.12473862300251*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052829*G0_1_1 - 0.0623693115012551*G0_1_2 - 0.271397840842283*G0_2_0 - 0.198068231922396*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1284] = 0.0260959462348332*G0_0_0 + 0.0104383784939321*G0_0_1 + 0.0260959462348339*G0_0_2 + 0.0104383784939329*G0_1_0 + 0.180714427676231*G0_1_1 + 0.0104383784939331*G0_1_2 + 0.0260959462348332*G0_2_0 + 0.010438378493932*G0_2_1 + 0.0260959462348341*G0_2_2;
    A[1285] = 0.0260959462348359*G0_0_0 - 0.00587158790283753*G0_0_1 + 0.0260959462348353*G0_0_2 - 0.00587158790283747*G0_1_0 + 0.00130479731174141*G0_1_1 - 0.00587158790283787*G0_1_2 + 0.026095946234836*G0_2_0 - 0.00587158790283765*G0_2_1 + 0.0260959462348353*G0_2_2;
    A[1286] = -0.185933616923197*G0_0_0 - 0.095576403085081*G0_0_1 - 0.185933616923198*G0_0_2 - 0.0955764030850823*G0_1_0 - 0.161794866655976*G0_1_1 - 0.0955764030850822*G0_1_2 - 0.185933616923197*G0_2_0 - 0.0955764030850809*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1287] = 0.0260959462348349*G0_0_0 - 0.00587158790283846*G0_0_1 + 0.0260959462348352*G0_0_2 - 0.00587158790283797*G0_1_0 + 0.00130479731174156*G0_1_1 - 0.00587158790283788*G0_1_2 + 0.0260959462348348*G0_2_0 - 0.00587158790283853*G0_2_1 + 0.0260959462348351*G0_2_2;
    A[1288] = 0.00480034930989779*G0_0_0 + 0.0048003493098978*G0_0_1 + 0.0048003493098978*G0_0_2 + 0.0153313684129655*G0_1_0 + 0.0153313684129655*G0_1_1 + 0.0153313684129655*G0_1_2 + 0.00480034930989778*G0_2_0 + 0.0048003493098978*G0_2_1 + 0.0048003493098978*G0_2_2;
    A[1289] = -0.00480034930989781*G0_0_0 - 0.0048003493098978*G0_2_0;
    A[1290] = -0.0372571824394738*G0_0_1 - 0.0551276864210887*G0_1_1 - 0.0372571824394738*G0_2_1;
    A[1291] = -0.00480034930989793*G0_0_2 - 0.00480034930989792*G0_2_2;
    A[1292] = -0.0699110399631224*G0_0_1 - 0.139822079926245*G0_0_2 + 0.0196371995417132*G0_1_1 - 0.0699110399631228*G0_1_2 - 0.0699110399631224*G0_2_1 - 0.139822079926245*G0_2_2;
    A[1293] = 0.0387263842124946*G0_0_1 + 0.108637424175617*G0_0_2 - 0.0186912214907006*G0_1_1 + 0.0387263842124949*G0_1_2 + 0.0387263842124946*G0_2_1 + 0.108637424175617*G0_2_2;
    A[1294] = -0.0189782768992835*G0_0_1 - 0.0788945694544646*G0_0_2 + 0.00939454064454053*G0_1_1 - 0.0189782768992838*G0_1_2 - 0.0189782768992834*G0_2_1 - 0.0788945694544645*G0_2_2;
    A[1295] = 0.00537576492437595*G0_0_1 + 0.0328352243499812*G0_0_2 + 0.00537576492437594*G0_1_2 + 0.00537576492437596*G0_2_1 + 0.0328352243499812*G0_2_2;
    A[1296] = 0.0053757649243759*G0_0_0 + 0.005375764924376*G0_0_2 + 0.00537576492437587*G0_2_0 + 0.00537576492437598*G0_2_2;
    A[1297] = 0.00228991928210681*G0_0_0 + 0.00228991928210659*G0_0_2 + 0.00228991928210684*G0_2_0 + 0.00228991928210661*G0_2_2;
    A[1298] = 0.00228991928210672*G0_0_0 + 0.00228991928210674*G0_0_2 + 0.00228991928210667*G0_2_0 + 0.0022899192821067*G0_2_2;
    A[1299] = 0.00537576492437598*G0_0_0 + 0.00537576492437603*G0_0_2 + 0.00537576492437599*G0_2_0 + 0.00537576492437604*G0_2_2;
    A[1300] = 0.0328352243499808*G0_0_0 + 0.00537576492437601*G0_0_1 + 0.00537576492437593*G0_1_0 + 0.0328352243499808*G0_2_0 + 0.00537576492437599*G0_2_1;
    A[1301] = -0.0788945694544645*G0_0_0 - 0.0189782768992837*G0_0_1 - 0.0189782768992837*G0_1_0 + 0.00939454064454035*G0_1_1 - 0.0788945694544644*G0_2_0 - 0.0189782768992837*G0_2_1;
    A[1302] = 0.108637424175618*G0_0_0 + 0.0387263842124948*G0_0_1 + 0.0387263842124951*G0_1_0 - 0.0186912214907003*G0_1_1 + 0.108637424175617*G0_2_0 + 0.0387263842124948*G0_2_1;
    A[1303] = -0.139822079926245*G0_0_0 - 0.0699110399631225*G0_0_1 - 0.0699110399631228*G0_1_0 + 0.0196371995417132*G0_1_1 - 0.139822079926245*G0_2_0 - 0.0699110399631225*G0_2_1;
    A[1304] = -0.00537576492437602*G0_0_0 - 0.00537576492437607*G0_0_1 + 0.0220836945012288*G0_1_0 + 0.0220836945012289*G0_1_1 - 0.00537576492437592*G0_1_2 - 0.00537576492437605*G0_2_0 - 0.00537576492437611*G0_2_1;
    A[1305] = -0.00228991928210683*G0_0_0 - 0.00228991928210676*G0_0_1 + 0.00658922642429614*G0_1_0 + 0.00658922642429612*G0_1_1 - 0.00228991928210681*G0_1_2 - 0.00228991928210683*G0_2_0 - 0.00228991928210676*G0_2_1;
    A[1306] = -0.00228991928210676*G0_0_0 - 0.00228991928210679*G0_0_1 + 0.00939454064454026*G0_1_0 + 0.00939454064454027*G0_1_1 - 0.00228991928210677*G0_1_2 - 0.00228991928210676*G0_2_0 - 0.00228991928210678*G0_2_1;
    A[1307] = -0.00537576492437598*G0_0_0 - 0.00537576492437598*G0_0_1 - 0.00537576492437602*G0_1_2 - 0.00537576492437598*G0_2_0 - 0.00537576492437599*G0_2_1;
    A[1308] = -0.0328352243499809*G0_0_0 - 0.0274594594256048*G0_0_1 - 0.0328352243499809*G0_0_2 - 0.0537511252572013*G0_1_0 - 0.081210584682806*G0_1_1 - 0.0537511252572013*G0_1_2 - 0.0328352243499809*G0_2_0 - 0.0274594594256048*G0_2_1 - 0.032835224349981*G0_2_2;
    A[1309] = 0.0788945694544646*G0_0_0 + 0.0599162925551808*G0_0_1 + 0.0788945694544646*G0_0_2 + 0.103529142700149*G0_1_0 + 0.18526817029421*G0_1_1 + 0.103529142700149*G0_1_2 + 0.0788945694544646*G0_2_0 + 0.0599162925551808*G0_2_1 + 0.0788945694544647*G0_2_2;
    A[1310] = -0.108637424175618*G0_0_0 - 0.0699110399631226*G0_0_1 - 0.108637424175618*G0_0_2 - 0.135020425819036*G0_1_0 - 0.243357746612953*G0_1_1 - 0.135020425819036*G0_1_2 - 0.108637424175618*G0_2_0 - 0.0699110399631227*G0_2_1 - 0.108637424175618*G0_2_2;
    A[1311] = 0.139822079926245*G0_0_0 + 0.0699110399631228*G0_0_1 + 0.139822079926245*G0_0_2 + 0.0699110399631228*G0_1_0 + 0.179096479009672*G0_1_1 + 0.0699110399631228*G0_1_2 + 0.139822079926245*G0_2_0 + 0.0699110399631228*G0_2_1 + 0.139822079926245*G0_2_2;
    A[1312] = -0.00537576492437607*G0_0_1 - 0.00537576492437604*G0_0_2 - 0.00537576492437599*G0_1_0 + 0.0220836945012288*G0_1_1 + 0.0220836945012287*G0_1_2 - 0.00537576492437611*G0_2_1 - 0.00537576492437607*G0_2_2;
    A[1313] = -0.00228991928210679*G0_0_1 - 0.00228991928210683*G0_0_2 - 0.00228991928210666*G0_1_0 + 0.00658922642429582*G0_1_1 + 0.0065892264242958*G0_1_2 - 0.00228991928210677*G0_2_1 - 0.00228991928210681*G0_2_2;
    A[1314] = -0.00228991928210687*G0_0_1 - 0.00228991928210688*G0_0_2 - 0.00228991928210689*G0_1_0 + 0.00939454064454031*G0_1_1 + 0.00939454064454031*G0_1_2 - 0.00228991928210691*G0_2_1 - 0.00228991928210691*G0_2_2;
    A[1315] = -0.00537576492437607*G0_0_1 - 0.00537576492437601*G0_0_2 - 0.00537576492437598*G0_1_0 - 0.00537576492437605*G0_2_1 - 0.005375764924376*G0_2_2;
    A[1316] = -0.0118736555368497*G0_0_0 + 0.00939454064454061*G0_0_1 - 0.01187365553685*G0_0_2 + 0.00939454064454043*G0_1_0 + 0.00939454064454053*G0_1_2 - 0.0118736555368497*G0_2_0 + 0.00939454064454059*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1317] = 0.0429604514890966*G0_0_0 + 0.000685018588664209*G0_0_1 + 0.0429604514890966*G0_0_2 + 0.000685018588664466*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.000685018588664183*G0_1_2 + 0.0429604514890966*G0_2_0 + 0.000685018588664196*G0_2_1 + 0.0429604514890966*G0_2_2;
    A[1318] = -0.12473862300251*G0_0_0 - 0.0284445813959699*G0_0_1 - 0.12473862300251*G0_0_2 - 0.0284445813959701*G0_1_0 - 0.0606730749959911*G0_1_1 - 0.0284445813959698*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0284445813959699*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1319] = -0.00254435475789689*G0_0_0 + 0.00329461321214757*G0_0_1 - 0.00254435475789635*G0_0_2 + 0.00329461321214802*G0_1_0 + 0.00329461321214794*G0_1_2 - 0.00254435475789691*G0_2_0 + 0.00329461321214754*G0_2_1 - 0.0025443547578963*G0_2_2;
    A[1320] = 0.0429604514890967*G0_0_0 + 0.000685018588664465*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.000685018588664078*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.000685018588664173*G0_1_2 + 0.0429604514890967*G0_2_0 + 0.000685018588664537*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[1321] = -0.0118736555368495*G0_0_0 + 0.00939454064454069*G0_0_1 - 0.01187365553685*G0_0_2 + 0.00939454064454051*G0_1_0 + 0.00939454064454059*G0_1_2 - 0.0118736555368495*G0_2_0 + 0.00939454064454064*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1322] = 0.0118736555368496*G0_0_0 + 0.0212681961813906*G0_0_1 - 0.0146137298915078*G0_1_0 - 0.0693499771190737*G0_1_1 + 0.0212681961813907*G0_1_2 + 0.0118736555368496*G0_2_0 + 0.0212681961813907*G0_2_1;
    A[1323] = -0.0429604514890966*G0_0_0 - 0.0422754329004328*G0_0_1 - 0.0359145460056915*G0_1_0 + 0.0920860702761737*G0_1_1 - 0.0422754329004326*G0_1_2 - 0.0429604514890967*G0_2_0 - 0.0422754329004329*G0_2_1;
    A[1324] = 0.12473862300251*G0_0_0 + 0.0962940416065406*G0_0_1 + 0.0284445813959702*G0_1_0 - 0.0644569872000422*G0_1_1 + 0.0962940416065412*G0_1_2 + 0.12473862300251*G0_2_0 + 0.0962940416065407*G0_2_1;
    A[1325] = 0.00254435475789675*G0_0_0 + 0.00583896797004442*G0_0_1 - 0.00590420783563156*G0_1_0 - 0.0318696743392925*G0_1_1 + 0.00583896797004419*G0_1_2 + 0.00254435475789677*G0_2_0 + 0.00583896797004441*G0_2_1;
    A[1326] = -0.0429604514890967*G0_0_0 - 0.0422754329004324*G0_0_1 - 0.000685018588664079*G0_1_0 + 0.0439716694056969*G0_1_1 - 0.0422754329004327*G0_1_2 - 0.0429604514890967*G0_2_0 - 0.0422754329004323*G0_2_1;
    A[1327] = 0.0118736555368495*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.0093945406445405*G0_1_0 - 0.0187890812890808*G0_1_1 + 0.0212681961813907*G0_1_2 + 0.0118736555368495*G0_2_0 + 0.0212681961813902*G0_2_1;
    A[1328] = -0.00939454064454092*G0_0_1 - 0.00939454064454053*G0_1_0 + 0.0453417065830254*G0_1_1 - 0.0093945406445401*G0_1_2 - 0.00939454064454095*G0_2_1;
    A[1329] = -0.00329461321214796*G0_0_1 - 0.00329461321214799*G0_1_0 + 0.0226708532915125*G0_1_1 - 0.00329461321214775*G0_1_2 - 0.00329461321214799*G0_2_1;
    A[1330] = -0.0093945406445405*G0_0_1 - 0.0093945406445404*G0_1_0 - 0.00939454064454019*G0_1_2 - 0.00939454064454049*G0_2_1;
    A[1331] = -0.00329461321214782*G0_0_1 - 0.00329461321214818*G0_1_0 + 0.0226708532915128*G0_1_1 - 0.00329461321214837*G0_1_2 - 0.00329461321214781*G0_2_1;
    A[1332] = -0.00329461321214766*G0_0_1 - 0.00329461321214777*G0_1_0 - 0.00329461321214806*G0_1_2 - 0.00329461321214763*G0_2_1;
    A[1333] = -0.00939454064454068*G0_0_1 - 0.00939454064454037*G0_1_0 - 0.00939454064454018*G0_1_2 - 0.00939454064454063*G0_2_1;
    A[1334] = 0.0212681961813904*G0_0_1 + 0.0118736555368495*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.014613729891508*G0_1_2 + 0.0212681961813905*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[1335] = 0.00583896797004436*G0_0_1 + 0.00254435475789645*G0_0_2 + 0.00583896797004424*G0_1_0 - 0.031869674339292*G0_1_1 - 0.00590420783563163*G0_1_2 + 0.0058389679700444*G0_2_1 + 0.00254435475789647*G0_2_2;
    A[1336] = 0.0212681961813907*G0_0_1 + 0.0118736555368499*G0_0_2 + 0.0212681961813909*G0_1_0 - 0.0187890812890807*G0_1_1 - 0.00939454064454048*G0_1_2 + 0.0212681961813907*G0_2_1 + 0.01187365553685*G0_2_2;
    A[1337] = -0.0422754329004321*G0_0_1 - 0.0429604514890964*G0_0_2 - 0.0422754329004321*G0_1_0 + 0.0920860702761738*G0_1_1 - 0.0359145460056913*G0_1_2 - 0.0422754329004323*G0_2_1 - 0.0429604514890964*G0_2_2;
    A[1338] = -0.0422754329004325*G0_0_1 - 0.0429604514890967*G0_0_2 - 0.042275432900433*G0_1_0 + 0.0439716694056965*G0_1_1 - 0.000685018588664251*G0_1_2 - 0.0422754329004325*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[1339] = 0.0962940416065401*G0_0_1 + 0.12473862300251*G0_0_2 + 0.096294041606541*G0_1_0 - 0.0644569872000422*G0_1_1 + 0.0284445813959699*G0_1_2 + 0.0962940416065401*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1340] = 0.00260959462348461*G0_0_1 + 0.00260959462348404*G0_1_0 - 0.151356488162042*G0_1_1 + 0.00260959462348401*G0_1_2 + 0.00260959462348467*G0_2_1;
    A[1341] = 0.00260959462348371*G0_0_1 + 0.00260959462348341*G0_1_0 - 0.0453417065830252*G0_1_1 + 0.00260959462348363*G0_1_2 + 0.00260959462348375*G0_2_1;
    A[1342] = 0.0352295274170262*G0_0_1 + 0.035229527417027*G0_1_0 + 0.166687856575008*G0_1_1 + 0.0352295274170271*G0_1_2 + 0.0352295274170262*G0_2_1;
    A[1343] = 0.00260959462348353*G0_0_1 + 0.00260959462348354*G0_1_0 - 0.0453417065830255*G0_1_1 + 0.00260959462348341*G0_1_2 + 0.00260959462348343*G0_2_1;
    A[1344] = -0.0551276864210886*G0_0_0 - 0.0551276864210887*G0_0_1 - 0.0551276864210887*G0_0_2 - 0.0178705039816148*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0178705039816149*G0_1_2 - 0.0178705039816148*G0_2_0 - 0.0178705039816148*G0_2_1 - 0.0178705039816149*G0_2_2;
    A[1345] = 0.0153313684129654*G0_0_0 + 0.0105310191030675*G0_1_0 + 0.0105310191030675*G0_2_0;
    A[1346] = -0.00480034930989779*G0_1_1 - 0.00480034930989779*G0_2_1;
    A[1347] = -0.00480034930989786*G0_1_2 - 0.00480034930989787*G0_2_2;
    A[1348] = 0.00537576492437578*G0_1_1 + 0.00537576492437607*G0_1_2 + 0.00537576492437575*G0_2_1 + 0.00537576492437604*G0_2_2;
    A[1349] = 0.00228991928210699*G0_1_1 + 0.00228991928210656*G0_1_2 + 0.002289919282107*G0_2_1 + 0.0022899192821066*G0_2_2;
    A[1350] = 0.00228991928210648*G0_1_1 + 0.00228991928210689*G0_1_2 + 0.00228991928210648*G0_2_1 + 0.00228991928210687*G0_2_2;
    A[1351] = 0.00537576492437617*G0_1_1 + 0.00537576492437597*G0_1_2 + 0.00537576492437614*G0_2_1 + 0.00537576492437596*G0_2_2;
    A[1352] = 0.022083694501229*G0_0_0 + 0.0274594594256049*G0_0_2 + 0.0274594594256048*G0_1_0 + 0.0328352243499808*G0_1_2 + 0.0274594594256049*G0_2_0 + 0.0328352243499808*G0_2_2;
    A[1353] = 0.00658922642429593*G0_0_0 + 0.00887914570640269*G0_0_2 + 0.00887914570640261*G0_1_0 + 0.0111690649885093*G0_1_2 + 0.00887914570640261*G0_2_0 + 0.0111690649885093*G0_2_2;
    A[1354] = 0.00939454064454041*G0_0_0 + 0.0116844599266472*G0_0_2 + 0.0116844599266471*G0_1_0 + 0.013974379208754*G0_1_2 + 0.0116844599266471*G0_2_0 + 0.013974379208754*G0_2_2;
    A[1355] = 0.00537576492437606*G0_0_2 + 0.00537576492437616*G0_1_0 + 0.0107515298487521*G0_1_2 + 0.00537576492437613*G0_2_0 + 0.0107515298487521*G0_2_2;
    A[1356] = 0.022083694501229*G0_0_0 + 0.0274594594256049*G0_0_1 + 0.0274594594256049*G0_1_0 + 0.0328352243499807*G0_1_1 + 0.0274594594256049*G0_2_0 + 0.0328352243499808*G0_2_1;
    A[1357] = 0.0065892264242955*G0_0_0 + 0.00887914570640243*G0_0_1 + 0.00887914570640243*G0_1_0 + 0.0111690649885093*G0_1_1 + 0.00887914570640238*G0_2_0 + 0.0111690649885092*G0_2_1;
    A[1358] = 0.00939454064454065*G0_0_0 + 0.0116844599266473*G0_0_1 + 0.0116844599266472*G0_1_0 + 0.0139743792087539*G0_1_1 + 0.0116844599266472*G0_2_0 + 0.013974379208754*G0_2_1;
    A[1359] = 0.00537576492437584*G0_0_1 + 0.00537576492437609*G0_1_0 + 0.010751529848752*G0_1_1 + 0.00537576492437606*G0_2_0 + 0.0107515298487519*G0_2_1;
    A[1360] = 0.0196371995417131*G0_0_0 + 0.0196371995417131*G0_0_1 + 0.0895482395048358*G0_0_2 + 0.0895482395048356*G0_1_0 + 0.0895482395048356*G0_1_1 + 0.0196371995417131*G0_1_2 + 0.0895482395048355*G0_2_0 + 0.0895482395048356*G0_2_1 + 0.0196371995417131*G0_2_2;
    A[1361] = -0.0186912214907004*G0_0_0 - 0.0186912214907004*G0_0_1 - 0.0574176057031955*G0_0_2 - 0.0574176057031954*G0_1_0 - 0.0574176057031954*G0_1_1 + 0.0124934342599273*G0_1_2 - 0.0574176057031953*G0_2_0 - 0.0574176057031953*G0_2_1 + 0.0124934342599274*G0_2_2;
    A[1362] = 0.00939454064454052*G0_0_0 + 0.00939454064454051*G0_0_1 + 0.0283728175438243*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.0283728175438241*G0_1_1 - 0.0315434750113569*G0_1_2 + 0.0283728175438241*G0_2_0 + 0.0283728175438241*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1363] = -0.00537576492437605*G0_0_2 - 0.00537576492437614*G0_1_0 - 0.00537576492437612*G0_1_1 + 0.0220836945012291*G0_1_2 - 0.0053757649243761*G0_2_0 - 0.00537576492437608*G0_2_1 + 0.0220836945012291*G0_2_2;
    A[1364] = 0.0196371995417131*G0_0_0 + 0.0895482395048358*G0_0_1 + 0.0196371995417131*G0_0_2 + 0.0895482395048357*G0_1_0 + 0.0196371995417132*G0_1_1 + 0.0895482395048356*G0_1_2 + 0.0895482395048357*G0_2_0 + 0.0196371995417131*G0_2_1 + 0.0895482395048356*G0_2_2;
    A[1365] = -0.0186912214907003*G0_0_0 - 0.0574176057031955*G0_0_1 - 0.0186912214907004*G0_0_2 - 0.0574176057031954*G0_1_0 + 0.0124934342599271*G0_1_1 - 0.0574176057031953*G0_1_2 - 0.0574176057031954*G0_2_0 + 0.0124934342599271*G0_2_1 - 0.0574176057031953*G0_2_2;
    A[1366] = 0.00939454064454042*G0_0_0 + 0.0283728175438243*G0_0_1 + 0.00939454064454042*G0_0_2 + 0.0283728175438243*G0_1_0 - 0.0315434750113564*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.0283728175438242*G0_2_0 - 0.0315434750113564*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1367] = -0.00537576492437599*G0_0_1 - 0.00537576492437607*G0_1_0 + 0.0220836945012288*G0_1_1 - 0.00537576492437611*G0_1_2 - 0.00537576492437604*G0_2_0 + 0.0220836945012288*G0_2_1 - 0.00537576492437607*G0_2_2;
    A[1368] = 0.179096479009672*G0_0_0 + 0.109185439046549*G0_0_1 + 0.109185439046549*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009671*G0_1_1 + 0.179096479009671*G0_1_2 + 0.109185439046549*G0_2_0 + 0.179096479009671*G0_2_1 + 0.179096479009671*G0_2_2;
    A[1369] = -0.243357746612952*G0_0_0 - 0.108337320793917*G0_0_1 - 0.108337320793917*G0_0_2 - 0.17344670664983*G0_1_0 - 0.147063705006411*G0_1_1 - 0.147063705006411*G0_1_2 - 0.17344670664983*G0_2_0 - 0.147063705006411*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1370] = 0.18526817029421*G0_0_0 + 0.0817390275940613*G0_0_1 + 0.0817390275940613*G0_0_2 + 0.125351877739029*G0_1_0 + 0.100717304493345*G0_1_1 + 0.100717304493345*G0_1_2 + 0.125351877739029*G0_2_0 + 0.100717304493345*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1371] = -0.0812105846828059*G0_0_0 - 0.0274594594256049*G0_0_1 - 0.0274594594256049*G0_0_2 - 0.0537511252572009*G0_1_0 - 0.0328352243499808*G0_1_1 - 0.0328352243499807*G0_1_2 - 0.0537511252572009*G0_2_0 - 0.0328352243499808*G0_2_1 - 0.0328352243499808*G0_2_2;
    A[1372] = 0.0453417065830251*G0_0_0 + 0.0547362472275661*G0_0_1 + 0.0547362472275658*G0_0_2 + 0.0547362472275656*G0_1_0 + 0.0641307878721063*G0_1_1 + 0.0641307878721061*G0_1_2 + 0.0547362472275656*G0_2_0 + 0.0641307878721063*G0_2_1 + 0.0641307878721061*G0_2_2;
    A[1373] = 0.0226708532915127*G0_0_0 + 0.0259654665036602*G0_0_1 + 0.0259654665036603*G0_0_2 + 0.0259654665036605*G0_1_0 + 0.0292600797158083*G0_1_1 + 0.0292600797158082*G0_1_2 + 0.0259654665036605*G0_2_0 + 0.0292600797158082*G0_2_1 + 0.0292600797158081*G0_2_2;
    A[1374] = 0.00939454064454054*G0_0_1 + 0.00939454064454036*G0_0_2 + 0.00939454064454034*G0_1_0 + 0.0187890812890809*G0_1_1 + 0.0187890812890809*G0_1_2 + 0.00939454064454031*G0_2_0 + 0.0187890812890809*G0_2_1 + 0.0187890812890809*G0_2_2;
    A[1375] = 0.022670853291513*G0_0_0 + 0.0259654665036608*G0_0_1 + 0.0259654665036609*G0_0_2 + 0.0259654665036609*G0_1_0 + 0.0292600797158086*G0_1_1 + 0.0292600797158086*G0_1_2 + 0.0259654665036609*G0_2_0 + 0.0292600797158086*G0_2_1 + 0.0292600797158086*G0_2_2;
    A[1376] = 0.00329461321214775*G0_0_1 + 0.00329461321214785*G0_0_2 + 0.00329461321214781*G0_1_0 + 0.00658922642429584*G0_1_1 + 0.0065892264242957*G0_1_2 + 0.00329461321214786*G0_2_0 + 0.00658922642429586*G0_2_1 + 0.00658922642429578*G0_2_2;
    A[1377] = 0.0093945406445406*G0_0_1 + 0.00939454064454045*G0_0_2 + 0.00939454064454054*G0_1_0 + 0.0187890812890813*G0_1_1 + 0.0187890812890812*G0_1_2 + 0.00939454064454047*G0_2_0 + 0.0187890812890812*G0_2_1 + 0.0187890812890811*G0_2_2;
    A[1378] = -0.0606730749959907*G0_0_0 - 0.0322284936000209*G0_0_1 - 0.0322284936000208*G0_0_2 - 0.032228493600021*G0_1_0 - 0.128522535206561*G0_1_1 - 0.128522535206562*G0_1_2 - 0.032228493600021*G0_2_0 - 0.128522535206561*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1379] = 0.0226708532915127*G0_0_0 + 0.0219858347028483*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0219858347028482*G0_1_0 + 0.0642612676032805*G0_1_1 + 0.0642612676032808*G0_1_2 + 0.0219858347028483*G0_2_0 + 0.0642612676032805*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[1380] = -0.00939454064454046*G0_0_1 - 0.00939454064454059*G0_0_2 - 0.00939454064454039*G0_1_0 - 0.0306627368259306*G0_1_1 - 0.0306627368259307*G0_1_2 - 0.00939454064454038*G0_2_0 - 0.0306627368259306*G0_2_1 - 0.0306627368259308*G0_2_2;
    A[1381] = 0.0226708532915127*G0_0_0 + 0.0219858347028482*G0_0_1 + 0.0219858347028481*G0_0_2 + 0.0219858347028485*G0_1_0 + 0.0642612676032808*G0_1_1 + 0.064261267603281*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0642612676032807*G0_2_1 + 0.0642612676032809*G0_2_2;
    A[1382] = -0.00329461321214804*G0_0_1 - 0.00329461321214796*G0_0_2 - 0.00329461321214777*G0_1_0 - 0.00913358118219246*G0_1_1 - 0.00913358118219225*G0_1_2 - 0.00329461321214782*G0_2_0 - 0.00913358118219252*G0_2_1 - 0.00913358118219227*G0_2_2;
    A[1383] = -0.00939454064454025*G0_0_1 - 0.00939454064454039*G0_0_2 - 0.00939454064454051*G0_1_0 - 0.0306627368259305*G0_1_1 - 0.030662736825931*G0_1_2 - 0.00939454064454044*G0_2_0 - 0.0306627368259305*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[1384] = -0.0644569872000418*G0_0_0 - 0.0929015685960117*G0_0_1 - 0.160751028806582*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000421*G0_1_1 - 0.257045070413123*G0_1_2 - 0.160751028806582*G0_2_0 - 0.0644569872000421*G0_2_1 - 0.257045070413123*G0_2_2;
    A[1385] = 0.0920860702761734*G0_0_0 + 0.128000616281865*G0_0_1 + 0.134361503176606*G0_0_2 + 0.134361503176606*G0_1_0 + 0.127315597693201*G0_1_1 + 0.176636936077038*G0_1_2 + 0.134361503176606*G0_2_0 + 0.127315597693201*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1386] = -0.0693499771190737*G0_0_0 - 0.0547362472275662*G0_0_1 - 0.0906181733004639*G0_0_2 - 0.0906181733004639*G0_1_0 - 0.0641307878721065*G0_1_1 - 0.111886369481854*G0_1_2 - 0.0906181733004639*G0_2_0 - 0.0641307878721065*G0_2_1 - 0.111886369481854*G0_2_2;
    A[1387] = 0.0439716694056963*G0_0_0 + 0.0446566879943608*G0_0_1 + 0.0862471023061289*G0_0_2 + 0.0862471023061291*G0_1_0 + 0.0439716694056968*G0_1_1 + 0.128522535206562*G0_1_2 + 0.086247102306129*G0_2_0 + 0.0439716694056968*G0_2_1 + 0.128522535206562*G0_2_2;
    A[1388] = -0.031869674339292*G0_0_0 - 0.0259654665036609*G0_0_1 - 0.0377086423093363*G0_0_2 - 0.0377086423093365*G0_1_0 - 0.0292600797158085*G0_1_1 - 0.0435476102793807*G0_1_2 - 0.0377086423093365*G0_2_0 - 0.0292600797158085*G0_2_1 - 0.0435476102793807*G0_2_2;
    A[1389] = -0.0187890812890809*G0_0_0 - 0.00939454064454052*G0_0_1 - 0.0400572774704713*G0_0_2 - 0.0400572774704711*G0_1_0 - 0.0187890812890811*G0_1_1 - 0.0613254736518619*G0_1_2 - 0.0400572774704711*G0_2_0 - 0.0187890812890811*G0_2_1 - 0.0613254736518619*G0_2_2;
    A[1390] = -0.0644569872000417*G0_0_0 - 0.160751028806582*G0_0_1 - 0.0929015685960117*G0_0_2 - 0.160751028806582*G0_1_0 - 0.257045070413123*G0_1_1 - 0.0644569872000417*G0_1_2 - 0.160751028806582*G0_2_0 - 0.257045070413123*G0_2_1 - 0.0644569872000417*G0_2_2;
    A[1391] = 0.0920860702761733*G0_0_0 + 0.134361503176606*G0_0_1 + 0.128000616281865*G0_0_2 + 0.134361503176606*G0_1_0 + 0.176636936077038*G0_1_1 + 0.1273155976932*G0_1_2 + 0.134361503176605*G0_2_0 + 0.176636936077038*G0_2_1 + 0.1273155976932*G0_2_2;
    A[1392] = -0.0693499771190736*G0_0_0 - 0.0906181733004637*G0_0_1 - 0.0547362472275659*G0_0_2 - 0.0906181733004638*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0641307878721064*G0_1_2 - 0.0906181733004638*G0_2_0 - 0.111886369481854*G0_2_1 - 0.0641307878721063*G0_2_2;
    A[1393] = 0.0439716694056965*G0_0_0 + 0.086247102306129*G0_0_1 + 0.0446566879943607*G0_0_2 + 0.086247102306129*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0439716694056965*G0_1_2 + 0.086247102306129*G0_2_0 + 0.128522535206561*G0_2_1 + 0.0439716694056965*G0_2_2;
    A[1394] = -0.0318696743392916*G0_0_0 - 0.0377086423093364*G0_0_1 - 0.0259654665036603*G0_0_2 - 0.0377086423093361*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.0292600797158081*G0_1_2 - 0.037708642309336*G0_2_0 - 0.0435476102793806*G0_2_1 - 0.029260079715808*G0_2_2;
    A[1395] = -0.0187890812890811*G0_0_0 - 0.0400572774704712*G0_0_1 - 0.00939454064454035*G0_0_2 - 0.0400572774704714*G0_1_0 - 0.0613254736518618*G0_1_1 - 0.0187890812890809*G0_1_2 - 0.0400572774704714*G0_2_0 - 0.0613254736518617*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[1396] = 0.166687856575007*G0_0_0 + 0.13145832915798*G0_0_1 + 0.13145832915798*G0_0_2 + 0.131458329157981*G0_1_0 + 0.0962288017409535*G0_1_1 + 0.0962288017409537*G0_1_2 + 0.131458329157981*G0_2_0 + 0.0962288017409535*G0_2_1 + 0.0962288017409537*G0_2_2;
    A[1397] = -0.151356488162041*G0_0_0 - 0.153966082785525*G0_0_1 - 0.153966082785526*G0_0_2 - 0.153966082785525*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.153966082785525*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1398] = -0.0453417065830253*G0_0_0 - 0.0479513012065086*G0_0_1 - 0.0479513012065085*G0_0_2 - 0.0479513012065087*G0_1_0 - 0.0505608958299922*G0_1_1 - 0.0505608958299923*G0_1_2 - 0.0479513012065087*G0_2_0 - 0.0505608958299923*G0_2_1 - 0.0505608958299923*G0_2_2;
    A[1399] = -0.0453417065830257*G0_0_0 - 0.0479513012065085*G0_0_1 - 0.0479513012065088*G0_0_2 - 0.0479513012065094*G0_1_0 - 0.0505608958299928*G0_1_1 - 0.0505608958299929*G0_1_2 - 0.0479513012065094*G0_2_0 - 0.0505608958299928*G0_2_1 - 0.0505608958299929*G0_2_2;
    A[1400] = 0.0551276864210887*G0_0_0 + 0.0551276864210887*G0_0_1 + 0.0551276864210887*G0_0_2 + 0.0214691349673986*G0_1_0 + 0.0214691349673986*G0_1_1 + 0.0214691349673986*G0_1_2 + 0.0214691349673986*G0_2_0 + 0.0214691349673986*G0_2_1 + 0.0214691349673986*G0_2_2;
    A[1401] = -0.0388177200243168*G0_0_0 - 0.0232632312710434*G0_1_0 - 0.0232632312710434*G0_2_0;
    A[1402] = -0.00336507226698194*G0_1_1 - 0.00336507226698194*G0_2_1;
    A[1403] = -0.00336507226698201*G0_1_2 - 0.00336507226698199*G0_2_2;
    A[1404] = 0.00228991928210674*G0_1_1 + 0.00228991928210673*G0_1_2 + 0.00228991928210677*G0_2_1 + 0.00228991928210677*G0_2_2;
    A[1405] = 0.00178757231708607*G0_1_1 + 0.00178757231708616*G0_1_2 + 0.00178757231708607*G0_2_1 + 0.00178757231708614*G0_2_2;
    A[1406] = 0.00178757231708635*G0_1_1 + 0.00178757231708618*G0_1_2 + 0.00178757231708629*G0_2_1 + 0.00178757231708617*G0_2_2;
    A[1407] = 0.00228991928210654*G0_1_1 + 0.00228991928210672*G0_1_2 + 0.00228991928210659*G0_2_1 + 0.00228991928210674*G0_2_2;
    A[1408] = -0.0315434750113566*G0_0_0 - 0.0599162925551805*G0_0_2 - 0.0599162925551806*G0_1_0 - 0.078894569454464*G0_1_2 - 0.0599162925551806*G0_2_0 - 0.078894569454464*G0_2_2;
    A[1409] = -0.00626302709636066*G0_0_0 - 0.00578025209101627*G0_0_2 - 0.0057802520910162*G0_1_0 - 0.00523223722008479*G0_1_2 - 0.00578025209101623*G0_2_0 - 0.00523223722008482*G0_2_2;
    A[1410] = 6.52398655873289e-05*G0_0_0 - 0.0131523569023566*G0_0_2 - 0.0131523569023565*G0_1_0 - 0.0263047138047132*G0_1_2 - 0.0131523569023565*G0_2_0 - 0.0263047138047132*G0_2_2;
    A[1411] = 0.00228991928210662*G0_0_2 + 0.00228991928210655*G0_1_0 + 0.013974379208754*G0_1_2 + 0.0022899192821066*G0_2_0 + 0.013974379208754*G0_2_2;
    A[1412] = -0.0315434750113566*G0_0_0 - 0.0599162925551805*G0_0_1 - 0.0599162925551806*G0_1_0 - 0.078894569454464*G0_1_1 - 0.0599162925551806*G0_2_0 - 0.078894569454464*G0_2_1;
    A[1413] = -0.00626302709636011*G0_0_0 - 0.0057802520910159*G0_0_1 - 0.00578025209101586*G0_1_0 - 0.00523223722008461*G0_1_1 - 0.00578025209101578*G0_2_0 - 0.00523223722008452*G0_2_1;
    A[1414] = 6.52398655870419e-05*G0_0_0 - 0.0131523569023567*G0_0_1 - 0.0131523569023567*G0_1_0 - 0.0263047138047132*G0_1_1 - 0.0131523569023568*G0_2_0 - 0.0263047138047133*G0_2_1;
    A[1415] = 0.00228991928210677*G0_0_1 + 0.00228991928210677*G0_1_0 + 0.0139743792087539*G0_1_1 + 0.00228991928210681*G0_2_0 + 0.0139743792087539*G0_2_1;
    A[1416] = 0.0124934342599274*G0_0_0 + 0.0124934342599274*G0_0_1 - 0.0574176057031954*G0_0_2 - 0.0574176057031952*G0_1_0 - 0.0574176057031952*G0_1_1 - 0.0186912214907004*G0_1_2 - 0.0574176057031951*G0_2_0 - 0.0574176057031952*G0_2_1 - 0.0186912214907004*G0_2_2;
    A[1417] = 0.00026095946234824*G0_0_0 + 0.000260959462348207*G0_0_1 + 0.00506261356955817*G0_0_2 + 0.00506261356955799*G0_1_0 + 0.005062613569558*G0_1_1 + 0.000260959462348475*G0_1_2 + 0.00506261356955794*G0_2_0 + 0.00506261356955795*G0_2_1 + 0.000260959462348448*G0_2_2;
    A[1418] = 6.52398655870076e-05*G0_0_0 + 6.52398655870281e-05*G0_0_1 - 0.000482775005344642*G0_0_2 - 0.000482775005344541*G0_1_0 - 0.000482775005344553*G0_1_1 - 0.00626302709636057*G0_1_2 - 0.00048277500534449*G0_2_0 - 0.000482775005344503*G0_2_1 - 0.00626302709636053*G0_2_2;
    A[1419] = -0.00228991928210659*G0_0_2 - 0.00228991928210655*G0_1_0 - 0.00228991928210657*G0_1_1 + 0.00658922642429598*G0_1_2 - 0.00228991928210661*G0_2_0 - 0.00228991928210662*G0_2_1 + 0.00658922642429595*G0_2_2;
    A[1420] = 0.0124934342599274*G0_0_0 - 0.0574176057031954*G0_0_1 + 0.0124934342599274*G0_0_2 - 0.0574176057031953*G0_1_0 - 0.0186912214907004*G0_1_1 - 0.0574176057031952*G0_1_2 - 0.0574176057031952*G0_2_0 - 0.0186912214907005*G0_2_1 - 0.0574176057031951*G0_2_2;
    A[1421] = 0.000260959462348346*G0_0_0 + 0.00506261356955813*G0_0_1 + 0.000260959462348373*G0_0_2 + 0.00506261356955797*G0_1_0 + 0.000260959462348483*G0_1_1 + 0.00506261356955793*G0_1_2 + 0.00506261356955796*G0_2_0 + 0.000260959462348471*G0_2_1 + 0.00506261356955792*G0_2_2;
    A[1422] = 6.5239865586987e-05*G0_0_0 - 0.000482775005344689*G0_0_1 + 6.52398655869985e-05*G0_0_2 - 0.000482775005344477*G0_1_0 - 0.00626302709636048*G0_1_1 - 0.000482775005344382*G0_1_2 - 0.000482775005344467*G0_2_0 - 0.00626302709636046*G0_2_1 - 0.000482775005344372*G0_2_2;
    A[1423] = -0.00228991928210666*G0_0_1 - 0.00228991928210679*G0_1_0 + 0.00658922642429582*G0_1_1 - 0.00228991928210677*G0_1_2 - 0.00228991928210683*G0_2_0 + 0.0065892264242958*G0_2_1 - 0.00228991928210681*G0_2_2;
    A[1424] = -0.243357746612952*G0_0_0 - 0.17344670664983*G0_0_1 - 0.17344670664983*G0_0_2 - 0.108337320793917*G0_1_0 - 0.147063705006411*G0_1_1 - 0.147063705006411*G0_1_2 - 0.108337320793917*G0_2_0 - 0.147063705006411*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1425] = 0.428965164208216*G0_0_0 + 0.226095278178609*G0_0_1 + 0.226095278178609*G0_0_2 + 0.226095278178609*G0_1_0 + 0.304226541205704*G0_1_1 + 0.304226541205704*G0_1_2 + 0.226095278178609*G0_2_0 + 0.304226541205704*G0_2_1 + 0.304226541205704*G0_2_2;
    A[1426] = -0.387185554286245*G0_0_0 - 0.167692550505048*G0_0_1 - 0.167692550505048*G0_0_2 - 0.219493003781196*G0_1_0 - 0.204187731314465*G0_1_1 - 0.204187731314465*G0_1_2 - 0.219493003781196*G0_2_0 - 0.204187731314465*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1427] = 0.18526817029421*G0_0_0 + 0.0599162925551807*G0_0_1 + 0.0599162925551806*G0_0_2 + 0.103529142700148*G0_1_0 + 0.0788945694544641*G0_1_1 + 0.0788945694544641*G0_1_2 + 0.103529142700148*G0_2_0 + 0.0788945694544641*G0_2_1 + 0.0788945694544641*G0_2_2;
    A[1428] = -0.0665446628988286*G0_0_0 - 0.0893133759887222*G0_0_1 - 0.0893133759887218*G0_0_2 - 0.0893133759887221*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.089313375988722*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1429] = -0.000652398655870825*G0_0_0 - 0.00949240044292073*G0_0_1 - 0.00949240044292091*G0_0_2 - 0.00949240044292088*G0_1_0 - 0.0189848008858422*G0_1_1 - 0.0189848008858421*G0_1_2 - 0.0094924004429209*G0_2_0 - 0.0189848008858419*G0_2_1 - 0.0189848008858421*G0_2_2;
    A[1430] = 6.52398655870477e-05*G0_0_1 + 6.52398655871188e-05*G0_0_2 + 6.52398655870803e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538557*G0_1_2 + 6.52398655870936e-05*G0_2_0 - 0.0148746893538559*G0_2_1 - 0.0148746893538557*G0_2_2;
    A[1431] = -0.000652398655871015*G0_0_0 - 0.00949240044292156*G0_0_1 - 0.00949240044292161*G0_0_2 - 0.00949240044292136*G0_1_0 - 0.0189848008858427*G0_1_1 - 0.0189848008858426*G0_1_2 - 0.00949240044292136*G0_2_0 - 0.0189848008858428*G0_2_1 - 0.0189848008858427*G0_2_2;
    A[1432] = 0.00257697469069018*G0_0_1 + 0.00257697469069006*G0_0_2 + 0.00257697469068988*G0_1_0 + 0.00450155072550893*G0_1_1 + 0.0045015507255089*G0_1_2 + 0.00257697469068984*G0_2_0 + 0.00450155072550897*G0_2_1 + 0.00450155072550893*G0_2_2;
    A[1433] = 6.52398655871042e-05*G0_0_1 + 6.52398655872797e-05*G0_0_2 + 6.52398655871688e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538557*G0_1_2 + 6.52398655872003e-05*G0_2_0 - 0.0148746893538556*G0_2_1 - 0.0148746893538557*G0_2_2;
    A[1434] = 0.0476251018785733*G0_0_0 - 0.014744209622682*G0_0_1 - 0.0147442096226821*G0_0_2 - 0.0147442096226818*G0_1_0 + 0.0476251018785733*G0_1_1 + 0.0476251018785734*G0_1_2 - 0.0147442096226818*G0_2_0 + 0.0476251018785732*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[1435] = -0.000652398655870761*G0_0_0 + 0.00231601522834152*G0_0_1 + 0.00231601522834161*G0_0_2 + 0.00231601522834154*G0_1_0 + 0.0088073818542569*G0_1_1 + 0.00880738185425667*G0_1_2 + 0.00231601522834153*G0_2_0 + 0.00880738185425687*G0_2_1 + 0.00880738185425665*G0_2_2;
    A[1436] = -6.52398655870311e-05*G0_0_1 - 6.52398655869944e-05*G0_0_2 - 6.52398655870105e-05*G0_1_0 - 0.00130479731174173*G0_1_1 - 0.00130479731174166*G0_1_2 - 6.52398655870249e-05*G0_2_0 - 0.00130479731174174*G0_2_1 - 0.00130479731174166*G0_2_2;
    A[1437] = -0.000652398655870789*G0_0_0 + 0.002316015228342*G0_0_1 + 0.00231601522834191*G0_0_2 + 0.00231601522834171*G0_1_0 + 0.00880738185425689*G0_1_1 + 0.00880738185425699*G0_1_2 + 0.00231601522834169*G0_2_0 + 0.00880738185425692*G0_2_1 + 0.00880738185425703*G0_2_2;
    A[1438] = -0.00257697469069*G0_0_1 - 0.00257697469069006*G0_0_2 - 0.00257697469068995*G0_1_0 - 0.0156575677409008*G0_1_1 - 0.015657567740901*G0_1_2 - 0.00257697469068991*G0_2_0 - 0.0156575677409008*G0_2_1 - 0.0156575677409009*G0_2_2;
    A[1439] = -6.52398655873132e-05*G0_0_1 - 6.52398655871124e-05*G0_0_2 - 6.52398655871477e-05*G0_1_0 - 0.00130479731174191*G0_1_1 - 0.00130479731174186*G0_1_2 - 6.52398655871788e-05*G0_2_0 - 0.00130479731174191*G0_2_1 - 0.00130479731174191*G0_2_2;
    A[1440] = -0.00339247301052908*G0_0_0 + 0.0589768384907264*G0_0_1 + 0.194675758911868*G0_0_2 + 0.194675758911868*G0_1_0 + 0.132306447410612*G0_1_1 + 0.121346149991982*G0_1_2 + 0.194675758911868*G0_2_0 + 0.132306447410612*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1441] = -0.0622388317700809*G0_0_0 - 0.160783648739376*G0_0_1 - 0.250814663249556*G0_0_2 - 0.250814663249556*G0_1_0 - 0.166948816037356*G0_1_1 - 0.334680510461757*G0_1_2 - 0.250814663249556*G0_2_0 - 0.166948816037356*G0_2_1 - 0.334680510461757*G0_2_2;
    A[1442] = 0.0846813455320391*G0_0_0 + 0.0893133759887224*G0_0_1 + 0.173472802596065*G0_0_2 + 0.173472802596065*G0_1_0 + 0.127087258163646*G0_1_1 + 0.20941996853455*G0_1_2 + 0.173472802596065*G0_2_0 + 0.127087258163646*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1443] = -0.00678494602105664*G0_0_0 - 0.00975335990526936*G0_0_1 - 0.0371541034518458*G0_0_2 - 0.0371541034518458*G0_1_0 - 0.0436454700777608*G0_1_1 + 0.0371867233846403*G0_1_2 - 0.0371541034518458*G0_2_0 - 0.0436454700777608*G0_2_1 + 0.0371867233846403*G0_2_2;
    A[1444] = 0.0127870136550692*G0_0_0 + 0.0094924004429216*G0_0_1 + 0.0378717419733041*G0_0_2 + 0.0378717419733041*G0_1_0 + 0.0189848008858426*G0_1_1 + 0.0567586830607657*G0_1_2 + 0.0378717419733042*G0_2_0 + 0.0189848008858427*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[1445] = -0.000130479731174406*G0_0_0 - 6.52398655871398e-05*G0_0_1 + 0.0136351319077011*G0_0_2 + 0.0136351319077009*G0_1_0 + 0.0148746893538556*G0_1_1 - 0.0254435475789643*G0_1_2 + 0.0136351319077009*G0_2_0 + 0.0148746893538555*G0_2_1 - 0.0254435475789644*G0_2_2;
    A[1446] = -0.00339247301052928*G0_0_0 + 0.194675758911868*G0_0_1 + 0.0589768384907265*G0_0_2 + 0.194675758911868*G0_1_0 + 0.121346149991982*G0_1_1 + 0.132306447410612*G0_1_2 + 0.194675758911868*G0_2_0 + 0.121346149991982*G0_2_1 + 0.132306447410612*G0_2_2;
    A[1447] = -0.0622388317700807*G0_0_0 - 0.250814663249556*G0_0_1 - 0.160783648739376*G0_0_2 - 0.250814663249556*G0_1_0 - 0.334680510461757*G0_1_1 - 0.166948816037356*G0_1_2 - 0.250814663249556*G0_2_0 - 0.334680510461757*G0_2_1 - 0.166948816037356*G0_2_2;
    A[1448] = 0.084681345532039*G0_0_0 + 0.173472802596064*G0_0_1 + 0.089313375988722*G0_0_2 + 0.173472802596064*G0_1_0 + 0.209419968534549*G0_1_1 + 0.127087258163645*G0_1_2 + 0.173472802596064*G0_2_0 + 0.209419968534549*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1449] = -0.00678494602105682*G0_0_0 - 0.0371541034518461*G0_0_1 - 0.00975335990526924*G0_0_2 - 0.0371541034518462*G0_1_0 + 0.0371867233846398*G0_1_1 - 0.0436454700777614*G0_1_2 - 0.0371541034518461*G0_2_0 + 0.0371867233846397*G0_2_1 - 0.0436454700777613*G0_2_2;
    A[1450] = 0.0127870136550686*G0_0_0 + 0.0378717419733041*G0_0_1 + 0.00949240044292088*G0_0_2 + 0.037871741973304*G0_1_0 + 0.0567586830607655*G0_1_1 + 0.0189848008858421*G0_1_2 + 0.0378717419733039*G0_2_0 + 0.0567586830607656*G0_2_1 + 0.0189848008858421*G0_2_2;
    A[1451] = -0.000130479731174023*G0_0_0 + 0.0136351319077012*G0_0_1 - 6.52398655871269e-05*G0_0_2 + 0.0136351319077012*G0_1_0 - 0.0254435475789638*G0_1_1 + 0.0148746893538558*G0_1_2 + 0.0136351319077013*G0_2_0 - 0.0254435475789639*G0_2_1 + 0.0148746893538558*G0_2_2;
    A[1452] = -0.161794866655975*G0_0_0 - 0.0662184635708925*G0_0_1 - 0.0662184635708929*G0_0_2 - 0.0662184635708932*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.0662184635708932*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1453] = 0.18071442767623*G0_0_0 + 0.170276049182297*G0_0_1 + 0.170276049182298*G0_0_2 + 0.170276049182297*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182297*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923198*G0_2_2;
    A[1454] = 0.00130479731174151*G0_0_0 + 0.00717638521457906*G0_0_1 + 0.00717638521457904*G0_0_2 + 0.00717638521457926*G0_1_0 + 0.0391439193522521*G0_1_1 + 0.0391439193522524*G0_1_2 + 0.0071763852145793*G0_2_0 + 0.0391439193522522*G0_2_1 + 0.0391439193522523*G0_2_2;
    A[1455] = 0.00130479731174177*G0_0_0 + 0.00717638521457912*G0_0_1 + 0.00717638521457916*G0_0_2 + 0.00717638521457959*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0391439193522519*G0_1_2 + 0.0071763852145796*G0_2_0 + 0.0391439193522519*G0_2_1 + 0.0391439193522519*G0_2_2;
    A[1456] = -0.0388177200243169*G0_0_0 - 0.0388177200243169*G0_0_1 - 0.0388177200243169*G0_0_2 - 0.0155544887532733*G0_1_0 - 0.0155544887532733*G0_1_1 - 0.0155544887532733*G0_1_2 - 0.0155544887532733*G0_2_0 - 0.0155544887532733*G0_2_1 - 0.0155544887532733*G0_2_2;
    A[1457] = 0.0551276864210885*G0_0_0 + 0.0336585514536899*G0_1_0 + 0.0336585514536899*G0_2_0;
    A[1458] = -0.00336507226698191*G0_1_1 - 0.0033650722669819*G0_2_1;
    A[1459] = -0.00336507226698194*G0_1_2 - 0.00336507226698195*G0_2_2;
    A[1460] = 0.00228991928210667*G0_1_1 + 0.00228991928210687*G0_1_2 + 0.00228991928210667*G0_2_1 + 0.00228991928210687*G0_2_2;
    A[1461] = 0.00178757231708628*G0_1_1 + 0.00178757231708595*G0_1_2 + 0.0017875723170863*G0_2_1 + 0.00178757231708596*G0_2_2;
    A[1462] = 0.00178757231708619*G0_1_1 + 0.00178757231708654*G0_1_2 + 0.00178757231708615*G0_2_1 + 0.00178757231708651*G0_2_2;
    A[1463] = 0.00228991928210683*G0_1_1 + 0.00228991928210668*G0_1_2 + 0.00228991928210686*G0_2_1 + 0.00228991928210672*G0_2_2;
    A[1464] = 0.0124934342599271*G0_0_0 + 0.0699110399631222*G0_0_2 + 0.0699110399631224*G0_1_0 + 0.108637424175617*G0_1_2 + 0.0699110399631224*G0_2_0 + 0.108637424175617*G0_2_2;
    A[1465] = 0.000260959462348538*G0_0_0 - 0.00480165410720919*G0_0_2 - 0.00480165410720943*G0_1_0 - 0.00960330821441885*G0_1_2 - 0.00480165410720938*G0_2_0 - 0.00960330821441876*G0_2_2;
    A[1466] = 6.52398655869738e-05*G0_0_0 + 0.00054801487093131*G0_0_2 + 0.000548014870931328*G0_1_0 - 0.00523223722008477*G0_1_2 + 0.000548014870931269*G0_2_0 - 0.00523223722008486*G0_2_2;
    A[1467] = 0.00228991928210683*G0_0_2 + 0.00228991928210685*G0_1_0 + 0.0111690649885094*G0_1_2 + 0.00228991928210688*G0_2_0 + 0.0111690649885095*G0_2_2;
    A[1468] = 0.0124934342599271*G0_0_0 + 0.0699110399631222*G0_0_1 + 0.0699110399631225*G0_1_0 + 0.108637424175617*G0_1_1 + 0.0699110399631224*G0_2_0 + 0.108637424175617*G0_2_1;
    A[1469] = 0.000260959462348321*G0_0_0 - 0.00480165410720943*G0_0_1 - 0.00480165410720953*G0_1_0 - 0.0096033082144188*G0_1_1 - 0.00480165410720957*G0_2_0 - 0.00960330821441887*G0_2_1;
    A[1470] = 6.52398655870839e-05*G0_0_0 + 0.000548014870931433*G0_0_1 + 0.000548014870931444*G0_1_0 - 0.0052322372200847*G0_1_1 + 0.000548014870931461*G0_2_0 - 0.00523223722008467*G0_2_1;
    A[1471] = 0.00228991928210683*G0_0_1 + 0.00228991928210687*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.00228991928210687*G0_2_0 + 0.0111690649885095*G0_2_1;
    A[1472] = -0.0315434750113567*G0_0_0 - 0.0315434750113568*G0_0_1 + 0.0283728175438243*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.028372817543824*G0_1_1 + 0.00939454064454056*G0_1_2 + 0.028372817543824*G0_2_0 + 0.028372817543824*G0_2_1 + 0.00939454064454056*G0_2_2;
    A[1473] = -0.00626302709636031*G0_0_0 - 0.00626302709636027*G0_0_1 - 0.000482775005344617*G0_0_2 - 0.000482775005344322*G0_1_0 - 0.000482775005344269*G0_1_1 + 6.52398655869526e-05*G0_1_2 - 0.000482775005344296*G0_2_0 - 0.000482775005344242*G0_2_1 + 6.52398655869697e-05*G0_2_2;
    A[1474] = 6.52398655871235e-05*G0_0_0 + 6.52398655870948e-05*G0_0_1 + 0.0132175967679441*G0_0_2 + 0.0132175967679438*G0_1_0 + 0.0132175967679438*G0_1_1 + 6.52398655871867e-05*G0_1_2 + 0.0132175967679438*G0_2_0 + 0.0132175967679438*G0_2_1 + 6.52398655871587e-05*G0_2_2;
    A[1475] = -0.00228991928210687*G0_0_2 - 0.00228991928210684*G0_1_0 - 0.00228991928210684*G0_1_1 + 0.00939454064454049*G0_1_2 - 0.00228991928210687*G0_2_0 - 0.00228991928210687*G0_2_1 + 0.00939454064454051*G0_2_2;
    A[1476] = -0.0315434750113567*G0_0_0 + 0.0283728175438243*G0_0_1 - 0.0315434750113567*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.00939454064454066*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.0283728175438241*G0_2_0 + 0.00939454064454065*G0_2_1 + 0.028372817543824*G0_2_2;
    A[1477] = -0.0062630270963605*G0_0_0 - 0.000482775005344668*G0_0_1 - 0.00626302709636053*G0_0_2 - 0.000482775005344458*G0_1_0 + 6.52398655867423e-05*G0_1_1 - 0.000482775005344465*G0_1_2 - 0.000482775005344466*G0_2_0 + 6.52398655867655e-05*G0_2_1 - 0.000482775005344474*G0_2_2;
    A[1478] = 6.52398655872324e-05*G0_0_0 + 0.0132175967679441*G0_0_1 + 6.52398655872193e-05*G0_0_2 + 0.0132175967679439*G0_1_0 + 6.52398655875123e-05*G0_1_1 + 0.0132175967679438*G0_1_2 + 0.0132175967679439*G0_2_0 + 6.52398655874889e-05*G0_2_1 + 0.0132175967679439*G0_2_2;
    A[1479] = -0.00228991928210689*G0_0_1 - 0.00228991928210687*G0_1_0 + 0.00939454064454031*G0_1_1 - 0.00228991928210691*G0_1_2 - 0.00228991928210688*G0_2_0 + 0.00939454064454032*G0_2_1 - 0.00228991928210691*G0_2_2;
    A[1480] = 0.18526817029421*G0_0_0 + 0.125351877739029*G0_0_1 + 0.125351877739029*G0_0_2 + 0.0817390275940613*G0_1_0 + 0.100717304493345*G0_1_1 + 0.100717304493345*G0_1_2 + 0.0817390275940613*G0_2_0 + 0.100717304493345*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1481] = -0.387185554286245*G0_0_0 - 0.219493003781196*G0_0_1 - 0.219493003781196*G0_0_2 - 0.167692550505048*G0_1_0 - 0.204187731314465*G0_1_1 - 0.204187731314465*G0_1_2 - 0.167692550505048*G0_2_0 - 0.204187731314465*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1482] = 0.428965164208215*G0_0_0 + 0.202869886029606*G0_0_1 + 0.202869886029606*G0_0_2 + 0.202869886029606*G0_1_0 + 0.281001149056702*G0_1_1 + 0.281001149056701*G0_1_2 + 0.202869886029606*G0_2_0 + 0.281001149056702*G0_2_1 + 0.281001149056701*G0_2_2;
    A[1483] = -0.243357746612952*G0_0_0 - 0.0699110399631223*G0_0_1 - 0.0699110399631223*G0_0_2 - 0.135020425819035*G0_1_0 - 0.108637424175617*G0_1_1 - 0.108637424175617*G0_1_2 - 0.135020425819035*G0_2_0 - 0.108637424175617*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1484] = 0.0476251018785737*G0_0_0 + 0.0623693115012553*G0_0_1 + 0.0623693115012551*G0_0_2 + 0.0623693115012553*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0623693115012552*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1485] = -0.00065239865587092*G0_0_0 - 0.00296841388421264*G0_0_1 - 0.00296841388421249*G0_0_2 - 0.00296841388421282*G0_1_0 + 0.00352295274170253*G0_1_1 + 0.00352295274170249*G0_1_2 - 0.00296841388421274*G0_2_0 + 0.00352295274170246*G0_2_1 + 0.00352295274170249*G0_2_2;
    A[1486] = 6.52398655871564e-05*G0_0_1 + 6.52398655871937e-05*G0_0_2 + 6.52398655870006e-05*G0_1_0 - 0.00117431758056774*G0_1_1 - 0.00117431758056754*G0_1_2 + 6.52398655870012e-05*G0_2_0 - 0.00117431758056769*G0_2_1 - 0.00117431758056754*G0_2_2;
    A[1487] = -0.000652398655870929*G0_0_0 - 0.00296841388421225*G0_0_1 - 0.00296841388421222*G0_0_2 - 0.00296841388421248*G0_1_0 + 0.00352295274170286*G0_1_1 + 0.00352295274170289*G0_1_2 - 0.00296841388421249*G0_2_0 + 0.00352295274170286*G0_2_1 + 0.00352295274170298*G0_2_2;
    A[1488] = 0.00257697469068994*G0_0_1 + 0.00257697469068988*G0_0_2 + 0.00257697469069014*G0_1_0 - 0.0105036183595209*G0_1_1 - 0.0105036183595212*G0_1_2 + 0.00257697469069014*G0_2_0 - 0.0105036183595208*G0_2_1 - 0.0105036183595211*G0_2_2;
    A[1489] = 6.5239865586939e-05*G0_0_1 + 6.52398655869821e-05*G0_0_2 + 6.5239865587128e-05*G0_1_0 - 0.00117431758056771*G0_1_1 - 0.00117431758056759*G0_1_2 + 6.5239865587135e-05*G0_2_0 - 0.00117431758056772*G0_2_1 - 0.00117431758056767*G0_2_2;
    A[1490] = -0.0665446628988288*G0_0_0 + 0.0227687130898935*G0_0_1 + 0.0227687130898936*G0_0_2 + 0.0227687130898938*G0_1_0 - 0.0150051690850296*G0_1_1 - 0.0150051690850298*G0_1_2 + 0.0227687130898937*G0_2_0 - 0.0150051690850297*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[1491] = -0.000652398655870807*G0_0_0 + 0.00884000178705047*G0_0_1 + 0.00884000178705042*G0_0_2 + 0.00884000178705013*G0_1_0 - 0.000652398655871291*G0_1_1 - 0.000652398655870894*G0_1_2 + 0.00884000178705019*G0_2_0 - 0.000652398655871185*G0_2_1 - 0.000652398655870909*G0_2_2;
    A[1492] = -6.52398655872219e-05*G0_0_1 - 6.52398655871638e-05*G0_0_2 - 6.52398655870355e-05*G0_1_0 - 0.0150051690850298*G0_1_1 - 0.0150051690850297*G0_1_2 - 6.52398655870334e-05*G0_2_0 - 0.0150051690850299*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[1493] = -0.000652398655871017*G0_0_0 + 0.00884000178705006*G0_0_1 + 0.00884000178705008*G0_0_2 + 0.00884000178705033*G0_1_0 - 0.000652398655871156*G0_1_1 - 0.000652398655871033*G0_1_2 + 0.00884000178705032*G0_2_0 - 0.000652398655871205*G0_2_1 - 0.000652398655871025*G0_2_2;
    A[1494] = -0.00257697469068985*G0_0_1 - 0.00257697469069001*G0_0_2 - 0.00257697469069008*G0_1_0 - 0.000652398655870914*G0_1_1 - 0.00065239865587118*G0_1_2 - 0.00257697469069008*G0_2_0 - 0.000652398655870906*G0_2_1 - 0.000652398655871122*G0_2_2;
    A[1495] = -6.52398655870897e-05*G0_0_1 - 6.52398655869774e-05*G0_0_2 - 6.52398655871797e-05*G0_1_0 - 0.0150051690850299*G0_1_1 - 0.0150051690850299*G0_1_2 - 6.52398655871864e-05*G0_2_0 - 0.0150051690850299*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[1496] = 0.0846813455320395*G0_0_0 - 0.004632030456683*G0_0_1 - 0.0887914570640252*G0_0_2 - 0.0887914570640256*G0_1_0 - 0.0510175748891021*G0_1_1 - 0.0528442911255403*G0_1_2 - 0.0887914570640255*G0_2_0 - 0.0510175748891021*G0_2_1 - 0.0528442911255402*G0_2_2;
    A[1497] = -0.0622388317700814*G0_0_0 + 0.0985448169692953*G0_0_1 + 0.188575831479475*G0_0_2 + 0.188575831479476*G0_1_0 + 0.182410664181496*G0_1_1 + 0.104709984267275*G0_1_2 + 0.188575831479475*G0_2_0 + 0.182410664181495*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1498] = -0.00339247301052845*G0_0_0 - 0.0623693115012556*G0_0_1 - 0.198068231922397*G0_0_2 - 0.198068231922397*G0_1_0 - 0.124738623002511*G0_1_1 - 0.271397840842283*G0_1_2 - 0.198068231922396*G0_2_0 - 0.124738623002511*G0_2_1 - 0.271397840842283*G0_2_2;
    A[1499] = 0.0127870136550689*G0_0_0 + 0.00329461321214776*G0_0_1 - 0.0250847283182353*G0_0_2 - 0.0250847283182353*G0_1_0 - 0.0155923278753139*G0_1_1 - 0.00619778723077345*G0_1_2 - 0.0250847283182353*G0_2_0 - 0.0155923278753139*G0_2_1 - 0.00619778723077344*G0_2_2;
    A[1500] = -0.00678494602105717*G0_0_0 + 0.0029684138842122*G0_0_1 + 0.0303691574307891*G0_0_2 + 0.030369157430789*G0_1_0 - 0.00352295274170282*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0303691574307889*G0_2_0 - 0.00352295274170282*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1501] = -0.000130479731174063*G0_0_0 - 6.52398655869275e-05*G0_0_1 - 0.0137656116388753*G0_0_2 - 0.0137656116388751*G0_1_0 + 0.00117431758056777*G0_1_1 - 0.0528442911255406*G0_1_2 - 0.013765611638875*G0_2_0 + 0.00117431758056779*G0_2_1 - 0.0528442911255406*G0_2_2;
    A[1502] = 0.0846813455320398*G0_0_0 - 0.0887914570640255*G0_0_1 - 0.00463203045668316*G0_0_2 - 0.0887914570640256*G0_1_0 - 0.0528442911255405*G0_1_1 - 0.0510175748891021*G0_1_2 - 0.0887914570640255*G0_2_0 - 0.0528442911255405*G0_2_1 - 0.0510175748891021*G0_2_2;
    A[1503] = -0.0622388317700815*G0_0_0 + 0.188575831479475*G0_0_1 + 0.0985448169692954*G0_0_2 + 0.188575831479476*G0_1_0 + 0.104709984267275*G0_1_1 + 0.182410664181496*G0_1_2 + 0.188575831479475*G0_2_0 + 0.104709984267275*G0_2_1 + 0.182410664181496*G0_2_2;
    A[1504] = -0.00339247301052834*G0_0_0 - 0.198068231922397*G0_0_1 - 0.0623693115012553*G0_0_2 - 0.198068231922397*G0_1_0 - 0.271397840842283*G0_1_1 - 0.124738623002511*G0_1_2 - 0.198068231922396*G0_2_0 - 0.271397840842283*G0_2_1 - 0.124738623002511*G0_2_2;
    A[1505] = 0.0127870136550691*G0_0_0 - 0.025084728318235*G0_0_1 + 0.00329461321214793*G0_0_2 - 0.0250847283182349*G0_1_0 - 0.00619778723077344*G0_1_1 - 0.0155923278753137*G0_1_2 - 0.0250847283182349*G0_2_0 - 0.00619778723077334*G0_2_1 - 0.0155923278753137*G0_2_2;
    A[1506] = -0.00678494602105695*G0_0_0 + 0.0303691574307889*G0_0_1 + 0.00296841388421249*G0_0_2 + 0.0303691574307889*G0_1_0 + 0.104709984267275*G0_1_1 - 0.00352295274170258*G0_1_2 + 0.030369157430789*G0_2_0 + 0.104709984267275*G0_2_1 - 0.00352295274170258*G0_2_2;
    A[1507] = -0.00013047973117433*G0_0_0 - 0.0137656116388755*G0_0_1 - 6.52398655871891e-05*G0_0_2 - 0.0137656116388753*G0_1_0 - 0.0528442911255407*G0_1_1 + 0.00117431758056757*G0_1_2 - 0.0137656116388754*G0_2_0 - 0.0528442911255407*G0_2_1 + 0.00117431758056757*G0_2_2;
    A[1508] = 0.180714427676231*G0_0_0 + 0.0104383784939335*G0_0_1 + 0.0104383784939336*G0_0_2 + 0.0104383784939329*G0_1_0 + 0.0260959462348345*G0_1_1 + 0.0260959462348345*G0_1_2 + 0.010438378493933*G0_2_0 + 0.0260959462348344*G0_2_1 + 0.0260959462348345*G0_2_2;
    A[1509] = -0.161794866655976*G0_0_0 - 0.0955764030850823*G0_0_1 - 0.0955764030850827*G0_0_2 - 0.095576403085082*G0_1_0 - 0.185933616923198*G0_1_1 - 0.185933616923198*G0_1_2 - 0.0955764030850819*G0_2_0 - 0.185933616923198*G0_2_1 - 0.185933616923198*G0_2_2;
    A[1510] = 0.00130479731174182*G0_0_0 - 0.00587158790283768*G0_0_1 - 0.0058715879028377*G0_0_2 - 0.00587158790283729*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.0260959462348351*G0_1_2 - 0.00587158790283742*G0_2_0 + 0.0260959462348354*G0_2_1 + 0.026095946234835*G0_2_2;
    A[1511] = 0.00130479731174197*G0_0_0 - 0.00587158790283764*G0_0_1 - 0.00587158790283756*G0_0_2 - 0.00587158790283782*G0_1_0 + 0.0260959462348348*G0_1_1 + 0.0260959462348351*G0_1_2 - 0.00587158790283779*G0_2_0 + 0.0260959462348348*G0_2_1 + 0.0260959462348352*G0_2_2;
    A[1512] = 0.0153313684129655*G0_0_0 + 0.0153313684129655*G0_0_1 + 0.0153313684129655*G0_0_2 + 0.0048003493098979*G0_1_0 + 0.00480034930989792*G0_1_1 + 0.00480034930989795*G0_1_2 + 0.0048003493098979*G0_2_0 + 0.00480034930989792*G0_2_1 + 0.00480034930989795*G0_2_2;
    A[1513] = -0.0551276864210886*G0_0_0 - 0.0372571824394737*G0_1_0 - 0.0372571824394737*G0_2_0;
    A[1514] = -0.00480034930989783*G0_1_1 - 0.00480034930989783*G0_2_1;
    A[1515] = -0.00480034930989797*G0_1_2 - 0.00480034930989798*G0_2_2;
    A[1516] = 0.00537576492437596*G0_1_1 + 0.00537576492437602*G0_1_2 + 0.0053757649243759*G0_2_1 + 0.00537576492437596*G0_2_2;
    A[1517] = 0.00228991928210653*G0_1_1 + 0.00228991928210665*G0_1_2 + 0.00228991928210657*G0_2_1 + 0.00228991928210669*G0_2_2;
    A[1518] = 0.00228991928210684*G0_1_1 + 0.00228991928210664*G0_1_2 + 0.00228991928210681*G0_2_1 + 0.00228991928210659*G0_2_2;
    A[1519] = 0.00537576492437584*G0_1_1 + 0.00537576492437603*G0_1_2 + 0.00537576492437582*G0_2_1 + 0.00537576492437603*G0_2_2;
    A[1520] = 0.0196371995417135*G0_0_0 - 0.0699110399631223*G0_0_2 - 0.0699110399631221*G0_1_0 - 0.139822079926245*G0_1_2 - 0.069911039963122*G0_2_0 - 0.139822079926245*G0_2_2;
    A[1521] = -0.0186912214907007*G0_0_0 + 0.0387263842124948*G0_0_2 + 0.0387263842124943*G0_1_0 + 0.108637424175617*G0_1_2 + 0.0387263842124943*G0_2_0 + 0.108637424175617*G0_2_2;
    A[1522] = 0.00939454064454067*G0_0_0 - 0.0189782768992835*G0_0_2 - 0.0189782768992833*G0_1_0 - 0.0788945694544644*G0_1_2 - 0.0189782768992832*G0_2_0 - 0.0788945694544643*G0_2_2;
    A[1523] = 0.00537576492437604*G0_0_2 + 0.00537576492437584*G0_1_0 + 0.032835224349981*G0_1_2 + 0.00537576492437582*G0_2_0 + 0.032835224349981*G0_2_2;
    A[1524] = 0.0196371995417134*G0_0_0 - 0.0699110399631224*G0_0_1 - 0.0699110399631224*G0_1_0 - 0.139822079926245*G0_1_1 - 0.0699110399631223*G0_2_0 - 0.139822079926245*G0_2_1;
    A[1525] = -0.0186912214907006*G0_0_0 + 0.0387263842124947*G0_0_1 + 0.0387263842124949*G0_1_0 + 0.108637424175617*G0_1_1 + 0.0387263842124949*G0_2_0 + 0.108637424175617*G0_2_1;
    A[1526] = 0.00939454064454058*G0_0_0 - 0.0189782768992835*G0_0_1 - 0.0189782768992837*G0_1_0 - 0.078894569454464*G0_1_1 - 0.0189782768992836*G0_2_0 - 0.0788945694544639*G0_2_1;
    A[1527] = 0.00537576492437596*G0_0_1 + 0.00537576492437607*G0_1_0 + 0.0328352243499807*G0_1_1 + 0.00537576492437602*G0_2_0 + 0.0328352243499806*G0_2_1;
    A[1528] = 0.0220836945012291*G0_0_0 + 0.0220836945012291*G0_0_1 - 0.00537576492437595*G0_0_2 - 0.00537576492437593*G0_1_0 - 0.0053757649243759*G0_1_1 - 0.00537576492437593*G0_2_0 - 0.00537576492437589*G0_2_1;
    A[1529] = 0.00658922642429581*G0_0_0 + 0.00658922642429579*G0_0_1 - 0.00228991928210671*G0_0_2 - 0.0022899192821065*G0_1_0 - 0.00228991928210653*G0_1_1 - 0.0022899192821065*G0_2_0 - 0.00228991928210652*G0_2_1;
    A[1530] = 0.0093945406445407*G0_0_0 + 0.00939454064454073*G0_0_1 - 0.00228991928210682*G0_0_2 - 0.00228991928210683*G0_1_0 - 0.00228991928210685*G0_1_1 - 0.00228991928210683*G0_2_0 - 0.00228991928210685*G0_2_1;
    A[1531] = -0.00537576492437596*G0_0_2 - 0.00537576492437586*G0_1_0 - 0.00537576492437585*G0_1_1 - 0.00537576492437584*G0_2_0 - 0.00537576492437583*G0_2_1;
    A[1532] = 0.0220836945012291*G0_0_0 - 0.00537576492437596*G0_0_1 + 0.0220836945012291*G0_0_2 - 0.00537576492437592*G0_1_0 - 0.00537576492437585*G0_1_2 - 0.00537576492437592*G0_2_0 - 0.00537576492437585*G0_2_2;
    A[1533] = 0.00658922642429579*G0_0_0 - 0.00228991928210673*G0_0_1 + 0.00658922642429578*G0_0_2 - 0.0022899192821069*G0_1_0 - 0.00228991928210691*G0_1_2 - 0.00228991928210685*G0_2_0 - 0.00228991928210686*G0_2_2;
    A[1534] = 0.00939454064454071*G0_0_0 - 0.00228991928210681*G0_0_1 + 0.00939454064454072*G0_0_2 - 0.00228991928210662*G0_1_0 - 0.00228991928210653*G0_1_2 - 0.00228991928210664*G0_2_0 - 0.00228991928210655*G0_2_2;
    A[1535] = -0.00537576492437599*G0_0_1 - 0.00537576492437607*G0_1_0 - 0.00537576492437605*G0_1_2 - 0.00537576492437601*G0_2_0 - 0.005375764924376*G0_2_2;
    A[1536] = -0.0812105846828059*G0_0_0 - 0.0537511252572009*G0_0_1 - 0.0537511252572009*G0_0_2 - 0.027459459425605*G0_1_0 - 0.0328352243499808*G0_1_1 - 0.0328352243499808*G0_1_2 - 0.0274594594256049*G0_2_0 - 0.0328352243499807*G0_2_1 - 0.0328352243499808*G0_2_2;
    A[1537] = 0.18526817029421*G0_0_0 + 0.103529142700148*G0_0_1 + 0.103529142700148*G0_0_2 + 0.0599162925551807*G0_1_0 + 0.0788945694544641*G0_1_1 + 0.0788945694544641*G0_1_2 + 0.0599162925551806*G0_2_0 + 0.0788945694544641*G0_2_1 + 0.0788945694544641*G0_2_2;
    A[1538] = -0.243357746612952*G0_0_0 - 0.135020425819035*G0_0_1 - 0.135020425819035*G0_0_2 - 0.0699110399631223*G0_1_0 - 0.108637424175617*G0_1_1 - 0.108637424175617*G0_1_2 - 0.0699110399631223*G0_2_0 - 0.108637424175617*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1539] = 0.179096479009672*G0_0_0 + 0.0699110399631224*G0_0_1 + 0.0699110399631224*G0_0_2 + 0.0699110399631224*G0_1_0 + 0.139822079926245*G0_1_1 + 0.139822079926245*G0_1_2 + 0.0699110399631224*G0_2_0 + 0.139822079926245*G0_2_1 + 0.139822079926245*G0_2_2;
    A[1540] = -0.0606730749959915*G0_0_0 - 0.0284445813959698*G0_0_1 - 0.0284445813959699*G0_0_2 - 0.0284445813959705*G0_1_0 - 0.12473862300251*G0_1_1 - 0.12473862300251*G0_1_2 - 0.0284445813959705*G0_2_0 - 0.12473862300251*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1541] = 0.022670853291513*G0_0_0 + 0.000685018588664287*G0_0_1 + 0.000685018588664379*G0_0_2 + 0.000685018588664881*G0_1_0 + 0.0429604514890966*G0_1_1 + 0.042960451489097*G0_1_2 + 0.00068501858866483*G0_2_0 + 0.0429604514890965*G0_2_1 + 0.042960451489097*G0_2_2;
    A[1542] = 0.00939454064454056*G0_0_1 + 0.0093945406445406*G0_0_2 + 0.00939454064454035*G0_1_0 - 0.0118736555368497*G0_1_1 - 0.0118736555368499*G0_1_2 + 0.00939454064454028*G0_2_0 - 0.0118736555368497*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1543] = 0.022670853291513*G0_0_0 + 0.000685018588664298*G0_0_1 + 0.000685018588664287*G0_0_2 + 0.000685018588664421*G0_1_0 + 0.042960451489096*G0_1_1 + 0.0429604514890962*G0_1_2 + 0.000685018588664476*G0_2_0 + 0.0429604514890961*G0_2_1 + 0.0429604514890962*G0_2_2;
    A[1544] = 0.00329461321214783*G0_0_1 + 0.00329461321214784*G0_0_2 + 0.00329461321214768*G0_1_0 - 0.00254435475789629*G0_1_1 - 0.00254435475789634*G0_1_2 + 0.00329461321214768*G0_2_0 - 0.00254435475789637*G0_2_1 - 0.00254435475789636*G0_2_2;
    A[1545] = 0.00939454064454091*G0_0_1 + 0.00939454064454074*G0_0_2 + 0.00939454064454084*G0_1_0 - 0.0118736555368493*G0_1_1 - 0.0118736555368495*G0_1_2 + 0.00939454064454079*G0_2_0 - 0.0118736555368492*G0_2_1 - 0.0118736555368495*G0_2_2;
    A[1546] = 0.0453417065830257*G0_0_0 - 0.00939454064454055*G0_0_1 - 0.0093945406445405*G0_0_2 - 0.00939454064454057*G0_1_0 - 0.00939454064454049*G0_2_0;
    A[1547] = 0.0226708532915126*G0_0_0 - 0.00329461321214785*G0_0_1 - 0.00329461321214807*G0_0_2 - 0.00329461321214816*G0_1_0 - 0.00329461321214822*G0_2_0;
    A[1548] = -0.00939454064454069*G0_0_1 - 0.00939454064454061*G0_0_2 - 0.00939454064454034*G0_1_0 - 0.00939454064454028*G0_2_0;
    A[1549] = 0.022670853291513*G0_0_0 - 0.00329461321214799*G0_0_1 - 0.00329461321214781*G0_0_2 - 0.00329461321214798*G0_1_0 - 0.00329461321214801*G0_2_0;
    A[1550] = -0.00329461321214768*G0_0_1 - 0.00329461321214758*G0_0_2 - 0.0032946132121477*G0_1_0 - 0.0032946132121477*G0_2_0;
    A[1551] = -0.00939454064454064*G0_0_1 - 0.00939454064454086*G0_0_2 - 0.00939454064454082*G0_1_0 - 0.00939454064454077*G0_2_0;
    A[1552] = -0.0693499771190739*G0_0_0 - 0.0146137298915074*G0_0_1 + 0.0212681961813902*G0_0_2 + 0.0212681961813905*G0_1_0 + 0.0118736555368504*G0_1_1 + 0.0212681961813904*G0_2_0 + 0.0118736555368504*G0_2_1;
    A[1553] = 0.0920860702761738*G0_0_0 - 0.0359145460056915*G0_0_1 - 0.0422754329004323*G0_0_2 - 0.0422754329004327*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.0422754329004326*G0_2_0 - 0.0429604514890973*G0_2_1;
    A[1554] = -0.0644569872000426*G0_0_0 + 0.0284445813959701*G0_0_1 + 0.0962940416065405*G0_0_2 + 0.0962940416065403*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0962940416065401*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1555] = -0.0318696743392921*G0_0_0 - 0.00590420783563113*G0_0_1 + 0.00583896797004442*G0_0_2 + 0.00583896797004413*G0_1_0 + 0.00254435475789674*G0_1_1 + 0.00583896797004413*G0_2_0 + 0.00254435475789673*G0_2_1;
    A[1556] = 0.0439716694056971*G0_0_0 - 0.000685018588664278*G0_0_1 - 0.0422754329004325*G0_0_2 - 0.0422754329004319*G0_1_0 - 0.0429604514890962*G0_1_1 - 0.0422754329004319*G0_2_0 - 0.0429604514890963*G0_2_1;
    A[1557] = -0.0187890812890814*G0_0_0 - 0.00939454064454091*G0_0_1 + 0.0212681961813901*G0_0_2 + 0.0212681961813901*G0_1_0 + 0.0118736555368493*G0_1_1 + 0.02126819618139*G0_2_0 + 0.0118736555368492*G0_2_1;
    A[1558] = -0.0693499771190741*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.0146137298915075*G0_0_2 + 0.0212681961813904*G0_1_0 + 0.0118736555368501*G0_1_2 + 0.0212681961813904*G0_2_0 + 0.0118736555368501*G0_2_2;
    A[1559] = 0.092086070276174*G0_0_0 - 0.0422754329004326*G0_0_1 - 0.0359145460056917*G0_0_2 - 0.0422754329004326*G0_1_0 - 0.0429604514890974*G0_1_2 - 0.0422754329004326*G0_2_0 - 0.0429604514890974*G0_2_2;
    A[1560] = -0.0644569872000424*G0_0_0 + 0.0962940416065406*G0_0_1 + 0.0284445813959702*G0_0_2 + 0.0962940416065405*G0_1_0 + 0.124738623002511*G0_1_2 + 0.0962940416065404*G0_2_0 + 0.124738623002511*G0_2_2;
    A[1561] = -0.0318696743392922*G0_0_0 + 0.00583896797004444*G0_0_1 - 0.00590420783563147*G0_0_2 + 0.00583896797004436*G0_1_0 + 0.00254435475789624*G0_1_2 + 0.00583896797004432*G0_2_0 + 0.0025443547578962*G0_2_2;
    A[1562] = 0.043971669405697*G0_0_0 - 0.0422754329004324*G0_0_1 - 0.000685018588664342*G0_0_2 - 0.0422754329004324*G0_1_0 - 0.042960451489097*G0_1_2 - 0.0422754329004323*G0_2_0 - 0.042960451489097*G0_2_2;
    A[1563] = -0.0187890812890812*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.00939454064454062*G0_0_2 + 0.0212681961813903*G0_1_0 + 0.0118736555368499*G0_1_2 + 0.0212681961813903*G0_2_0 + 0.0118736555368499*G0_2_2;
    A[1564] = -0.151356488162042*G0_0_0 + 0.0026095946234832*G0_0_1 + 0.00260959462348352*G0_0_2 + 0.00260959462348379*G0_1_0 + 0.0026095946234837*G0_2_0;
    A[1565] = 0.166687856575008*G0_0_0 + 0.0352295274170269*G0_0_1 + 0.0352295274170271*G0_0_2 + 0.0352295274170272*G0_1_0 + 0.0352295274170272*G0_2_0;
    A[1566] = -0.0453417065830257*G0_0_0 + 0.00260959462348381*G0_0_1 + 0.00260959462348354*G0_0_2 + 0.00260959462348329*G0_1_0 + 0.0026095946234834*G0_2_0;
    A[1567] = -0.0453417065830261*G0_0_0 + 0.00260959462348326*G0_0_1 + 0.00260959462348335*G0_0_2 + 0.00260959462348357*G0_1_0 + 0.00260959462348355*G0_2_0;
    A[1568] = -0.0137421252872639*G0_0_0 - 0.0137421252872639*G0_0_1 - 0.0137421252872639*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.0137421252872641*G0_1_2 - 0.013742125287264*G0_2_0 - 0.013742125287264*G0_2_1 - 0.013742125287264*G0_2_2;
    A[1569] = 0.047993054720485*G0_0_0 + 0.0143945239431347*G0_1_0 + 0.0143945239431349*G0_2_0;
    A[1570] = -0.00517743573299126*G0_0_1 + 0.013742125287264*G0_1_1 - 0.0051774357329913*G0_2_1;
    A[1571] = -0.00517743573299137*G0_0_2 - 0.00517743573299121*G0_1_2 + 0.0137421252872643*G0_2_2;
    A[1572] = 0.0425363923627808*G0_0_1 + 0.0118736555368496*G0_0_2 - 0.0187890812890812*G0_1_1 - 0.00939454064454054*G0_1_2 + 0.0212681961813905*G0_2_1 + 0.01187365553685*G0_2_2;
    A[1573] = 0.0403182369328198*G0_0_1 + 0.0540186087061085*G0_0_2 + 0.0148746893538562*G0_1_1 + 0.0149399292194431*G0_1_2 + 0.00123955744615479*G0_2_1 + 0.00117431758056753*G0_2_2;
    A[1574] = 0.0540186087061087*G0_0_1 + 0.0403182369328201*G0_0_2 + 0.00117431758056746*G0_1_1 + 0.00123955744615476*G0_1_2 + 0.0149399292194428*G0_2_1 + 0.0148746893538562*G0_2_2;
    A[1575] = 0.0118736555368493*G0_0_1 + 0.0425363923627805*G0_0_2 + 0.0118736555368503*G0_1_1 + 0.0212681961813906*G0_1_2 - 0.00939454064454026*G0_2_1 - 0.0187890812890813*G0_2_2;
    A[1576] = -0.064456987200042*G0_0_0 + 0.0284445813959705*G0_0_2 + 0.192588083213081*G0_1_0 + 0.12473862300251*G0_1_2 + 0.09629404160654*G0_2_0 + 0.12473862300251*G0_2_2;
    A[1577] = 0.132306447410612*G0_0_0 + 0.0733296089198852*G0_0_2 + 0.0109602974186311*G0_1_0 + 0.146659217839773*G0_1_2 - 0.0623693115012545*G0_2_0 - 0.12473862300251*G0_2_2;
    A[1578] = -0.0510175748891015*G0_0_0 - 0.0463855444324184*G0_0_2 + 0.00182671623643824*G0_1_0 - 0.0823327103709048*G0_1_2 + 0.0377738821749229*G0_2_0 + 0.127087258163646*G0_2_2;
    A[1579] = 0.0118736555368492*G0_0_0 + 0.0264873854283569*G0_0_2 + 0.0118736555368502*G0_1_0 + 0.0477555816097483*G0_1_2 - 0.00939454064454027*G0_2_0 - 0.0641307878721069*G0_2_2;
    A[1580] = -0.0644569872000421*G0_0_0 + 0.0284445813959704*G0_0_1 + 0.0962940416065403*G0_1_0 + 0.12473862300251*G0_1_1 + 0.192588083213081*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1581] = 0.132306447410613*G0_0_0 + 0.0733296089198858*G0_0_1 - 0.0623693115012553*G0_1_0 - 0.12473862300251*G0_1_1 + 0.0109602974186309*G0_2_0 + 0.146659217839773*G0_2_1;
    A[1582] = -0.0510175748891022*G0_0_0 - 0.0463855444324186*G0_0_1 + 0.0377738821749235*G0_1_0 + 0.127087258163646*G0_1_1 + 0.0018267162364384*G0_2_0 - 0.0823327103709047*G0_2_1;
    A[1583] = 0.0118736555368497*G0_0_0 + 0.026487385428357*G0_0_1 - 0.00939454064454059*G0_1_0 - 0.0641307878721065*G0_1_1 + 0.0118736555368499*G0_2_0 + 0.0477555816097483*G0_2_1;
    A[1584] = 0.0187890812890807*G0_0_0 + 0.0187890812890806*G0_0_1 + 0.00939454064454018*G0_0_2 + 0.0187890812890808*G0_1_0 + 0.0187890812890808*G0_1_1 + 0.0093945406445406*G0_1_2 + 0.00939454064454034*G0_2_0 + 0.0093945406445403*G0_2_1;
    A[1585] = -0.0148746893538554*G0_0_0 - 0.0148746893538555*G0_0_1 + 6.52398655874749e-05*G0_0_2 - 0.0148746893538559*G0_1_0 - 0.0148746893538559*G0_1_1 + 6.5239865586866e-05*G0_1_2 + 6.52398655870074e-05*G0_2_0 + 6.52398655869971e-05*G0_2_1;
    A[1586] = -0.00117431758056813*G0_0_0 - 0.00117431758056818*G0_0_1 + 6.52398655868843e-05*G0_0_2 - 0.00117431758056742*G0_1_0 - 0.00117431758056742*G0_1_1 + 6.52398655874344e-05*G0_1_2 + 6.52398655872846e-05*G0_2_0 + 6.52398655873524e-05*G0_2_1;
    A[1587] = -0.0118736555368493*G0_0_0 - 0.0118736555368493*G0_0_1 + 0.00939454064454073*G0_0_2 - 0.0118736555368502*G0_1_0 - 0.0118736555368502*G0_1_1 + 0.00939454064454028*G0_1_2 + 0.00939454064454028*G0_2_0 + 0.00939454064454028*G0_2_1;
    A[1588] = 0.0187890812890808*G0_0_0 + 0.00939454064454042*G0_0_1 + 0.0187890812890807*G0_0_2 + 0.00939454064454041*G0_1_0 + 0.00939454064454028*G0_1_2 + 0.0187890812890808*G0_2_0 + 0.00939454064454037*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1589] = -0.0148746893538558*G0_0_0 + 6.52398655871214e-05*G0_0_1 - 0.0148746893538556*G0_0_2 + 6.52398655871438e-05*G0_1_0 + 6.52398655872704e-05*G0_1_2 - 0.014874689353856*G0_2_0 + 6.52398655870539e-05*G0_2_1 - 0.0148746893538559*G0_2_2;
    A[1590] = -0.00117431758056779*G0_0_0 + 6.52398655871205e-05*G0_0_1 - 0.00117431758056782*G0_0_2 + 6.5239865587218e-05*G0_1_0 + 6.52398655870566e-05*G0_1_2 - 0.00117431758056736*G0_2_0 + 6.52398655871964e-05*G0_2_1 - 0.00117431758056744*G0_2_2;
    A[1591] = -0.0118736555368497*G0_0_0 + 0.00939454064454043*G0_0_1 - 0.0118736555368497*G0_0_2 + 0.00939454064454061*G0_1_0 + 0.00939454064454059*G0_1_2 - 0.0118736555368499*G0_2_0 + 0.00939454064454053*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1592] = 0.0453417065830251*G0_0_0 + 0.0547362472275656*G0_0_1 + 0.0547362472275656*G0_0_2 + 0.0547362472275661*G0_1_0 + 0.0641307878721063*G0_1_1 + 0.0641307878721063*G0_1_2 + 0.0547362472275658*G0_2_0 + 0.0641307878721061*G0_2_1 + 0.0641307878721061*G0_2_2;
    A[1593] = -0.0665446628988286*G0_0_0 - 0.0893133759887221*G0_0_1 - 0.089313375988722*G0_0_2 - 0.0893133759887222*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.0893133759887218*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1594] = 0.0476251018785737*G0_0_0 + 0.0623693115012553*G0_0_1 + 0.0623693115012552*G0_0_2 + 0.0623693115012553*G0_1_0 + 0.12473862300251*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0623693115012551*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1595] = -0.0606730749959915*G0_0_0 - 0.0284445813959705*G0_0_1 - 0.0284445813959705*G0_0_2 - 0.0284445813959698*G0_1_0 - 0.12473862300251*G0_1_1 - 0.12473862300251*G0_1_2 - 0.02844458139597*G0_2_0 - 0.12473862300251*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1596] = 0.647179466623905*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684565*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684565*G0_2_2;
    A[1597] = -0.268135847562929*G0_0_0 + 0.180714427676231*G0_0_1 + 0.0225077536275451*G0_0_2 - 0.0737210481134093*G0_1_0 - 0.209419968534551*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0459941052388973*G0_2_0 + 0.0812236326559239*G0_2_1 - 0.0234863516113513*G0_2_2;
    A[1598] = 0.105688582251081*G0_0_0 - 0.0743734467692796*G0_0_1 + 0.000652398655870263*G0_0_2 + 0.0300103381700601*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255404*G0_1_2 + 0.0528442911255407*G0_2_0 + 0.000652398655870209*G0_2_1 + 0.0534966897814114*G0_2_2;
    A[1599] = -0.268135847562928*G0_0_0 + 0.0225077536275447*G0_0_1 + 0.180714427676231*G0_0_2 - 0.0459941052388966*G0_1_0 - 0.0234863516113506*G0_1_1 + 0.0812236326559239*G0_1_2 - 0.073721048113408*G0_2_0 - 0.104709984267274*G0_2_1 - 0.209419968534549*G0_2_2;
    A[1600] = 0.113517366121532*G0_0_0 - 0.0851380245911486*G0_0_1 - 0.0851380245911487*G0_0_2 + 0.0127217737894821*G0_1_0 - 0.0137003717732887*G0_1_1 - 0.0198981590040618*G0_1_2 + 0.0127217737894819*G0_2_0 - 0.0198981590040618*G0_2_1 - 0.0137003717732891*G0_2_2;
    A[1601] = 0.105688582251081*G0_0_0 + 0.000652398655870793*G0_0_1 - 0.0743734467692791*G0_0_2 + 0.0528442911255409*G0_1_0 + 0.0534966897814119*G0_1_1 + 0.0006523986558709*G0_1_2 + 0.0300103381700598*G0_2_0 + 0.05284429112554*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1602] = -0.0600206763401203*G0_0_0 - 0.0300103381700596*G0_0_1 - 0.0300103381700597*G0_0_2 - 0.03001033817006*G0_1_0 - 0.0150051690850299*G0_1_2 - 0.0300103381700604*G0_2_0 - 0.0150051690850296*G0_2_1;
    A[1603] = -0.0254435475789637*G0_0_0 - 0.00130479731174211*G0_0_1 - 0.012721773789482*G0_0_2 - 0.00130479731174144*G0_1_0 - 0.000652398655871149*G0_1_2 - 0.0127217737894818*G0_2_0 - 0.000652398655870803*G0_2_1;
    A[1604] = -0.105688582251081*G0_0_0 - 0.0300103381700597*G0_0_1 - 0.0528442911255405*G0_0_2 - 0.0300103381700601*G0_1_0 - 0.0150051690850303*G0_1_2 - 0.0528442911255406*G0_2_0 - 0.0150051690850303*G0_2_1;
    A[1605] = -0.0254435475789646*G0_0_0 - 0.0127217737894822*G0_0_1 - 0.00130479731174235*G0_0_2 - 0.0127217737894822*G0_1_0 - 0.000652398655871056*G0_1_2 - 0.00130479731174185*G0_2_0 - 0.000652398655870919*G0_2_1;
    A[1606] = -0.113517366121532*G0_0_0 - 0.012721773789482*G0_0_1 - 0.0127217737894822*G0_0_2 - 0.0127217737894821*G0_1_0 - 0.000652398655870692*G0_1_2 - 0.0127217737894819*G0_2_0 - 0.000652398655870921*G0_2_1;
    A[1607] = -0.105688582251081*G0_0_0 - 0.0528442911255403*G0_0_1 - 0.0300103381700593*G0_0_2 - 0.0528442911255408*G0_1_0 - 0.01500516908503*G0_1_2 - 0.0300103381700597*G0_2_0 - 0.01500516908503*G0_2_1;
    A[1608] = -0.0456679059109597*G0_0_0 - 0.0756782440810203*G0_0_1 - 0.0378391220405093*G0_0_2 - 0.0756782440810205*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0528442911255401*G0_1_2 - 0.0378391220405099*G0_2_0 - 0.0528442911255407*G0_2_1;
    A[1609] = 0.0391439193522506*G0_0_0 + 0.135698920421141*G0_0_1 + 0.0678494602105699*G0_0_2 + 0.135698920421141*G0_1_0 + 0.20941996853455*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0678494602105699*G0_2_0 + 0.104709984267275*G0_2_1;
    A[1610] = 0.0521918924696705*G0_0_0 - 0.271397840842283*G0_0_1 - 0.13569892042114*G0_0_2 - 0.271397840842282*G0_1_0 - 0.542795681684566*G0_1_1 - 0.271397840842284*G0_1_2 - 0.13569892042114*G0_2_0 - 0.271397840842282*G0_2_1;
    A[1611] = 0.0391439193522518*G0_0_0 + 0.0264221455627695*G0_0_1 - 0.00554538857490352*G0_0_2 + 0.02642214556277*G0_1_0 + 0.0137003717732881*G0_1_1 - 0.00619778723077387*G0_1_2 - 0.00554538857490255*G0_2_0 - 0.00619778723077356*G0_2_1;
    A[1612] = -0.156575677409009*G0_0_0 - 0.0225077536275445*G0_0_1 + 0.0678494602105707*G0_0_2 - 0.0225077536275452*G0_1_0 + 0.0234863516113511*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0678494602105699*G0_2_0 + 0.104709984267274*G0_2_1;
    A[1613] = 0.0521918924696698*G0_0_0 - 0.00065239865587069*G0_0_1 - 0.0378391220405096*G0_0_2 - 0.000652398655870718*G0_1_0 - 0.0534966897814118*G0_1_1 - 0.0528442911255406*G0_1_2 - 0.0378391220405101*G0_2_0 - 0.05284429112554*G0_2_1;
    A[1614] = -0.0456679059109599*G0_0_0 - 0.0378391220405095*G0_0_1 - 0.0756782440810205*G0_0_2 - 0.0378391220405102*G0_1_0 - 0.0528442911255406*G0_1_2 - 0.0756782440810205*G0_2_0 - 0.05284429112554*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1615] = 0.039143919352251*G0_0_0 + 0.0678494602105702*G0_0_1 + 0.135698920421141*G0_0_2 + 0.0678494602105704*G0_1_0 + 0.104709984267275*G0_1_2 + 0.135698920421141*G0_2_0 + 0.104709984267275*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1616] = 0.0521918924696705*G0_0_0 - 0.135698920421141*G0_0_1 - 0.271397840842283*G0_0_2 - 0.135698920421141*G0_1_0 - 0.271397840842283*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842283*G0_2_1 - 0.542795681684566*G0_2_2;
    A[1617] = 0.0391439193522524*G0_0_0 - 0.00554538857490311*G0_0_1 + 0.0264221455627705*G0_0_2 - 0.0055453885749023*G0_1_0 - 0.00619778723077281*G0_1_2 + 0.0264221455627705*G0_2_0 - 0.00619778723077308*G0_2_1 + 0.0137003717732888*G0_2_2;
    A[1618] = -0.15657567740901*G0_0_0 + 0.0678494602105706*G0_0_1 - 0.0225077536275451*G0_0_2 + 0.0678494602105704*G0_1_0 + 0.104709984267275*G0_1_2 - 0.0225077536275455*G0_2_0 + 0.104709984267275*G0_2_1 + 0.0234863516113513*G0_2_2;
    A[1619] = 0.05219189246967*G0_0_0 - 0.0378391220405099*G0_0_1 - 0.000652398655870217*G0_0_2 - 0.0378391220405107*G0_1_0 - 0.0528442911255403*G0_1_2 - 0.000652398655870997*G0_2_0 - 0.0528442911255412*G0_2_1 - 0.0534966897814113*G0_2_2;
    A[1620] = 0.195719596761262*G0_0_0 + 0.0750258454251505*G0_0_1 + 0.0750258454251499*G0_0_2 + 0.0750258454251501*G0_1_0 + 0.0260959462348348*G0_1_2 + 0.0750258454251508*G0_2_0 + 0.0260959462348348*G0_2_1;
    A[1621] = -0.782878387045047*G0_0_0 - 0.316413348097372*G0_0_1 - 0.316413348097372*G0_0_2 - 0.316413348097373*G0_1_0 - 0.185933616923198*G0_1_2 - 0.316413348097374*G0_2_0 - 0.185933616923198*G0_2_1;
    A[1622] = 0.293579395141893*G0_0_0 + 0.0750258454251499*G0_0_1 + 0.0587158790283781*G0_0_2 + 0.0750258454251508*G0_1_0 + 0.0260959462348348*G0_1_2 + 0.0587158790283793*G0_2_0 + 0.0260959462348355*G0_2_1;
    A[1623] = 0.293579395141893*G0_0_0 + 0.058715879028379*G0_0_1 + 0.0750258454251499*G0_0_2 + 0.0587158790283791*G0_1_0 + 0.0260959462348348*G0_1_2 + 0.0750258454251501*G0_2_0 + 0.0260959462348352*G0_2_1;
    A[1624] = -0.00946499969937454*G0_0_0 - 0.00946499969937458*G0_0_1 - 0.00946499969937458*G0_0_2 - 0.00946499969937446*G0_1_0 - 0.00946499969937447*G0_1_1 - 0.00946499969937447*G0_1_2 - 0.00946499969937451*G0_2_0 - 0.00946499969937453*G0_2_1 - 0.00946499969937451*G0_2_2;
    A[1625] = -0.0345719095719093*G0_0_0 - 0.00472467106581669*G0_1_0 - 0.0047246710658168*G0_2_0;
    A[1626] = -0.0047246710658168*G0_0_1 - 0.0345719095719093*G0_1_1 - 0.00472467106581676*G0_2_1;
    A[1627] = -0.00227817610630093*G0_0_2 - 0.00227817610630105*G0_1_2 + 0.0094649996993747*G0_2_2;
    A[1628] = -0.0845508658008648*G0_0_1 - 0.0429604514890965*G0_0_2 + 0.043971669405697*G0_1_1 - 0.000685018588664194*G0_1_2 - 0.0422754329004322*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[1629] = -0.0808321934624013*G0_0_1 - 0.108232937008979*G0_0_2 - 0.0436454700777617*G0_1_1 - 0.0338921101724922*G0_1_2 - 0.00649136662591533*G0_2_1 - 0.00352295274170302*G0_2_2;
    A[1630] = -0.00939454064454073*G0_0_1 - 0.0377738821749231*G0_0_2 - 0.0155923278753138*G0_1_1 - 0.0188869410874615*G0_1_2 + 0.0094924004429213*G0_2_1 + 0.0189848008858427*G0_2_2;
    A[1631] = 0.00254435475789689*G0_0_1 + 0.0142875305635722*G0_0_2 + 0.00254435475789623*G0_1_1 + 0.00844856259352763*G0_1_2 - 0.00329461321214769*G0_2_1 - 0.0292600797158085*G0_2_2;
    A[1632] = 0.0439716694056961*G0_0_0 - 0.000685018588664941*G0_0_2 - 0.0845508658008653*G0_1_0 - 0.0429604514890966*G0_1_2 - 0.0422754329004328*G0_2_0 - 0.042960451489097*G0_2_2;
    A[1633] = -0.0436454700777607*G0_0_0 - 0.0338921101724915*G0_0_2 - 0.0808321934624015*G0_1_0 - 0.108232937008979*G0_1_2 - 0.00649136662591492*G0_2_0 - 0.00352295274170286*G0_2_2;
    A[1634] = -0.015592327875315*G0_0_0 - 0.0188869410874624*G0_0_2 - 0.00939454064454066*G0_1_0 - 0.0377738821749231*G0_1_2 + 0.00949240044292063*G0_2_0 + 0.0189848008858423*G0_2_2;
    A[1635] = 0.00254435475789687*G0_0_0 + 0.00844856259352787*G0_0_2 + 0.00254435475789625*G0_1_0 + 0.0142875305635717*G0_1_2 - 0.00329461321214767*G0_2_0 - 0.0292600797158087*G0_2_2;
    A[1636] = 0.1273155976932*G0_0_0 - 0.000685018588664869*G0_0_1 - 0.00704590548340553*G0_1_0 - 0.0429604514890966*G0_1_1 - 0.0493213383838382*G0_2_0 - 0.0429604514890971*G0_2_1;
    A[1637] = -0.166948816037356*G0_0_0 - 0.00616516729797949*G0_0_1 + 0.0838658472122008*G0_1_0 + 0.182410664181496*G0_1_1 + 0.167731694424402*G0_2_0 + 0.077700679914221*G0_2_1;
    A[1638] = 0.182410664181495*G0_0_0 + 0.0838658472121996*G0_0_1 - 0.00616516729798044*G0_1_0 - 0.166948816037357*G0_1_1 + 0.07770067991422*G0_2_0 + 0.1677316944244*G0_2_1;
    A[1639] = -0.0429604514890966*G0_0_0 - 0.007045905483405*G0_0_1 - 0.0006850185886642*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0429604514890966*G0_2_0 - 0.0493213383838378*G0_2_1;
    A[1640] = 0.00658922642429541*G0_0_0 + 0.00658922642429556*G0_0_1 + 0.00329461321214785*G0_0_2 + 0.00658922642429554*G0_1_0 + 0.00658922642429567*G0_1_1 + 0.00329461321214772*G0_1_2 + 0.00329461321214756*G0_2_0 + 0.00329461321214765*G0_2_1;
    A[1641] = 0.00450155072550866*G0_0_0 + 0.00450155072550857*G0_0_1 + 0.00257697469068985*G0_0_2 + 0.00450155072550884*G0_1_0 + 0.00450155072550872*G0_1_1 + 0.00257697469068997*G0_1_2 + 0.00257697469068982*G0_2_0 + 0.00257697469068977*G0_2_1;
    A[1642] = -0.0105036183595208*G0_0_0 - 0.0105036183595208*G0_0_1 + 0.00257697469069023*G0_0_2 - 0.0105036183595211*G0_1_0 - 0.0105036183595211*G0_1_1 + 0.00257697469069004*G0_1_2 + 0.00257697469069014*G0_2_0 + 0.00257697469069013*G0_2_1;
    A[1643] = -0.00254435475789674*G0_0_0 - 0.00254435475789679*G0_0_1 + 0.0032946132121476*G0_0_2 - 0.00254435475789623*G0_1_0 - 0.00254435475789625*G0_1_1 + 0.00329461321214783*G0_1_2 + 0.00329461321214771*G0_2_0 + 0.0032946132121477*G0_2_1;
    A[1644] = 0.0292600797158083*G0_0_0 + 0.0259654665036606*G0_0_1 + 0.0292600797158084*G0_0_2 + 0.0259654665036604*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0259654665036605*G0_1_2 + 0.0292600797158082*G0_2_0 + 0.0259654665036605*G0_2_1 + 0.0292600797158083*G0_2_2;
    A[1645] = -0.0189848008858416*G0_0_0 - 0.00949240044292101*G0_0_1 - 0.0189848008858418*G0_0_2 - 0.00949240044292075*G0_1_0 - 0.000652398655870843*G0_1_1 - 0.00949240044292089*G0_1_2 - 0.0189848008858419*G0_2_0 - 0.00949240044292115*G0_2_1 - 0.018984800885842*G0_2_2;
    A[1646] = 0.00352295274170223*G0_0_0 - 0.00296841388421271*G0_0_1 + 0.00352295274170218*G0_0_2 - 0.00296841388421298*G0_1_0 - 0.000652398655870827*G0_1_1 - 0.00296841388421286*G0_1_2 + 0.00352295274170245*G0_2_0 - 0.00296841388421228*G0_2_1 + 0.0035229527417025*G0_2_2;
    A[1647] = 0.0429604514890967*G0_0_0 + 0.000685018588664463*G0_0_1 + 0.0429604514890966*G0_0_2 + 0.000685018588664209*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.000685018588664195*G0_1_2 + 0.0429604514890966*G0_2_0 + 0.000685018588664183*G0_2_1 + 0.0429604514890966*G0_2_2;
    A[1648] = 0.0226708532915127*G0_0_0 + 0.0259654665036605*G0_0_1 + 0.0259654665036605*G0_0_2 + 0.0259654665036602*G0_1_0 + 0.0292600797158083*G0_1_1 + 0.0292600797158082*G0_1_2 + 0.0259654665036603*G0_2_0 + 0.0292600797158082*G0_2_1 + 0.0292600797158082*G0_2_2;
    A[1649] = -0.000652398655870817*G0_0_0 - 0.00949240044292088*G0_0_1 - 0.00949240044292089*G0_0_2 - 0.00949240044292072*G0_1_0 - 0.0189848008858422*G0_1_1 - 0.018984800885842*G0_1_2 - 0.00949240044292091*G0_2_0 - 0.0189848008858421*G0_2_1 - 0.0189848008858421*G0_2_2;
    A[1650] = -0.000652398655870921*G0_0_0 - 0.00296841388421282*G0_0_1 - 0.00296841388421274*G0_0_2 - 0.00296841388421264*G0_1_0 + 0.00352295274170253*G0_1_1 + 0.00352295274170247*G0_1_2 - 0.0029684138842125*G0_2_0 + 0.00352295274170249*G0_2_1 + 0.00352295274170249*G0_2_2;
    A[1651] = 0.022670853291513*G0_0_0 + 0.00068501858866488*G0_0_1 + 0.000685018588664831*G0_0_2 + 0.000685018588664279*G0_1_0 + 0.0429604514890966*G0_1_1 + 0.0429604514890965*G0_1_2 + 0.000685018588664379*G0_2_0 + 0.042960451489097*G0_2_1 + 0.042960451489097*G0_2_2;
    A[1652] = -0.268135847562929*G0_0_0 - 0.0737210481134093*G0_0_1 - 0.0459941052388973*G0_0_2 + 0.180714427676231*G0_1_0 - 0.209419968534551*G0_1_1 + 0.0812236326559239*G0_1_2 + 0.0225077536275451*G0_2_0 - 0.104709984267275*G0_2_1 - 0.0234863516113513*G0_2_2;
    A[1653] = 0.726119703984281*G0_0_0 + 0.270093043530541*G0_0_1 + 0.27009304353054*G0_0_2 + 0.270093043530541*G0_1_0 + 0.726119703984282*G0_1_1 + 0.27009304353054*G0_1_2 + 0.27009304353054*G0_2_0 + 0.27009304353054*G0_2_1 + 0.54018608706108*G0_2_2;
    A[1654] = -0.20941996853455*G0_0_0 + 0.180714427676231*G0_0_1 + 0.0812236326559233*G0_0_2 - 0.0737210481134085*G0_1_0 - 0.26813584756293*G0_1_1 - 0.0459941052388969*G0_1_2 - 0.104709984267275*G0_2_0 + 0.0225077536275445*G0_2_1 - 0.0234863516113516*G0_2_2;
    A[1655] = 0.0655660649150216*G0_0_0 - 0.0748627457611832*G0_0_1 + 0.0278900425384803*G0_0_2 + 0.0278900425384801*G0_1_0 - 0.149725491522366*G0_1_1 + 0.177126235068942*G0_1_2 - 0.0748627457611829*G0_2_0 - 0.0283793415303837*G0_2_1 - 0.149725491522366*G0_2_2;
    A[1656] = -0.149725491522365*G0_0_0 + 0.0278900425384809*G0_0_1 + 0.177126235068943*G0_0_2 - 0.0748627457611826*G0_1_0 + 0.0655660649150238*G0_1_1 + 0.0278900425384805*G0_1_2 - 0.0283793415303825*G0_2_0 - 0.0748627457611813*G0_2_1 - 0.149725491522365*G0_2_2;
    A[1657] = -0.0137003717732884*G0_0_0 - 0.0198981590040627*G0_0_1 - 0.0851380245911501*G0_0_2 - 0.0198981590040619*G0_1_0 - 0.0137003717732889*G0_1_1 - 0.0851380245911493*G0_1_2 + 0.0127217737894825*G0_2_0 + 0.0127217737894817*G0_2_1 + 0.113517366121532*G0_2_2;
    A[1658] = -0.0254435475789637*G0_0_0 - 0.0241387502672216*G0_0_1 - 0.012721773789482*G0_0_2 - 0.0241387502672222*G0_1_0 - 0.0228339529554803*G0_1_1 - 0.0120693751336111*G0_1_2 - 0.0127217737894821*G0_2_0 - 0.0120693751336106*G0_2_1;
    A[1659] = 0.0567586830607656*G0_0_0 + 0.0283793415303823*G0_0_1 + 0.0283793415303825*G0_0_2 + 0.0283793415303828*G0_1_0 - 0.00489298991903206*G0_1_1 + 0.0141896707651915*G0_1_2 + 0.0283793415303827*G0_2_0 + 0.0141896707651907*G0_2_1;
    A[1660] = 0.20941996853455*G0_0_0 + 0.0737210481134088*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0737210481134086*G0_1_0 - 0.0228339529554803*G0_1_1 + 0.0368605240567052*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0368605240567043*G0_2_1;
    A[1661] = -0.0166361657247075*G0_0_0 + 0.00146789697570987*G0_0_1 - 0.0181040627004165*G0_0_2 + 0.00146789697570946*G0_1_0 + 0.0195719596761268*G0_1_1 - 0.010275278829966*G0_1_2 - 0.0181040627004168*G0_2_0 - 0.0102752788299661*G0_2_1;
    A[1662] = 0.149725491522365*G0_0_0 + 0.0748627457611828*G0_0_1 + 0.0283793415303825*G0_0_2 + 0.0748627457611826*G0_1_0 + 0.0195719596761262*G0_1_1 + 0.0141896707651911*G0_1_2 + 0.0283793415303824*G0_2_0 + 0.0141896707651917*G0_2_1;
    A[1663] = 0.0137003717732882*G0_0_0 + 0.0198981590040614*G0_0_1 - 0.0127217737894821*G0_0_2 + 0.0198981590040618*G0_1_0 + 0.0260959462348348*G0_1_1 - 0.012069375133611*G0_1_2 - 0.0127217737894825*G0_2_0 - 0.0120693751336115*G0_2_1;
    A[1664] = -0.0228339529554799*G0_0_0 - 0.0241387502672225*G0_0_1 - 0.0120693751336113*G0_0_2 - 0.0241387502672215*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.0127217737894815*G0_1_2 - 0.0120693751336105*G0_2_0 - 0.0127217737894821*G0_2_1;
    A[1665] = -0.00489298991903143*G0_0_0 + 0.0283793415303835*G0_0_1 + 0.014189670765192*G0_0_2 + 0.028379341530382*G0_1_0 + 0.0567586830607654*G0_1_1 + 0.0283793415303824*G0_1_2 + 0.0141896707651912*G0_2_0 + 0.0283793415303826*G0_2_1;
    A[1666] = -0.0228339529554802*G0_0_0 + 0.0737210481134093*G0_0_1 + 0.0368605240567044*G0_0_2 + 0.0737210481134092*G0_1_0 + 0.209419968534551*G0_1_1 + 0.104709984267276*G0_1_2 + 0.0368605240567046*G0_2_0 + 0.104709984267275*G0_2_1;
    A[1667] = 0.0195719596761265*G0_0_0 + 0.00146789697570922*G0_0_1 - 0.010275278829966*G0_0_2 + 0.0014678969757098*G0_1_0 - 0.0166361657247072*G0_1_1 - 0.0181040627004167*G0_1_2 - 0.0102752788299663*G0_2_0 - 0.0181040627004168*G0_2_1;
    A[1668] = 0.0195719596761254*G0_0_0 + 0.0748627457611829*G0_0_1 + 0.0141896707651914*G0_0_2 + 0.0748627457611827*G0_1_0 + 0.149725491522366*G0_1_1 + 0.0283793415303826*G0_1_2 + 0.0141896707651913*G0_2_0 + 0.0283793415303834*G0_2_1;
    A[1669] = 0.0260959462348357*G0_0_0 + 0.0198981590040627*G0_0_1 - 0.0120693751336111*G0_0_2 + 0.0198981590040618*G0_1_0 + 0.0137003717732889*G0_1_1 - 0.0127217737894817*G0_1_2 - 0.0120693751336108*G0_2_0 - 0.0127217737894817*G0_2_1;
    A[1670] = -0.0880738185425672*G0_0_0 - 0.088726217198438*G0_0_1 - 0.100795592332049*G0_0_2 - 0.0887262171984376*G0_1_0 - 0.0880738185425667*G0_1_1 - 0.100795592332049*G0_1_2 - 0.100795592332049*G0_2_0 - 0.100795592332049*G0_2_1 - 0.113517366121531*G0_2_2;
    A[1671] = 0.019571959676126*G0_0_0 + 0.0606730749959911*G0_0_1 + 0.0748627457611827*G0_0_2 + 0.0606730749959899*G0_1_0 + 0.0929668084615988*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0748627457611816*G0_2_0 + 0.121346149991982*G0_2_1 + 0.149725491522364*G0_2_2;
    A[1672] = -0.0880738185425675*G0_0_0 + 0.00913358118219229*G0_0_1 + 0.0459941052388973*G0_0_2 + 0.00913358118219277*G0_1_0 - 0.185933616923198*G0_1_1 - 0.0812236326559236*G0_1_2 + 0.045994105238897*G0_2_0 - 0.0812236326559234*G0_2_1 + 0.0234863516113518*G0_2_2;
    A[1673] = 0.0929668084615984*G0_0_0 + 0.0606730749959903*G0_0_1 + 0.121346149991982*G0_0_2 + 0.0606730749959904*G0_1_0 + 0.0195719596761248*G0_1_1 + 0.0748627457611818*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0748627457611813*G0_2_1 + 0.149725491522364*G0_2_2;
    A[1674] = 0.0929668084615995*G0_0_0 - 0.13504652176527*G0_0_1 - 0.27009304353054*G0_0_2 - 0.13504652176527*G0_1_0 + 0.0929668084615999*G0_1_1 - 0.270093043530541*G0_1_2 - 0.270093043530541*G0_2_0 - 0.27009304353054*G0_2_1 - 0.540186087061081*G0_2_2;
    A[1675] = -0.185933616923197*G0_0_0 + 0.00913358118219219*G0_0_1 - 0.0812236326559232*G0_0_2 + 0.00913358118219349*G0_1_0 - 0.0880738185425669*G0_1_1 + 0.045994105238897*G0_1_2 - 0.0812236326559223*G0_2_0 + 0.0459941052388969*G0_2_1 + 0.0234863516113517*G0_2_2;
    A[1676] = 0.146789697570946*G0_0_0 + 0.141896707651915*G0_0_1 + 0.0929668084615987*G0_0_2 + 0.141896707651915*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0929668084615985*G0_1_2 + 0.0929668084615993*G0_2_0 + 0.0929668084615998*G0_2_1;
    A[1677] = 0.146789697570946*G0_0_0 - 0.298472385060924*G0_0_1 - 0.102752788299663*G0_0_2 - 0.298472385060924*G0_1_0 - 0.782878387045048*G0_1_1 - 0.298472385060924*G0_1_2 - 0.102752788299662*G0_2_0 - 0.298472385060924*G0_2_1;
    A[1678] = -0.782878387045047*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060924*G0_0_2 - 0.298472385060924*G0_1_0 + 0.146789697570947*G0_1_1 - 0.102752788299663*G0_1_2 - 0.298472385060923*G0_2_0 - 0.102752788299662*G0_2_1;
    A[1679] = -0.0489298991903146*G0_0_0 - 0.0293579395141899*G0_0_1 + 0.0929668084616003*G0_0_2 - 0.0293579395141895*G0_1_0 - 0.0489298991903161*G0_1_1 + 0.0929668084615995*G0_1_2 + 0.0929668084615997*G0_2_0 + 0.0929668084615983*G0_2_1;
    A[1680] = -0.013742125287264*G0_0_0 - 0.013742125287264*G0_0_1 - 0.013742125287264*G0_0_2 - 0.013742125287264*G0_1_0 - 0.013742125287264*G0_1_1 - 0.013742125287264*G0_1_2 - 0.0137421252872639*G0_2_0 - 0.013742125287264*G0_2_1 - 0.0137421252872639*G0_2_2;
    A[1681] = 0.0137421252872641*G0_0_0 - 0.00517743573299126*G0_1_0 - 0.00517743573299132*G0_2_0;
    A[1682] = 0.0143945239431349*G0_0_1 + 0.0479930547204849*G0_1_1 + 0.0143945239431348*G0_2_1;
    A[1683] = -0.00517743573299124*G0_0_2 - 0.0051774357329912*G0_1_2 + 0.0137421252872641*G0_2_2;
    A[1684] = 0.192588083213081*G0_0_1 + 0.12473862300251*G0_0_2 - 0.0644569872000425*G0_1_1 + 0.0284445813959699*G0_1_2 + 0.0962940416065402*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1685] = 0.0109602974186308*G0_0_1 + 0.146659217839773*G0_0_2 + 0.132306447410613*G0_1_1 + 0.073329608919886*G0_1_2 - 0.062369311501255*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1686] = 0.00182671623643825*G0_0_1 - 0.0823327103709047*G0_0_2 - 0.0510175748891022*G0_1_1 - 0.046385544432419*G0_1_2 + 0.0377738821749234*G0_2_1 + 0.127087258163646*G0_2_2;
    A[1687] = 0.0118736555368497*G0_0_1 + 0.0477555816097478*G0_0_2 + 0.01187365553685*G0_1_1 + 0.0264873854283576*G0_1_2 - 0.00939454064454079*G0_2_1 - 0.0641307878721072*G0_2_2;
    A[1688] = -0.0187890812890808*G0_0_0 - 0.00939454064454019*G0_0_2 + 0.0425363923627808*G0_1_0 + 0.0118736555368498*G0_1_2 + 0.0212681961813907*G0_2_0 + 0.01187365553685*G0_2_2;
    A[1689] = 0.0148746893538553*G0_0_0 + 0.0149399292194424*G0_0_2 + 0.0403182369328201*G0_1_0 + 0.0540186087061083*G0_1_2 + 0.00123955744615429*G0_2_0 + 0.00117431758056746*G0_2_2;
    A[1690] = 0.00117431758056825*G0_0_0 + 0.00123955744615525*G0_0_2 + 0.0540186087061081*G0_1_0 + 0.0403182369328198*G0_1_2 + 0.0149399292194433*G0_2_0 + 0.0148746893538562*G0_2_2;
    A[1691] = 0.0118736555368496*G0_0_0 + 0.0212681961813902*G0_0_2 + 0.01187365553685*G0_1_0 + 0.0425363923627809*G0_1_2 - 0.00939454064454084*G0_2_0 - 0.0187890812890815*G0_2_2;
    A[1692] = -0.0641307878721065*G0_0_0 - 0.00939454064454032*G0_0_1 + 0.0264873854283573*G0_1_0 + 0.0118736555368496*G0_1_1 + 0.0477555816097482*G0_2_0 + 0.0118736555368499*G0_2_1;
    A[1693] = 0.127087258163646*G0_0_0 + 0.0377738821749232*G0_0_1 - 0.0463855444324192*G0_1_0 - 0.0510175748891022*G0_1_1 - 0.0823327103709046*G0_2_0 + 0.00182671623643828*G0_2_1;
    A[1694] = -0.12473862300251*G0_0_0 - 0.0623693115012549*G0_0_1 + 0.0733296089198861*G0_1_0 + 0.132306447410613*G0_1_1 + 0.146659217839773*G0_2_0 + 0.0109602974186309*G0_2_1;
    A[1695] = 0.12473862300251*G0_0_0 + 0.0962940416065402*G0_0_1 + 0.02844458139597*G0_1_0 - 0.0644569872000423*G0_1_1 + 0.12473862300251*G0_2_0 + 0.192588083213081*G0_2_1;
    A[1696] = 0.0187890812890811*G0_0_0 + 0.0187890812890811*G0_0_1 + 0.00939454064454063*G0_0_2 + 0.0187890812890809*G0_1_0 + 0.0187890812890808*G0_1_1 + 0.00939454064454058*G0_1_2 + 0.00939454064454027*G0_2_0 + 0.00939454064454035*G0_2_1;
    A[1697] = -0.0148746893538554*G0_0_0 - 0.0148746893538554*G0_0_1 + 6.52398655870295e-05*G0_0_2 - 0.0148746893538558*G0_1_0 - 0.0148746893538557*G0_1_1 + 6.52398655870815e-05*G0_1_2 + 6.52398655874953e-05*G0_2_0 + 6.52398655873763e-05*G0_2_1;
    A[1698] = -0.00117431758056807*G0_0_0 - 0.00117431758056803*G0_0_1 + 6.52398655871632e-05*G0_0_2 - 0.00117431758056781*G0_1_0 - 0.00117431758056783*G0_1_1 + 6.5239865587186e-05*G0_1_2 + 6.52398655866854e-05*G0_2_0 + 6.52398655867253e-05*G0_2_1;
    A[1699] = -0.0118736555368497*G0_0_0 - 0.0118736555368496*G0_0_1 + 0.00939454064454055*G0_0_2 - 0.0118736555368499*G0_1_0 - 0.0118736555368499*G0_1_1 + 0.00939454064454046*G0_1_2 + 0.00939454064454087*G0_2_0 + 0.00939454064454088*G0_2_1;
    A[1700] = 0.0641307878721064*G0_0_0 + 0.054736247227566*G0_0_1 + 0.0641307878721064*G0_0_2 + 0.0547362472275659*G0_1_0 + 0.0453417065830254*G0_1_1 + 0.0547362472275658*G0_1_2 + 0.0641307878721062*G0_2_0 + 0.0547362472275658*G0_2_1 + 0.0641307878721061*G0_2_2;
    A[1701] = -0.127087258163646*G0_0_0 - 0.0893133759887223*G0_0_1 - 0.127087258163646*G0_0_2 - 0.089313375988722*G0_1_0 - 0.0665446628988287*G0_1_1 - 0.089313375988722*G0_1_2 - 0.127087258163645*G0_2_0 - 0.0893133759887218*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1702] = 0.124738623002511*G0_0_0 + 0.0623693115012554*G0_0_1 + 0.124738623002511*G0_0_2 + 0.0623693115012552*G0_1_0 + 0.0476251018785735*G0_1_1 + 0.0623693115012552*G0_1_2 + 0.12473862300251*G0_2_0 + 0.062369311501255*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1703] = -0.12473862300251*G0_0_0 - 0.0284445813959701*G0_0_1 - 0.12473862300251*G0_0_2 - 0.0284445813959699*G0_1_0 - 0.0606730749959911*G0_1_1 - 0.0284445813959699*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0284445813959698*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1704] = 0.00939454064454034*G0_0_1 + 0.00939454064454031*G0_0_2 + 0.00939454064454054*G0_1_0 + 0.0187890812890809*G0_1_1 + 0.0187890812890809*G0_1_2 + 0.00939454064454036*G0_2_0 + 0.0187890812890809*G0_2_1 + 0.0187890812890809*G0_2_2;
    A[1705] = 6.52398655870789e-05*G0_0_1 + 6.52398655870938e-05*G0_0_2 + 6.52398655870458e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538558*G0_1_2 + 6.52398655871194e-05*G0_2_0 - 0.0148746893538557*G0_2_1 - 0.0148746893538557*G0_2_2;
    A[1706] = 6.52398655869999e-05*G0_0_1 + 6.52398655869994e-05*G0_0_2 + 6.52398655871545e-05*G0_1_0 - 0.00117431758056774*G0_1_1 - 0.0011743175805677*G0_1_2 + 6.52398655871945e-05*G0_2_0 - 0.00117431758056754*G0_2_1 - 0.00117431758056754*G0_2_2;
    A[1707] = 0.00939454064454035*G0_0_1 + 0.00939454064454027*G0_0_2 + 0.00939454064454056*G0_1_0 - 0.0118736555368497*G0_1_1 - 0.0118736555368497*G0_1_2 + 0.0093945406445406*G0_2_0 - 0.0118736555368499*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1708] = 0.105688582251081*G0_0_0 + 0.0300103381700601*G0_0_1 + 0.0528442911255407*G0_0_2 - 0.0743734467692797*G0_1_0 + 0.105688582251081*G0_1_1 + 0.000652398655870211*G0_1_2 + 0.000652398655870264*G0_2_0 + 0.0528442911255404*G0_2_1 + 0.0534966897814114*G0_2_2;
    A[1709] = -0.20941996853455*G0_0_0 - 0.0737210481134085*G0_0_1 - 0.104709984267275*G0_0_2 + 0.180714427676231*G0_1_0 - 0.26813584756293*G0_1_1 + 0.0225077536275445*G0_1_2 + 0.0812236326559233*G0_2_0 - 0.0459941052388969*G0_2_1 - 0.0234863516113516*G0_2_2;
    A[1710] = 0.542795681684565*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623905*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684564*G0_2_2;
    A[1711] = -0.0137003717732874*G0_0_0 + 0.0127217737894821*G0_0_1 - 0.0198981590040618*G0_0_2 - 0.0851380245911493*G0_1_0 + 0.113517366121532*G0_1_1 - 0.085138024591149*G0_1_2 - 0.0198981590040607*G0_2_0 + 0.0127217737894827*G0_2_1 - 0.0137003717732885*G0_2_2;
    A[1712] = -0.0234863516113521*G0_0_0 - 0.0459941052388974*G0_0_1 + 0.0812236326559232*G0_0_2 + 0.0225077536275451*G0_1_0 - 0.268135847562929*G0_1_1 + 0.180714427676231*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0737210481134087*G0_2_1 - 0.20941996853455*G0_2_2;
    A[1713] = 0.0534966897814113*G0_0_0 + 0.0528442911255414*G0_0_1 + 0.00065239865587144*G0_0_2 + 0.000652398655871055*G0_1_0 + 0.105688582251081*G0_1_1 - 0.0743734467692795*G0_1_2 + 0.0528442911255396*G0_2_0 + 0.0300103381700596*G0_2_1 + 0.105688582251082*G0_2_2;
    A[1714] = -0.105688582251081*G0_0_0 - 0.0756782440810213*G0_0_1 - 0.0528442911255401*G0_0_2 - 0.0756782440810207*G0_1_0 - 0.0456679059109602*G0_1_1 - 0.0378391220405102*G0_1_2 - 0.05284429112554*G0_2_0 - 0.0378391220405103*G0_2_1;
    A[1715] = 0.20941996853455*G0_0_0 + 0.135698920421142*G0_0_1 + 0.104709984267275*G0_0_2 + 0.135698920421142*G0_1_0 + 0.0391439193522515*G0_1_1 + 0.0678494602105704*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0678494602105705*G0_2_1;
    A[1716] = -0.542795681684565*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842283*G0_0_2 - 0.271397840842283*G0_1_0 + 0.0521918924696704*G0_1_1 - 0.135698920421141*G0_1_2 - 0.271397840842282*G0_2_0 - 0.135698920421141*G0_2_1;
    A[1717] = 0.0137003717732878*G0_0_0 + 0.0264221455627698*G0_0_1 - 0.0061977872307743*G0_0_2 + 0.0264221455627699*G0_1_0 + 0.0391439193522521*G0_1_1 - 0.00554538857490298*G0_1_2 - 0.00619778723077422*G0_2_0 - 0.00554538857490259*G0_2_1;
    A[1718] = 0.0234863516113523*G0_0_0 - 0.022507753627545*G0_0_1 + 0.104709984267276*G0_0_2 - 0.0225077536275449*G0_1_0 - 0.156575677409009*G0_1_1 + 0.0678494602105708*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0678494602105704*G0_2_1;
    A[1719] = -0.053496689781411*G0_0_0 - 0.000652398655870083*G0_0_1 - 0.05284429112554*G0_0_2 - 0.000652398655870868*G0_1_0 + 0.0521918924696702*G0_1_1 - 0.0378391220405103*G0_1_2 - 0.0528442911255395*G0_2_0 - 0.03783912204051*G0_2_1;
    A[1720] = -0.0300103381700596*G0_0_1 - 0.0150051690850295*G0_0_2 - 0.03001033817006*G0_1_0 - 0.0600206763401202*G0_1_1 - 0.0300103381700601*G0_1_2 - 0.0150051690850296*G0_2_0 - 0.0300103381700596*G0_2_1;
    A[1721] = -0.00130479731174212*G0_0_1 - 0.000652398655870949*G0_0_2 - 0.00130479731174152*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.0127217737894816*G0_1_2 - 0.000652398655871052*G0_2_0 - 0.012721773789482*G0_2_1;
    A[1722] = -0.0300103381700601*G0_0_1 - 0.0150051690850296*G0_0_2 - 0.0300103381700601*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0528442911255407*G0_1_2 - 0.0150051690850303*G0_2_0 - 0.0528442911255406*G0_2_1;
    A[1723] = -0.0127217737894822*G0_0_1 - 0.000652398655871743*G0_0_2 - 0.0127217737894823*G0_1_0 - 0.0254435475789648*G0_1_1 - 0.00130479731174219*G0_1_2 - 0.00065239865587116*G0_2_0 - 0.00130479731174264*G0_2_1;
    A[1724] = -0.012721773789482*G0_0_1 - 0.000652398655871372*G0_0_2 - 0.0127217737894821*G0_1_0 - 0.113517366121532*G0_1_1 - 0.0127217737894823*G0_1_2 - 0.000652398655870722*G0_2_0 - 0.0127217737894824*G0_2_1;
    A[1725] = -0.0528442911255413*G0_0_1 - 0.0150051690850294*G0_0_2 - 0.0528442911255403*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0300103381700596*G0_1_2 - 0.01500516908503*G0_2_0 - 0.0300103381700596*G0_2_1;
    A[1726] = -0.0378391220405101*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0378391220405103*G0_1_0 - 0.0456679059109604*G0_1_1 - 0.0756782440810207*G0_1_2 - 0.0528442911255398*G0_2_0 - 0.0756782440810206*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1727] = -0.00554538857490263*G0_0_1 - 0.00619778723077418*G0_0_2 - 0.00554538857490255*G0_1_0 + 0.0391439193522522*G0_1_1 + 0.0264221455627698*G0_1_2 - 0.00619778723077364*G0_2_0 + 0.0264221455627701*G0_2_1 + 0.0137003717732882*G0_2_2;
    A[1728] = -0.0378391220405103*G0_0_1 - 0.0528442911255407*G0_0_2 - 0.0378391220405103*G0_1_0 + 0.05219189246967*G0_1_1 - 0.000652398655870324*G0_1_2 - 0.0528442911255409*G0_2_0 - 0.000652398655870832*G0_2_1 - 0.0534966897814115*G0_2_2;
    A[1729] = 0.0678494602105705*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105705*G0_1_0 + 0.039143919352252*G0_1_1 + 0.135698920421142*G0_1_2 + 0.104709984267275*G0_2_0 + 0.135698920421141*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1730] = 0.0678494602105705*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105709*G0_1_0 - 0.156575677409009*G0_1_1 - 0.0225077536275442*G0_1_2 + 0.104709984267276*G0_2_0 - 0.0225077536275451*G0_2_1 + 0.0234863516113518*G0_2_2;
    A[1731] = -0.135698920421141*G0_0_1 - 0.271397840842282*G0_0_2 - 0.135698920421141*G0_1_0 + 0.0521918924696699*G0_1_1 - 0.271397840842282*G0_1_2 - 0.271397840842283*G0_2_0 - 0.271397840842282*G0_2_1 - 0.542795681684565*G0_2_2;
    A[1732] = 0.0750258454251495*G0_0_1 + 0.0260959462348353*G0_0_2 + 0.07502584542515*G0_1_0 + 0.195719596761262*G0_1_1 + 0.0750258454251505*G0_1_2 + 0.0260959462348337*G0_2_0 + 0.0750258454251492*G0_2_1;
    A[1733] = 0.0750258454251505*G0_0_1 + 0.0260959462348361*G0_0_2 + 0.0750258454251503*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0587158790283792*G0_1_2 + 0.0260959462348359*G0_2_0 + 0.0587158790283791*G0_2_1;
    A[1734] = -0.316413348097372*G0_0_1 - 0.185933616923198*G0_0_2 - 0.316413348097373*G0_1_0 - 0.782878387045047*G0_1_1 - 0.316413348097373*G0_1_2 - 0.185933616923198*G0_2_0 - 0.316413348097372*G0_2_1;
    A[1735] = 0.0587158790283794*G0_0_1 + 0.0260959462348338*G0_0_2 + 0.0587158790283795*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0750258454251498*G0_1_2 + 0.026095946234835*G0_2_0 + 0.075025845425151*G0_2_1;
    A[1736] = -0.00946499969937465*G0_0_0 - 0.00946499969937469*G0_0_1 - 0.00946499969937468*G0_0_2 - 0.00946499969937462*G0_1_0 - 0.00946499969937464*G0_1_1 - 0.00946499969937462*G0_1_2 - 0.00946499969937463*G0_2_0 - 0.00946499969937465*G0_2_1 - 0.00946499969937464*G0_2_2;
    A[1737] = -0.0345719095719093*G0_0_0 - 0.00472467106581676*G0_1_0 - 0.00472467106581678*G0_2_0;
    A[1738] = -0.00227817610630114*G0_0_1 + 0.00946499969937467*G0_1_1 - 0.00227817610630111*G0_2_1;
    A[1739] = -0.0047246710658169*G0_0_2 - 0.00472467106581705*G0_1_2 - 0.0345719095719096*G0_2_2;
    A[1740] = 0.0142875305635729*G0_0_1 + 0.00254435475789695*G0_0_2 - 0.0292600797158081*G0_1_1 - 0.00329461321214759*G0_1_2 + 0.00844856259352797*G0_2_1 + 0.00254435475789638*G0_2_2;
    A[1741] = -0.0377738821749242*G0_0_1 - 0.00939454064454088*G0_0_2 + 0.0189848008858413*G0_1_1 + 0.00949240044292073*G0_1_2 - 0.0188869410874623*G0_2_1 - 0.0155923278753139*G0_2_2;
    A[1742] = -0.108232937008977*G0_0_1 - 0.0808321934624009*G0_0_2 - 0.00352295274170158*G0_1_1 - 0.00649136662591481*G0_1_2 - 0.0338921101724913*G0_2_1 - 0.0436454700777615*G0_2_2;
    A[1743] = -0.0429604514890972*G0_0_1 - 0.0845508658008652*G0_0_2 - 0.0429604514890981*G0_1_1 - 0.0422754329004332*G0_1_2 - 0.000685018588665159*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[1744] = 0.127315597693201*G0_0_0 - 0.000685018588664525*G0_0_2 - 0.049321338383837*G0_1_0 - 0.0429604514890961*G0_1_2 - 0.00704590548340457*G0_2_0 - 0.0429604514890961*G0_2_2;
    A[1745] = -0.166948816037357*G0_0_0 - 0.0061651672979797*G0_0_2 + 0.1677316944244*G0_1_0 + 0.0777006799142198*G0_1_2 + 0.0838658472121989*G0_2_0 + 0.182410664181495*G0_2_2;
    A[1746] = 0.182410664181496*G0_0_0 + 0.0838658472122007*G0_0_2 + 0.0777006799142218*G0_1_0 + 0.167731694424402*G0_1_2 - 0.0061651672979788*G0_2_0 - 0.166948816037356*G0_2_2;
    A[1747] = -0.0429604514890972*G0_0_0 - 0.0070459054834056*G0_0_2 - 0.0429604514890981*G0_1_0 - 0.049321338383839*G0_1_2 - 0.000685018588665164*G0_2_0 + 0.1273155976932*G0_2_2;
    A[1748] = 0.0439716694056967*G0_0_0 - 0.000685018588664461*G0_0_1 - 0.0422754329004318*G0_1_0 - 0.0429604514890962*G0_1_1 - 0.0845508658008643*G0_2_0 - 0.0429604514890961*G0_2_1;
    A[1749] = -0.0436454700777611*G0_0_0 - 0.0338921101724914*G0_0_1 - 0.0064913666259153*G0_1_0 - 0.00352295274170299*G0_1_1 - 0.0808321934624012*G0_2_0 - 0.108232937008978*G0_2_1;
    A[1750] = -0.0155923278753141*G0_0_0 - 0.0188869410874623*G0_0_1 + 0.00949240044292112*G0_1_0 + 0.0189848008858427*G0_1_1 - 0.00939454064454064*G0_2_0 - 0.0377738821749232*G0_2_1;
    A[1751] = 0.00254435475789691*G0_0_0 + 0.00844856259352858*G0_0_1 - 0.00329461321214755*G0_1_0 - 0.0292600797158085*G0_1_1 + 0.00254435475789637*G0_2_0 + 0.0142875305635719*G0_2_1;
    A[1752] = 0.0292600797158087*G0_0_0 + 0.0292600797158087*G0_0_1 + 0.0259654665036608*G0_0_2 + 0.0292600797158083*G0_1_0 + 0.0292600797158083*G0_1_1 + 0.0259654665036605*G0_1_2 + 0.0259654665036603*G0_2_0 + 0.0259654665036604*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[1753] = -0.0189848008858424*G0_0_0 - 0.0189848008858424*G0_0_1 - 0.00949240044292134*G0_0_2 - 0.0189848008858416*G0_1_0 - 0.0189848008858416*G0_1_1 - 0.00949240044292077*G0_1_2 - 0.00949240044292067*G0_2_0 - 0.00949240044292074*G0_2_1 - 0.000652398655870927*G0_2_2;
    A[1754] = 0.00352295274170286*G0_0_0 + 0.003522952741703*G0_0_1 - 0.00296841388421249*G0_0_2 + 0.00352295274170191*G0_1_0 + 0.00352295274170195*G0_1_1 - 0.00296841388421302*G0_1_2 - 0.002968413884213*G0_2_0 - 0.00296841388421302*G0_2_1 - 0.000652398655870975*G0_2_2;
    A[1755] = 0.0429604514890973*G0_0_0 + 0.0429604514890972*G0_0_1 + 0.000685018588664468*G0_0_2 + 0.0429604514890982*G0_1_0 + 0.0429604514890982*G0_1_1 + 0.00068501858866488*G0_1_2 + 0.0006850185886652*G0_2_0 + 0.00068501858866518*G0_2_1 + 0.0226708532915133*G0_2_2;
    A[1756] = 0.00658922642429589*G0_0_0 + 0.00329461321214796*G0_0_1 + 0.00658922642429592*G0_0_2 + 0.00329461321214786*G0_1_0 + 0.00329461321214787*G0_1_2 + 0.00658922642429574*G0_2_0 + 0.00329461321214825*G0_2_1 + 0.00658922642429579*G0_2_2;
    A[1757] = 0.00450155072550887*G0_0_0 + 0.0025769746906899*G0_0_1 + 0.00450155072550876*G0_0_2 + 0.00257697469068995*G0_1_0 + 0.00257697469068994*G0_1_2 + 0.00450155072550929*G0_2_0 + 0.0025769746906898*G0_2_1 + 0.0045015507255093*G0_2_2;
    A[1758] = -0.0105036183595206*G0_0_0 + 0.00257697469068985*G0_0_1 - 0.0105036183595206*G0_0_2 + 0.00257697469069019*G0_1_0 + 0.00257697469069019*G0_1_2 - 0.0105036183595213*G0_2_0 + 0.00257697469068981*G0_2_1 - 0.0105036183595213*G0_2_2;
    A[1759] = -0.00254435475789689*G0_0_0 + 0.00329461321214803*G0_0_1 - 0.00254435475789691*G0_0_2 + 0.00329461321214757*G0_1_0 + 0.00329461321214754*G0_1_2 - 0.00254435475789635*G0_2_0 + 0.00329461321214794*G0_2_1 - 0.0025443547578963*G0_2_2;
    A[1760] = 0.0226708532915129*G0_0_0 + 0.0259654665036609*G0_0_1 + 0.0259654665036609*G0_0_2 + 0.0259654665036608*G0_1_0 + 0.0292600797158086*G0_1_1 + 0.0292600797158086*G0_1_2 + 0.0259654665036609*G0_2_0 + 0.0292600797158086*G0_2_1 + 0.0292600797158086*G0_2_2;
    A[1761] = -0.000652398655871016*G0_0_0 - 0.00949240044292136*G0_0_1 - 0.00949240044292135*G0_0_2 - 0.00949240044292155*G0_1_0 - 0.0189848008858427*G0_1_1 - 0.0189848008858428*G0_1_2 - 0.0094924004429216*G0_2_0 - 0.0189848008858427*G0_2_1 - 0.0189848008858427*G0_2_2;
    A[1762] = -0.00065239865587093*G0_0_0 - 0.00296841388421248*G0_0_1 - 0.00296841388421249*G0_0_2 - 0.00296841388421224*G0_1_0 + 0.00352295274170286*G0_1_1 + 0.00352295274170287*G0_1_2 - 0.00296841388421222*G0_2_0 + 0.00352295274170288*G0_2_1 + 0.00352295274170297*G0_2_2;
    A[1763] = 0.022670853291513*G0_0_0 + 0.000685018588664423*G0_0_1 + 0.000685018588664476*G0_0_2 + 0.000685018588664297*G0_1_0 + 0.042960451489096*G0_1_1 + 0.0429604514890961*G0_1_2 + 0.000685018588664287*G0_2_0 + 0.0429604514890962*G0_2_1 + 0.0429604514890962*G0_2_2;
    A[1764] = -0.268135847562928*G0_0_0 - 0.0459941052388966*G0_0_1 - 0.073721048113408*G0_0_2 + 0.0225077536275447*G0_1_0 - 0.0234863516113506*G0_1_1 - 0.104709984267273*G0_1_2 + 0.180714427676231*G0_2_0 + 0.0812236326559239*G0_2_1 - 0.209419968534549*G0_2_2;
    A[1765] = 0.0655660649150217*G0_0_0 + 0.0278900425384801*G0_0_1 - 0.0748627457611829*G0_0_2 - 0.0748627457611832*G0_1_0 - 0.149725491522366*G0_1_1 - 0.0283793415303837*G0_1_2 + 0.0278900425384804*G0_2_0 + 0.177126235068942*G0_2_1 - 0.149725491522365*G0_2_2;
    A[1766] = -0.0137003717732874*G0_0_0 - 0.0851380245911493*G0_0_1 - 0.0198981590040607*G0_0_2 + 0.0127217737894821*G0_1_0 + 0.113517366121532*G0_1_1 + 0.0127217737894827*G0_1_2 - 0.0198981590040618*G0_2_0 - 0.085138024591149*G0_2_1 - 0.0137003717732885*G0_2_2;
    A[1767] = 0.726119703984281*G0_0_0 + 0.270093043530541*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530541*G0_1_0 + 0.540186087061082*G0_1_1 + 0.270093043530539*G0_1_2 + 0.270093043530541*G0_2_0 + 0.270093043530539*G0_2_1 + 0.72611970398428*G0_2_2;
    A[1768] = -0.149725491522365*G0_0_0 + 0.177126235068943*G0_0_1 + 0.0278900425384793*G0_0_2 - 0.0283793415303826*G0_1_0 - 0.149725491522364*G0_1_1 - 0.0748627457611828*G0_1_2 - 0.0748627457611823*G0_2_0 + 0.0278900425384805*G0_2_1 + 0.0655660649150236*G0_2_2;
    A[1769] = -0.20941996853455*G0_0_0 + 0.0812236326559233*G0_0_1 + 0.180714427676232*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0234863516113524*G0_1_1 + 0.022507753627546*G0_1_2 - 0.073721048113409*G0_2_0 - 0.0459941052388966*G0_2_1 - 0.268135847562929*G0_2_2;
    A[1770] = -0.025443547578964*G0_0_0 - 0.0127217737894823*G0_0_1 - 0.0241387502672222*G0_0_2 - 0.0127217737894817*G0_1_0 - 0.0120693751336107*G0_1_2 - 0.0241387502672216*G0_2_0 - 0.0120693751336111*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[1771] = -0.0166361657247078*G0_0_0 - 0.0181040627004162*G0_0_1 + 0.00146789697570886*G0_0_2 - 0.0181040627004172*G0_1_0 - 0.010275278829967*G0_1_2 + 0.00146789697570958*G0_2_0 - 0.0102752788299664*G0_2_1 + 0.0195719596761259*G0_2_2;
    A[1772] = 0.0137003717732873*G0_0_0 - 0.012721773789483*G0_0_1 + 0.0198981590040614*G0_0_2 - 0.0127217737894822*G0_1_0 - 0.0120693751336109*G0_1_2 + 0.0198981590040617*G0_2_0 - 0.0120693751336111*G0_2_1 + 0.026095946234835*G0_2_2;
    A[1773] = 0.0567586830607659*G0_0_0 + 0.0283793415303828*G0_0_1 + 0.0283793415303833*G0_0_2 + 0.0283793415303823*G0_1_0 + 0.014189670765191*G0_1_2 + 0.0283793415303826*G0_2_0 + 0.0141896707651911*G0_2_1 - 0.00489298991903181*G0_2_2;
    A[1774] = 0.149725491522365*G0_0_0 + 0.0283793415303837*G0_0_1 + 0.0748627457611832*G0_0_2 + 0.0283793415303826*G0_1_0 + 0.0141896707651916*G0_1_2 + 0.0748627457611823*G0_2_0 + 0.0141896707651919*G0_2_1 + 0.0195719596761266*G0_2_2;
    A[1775] = 0.20941996853455*G0_0_0 + 0.104709984267274*G0_0_1 + 0.0737210481134084*G0_0_2 + 0.104709984267276*G0_1_0 + 0.036860524056705*G0_1_2 + 0.073721048113409*G0_2_0 + 0.0368605240567044*G0_2_1 - 0.0228339529554806*G0_2_2;
    A[1776] = -0.0880738185425679*G0_0_0 - 0.10079559233205*G0_0_1 - 0.0887262171984385*G0_0_2 - 0.100795592332049*G0_1_0 - 0.113517366121531*G0_1_1 - 0.100795592332049*G0_1_2 - 0.0887262171984379*G0_2_0 - 0.100795592332049*G0_2_1 - 0.088073818542567*G0_2_2;
    A[1777] = 0.0195719596761266*G0_0_0 + 0.0748627457611831*G0_0_1 + 0.0606730749959916*G0_0_2 + 0.0748627457611824*G0_1_0 + 0.149725491522365*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0606730749959905*G0_2_0 + 0.121346149991982*G0_2_1 + 0.0929668084615989*G0_2_2;
    A[1778] = -0.0880738185425682*G0_0_0 + 0.0459941052388964*G0_0_1 + 0.00913358118219217*G0_0_2 + 0.0459941052388958*G0_1_0 + 0.023486351611351*G0_1_1 - 0.0812236326559225*G0_1_2 + 0.00913358118219169*G0_2_0 - 0.0812236326559237*G0_2_1 - 0.185933616923198*G0_2_2;
    A[1779] = 0.0929668084615992*G0_0_0 + 0.121346149991982*G0_0_1 + 0.0606730749959906*G0_0_2 + 0.121346149991982*G0_1_0 + 0.149725491522364*G0_1_1 + 0.0748627457611813*G0_1_2 + 0.0606730749959904*G0_2_0 + 0.0748627457611818*G0_2_1 + 0.0195719596761257*G0_2_2;
    A[1780] = 0.0929668084615998*G0_0_0 - 0.270093043530541*G0_0_1 - 0.135046521765271*G0_0_2 - 0.27009304353054*G0_1_0 - 0.540186087061081*G0_1_1 - 0.270093043530542*G0_1_2 - 0.135046521765269*G0_2_0 - 0.270093043530539*G0_2_1 + 0.0929668084615998*G0_2_2;
    A[1781] = -0.185933616923199*G0_0_0 - 0.0812236326559233*G0_0_1 + 0.0091335811821926*G0_0_2 - 0.0812236326559237*G0_1_0 + 0.0234863516113524*G0_1_1 + 0.0459941052388983*G0_1_2 + 0.00913358118219185*G0_2_0 + 0.0459941052388967*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[1782] = -0.0228339529554807*G0_0_0 - 0.0120693751336115*G0_0_1 - 0.0241387502672224*G0_0_2 - 0.0120693751336112*G0_1_0 - 0.0127217737894819*G0_1_2 - 0.024138750267222*G0_2_0 - 0.0127217737894816*G0_2_1 - 0.0254435475789634*G0_2_2;
    A[1783] = -0.00489298991903107*G0_0_0 + 0.0141896707651919*G0_0_1 + 0.0283793415303836*G0_0_2 + 0.0141896707651917*G0_1_0 + 0.0283793415303828*G0_1_2 + 0.0283793415303829*G0_2_0 + 0.0283793415303832*G0_2_1 + 0.056758683060766*G0_2_2;
    A[1784] = -0.0228339529554809*G0_0_0 + 0.0368605240567039*G0_0_1 + 0.073721048113408*G0_0_2 + 0.0368605240567034*G0_1_0 + 0.104709984267274*G0_1_2 + 0.0737210481134077*G0_2_0 + 0.104709984267275*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1785] = 0.019571959676126*G0_0_0 - 0.0102752788299657*G0_0_1 + 0.00146789697570963*G0_0_2 - 0.0102752788299663*G0_1_0 - 0.0181040627004165*G0_1_2 + 0.00146789697570936*G0_2_0 - 0.0181040627004171*G0_2_1 - 0.016636165724707*G0_2_2;
    A[1786] = 0.0195719596761261*G0_0_0 + 0.0141896707651917*G0_0_1 + 0.0748627457611828*G0_0_2 + 0.0141896707651917*G0_1_0 + 0.0283793415303837*G0_1_2 + 0.0748627457611826*G0_2_0 + 0.0283793415303826*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1787] = 0.0260959462348347*G0_0_0 - 0.0120693751336122*G0_0_1 + 0.0198981590040607*G0_0_2 - 0.0120693751336114*G0_1_0 - 0.0127217737894826*G0_1_2 + 0.0198981590040619*G0_2_0 - 0.0127217737894817*G0_2_1 + 0.0137003717732885*G0_2_2;
    A[1788] = 0.146789697570946*G0_0_0 + 0.0929668084615985*G0_0_1 + 0.141896707651914*G0_0_2 + 0.0929668084615992*G0_1_0 + 0.0929668084615986*G0_1_2 + 0.141896707651914*G0_2_0 + 0.0929668084615988*G0_2_1 + 0.146789697570945*G0_2_2;
    A[1789] = 0.146789697570945*G0_0_0 - 0.102752788299663*G0_0_1 - 0.298472385060925*G0_0_2 - 0.102752788299662*G0_1_0 - 0.298472385060923*G0_1_2 - 0.298472385060923*G0_2_0 - 0.298472385060924*G0_2_1 - 0.782878387045045*G0_2_2;
    A[1790] = -0.0489298991903137*G0_0_0 + 0.0929668084616004*G0_0_1 - 0.0293579395141888*G0_0_2 + 0.0929668084616005*G0_1_0 + 0.0929668084615992*G0_1_2 - 0.0293579395141897*G0_2_0 + 0.0929668084615992*G0_2_1 - 0.0489298991903165*G0_2_2;
    A[1791] = -0.782878387045047*G0_0_0 - 0.298472385060925*G0_0_1 - 0.298472385060924*G0_0_2 - 0.298472385060924*G0_1_0 - 0.102752788299662*G0_1_2 - 0.298472385060924*G0_2_0 - 0.102752788299662*G0_2_1 + 0.146789697570947*G0_2_2;
    A[1792] = -0.00946499969937457*G0_0_0 - 0.00946499969937454*G0_0_1 - 0.00946499969937453*G0_0_2 - 0.00946499969937457*G0_1_0 - 0.00946499969937457*G0_1_1 - 0.00946499969937456*G0_1_2 - 0.00946499969937458*G0_2_0 - 0.00946499969937457*G0_2_1 - 0.00946499969937459*G0_2_2;
    A[1793] = 0.00946499969937471*G0_0_0 - 0.00227817610630112*G0_1_0 - 0.00227817610630103*G0_2_0;
    A[1794] = -0.0047246710658167*G0_0_1 - 0.0345719095719093*G0_1_1 - 0.00472467106581673*G0_2_1;
    A[1795] = -0.00472467106581683*G0_0_2 - 0.00472467106581674*G0_1_2 - 0.0345719095719092*G0_2_2;
    A[1796] = -0.0493213383838383*G0_0_1 - 0.0429604514890968*G0_0_2 + 0.1273155976932*G0_1_1 - 0.000685018588664536*G0_1_2 - 0.00704590548340569*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[1797] = 0.167731694424402*G0_0_1 + 0.0777006799142205*G0_0_2 - 0.166948816037356*G0_1_1 - 0.00616516729797982*G0_1_2 + 0.0838658472122008*G0_2_1 + 0.182410664181495*G0_2_2;
    A[1798] = 0.0777006799142207*G0_0_1 + 0.167731694424402*G0_0_2 + 0.182410664181495*G0_1_1 + 0.0838658472122004*G0_1_2 - 0.00616516729798061*G0_2_1 - 0.166948816037357*G0_2_2;
    A[1799] = -0.0429604514890968*G0_0_1 - 0.0493213383838381*G0_0_2 - 0.0429604514890964*G0_1_1 - 0.00704590548340545*G0_1_2 - 0.000685018588663634*G0_2_1 + 0.127315597693201*G0_2_2;
    A[1800] = -0.0292600797158087*G0_0_0 - 0.00329461321214766*G0_0_2 + 0.0142875305635719*G0_1_0 + 0.00254435475789634*G0_1_2 + 0.0084485625935274*G0_2_0 + 0.00254435475789637*G0_2_2;
    A[1801] = 0.0189848008858427*G0_0_0 + 0.00949240044292111*G0_0_2 - 0.0377738821749233*G0_1_0 - 0.00939454064454027*G0_1_2 - 0.0188869410874611*G0_2_0 - 0.0155923278753138*G0_2_2;
    A[1802] = -0.00352295274170279*G0_0_0 - 0.00649136662591513*G0_0_2 - 0.108232937008978*G0_1_0 - 0.0808321934624011*G0_1_2 - 0.0338921101724925*G0_2_0 - 0.0436454700777616*G0_2_2;
    A[1803] = -0.0429604514890969*G0_0_0 - 0.0422754329004325*G0_0_2 - 0.0429604514890965*G0_1_0 - 0.0845508658008648*G0_1_2 - 0.000685018588663687*G0_2_0 + 0.0439716694056974*G0_2_2;
    A[1804] = -0.0292600797158087*G0_0_0 - 0.00329461321214767*G0_0_1 + 0.00844856259352788*G0_1_0 + 0.00254435475789649*G0_1_1 + 0.0142875305635719*G0_2_0 + 0.00254435475789637*G0_2_1;
    A[1805] = 0.0189848008858424*G0_0_0 + 0.00949240044292091*G0_0_1 - 0.0188869410874619*G0_1_0 - 0.0155923278753139*G0_1_1 - 0.0377738821749235*G0_2_0 - 0.00939454064454069*G0_2_1;
    A[1806] = -0.00352295274170235*G0_0_0 - 0.00649136662591467*G0_0_1 - 0.0338921101724912*G0_1_0 - 0.0436454700777609*G0_1_1 - 0.108232937008978*G0_2_0 - 0.080832193462401*G0_2_1;
    A[1807] = -0.0429604514890967*G0_0_0 - 0.0422754329004325*G0_0_1 - 0.000685018588664455*G0_1_0 + 0.0439716694056968*G0_1_1 - 0.0429604514890967*G0_2_0 - 0.0845508658008649*G0_2_1;
    A[1808] = 0.0292600797158085*G0_0_0 + 0.0292600797158084*G0_0_1 + 0.0259654665036604*G0_0_2 + 0.0292600797158086*G0_1_0 + 0.0292600797158087*G0_1_1 + 0.0259654665036606*G0_1_2 + 0.0259654665036607*G0_2_0 + 0.0259654665036606*G0_2_1 + 0.0226708532915127*G0_2_2;
    A[1809] = -0.0189848008858421*G0_0_0 - 0.018984800885842*G0_0_1 - 0.0094924004429208*G0_0_2 - 0.0189848008858422*G0_1_0 - 0.0189848008858424*G0_1_1 - 0.00949240044292117*G0_1_2 - 0.00949240044292125*G0_2_0 - 0.00949240044292119*G0_2_1 - 0.000652398655871127*G0_2_2;
    A[1810] = 0.00352295274170281*G0_0_0 + 0.00352295274170278*G0_0_1 - 0.00296841388421277*G0_0_2 + 0.00352295274170329*G0_1_0 + 0.00352295274170331*G0_1_1 - 0.00296841388421238*G0_1_2 - 0.00296841388421203*G0_2_0 - 0.00296841388421207*G0_2_1 - 0.000652398655870552*G0_2_2;
    A[1811] = 0.0429604514890968*G0_0_0 + 0.0429604514890969*G0_0_1 + 0.000685018588664491*G0_0_2 + 0.0429604514890965*G0_1_0 + 0.0429604514890965*G0_1_1 + 0.000685018588664225*G0_1_2 + 0.000685018588663695*G0_2_0 + 0.000685018588663727*G0_2_1 + 0.0226708532915127*G0_2_2;
    A[1812] = 0.0292600797158087*G0_0_0 + 0.0259654665036606*G0_0_1 + 0.0292600797158087*G0_0_2 + 0.0259654665036607*G0_1_0 + 0.0226708532915128*G0_1_1 + 0.0259654665036608*G0_1_2 + 0.0292600797158086*G0_2_0 + 0.0259654665036606*G0_2_1 + 0.0292600797158086*G0_2_2;
    A[1813] = -0.0189848008858426*G0_0_0 - 0.00949240044292131*G0_0_1 - 0.0189848008858426*G0_0_2 - 0.00949240044292125*G0_1_0 - 0.000652398655870921*G0_1_1 - 0.00949240044292135*G0_1_2 - 0.0189848008858424*G0_2_0 - 0.0094924004429212*G0_2_1 - 0.0189848008858424*G0_2_2;
    A[1814] = 0.00352295274170326*G0_0_0 - 0.00296841388421216*G0_0_1 + 0.00352295274170333*G0_0_2 - 0.00296841388421194*G0_1_0 - 0.00065239865587075*G0_1_1 - 0.00296841388421187*G0_1_2 + 0.00352295274170332*G0_2_0 - 0.00296841388421231*G0_2_1 + 0.00352295274170344*G0_2_2;
    A[1815] = 0.0429604514890967*G0_0_0 + 0.000685018588664078*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.000685018588664463*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.000685018588664536*G0_1_2 + 0.0429604514890967*G0_2_0 + 0.000685018588664173*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[1816] = 0.00329461321214781*G0_0_1 + 0.00329461321214786*G0_0_2 + 0.00329461321214775*G0_1_0 + 0.00658922642429584*G0_1_1 + 0.00658922642429586*G0_1_2 + 0.00329461321214785*G0_2_0 + 0.0065892264242957*G0_2_1 + 0.00658922642429578*G0_2_2;
    A[1817] = 0.00257697469068988*G0_0_1 + 0.00257697469068984*G0_0_2 + 0.00257697469069018*G0_1_0 + 0.00450155072550894*G0_1_1 + 0.00450155072550897*G0_1_2 + 0.00257697469069006*G0_2_0 + 0.00450155072550889*G0_2_1 + 0.00450155072550893*G0_2_2;
    A[1818] = 0.00257697469069014*G0_0_1 + 0.00257697469069014*G0_0_2 + 0.00257697469068994*G0_1_0 - 0.0105036183595209*G0_1_1 - 0.0105036183595208*G0_1_2 + 0.00257697469068988*G0_2_0 - 0.0105036183595212*G0_2_1 - 0.0105036183595211*G0_2_2;
    A[1819] = 0.00329461321214768*G0_0_1 + 0.00329461321214768*G0_0_2 + 0.00329461321214783*G0_1_0 - 0.00254435475789629*G0_1_1 - 0.00254435475789637*G0_1_2 + 0.00329461321214784*G0_2_0 - 0.00254435475789634*G0_2_1 - 0.00254435475789636*G0_2_2;
    A[1820] = 0.113517366121532*G0_0_0 + 0.0127217737894821*G0_0_1 + 0.0127217737894819*G0_0_2 - 0.0851380245911486*G0_1_0 - 0.0137003717732887*G0_1_1 - 0.0198981590040618*G0_1_2 - 0.0851380245911487*G0_2_0 - 0.0198981590040618*G0_2_1 - 0.0137003717732891*G0_2_2;
    A[1821] = -0.149725491522365*G0_0_0 - 0.0748627457611826*G0_0_1 - 0.0283793415303825*G0_0_2 + 0.0278900425384809*G0_1_0 + 0.0655660649150238*G0_1_1 - 0.0748627457611813*G0_1_2 + 0.177126235068943*G0_2_0 + 0.0278900425384805*G0_2_1 - 0.149725491522364*G0_2_2;
    A[1822] = -0.0234863516113521*G0_0_0 + 0.0225077536275451*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0459941052388973*G0_1_0 - 0.268135847562929*G0_1_1 - 0.0737210481134087*G0_1_2 + 0.0812236326559232*G0_2_0 + 0.180714427676231*G0_2_1 - 0.20941996853455*G0_2_2;
    A[1823] = -0.149725491522365*G0_0_0 - 0.0283793415303826*G0_0_1 - 0.0748627457611823*G0_0_2 + 0.177126235068943*G0_1_0 - 0.149725491522364*G0_1_1 + 0.0278900425384805*G0_1_2 + 0.0278900425384793*G0_2_0 - 0.0748627457611828*G0_2_1 + 0.0655660649150237*G0_2_2;
    A[1824] = 0.540186087061083*G0_0_0 + 0.270093043530541*G0_0_1 + 0.27009304353054*G0_0_2 + 0.270093043530541*G0_1_0 + 0.726119703984281*G0_1_1 + 0.270093043530541*G0_1_2 + 0.27009304353054*G0_2_0 + 0.270093043530541*G0_2_1 + 0.726119703984281*G0_2_2;
    A[1825] = -0.0234863516113516*G0_0_0 - 0.104709984267275*G0_0_1 + 0.0225077536275453*G0_0_2 + 0.0812236326559228*G0_1_0 - 0.20941996853455*G0_1_1 + 0.180714427676232*G0_1_2 - 0.0459941052388963*G0_2_0 - 0.0737210481134081*G0_2_1 - 0.26813584756293*G0_2_2;
    A[1826] = -0.113517366121531*G0_0_0 - 0.100795592332049*G0_0_1 - 0.100795592332049*G0_0_2 - 0.100795592332049*G0_1_0 - 0.0880738185425677*G0_1_1 - 0.0887262171984383*G0_1_2 - 0.100795592332049*G0_2_0 - 0.088726217198438*G0_2_1 - 0.0880738185425672*G0_2_2;
    A[1827] = 0.149725491522365*G0_0_0 + 0.0748627457611819*G0_0_1 + 0.121346149991982*G0_0_2 + 0.074862745761182*G0_1_0 + 0.0195719596761266*G0_1_1 + 0.0606730749959905*G0_1_2 + 0.121346149991982*G0_2_0 + 0.0606730749959909*G0_2_1 + 0.0929668084615984*G0_2_2;
    A[1828] = 0.023486351611352*G0_0_0 + 0.045994105238897*G0_0_1 - 0.0812236326559235*G0_0_2 + 0.0459941052388974*G0_1_0 - 0.0880738185425679*G0_1_1 + 0.0091335811821918*G0_1_2 - 0.0812236326559232*G0_2_0 + 0.00913358118219226*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1829] = 0.149725491522365*G0_0_0 + 0.121346149991981*G0_0_1 + 0.0748627457611822*G0_0_2 + 0.121346149991982*G0_1_0 + 0.0929668084615982*G0_1_1 + 0.0606730749959909*G0_1_2 + 0.0748627457611824*G0_2_0 + 0.0606730749959901*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[1830] = -0.540186087061083*G0_0_0 - 0.270093043530541*G0_0_1 - 0.270093043530541*G0_0_2 - 0.270093043530541*G0_1_0 + 0.0929668084615999*G0_1_1 - 0.135046521765269*G0_1_2 - 0.27009304353054*G0_2_0 - 0.13504652176527*G0_2_1 + 0.0929668084616012*G0_2_2;
    A[1831] = 0.0234863516113518*G0_0_0 - 0.0812236326559233*G0_0_1 + 0.0459941052388967*G0_0_2 - 0.0812236326559227*G0_1_0 - 0.185933616923198*G0_1_1 + 0.00913358118219214*G0_1_2 + 0.0459941052388963*G0_2_0 + 0.00913358118219277*G0_2_1 - 0.0880738185425688*G0_2_2;
    A[1832] = -0.0127217737894816*G0_0_1 - 0.0120693751336106*G0_0_2 - 0.0127217737894818*G0_1_0 - 0.0254435475789635*G0_1_1 - 0.0241387502672218*G0_1_2 - 0.0120693751336112*G0_2_0 - 0.0241387502672219*G0_2_1 - 0.0228339529554802*G0_2_2;
    A[1833] = -0.0181040627004173*G0_0_1 - 0.0102752788299666*G0_0_2 - 0.0181040627004173*G0_1_0 - 0.0166361657247074*G0_1_1 + 0.0014678969757092*G0_1_2 - 0.0102752788299663*G0_2_0 + 0.00146789697570953*G0_2_1 + 0.0195719596761266*G0_2_2;
    A[1834] = -0.0127217737894822*G0_0_1 - 0.0120693751336113*G0_0_2 - 0.0127217737894815*G0_1_0 + 0.0137003717732883*G0_1_1 + 0.0198981590040612*G0_1_2 - 0.0120693751336106*G0_2_0 + 0.0198981590040618*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[1835] = 0.0283793415303833*G0_0_1 + 0.0141896707651911*G0_0_2 + 0.028379341530383*G0_1_0 + 0.0567586830607669*G0_1_1 + 0.0283793415303828*G0_1_2 + 0.0141896707651914*G0_2_0 + 0.0283793415303837*G0_2_1 - 0.00489298991903172*G0_2_2;
    A[1836] = 0.0283793415303827*G0_0_1 + 0.0141896707651917*G0_0_2 + 0.0283793415303824*G0_1_0 + 0.149725491522364*G0_1_1 + 0.0748627457611828*G0_1_2 + 0.0141896707651909*G0_2_0 + 0.0748627457611827*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[1837] = 0.104709984267275*G0_0_1 + 0.0368605240567042*G0_0_2 + 0.104709984267275*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0737210481134079*G0_1_2 + 0.0368605240567045*G0_2_0 + 0.0737210481134081*G0_2_1 - 0.0228339529554814*G0_2_2;
    A[1838] = -0.0120693751336111*G0_0_1 - 0.0127217737894821*G0_0_2 - 0.0120693751336112*G0_1_0 - 0.0228339529554806*G0_1_1 - 0.0241387502672223*G0_1_2 - 0.0127217737894821*G0_2_0 - 0.0241387502672221*G0_2_1 - 0.025443547578964*G0_2_2;
    A[1839] = -0.0102752788299663*G0_0_1 - 0.018104062700417*G0_0_2 - 0.0102752788299663*G0_1_0 + 0.0195719596761261*G0_1_1 + 0.0014678969757096*G0_1_2 - 0.0181040627004166*G0_2_0 + 0.00146789697570961*G0_2_1 - 0.0166361657247074*G0_2_2;
    A[1840] = -0.0120693751336111*G0_0_1 - 0.012721773789482*G0_0_2 - 0.012069375133611*G0_1_0 + 0.0260959462348345*G0_1_1 + 0.0198981590040616*G0_1_2 - 0.0127217737894817*G0_2_0 + 0.019898159004062*G0_2_1 + 0.0137003717732888*G0_2_2;
    A[1841] = 0.014189670765192*G0_0_1 + 0.0283793415303838*G0_0_2 + 0.0141896707651918*G0_1_0 - 0.004892989919031*G0_1_1 + 0.0283793415303837*G0_1_2 + 0.0283793415303832*G0_2_0 + 0.0283793415303836*G0_2_1 + 0.0567586830607672*G0_2_2;
    A[1842] = 0.0141896707651913*G0_0_1 + 0.0283793415303828*G0_0_2 + 0.0141896707651913*G0_1_0 + 0.0195719596761259*G0_1_1 + 0.0748627457611816*G0_1_2 + 0.0283793415303828*G0_2_0 + 0.0748627457611825*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1843] = 0.0368605240567037*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0368605240567032*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.0737210481134086*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0737210481134077*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1844] = 0.092966808461599*G0_0_1 + 0.0929668084615993*G0_0_2 + 0.092966808461599*G0_1_0 + 0.146789697570945*G0_1_1 + 0.141896707651914*G0_1_2 + 0.0929668084615991*G0_2_0 + 0.141896707651914*G0_2_1 + 0.146789697570946*G0_2_2;
    A[1845] = 0.0929668084615999*G0_0_1 + 0.0929668084615996*G0_0_2 + 0.0929668084615986*G0_1_0 - 0.0489298991903158*G0_1_1 - 0.0293579395141898*G0_1_2 + 0.0929668084615985*G0_2_0 - 0.0293579395141899*G0_2_1 - 0.0489298991903155*G0_2_2;
    A[1846] = -0.102752788299663*G0_0_1 - 0.298472385060924*G0_0_2 - 0.102752788299663*G0_1_0 + 0.146789697570947*G0_1_1 - 0.298472385060924*G0_1_2 - 0.298472385060925*G0_2_0 - 0.298472385060924*G0_2_1 - 0.782878387045048*G0_2_2;
    A[1847] = -0.298472385060924*G0_0_1 - 0.102752788299662*G0_0_2 - 0.298472385060925*G0_1_0 - 0.782878387045047*G0_1_1 - 0.298472385060924*G0_1_2 - 0.102752788299662*G0_2_0 - 0.298472385060924*G0_2_1 + 0.146789697570947*G0_2_2;
    A[1848] = -0.0137421252872641*G0_0_0 - 0.013742125287264*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.013742125287264*G0_1_1 - 0.0137421252872641*G0_1_2 - 0.0137421252872641*G0_2_0 - 0.0137421252872641*G0_2_1 - 0.013742125287264*G0_2_2;
    A[1849] = 0.0137421252872639*G0_0_0 - 0.00517743573299147*G0_1_0 - 0.00517743573299141*G0_2_0;
    A[1850] = -0.00517743573299126*G0_0_1 + 0.013742125287264*G0_1_1 - 0.0051774357329913*G0_2_1;
    A[1851] = 0.0143945239431349*G0_0_2 + 0.0143945239431349*G0_1_2 + 0.0479930547204849*G0_2_2;
    A[1852] = 0.0477555816097474*G0_0_1 + 0.0118736555368495*G0_0_2 - 0.0641307878721068*G0_1_1 - 0.00939454064454059*G0_1_2 + 0.0264873854283577*G0_2_1 + 0.01187365553685*G0_2_2;
    A[1853] = -0.0823327103709032*G0_0_1 + 0.00182671623643896*G0_0_2 + 0.127087258163647*G0_1_1 + 0.037773882174924*G0_1_2 - 0.0463855444324188*G0_2_1 - 0.051017574889102*G0_2_2;
    A[1854] = 0.146659217839772*G0_0_1 + 0.0109602974186304*G0_0_2 - 0.124738623002511*G0_1_1 - 0.0623693115012556*G0_1_2 + 0.0733296089198861*G0_2_1 + 0.132306447410613*G0_2_2;
    A[1855] = 0.124738623002511*G0_0_1 + 0.192588083213082*G0_0_2 + 0.124738623002512*G0_1_1 + 0.0962940416065417*G0_1_2 + 0.02844458139597*G0_2_1 - 0.0644569872000426*G0_2_2;
    A[1856] = -0.0641307878721065*G0_0_0 - 0.00939454064454063*G0_0_2 + 0.047755581609748*G0_1_0 + 0.0118736555368494*G0_1_2 + 0.0264873854283575*G0_2_0 + 0.0118736555368495*G0_2_2;
    A[1857] = 0.127087258163646*G0_0_0 + 0.0377738821749238*G0_0_2 - 0.0823327103709043*G0_1_0 + 0.00182671623643922*G0_1_2 - 0.0463855444324192*G0_2_0 - 0.0510175748891016*G0_2_2;
    A[1858] = -0.124738623002511*G0_0_0 - 0.0623693115012557*G0_0_2 + 0.146659217839772*G0_1_0 + 0.0109602974186298*G0_1_2 + 0.0733296089198866*G0_2_0 + 0.132306447410613*G0_2_2;
    A[1859] = 0.124738623002511*G0_0_0 + 0.0962940416065416*G0_0_2 + 0.124738623002512*G0_1_0 + 0.192588083213083*G0_1_2 + 0.02844458139597*G0_2_0 - 0.0644569872000423*G0_2_2;
    A[1860] = -0.0187890812890812*G0_0_0 - 0.00939454064454074*G0_0_1 + 0.0212681961813901*G0_1_0 + 0.0118736555368493*G0_1_1 + 0.0425363923627806*G0_2_0 + 0.0118736555368495*G0_2_1;
    A[1861] = 0.0148746893538561*G0_0_0 + 0.014939929219443*G0_0_1 + 0.00123955744615497*G0_1_0 + 0.00117431758056787*G0_1_1 + 0.0403182369328202*G0_2_0 + 0.0540186087061087*G0_2_1;
    A[1862] = 0.00117431758056761*G0_0_0 + 0.00123955744615494*G0_0_1 + 0.0149399292194427*G0_1_0 + 0.0148746893538555*G0_1_1 + 0.0540186087061083*G0_2_0 + 0.0403182369328197*G0_2_1;
    A[1863] = 0.0118736555368495*G0_0_0 + 0.0212681961813898*G0_0_1 - 0.00939454064454066*G0_1_0 - 0.0187890812890812*G0_1_1 + 0.01187365553685*G0_2_0 + 0.0425363923627811*G0_2_1;
    A[1864] = 0.0641307878721067*G0_0_0 + 0.0641307878721067*G0_0_1 + 0.0547362472275663*G0_0_2 + 0.0641307878721068*G0_1_0 + 0.0641307878721067*G0_1_1 + 0.0547362472275662*G0_1_2 + 0.0547362472275662*G0_2_0 + 0.0547362472275663*G0_2_1 + 0.0453417065830256*G0_2_2;
    A[1865] = -0.127087258163646*G0_0_0 - 0.127087258163646*G0_0_1 - 0.0893133759887224*G0_0_2 - 0.127087258163646*G0_1_0 - 0.127087258163646*G0_1_1 - 0.0893133759887224*G0_1_2 - 0.0893133759887223*G0_2_0 - 0.0893133759887223*G0_2_1 - 0.0665446628988286*G0_2_2;
    A[1866] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0623693115012553*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0623693115012553*G0_1_2 + 0.0623693115012556*G0_2_0 + 0.0623693115012556*G0_2_1 + 0.0476251018785734*G0_2_2;
    A[1867] = -0.124738623002511*G0_0_0 - 0.124738623002511*G0_0_1 - 0.02844458139597*G0_0_2 - 0.124738623002512*G0_1_0 - 0.124738623002512*G0_1_1 - 0.02844458139597*G0_1_2 - 0.02844458139597*G0_2_0 - 0.02844458139597*G0_2_1 - 0.0606730749959912*G0_2_2;
    A[1868] = 0.0187890812890811*G0_0_0 + 0.00939454064454063*G0_0_1 + 0.0187890812890811*G0_0_2 + 0.00939454064454062*G0_1_0 + 0.00939454064454054*G0_1_2 + 0.0187890812890811*G0_2_0 + 0.00939454064454049*G0_2_1 + 0.0187890812890811*G0_2_2;
    A[1869] = -0.0148746893538557*G0_0_0 + 6.5239865587016e-05*G0_0_1 - 0.0148746893538556*G0_0_2 + 6.52398655872489e-05*G0_1_0 + 6.52398655872703e-05*G0_1_2 - 0.0148746893538561*G0_2_0 + 6.5239865587004e-05*G0_2_1 - 0.0148746893538562*G0_2_2;
    A[1870] = -0.00117431758056765*G0_0_0 + 6.52398655871987e-05*G0_0_1 - 0.00117431758056763*G0_0_2 + 6.52398655870681e-05*G0_1_0 + 6.52398655870821e-05*G0_1_2 - 0.00117431758056721*G0_2_0 + 6.52398655872025e-05*G0_2_1 - 0.00117431758056717*G0_2_2;
    A[1871] = -0.0118736555368495*G0_0_0 + 0.00939454064454051*G0_0_1 - 0.0118736555368495*G0_0_2 + 0.00939454064454069*G0_1_0 + 0.00939454064454064*G0_1_2 - 0.01187365553685*G0_2_0 + 0.00939454064454059*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1872] = 0.00939454064454055*G0_0_1 + 0.00939454064454047*G0_0_2 + 0.0093945406445406*G0_1_0 + 0.0187890812890813*G0_1_1 + 0.0187890812890812*G0_1_2 + 0.00939454064454045*G0_2_0 + 0.0187890812890812*G0_2_1 + 0.0187890812890811*G0_2_2;
    A[1873] = 6.52398655871676e-05*G0_0_1 + 6.52398655872001e-05*G0_0_2 + 6.52398655871007e-05*G0_1_0 - 0.0148746893538557*G0_1_1 - 0.0148746893538556*G0_1_2 + 6.52398655872906e-05*G0_2_0 - 0.0148746893538557*G0_2_1 - 0.0148746893538557*G0_2_2;
    A[1874] = 6.52398655871287e-05*G0_0_1 + 6.52398655871354e-05*G0_0_2 + 6.52398655869407e-05*G0_1_0 - 0.00117431758056771*G0_1_1 - 0.00117431758056772*G0_1_2 + 6.52398655869857e-05*G0_2_0 - 0.00117431758056758*G0_2_1 - 0.00117431758056767*G0_2_2;
    A[1875] = 0.00939454064454084*G0_0_1 + 0.00939454064454079*G0_0_2 + 0.00939454064454091*G0_1_0 - 0.0118736555368493*G0_1_1 - 0.0118736555368492*G0_1_2 + 0.00939454064454075*G0_2_0 - 0.0118736555368495*G0_2_1 - 0.0118736555368495*G0_2_2;
    A[1876] = 0.105688582251081*G0_0_0 + 0.0528442911255409*G0_0_1 + 0.0300103381700598*G0_0_2 + 0.00065239865587079*G0_1_0 + 0.0534966897814119*G0_1_1 + 0.05284429112554*G0_1_2 - 0.0743734467692791*G0_2_0 + 0.000652398655870899*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1877] = -0.0137003717732884*G0_0_0 - 0.0198981590040619*G0_0_1 + 0.0127217737894825*G0_0_2 - 0.0198981590040627*G0_1_0 - 0.0137003717732889*G0_1_1 + 0.0127217737894817*G0_1_2 - 0.0851380245911501*G0_2_0 - 0.0851380245911492*G0_2_1 + 0.113517366121532*G0_2_2;
    A[1878] = 0.0534966897814114*G0_0_0 + 0.000652398655871054*G0_0_1 + 0.0528442911255396*G0_0_2 + 0.0528442911255414*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0300103381700596*G0_1_2 + 0.000652398655871439*G0_2_0 - 0.0743734467692795*G0_2_1 + 0.105688582251082*G0_2_2;
    A[1879] = -0.20941996853455*G0_0_0 - 0.104709984267276*G0_0_1 - 0.073721048113409*G0_0_2 + 0.0812236326559233*G0_1_0 - 0.0234863516113524*G0_1_1 - 0.0459941052388966*G0_1_2 + 0.180714427676232*G0_2_0 + 0.022507753627546*G0_2_1 - 0.268135847562929*G0_2_2;
    A[1880] = -0.0234863516113516*G0_0_0 + 0.0812236326559228*G0_0_1 - 0.0459941052388963*G0_0_2 - 0.104709984267275*G0_1_0 - 0.20941996853455*G0_1_1 - 0.0737210481134081*G0_1_2 + 0.0225077536275453*G0_2_0 + 0.180714427676232*G0_2_1 - 0.26813584756293*G0_2_2;
    A[1881] = 0.542795681684567*G0_0_0 + 0.271397840842285*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842285*G0_1_0 + 0.542795681684568*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842284*G0_2_1 + 0.647179466623907*G0_2_2;
    A[1882] = -0.105688582251081*G0_0_0 - 0.0528442911255405*G0_0_1 - 0.0756782440810214*G0_0_2 - 0.052844291125541*G0_1_0 - 0.0378391220405109*G0_1_2 - 0.0756782440810216*G0_2_0 - 0.0378391220405108*G0_2_1 - 0.045667905910961*G0_2_2;
    A[1883] = 0.0137003717732886*G0_0_0 - 0.00619778723077315*G0_0_1 + 0.0264221455627703*G0_0_2 - 0.00619778723077294*G0_1_0 - 0.00554538857490256*G0_1_2 + 0.0264221455627703*G0_2_0 - 0.00554538857490179*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[1884] = -0.0534966897814111*G0_0_0 - 0.0528442911255406*G0_0_1 - 0.000652398655871463*G0_0_2 - 0.0528442911255412*G0_1_0 - 0.0378391220405113*G0_1_2 - 0.000652398655871304*G0_2_0 - 0.0378391220405113*G0_2_1 + 0.0521918924696694*G0_2_2;
    A[1885] = 0.20941996853455*G0_0_0 + 0.104709984267274*G0_0_1 + 0.135698920421142*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105714*G0_1_2 + 0.135698920421141*G0_2_0 + 0.0678494602105703*G0_2_1 + 0.039143919352252*G0_2_2;
    A[1886] = 0.0234863516113515*G0_0_0 + 0.104709984267275*G0_0_1 - 0.0225077536275446*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105712*G0_1_2 - 0.0225077536275454*G0_2_0 + 0.0678494602105709*G0_2_1 - 0.15657567740901*G0_2_2;
    A[1887] = -0.542795681684566*G0_0_0 - 0.271397840842282*G0_0_1 - 0.271397840842284*G0_0_2 - 0.271397840842285*G0_1_0 - 0.135698920421142*G0_1_2 - 0.271397840842283*G0_2_0 - 0.135698920421141*G0_2_1 + 0.0521918924696706*G0_2_2;
    A[1888] = -0.0528442911255405*G0_0_1 - 0.0378391220405107*G0_0_2 - 0.0528442911255407*G0_1_0 - 0.105688582251082*G0_1_1 - 0.075678244081022*G0_1_2 - 0.0378391220405106*G0_2_0 - 0.0756782440810213*G0_2_1 - 0.045667905910961*G0_2_2;
    A[1889] = -0.00619778723077325*G0_0_1 - 0.00554538857490259*G0_0_2 - 0.00619778723077245*G0_1_0 + 0.013700371773289*G0_1_1 + 0.0264221455627706*G0_1_2 - 0.00554538857490221*G0_2_0 + 0.0264221455627706*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[1890] = -0.0528442911255409*G0_0_1 - 0.0378391220405112*G0_0_2 - 0.0528442911255417*G0_1_0 - 0.0534966897814119*G0_1_1 - 0.000652398655872018*G0_1_2 - 0.0378391220405109*G0_2_0 - 0.000652398655871069*G0_2_1 + 0.0521918924696691*G0_2_2;
    A[1891] = 0.104709984267275*G0_0_1 + 0.0678494602105713*G0_0_2 + 0.104709984267275*G0_1_0 + 0.20941996853455*G0_1_1 + 0.135698920421142*G0_1_2 + 0.0678494602105705*G0_2_0 + 0.135698920421142*G0_2_1 + 0.0391439193522519*G0_2_2;
    A[1892] = 0.104709984267276*G0_0_1 + 0.0678494602105713*G0_0_2 + 0.104709984267276*G0_1_0 + 0.0234863516113523*G0_1_1 - 0.0225077536275443*G0_1_2 + 0.0678494602105708*G0_2_0 - 0.0225077536275458*G0_2_1 - 0.15657567740901*G0_2_2;
    A[1893] = -0.271397840842285*G0_0_1 - 0.135698920421143*G0_0_2 - 0.271397840842283*G0_1_0 - 0.542795681684568*G0_1_1 - 0.271397840842285*G0_1_2 - 0.135698920421142*G0_2_0 - 0.271397840842283*G0_2_1 + 0.0521918924696701*G0_2_2;
    A[1894] = -0.0150051690850297*G0_0_1 - 0.0300103381700597*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0300103381700597*G0_1_2 - 0.03001033817006*G0_2_0 - 0.0300103381700604*G0_2_1 - 0.0600206763401204*G0_2_2;
    A[1895] = -0.000652398655871111*G0_0_1 - 0.0013047973117423*G0_0_2 - 0.000652398655870717*G0_1_0 - 0.012721773789482*G0_1_2 - 0.00130479731174189*G0_2_0 - 0.0127217737894823*G0_2_1 - 0.0254435475789643*G0_2_2;
    A[1896] = -0.0150051690850302*G0_0_1 - 0.0300103381700599*G0_0_2 - 0.0150051690850301*G0_1_0 - 0.05284429112554*G0_1_2 - 0.0300103381700598*G0_2_0 - 0.0528442911255411*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1897] = -0.000652398655871093*G0_0_1 - 0.0127217737894819*G0_0_2 - 0.00065239865587133*G0_1_0 - 0.00130479731174175*G0_1_2 - 0.0127217737894822*G0_2_0 - 0.00130479731174132*G0_2_1 - 0.0254435475789641*G0_2_2;
    A[1898] = -0.0006523986558709*G0_0_1 - 0.0127217737894824*G0_0_2 - 0.000652398655870878*G0_1_0 - 0.0127217737894817*G0_1_2 - 0.0127217737894819*G0_2_0 - 0.0127217737894814*G0_2_1 - 0.113517366121532*G0_2_2;
    A[1899] = -0.0150051690850296*G0_0_1 - 0.0528442911255397*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0300103381700597*G0_1_2 - 0.0528442911255411*G0_2_0 - 0.0300103381700606*G0_2_1 - 0.105688582251082*G0_2_2;
    A[1900] = 0.0260959462348347*G0_0_1 + 0.07502584542515*G0_0_2 + 0.026095946234835*G0_1_0 + 0.0750258454251498*G0_1_2 + 0.075025845425151*G0_2_0 + 0.0750258454251503*G0_2_1 + 0.195719596761262*G0_2_2;
    A[1901] = 0.026095946234835*G0_0_1 + 0.0750258454251514*G0_0_2 + 0.0260959462348353*G0_1_0 + 0.0587158790283785*G0_1_2 + 0.0750258454251497*G0_2_0 + 0.0587158790283788*G0_2_1 + 0.293579395141893*G0_2_2;
    A[1902] = 0.0260959462348351*G0_0_1 + 0.0587158790283782*G0_0_2 + 0.0260959462348355*G0_1_0 + 0.07502584542515*G0_1_2 + 0.0587158790283797*G0_2_0 + 0.0750258454251505*G0_2_1 + 0.293579395141894*G0_2_2;
    A[1903] = -0.185933616923198*G0_0_1 - 0.316413348097373*G0_0_2 - 0.185933616923198*G0_1_0 - 0.316413348097374*G0_1_2 - 0.316413348097374*G0_2_0 - 0.316413348097374*G0_2_1 - 0.782878387045049*G0_2_2;
    A[1904] = 0.0479930547204847*G0_0_0 + 0.0479930547204847*G0_0_1 + 0.0479930547204847*G0_0_2 + 0.0335985307773499*G0_1_0 + 0.0335985307773499*G0_1_1 + 0.0335985307773499*G0_1_2 + 0.0335985307773498*G0_2_0 + 0.0335985307773499*G0_2_1 + 0.0335985307773499*G0_2_2;
    A[1905] = -0.0137421252872641*G0_0_0;
    A[1906] = 0.00517743573299132*G0_0_1 + 0.0189195610202553*G0_1_1;
    A[1907] = 0.00517743573299131*G0_0_2 + 0.0189195610202554*G0_2_2;
    A[1908] = -0.0425363923627807*G0_0_1 - 0.0118736555368496*G0_0_2 - 0.0613254736518622*G0_1_1 - 0.0212681961813906*G0_1_2 - 0.02126819618139*G0_2_1;
    A[1909] = -0.0403182369328201*G0_0_1 - 0.0540186087061084*G0_0_2 - 0.0254435475789639*G0_1_1 - 0.0390786794866652*G0_1_2 - 0.0390786794866655*G0_2_1 - 0.0528442911255406*G0_2_2;
    A[1910] = -0.0540186087061083*G0_0_1 - 0.0403182369328201*G0_0_2 - 0.0528442911255407*G0_1_1 - 0.0390786794866655*G0_1_2 - 0.0390786794866652*G0_2_1 - 0.0254435475789642*G0_2_2;
    A[1911] = -0.0118736555368497*G0_0_1 - 0.0425363923627807*G0_0_2 - 0.0212681961813901*G0_1_2 - 0.0212681961813905*G0_2_1 - 0.0613254736518622*G0_2_2;
    A[1912] = 0.0187890812890811*G0_0_0 + 0.00939454064454043*G0_0_2 + 0.00939454064454059*G0_2_0;
    A[1913] = -0.0148746893538554*G0_0_0 - 0.0149399292194425*G0_0_2 - 0.0149399292194428*G0_2_0 - 0.0150051690850296*G0_2_2;
    A[1914] = -0.001174317580568*G0_0_0 - 0.00123955744615513*G0_0_2 - 0.00123955744615459*G0_2_0 - 0.00130479731174192*G0_2_2;
    A[1915] = -0.0118736555368496*G0_0_0 - 0.0212681961813902*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.0306627368259311*G0_2_2;
    A[1916] = 0.0187890812890812*G0_0_0 + 0.00939454064454051*G0_0_1 + 0.00939454064454081*G0_1_0;
    A[1917] = -0.0148746893538561*G0_0_0 - 0.0149399292194431*G0_0_1 - 0.0149399292194432*G0_1_0 - 0.0150051690850303*G0_1_1;
    A[1918] = -0.00117431758056746*G0_0_0 - 0.00123955744615464*G0_0_1 - 0.00123955744615421*G0_1_0 - 0.00130479731174134*G0_1_1;
    A[1919] = -0.0118736555368497*G0_0_0 - 0.0212681961813901*G0_0_1 - 0.0212681961813907*G0_1_0 - 0.0306627368259312*G0_1_1;
    A[1920] = -0.064456987200042*G0_0_0 - 0.064456987200042*G0_0_1 - 0.0929015685960119*G0_0_2 - 0.257045070413123*G0_1_0 - 0.257045070413123*G0_1_1 - 0.160751028806583*G0_1_2 - 0.160751028806582*G0_2_0 - 0.160751028806583*G0_2_1 - 0.0644569872000419*G0_2_2;
    A[1921] = 0.132306447410613*G0_0_0 + 0.132306447410613*G0_0_1 + 0.0589768384907264*G0_0_2 + 0.121346149991982*G0_1_0 + 0.121346149991982*G0_1_1 + 0.194675758911868*G0_1_2 + 0.194675758911868*G0_2_0 + 0.194675758911868*G0_2_1 - 0.00339247301052914*G0_2_2;
    A[1922] = -0.0510175748891017*G0_0_0 - 0.0510175748891017*G0_0_1 - 0.00463203045668257*G0_0_2 - 0.0528442911255397*G0_1_0 - 0.0528442911255397*G0_1_1 - 0.0887914570640255*G0_1_2 - 0.0887914570640252*G0_2_0 - 0.0887914570640251*G0_2_1 + 0.0846813455320401*G0_2_2;
    A[1923] = 0.0118736555368495*G0_0_0 + 0.0118736555368496*G0_0_1 - 0.0146137298915079*G0_0_2 + 0.0212681961813903*G0_1_2 + 0.0212681961813903*G0_2_0 + 0.0212681961813903*G0_2_1 - 0.0693499771190744*G0_2_2;
    A[1924] = -0.0644569872000419*G0_0_0 - 0.0929015685960119*G0_0_1 - 0.0644569872000421*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000423*G0_1_1 - 0.160751028806582*G0_1_2 - 0.257045070413124*G0_2_0 - 0.160751028806583*G0_2_1 - 0.257045070413124*G0_2_2;
    A[1925] = 0.132306447410613*G0_0_0 + 0.0589768384907264*G0_0_1 + 0.132306447410613*G0_0_2 + 0.194675758911868*G0_1_0 - 0.00339247301052845*G0_1_1 + 0.194675758911868*G0_1_2 + 0.121346149991982*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1926] = -0.0510175748891014*G0_0_0 - 0.0046320304566826*G0_0_1 - 0.0510175748891015*G0_0_2 - 0.0887914570640253*G0_1_0 + 0.0846813455320393*G0_1_1 - 0.0887914570640253*G0_1_2 - 0.0528442911255402*G0_2_0 - 0.088791457064026*G0_2_1 - 0.05284429112554*G0_2_2;
    A[1927] = 0.0118736555368496*G0_0_0 - 0.0146137298915078*G0_0_1 + 0.0118736555368496*G0_0_2 + 0.0212681961813906*G0_1_0 - 0.0693499771190737*G0_1_1 + 0.0212681961813907*G0_1_2 + 0.0212681961813907*G0_2_1;
    A[1928] = -0.0606730749959907*G0_0_0 - 0.032228493600021*G0_0_1 - 0.032228493600021*G0_0_2 - 0.0322284936000209*G0_1_0 - 0.128522535206561*G0_1_1 - 0.128522535206561*G0_1_2 - 0.0322284936000208*G0_2_0 - 0.128522535206562*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1929] = 0.0476251018785733*G0_0_0 - 0.0147442096226818*G0_0_1 - 0.0147442096226818*G0_0_2 - 0.014744209622682*G0_1_0 + 0.0476251018785733*G0_1_1 + 0.0476251018785732*G0_1_2 - 0.0147442096226821*G0_2_0 + 0.0476251018785734*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[1930] = -0.0665446628988288*G0_0_0 + 0.0227687130898938*G0_0_1 + 0.0227687130898937*G0_0_2 + 0.0227687130898935*G0_1_0 - 0.0150051690850296*G0_1_1 - 0.0150051690850297*G0_1_2 + 0.0227687130898936*G0_2_0 - 0.0150051690850298*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[1931] = 0.0453417065830257*G0_0_0 - 0.00939454064454057*G0_0_1 - 0.00939454064454049*G0_0_2 - 0.00939454064454058*G0_1_0 - 0.0093945406445405*G0_2_0;
    A[1932] = -0.0600206763401203*G0_0_0 - 0.03001033817006*G0_0_1 - 0.0300103381700604*G0_0_2 - 0.0300103381700596*G0_1_0 - 0.0150051690850296*G0_1_2 - 0.0300103381700597*G0_2_0 - 0.0150051690850299*G0_2_1;
    A[1933] = -0.0254435475789637*G0_0_0 - 0.0241387502672222*G0_0_1 - 0.0127217737894821*G0_0_2 - 0.0241387502672216*G0_1_0 - 0.0228339529554803*G0_1_1 - 0.0120693751336106*G0_1_2 - 0.012721773789482*G0_2_0 - 0.0120693751336111*G0_2_1;
    A[1934] = -0.105688582251081*G0_0_0 - 0.0756782440810207*G0_0_1 - 0.05284429112554*G0_0_2 - 0.0756782440810213*G0_1_0 - 0.0456679059109602*G0_1_1 - 0.0378391220405103*G0_1_2 - 0.0528442911255401*G0_2_0 - 0.0378391220405101*G0_2_1;
    A[1935] = -0.0254435475789641*G0_0_0 - 0.0127217737894817*G0_0_1 - 0.0241387502672217*G0_0_2 - 0.0127217737894823*G0_1_0 - 0.0120693751336111*G0_1_2 - 0.0241387502672222*G0_2_0 - 0.0120693751336107*G0_2_1 - 0.0228339529554804*G0_2_2;
    A[1936] = -0.113517366121531*G0_0_0 - 0.100795592332049*G0_0_1 - 0.100795592332049*G0_0_2 - 0.100795592332049*G0_1_0 - 0.0880738185425677*G0_1_1 - 0.088726217198438*G0_1_2 - 0.100795592332049*G0_2_0 - 0.0887262171984383*G0_2_1 - 0.0880738185425672*G0_2_2;
    A[1937] = -0.105688582251081*G0_0_0 - 0.052844291125541*G0_0_1 - 0.0756782440810216*G0_0_2 - 0.0528442911255405*G0_1_0 - 0.0378391220405108*G0_1_2 - 0.0756782440810214*G0_2_0 - 0.0378391220405109*G0_2_1 - 0.045667905910961*G0_2_2;
    A[1938] = 0.647179466623906*G0_0_0 + 0.375781625781623*G0_0_1 + 0.375781625781623*G0_0_2 + 0.375781625781623*G0_1_0 + 0.647179466623905*G0_1_1 + 0.375781625781622*G0_1_2 + 0.375781625781623*G0_2_0 + 0.375781625781622*G0_2_1 + 0.647179466623906*G0_2_2;
    A[1939] = -0.268135847562929*G0_0_0 - 0.448850275239161*G0_0_1 - 0.290643601190474*G0_0_2 - 0.19441479944952*G0_1_0 - 0.584549195660302*G0_1_1 - 0.321632537344341*G0_1_2 - 0.222141742324032*G0_2_0 - 0.32163253734434*G0_2_1 - 0.268135847562929*G0_2_2;
    A[1940] = 0.105688582251081*G0_0_0 + 0.18006202902036*G0_0_1 + 0.10503618359521*G0_0_2 + 0.0756782440810214*G0_1_0 + 0.255740273101382*G0_1_1 + 0.12787013655069*G0_1_2 + 0.0528442911255402*G0_2_0 + 0.12787013655069*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1941] = -0.268135847562928*G0_0_0 - 0.290643601190474*G0_0_1 - 0.448850275239161*G0_0_2 - 0.222141742324031*G0_1_0 - 0.268135847562928*G0_1_1 - 0.32163253734434*G0_1_2 - 0.194414799449521*G0_2_0 - 0.321632537344341*G0_2_1 - 0.584549195660303*G0_2_2;
    A[1942] = 0.113517366121531*G0_0_0 + 0.198655390712681*G0_0_1 + 0.198655390712681*G0_0_2 + 0.100795592332049*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919138*G0_1_2 + 0.100795592332049*G0_2_0 + 0.166035457919138*G0_2_1 + 0.172233245149911*G0_2_2;
    A[1943] = 0.105688582251081*G0_0_0 + 0.10503618359521*G0_0_1 + 0.18006202902036*G0_0_2 + 0.0528442911255403*G0_1_0 + 0.10568858225108*G0_1_1 + 0.12787013655069*G0_1_2 + 0.075678244081021*G0_2_0 + 0.12787013655069*G0_2_1 + 0.255740273101382*G0_2_2;
    A[1944] = 0.0521918924696692*G0_0_0 + 0.323589733311953*G0_0_1 + 0.187890812890811*G0_0_2 + 0.323589733311952*G0_1_0 + 0.0521918924696695*G0_1_1 + 0.18789081289081*G0_1_2 + 0.187890812890811*G0_2_0 + 0.187890812890812*G0_2_1 + 0.323589733311953*G0_2_2;
    A[1945] = 0.039143919352253*G0_0_0 - 0.0965550010688889*G0_0_1 - 0.0287055408583184*G0_0_2 - 0.0965550010688886*G0_1_0 - 0.0228339529554804*G0_1_1 - 0.0596944770121848*G0_1_2 - 0.028705540858318*G0_2_0 - 0.059694477012185*G0_2_1 - 0.096555001068889*G0_2_2;
    A[1946] = -0.0456679059109618*G0_0_0 + 0.0300103381700601*G0_0_1 - 0.0078287838704511*G0_0_2 + 0.0300103381700595*G0_1_0 + 0.0150051690850298*G0_1_2 - 0.00782878387045089*G0_2_0 + 0.0150051690850302*G0_2_1 + 0.0300103381700599*G0_2_2;
    A[1947] = -0.156575677409009*G0_0_0 - 0.134067923781463*G0_0_1 - 0.22442513761958*G0_0_2 - 0.134067923781463*G0_1_0 - 0.0880738185425666*G0_1_1 - 0.0972073997247584*G0_1_2 - 0.22442513761958*G0_2_0 - 0.09720739972476*G0_2_1 - 0.292274597830151*G0_2_2;
    A[1948] = 0.0391439193522525*G0_0_0 + 0.0127217737894818*G0_0_1 + 0.0446893079271553*G0_0_2 + 0.0127217737894823*G0_1_0 + 0.0120693751336111*G0_1_2 + 0.0446893079271553*G0_2_0 + 0.0120693751336107*G0_2_1 + 0.0502346965020572*G0_2_2;
    A[1949] = 0.0521918924696694*G0_0_0 + 0.0528442911255407*G0_0_1 + 0.0900310145101798*G0_0_2 + 0.0528442911255391*G0_1_0 + 0.0378391220405092*G0_1_2 + 0.0900310145101795*G0_2_0 + 0.0378391220405106*G0_2_1 + 0.127870136550691*G0_2_2;
    A[1950] = 0.0521918924696691*G0_0_0 + 0.187890812890811*G0_0_1 + 0.323589733311953*G0_0_2 + 0.18789081289081*G0_1_0 + 0.323589733311952*G0_1_1 + 0.187890812890811*G0_1_2 + 0.323589733311952*G0_2_0 + 0.187890812890811*G0_2_1 + 0.0521918924696693*G0_2_2;
    A[1951] = 0.0391439193522531*G0_0_0 - 0.0287055408583184*G0_0_1 - 0.0965550010688886*G0_0_2 - 0.0287055408583177*G0_1_0 - 0.096555001068889*G0_1_1 - 0.0596944770121842*G0_1_2 - 0.0965550010688889*G0_2_0 - 0.0596944770121845*G0_2_1 - 0.0228339529554801*G0_2_2;
    A[1952] = -0.0456679059109618*G0_0_0 - 0.00782878387045077*G0_0_1 + 0.0300103381700603*G0_0_2 - 0.00782878387045116*G0_1_0 + 0.03001033817006*G0_1_1 + 0.0150051690850298*G0_1_2 + 0.0300103381700599*G0_2_0 + 0.0150051690850294*G0_2_1;
    A[1953] = -0.156575677409009*G0_0_0 - 0.22442513761958*G0_0_1 - 0.134067923781464*G0_0_2 - 0.22442513761958*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0972073997247597*G0_1_2 - 0.134067923781464*G0_2_0 - 0.0972073997247592*G0_2_1 - 0.0880738185425672*G0_2_2;
    A[1954] = 0.0391439193522528*G0_0_0 + 0.0446893079271548*G0_0_1 + 0.0127217737894819*G0_0_2 + 0.044689307927155*G0_1_0 + 0.0502346965020568*G0_1_1 + 0.0120693751336103*G0_1_2 + 0.0127217737894817*G0_2_0 + 0.0120693751336113*G0_2_1;
    A[1955] = 0.0521918924696689*G0_0_0 + 0.0900310145101796*G0_0_1 + 0.05284429112554*G0_0_2 + 0.0900310145101795*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0378391220405102*G0_1_2 + 0.0528442911255401*G0_2_0 + 0.0378391220405096*G0_2_1;
    A[1956] = -0.782878387045049*G0_0_0 - 0.466465038947675*G0_0_1 - 0.466465038947674*G0_0_2 - 0.466465038947674*G0_1_0 - 0.150051690850301*G0_1_1 - 0.3359853077735*G0_1_2 - 0.466465038947675*G0_2_0 - 0.3359853077735*G0_2_1 - 0.150051690850301*G0_2_2;
    A[1957] = 0.195719596761262*G0_0_0 + 0.120693751336111*G0_0_1 + 0.12069375133611*G0_0_2 + 0.120693751336111*G0_1_0 + 0.0456679059109606*G0_1_1 + 0.0717638521457953*G0_1_2 + 0.120693751336112*G0_2_0 + 0.0717638521457957*G0_2_1 + 0.0456679059109604*G0_2_2;
    A[1958] = 0.293579395141892*G0_0_0 + 0.218553549716742*G0_0_1 + 0.234863516113513*G0_0_2 + 0.218553549716742*G0_1_0 + 0.143527704291591*G0_1_1 + 0.185933616923198*G0_1_2 + 0.234863516113514*G0_2_0 + 0.185933616923199*G0_2_1 + 0.176147637085134*G0_2_2;
    A[1959] = 0.293579395141892*G0_0_0 + 0.234863516113513*G0_0_1 + 0.218553549716743*G0_0_2 + 0.234863516113514*G0_1_0 + 0.176147637085135*G0_1_1 + 0.185933616923199*G0_1_2 + 0.218553549716743*G0_2_0 + 0.185933616923199*G0_2_1 + 0.143527704291593*G0_2_2;
    A[1960] = -0.0345719095719093*G0_0_0 - 0.0345719095719093*G0_0_1 - 0.0345719095719093*G0_0_2 - 0.0298472385060924*G0_1_0 - 0.0298472385060925*G0_1_1 - 0.0298472385060924*G0_1_2 - 0.0298472385060925*G0_2_0 - 0.0298472385060926*G0_2_1 - 0.0298472385060925*G0_2_2;
    A[1961] = -0.00946499969937446*G0_0_0;
    A[1962] = 0.00472467106581694*G0_0_1 - 0.0298472385060924*G0_1_1;
    A[1963] = 0.00227817610630113*G0_0_2 + 0.0117431758056757*G0_2_2;
    A[1964] = 0.0845508658008647*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.128522535206562*G0_1_1 + 0.0422754329004327*G0_1_2 + 0.0422754329004328*G0_2_1;
    A[1965] = 0.0808321934624015*G0_0_1 + 0.108232937008978*G0_0_2 + 0.0371867233846394*G0_1_1 + 0.0743408268364859*G0_1_2 + 0.0743408268364857*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1966] = 0.00939454064454045*G0_0_1 + 0.0377738821749235*G0_0_2 - 0.00619778723077319*G0_1_1 + 0.018886941087462*G0_1_2 + 0.018886941087462*G0_2_1 + 0.0567586830607664*G0_2_2;
    A[1967] = -0.00254435475789633*G0_0_1 - 0.0142875305635721*G0_0_2 - 0.00583896797004464*G0_1_2 - 0.00583896797004453*G0_2_1 - 0.0435476102793812*G0_2_2;
    A[1968] = 0.00658922642429572*G0_0_0 + 0.00329461321214823*G0_0_2 + 0.00329461321214784*G0_2_0;
    A[1969] = 0.00450155072550914*G0_0_0 + 0.0019245760348187*G0_0_2 + 0.0019245760348188*G0_2_0 - 0.000652398655871161*G0_2_2;
    A[1970] = -0.0105036183595212*G0_0_0 - 0.0130805930502109*G0_0_2 - 0.013080593050211*G0_2_0 - 0.0156575677409011*G0_2_2;
    A[1971] = -0.00254435475789632*G0_0_0 - 0.00583896797004434*G0_0_2 - 0.00583896797004453*G0_2_0 - 0.0091335811821924*G0_2_2;
    A[1972] = 0.0292600797158086*G0_0_0 + 0.00329461321214826*G0_0_1 + 0.00329461321214787*G0_1_0;
    A[1973] = -0.0189848008858422*G0_0_0 - 0.00949240044292161*G0_0_1 - 0.00949240044292087*G0_1_0 - 0.000652398655870534*G0_1_1;
    A[1974] = 0.0035229527417024*G0_0_0 + 0.00649136662591558*G0_0_1 + 0.00649136662591454*G0_1_0 + 0.0088073818542564*G0_1_1;
    A[1975] = 0.0429604514890968*G0_0_0 + 0.042275432900432*G0_0_1 + 0.0422754329004329*G0_1_0 + 0.0642612676032811*G0_1_1;
    A[1976] = 0.0439716694056966*G0_0_0 + 0.0439716694056967*G0_0_1 + 0.0446566879943612*G0_0_2 + 0.128522535206561*G0_1_0 + 0.128522535206561*G0_1_1 + 0.0862471023061294*G0_1_2 + 0.0862471023061293*G0_2_0 + 0.0862471023061293*G0_2_1 + 0.0439716694056972*G0_2_2;
    A[1977] = -0.0436454700777612*G0_0_0 - 0.0436454700777612*G0_0_1 - 0.00975335990526952*G0_0_2 + 0.0371867233846406*G0_1_0 + 0.0371867233846405*G0_1_1 - 0.037154103451846*G0_1_2 - 0.0371541034518457*G0_2_0 - 0.0371541034518458*G0_2_1 - 0.00678494602105737*G0_2_2;
    A[1978] = -0.0155923278753145*G0_0_0 - 0.0155923278753145*G0_0_1 + 0.00329461321214773*G0_0_2 - 0.00619778723077418*G0_1_0 - 0.00619778723077416*G0_1_1 - 0.0250847283182353*G0_1_2 - 0.0250847283182357*G0_2_0 - 0.0250847283182357*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[1979] = 0.00254435475789643*G0_0_0 + 0.0025443547578964*G0_0_1 - 0.00590420783563132*G0_0_2 + 0.00583896797004432*G0_1_2 + 0.00583896797004461*G0_2_0 + 0.0058389679700446*G0_2_1 - 0.0318696743392922*G0_2_2;
    A[1980] = 0.1273155976932*G0_0_0 + 0.128000616281865*G0_0_1 + 0.1273155976932*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761737*G0_1_1 + 0.134361503176606*G0_1_2 + 0.176636936077039*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077039*G0_2_2;
    A[1981] = -0.166948816037356*G0_0_0 - 0.160783648739376*G0_0_1 - 0.166948816037356*G0_0_2 - 0.250814663249557*G0_1_0 - 0.0622388317700813*G0_1_1 - 0.250814663249557*G0_1_2 - 0.334680510461758*G0_2_0 - 0.250814663249557*G0_2_1 - 0.334680510461758*G0_2_2;
    A[1982] = 0.182410664181495*G0_0_0 + 0.0985448169692951*G0_0_1 + 0.182410664181495*G0_0_2 + 0.188575831479475*G0_1_0 - 0.0622388317700814*G0_1_1 + 0.188575831479475*G0_1_2 + 0.104709984267275*G0_2_0 + 0.188575831479476*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1983] = -0.0429604514890966*G0_0_0 - 0.0359145460056916*G0_0_1 - 0.0429604514890967*G0_0_2 - 0.0422754329004328*G0_1_0 + 0.0920860702761737*G0_1_1 - 0.0422754329004329*G0_1_2 - 0.0422754329004326*G0_2_1;
    A[1984] = 0.0226708532915127*G0_0_0 + 0.0219858347028482*G0_0_1 + 0.0219858347028483*G0_0_2 + 0.0219858347028483*G0_1_0 + 0.0642612676032805*G0_1_1 + 0.0642612676032805*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0642612676032808*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[1985] = -0.000652398655870761*G0_0_0 + 0.00231601522834155*G0_0_1 + 0.00231601522834153*G0_0_2 + 0.00231601522834152*G0_1_0 + 0.0088073818542569*G0_1_1 + 0.00880738185425687*G0_1_2 + 0.00231601522834161*G0_2_0 + 0.00880738185425667*G0_2_1 + 0.00880738185425665*G0_2_2;
    A[1986] = -0.000652398655870812*G0_0_0 + 0.00884000178705013*G0_0_1 + 0.00884000178705018*G0_0_2 + 0.00884000178705048*G0_1_0 - 0.000652398655871293*G0_1_1 - 0.000652398655871186*G0_1_2 + 0.00884000178705041*G0_2_0 - 0.000652398655870894*G0_2_1 - 0.000652398655870909*G0_2_2;
    A[1987] = 0.0226708532915126*G0_0_0 - 0.00329461321214816*G0_0_1 - 0.00329461321214822*G0_0_2 - 0.00329461321214786*G0_1_0 - 0.00329461321214807*G0_2_0;
    A[1988] = -0.0254435475789637*G0_0_0 - 0.00130479731174144*G0_0_1 - 0.0127217737894818*G0_0_2 - 0.00130479731174212*G0_1_0 - 0.000652398655870803*G0_1_2 - 0.012721773789482*G0_2_0 - 0.000652398655871149*G0_2_1;
    A[1989] = 0.0567586830607656*G0_0_0 + 0.0283793415303828*G0_0_1 + 0.0283793415303827*G0_0_2 + 0.0283793415303823*G0_1_0 - 0.00489298991903206*G0_1_1 + 0.0141896707651907*G0_1_2 + 0.0283793415303825*G0_2_0 + 0.0141896707651915*G0_2_1;
    A[1990] = 0.20941996853455*G0_0_0 + 0.135698920421142*G0_0_1 + 0.104709984267274*G0_0_2 + 0.135698920421142*G0_1_0 + 0.0391439193522515*G0_1_1 + 0.0678494602105705*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0678494602105705*G0_2_1;
    A[1991] = -0.0166361657247078*G0_0_0 - 0.0181040627004172*G0_0_1 + 0.00146789697570957*G0_0_2 - 0.0181040627004162*G0_1_0 - 0.0102752788299664*G0_1_2 + 0.00146789697570886*G0_2_0 - 0.010275278829967*G0_2_1 + 0.0195719596761259*G0_2_2;
    A[1992] = 0.149725491522365*G0_0_0 + 0.074862745761182*G0_0_1 + 0.121346149991982*G0_0_2 + 0.0748627457611819*G0_1_0 + 0.0195719596761266*G0_1_1 + 0.0606730749959908*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0606730749959905*G0_2_1 + 0.0929668084615984*G0_2_2;
    A[1993] = 0.0137003717732886*G0_0_0 - 0.00619778723077294*G0_0_1 + 0.0264221455627703*G0_0_2 - 0.00619778723077315*G0_1_0 - 0.00554538857490178*G0_1_2 + 0.0264221455627703*G0_2_0 - 0.00554538857490256*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[1994] = -0.268135847562929*G0_0_0 - 0.19441479944952*G0_0_1 - 0.222141742324032*G0_0_2 - 0.448850275239161*G0_1_0 - 0.584549195660302*G0_1_1 - 0.32163253734434*G0_1_2 - 0.290643601190474*G0_2_0 - 0.321632537344341*G0_2_1 - 0.268135847562929*G0_2_2;
    A[1995] = 0.726119703984281*G0_0_0 + 0.45602666045374*G0_0_1 + 0.456026660453741*G0_0_2 + 0.45602666045374*G0_1_0 + 0.91205332090748*G0_1_1 + 0.456026660453741*G0_1_2 + 0.456026660453741*G0_2_0 + 0.456026660453741*G0_2_1 + 0.726119703984282*G0_2_2;
    A[1996] = -0.20941996853455*G0_0_0 - 0.390134396210781*G0_0_1 - 0.290643601190473*G0_0_2 - 0.135698920421142*G0_1_0 - 0.584549195660303*G0_1_1 - 0.262916658315961*G0_1_2 - 0.104709984267276*G0_2_0 - 0.262916658315962*G0_2_1 - 0.209419968534549*G0_2_2;
    A[1997] = 0.065566064915023*G0_0_0 + 0.140428810676206*G0_0_1 + 0.0376760223765423*G0_0_2 + 0.0376760223765418*G0_1_0 - 0.0371867233846406*G0_1_1 + 0.186912214907004*G0_1_2 + 0.140428810676206*G0_2_0 + 0.186912214907005*G0_2_1 - 0.0371867233846404*G0_2_2;
    A[1998] = -0.149725491522365*G0_0_0 - 0.177615534060846*G0_0_1 - 0.326851726591308*G0_0_2 - 0.0748627457611819*G0_1_0 - 0.0371867233846392*G0_1_1 - 0.224098938291645*G0_1_2 - 0.121346149991981*G0_2_0 - 0.224098938291645*G0_2_1 - 0.448197876583291*G0_2_2;
    A[1999] = -0.0137003717732886*G0_0_0 + 0.0061977872307743*G0_0_1 + 0.0714376528178613*G0_0_2 + 0.00619778723077336*G0_1_0 + 0.0123955744615476*G0_1_1 + 0.00619778723077465*G0_1_2 - 0.0264221455627702*G0_2_0 + 0.00619778723077389*G0_2_1 + 0.172233245149912*G0_2_2;
    A[2000] = -0.0228339529554807*G0_0_0 - 0.0965550010688895*G0_0_1 - 0.0596944770121848*G0_0_2 - 0.0965550010688884*G0_1_0 + 0.0391439193522523*G0_1_1 - 0.0287055408583175*G0_1_2 - 0.0596944770121852*G0_2_0 - 0.028705540858319*G0_2_1 - 0.0965550010688888*G0_2_2;
    A[2001] = -0.00489298991903159*G0_0_0 - 0.0332723314494149*G0_0_1 - 0.0190826606842232*G0_0_2 - 0.0332723314494148*G0_1_0 - 0.00489298991903171*G0_1_1 - 0.0190826606842229*G0_1_2 - 0.0190826606842226*G0_2_0 - 0.0190826606842232*G0_2_1 - 0.0332723314494143*G0_2_2;
    A[2002] = -0.02283395295548*G0_0_0 + 0.00130479731174133*G0_0_1 - 0.010764577821869*G0_0_2 + 0.001304797311742*G0_1_0 + 0.000652398655871016*G0_1_2 - 0.0107645778218691*G0_2_0 + 0.000652398655871101*G0_2_1 + 0.00130479731174196*G0_2_2;
    A[2003] = 0.0195719596761261*G0_0_0 - 0.0552907860850568*G0_0_1 + 0.00538228891093432*G0_0_2 - 0.0552907860850576*G0_1_0 + 0.0195719596761252*G0_1_1 - 0.0411011153198664*G0_1_2 + 0.00538228891093429*G0_2_0 - 0.041101115319865*G0_2_1 - 0.00880738185425743*G0_2_2;
    A[2004] = 0.0195719596761257*G0_0_0 + 0.0181040627004171*G0_0_1 + 0.0298472385060926*G0_0_2 + 0.0181040627004167*G0_1_0 + 0.0102752788299663*G0_1_2 + 0.0298472385060923*G0_2_0 + 0.0102752788299668*G0_2_1 + 0.0401225173360589*G0_2_2;
    A[2005] = 0.0260959462348358*G0_0_0 + 0.00619778723077315*G0_0_1 + 0.0381653213684464*G0_0_2 + 0.00619778723077453*G0_1_0 + 0.00554538857490355*G0_1_2 + 0.0381653213684468*G0_2_0 + 0.00554538857490277*G0_2_1 + 0.0502346965020578*G0_2_2;
    A[2006] = -0.0880738185425675*G0_0_0 - 0.0972073997247599*G0_0_1 - 0.134067923781465*G0_0_2 - 0.097207399724759*G0_1_0 - 0.292274597830151*G0_1_1 - 0.224425137619581*G0_1_2 - 0.134067923781464*G0_2_0 - 0.22442513761958*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2007] = 0.019571959676126*G0_0_0 - 0.0411011153198645*G0_0_1 - 0.0552907860850568*G0_0_2 - 0.0411011153198654*G0_1_0 - 0.00880738185425634*G0_1_1 + 0.00538228891093442*G0_1_2 - 0.0552907860850566*G0_2_0 + 0.005382288910935*G0_2_1 + 0.0195719596761258*G0_2_2;
    A[2008] = -0.0880738185425674*G0_0_0 + 0.000652398655871306*G0_0_1 + 0.0127217737894817*G0_0_2 + 0.000652398655871069*G0_1_0 + 0.00130479731174178*G0_1_1 + 0.000652398655870629*G0_1_2 + 0.0127217737894827*G0_2_0 + 0.000652398655870883*G0_2_1;
    A[2009] = 0.0929668084615991*G0_0_0 + 0.228013330226869*G0_0_1 + 0.363059851992139*G0_0_2 + 0.22801333022687*G0_1_0 + 0.45602666045374*G0_1_1 + 0.228013330226869*G0_1_2 + 0.363059851992141*G0_2_0 + 0.22801333022687*G0_2_1 + 0.0929668084615995*G0_2_2;
    A[2010] = 0.0929668084615993*G0_0_0 + 0.0322937334656078*G0_0_1 - 0.0283793415303824*G0_0_2 + 0.0322937334656083*G0_1_0 - 0.00880738185425665*G0_1_1 - 0.0141896707651905*G0_1_2 - 0.0283793415303835*G0_2_0 - 0.0141896707651913*G0_2_1;
    A[2011] = -0.185933616923197*G0_0_0 - 0.195067198105389*G0_0_1 - 0.104709984267275*G0_0_2 - 0.19506719810539*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105704*G0_1_2 - 0.104709984267274*G0_2_0 - 0.0678494602105702*G0_2_1;
    A[2012] = 0.146789697570946*G0_0_0 + 0.445262082631871*G0_0_1 + 0.24954248587061*G0_0_2 + 0.44526208263187*G0_1_0 - 0.0391439193522535*G0_1_1 + 0.249542485870609*G0_1_2 + 0.24954248587061*G0_2_0 + 0.249542485870611*G0_2_1 + 0.352295274170274*G0_2_2;
    A[2013] = 0.146789697570946*G0_0_0 + 0.00489298991903234*G0_0_1 + 0.0538228891093476*G0_0_2 + 0.00489298991903254*G0_1_0 + 0.00978597983806359*G0_1_1 + 0.00489298991903195*G0_1_2 + 0.0538228891093462*G0_2_0 + 0.00489298991903181*G0_2_1 - 0.0391439193522516*G0_2_2;
    A[2014] = -0.782878387045047*G0_0_0 - 0.484406001984124*G0_0_1 - 0.484406001984122*G0_0_2 - 0.484406001984122*G0_1_0 - 0.0391439193522503*G0_1_1 - 0.28868640522286*G0_1_2 - 0.484406001984123*G0_2_0 - 0.288686405222863*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2015] = -0.0489298991903154*G0_0_0 - 0.0195719596761254*G0_0_1 - 0.141896707651916*G0_0_2 - 0.0195719596761257*G0_1_0 - 0.0391439193522523*G0_1_1 - 0.0195719596761262*G0_1_2 - 0.141896707651915*G0_2_0 - 0.0195719596761256*G0_2_1 - 0.234863516113516*G0_2_2;
    A[2016] = 0.013742125287264*G0_0_0 + 0.013742125287264*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0189195610202553*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[2017] = -0.0137421252872641*G0_0_0;
    A[2018] = -0.0143945239431349*G0_0_1 + 0.03359853077735*G0_1_1;
    A[2019] = 0.00517743573299125*G0_0_2 + 0.0189195610202553*G0_2_2;
    A[2020] = -0.192588083213081*G0_0_1 - 0.12473862300251*G0_0_2 - 0.257045070413124*G0_1_1 - 0.0962940416065406*G0_1_2 - 0.0962940416065408*G0_2_1;
    A[2021] = -0.0109602974186309*G0_0_1 - 0.146659217839773*G0_0_2 + 0.121346149991982*G0_1_1 - 0.0733296089198864*G0_1_2 - 0.0733296089198859*G0_2_1 - 0.271397840842283*G0_2_2;
    A[2022] = -0.0018267162364382*G0_0_1 + 0.0823327103709046*G0_0_2 - 0.0528442911255405*G0_1_1 + 0.0359471659384853*G0_1_2 + 0.0359471659384851*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2023] = -0.0118736555368496*G0_0_1 - 0.0477555816097478*G0_0_2 - 0.0212681961813902*G0_1_2 - 0.0212681961813905*G0_2_1 - 0.111886369481855*G0_2_2;
    A[2024] = 0.0187890812890808*G0_0_0 + 0.00939454064454019*G0_0_2 + 0.00939454064454048*G0_2_0;
    A[2025] = -0.0148746893538553*G0_0_0 - 0.0149399292194425*G0_0_2 - 0.014939929219443*G0_2_0 - 0.0150051690850299*G0_2_2;
    A[2026] = -0.00117431758056827*G0_0_0 - 0.00123955744615526*G0_0_2 - 0.00123955744615468*G0_2_0 - 0.00130479731174192*G0_2_2;
    A[2027] = -0.0118736555368496*G0_0_0 - 0.0212681961813902*G0_0_2 - 0.0212681961813905*G0_2_0 - 0.0306627368259311*G0_2_2;
    A[2028] = 0.0641307878721066*G0_0_0 + 0.00939454064454031*G0_0_1 + 0.00939454064454029*G0_1_0;
    A[2029] = -0.127087258163646*G0_0_0 - 0.0377738821749232*G0_0_1 - 0.0377738821749235*G0_1_0 - 0.0150051690850301*G0_1_1;
    A[2030] = 0.12473862300251*G0_0_0 + 0.0623693115012549*G0_0_1 + 0.0623693115012555*G0_1_0 + 0.0476251018785735*G0_1_1;
    A[2031] = -0.124738623002511*G0_0_0 - 0.0962940416065402*G0_0_1 - 0.0962940416065407*G0_1_0 - 0.128522535206562*G0_1_1;
    A[2032] = -0.0187890812890812*G0_0_0 - 0.0187890812890811*G0_0_1 - 0.00939454064454065*G0_0_2 - 0.0613254736518616*G0_1_0 - 0.0613254736518617*G0_1_1 - 0.0400572774704716*G0_1_2 - 0.0400572774704713*G0_2_0 - 0.0400572774704713*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[2033] = 0.0148746893538554*G0_0_0 + 0.0148746893538554*G0_0_1 - 6.52398655870367e-05*G0_0_2 - 0.025443547578965*G0_1_0 - 0.0254435475789649*G0_1_1 + 0.0136351319077012*G0_1_2 + 0.0136351319077009*G0_2_0 + 0.0136351319077009*G0_2_1 - 0.000130479731174106*G0_2_2;
    A[2034] = 0.00117431758056813*G0_0_0 + 0.00117431758056808*G0_0_1 - 6.52398655871379e-05*G0_0_2 - 0.0528442911255401*G0_1_0 - 0.0528442911255401*G0_1_1 - 0.0137656116388754*G0_1_2 - 0.0137656116388752*G0_2_0 - 0.0137656116388752*G0_2_1 - 0.000130479731174183*G0_2_2;
    A[2035] = 0.0118736555368496*G0_0_0 + 0.0118736555368496*G0_0_1 - 0.00939454064454056*G0_0_2 + 0.0212681961813906*G0_1_2 + 0.0212681961813905*G0_2_0 + 0.0212681961813905*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[2036] = -0.0641307878721065*G0_0_0 - 0.0547362472275661*G0_0_1 - 0.0641307878721065*G0_0_2 - 0.0906181733004636*G0_1_0 - 0.0693499771190737*G0_1_1 - 0.0906181733004636*G0_1_2 - 0.111886369481854*G0_2_0 - 0.0906181733004642*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2037] = 0.127087258163646*G0_0_0 + 0.0893133759887224*G0_0_1 + 0.127087258163646*G0_0_2 + 0.173472802596065*G0_1_0 + 0.0846813455320393*G0_1_1 + 0.173472802596065*G0_1_2 + 0.20941996853455*G0_2_0 + 0.173472802596065*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2038] = -0.124738623002511*G0_0_0 - 0.0623693115012555*G0_0_1 - 0.124738623002511*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052862*G0_1_1 - 0.198068231922397*G0_1_2 - 0.271397840842283*G0_2_0 - 0.198068231922397*G0_2_1 - 0.271397840842283*G0_2_2;
    A[2039] = 0.12473862300251*G0_0_0 + 0.0284445813959702*G0_0_1 + 0.12473862300251*G0_0_2 + 0.0962940416065406*G0_1_0 - 0.0644569872000422*G0_1_1 + 0.0962940416065407*G0_1_2 + 0.0962940416065412*G0_2_1;
    A[2040] = -0.00939454064454039*G0_0_1 - 0.00939454064454038*G0_0_2 - 0.00939454064454046*G0_1_0 - 0.0306627368259306*G0_1_1 - 0.0306627368259306*G0_1_2 - 0.00939454064454059*G0_2_0 - 0.0306627368259307*G0_2_1 - 0.0306627368259308*G0_2_2;
    A[2041] = -6.52398655870091e-05*G0_0_1 - 6.5239865587025e-05*G0_0_2 - 6.52398655870362e-05*G0_1_0 - 0.00130479731174173*G0_1_1 - 0.00130479731174174*G0_1_2 - 6.52398655869959e-05*G0_2_0 - 0.00130479731174166*G0_2_1 - 0.00130479731174166*G0_2_2;
    A[2042] = -6.52398655870354e-05*G0_0_1 - 6.52398655870323e-05*G0_0_2 - 6.52398655872252e-05*G0_1_0 - 0.0150051690850298*G0_1_1 - 0.0150051690850299*G0_1_2 - 6.52398655871646e-05*G0_2_0 - 0.0150051690850297*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[2043] = -0.00939454064454034*G0_0_1 - 0.00939454064454028*G0_0_2 - 0.0093945406445407*G0_1_0 - 0.00939454064454061*G0_2_0;
    A[2044] = -0.105688582251081*G0_0_0 - 0.0300103381700601*G0_0_1 - 0.0528442911255406*G0_0_2 - 0.0300103381700597*G0_1_0 - 0.0150051690850303*G0_1_2 - 0.0528442911255405*G0_2_0 - 0.0150051690850303*G0_2_1;
    A[2045] = 0.20941996853455*G0_0_0 + 0.0737210481134086*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0737210481134088*G0_1_0 - 0.0228339529554803*G0_1_1 + 0.0368605240567043*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0368605240567052*G0_2_1;
    A[2046] = -0.542795681684565*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842282*G0_0_2 - 0.271397840842283*G0_1_0 + 0.0521918924696704*G0_1_1 - 0.135698920421141*G0_1_2 - 0.271397840842283*G0_2_0 - 0.135698920421141*G0_2_1;
    A[2047] = 0.0137003717732873*G0_0_0 - 0.0127217737894822*G0_0_1 + 0.0198981590040617*G0_0_2 - 0.012721773789483*G0_1_0 - 0.0120693751336111*G0_1_2 + 0.0198981590040614*G0_2_0 - 0.0120693751336109*G0_2_1 + 0.026095946234835*G0_2_2;
    A[2048] = 0.023486351611352*G0_0_0 + 0.0459941052388974*G0_0_1 - 0.0812236326559232*G0_0_2 + 0.045994105238897*G0_1_0 - 0.0880738185425679*G0_1_1 + 0.00913358118219226*G0_1_2 - 0.0812236326559235*G0_2_0 + 0.0091335811821918*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2049] = -0.0534966897814111*G0_0_0 - 0.0528442911255412*G0_0_1 - 0.000652398655871308*G0_0_2 - 0.0528442911255406*G0_1_0 - 0.0378391220405113*G0_1_2 - 0.000652398655871462*G0_2_0 - 0.0378391220405113*G0_2_1 + 0.0521918924696694*G0_2_2;
    A[2050] = 0.105688582251081*G0_0_0 + 0.0756782440810214*G0_0_1 + 0.0528442911255402*G0_0_2 + 0.18006202902036*G0_1_0 + 0.255740273101382*G0_1_1 + 0.12787013655069*G0_1_2 + 0.10503618359521*G0_2_0 + 0.12787013655069*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2051] = -0.20941996853455*G0_0_0 - 0.135698920421142*G0_0_1 - 0.104709984267276*G0_0_2 - 0.390134396210781*G0_1_0 - 0.584549195660303*G0_1_1 - 0.262916658315962*G0_1_2 - 0.290643601190473*G0_2_0 - 0.262916658315961*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2052] = 0.542795681684566*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842284*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623906*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842284*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684566*G0_2_2;
    A[2053] = -0.0137003717732877*G0_0_0 - 0.0264221455627698*G0_0_1 + 0.00619778723077431*G0_0_2 + 0.0714376528178617*G0_1_0 + 0.172233245149912*G0_1_1 + 0.00619778723077505*G0_1_2 + 0.00619778723077341*G0_2_0 + 0.00619778723077404*G0_2_1 + 0.0123955744615469*G0_2_2;
    A[2054] = -0.0234863516113522*G0_0_0 + 0.0225077536275451*G0_0_1 - 0.104709984267276*G0_0_2 - 0.0459941052388971*G0_1_0 - 0.268135847562929*G0_1_1 + 0.0534966897814114*G0_1_2 + 0.0812236326559233*G0_2_0 + 0.053496689781411*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2055] = 0.0534966897814108*G0_0_0 + 0.00065239865586998*G0_0_1 + 0.0528442911255399*G0_0_2 + 0.0528442911255405*G0_1_0 + 0.105688582251081*G0_1_1 - 0.022181554299611*G0_1_2 + 0.000652398655871379*G0_2_0 - 0.02218155429961*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2056] = 0.0300103381700597*G0_0_1 + 0.0150051690850297*G0_0_2 + 0.0300103381700597*G0_1_0 - 0.0456679059109615*G0_1_1 - 0.00782878387045154*G0_1_2 + 0.0150051690850299*G0_2_0 - 0.00782878387045053*G0_2_1 + 0.0300103381700597*G0_2_2;
    A[2057] = 0.00130479731174202*G0_0_1 + 0.000652398655870795*G0_0_2 + 0.00130479731174179*G0_1_0 - 0.0228339529554803*G0_1_1 - 0.0107645778218694*G0_1_2 + 0.00065239865587093*G0_2_0 - 0.0107645778218694*G0_2_1 + 0.00130479731174134*G0_2_2;
    A[2058] = 0.0300103381700601*G0_0_1 + 0.0150051690850296*G0_0_2 + 0.0300103381700604*G0_1_0 + 0.01500516908503*G0_1_2 + 0.01500516908503*G0_2_0 + 0.0150051690850302*G0_2_1 + 0.0300103381700599*G0_2_2;
    A[2059] = 0.0127217737894823*G0_0_1 + 0.000652398655871727*G0_0_2 + 0.0127217737894831*G0_1_0 - 0.0880738185425673*G0_1_1 + 0.000652398655872321*G0_1_2 + 0.000652398655871317*G0_2_0 + 0.000652398655870769*G0_2_1 + 0.00130479731174231*G0_2_2;
    A[2060] = 0.0127217737894821*G0_0_1 + 0.000652398655871449*G0_0_2 + 0.0127217737894816*G0_1_0 + 0.0120693751336111*G0_1_2 + 0.000652398655870789*G0_2_0 + 0.0120693751336109*G0_2_1 + 0.00130479731174214*G0_2_2;
    A[2061] = 0.0528442911255411*G0_0_1 + 0.0150051690850294*G0_0_2 + 0.0528442911255403*G0_1_0 + 0.0378391220405098*G0_1_2 + 0.0150051690850298*G0_2_0 + 0.0378391220405111*G0_2_1 + 0.0300103381700599*G0_2_2;
    A[2062] = 0.0378391220405102*G0_0_1 + 0.0528442911255405*G0_0_2 + 0.0378391220405098*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0900310145101808*G0_1_2 + 0.0528442911255402*G0_2_0 + 0.09003101451018*G0_2_1 + 0.0521918924696698*G0_2_2;
    A[2063] = 0.00554538857490258*G0_0_1 + 0.00619778723077419*G0_0_2 + 0.00554538857490255*G0_1_0 + 0.0502346965020567*G0_1_1 + 0.0381653213684461*G0_1_2 + 0.0061977872307732*G0_2_0 + 0.0381653213684456*G0_2_1 + 0.0260959462348353*G0_2_2;
    A[2064] = 0.0378391220405102*G0_0_1 + 0.0528442911255407*G0_0_2 + 0.0378391220405109*G0_1_0 + 0.0300103381700601*G0_1_1 + 0.0150051690850304*G0_1_2 + 0.0528442911255408*G0_2_0 + 0.0150051690850297*G0_2_1;
    A[2065] = -0.0678494602105706*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105704*G0_1_0 - 0.292274597830151*G0_1_1 - 0.19506719810539*G0_1_2 - 0.104709984267275*G0_2_0 - 0.195067198105391*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2066] = -0.0678494602105705*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105711*G0_1_0 - 0.0965550010688887*G0_1_1 - 0.0368605240567045*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0368605240567042*G0_2_1;
    A[2067] = 0.135698920421141*G0_0_1 + 0.271397840842282*G0_0_2 + 0.135698920421141*G0_1_0 + 0.323589733311952*G0_1_1 + 0.135698920421141*G0_1_2 + 0.271397840842283*G0_2_0 + 0.135698920421141*G0_2_1;
    A[2068] = -0.0750258454251497*G0_0_1 - 0.0260959462348354*G0_0_2 - 0.075025845425149*G0_1_0 + 0.143527704291594*G0_1_1 - 0.0424059126316066*G0_1_2 - 0.0260959462348341*G0_2_0 - 0.0424059126316058*G0_2_1 - 0.05219189246967*G0_2_2;
    A[2069] = -0.0750258454251504*G0_0_1 - 0.0260959462348361*G0_0_2 - 0.0750258454251517*G0_1_0 + 0.0456679059109607*G0_1_1 - 0.0260959462348352*G0_1_2 - 0.0260959462348352*G0_2_0 - 0.0260959462348355*G0_2_1 - 0.0521918924696707*G0_2_2;
    A[2070] = 0.316413348097372*G0_0_1 + 0.185933616923199*G0_0_2 + 0.316413348097372*G0_1_0 - 0.150051690850303*G0_1_1 + 0.185933616923198*G0_1_2 + 0.185933616923197*G0_2_0 + 0.185933616923198*G0_2_1 + 0.371867233846398*G0_2_2;
    A[2071] = -0.0587158790283794*G0_0_1 - 0.0260959462348339*G0_0_2 - 0.0587158790283785*G0_1_0 + 0.176147637085136*G0_1_1 - 0.00978597983806311*G0_1_2 - 0.0260959462348349*G0_2_0 - 0.0097859798380625*G0_2_1 - 0.0521918924696694*G0_2_2;
    A[2072] = -0.0345719095719092*G0_0_0 - 0.0345719095719091*G0_0_1 - 0.0345719095719092*G0_0_2 - 0.0298472385060924*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0298472385060924*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.0298472385060923*G0_2_2;
    A[2073] = -0.00946499969937471*G0_0_0;
    A[2074] = 0.0022781761063011*G0_0_1 + 0.0117431758056758*G0_1_1;
    A[2075] = 0.00472467106581673*G0_0_2 - 0.0298472385060927*G0_2_2;
    A[2076] = -0.0142875305635726*G0_0_1 - 0.00254435475789679*G0_0_2 - 0.0435476102793812*G0_1_1 - 0.00583896797004446*G0_1_2 - 0.00583896797004512*G0_2_1;
    A[2077] = 0.0377738821749241*G0_0_1 + 0.00939454064454089*G0_0_2 + 0.0567586830607661*G0_1_1 + 0.0188869410874615*G0_1_2 + 0.0188869410874625*G0_2_1 - 0.00619778723077291*G0_2_2;
    A[2078] = 0.108232937008977*G0_0_1 + 0.080832193462401*G0_0_2 + 0.104709984267275*G0_1_1 + 0.0743408268364861*G0_1_2 + 0.0743408268364854*G0_2_1 + 0.0371867233846393*G0_2_2;
    A[2079] = 0.042960451489097*G0_0_1 + 0.0845508658008653*G0_0_2 + 0.0422754329004322*G0_1_2 + 0.0422754329004327*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2080] = 0.0292600797158088*G0_0_0 + 0.00329461321214814*G0_0_2 + 0.00329461321214805*G0_2_0;
    A[2081] = -0.0189848008858428*G0_0_0 - 0.0094924004429216*G0_0_2 - 0.00949240044292153*G0_2_0 - 0.000652398655871687*G0_2_2;
    A[2082] = 0.00352295274170265*G0_0_0 + 0.0064913666259153*G0_0_2 + 0.00649136662591508*G0_2_0 + 0.00880738185425734*G0_2_2;
    A[2083] = 0.042960451489097*G0_0_0 + 0.0422754329004325*G0_0_2 + 0.0422754329004327*G0_2_0 + 0.0642612676032811*G0_2_2;
    A[2084] = 0.006589226424296*G0_0_0 + 0.00329461321214804*G0_0_1 + 0.003294613212148*G0_1_0;
    A[2085] = 0.00450155072550902*G0_0_0 + 0.00192457603481895*G0_0_1 + 0.00192457603481896*G0_1_0 - 0.000652398655871434*G0_1_1;
    A[2086] = -0.0105036183595211*G0_0_0 - 0.0130805930502109*G0_0_1 - 0.013080593050211*G0_1_0 - 0.0156575677409005*G0_1_1;
    A[2087] = -0.00254435475789676*G0_0_0 - 0.00583896797004473*G0_0_1 - 0.00583896797004438*G0_1_0 - 0.00913358118219253*G0_1_1;
    A[2088] = 0.1273155976932*G0_0_0 + 0.1273155976932*G0_0_1 + 0.128000616281865*G0_0_2 + 0.176636936077039*G0_1_0 + 0.176636936077038*G0_1_1 + 0.134361503176606*G0_1_2 + 0.134361503176606*G0_2_0 + 0.134361503176606*G0_2_1 + 0.092086070276173*G0_2_2;
    A[2089] = -0.166948816037356*G0_0_0 - 0.166948816037356*G0_0_1 - 0.160783648739376*G0_0_2 - 0.334680510461758*G0_1_0 - 0.334680510461758*G0_1_1 - 0.250814663249557*G0_1_2 - 0.250814663249557*G0_2_0 - 0.250814663249557*G0_2_1 - 0.0622388317700801*G0_2_2;
    A[2090] = 0.182410664181495*G0_0_0 + 0.182410664181495*G0_0_1 + 0.0985448169692946*G0_0_2 + 0.104709984267274*G0_1_0 + 0.104709984267274*G0_1_1 + 0.188575831479475*G0_1_2 + 0.188575831479475*G0_2_0 + 0.188575831479475*G0_2_1 - 0.0622388317700828*G0_2_2;
    A[2091] = -0.0429604514890969*G0_0_0 - 0.0429604514890969*G0_0_1 - 0.035914546005691*G0_0_2 - 0.0422754329004321*G0_1_2 - 0.0422754329004326*G0_2_0 - 0.0422754329004326*G0_2_1 + 0.0920860702761748*G0_2_2;
    A[2092] = 0.043971669405697*G0_0_0 + 0.0446566879943612*G0_0_1 + 0.043971669405697*G0_0_2 + 0.0862471023061294*G0_1_0 + 0.0439716694056969*G0_1_1 + 0.0862471023061294*G0_1_2 + 0.128522535206562*G0_2_0 + 0.0862471023061291*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2093] = -0.043645470077761*G0_0_0 - 0.00975335990526969*G0_0_1 - 0.0436454700777611*G0_0_2 - 0.0371541034518459*G0_1_0 - 0.00678494602105721*G0_1_1 - 0.0371541034518459*G0_1_2 + 0.0371867233846405*G0_2_0 - 0.0371541034518461*G0_2_1 + 0.0371867233846406*G0_2_2;
    A[2094] = -0.0155923278753143*G0_0_0 + 0.00329461321214811*G0_0_1 - 0.0155923278753142*G0_0_2 - 0.0250847283182354*G0_1_0 + 0.0127870136550693*G0_1_1 - 0.0250847283182354*G0_1_2 - 0.00619778723077391*G0_2_0 - 0.025084728318235*G0_2_1 - 0.006197787230774*G0_2_2;
    A[2095] = 0.00254435475789675*G0_0_0 - 0.00590420783563155*G0_0_1 + 0.00254435475789677*G0_0_2 + 0.00583896797004442*G0_1_0 - 0.0318696743392925*G0_1_1 + 0.00583896797004441*G0_1_2 + 0.00583896797004419*G0_2_1;
    A[2096] = 0.0226708532915127*G0_0_0 + 0.0219858347028485*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0219858347028482*G0_1_0 + 0.0642612676032808*G0_1_1 + 0.0642612676032807*G0_1_2 + 0.021985834702848*G0_2_0 + 0.064261267603281*G0_2_1 + 0.0642612676032809*G0_2_2;
    A[2097] = -0.00065239865587079*G0_0_0 + 0.00231601522834171*G0_0_1 + 0.00231601522834169*G0_0_2 + 0.002316015228342*G0_1_0 + 0.00880738185425689*G0_1_1 + 0.00880738185425693*G0_1_2 + 0.00231601522834191*G0_2_0 + 0.00880738185425699*G0_2_1 + 0.00880738185425703*G0_2_2;
    A[2098] = -0.000652398655871017*G0_0_0 + 0.00884000178705033*G0_0_1 + 0.00884000178705032*G0_0_2 + 0.00884000178705007*G0_1_0 - 0.000652398655871157*G0_1_1 - 0.000652398655871207*G0_1_2 + 0.00884000178705008*G0_2_0 - 0.000652398655871036*G0_2_1 - 0.000652398655871026*G0_2_2;
    A[2099] = 0.022670853291513*G0_0_0 - 0.00329461321214798*G0_0_1 - 0.00329461321214801*G0_0_2 - 0.00329461321214799*G0_1_0 - 0.00329461321214781*G0_2_0;
    A[2100] = -0.0254435475789646*G0_0_0 - 0.0127217737894822*G0_0_1 - 0.00130479731174185*G0_0_2 - 0.0127217737894822*G0_1_0 - 0.000652398655870916*G0_1_2 - 0.00130479731174235*G0_2_0 - 0.000652398655871063*G0_2_1;
    A[2101] = -0.0166361657247075*G0_0_0 + 0.00146789697570946*G0_0_1 - 0.0181040627004168*G0_0_2 + 0.00146789697570987*G0_1_0 + 0.0195719596761268*G0_1_1 - 0.0102752788299661*G0_1_2 - 0.0181040627004165*G0_2_0 - 0.010275278829966*G0_2_1;
    A[2102] = 0.0137003717732878*G0_0_0 + 0.0264221455627699*G0_0_1 - 0.00619778723077422*G0_0_2 + 0.0264221455627698*G0_1_0 + 0.0391439193522521*G0_1_1 - 0.00554538857490259*G0_1_2 - 0.00619778723077431*G0_2_0 - 0.00554538857490297*G0_2_1;
    A[2103] = 0.0567586830607659*G0_0_0 + 0.0283793415303823*G0_0_1 + 0.0283793415303826*G0_0_2 + 0.0283793415303828*G0_1_0 + 0.0141896707651911*G0_1_2 + 0.0283793415303833*G0_2_0 + 0.014189670765191*G0_2_1 - 0.00489298991903181*G0_2_2;
    A[2104] = 0.149725491522365*G0_0_0 + 0.121346149991982*G0_0_1 + 0.0748627457611824*G0_0_2 + 0.121346149991981*G0_1_0 + 0.0929668084615982*G0_1_1 + 0.0606730749959901*G0_1_2 + 0.0748627457611822*G0_2_0 + 0.0606730749959909*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2105] = 0.20941996853455*G0_0_0 + 0.104709984267275*G0_0_1 + 0.135698920421141*G0_0_2 + 0.104709984267274*G0_1_0 + 0.0678494602105703*G0_1_2 + 0.135698920421142*G0_2_0 + 0.0678494602105714*G0_2_1 + 0.039143919352252*G0_2_2;
    A[2106] = -0.268135847562928*G0_0_0 - 0.222141742324031*G0_0_1 - 0.194414799449521*G0_0_2 - 0.290643601190474*G0_1_0 - 0.268135847562928*G0_1_1 - 0.321632537344341*G0_1_2 - 0.448850275239161*G0_2_0 - 0.32163253734434*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2107] = 0.065566064915023*G0_0_0 + 0.0376760223765418*G0_0_1 + 0.140428810676206*G0_0_2 + 0.140428810676206*G0_1_0 - 0.0371867233846406*G0_1_1 + 0.186912214907005*G0_1_2 + 0.0376760223765423*G0_2_0 + 0.186912214907004*G0_2_1 - 0.0371867233846404*G0_2_2;
    A[2108] = -0.0137003717732877*G0_0_0 + 0.0714376528178617*G0_0_1 + 0.00619778723077341*G0_0_2 - 0.0264221455627698*G0_1_0 + 0.172233245149912*G0_1_1 + 0.00619778723077404*G0_1_2 + 0.00619778723077432*G0_2_0 + 0.00619778723077506*G0_2_1 + 0.0123955744615469*G0_2_2;
    A[2109] = 0.726119703984281*G0_0_0 + 0.456026660453741*G0_0_1 + 0.456026660453741*G0_0_2 + 0.456026660453741*G0_1_0 + 0.726119703984283*G0_1_1 + 0.456026660453741*G0_1_2 + 0.45602666045374*G0_2_0 + 0.456026660453741*G0_2_1 + 0.912053320907482*G0_2_2;
    A[2110] = -0.149725491522365*G0_0_0 - 0.326851726591308*G0_0_1 - 0.177615534060846*G0_0_2 - 0.121346149991982*G0_1_0 - 0.448197876583291*G0_1_1 - 0.224098938291646*G0_1_2 - 0.0748627457611824*G0_2_0 - 0.224098938291647*G0_2_1 - 0.0371867233846394*G0_2_2;
    A[2111] = -0.20941996853455*G0_0_0 - 0.290643601190473*G0_0_1 - 0.390134396210782*G0_0_2 - 0.104709984267274*G0_1_0 - 0.209419968534549*G0_1_1 - 0.262916658315962*G0_1_2 - 0.135698920421141*G0_2_0 - 0.26291665831596*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2112] = -0.0880738185425674*G0_0_0 - 0.134067923781464*G0_0_1 - 0.0972073997247602*G0_0_2 - 0.134067923781464*G0_1_0 - 0.15657567740901*G0_1_1 - 0.22442513761958*G0_1_2 - 0.0972073997247591*G0_2_0 - 0.22442513761958*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2113] = 0.0195719596761264*G0_0_0 - 0.0552907860850563*G0_0_1 - 0.041101115319865*G0_0_2 - 0.055290786085057*G0_1_0 + 0.0195719596761262*G0_1_1 + 0.00538228891093482*G0_1_2 - 0.0411011153198653*G0_2_0 + 0.00538228891093511*G0_2_1 - 0.008807381854257*G0_2_2;
    A[2114] = -0.0880738185425683*G0_0_0 + 0.012721773789482*G0_0_1 + 0.000652398655870979*G0_0_2 + 0.0127217737894827*G0_1_0 + 0.000652398655872118*G0_1_2 + 0.000652398655870911*G0_2_0 + 0.00065239865587068*G0_2_1 + 0.0013047973117422*G0_2_2;
    A[2115] = 0.0929668084615987*G0_0_0 + 0.363059851992139*G0_0_1 + 0.22801333022687*G0_0_2 + 0.363059851992141*G0_1_0 + 0.0929668084615991*G0_1_1 + 0.22801333022687*G0_1_2 + 0.22801333022687*G0_2_0 + 0.22801333022687*G0_2_1 + 0.456026660453741*G0_2_2;
    A[2116] = 0.0929668084616*G0_0_0 - 0.0283793415303824*G0_0_1 + 0.0322937334656083*G0_0_2 - 0.0283793415303836*G0_1_0 - 0.0141896707651925*G0_1_2 + 0.0322937334656081*G0_2_0 - 0.014189670765191*G0_2_1 - 0.00880738185425697*G0_2_2;
    A[2117] = -0.185933616923198*G0_0_0 - 0.104709984267275*G0_0_1 - 0.195067198105391*G0_0_2 - 0.104709984267274*G0_1_0 - 0.0678494602105696*G0_1_2 - 0.19506719810539*G0_2_0 - 0.0678494602105711*G0_2_1 - 0.292274597830152*G0_2_2;
    A[2118] = -0.0228339529554807*G0_0_0 - 0.0596944770121847*G0_0_1 - 0.0965550010688884*G0_0_2 - 0.0596944770121851*G0_1_0 - 0.0965550010688889*G0_1_1 - 0.0287055408583184*G0_1_2 - 0.0965550010688891*G0_2_0 - 0.0287055408583181*G0_2_1 + 0.0391439193522531*G0_2_2;
    A[2119] = -0.00489298991903138*G0_0_0 - 0.019082660684223*G0_0_1 - 0.0332723314494146*G0_0_2 - 0.0190826606842232*G0_1_0 - 0.0332723314494147*G0_1_1 - 0.0190826606842238*G0_1_2 - 0.0332723314494143*G0_2_0 - 0.019082660684223*G0_2_1 - 0.00489298991903137*G0_2_2;
    A[2120] = -0.0228339529554809*G0_0_0 - 0.0107645778218694*G0_0_1 + 0.00130479731174161*G0_0_2 - 0.010764577821869*G0_1_0 + 0.0013047973117425*G0_1_1 + 0.000652398655870736*G0_1_2 + 0.00130479731174122*G0_2_0 + 0.000652398655871268*G0_2_1;
    A[2121] = 0.019571959676126*G0_0_0 + 0.00538228891093421*G0_0_1 - 0.0552907860850565*G0_0_2 + 0.00538228891093467*G0_1_0 - 0.00880738185425716*G0_1_1 - 0.0411011153198647*G0_1_2 - 0.0552907860850571*G0_2_0 - 0.0411011153198656*G0_2_1 + 0.0195719596761257*G0_2_2;
    A[2122] = 0.0195719596761259*G0_0_0 + 0.0298472385060924*G0_0_1 + 0.0181040627004169*G0_0_2 + 0.0298472385060923*G0_1_0 + 0.0401225173360584*G0_1_1 + 0.0102752788299662*G0_1_2 + 0.0181040627004168*G0_2_0 + 0.0102752788299657*G0_2_1;
    A[2123] = 0.0260959462348354*G0_0_0 + 0.0381653213684467*G0_0_1 + 0.00619778723077428*G0_0_2 + 0.0381653213684464*G0_1_0 + 0.0502346965020579*G0_1_1 + 0.00554538857490266*G0_1_2 + 0.00619778723077368*G0_2_0 + 0.00554538857490344*G0_2_1;
    A[2124] = 0.146789697570946*G0_0_0 + 0.249542485870609*G0_0_1 + 0.44526208263187*G0_0_2 + 0.249542485870611*G0_1_0 + 0.352295274170273*G0_1_1 + 0.249542485870611*G0_1_2 + 0.445262082631871*G0_2_0 + 0.249542485870609*G0_2_1 - 0.0391439193522533*G0_2_2;
    A[2125] = 0.146789697570947*G0_0_0 + 0.053822889109347*G0_0_1 + 0.00489298991903221*G0_0_2 + 0.0538228891093459*G0_1_0 - 0.0391439193522524*G0_1_1 + 0.00489298991903281*G0_1_2 + 0.00489298991903178*G0_2_0 + 0.00489298991903132*G0_2_1 + 0.00978597983806339*G0_2_2;
    A[2126] = -0.0489298991903153*G0_0_0 - 0.141896707651915*G0_0_1 - 0.019571959676126*G0_0_2 - 0.141896707651916*G0_1_0 - 0.234863516113516*G0_1_1 - 0.0195719596761256*G0_1_2 - 0.0195719596761257*G0_2_0 - 0.0195719596761253*G0_2_1 - 0.039143919352252*G0_2_2;
    A[2127] = -0.782878387045048*G0_0_0 - 0.484406001984122*G0_0_1 - 0.484406001984123*G0_0_2 - 0.484406001984124*G0_1_0 - 0.185933616923198*G0_1_1 - 0.288686405222862*G0_1_2 - 0.484406001984124*G0_2_0 - 0.288686405222861*G0_2_1 - 0.039143919352252*G0_2_2;
    A[2128] = 0.00946499969937457*G0_0_0 + 0.00946499969937454*G0_0_1 + 0.00946499969937453*G0_0_2 + 0.0117431758056758*G0_1_0 + 0.0117431758056757*G0_1_1 + 0.0117431758056758*G0_1_2 + 0.0117431758056758*G0_2_0 + 0.0117431758056758*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[2129] = -0.00946499969937473*G0_0_0;
    A[2130] = 0.00472467106581671*G0_0_1 - 0.0298472385060926*G0_1_1;
    A[2131] = 0.00472467106581683*G0_0_2 - 0.0298472385060924*G0_2_2;
    A[2132] = 0.0493213383838383*G0_0_1 + 0.0429604514890968*G0_0_2 + 0.176636936077039*G0_1_1 + 0.0422754329004324*G0_1_2 + 0.0422754329004327*G0_2_1;
    A[2133] = -0.167731694424402*G0_0_1 - 0.0777006799142204*G0_0_2 - 0.334680510461758*G0_1_1 - 0.0838658472122002*G0_1_2 - 0.0838658472122007*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2134] = -0.0777006799142207*G0_0_1 - 0.167731694424402*G0_0_2 + 0.104709984267275*G0_1_1 - 0.0838658472122012*G0_1_2 - 0.0838658472122011*G0_2_1 - 0.334680510461758*G0_2_2;
    A[2135] = 0.0429604514890968*G0_0_1 + 0.0493213383838381*G0_0_2 + 0.0422754329004325*G0_1_2 + 0.042275432900433*G0_2_1 + 0.17663693607704*G0_2_2;
    A[2136] = 0.0292600797158088*G0_0_0 + 0.00329461321214768*G0_0_2 + 0.00329461321214791*G0_2_0;
    A[2137] = -0.0189848008858426*G0_0_0 - 0.00949240044292107*G0_0_2 - 0.00949240044292088*G0_2_0 - 0.000652398655870533*G0_2_2;
    A[2138] = 0.00352295274170272*G0_0_0 + 0.00649136662591511*G0_0_2 + 0.00649136662591487*G0_2_0 + 0.0088073818542567*G0_2_2;
    A[2139] = 0.0429604514890968*G0_0_0 + 0.0422754329004325*G0_0_2 + 0.042275432900433*G0_2_0 + 0.0642612676032813*G0_2_2;
    A[2140] = 0.0292600797158087*G0_0_0 + 0.00329461321214769*G0_0_1 + 0.00329461321214795*G0_1_0;
    A[2141] = -0.0189848008858424*G0_0_0 - 0.00949240044292092*G0_0_1 - 0.00949240044292126*G0_1_0 - 0.000652398655870431*G0_1_1;
    A[2142] = 0.00352295274170233*G0_0_0 + 0.00649136662591467*G0_0_1 + 0.00649136662591529*G0_1_0 + 0.00880738185425654*G0_1_1;
    A[2143] = 0.0429604514890967*G0_0_0 + 0.0422754329004325*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.0642612676032809*G0_1_1;
    A[2144] = -0.0292600797158085*G0_0_0 - 0.0292600797158084*G0_0_1 - 0.0259654665036604*G0_0_2 - 0.0435476102793811*G0_1_0 - 0.043547610279381*G0_1_1 - 0.0377086423093366*G0_1_2 - 0.0377086423093365*G0_2_0 - 0.0377086423093365*G0_2_1 - 0.0318696743392922*G0_2_2;
    A[2145] = 0.0189848008858422*G0_0_0 + 0.0189848008858422*G0_0_1 + 0.00949240044292086*G0_0_2 + 0.0567586830607664*G0_1_0 + 0.0567586830607663*G0_1_1 + 0.0378717419733043*G0_1_2 + 0.0378717419733038*G0_2_0 + 0.0378717419733039*G0_2_1 + 0.012787013655069*G0_2_2;
    A[2146] = -0.00352295274170292*G0_0_0 - 0.00352295274170291*G0_0_1 + 0.00296841388421271*G0_0_2 + 0.104709984267275*G0_1_0 + 0.104709984267275*G0_1_1 + 0.0303691574307894*G0_1_2 + 0.0303691574307896*G0_2_0 + 0.0303691574307896*G0_2_1 - 0.0067849460210568*G0_2_2;
    A[2147] = -0.0429604514890968*G0_0_0 - 0.0429604514890968*G0_0_1 - 0.000685018588664468*G0_0_2 - 0.0422754329004329*G0_1_2 - 0.042275432900433*G0_2_0 - 0.042275432900433*G0_2_1 + 0.0439716694056968*G0_2_2;
    A[2148] = -0.0292600797158086*G0_0_0 - 0.0259654665036606*G0_0_1 - 0.0292600797158086*G0_0_2 - 0.0377086423093369*G0_1_0 - 0.0318696743392923*G0_1_1 - 0.0377086423093369*G0_1_2 - 0.0435476102793809*G0_2_0 - 0.0377086423093366*G0_2_1 - 0.043547610279381*G0_2_2;
    A[2149] = 0.0189848008858426*G0_0_0 + 0.00949240044292128*G0_0_1 + 0.0189848008858426*G0_0_2 + 0.0378717419733043*G0_1_0 + 0.0127870136550692*G0_1_1 + 0.0378717419733042*G0_1_2 + 0.0567586830607661*G0_2_0 + 0.0378717419733044*G0_2_1 + 0.0567586830607662*G0_2_2;
    A[2150] = -0.0035229527417032*G0_0_0 + 0.00296841388421218*G0_0_1 - 0.00352295274170327*G0_0_2 + 0.0303691574307893*G0_1_0 - 0.00678494602105698*G0_1_1 + 0.0303691574307892*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0303691574307892*G0_2_1 + 0.104709984267276*G0_2_2;
    A[2151] = -0.0429604514890967*G0_0_0 - 0.000685018588664079*G0_0_1 - 0.0429604514890967*G0_0_2 - 0.0422754329004324*G0_1_0 + 0.0439716694056969*G0_1_1 - 0.0422754329004323*G0_1_2 - 0.0422754329004327*G0_2_1;
    A[2152] = -0.00329461321214777*G0_0_1 - 0.00329461321214782*G0_0_2 - 0.00329461321214804*G0_1_0 - 0.00913358118219246*G0_1_1 - 0.00913358118219252*G0_1_2 - 0.00329461321214796*G0_2_0 - 0.00913358118219225*G0_2_1 - 0.00913358118219227*G0_2_2;
    A[2153] = -0.00257697469068995*G0_0_1 - 0.00257697469068991*G0_0_2 - 0.00257697469069*G0_1_0 - 0.0156575677409008*G0_1_1 - 0.0156575677409008*G0_1_2 - 0.00257697469069006*G0_2_0 - 0.015657567740901*G0_2_1 - 0.0156575677409009*G0_2_2;
    A[2154] = -0.00257697469069008*G0_0_1 - 0.00257697469069008*G0_0_2 - 0.00257697469068985*G0_1_0 - 0.000652398655870914*G0_1_1 - 0.000652398655870907*G0_1_2 - 0.00257697469069001*G0_2_0 - 0.00065239865587118*G0_2_1 - 0.000652398655871123*G0_2_2;
    A[2155] = -0.0032946132121477*G0_0_1 - 0.0032946132121477*G0_0_2 - 0.00329461321214768*G0_1_0 - 0.00329461321214758*G0_2_0;
    A[2156] = -0.113517366121532*G0_0_0 - 0.0127217737894821*G0_0_1 - 0.0127217737894819*G0_0_2 - 0.012721773789482*G0_1_0 - 0.000652398655870926*G0_1_2 - 0.0127217737894822*G0_2_0 - 0.000652398655870693*G0_2_1;
    A[2157] = 0.149725491522365*G0_0_0 + 0.0748627457611826*G0_0_1 + 0.0283793415303824*G0_0_2 + 0.0748627457611828*G0_1_0 + 0.0195719596761262*G0_1_1 + 0.0141896707651917*G0_1_2 + 0.0283793415303825*G0_2_0 + 0.0141896707651911*G0_2_1;
    A[2158] = 0.0234863516113523*G0_0_0 - 0.0225077536275449*G0_0_1 + 0.104709984267275*G0_0_2 - 0.022507753627545*G0_1_0 - 0.156575677409009*G0_1_1 + 0.0678494602105704*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0678494602105708*G0_2_1;
    A[2159] = 0.149725491522365*G0_0_0 + 0.0283793415303826*G0_0_1 + 0.0748627457611824*G0_0_2 + 0.0283793415303837*G0_1_0 + 0.0141896707651919*G0_1_2 + 0.0748627457611832*G0_2_0 + 0.0141896707651916*G0_2_1 + 0.0195719596761266*G0_2_2;
    A[2160] = -0.540186087061083*G0_0_0 - 0.270093043530541*G0_0_1 - 0.27009304353054*G0_0_2 - 0.270093043530541*G0_1_0 + 0.0929668084615999*G0_1_1 - 0.13504652176527*G0_1_2 - 0.270093043530541*G0_2_0 - 0.135046521765269*G0_2_1 + 0.0929668084616012*G0_2_2;
    A[2161] = 0.0234863516113515*G0_0_0 + 0.104709984267275*G0_0_1 - 0.0225077536275454*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105709*G0_1_2 - 0.0225077536275446*G0_2_0 + 0.0678494602105712*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2162] = 0.113517366121531*G0_0_0 + 0.100795592332049*G0_0_1 + 0.100795592332049*G0_0_2 + 0.198655390712681*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919138*G0_1_2 + 0.198655390712681*G0_2_0 + 0.166035457919138*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2163] = -0.149725491522365*G0_0_0 - 0.0748627457611819*G0_0_1 - 0.121346149991981*G0_0_2 - 0.177615534060846*G0_1_0 - 0.0371867233846392*G0_1_1 - 0.224098938291645*G0_1_2 - 0.326851726591308*G0_2_0 - 0.224098938291645*G0_2_1 - 0.448197876583291*G0_2_2;
    A[2164] = -0.0234863516113522*G0_0_0 - 0.0459941052388971*G0_0_1 + 0.0812236326559233*G0_0_2 + 0.0225077536275451*G0_1_0 - 0.268135847562929*G0_1_1 + 0.053496689781411*G0_1_2 - 0.104709984267276*G0_2_0 + 0.0534966897814114*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2165] = -0.149725491522365*G0_0_0 - 0.121346149991982*G0_0_1 - 0.0748627457611824*G0_0_2 - 0.326851726591308*G0_1_0 - 0.448197876583291*G0_1_1 - 0.224098938291647*G0_1_2 - 0.177615534060846*G0_2_0 - 0.224098938291646*G0_2_1 - 0.0371867233846394*G0_2_2;
    A[2166] = 0.540186087061082*G0_0_0 + 0.270093043530541*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530541*G0_1_0 + 0.726119703984283*G0_1_1 + 0.270093043530542*G0_1_2 + 0.270093043530541*G0_2_0 + 0.270093043530542*G0_2_1 + 0.726119703984283*G0_2_2;
    A[2167] = -0.0234863516113517*G0_0_0 + 0.0812236326559234*G0_0_1 - 0.0459941052388966*G0_0_2 - 0.104709984267275*G0_1_0 - 0.20941996853455*G0_1_1 + 0.0534966897814122*G0_1_2 + 0.0225077536275445*G0_2_0 + 0.0534966897814113*G0_2_1 - 0.26813584756293*G0_2_2;
    A[2168] = 0.0127217737894816*G0_0_1 + 0.0120693751336106*G0_0_2 + 0.0127217737894828*G0_1_0 + 0.039143919352253*G0_1_1 + 0.0446893079271557*G0_1_2 + 0.0120693751336115*G0_2_0 + 0.0446893079271551*G0_2_1 + 0.0502346965020571*G0_2_2;
    A[2169] = 0.0181040627004174*G0_0_1 + 0.0102752788299667*G0_0_2 + 0.0181040627004166*G0_1_0 + 0.0195719596761266*G0_1_1 + 0.0298472385060924*G0_1_2 + 0.0102752788299663*G0_2_0 + 0.0298472385060931*G0_2_1 + 0.0401225173360593*G0_2_2;
    A[2170] = 0.0127217737894821*G0_0_1 + 0.0120693751336112*G0_0_2 + 0.0127217737894816*G0_1_0 + 0.000652398655870062*G0_1_2 + 0.0120693751336107*G0_2_0 + 0.00065239865587085*G0_2_1 + 0.00130479731174154*G0_2_2;
    A[2171] = -0.0283793415303834*G0_0_1 - 0.0141896707651912*G0_0_2 - 0.0283793415303837*G0_1_0 + 0.0929668084615991*G0_1_1 + 0.0322937334656073*G0_1_2 - 0.0141896707651915*G0_2_0 + 0.0322937334656083*G0_2_1 - 0.00880738185425669*G0_2_2;
    A[2172] = -0.0283793415303827*G0_0_1 - 0.0141896707651918*G0_0_2 - 0.0283793415303827*G0_1_0 - 0.0141896707651912*G0_1_2 - 0.0141896707651913*G0_2_0 - 0.0141896707651916*G0_2_1 - 0.033272331449415*G0_2_2;
    A[2173] = -0.104709984267275*G0_0_1 - 0.036860524056704*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105707*G0_1_2 - 0.0368605240567045*G0_2_0 - 0.0678494602105713*G0_2_1 - 0.0965550010688895*G0_2_2;
    A[2174] = 0.0120693751336111*G0_0_1 + 0.0127217737894821*G0_0_2 + 0.0120693751336119*G0_1_0 + 0.0502346965020576*G0_1_1 + 0.0446893079271548*G0_1_2 + 0.0127217737894825*G0_2_0 + 0.0446893079271553*G0_2_1 + 0.039143919352253*G0_2_2;
    A[2175] = 0.0102752788299664*G0_0_1 + 0.018104062700417*G0_0_2 + 0.0102752788299661*G0_1_0 + 0.0401225173360588*G0_1_1 + 0.029847238506093*G0_1_2 + 0.0181040627004168*G0_2_0 + 0.0298472385060927*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2176] = 0.012069375133611*G0_0_1 + 0.012721773789482*G0_0_2 + 0.0120693751336108*G0_1_0 + 0.00130479731174129*G0_1_1 + 0.000652398655870975*G0_1_2 + 0.0127217737894814*G0_2_0 + 0.000652398655870885*G0_2_1;
    A[2177] = -0.014189670765192*G0_0_1 - 0.0283793415303838*G0_0_2 - 0.0141896707651918*G0_1_0 - 0.00880738185425697*G0_1_1 + 0.0322937334656075*G0_1_2 - 0.0283793415303837*G0_2_0 + 0.032293733465608*G0_2_1 + 0.0929668084615992*G0_2_2;
    A[2178] = -0.0141896707651913*G0_0_1 - 0.0283793415303827*G0_0_2 - 0.0141896707651912*G0_1_0 - 0.0332723314494146*G0_1_1 - 0.0141896707651917*G0_1_2 - 0.0283793415303823*G0_2_0 - 0.0141896707651914*G0_2_1;
    A[2179] = -0.0368605240567037*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0368605240567045*G0_1_0 - 0.096555001068889*G0_1_1 - 0.0678494602105705*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105711*G0_2_1;
    A[2180] = -0.092966808461599*G0_0_1 - 0.0929668084615993*G0_0_2 - 0.0929668084616007*G0_1_0 - 0.234863516113517*G0_1_1 - 0.215291556437389*G0_1_2 - 0.0929668084616006*G0_2_0 - 0.21529155643739*G0_2_1 - 0.234863516113516*G0_2_2;
    A[2181] = -0.0929668084615999*G0_0_1 - 0.0929668084615996*G0_0_2 - 0.0929668084615986*G0_1_0 - 0.0391439193522526*G0_1_1 - 0.0440369092712852*G0_1_2 - 0.0929668084615985*G0_2_0 - 0.0440369092712846*G0_2_1 - 0.0391439193522529*G0_2_2;
    A[2182] = 0.102752788299662*G0_0_1 + 0.298472385060924*G0_0_2 + 0.102752788299663*G0_1_0 + 0.352295274170273*G0_1_1 + 0.102752788299662*G0_1_2 + 0.298472385060925*G0_2_0 + 0.102752788299665*G0_2_1 - 0.185933616923201*G0_2_2;
    A[2183] = 0.298472385060924*G0_0_1 + 0.102752788299662*G0_0_2 + 0.298472385060925*G0_1_0 - 0.185933616923199*G0_1_1 + 0.102752788299664*G0_1_2 + 0.102752788299662*G0_2_0 + 0.102752788299661*G0_2_1 + 0.352295274170273*G0_2_2;
    A[2184] = 0.013742125287264*G0_0_0 + 0.013742125287264*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202552*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202552*G0_1_2 + 0.0189195610202552*G0_2_0 + 0.0189195610202551*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[2185] = -0.0137421252872639*G0_0_0;
    A[2186] = 0.00517743573299127*G0_0_1 + 0.0189195610202553*G0_1_1;
    A[2187] = -0.0143945239431349*G0_0_2 + 0.0335985307773502*G0_2_2;
    A[2188] = -0.0477555816097473*G0_0_1 - 0.0118736555368495*G0_0_2 - 0.111886369481854*G0_1_1 - 0.0212681961813902*G0_1_2 - 0.0212681961813898*G0_2_1;
    A[2189] = 0.0823327103709033*G0_0_1 - 0.00182671623643887*G0_0_2 + 0.20941996853455*G0_1_1 + 0.0359471659384851*G0_1_2 + 0.0359471659384844*G0_2_1 - 0.052844291125541*G0_2_2;
    A[2190] = -0.146659217839771*G0_0_1 - 0.0109602974186303*G0_0_2 - 0.271397840842283*G0_1_1 - 0.0733296089198861*G0_1_2 - 0.0733296089198854*G0_2_1 + 0.121346149991983*G0_2_2;
    A[2191] = -0.124738623002511*G0_0_1 - 0.192588083213082*G0_0_2 - 0.0962940416065403*G0_1_2 - 0.0962940416065413*G0_2_1 - 0.257045070413125*G0_2_2;
    A[2192] = 0.0641307878721064*G0_0_0 + 0.0093945406445406*G0_0_2 + 0.00939454064454042*G0_2_0;
    A[2193] = -0.127087258163646*G0_0_0 - 0.0377738821749238*G0_0_2 - 0.0377738821749234*G0_2_0 - 0.0150051690850299*G0_2_2;
    A[2194] = 0.124738623002511*G0_0_0 + 0.0623693115012557*G0_0_2 + 0.0623693115012557*G0_2_0 + 0.0476251018785737*G0_2_2;
    A[2195] = -0.124738623002511*G0_0_0 - 0.0962940416065414*G0_0_2 - 0.0962940416065412*G0_2_0 - 0.128522535206563*G0_2_2;
    A[2196] = 0.0187890812890811*G0_0_0 + 0.00939454064454072*G0_0_1 + 0.00939454064454043*G0_1_0;
    A[2197] = -0.014874689353856*G0_0_0 - 0.0149399292194429*G0_0_1 - 0.0149399292194429*G0_1_0 - 0.01500516908503*G0_1_1;
    A[2198] = -0.00117431758056766*G0_0_0 - 0.00123955744615495*G0_0_1 - 0.00123955744615483*G0_1_0 - 0.00130479731174205*G0_1_1;
    A[2199] = -0.0118736555368495*G0_0_0 - 0.0212681961813898*G0_0_1 - 0.0212681961813903*G0_1_0 - 0.0306627368259307*G0_1_1;
    A[2200] = -0.0641307878721066*G0_0_0 - 0.0641307878721066*G0_0_1 - 0.0547362472275661*G0_0_2 - 0.111886369481854*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0906181733004638*G0_1_2 - 0.0906181733004637*G0_2_0 - 0.0906181733004637*G0_2_1 - 0.0693499771190733*G0_2_2;
    A[2201] = 0.127087258163646*G0_0_0 + 0.127087258163646*G0_0_1 + 0.0893133759887222*G0_0_2 + 0.209419968534549*G0_1_0 + 0.209419968534549*G0_1_1 + 0.173472802596064*G0_1_2 + 0.173472802596065*G0_2_0 + 0.173472802596065*G0_2_1 + 0.0846813455320386*G0_2_2;
    A[2202] = -0.124738623002511*G0_0_0 - 0.124738623002511*G0_0_1 - 0.0623693115012552*G0_0_2 - 0.271397840842282*G0_1_0 - 0.271397840842282*G0_1_1 - 0.198068231922396*G0_1_2 - 0.198068231922397*G0_2_0 - 0.198068231922397*G0_2_1 - 0.00339247301052763*G0_2_2;
    A[2203] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0284445813959699*G0_0_2 + 0.0962940416065406*G0_1_2 + 0.0962940416065412*G0_2_0 + 0.0962940416065412*G0_2_1 - 0.064456987200043*G0_2_2;
    A[2204] = -0.0187890812890811*G0_0_0 - 0.00939454064454057*G0_0_1 - 0.0187890812890811*G0_0_2 - 0.0400572774704711*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.0400572774704711*G0_1_2 - 0.0613254736518619*G0_2_0 - 0.0400572774704713*G0_2_1 - 0.0613254736518619*G0_2_2;
    A[2205] = 0.0148746893538556*G0_0_0 - 6.52398655870356e-05*G0_0_1 + 0.0148746893538555*G0_0_2 + 0.0136351319077009*G0_1_0 - 0.000130479731174048*G0_1_1 + 0.013635131907701*G0_1_2 - 0.0254435475789647*G0_2_0 + 0.0136351319077011*G0_2_1 - 0.0254435475789647*G0_2_2;
    A[2206] = 0.00117431758056765*G0_0_0 - 6.52398655872122e-05*G0_0_1 + 0.00117431758056762*G0_0_2 - 0.0137656116388753*G0_1_0 - 0.000130479731174683*G0_1_1 - 0.0137656116388753*G0_1_2 - 0.0528442911255405*G0_2_0 - 0.0137656116388756*G0_2_1 - 0.0528442911255405*G0_2_2;
    A[2207] = 0.0118736555368495*G0_0_0 - 0.0093945406445405*G0_0_1 + 0.0118736555368495*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.0187890812890808*G0_1_1 + 0.0212681961813902*G0_1_2 + 0.0212681961813907*G0_2_1;
    A[2208] = -0.00939454064454051*G0_0_1 - 0.00939454064454044*G0_0_2 - 0.00939454064454025*G0_1_0 - 0.0306627368259305*G0_1_1 - 0.0306627368259305*G0_1_2 - 0.0093945406445404*G0_2_0 - 0.030662736825931*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[2209] = -6.52398655871478e-05*G0_0_1 - 6.52398655871789e-05*G0_0_2 - 6.52398655873144e-05*G0_1_0 - 0.00130479731174191*G0_1_1 - 0.00130479731174191*G0_1_2 - 6.5239865587115e-05*G0_2_0 - 0.00130479731174186*G0_2_1 - 0.00130479731174191*G0_2_2;
    A[2210] = -6.52398655871797e-05*G0_0_1 - 6.52398655871864e-05*G0_0_2 - 6.52398655870905e-05*G0_1_0 - 0.0150051690850299*G0_1_1 - 0.0150051690850299*G0_1_2 - 6.5239865586971e-05*G0_2_0 - 0.0150051690850299*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[2211] = -0.00939454064454082*G0_0_1 - 0.00939454064454077*G0_0_2 - 0.00939454064454064*G0_1_0 - 0.00939454064454086*G0_2_0;
    A[2212] = -0.105688582251081*G0_0_0 - 0.0528442911255408*G0_0_1 - 0.0300103381700597*G0_0_2 - 0.0528442911255403*G0_1_0 - 0.01500516908503*G0_1_2 - 0.0300103381700593*G0_2_0 - 0.01500516908503*G0_2_1;
    A[2213] = 0.0137003717732882*G0_0_0 + 0.0198981590040618*G0_0_1 - 0.0127217737894825*G0_0_2 + 0.0198981590040614*G0_1_0 + 0.0260959462348348*G0_1_1 - 0.0120693751336115*G0_1_2 - 0.0127217737894821*G0_2_0 - 0.012069375133611*G0_2_1;
    A[2214] = -0.053496689781411*G0_0_0 - 0.000652398655870863*G0_0_1 - 0.0528442911255395*G0_0_2 - 0.000652398655870085*G0_1_0 + 0.0521918924696702*G0_1_1 - 0.03783912204051*G0_1_2 - 0.05284429112554*G0_2_0 - 0.0378391220405103*G0_2_1;
    A[2215] = 0.20941996853455*G0_0_0 + 0.104709984267276*G0_0_1 + 0.073721048113409*G0_0_2 + 0.104709984267274*G0_1_0 + 0.0368605240567044*G0_1_2 + 0.0737210481134084*G0_2_0 + 0.036860524056705*G0_2_1 - 0.0228339529554806*G0_2_2;
    A[2216] = 0.0234863516113518*G0_0_0 - 0.0812236326559227*G0_0_1 + 0.0459941052388963*G0_0_2 - 0.0812236326559233*G0_1_0 - 0.185933616923198*G0_1_1 + 0.00913358118219277*G0_1_2 + 0.0459941052388967*G0_2_0 + 0.00913358118219213*G0_2_1 - 0.0880738185425688*G0_2_2;
    A[2217] = -0.542795681684566*G0_0_0 - 0.271397840842285*G0_0_1 - 0.271397840842283*G0_0_2 - 0.271397840842282*G0_1_0 - 0.135698920421141*G0_1_2 - 0.271397840842284*G0_2_0 - 0.135698920421142*G0_2_1 + 0.0521918924696707*G0_2_2;
    A[2218] = 0.105688582251081*G0_0_0 + 0.0528442911255402*G0_0_1 + 0.075678244081021*G0_0_2 + 0.105036183595209*G0_1_0 + 0.10568858225108*G0_1_1 + 0.12787013655069*G0_1_2 + 0.18006202902036*G0_2_0 + 0.12787013655069*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2219] = -0.0137003717732886*G0_0_0 + 0.00619778723077336*G0_0_1 - 0.0264221455627702*G0_0_2 + 0.00619778723077429*G0_1_0 + 0.0123955744615476*G0_1_1 + 0.00619778723077388*G0_1_2 + 0.0714376528178614*G0_2_0 + 0.00619778723077465*G0_2_1 + 0.172233245149912*G0_2_2;
    A[2220] = 0.0534966897814108*G0_0_0 + 0.0528442911255406*G0_0_1 + 0.00065239865587138*G0_0_2 + 0.000652398655869982*G0_1_0 + 0.105688582251081*G0_1_1 - 0.02218155429961*G0_1_2 + 0.0528442911255399*G0_2_0 - 0.022181554299611*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2221] = -0.20941996853455*G0_0_0 - 0.104709984267274*G0_0_1 - 0.135698920421141*G0_0_2 - 0.290643601190473*G0_1_0 - 0.209419968534549*G0_1_1 - 0.26291665831596*G0_1_2 - 0.390134396210782*G0_2_0 - 0.262916658315962*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2222] = -0.0234863516113517*G0_0_0 - 0.104709984267275*G0_0_1 + 0.0225077536275445*G0_0_2 + 0.0812236326559234*G0_1_0 - 0.20941996853455*G0_1_1 + 0.0534966897814113*G0_1_2 - 0.0459941052388966*G0_2_0 + 0.0534966897814122*G0_2_1 - 0.26813584756293*G0_2_2;
    A[2223] = 0.542795681684566*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842282*G0_1_0 + 0.542795681684565*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842282*G0_2_1 + 0.647179466623907*G0_2_2;
    A[2224] = 0.0528442911255401*G0_0_1 + 0.0378391220405105*G0_0_2 + 0.0528442911255396*G0_1_0 + 0.0521918924696691*G0_1_1 + 0.090031014510179*G0_1_2 + 0.0378391220405099*G0_2_0 + 0.0900310145101802*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2225] = 0.00619778723077332*G0_0_1 + 0.00554538857490261*G0_0_2 + 0.00619778723077375*G0_1_0 + 0.0260959462348348*G0_1_1 + 0.0381653213684461*G0_1_2 + 0.00554538857490223*G0_2_0 + 0.0381653213684456*G0_2_1 + 0.0502346965020571*G0_2_2;
    A[2226] = 0.0528442911255408*G0_0_1 + 0.0378391220405112*G0_0_2 + 0.0528442911255407*G0_1_0 + 0.01500516908503*G0_1_2 + 0.0378391220405111*G0_2_0 + 0.0150051690850301*G0_2_1 + 0.0300103381700603*G0_2_2;
    A[2227] = -0.104709984267275*G0_0_1 - 0.067849460210571*G0_0_2 - 0.104709984267274*G0_1_0 - 0.185933616923198*G0_1_1 - 0.195067198105389*G0_1_2 - 0.0678494602105702*G0_2_0 - 0.195067198105391*G0_2_1 - 0.292274597830152*G0_2_2;
    A[2228] = -0.104709984267276*G0_0_1 - 0.0678494602105713*G0_0_2 - 0.104709984267275*G0_1_0 - 0.036860524056704*G0_1_2 - 0.0678494602105711*G0_2_0 - 0.0368605240567049*G0_2_1 - 0.0965550010688893*G0_2_2;
    A[2229] = 0.271397840842284*G0_0_1 + 0.135698920421143*G0_0_2 + 0.271397840842283*G0_1_0 + 0.13569892042114*G0_1_2 + 0.135698920421141*G0_2_0 + 0.135698920421142*G0_2_1 + 0.323589733311954*G0_2_2;
    A[2230] = 0.0150051690850296*G0_0_1 + 0.0300103381700594*G0_0_2 + 0.0150051690850295*G0_1_0 + 0.0300103381700594*G0_1_1 - 0.00782878387045056*G0_1_2 + 0.0300103381700598*G0_2_0 - 0.00782878387045105*G0_2_1 - 0.045667905910962*G0_2_2;
    A[2231] = 0.000652398655871151*G0_0_1 + 0.00130479731174239*G0_0_2 + 0.000652398655871068*G0_1_0 + 0.0013047973117417*G0_1_1 - 0.0107645778218693*G0_1_2 + 0.00130479731174162*G0_2_0 - 0.0107645778218697*G0_2_1 - 0.0228339529554808*G0_2_2;
    A[2232] = 0.0150051690850302*G0_0_1 + 0.0300103381700598*G0_0_2 + 0.0150051690850301*G0_1_0 + 0.0300103381700601*G0_1_1 + 0.0150051690850302*G0_1_2 + 0.0300103381700608*G0_2_0 + 0.01500516908503*G0_2_1;
    A[2233] = 0.000652398655871187*G0_0_1 + 0.0127217737894819*G0_0_2 + 0.000652398655871117*G0_1_0 + 0.00130479731174228*G0_1_1 + 0.000652398655871226*G0_1_2 + 0.0127217737894827*G0_2_0 + 0.000652398655871577*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[2234] = 0.000652398655870845*G0_0_1 + 0.0127217737894824*G0_0_2 + 0.00065239865587091*G0_1_0 + 0.00130479731174212*G0_1_1 + 0.0120693751336113*G0_1_2 + 0.0127217737894818*G0_2_0 + 0.0120693751336115*G0_2_1;
    A[2235] = 0.0150051690850296*G0_0_1 + 0.0528442911255395*G0_0_2 + 0.0150051690850301*G0_1_0 + 0.0300103381700595*G0_1_1 + 0.0378391220405101*G0_1_2 + 0.0528442911255405*G0_2_0 + 0.03783912204051*G0_2_1;
    A[2236] = -0.0260959462348344*G0_0_1 - 0.0750258454251496*G0_0_2 - 0.0260959462348341*G0_1_0 - 0.0521918924696684*G0_1_1 - 0.0424059126316064*G0_1_2 - 0.0750258454251494*G0_2_0 - 0.0424059126316054*G0_2_1 + 0.143527704291594*G0_2_2;
    A[2237] = -0.026095946234835*G0_0_1 - 0.0750258454251514*G0_0_2 - 0.0260959462348353*G0_1_0 - 0.0521918924696699*G0_1_1 - 0.0260959462348352*G0_1_2 - 0.0750258454251517*G0_2_0 - 0.0260959462348346*G0_2_1 + 0.0456679059109609*G0_2_2;
    A[2238] = -0.0260959462348353*G0_0_1 - 0.0587158790283782*G0_0_2 - 0.0260959462348357*G0_1_0 - 0.0521918924696709*G0_1_1 - 0.00978597983806372*G0_1_2 - 0.0587158790283791*G0_2_0 - 0.00978597983806488*G0_2_1 + 0.176147637085136*G0_2_2;
    A[2239] = 0.185933616923198*G0_0_1 + 0.316413348097373*G0_0_2 + 0.185933616923198*G0_1_0 + 0.371867233846397*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097374*G0_2_0 + 0.185933616923199*G0_2_1 - 0.150051690850302*G0_2_2;
    A[2240] = 0.0335985307773498*G0_0_0 + 0.0335985307773498*G0_0_1 + 0.0335985307773498*G0_0_2 + 0.0479930547204846*G0_1_0 + 0.0479930547204846*G0_1_1 + 0.0479930547204846*G0_1_2 + 0.0335985307773497*G0_2_0 + 0.0335985307773497*G0_2_1 + 0.0335985307773497*G0_2_2;
    A[2241] = 0.0189195610202554*G0_0_0 + 0.00517743573299117*G0_1_0;
    A[2242] = -0.013742125287264*G0_1_1;
    A[2243] = 0.0051774357329913*G0_1_2 + 0.0189195610202554*G0_2_2;
    A[2244] = 0.0187890812890814*G0_1_1 + 0.00939454064454083*G0_1_2 + 0.0093945406445412*G0_2_1;
    A[2245] = -0.014874689353856*G0_1_1 - 0.014939929219443*G0_1_2 - 0.0149399292194436*G0_2_1 - 0.0150051690850298*G0_2_2;
    A[2246] = -0.00117431758056745*G0_1_1 - 0.00123955744615462*G0_1_2 - 0.00123955744615397*G0_2_1 - 0.00130479731174177*G0_2_2;
    A[2247] = -0.0118736555368498*G0_1_1 - 0.0212681961813904*G0_1_2 - 0.0212681961813908*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[2248] = -0.061325473651862*G0_0_0 - 0.0212681961813904*G0_0_2 - 0.0425363923627812*G0_1_0 - 0.0118736555368504*G0_1_2 - 0.0212681961813905*G0_2_0;
    A[2249] = -0.0254435475789637*G0_0_0 - 0.0390786794866649*G0_0_2 - 0.0403182369328191*G0_1_0 - 0.0540186087061073*G0_1_2 - 0.0390786794866649*G0_2_0 - 0.05284429112554*G0_2_2;
    A[2250] = -0.0528442911255404*G0_0_0 - 0.0390786794866652*G0_0_2 - 0.0540186087061086*G0_1_0 - 0.0403182369328204*G0_1_2 - 0.0390786794866651*G0_2_0 - 0.0254435475789642*G0_2_2;
    A[2251] = -0.0212681961813904*G0_0_2 - 0.0118736555368498*G0_1_0 - 0.0425363923627807*G0_1_2 - 0.0212681961813908*G0_2_0 - 0.0613254736518622*G0_2_2;
    A[2252] = -0.030662736825931*G0_0_0 - 0.0212681961813904*G0_0_1 - 0.0212681961813906*G0_1_0 - 0.0118736555368503*G0_1_1;
    A[2253] = -0.00130479731174161*G0_0_0 - 0.00123955744615443*G0_0_1 - 0.00123955744615445*G0_1_0 - 0.00117431758056696*G0_1_1;
    A[2254] = -0.0150051690850299*G0_0_0 - 0.0149399292194428*G0_0_1 - 0.0149399292194432*G0_1_0 - 0.0148746893538563*G0_1_1;
    A[2255] = 0.00939454064454066*G0_0_1 + 0.00939454064454087*G0_1_0 + 0.0187890812890815*G0_1_1;
    A[2256] = -0.257045070413123*G0_0_0 - 0.257045070413123*G0_0_1 - 0.160751028806583*G0_0_2 - 0.0644569872000418*G0_1_0 - 0.0644569872000418*G0_1_1 - 0.0929015685960116*G0_1_2 - 0.160751028806582*G0_2_0 - 0.160751028806582*G0_2_1 - 0.0644569872000416*G0_2_2;
    A[2257] = 0.121346149991982*G0_0_0 + 0.121346149991982*G0_0_1 + 0.194675758911868*G0_0_2 + 0.132306447410613*G0_1_0 + 0.132306447410613*G0_1_1 + 0.0589768384907261*G0_1_2 + 0.194675758911868*G0_2_0 + 0.194675758911868*G0_2_1 - 0.00339247301052939*G0_2_2;
    A[2258] = -0.0528442911255402*G0_0_0 - 0.0528442911255402*G0_0_1 - 0.0887914570640255*G0_0_2 - 0.051017574889102*G0_1_0 - 0.051017574889102*G0_1_1 - 0.00463203045668253*G0_1_2 - 0.0887914570640254*G0_2_0 - 0.0887914570640254*G0_2_1 + 0.0846813455320403*G0_2_2;
    A[2259] = 0.0212681961813903*G0_0_2 + 0.0118736555368497*G0_1_0 + 0.0118736555368498*G0_1_1 - 0.0146137298915079*G0_1_2 + 0.0212681961813907*G0_2_0 + 0.0212681961813907*G0_2_1 - 0.0693499771190743*G0_2_2;
    A[2260] = -0.128522535206562*G0_0_0 - 0.032228493600021*G0_0_1 - 0.128522535206561*G0_0_2 - 0.032228493600021*G0_1_0 - 0.0606730749959907*G0_1_1 - 0.032228493600021*G0_1_2 - 0.128522535206562*G0_2_0 - 0.0322284936000208*G0_2_1 - 0.128522535206562*G0_2_2;
    A[2261] = 0.0476251018785734*G0_0_0 - 0.0147442096226816*G0_0_1 + 0.0476251018785734*G0_0_2 - 0.0147442096226819*G0_1_0 + 0.0476251018785734*G0_1_1 - 0.0147442096226818*G0_1_2 + 0.0476251018785734*G0_2_0 - 0.0147442096226817*G0_2_1 + 0.0476251018785733*G0_2_2;
    A[2262] = -0.0150051690850296*G0_0_0 + 0.0227687130898934*G0_0_1 - 0.0150051690850295*G0_0_2 + 0.0227687130898939*G0_1_0 - 0.0665446628988285*G0_1_1 + 0.0227687130898939*G0_1_2 - 0.01500516908503*G0_2_0 + 0.022768713089893*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[2263] = -0.00939454064454053*G0_0_1 - 0.00939454064454092*G0_1_0 + 0.0453417065830254*G0_1_1 - 0.00939454064454094*G0_1_2 - 0.0093945406445401*G0_2_1;
    A[2264] = -0.0644569872000417*G0_0_0 - 0.160751028806582*G0_0_1 - 0.160751028806582*G0_0_2 - 0.0929015685960117*G0_1_0 - 0.0644569872000421*G0_1_1 - 0.0644569872000421*G0_1_2 - 0.160751028806582*G0_2_0 - 0.257045070413123*G0_2_1 - 0.257045070413123*G0_2_2;
    A[2265] = -0.00339247301052907*G0_0_0 + 0.194675758911868*G0_0_1 + 0.194675758911868*G0_0_2 + 0.0589768384907264*G0_1_0 + 0.132306447410612*G0_1_1 + 0.132306447410612*G0_1_2 + 0.194675758911868*G0_2_0 + 0.121346149991982*G0_2_1 + 0.121346149991982*G0_2_2;
    A[2266] = 0.0846813455320395*G0_0_0 - 0.0887914570640256*G0_0_1 - 0.0887914570640255*G0_0_2 - 0.00463203045668301*G0_1_0 - 0.0510175748891021*G0_1_1 - 0.0510175748891021*G0_1_2 - 0.0887914570640252*G0_2_0 - 0.0528442911255403*G0_2_1 - 0.0528442911255402*G0_2_2;
    A[2267] = -0.0693499771190739*G0_0_0 + 0.0212681961813905*G0_0_1 + 0.0212681961813904*G0_0_2 - 0.0146137298915074*G0_1_0 + 0.0118736555368504*G0_1_1 + 0.0118736555368504*G0_1_2 + 0.0212681961813903*G0_2_0;
    A[2268] = -0.0456679059109597*G0_0_0 - 0.0756782440810205*G0_0_1 - 0.0378391220405099*G0_0_2 - 0.0756782440810203*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0528442911255407*G0_1_2 - 0.0378391220405094*G0_2_0 - 0.0528442911255401*G0_2_1;
    A[2269] = -0.0228339529554799*G0_0_0 - 0.0241387502672215*G0_0_1 - 0.0120693751336105*G0_0_2 - 0.0241387502672225*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.0127217737894821*G0_1_2 - 0.0120693751336113*G0_2_0 - 0.0127217737894815*G0_2_1;
    A[2270] = -0.03001033817006*G0_0_1 - 0.0150051690850296*G0_0_2 - 0.0300103381700597*G0_1_0 - 0.0600206763401202*G0_1_1 - 0.0300103381700596*G0_1_2 - 0.0150051690850295*G0_2_0 - 0.0300103381700601*G0_2_1;
    A[2271] = -0.0880738185425679*G0_0_0 - 0.100795592332049*G0_0_1 - 0.0887262171984379*G0_0_2 - 0.10079559233205*G0_1_0 - 0.113517366121531*G0_1_1 - 0.100795592332049*G0_1_2 - 0.0887262171984385*G0_2_0 - 0.100795592332049*G0_2_1 - 0.088073818542567*G0_2_2;
    A[2272] = -0.0127217737894818*G0_0_1 - 0.0120693751336112*G0_0_2 - 0.0127217737894816*G0_1_0 - 0.0254435475789635*G0_1_1 - 0.0241387502672219*G0_1_2 - 0.0120693751336106*G0_2_0 - 0.0241387502672218*G0_2_1 - 0.0228339529554802*G0_2_2;
    A[2273] = -0.0528442911255406*G0_0_1 - 0.0378391220405106*G0_0_2 - 0.0528442911255405*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0756782440810213*G0_1_2 - 0.0378391220405107*G0_2_0 - 0.075678244081022*G0_2_1 - 0.045667905910961*G0_2_2;
    A[2274] = 0.0521918924696692*G0_0_0 + 0.323589733311952*G0_0_1 + 0.187890812890811*G0_0_2 + 0.323589733311953*G0_1_0 + 0.0521918924696695*G0_1_1 + 0.187890812890812*G0_1_2 + 0.187890812890811*G0_2_0 + 0.18789081289081*G0_2_1 + 0.323589733311953*G0_2_2;
    A[2275] = -0.0228339529554807*G0_0_0 - 0.0965550010688884*G0_0_1 - 0.0596944770121852*G0_0_2 - 0.0965550010688895*G0_1_0 + 0.0391439193522523*G0_1_1 - 0.028705540858319*G0_1_2 - 0.0596944770121848*G0_2_0 - 0.0287055408583175*G0_2_1 - 0.0965550010688888*G0_2_2;
    A[2276] = 0.0300103381700597*G0_0_1 + 0.0150051690850299*G0_0_2 + 0.0300103381700597*G0_1_0 - 0.0456679059109615*G0_1_1 - 0.00782878387045053*G0_1_2 + 0.0150051690850297*G0_2_0 - 0.00782878387045154*G0_2_1 + 0.0300103381700597*G0_2_2;
    A[2277] = -0.0880738185425674*G0_0_0 - 0.134067923781464*G0_0_1 - 0.0972073997247591*G0_0_2 - 0.134067923781464*G0_1_0 - 0.15657567740901*G0_1_1 - 0.22442513761958*G0_1_2 - 0.0972073997247602*G0_2_0 - 0.22442513761958*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2278] = 0.0127217737894828*G0_0_1 + 0.0120693751336115*G0_0_2 + 0.0127217737894816*G0_1_0 + 0.039143919352253*G0_1_1 + 0.0446893079271551*G0_1_2 + 0.0120693751336106*G0_2_0 + 0.0446893079271556*G0_2_1 + 0.0502346965020571*G0_2_2;
    A[2279] = 0.0528442911255396*G0_0_1 + 0.0378391220405099*G0_0_2 + 0.0528442911255401*G0_1_0 + 0.0521918924696691*G0_1_1 + 0.0900310145101802*G0_1_2 + 0.0378391220405105*G0_2_0 + 0.090031014510179*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2280] = 0.647179466623904*G0_0_0 + 0.375781625781621*G0_0_1 + 0.375781625781621*G0_0_2 + 0.375781625781621*G0_1_0 + 0.647179466623905*G0_1_1 + 0.375781625781622*G0_1_2 + 0.375781625781621*G0_2_0 + 0.375781625781622*G0_2_1 + 0.647179466623905*G0_2_2;
    A[2281] = -0.584549195660302*G0_0_0 - 0.194414799449519*G0_0_1 - 0.321632537344339*G0_0_2 - 0.44885027523916*G0_1_0 - 0.268135847562928*G0_1_1 - 0.290643601190473*G0_1_2 - 0.32163253734434*G0_2_0 - 0.222141742324032*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2282] = 0.255740273101382*G0_0_0 + 0.0756782440810206*G0_0_1 + 0.127870136550691*G0_0_2 + 0.180062029020361*G0_1_0 + 0.105688582251081*G0_1_1 + 0.10503618359521*G0_1_2 + 0.127870136550691*G0_2_0 + 0.0528442911255404*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2283] = -0.268135847562927*G0_0_0 - 0.22214174232403*G0_0_1 - 0.321632537344339*G0_0_2 - 0.290643601190473*G0_1_0 - 0.268135847562927*G0_1_1 - 0.44885027523916*G0_1_2 - 0.32163253734434*G0_2_0 - 0.19441479944952*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2284] = 0.172233245149909*G0_0_0 + 0.100795592332049*G0_0_1 + 0.166035457919137*G0_0_2 + 0.19865539071268*G0_1_0 + 0.113517366121531*G0_1_1 + 0.198655390712681*G0_1_2 + 0.166035457919137*G0_2_0 + 0.100795592332049*G0_2_1 + 0.17223324514991*G0_2_2;
    A[2285] = 0.10568858225108*G0_0_0 + 0.0528442911255403*G0_0_1 + 0.12787013655069*G0_0_2 + 0.10503618359521*G0_1_0 + 0.105688582251081*G0_1_1 + 0.180062029020361*G0_1_2 + 0.12787013655069*G0_2_0 + 0.0756782440810216*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2286] = 0.323589733311952*G0_0_0 + 0.18789081289081*G0_0_1 + 0.18789081289081*G0_0_2 + 0.187890812890811*G0_1_0 + 0.0521918924696695*G0_1_1 + 0.323589733311952*G0_1_2 + 0.187890812890811*G0_2_0 + 0.323589733311952*G0_2_1 + 0.0521918924696692*G0_2_2;
    A[2287] = -0.292274597830151*G0_0_0 - 0.224425137619579*G0_0_1 - 0.0972073997247595*G0_0_2 - 0.224425137619579*G0_1_0 - 0.156575677409009*G0_1_1 - 0.134067923781464*G0_1_2 - 0.0972073997247595*G0_2_0 - 0.134067923781464*G0_2_1 - 0.0880738185425671*G0_2_2;
    A[2288] = 0.127870136550691*G0_0_0 + 0.09003101451018*G0_0_1 + 0.0378391220405101*G0_0_2 + 0.0900310145101803*G0_1_0 + 0.052191892469669*G0_1_1 + 0.0528442911255406*G0_1_2 + 0.0378391220405104*G0_2_0 + 0.0528442911255397*G0_2_1;
    A[2289] = -0.0965550010688888*G0_0_0 - 0.0287055408583179*G0_0_1 - 0.0596944770121843*G0_0_2 - 0.0287055408583184*G0_1_0 + 0.0391439193522523*G0_1_1 - 0.0965550010688891*G0_1_2 - 0.0596944770121845*G0_2_0 - 0.0965550010688887*G0_2_1 - 0.0228339529554803*G0_2_2;
    A[2290] = 0.0502346965020569*G0_0_0 + 0.0446893079271543*G0_0_1 + 0.0120693751336104*G0_0_2 + 0.0446893079271547*G0_1_0 + 0.0391439193522525*G0_1_1 + 0.012721773789482*G0_1_2 + 0.0120693751336108*G0_2_0 + 0.0127217737894823*G0_2_1;
    A[2291] = 0.0300103381700595*G0_0_0 - 0.00782878387045089*G0_0_1 + 0.0150051690850296*G0_0_2 - 0.0078287838704507*G0_1_0 - 0.0456679059109614*G0_1_1 + 0.0300103381700597*G0_1_2 + 0.0150051690850297*G0_2_0 + 0.0300103381700594*G0_2_1;
    A[2292] = -0.150051690850299*G0_0_0 - 0.466465038947673*G0_0_1 - 0.335985307773498*G0_0_2 - 0.466465038947674*G0_1_0 - 0.782878387045047*G0_1_1 - 0.466465038947672*G0_1_2 - 0.3359853077735*G0_2_0 - 0.466465038947674*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2293] = 0.14352770429159*G0_0_0 + 0.218553549716741*G0_0_1 + 0.185933616923197*G0_0_2 + 0.218553549716741*G0_1_0 + 0.293579395141891*G0_1_1 + 0.234863516113513*G0_1_2 + 0.185933616923198*G0_2_0 + 0.234863516113513*G0_2_1 + 0.176147637085134*G0_2_2;
    A[2294] = 0.0456679059109605*G0_0_0 + 0.120693751336111*G0_0_1 + 0.0717638521457955*G0_0_2 + 0.120693751336112*G0_1_0 + 0.195719596761262*G0_1_1 + 0.120693751336111*G0_1_2 + 0.0717638521457959*G0_2_0 + 0.120693751336112*G0_2_1 + 0.0456679059109607*G0_2_2;
    A[2295] = 0.176147637085135*G0_0_0 + 0.234863516113512*G0_0_1 + 0.185933616923198*G0_0_2 + 0.234863516113514*G0_1_0 + 0.293579395141891*G0_1_1 + 0.218553549716742*G0_1_2 + 0.185933616923199*G0_2_0 + 0.218553549716742*G0_2_1 + 0.143527704291593*G0_2_2;
    A[2296] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0298472385060923*G0_0_2 - 0.0345719095719091*G0_1_0 - 0.0345719095719091*G0_1_1 - 0.0345719095719091*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[2297] = -0.0298472385060925*G0_0_0 + 0.00472467106581687*G0_1_0;
    A[2298] = -0.00946499969937448*G0_1_1;
    A[2299] = 0.00227817610630117*G0_1_2 + 0.0117431758056757*G0_2_2;
    A[2300] = 0.00658922642429554*G0_1_1 + 0.00329461321214796*G0_1_2 + 0.00329461321214781*G0_2_1;
    A[2301] = 0.00450155072550904*G0_1_1 + 0.00192457603481882*G0_1_2 + 0.0019245760348191*G0_2_1 - 0.000652398655870661*G0_2_2;
    A[2302] = -0.0105036183595215*G0_1_1 - 0.0130805930502112*G0_1_2 - 0.0130805930502112*G0_2_1 - 0.0156575677409014*G0_2_2;
    A[2303] = -0.00254435475789612*G0_1_1 - 0.00583896797004416*G0_1_2 - 0.0058389679700442*G0_2_1 - 0.00913358118219213*G0_2_2;
    A[2304] = 0.128522535206562*G0_0_0 + 0.0422754329004326*G0_0_2 + 0.0845508658008653*G0_1_0 + 0.0429604514890972*G0_1_2 + 0.0422754329004328*G0_2_0;
    A[2305] = 0.0371867233846395*G0_0_0 + 0.0743408268364857*G0_0_2 + 0.0808321934624006*G0_1_0 + 0.108232937008977*G0_1_2 + 0.0743408268364856*G0_2_0 + 0.104709984267275*G0_2_2;
    A[2306] = -0.00619778723077341*G0_0_0 + 0.0188869410874618*G0_0_2 + 0.00939454064454078*G0_1_0 + 0.0377738821749238*G0_1_2 + 0.0188869410874619*G0_2_0 + 0.0567586830607661*G0_2_2;
    A[2307] = -0.00583896797004399*G0_0_2 - 0.00254435475789609*G0_1_0 - 0.014287530563572*G0_1_2 - 0.0058389679700442*G0_2_0 - 0.0435476102793807*G0_2_2;
    A[2308] = 0.064261267603281*G0_0_0 + 0.0422754329004325*G0_0_1 + 0.0422754329004328*G0_1_0 + 0.0429604514890972*G0_1_1;
    A[2309] = 0.00880738185425657*G0_0_0 + 0.00649136662591476*G0_0_1 + 0.00649136662591475*G0_1_0 + 0.00352295274170217*G0_1_1;
    A[2310] = -0.000652398655870869*G0_0_0 - 0.00949240044292103*G0_0_1 - 0.00949240044292113*G0_1_0 - 0.018984800885842*G0_1_1;
    A[2311] = 0.00329461321214776*G0_0_1 + 0.00329461321214797*G0_1_0 + 0.0292600797158083*G0_1_1;
    A[2312] = 0.128522535206562*G0_0_0 + 0.128522535206562*G0_0_1 + 0.0862471023061291*G0_0_2 + 0.0439716694056968*G0_1_0 + 0.0439716694056968*G0_1_1 + 0.0446566879943609*G0_1_2 + 0.0862471023061291*G0_2_0 + 0.0862471023061292*G0_2_1 + 0.0439716694056964*G0_2_2;
    A[2313] = 0.0371867233846398*G0_0_0 + 0.0371867233846398*G0_0_1 - 0.0371541034518462*G0_0_2 - 0.0436454700777618*G0_1_0 - 0.0436454700777619*G0_1_1 - 0.0097533599052694*G0_1_2 - 0.0371541034518464*G0_2_0 - 0.0371541034518465*G0_2_1 - 0.00678494602105683*G0_2_2;
    A[2314] = -0.00619778723077294*G0_0_0 - 0.0061977872307729*G0_0_1 - 0.0250847283182351*G0_0_2 - 0.0155923278753133*G0_1_0 - 0.0155923278753133*G0_1_1 + 0.00329461321214789*G0_1_2 - 0.0250847283182348*G0_2_0 - 0.0250847283182348*G0_2_1 + 0.012787013655069*G0_2_2;
    A[2315] = 0.00583896797004413*G0_0_2 + 0.00254435475789612*G0_1_0 + 0.00254435475789608*G0_1_1 - 0.00590420783563148*G0_1_2 + 0.0058389679700442*G0_2_0 + 0.0058389679700442*G0_2_1 - 0.0318696743392921*G0_2_2;
    A[2316] = 0.0642612676032809*G0_0_0 + 0.0219858347028483*G0_0_1 + 0.0642612676032808*G0_0_2 + 0.0219858347028484*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0219858347028484*G0_1_2 + 0.0642612676032809*G0_2_0 + 0.0219858347028481*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[2317] = 0.00880738185425671*G0_0_0 + 0.00231601522834134*G0_0_1 + 0.0088073818542567*G0_0_2 + 0.00231601522834157*G0_1_0 - 0.000652398655871007*G0_1_1 + 0.00231601522834151*G0_1_2 + 0.00880738185425674*G0_2_0 + 0.00231601522834172*G0_2_1 + 0.00880738185425676*G0_2_2;
    A[2318] = -0.000652398655870951*G0_0_0 + 0.00884000178705049*G0_0_1 - 0.000652398655870997*G0_0_2 + 0.00884000178705049*G0_1_0 - 0.000652398655870507*G0_1_1 + 0.0088400017870505*G0_1_2 - 0.000652398655870857*G0_2_0 + 0.00884000178705013*G0_2_1 - 0.000652398655870894*G0_2_2;
    A[2319] = -0.00329461321214799*G0_0_1 - 0.00329461321214796*G0_1_0 + 0.0226708532915125*G0_1_1 - 0.003294613212148*G0_1_2 - 0.00329461321214775*G0_2_1;
    A[2320] = 0.0920860702761734*G0_0_0 + 0.134361503176606*G0_0_1 + 0.134361503176606*G0_0_2 + 0.128000616281865*G0_1_0 + 0.127315597693201*G0_1_1 + 0.127315597693201*G0_1_2 + 0.134361503176606*G0_2_0 + 0.176636936077038*G0_2_1 + 0.176636936077038*G0_2_2;
    A[2321] = -0.062238831770081*G0_0_0 - 0.250814663249556*G0_0_1 - 0.250814663249556*G0_0_2 - 0.160783648739376*G0_1_0 - 0.166948816037356*G0_1_1 - 0.166948816037356*G0_1_2 - 0.250814663249556*G0_2_0 - 0.334680510461757*G0_2_1 - 0.334680510461757*G0_2_2;
    A[2322] = -0.0622388317700814*G0_0_0 + 0.188575831479476*G0_0_1 + 0.188575831479475*G0_0_2 + 0.0985448169692953*G0_1_0 + 0.182410664181496*G0_1_1 + 0.182410664181495*G0_1_2 + 0.188575831479475*G0_2_0 + 0.104709984267275*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2323] = 0.0920860702761738*G0_0_0 - 0.0422754329004327*G0_0_1 - 0.0422754329004326*G0_0_2 - 0.0359145460056915*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.0429604514890973*G0_1_2 - 0.0422754329004323*G0_2_0;
    A[2324] = 0.0391439193522506*G0_0_0 + 0.135698920421141*G0_0_1 + 0.06784946021057*G0_0_2 + 0.135698920421141*G0_1_0 + 0.20941996853455*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0678494602105699*G0_2_0 + 0.104709984267275*G0_2_1;
    A[2325] = -0.00489298991903143*G0_0_0 + 0.0283793415303821*G0_0_1 + 0.0141896707651912*G0_0_2 + 0.0283793415303835*G0_1_0 + 0.0567586830607653*G0_1_1 + 0.0283793415303826*G0_1_2 + 0.014189670765192*G0_2_0 + 0.0283793415303824*G0_2_1;
    A[2326] = -0.00130479731174151*G0_0_1 - 0.000652398655871049*G0_0_2 - 0.00130479731174212*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.012721773789482*G0_1_2 - 0.000652398655870949*G0_2_0 - 0.0127217737894816*G0_2_1;
    A[2327] = 0.0195719596761266*G0_0_0 + 0.0748627457611824*G0_0_1 + 0.0606730749959905*G0_0_2 + 0.0748627457611831*G0_1_0 + 0.149725491522365*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0606730749959916*G0_2_0 + 0.121346149991982*G0_2_1 + 0.0929668084615989*G0_2_2;
    A[2328] = -0.0181040627004173*G0_0_1 - 0.0102752788299663*G0_0_2 - 0.0181040627004173*G0_1_0 - 0.0166361657247074*G0_1_1 + 0.00146789697570954*G0_1_2 - 0.0102752788299666*G0_2_0 + 0.0014678969757092*G0_2_1 + 0.0195719596761266*G0_2_2;
    A[2329] = -0.00619778723077245*G0_0_1 - 0.0055453885749022*G0_0_2 - 0.00619778723077325*G0_1_0 + 0.013700371773289*G0_1_1 + 0.0264221455627706*G0_1_2 - 0.00554538857490259*G0_2_0 + 0.0264221455627706*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[2330] = 0.039143919352253*G0_0_0 - 0.0965550010688885*G0_0_1 - 0.028705540858318*G0_0_2 - 0.0965550010688889*G0_1_0 - 0.0228339529554804*G0_1_1 - 0.059694477012185*G0_1_2 - 0.0287055408583184*G0_2_0 - 0.0596944770121847*G0_2_1 - 0.096555001068889*G0_2_2;
    A[2331] = -0.00489298991903159*G0_0_0 - 0.0332723314494148*G0_0_1 - 0.0190826606842226*G0_0_2 - 0.0332723314494149*G0_1_0 - 0.0048929899190317*G0_1_1 - 0.0190826606842232*G0_1_2 - 0.0190826606842232*G0_2_0 - 0.0190826606842229*G0_2_1 - 0.0332723314494143*G0_2_2;
    A[2332] = 0.00130479731174179*G0_0_1 + 0.000652398655870934*G0_0_2 + 0.00130479731174202*G0_1_0 - 0.0228339529554803*G0_1_1 - 0.0107645778218694*G0_1_2 + 0.000652398655870796*G0_2_0 - 0.0107645778218694*G0_2_1 + 0.00130479731174134*G0_2_2;
    A[2333] = 0.0195719596761264*G0_0_0 - 0.055290786085057*G0_0_1 - 0.0411011153198653*G0_0_2 - 0.0552907860850563*G0_1_0 + 0.0195719596761261*G0_1_1 + 0.00538228891093511*G0_1_2 - 0.041101115319865*G0_2_0 + 0.00538228891093482*G0_2_1 - 0.00880738185425702*G0_2_2;
    A[2334] = 0.0181040627004166*G0_0_1 + 0.0102752788299663*G0_0_2 + 0.0181040627004174*G0_1_0 + 0.0195719596761266*G0_1_1 + 0.0298472385060931*G0_1_2 + 0.0102752788299667*G0_2_0 + 0.0298472385060924*G0_2_1 + 0.0401225173360593*G0_2_2;
    A[2335] = 0.00619778723077375*G0_0_1 + 0.00554538857490223*G0_0_2 + 0.00619778723077332*G0_1_0 + 0.0260959462348348*G0_1_1 + 0.0381653213684456*G0_1_2 + 0.00554538857490261*G0_2_0 + 0.0381653213684461*G0_2_1 + 0.0502346965020571*G0_2_2;
    A[2336] = -0.584549195660302*G0_0_0 - 0.44885027523916*G0_0_1 - 0.32163253734434*G0_0_2 - 0.19441479944952*G0_1_0 - 0.268135847562928*G0_1_1 - 0.222141742324032*G0_1_2 - 0.321632537344339*G0_2_0 - 0.290643601190473*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2337] = 0.912053320907481*G0_0_0 + 0.456026660453739*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453739*G0_1_0 + 0.72611970398428*G0_1_1 + 0.45602666045374*G0_1_2 + 0.456026660453739*G0_2_0 + 0.45602666045374*G0_2_1 + 0.726119703984281*G0_2_2;
    A[2338] = -0.584549195660303*G0_0_0 - 0.135698920421141*G0_0_1 - 0.262916658315961*G0_0_2 - 0.390134396210782*G0_1_0 - 0.20941996853455*G0_1_1 - 0.290643601190473*G0_1_2 - 0.262916658315962*G0_2_0 - 0.104709984267275*G0_2_1 - 0.20941996853455*G0_2_2;
    A[2339] = -0.0371867233846408*G0_0_0 + 0.0376760223765422*G0_0_1 + 0.186912214907005*G0_0_2 + 0.140428810676206*G0_1_0 + 0.0655660649150232*G0_1_1 + 0.0376760223765429*G0_1_2 + 0.186912214907005*G0_2_0 + 0.140428810676205*G0_2_1 - 0.0371867233846405*G0_2_2;
    A[2340] = -0.0371867233846386*G0_0_0 - 0.0748627457611824*G0_0_1 - 0.224098938291644*G0_0_2 - 0.177615534060845*G0_1_0 - 0.149725491522365*G0_1_1 - 0.326851726591308*G0_1_2 - 0.224098938291644*G0_2_0 - 0.121346149991982*G0_2_1 - 0.44819787658329*G0_2_2;
    A[2341] = 0.0123955744615466*G0_0_0 + 0.0061977872307727*G0_0_1 + 0.00619778723077316*G0_0_2 + 0.00619778723077278*G0_1_0 - 0.0137003717732888*G0_1_1 + 0.0714376528178605*G0_1_2 + 0.00619778723077311*G0_2_0 - 0.0264221455627704*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2342] = -0.292274597830151*G0_0_0 - 0.0972073997247596*G0_0_1 - 0.224425137619579*G0_0_2 - 0.0972073997247596*G0_1_0 - 0.0880738185425675*G0_1_1 - 0.134067923781464*G0_1_2 - 0.22442513761958*G0_2_0 - 0.134067923781464*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2343] = 0.45602666045374*G0_0_0 + 0.228013330226869*G0_0_1 + 0.228013330226869*G0_0_2 + 0.228013330226869*G0_1_0 + 0.0929668084615993*G0_1_1 + 0.36305985199214*G0_1_2 + 0.228013330226869*G0_2_0 + 0.363059851992139*G0_2_1 + 0.0929668084615989*G0_2_2;
    A[2344] = -0.292274597830151*G0_0_0 - 0.19506719810539*G0_0_1 - 0.0678494602105703*G0_0_2 - 0.195067198105391*G0_1_0 - 0.185933616923198*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0678494602105705*G0_2_0 - 0.104709984267274*G0_2_1;
    A[2345] = -0.00880738185425662*G0_0_0 - 0.041101115319865*G0_0_1 + 0.00538228891093456*G0_0_2 - 0.041101115319865*G0_1_0 + 0.0195719596761258*G0_1_1 - 0.0552907860850565*G0_1_2 + 0.0053822889109346*G0_2_0 - 0.0552907860850564*G0_2_1 + 0.0195719596761258*G0_2_2;
    A[2346] = -0.00880738185425664*G0_0_0 + 0.0322937334656084*G0_0_1 - 0.0141896707651912*G0_0_2 + 0.0322937334656086*G0_1_0 + 0.0929668084615989*G0_1_1 - 0.0283793415303826*G0_1_2 - 0.014189670765191*G0_2_0 - 0.0283793415303829*G0_2_1;
    A[2347] = 0.00130479731174186*G0_0_0 + 0.000652398655870914*G0_0_1 + 0.000652398655871052*G0_0_2 + 0.000652398655870667*G0_1_0 - 0.0880738185425673*G0_1_1 + 0.0127217737894821*G0_1_2 + 0.00065239865587071*G0_2_0 + 0.0127217737894822*G0_2_1;
    A[2348] = -0.0391439193522546*G0_0_0 + 0.445262082631869*G0_0_1 + 0.249542485870607*G0_0_2 + 0.445262082631871*G0_1_0 + 0.146789697570946*G0_1_1 + 0.249542485870608*G0_1_2 + 0.249542485870609*G0_2_0 + 0.249542485870608*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2349] = -0.039143919352249*G0_0_0 - 0.484406001984121*G0_0_1 - 0.288686405222859*G0_0_2 - 0.484406001984122*G0_1_0 - 0.782878387045046*G0_1_1 - 0.484406001984121*G0_1_2 - 0.288686405222861*G0_2_0 - 0.484406001984122*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2350] = 0.00978597983806307*G0_0_0 + 0.00489298991903214*G0_0_1 + 0.00489298991903189*G0_0_2 + 0.00489298991903166*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0538228891093469*G0_1_2 + 0.00489298991903135*G0_2_0 + 0.0538228891093465*G0_2_1 - 0.0391439193522525*G0_2_2;
    A[2351] = -0.0391439193522531*G0_0_0 - 0.0195719596761251*G0_0_1 - 0.0195719596761256*G0_0_2 - 0.0195719596761267*G0_1_0 - 0.0489298991903157*G0_1_1 - 0.141896707651915*G0_1_2 - 0.0195719596761266*G0_2_0 - 0.141896707651914*G0_2_1 - 0.234863516113514*G0_2_2;
    A[2352] = 0.0189195610202553*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202554*G0_0_2 + 0.0137421252872641*G0_1_0 + 0.0137421252872641*G0_1_1 + 0.0137421252872641*G0_1_2 + 0.0189195610202552*G0_2_0 + 0.0189195610202552*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[2353] = 0.0335985307773502*G0_0_0 - 0.0143945239431349*G0_1_0;
    A[2354] = -0.0137421252872641*G0_1_1;
    A[2355] = 0.00517743573299119*G0_1_2 + 0.0189195610202555*G0_2_2;
    A[2356] = 0.0187890812890811*G0_1_1 + 0.00939454064454043*G0_1_2 + 0.00939454064454076*G0_2_1;
    A[2357] = -0.0148746893538561*G0_1_1 - 0.0149399292194429*G0_1_2 - 0.0149399292194429*G0_2_1 - 0.0150051690850296*G0_2_2;
    A[2358] = -0.00117431758056754*G0_1_1 - 0.00123955744615492*G0_1_2 - 0.00123955744615475*G0_2_1 - 0.00130479731174205*G0_2_2;
    A[2359] = -0.0118736555368502*G0_1_1 - 0.0212681961813905*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.030662736825931*G0_2_2;
    A[2360] = -0.257045070413124*G0_0_0 - 0.0962940416065401*G0_0_2 - 0.192588083213081*G0_1_0 - 0.12473862300251*G0_1_2 - 0.0962940416065414*G0_2_0;
    A[2361] = 0.121346149991982*G0_0_0 - 0.0733296089198868*G0_0_2 - 0.0109602974186307*G0_1_0 - 0.146659217839773*G0_1_2 - 0.0733296089198851*G0_2_0 - 0.271397840842283*G0_2_2;
    A[2362] = -0.0528442911255403*G0_0_0 + 0.0359471659384855*G0_0_2 - 0.00182671623643837*G0_1_0 + 0.0823327103709046*G0_1_2 + 0.0359471659384845*G0_2_0 + 0.20941996853455*G0_2_2;
    A[2363] = -0.0212681961813911*G0_0_2 - 0.0118736555368502*G0_1_0 - 0.0477555816097482*G0_1_2 - 0.0212681961813904*G0_2_0 - 0.111886369481855*G0_2_2;
    A[2364] = -0.128522535206562*G0_0_0 - 0.0962940416065401*G0_0_1 - 0.0962940416065406*G0_1_0 - 0.12473862300251*G0_1_1;
    A[2365] = 0.0476251018785738*G0_0_0 + 0.0623693115012553*G0_0_1 + 0.0623693115012555*G0_1_0 + 0.124738623002511*G0_1_1;
    A[2366] = -0.0150051690850301*G0_0_0 - 0.0377738821749233*G0_0_1 - 0.0377738821749235*G0_1_0 - 0.127087258163646*G0_1_1;
    A[2367] = 0.00939454064454027*G0_0_1 + 0.00939454064454048*G0_1_0 + 0.0641307878721065*G0_1_1;
    A[2368] = -0.0613254736518621*G0_0_0 - 0.061325473651862*G0_0_1 - 0.0400572774704718*G0_0_2 - 0.018789081289081*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.00939454064454069*G0_1_2 - 0.0400572774704714*G0_2_0 - 0.0400572774704713*G0_2_1 - 0.0187890812890809*G0_2_2;
    A[2369] = -0.0254435475789638*G0_0_0 - 0.0254435475789638*G0_0_1 + 0.0136351319077017*G0_0_2 + 0.0148746893538561*G0_1_0 + 0.0148746893538561*G0_1_1 - 6.52398655867551e-05*G0_1_2 + 0.0136351319077007*G0_2_0 + 0.0136351319077007*G0_2_1 - 0.00013047973117436*G0_2_2;
    A[2370] = -0.0528442911255415*G0_0_0 - 0.0528442911255415*G0_0_1 - 0.0137656116388759*G0_0_2 + 0.0011743175805672*G0_1_0 + 0.0011743175805672*G0_1_1 - 6.52398655875237e-05*G0_1_2 - 0.0137656116388753*G0_2_0 - 0.0137656116388753*G0_2_1 - 0.000130479731173912*G0_2_2;
    A[2371] = 0.0212681961813909*G0_0_2 + 0.0118736555368502*G0_1_0 + 0.0118736555368502*G0_1_1 - 0.00939454064454024*G0_1_2 + 0.0212681961813904*G0_2_0 + 0.0212681961813904*G0_2_1 - 0.0187890812890814*G0_2_2;
    A[2372] = -0.030662736825931*G0_0_0 - 0.0093945406445406*G0_0_1 - 0.0306627368259309*G0_0_2 - 0.00939454064454055*G0_1_0 - 0.00939454064454042*G0_1_2 - 0.030662736825931*G0_2_0 - 0.00939454064454074*G0_2_1 - 0.030662736825931*G0_2_2;
    A[2373] = -0.00130479731174164*G0_0_0 - 6.52398655867187e-05*G0_0_1 - 0.00130479731174159*G0_0_2 - 6.5239865587096e-05*G0_1_0 - 6.52398655872229e-05*G0_1_2 - 0.0013047973117416*G0_2_0 - 6.52398655866864e-05*G0_2_1 - 0.00130479731174165*G0_2_2;
    A[2374] = -0.0150051690850304*G0_0_0 - 6.5239865587493e-05*G0_0_1 - 0.0150051690850303*G0_0_2 - 6.52398655873128e-05*G0_1_0 - 6.52398655871491e-05*G0_1_2 - 0.0150051690850303*G0_2_0 - 6.52398655875629e-05*G0_2_1 - 0.0150051690850303*G0_2_2;
    A[2375] = -0.0093945406445404*G0_0_1 - 0.0093945406445405*G0_1_0 - 0.00939454064454049*G0_1_2 - 0.00939454064454019*G0_2_1;
    A[2376] = -0.0693499771190737*G0_0_0 - 0.0906181733004639*G0_0_1 - 0.0906181733004639*G0_0_2 - 0.0547362472275662*G0_1_0 - 0.0641307878721065*G0_1_1 - 0.0641307878721065*G0_1_2 - 0.0906181733004639*G0_2_0 - 0.111886369481854*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2377] = 0.0846813455320391*G0_0_0 + 0.173472802596065*G0_0_1 + 0.173472802596065*G0_0_2 + 0.0893133759887225*G0_1_0 + 0.127087258163646*G0_1_1 + 0.127087258163646*G0_1_2 + 0.173472802596065*G0_2_0 + 0.20941996853455*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2378] = -0.00339247301052845*G0_0_0 - 0.198068231922397*G0_0_1 - 0.198068231922396*G0_0_2 - 0.0623693115012556*G0_1_0 - 0.124738623002511*G0_1_1 - 0.124738623002511*G0_1_2 - 0.198068231922397*G0_2_0 - 0.271397840842283*G0_2_1 - 0.271397840842283*G0_2_2;
    A[2379] = -0.0644569872000426*G0_0_0 + 0.0962940416065403*G0_0_1 + 0.0962940416065401*G0_0_2 + 0.0284445813959701*G0_1_0 + 0.124738623002511*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0962940416065405*G0_2_0;
    A[2380] = 0.0521918924696705*G0_0_0 - 0.271397840842282*G0_0_1 - 0.13569892042114*G0_0_2 - 0.271397840842283*G0_1_0 - 0.542795681684566*G0_1_1 - 0.271397840842282*G0_1_2 - 0.13569892042114*G0_2_0 - 0.271397840842284*G0_2_1;
    A[2381] = -0.0228339529554802*G0_0_0 + 0.0737210481134093*G0_0_1 + 0.0368605240567046*G0_0_2 + 0.0737210481134093*G0_1_0 + 0.209419968534551*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0368605240567044*G0_2_0 + 0.104709984267276*G0_2_1;
    A[2382] = -0.0300103381700601*G0_0_1 - 0.0150051690850303*G0_0_2 - 0.0300103381700601*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0528442911255406*G0_1_2 - 0.0150051690850296*G0_2_0 - 0.0528442911255407*G0_2_1;
    A[2383] = -0.0880738185425682*G0_0_0 + 0.0459941052388958*G0_0_1 + 0.00913358118219168*G0_0_2 + 0.0459941052388964*G0_1_0 + 0.023486351611351*G0_1_1 - 0.0812236326559237*G0_1_2 + 0.00913358118219218*G0_2_0 - 0.0812236326559225*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2384] = -0.0127217737894815*G0_0_1 - 0.0120693751336106*G0_0_2 - 0.0127217737894822*G0_1_0 + 0.0137003717732883*G0_1_1 + 0.0198981590040618*G0_1_2 - 0.0120693751336113*G0_2_0 + 0.0198981590040612*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[2385] = -0.0528442911255417*G0_0_1 - 0.0378391220405109*G0_0_2 - 0.0528442911255409*G0_1_0 - 0.053496689781412*G0_1_1 - 0.000652398655871067*G0_1_2 - 0.0378391220405112*G0_2_0 - 0.000652398655872019*G0_2_1 + 0.0521918924696691*G0_2_2;
    A[2386] = -0.0456679059109617*G0_0_0 + 0.0300103381700595*G0_0_1 - 0.00782878387045087*G0_0_2 + 0.0300103381700601*G0_1_0 + 0.0150051690850302*G0_1_2 - 0.0078287838704511*G0_2_0 + 0.0150051690850298*G0_2_1 + 0.0300103381700599*G0_2_2;
    A[2387] = -0.02283395295548*G0_0_0 + 0.00130479731174198*G0_0_1 - 0.0107645778218691*G0_0_2 + 0.00130479731174133*G0_1_0 + 0.000652398655871106*G0_1_2 - 0.010764577821869*G0_2_0 + 0.000652398655871014*G0_2_1 + 0.00130479731174197*G0_2_2;
    A[2388] = 0.0300103381700604*G0_0_1 + 0.01500516908503*G0_0_2 + 0.0300103381700601*G0_1_0 + 0.0150051690850302*G0_1_2 + 0.0150051690850296*G0_2_0 + 0.01500516908503*G0_2_1 + 0.0300103381700599*G0_2_2;
    A[2389] = -0.0880738185425683*G0_0_0 + 0.0127217737894828*G0_0_1 + 0.000652398655870908*G0_0_2 + 0.012721773789482*G0_1_0 + 0.000652398655870685*G0_1_2 + 0.000652398655870982*G0_2_0 + 0.000652398655872121*G0_2_1 + 0.0013047973117422*G0_2_2;
    A[2390] = 0.0127217737894816*G0_0_1 + 0.0120693751336107*G0_0_2 + 0.0127217737894821*G0_1_0 + 0.000652398655870849*G0_1_2 + 0.0120693751336112*G0_2_0 + 0.000652398655870064*G0_2_1 + 0.00130479731174154*G0_2_2;
    A[2391] = 0.0528442911255407*G0_0_1 + 0.0378391220405111*G0_0_2 + 0.0528442911255408*G0_1_0 + 0.0150051690850301*G0_1_2 + 0.0378391220405112*G0_2_0 + 0.01500516908503*G0_2_1 + 0.0300103381700603*G0_2_2;
    A[2392] = 0.255740273101382*G0_0_0 + 0.180062029020361*G0_0_1 + 0.127870136550691*G0_0_2 + 0.0756782440810206*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255404*G0_1_2 + 0.127870136550691*G0_2_0 + 0.10503618359521*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2393] = -0.584549195660303*G0_0_0 - 0.390134396210782*G0_0_1 - 0.262916658315962*G0_0_2 - 0.135698920421141*G0_1_0 - 0.20941996853455*G0_1_1 - 0.104709984267275*G0_1_2 - 0.262916658315961*G0_2_0 - 0.290643601190473*G0_2_1 - 0.20941996853455*G0_2_2;
    A[2394] = 0.647179466623906*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684566*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684566*G0_2_2;
    A[2395] = 0.172233245149911*G0_0_0 + 0.0714376528178599*G0_0_1 + 0.00619778723077323*G0_0_2 - 0.0264221455627702*G0_1_0 - 0.0137003717732883*G0_1_1 + 0.00619778723077349*G0_1_2 + 0.00619778723077396*G0_2_0 + 0.00619778723077288*G0_2_1 + 0.0123955744615465*G0_2_2;
    A[2396] = -0.268135847562929*G0_0_0 - 0.045994105238896*G0_0_1 + 0.0534966897814113*G0_0_2 + 0.0225077536275449*G0_1_0 - 0.0234863516113515*G0_1_1 - 0.104709984267275*G0_1_2 + 0.0534966897814105*G0_2_0 + 0.0812236326559225*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2397] = 0.105688582251082*G0_0_0 + 0.0528442911255415*G0_0_1 - 0.0221815542996091*G0_0_2 + 0.000652398655871061*G0_1_0 + 0.0534966897814118*G0_1_1 + 0.0528442911255408*G0_1_2 - 0.0221815542996092*G0_2_0 + 0.00065239865587185*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2398] = 0.127870136550691*G0_0_0 + 0.0378391220405105*G0_0_1 + 0.0900310145101804*G0_0_2 + 0.0378391220405104*G0_1_0 + 0.0528442911255405*G0_1_2 + 0.0900310145101803*G0_2_0 + 0.0528442911255405*G0_2_1 + 0.0521918924696694*G0_2_2;
    A[2399] = -0.292274597830151*G0_0_0 - 0.0678494602105708*G0_0_1 - 0.195067198105391*G0_0_2 - 0.0678494602105705*G0_1_0 - 0.104709984267275*G0_1_2 - 0.195067198105391*G0_2_0 - 0.104709984267275*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2400] = 0.323589733311953*G0_0_0 + 0.135698920421141*G0_0_1 + 0.135698920421141*G0_0_2 + 0.135698920421141*G0_1_0 + 0.271397840842283*G0_1_2 + 0.135698920421142*G0_2_0 + 0.271397840842282*G0_2_1;
    A[2401] = 0.050234696502057*G0_0_0 + 0.00554538857490207*G0_0_1 + 0.0381653213684459*G0_0_2 + 0.00554538857490235*G0_1_0 + 0.00619778723077279*G0_1_2 + 0.0381653213684458*G0_2_0 + 0.0061977872307733*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[2402] = -0.0965550010688892*G0_0_0 - 0.067849460210571*G0_0_1 - 0.0368605240567045*G0_0_2 - 0.0678494602105707*G0_1_0 - 0.104709984267275*G0_1_2 - 0.0368605240567043*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2403] = 0.0300103381700604*G0_0_0 + 0.0378391220405112*G0_0_1 + 0.0150051690850303*G0_0_2 + 0.0378391220405108*G0_1_0 + 0.0528442911255405*G0_1_2 + 0.0150051690850302*G0_2_0 + 0.0528442911255405*G0_2_1;
    A[2404] = 0.143527704291594*G0_0_0 - 0.0750258454251491*G0_0_1 - 0.0424059126316063*G0_0_2 - 0.0750258454251502*G0_1_0 - 0.0260959462348348*G0_1_2 - 0.0424059126316055*G0_2_0 - 0.0260959462348338*G0_2_1 - 0.0521918924696692*G0_2_2;
    A[2405] = -0.150051690850303*G0_0_0 + 0.316413348097372*G0_0_1 + 0.185933616923198*G0_0_2 + 0.316413348097373*G0_1_0 + 0.185933616923198*G0_1_2 + 0.185933616923197*G0_2_0 + 0.185933616923197*G0_2_1 + 0.371867233846395*G0_2_2;
    A[2406] = 0.0456679059109603*G0_0_0 - 0.0750258454251518*G0_0_1 - 0.0260959462348355*G0_0_2 - 0.0750258454251507*G0_1_0 - 0.0260959462348348*G0_1_2 - 0.0260959462348354*G0_2_0 - 0.0260959462348358*G0_2_1 - 0.0521918924696697*G0_2_2;
    A[2407] = 0.176147637085137*G0_0_0 - 0.0587158790283785*G0_0_1 - 0.00978597983806311*G0_0_2 - 0.0587158790283787*G0_1_0 - 0.0260959462348346*G0_1_2 - 0.00978597983806334*G0_2_0 - 0.0260959462348341*G0_2_1 - 0.0521918924696693*G0_2_2;
    A[2408] = -0.0298472385060922*G0_0_0 - 0.0298472385060922*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0345719095719089*G0_1_0 - 0.034571909571909*G0_1_1 - 0.034571909571909*G0_1_2 - 0.0298472385060921*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[2409] = 0.0117431758056757*G0_0_0 + 0.00227817610630087*G0_1_0;
    A[2410] = -0.00946499969937467*G0_1_1;
    A[2411] = 0.00472467106581668*G0_1_2 - 0.0298472385060926*G0_2_2;
    A[2412] = 0.0292600797158086*G0_1_1 + 0.00329461321214784*G0_1_2 + 0.00329461321214707*G0_2_1;
    A[2413] = -0.0189848008858422*G0_1_1 - 0.00949240044292104*G0_1_2 - 0.00949240044292042*G0_2_1 - 0.000652398655871181*G0_2_2;
    A[2414] = 0.00352295274170273*G0_1_1 + 0.00649136662591517*G0_1_2 + 0.0064913666259143*G0_2_1 + 0.00880738185425684*G0_2_2;
    A[2415] = 0.042960451489097*G0_1_1 + 0.0422754329004327*G0_1_2 + 0.0422754329004332*G0_2_1 + 0.0642612676032814*G0_2_2;
    A[2416] = -0.0435476102793806*G0_0_0 - 0.00583896797004407*G0_0_2 - 0.0142875305635719*G0_1_0 - 0.00254435475789666*G0_1_2 - 0.00583896797004439*G0_2_0;
    A[2417] = 0.0567586830607654*G0_0_0 + 0.0188869410874612*G0_0_2 + 0.0377738821749231*G0_1_0 + 0.0093945406445408*G0_1_2 + 0.0188869410874616*G0_2_0 - 0.00619778723077357*G0_2_2;
    A[2418] = 0.104709984267275*G0_0_0 + 0.0743408268364862*G0_0_2 + 0.108232937008977*G0_1_0 + 0.0808321934624005*G0_1_2 + 0.0743408268364853*G0_2_0 + 0.0371867233846393*G0_2_2;
    A[2419] = 0.0422754329004321*G0_0_2 + 0.042960451489097*G0_1_0 + 0.0845508658008652*G0_1_2 + 0.0422754329004332*G0_2_0 + 0.128522535206563*G0_2_2;
    A[2420] = -0.00913358118219209*G0_0_0 - 0.00583896797004412*G0_0_1 - 0.00583896797004424*G0_1_0 - 0.00254435475789672*G0_1_1;
    A[2421] = -0.0156575677409011*G0_0_0 - 0.0130805930502113*G0_0_1 - 0.0130805930502109*G0_1_0 - 0.0105036183595208*G0_1_1;
    A[2422] = -0.000652398655871078*G0_0_0 + 0.00192457603481911*G0_0_1 + 0.00192457603481873*G0_1_0 + 0.0045015507255086*G0_1_1;
    A[2423] = 0.00329461321214776*G0_0_1 + 0.00329461321214782*G0_1_0 + 0.00658922642429577*G0_1_1;
    A[2424] = 0.176636936077038*G0_0_0 + 0.176636936077038*G0_0_1 + 0.134361503176605*G0_0_2 + 0.1273155976932*G0_1_0 + 0.1273155976932*G0_1_1 + 0.128000616281864*G0_1_2 + 0.134361503176605*G0_2_0 + 0.134361503176605*G0_2_1 + 0.0920860702761728*G0_2_2;
    A[2425] = -0.334680510461758*G0_0_0 - 0.334680510461758*G0_0_1 - 0.250814663249556*G0_0_2 - 0.166948816037356*G0_1_0 - 0.166948816037356*G0_1_1 - 0.160783648739375*G0_1_2 - 0.250814663249557*G0_2_0 - 0.250814663249557*G0_2_1 - 0.0622388317700799*G0_2_2;
    A[2426] = 0.104709984267274*G0_0_0 + 0.104709984267274*G0_0_1 + 0.188575831479475*G0_0_2 + 0.182410664181496*G0_1_0 + 0.182410664181496*G0_1_1 + 0.0985448169692942*G0_1_2 + 0.188575831479476*G0_2_0 + 0.188575831479475*G0_2_1 - 0.0622388317700826*G0_2_2;
    A[2427] = -0.042275432900432*G0_0_2 - 0.0429604514890969*G0_1_0 - 0.042960451489097*G0_1_1 - 0.0359145460056909*G0_1_2 - 0.0422754329004331*G0_2_0 - 0.0422754329004331*G0_2_1 + 0.0920860702761745*G0_2_2;
    A[2428] = 0.0642612676032808*G0_0_0 + 0.0219858347028482*G0_0_1 + 0.0642612676032808*G0_0_2 + 0.0219858347028484*G0_1_0 + 0.0226708532915124*G0_1_1 + 0.0219858347028484*G0_1_2 + 0.0642612676032808*G0_2_0 + 0.021985834702848*G0_2_1 + 0.0642612676032808*G0_2_2;
    A[2429] = 0.0088073818542573*G0_0_0 + 0.00231601522834168*G0_0_1 + 0.00880738185425729*G0_0_2 + 0.00231601522834195*G0_1_0 - 0.000652398655870761*G0_1_1 + 0.00231601522834187*G0_1_2 + 0.00880738185425735*G0_2_0 + 0.00231601522834161*G0_2_1 + 0.00880738185425736*G0_2_2;
    A[2430] = -0.000652398655871092*G0_0_0 + 0.00884000178705046*G0_0_1 - 0.000652398655871144*G0_0_2 + 0.0088400017870502*G0_1_0 - 0.000652398655870839*G0_1_1 + 0.0088400017870502*G0_1_2 - 0.000652398655871118*G0_2_0 + 0.00884000178705075*G0_2_1 - 0.000652398655871195*G0_2_2;
    A[2431] = -0.00329461321214818*G0_0_1 - 0.00329461321214782*G0_1_0 + 0.0226708532915128*G0_1_1 - 0.00329461321214781*G0_1_2 - 0.00329461321214837*G0_2_1;
    A[2432] = 0.0439716694056963*G0_0_0 + 0.0862471023061291*G0_0_1 + 0.086247102306129*G0_0_2 + 0.0446566879943608*G0_1_0 + 0.0439716694056968*G0_1_1 + 0.0439716694056968*G0_1_2 + 0.0862471023061289*G0_2_0 + 0.128522535206562*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2433] = -0.00678494602105666*G0_0_0 - 0.0371541034518458*G0_0_1 - 0.0371541034518458*G0_0_2 - 0.00975335990526937*G0_1_0 - 0.0436454700777608*G0_1_1 - 0.0436454700777608*G0_1_2 - 0.0371541034518458*G0_2_0 + 0.0371867233846402*G0_2_1 + 0.0371867233846403*G0_2_2;
    A[2434] = 0.0127870136550689*G0_0_0 - 0.0250847283182353*G0_0_1 - 0.0250847283182353*G0_0_2 + 0.00329461321214777*G0_1_0 - 0.0155923278753139*G0_1_1 - 0.0155923278753139*G0_1_2 - 0.0250847283182353*G0_2_0 - 0.00619778723077345*G0_2_1 - 0.00619778723077344*G0_2_2;
    A[2435] = -0.0318696743392921*G0_0_0 + 0.00583896797004413*G0_0_1 + 0.00583896797004413*G0_0_2 - 0.00590420783563113*G0_1_0 + 0.00254435475789674*G0_1_1 + 0.00254435475789673*G0_1_2 + 0.0058389679700444*G0_2_0;
    A[2436] = 0.0391439193522518*G0_0_0 + 0.02642214556277*G0_0_1 - 0.00554538857490255*G0_0_2 + 0.0264221455627695*G0_1_0 + 0.0137003717732881*G0_1_1 - 0.00619778723077356*G0_1_2 - 0.00554538857490352*G0_2_0 - 0.00619778723077387*G0_2_1;
    A[2437] = 0.0195719596761265*G0_0_0 + 0.00146789697570979*G0_0_1 - 0.0102752788299663*G0_0_2 + 0.00146789697570923*G0_1_0 - 0.0166361657247072*G0_1_1 - 0.0181040627004168*G0_1_2 - 0.010275278829966*G0_2_0 - 0.0181040627004167*G0_2_1;
    A[2438] = -0.0127217737894823*G0_0_1 - 0.000652398655871159*G0_0_2 - 0.0127217737894822*G0_1_0 - 0.0254435475789648*G0_1_1 - 0.00130479731174264*G0_1_2 - 0.000652398655871741*G0_2_0 - 0.00130479731174219*G0_2_1;
    A[2439] = 0.0929668084615992*G0_0_0 + 0.121346149991982*G0_0_1 + 0.0606730749959904*G0_0_2 + 0.121346149991982*G0_1_0 + 0.149725491522364*G0_1_1 + 0.0748627457611818*G0_1_2 + 0.0606730749959907*G0_2_0 + 0.0748627457611813*G0_2_1 + 0.0195719596761257*G0_2_2;
    A[2440] = 0.028379341530383*G0_0_1 + 0.0141896707651914*G0_0_2 + 0.0283793415303833*G0_1_0 + 0.0567586830607669*G0_1_1 + 0.0283793415303837*G0_1_2 + 0.0141896707651911*G0_2_0 + 0.0283793415303828*G0_2_1 - 0.00489298991903172*G0_2_2;
    A[2441] = 0.104709984267275*G0_0_1 + 0.0678494602105705*G0_0_2 + 0.104709984267275*G0_1_0 + 0.20941996853455*G0_1_1 + 0.135698920421142*G0_1_2 + 0.0678494602105713*G0_2_0 + 0.135698920421142*G0_2_1 + 0.0391439193522519*G0_2_2;
    A[2442] = -0.156575677409009*G0_0_0 - 0.134067923781463*G0_0_1 - 0.22442513761958*G0_0_2 - 0.134067923781463*G0_1_0 - 0.0880738185425666*G0_1_1 - 0.09720739972476*G0_1_2 - 0.22442513761958*G0_2_0 - 0.0972073997247585*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2443] = 0.0195719596761261*G0_0_0 - 0.0552907860850576*G0_0_1 + 0.00538228891093429*G0_0_2 - 0.0552907860850568*G0_1_0 + 0.0195719596761253*G0_1_1 - 0.0411011153198651*G0_1_2 + 0.00538228891093432*G0_2_0 - 0.0411011153198665*G0_2_1 - 0.00880738185425742*G0_2_2;
    A[2444] = 0.0127217737894831*G0_0_1 + 0.000652398655871317*G0_0_2 + 0.0127217737894823*G0_1_0 - 0.0880738185425673*G0_1_1 + 0.000652398655870761*G0_1_2 + 0.000652398655871727*G0_2_0 + 0.00065239865587232*G0_2_1 + 0.00130479731174231*G0_2_2;
    A[2445] = 0.0929668084615987*G0_0_0 + 0.363059851992141*G0_0_1 + 0.22801333022687*G0_0_2 + 0.363059851992139*G0_1_0 + 0.0929668084615991*G0_1_1 + 0.22801333022687*G0_1_2 + 0.22801333022687*G0_2_0 + 0.22801333022687*G0_2_1 + 0.456026660453741*G0_2_2;
    A[2446] = -0.0283793415303837*G0_0_1 - 0.0141896707651915*G0_0_2 - 0.0283793415303835*G0_1_0 + 0.0929668084615991*G0_1_1 + 0.0322937334656083*G0_1_2 - 0.0141896707651912*G0_2_0 + 0.0322937334656073*G0_2_1 - 0.00880738185425669*G0_2_2;
    A[2447] = -0.104709984267274*G0_0_1 - 0.0678494602105702*G0_0_2 - 0.104709984267275*G0_1_0 - 0.185933616923198*G0_1_1 - 0.195067198105391*G0_1_2 - 0.067849460210571*G0_2_0 - 0.195067198105389*G0_2_1 - 0.292274597830152*G0_2_2;
    A[2448] = -0.268135847562927*G0_0_0 - 0.290643601190473*G0_0_1 - 0.32163253734434*G0_0_2 - 0.22214174232403*G0_1_0 - 0.268135847562927*G0_1_1 - 0.19441479944952*G0_1_2 - 0.321632537344339*G0_2_0 - 0.44885027523916*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2449] = -0.0371867233846409*G0_0_0 + 0.140428810676206*G0_0_1 + 0.186912214907005*G0_0_2 + 0.0376760223765422*G0_1_0 + 0.0655660649150231*G0_1_1 + 0.140428810676205*G0_1_2 + 0.186912214907005*G0_2_0 + 0.0376760223765428*G0_2_1 - 0.0371867233846405*G0_2_2;
    A[2450] = 0.172233245149911*G0_0_0 - 0.0264221455627702*G0_0_1 + 0.00619778723077395*G0_0_2 + 0.0714376528178599*G0_1_0 - 0.0137003717732883*G0_1_1 + 0.00619778723077287*G0_1_2 + 0.00619778723077324*G0_2_0 + 0.00619778723077349*G0_2_1 + 0.0123955744615465*G0_2_2;
    A[2451] = 0.726119703984281*G0_0_0 + 0.456026660453739*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453739*G0_1_0 + 0.726119703984278*G0_1_1 + 0.456026660453739*G0_1_2 + 0.456026660453739*G0_2_0 + 0.456026660453739*G0_2_1 + 0.912053320907482*G0_2_2;
    A[2452] = -0.448197876583289*G0_0_0 - 0.121346149991981*G0_0_1 - 0.224098938291645*G0_0_2 - 0.326851726591306*G0_1_0 - 0.149725491522364*G0_1_1 - 0.177615534060844*G0_1_2 - 0.224098938291644*G0_2_0 - 0.0748627457611813*G0_2_1 - 0.0371867233846387*G0_2_2;
    A[2453] = -0.20941996853455*G0_0_0 - 0.104709984267274*G0_0_1 - 0.262916658315961*G0_0_2 - 0.290643601190473*G0_1_0 - 0.20941996853455*G0_1_1 - 0.390134396210781*G0_1_2 - 0.262916658315961*G0_2_0 - 0.135698920421142*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2454] = -0.0965550010688887*G0_0_0 - 0.0596944770121842*G0_0_1 - 0.0287055408583175*G0_0_2 - 0.0596944770121842*G0_1_0 - 0.0228339529554803*G0_1_1 - 0.0965550010688876*G0_1_2 - 0.0287055408583179*G0_2_0 - 0.0965550010688886*G0_2_1 + 0.0391439193522533*G0_2_2;
    A[2455] = -0.00880738185425687*G0_0_0 + 0.00538228891093423*G0_0_1 - 0.041101115319865*G0_0_2 + 0.0053822889109347*G0_1_0 + 0.0195719596761259*G0_1_1 - 0.0552907860850559*G0_1_2 - 0.0411011153198649*G0_2_0 - 0.0552907860850564*G0_2_1 + 0.0195719596761264*G0_2_2;
    A[2456] = 0.050234696502057*G0_0_0 + 0.0381653213684467*G0_0_1 + 0.00554538857490261*G0_0_2 + 0.0381653213684454*G0_1_0 + 0.0260959462348348*G0_1_1 + 0.00619778723077356*G0_1_2 + 0.00554538857490209*G0_2_0 + 0.00619778723077362*G0_2_1;
    A[2457] = -0.0332723314494155*G0_0_0 - 0.0190826606842238*G0_0_1 - 0.0190826606842236*G0_0_2 - 0.0190826606842239*G0_1_0 - 0.00489298991903199*G0_1_1 - 0.0332723314494153*G0_1_2 - 0.0190826606842237*G0_2_0 - 0.0332723314494151*G0_2_1 - 0.00489298991903181*G0_2_2;
    A[2458] = 0.0401225173360593*G0_0_0 + 0.0298472385060923*G0_0_1 + 0.0102752788299664*G0_0_2 + 0.0298472385060928*G0_1_0 + 0.0195719596761263*G0_1_1 + 0.0181040627004169*G0_1_2 + 0.0102752788299664*G0_2_0 + 0.0181040627004161*G0_2_1;
    A[2459] = 0.00130479731174217*G0_0_0 - 0.0107645778218687*G0_0_1 + 0.000652398655871107*G0_0_2 - 0.0107645778218689*G0_1_0 - 0.0228339529554803*G0_1_1 + 0.00130479731174256*G0_1_2 + 0.000652398655871262*G0_2_0 + 0.00130479731174249*G0_2_1;
    A[2460] = 0.352295274170272*G0_0_0 + 0.249542485870609*G0_0_1 + 0.249542485870608*G0_0_2 + 0.249542485870607*G0_1_0 + 0.146789697570944*G0_1_1 + 0.445262082631868*G0_1_2 + 0.249542485870608*G0_2_0 + 0.44526208263187*G0_2_1 - 0.039143919352254*G0_2_2;
    A[2461] = -0.234863516113515*G0_0_0 - 0.141896707651915*G0_0_1 - 0.0195719596761256*G0_0_2 - 0.141896707651913*G0_1_0 - 0.0489298991903158*G0_1_1 - 0.0195719596761263*G0_1_2 - 0.0195719596761249*G0_2_0 - 0.0195719596761257*G0_2_1 - 0.0391439193522519*G0_2_2;
    A[2462] = -0.0391439193522524*G0_0_0 + 0.0538228891093463*G0_0_1 + 0.00489298991903199*G0_0_2 + 0.0538228891093478*G0_1_0 + 0.146789697570948*G0_1_1 + 0.00489298991903157*G0_1_2 + 0.00489298991903175*G0_2_0 + 0.00489298991903219*G0_2_1 + 0.00978597983806347*G0_2_2;
    A[2463] = -0.185933616923198*G0_0_0 - 0.484406001984122*G0_0_1 - 0.288686405222861*G0_0_2 - 0.484406001984122*G0_1_0 - 0.782878387045046*G0_1_1 - 0.484406001984122*G0_1_2 - 0.288686405222861*G0_2_0 - 0.484406001984122*G0_2_1 - 0.0391439193522515*G0_2_2;
    A[2464] = 0.0117431758056756*G0_0_0 + 0.0117431758056756*G0_0_1 + 0.0117431758056756*G0_0_2 + 0.00946499969937466*G0_1_0 + 0.00946499969937467*G0_1_1 + 0.00946499969937465*G0_1_2 + 0.0117431758056756*G0_2_0 + 0.0117431758056757*G0_2_1 + 0.0117431758056756*G0_2_2;
    A[2465] = -0.0298472385060925*G0_0_0 + 0.00472467106581678*G0_1_0;
    A[2466] = -0.00946499969937468*G0_1_1;
    A[2467] = 0.00472467106581704*G0_1_2 - 0.0298472385060925*G0_2_2;
    A[2468] = 0.0292600797158082*G0_1_1 + 0.00329461321214768*G0_1_2 + 0.00329461321214766*G0_2_1;
    A[2469] = -0.0189848008858416*G0_1_1 - 0.00949240044292085*G0_1_2 - 0.00949240044292096*G0_2_1 - 0.000652398655871198*G0_2_2;
    A[2470] = 0.00352295274170178*G0_1_1 + 0.00649136662591494*G0_1_2 + 0.00649136662591503*G0_2_1 + 0.00880738185425722*G0_2_2;
    A[2471] = 0.042960451489098*G0_1_1 + 0.0422754329004331*G0_1_2 + 0.0422754329004328*G0_2_1 + 0.064261267603281*G0_2_2;
    A[2472] = 0.176636936077038*G0_0_0 + 0.0422754329004319*G0_0_2 + 0.0493213383838372*G0_1_0 + 0.0429604514890962*G0_1_2 + 0.0422754329004328*G0_2_0;
    A[2473] = -0.334680510461757*G0_0_0 - 0.0838658472121997*G0_0_2 - 0.1677316944244*G0_1_0 - 0.0777006799142199*G0_1_2 - 0.083865847212201*G0_2_0 + 0.104709984267275*G0_2_2;
    A[2474] = 0.104709984267275*G0_0_0 - 0.0838658472122012*G0_0_2 - 0.0777006799142216*G0_1_0 - 0.167731694424402*G0_1_2 - 0.0838658472122006*G0_2_0 - 0.334680510461758*G0_2_2;
    A[2475] = 0.0422754329004333*G0_0_2 + 0.042960451489098*G0_1_0 + 0.0493213383838389*G0_1_2 + 0.0422754329004327*G0_2_0 + 0.176636936077039*G0_2_2;
    A[2476] = 0.0642612676032804*G0_0_0 + 0.042275432900432*G0_0_1 + 0.042275432900432*G0_1_0 + 0.0429604514890964*G0_1_1;
    A[2477] = 0.00880738185425689*G0_0_0 + 0.0064913666259154*G0_0_1 + 0.0064913666259152*G0_1_0 + 0.00352295274170283*G0_1_1;
    A[2478] = -0.000652398655870736*G0_0_0 - 0.00949240044292157*G0_0_1 - 0.00949240044292112*G0_1_0 - 0.0189848008858426*G0_1_1;
    A[2479] = 0.00329461321214817*G0_0_1 + 0.00329461321214764*G0_1_0 + 0.0292600797158085*G0_1_1;
    A[2480] = -0.0435476102793804*G0_0_0 - 0.0435476102793805*G0_0_1 - 0.0377086423093359*G0_0_2 - 0.0292600797158082*G0_1_0 - 0.0292600797158082*G0_1_1 - 0.0259654665036606*G0_1_2 - 0.0377086423093363*G0_2_0 - 0.0377086423093364*G0_2_1 - 0.0318696743392919*G0_2_2;
    A[2481] = 0.0567586830607656*G0_0_0 + 0.0567586830607658*G0_0_1 + 0.0378717419733034*G0_0_2 + 0.0189848008858416*G0_1_0 + 0.0189848008858416*G0_1_1 + 0.00949240044292085*G0_1_2 + 0.0378717419733045*G0_2_0 + 0.0378717419733046*G0_2_1 + 0.0127870136550689*G0_2_2;
    A[2482] = 0.104709984267276*G0_0_0 + 0.104709984267276*G0_0_1 + 0.0303691574307898*G0_0_2 - 0.00352295274170184*G0_1_0 - 0.00352295274170189*G0_1_1 + 0.00296841388421298*G0_1_2 + 0.0303691574307891*G0_2_0 + 0.030369157430789*G0_2_1 - 0.00678494602105701*G0_2_2;
    A[2483] = -0.0422754329004332*G0_0_2 - 0.042960451489098*G0_1_0 - 0.042960451489098*G0_1_1 - 0.00068501858866488*G0_1_2 - 0.0422754329004327*G0_2_0 - 0.0422754329004328*G0_2_1 + 0.0439716694056969*G0_2_2;
    A[2484] = -0.00913358118219245*G0_0_0 - 0.00329461321214802*G0_0_1 - 0.00913358118219242*G0_0_2 - 0.00329461321214783*G0_1_0 - 0.00329461321214783*G0_1_2 - 0.00913358118219215*G0_2_0 - 0.00329461321214774*G0_2_1 - 0.00913358118219212*G0_2_2;
    A[2485] = -0.0156575677409011*G0_0_0 - 0.00257697469068994*G0_0_1 - 0.0156575677409011*G0_0_2 - 0.00257697469068996*G0_1_0 - 0.00257697469068995*G0_1_2 - 0.0156575677409011*G0_2_0 - 0.00257697469069012*G0_2_1 - 0.015657567740901*G0_2_2;
    A[2486] = -0.00065239865587089*G0_0_0 - 0.00257697469068988*G0_0_1 - 0.000652398655870868*G0_0_2 - 0.00257697469069014*G0_1_0 - 0.00257697469069014*G0_1_2 - 0.000652398655870914*G0_2_0 - 0.00257697469068976*G0_2_1 - 0.000652398655870947*G0_2_2;
    A[2487] = -0.00329461321214777*G0_0_1 - 0.00329461321214766*G0_1_0 - 0.00329461321214763*G0_1_2 - 0.00329461321214806*G0_2_1;
    A[2488] = -0.031869674339292*G0_0_0 - 0.0377086423093365*G0_0_1 - 0.0377086423093365*G0_0_2 - 0.0259654665036609*G0_1_0 - 0.0292600797158085*G0_1_1 - 0.0292600797158085*G0_1_2 - 0.0377086423093363*G0_2_0 - 0.0435476102793807*G0_2_1 - 0.0435476102793807*G0_2_2;
    A[2489] = 0.0127870136550692*G0_0_0 + 0.0378717419733041*G0_0_1 + 0.0378717419733042*G0_0_2 + 0.00949240044292159*G0_1_0 + 0.0189848008858426*G0_1_1 + 0.0189848008858427*G0_1_2 + 0.0378717419733041*G0_2_0 + 0.0567586830607657*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[2490] = -0.00678494602105718*G0_0_0 + 0.030369157430789*G0_0_1 + 0.0303691574307889*G0_0_2 + 0.0029684138842122*G0_1_0 - 0.00352295274170283*G0_1_1 - 0.00352295274170282*G0_1_2 + 0.0303691574307891*G0_2_0 + 0.104709984267275*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2491] = 0.0439716694056971*G0_0_0 - 0.0422754329004319*G0_0_1 - 0.0422754329004319*G0_0_2 - 0.000685018588664278*G0_1_0 - 0.0429604514890962*G0_1_1 - 0.0429604514890963*G0_1_2 - 0.0422754329004325*G0_2_0;
    A[2492] = -0.156575677409009*G0_0_0 - 0.0225077536275452*G0_0_1 + 0.0678494602105699*G0_0_2 - 0.0225077536275445*G0_1_0 + 0.0234863516113511*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0678494602105707*G0_2_0 + 0.104709984267275*G0_2_1;
    A[2493] = 0.0195719596761254*G0_0_0 + 0.0748627457611827*G0_0_1 + 0.0141896707651913*G0_0_2 + 0.0748627457611829*G0_1_0 + 0.149725491522366*G0_1_1 + 0.0283793415303834*G0_1_2 + 0.0141896707651914*G0_2_0 + 0.0283793415303826*G0_2_1;
    A[2494] = -0.0127217737894821*G0_0_1 - 0.000652398655870722*G0_0_2 - 0.012721773789482*G0_1_0 - 0.113517366121532*G0_1_1 - 0.0127217737894824*G0_1_2 - 0.000652398655871371*G0_2_0 - 0.0127217737894822*G0_2_1;
    A[2495] = 0.0929668084615998*G0_0_0 - 0.27009304353054*G0_0_1 - 0.135046521765269*G0_0_2 - 0.270093043530541*G0_1_0 - 0.540186087061081*G0_1_1 - 0.270093043530539*G0_1_2 - 0.135046521765271*G0_2_0 - 0.270093043530542*G0_2_1 + 0.0929668084615998*G0_2_2;
    A[2496] = 0.0283793415303824*G0_0_1 + 0.0141896707651909*G0_0_2 + 0.0283793415303827*G0_1_0 + 0.149725491522364*G0_1_1 + 0.0748627457611827*G0_1_2 + 0.0141896707651917*G0_2_0 + 0.0748627457611828*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2497] = 0.104709984267276*G0_0_1 + 0.0678494602105708*G0_0_2 + 0.104709984267276*G0_1_0 + 0.0234863516113523*G0_1_1 - 0.0225077536275458*G0_1_2 + 0.0678494602105713*G0_2_0 - 0.0225077536275443*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2498] = 0.0391439193522525*G0_0_0 + 0.0127217737894823*G0_0_1 + 0.0446893079271553*G0_0_2 + 0.0127217737894818*G0_1_0 + 0.0120693751336107*G0_1_2 + 0.0446893079271553*G0_2_0 + 0.0120693751336111*G0_2_1 + 0.0502346965020572*G0_2_2;
    A[2499] = 0.0195719596761257*G0_0_0 + 0.0181040627004167*G0_0_1 + 0.0298472385060923*G0_0_2 + 0.0181040627004171*G0_1_0 + 0.0102752788299668*G0_1_2 + 0.0298472385060926*G0_2_0 + 0.0102752788299663*G0_2_1 + 0.0401225173360589*G0_2_2;
    A[2500] = 0.0127217737894816*G0_0_1 + 0.000652398655870789*G0_0_2 + 0.0127217737894821*G0_1_0 + 0.0120693751336109*G0_1_2 + 0.000652398655871448*G0_2_0 + 0.0120693751336111*G0_2_1 + 0.00130479731174214*G0_2_2;
    A[2501] = 0.0929668084616*G0_0_0 - 0.0283793415303836*G0_0_1 + 0.0322937334656081*G0_0_2 - 0.0283793415303824*G0_1_0 - 0.014189670765191*G0_1_2 + 0.0322937334656083*G0_2_0 - 0.0141896707651925*G0_2_1 - 0.00880738185425698*G0_2_2;
    A[2502] = -0.0283793415303827*G0_0_1 - 0.0141896707651913*G0_0_2 - 0.0283793415303827*G0_1_0 - 0.0141896707651916*G0_1_2 - 0.0141896707651918*G0_2_0 - 0.0141896707651912*G0_2_1 - 0.033272331449415*G0_2_2;
    A[2503] = -0.104709984267275*G0_0_1 - 0.0678494602105711*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0368605240567049*G0_1_2 - 0.0678494602105713*G0_2_0 - 0.036860524056704*G0_2_1 - 0.0965550010688893*G0_2_2;
    A[2504] = 0.172233245149909*G0_0_0 + 0.19865539071268*G0_0_1 + 0.166035457919137*G0_0_2 + 0.100795592332049*G0_1_0 + 0.113517366121531*G0_1_1 + 0.100795592332049*G0_1_2 + 0.166035457919137*G0_2_0 + 0.198655390712681*G0_2_1 + 0.17223324514991*G0_2_2;
    A[2505] = -0.0371867233846386*G0_0_0 - 0.177615534060845*G0_0_1 - 0.224098938291644*G0_0_2 - 0.0748627457611825*G0_1_0 - 0.149725491522365*G0_1_1 - 0.121346149991982*G0_1_2 - 0.224098938291645*G0_2_0 - 0.326851726591308*G0_2_1 - 0.44819787658329*G0_2_2;
    A[2506] = -0.268135847562929*G0_0_0 + 0.0225077536275449*G0_0_1 + 0.0534966897814105*G0_0_2 - 0.045994105238896*G0_1_0 - 0.0234863516113515*G0_1_1 + 0.0812236326559225*G0_1_2 + 0.0534966897814113*G0_2_0 - 0.104709984267276*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2507] = -0.448197876583289*G0_0_0 - 0.326851726591306*G0_0_1 - 0.224098938291644*G0_0_2 - 0.121346149991981*G0_1_0 - 0.149725491522364*G0_1_1 - 0.0748627457611813*G0_1_2 - 0.224098938291645*G0_2_0 - 0.177615534060844*G0_2_1 - 0.0371867233846388*G0_2_2;
    A[2508] = 0.726119703984281*G0_0_0 + 0.27009304353054*G0_0_1 + 0.270093043530542*G0_0_2 + 0.27009304353054*G0_1_0 + 0.540186087061081*G0_1_1 + 0.270093043530542*G0_1_2 + 0.270093043530542*G0_2_0 + 0.270093043530542*G0_2_1 + 0.726119703984283*G0_2_2;
    A[2509] = -0.209419968534551*G0_0_0 - 0.104709984267276*G0_0_1 + 0.0534966897814105*G0_0_2 + 0.0812236326559235*G0_1_0 - 0.0234863516113523*G0_1_1 - 0.0459941052388981*G0_1_2 + 0.0534966897814114*G0_2_0 + 0.0225077536275443*G0_2_1 - 0.26813584756293*G0_2_2;
    A[2510] = 0.0502346965020571*G0_0_0 + 0.012069375133611*G0_0_1 + 0.0446893079271539*G0_0_2 + 0.0120693751336112*G0_1_0 + 0.012721773789482*G0_1_2 + 0.0446893079271546*G0_2_0 + 0.0127217737894819*G0_2_1 + 0.0391439193522523*G0_2_2;
    A[2511] = -0.00880738185425665*G0_0_0 - 0.0141896707651912*G0_0_1 + 0.0322937334656082*G0_0_2 - 0.0141896707651918*G0_1_0 - 0.0283793415303829*G0_1_2 + 0.0322937334656084*G0_2_0 - 0.0283793415303824*G0_2_1 + 0.0929668084615994*G0_2_2;
    A[2512] = -0.0965550010688885*G0_0_0 - 0.0368605240567045*G0_0_1 - 0.0678494602105702*G0_0_2 - 0.0368605240567036*G0_1_0 - 0.104709984267274*G0_1_2 - 0.0678494602105709*G0_2_0 - 0.104709984267276*G0_2_1;
    A[2513] = 0.040122517336059*G0_0_0 + 0.0102752788299666*G0_0_1 + 0.0298472385060925*G0_0_2 + 0.0102752788299662*G0_1_0 + 0.0181040627004164*G0_1_2 + 0.0298472385060928*G0_2_0 + 0.0181040627004166*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2514] = -0.0332723314494149*G0_0_0 - 0.0141896707651912*G0_0_1 - 0.0141896707651913*G0_0_2 - 0.0141896707651915*G0_1_0 - 0.0283793415303834*G0_1_2 - 0.0141896707651919*G0_2_0 - 0.0283793415303832*G0_2_1;
    A[2515] = 0.00130479731174161*G0_0_0 + 0.0120693751336107*G0_0_1 + 0.000652398655870701*G0_0_2 + 0.0120693751336113*G0_1_0 + 0.0127217737894824*G0_1_2 + 0.000652398655871024*G0_2_0 + 0.012721773789482*G0_2_1;
    A[2516] = -0.234863516113515*G0_0_0 - 0.0929668084616*G0_0_1 - 0.215291556437387*G0_0_2 - 0.0929668084615994*G0_1_0 - 0.092966808461599*G0_1_2 - 0.215291556437389*G0_2_0 - 0.0929668084615997*G0_2_1 - 0.234863516113514*G0_2_2;
    A[2517] = 0.352295274170271*G0_0_0 + 0.102752788299663*G0_0_1 + 0.102752788299661*G0_0_2 + 0.102752788299662*G0_1_0 + 0.298472385060923*G0_1_2 + 0.102752788299663*G0_2_0 + 0.298472385060925*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2518] = -0.0391439193522512*G0_0_0 - 0.092966808461598*G0_0_1 - 0.0440369092712832*G0_0_2 - 0.0929668084616002*G0_1_0 - 0.0929668084615991*G0_1_2 - 0.0440369092712841*G0_2_0 - 0.0929668084615985*G0_2_1 - 0.0391439193522525*G0_2_2;
    A[2519] = -0.1859336169232*G0_0_0 + 0.298472385060924*G0_0_1 + 0.102752788299662*G0_0_2 + 0.298472385060924*G0_1_0 + 0.102752788299663*G0_1_2 + 0.102752788299663*G0_2_0 + 0.102752788299662*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2520] = 0.0189195610202552*G0_0_0 + 0.0189195610202552*G0_0_1 + 0.0189195610202552*G0_0_2 + 0.013742125287264*G0_1_0 + 0.013742125287264*G0_1_1 + 0.013742125287264*G0_1_2 + 0.0189195610202551*G0_2_0 + 0.0189195610202551*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[2521] = 0.0189195610202554*G0_0_0 + 0.00517743573299146*G0_1_0;
    A[2522] = -0.013742125287264*G0_1_1;
    A[2523] = -0.0143945239431349*G0_1_2 + 0.0335985307773501*G0_2_2;
    A[2524] = 0.0641307878721068*G0_1_1 + 0.00939454064454057*G0_1_2 + 0.0093945406445412*G0_2_1;
    A[2525] = -0.127087258163647*G0_1_1 - 0.0377738821749239*G0_1_2 - 0.0377738821749243*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[2526] = 0.124738623002511*G0_1_1 + 0.0623693115012556*G0_1_2 + 0.0623693115012562*G0_2_1 + 0.0476251018785737*G0_2_2;
    A[2527] = -0.124738623002512*G0_1_1 - 0.0962940416065415*G0_1_2 - 0.0962940416065416*G0_2_1 - 0.128522535206563*G0_2_2;
    A[2528] = -0.111886369481855*G0_0_0 - 0.0212681961813901*G0_0_2 - 0.047755581609748*G0_1_0 - 0.0118736555368494*G0_1_2 - 0.0212681961813906*G0_2_0;
    A[2529] = 0.20941996853455*G0_0_0 + 0.0359471659384848*G0_0_2 + 0.0823327103709042*G0_1_0 - 0.00182671623643911*G0_1_2 + 0.0359471659384854*G0_2_0 - 0.0528442911255409*G0_2_2;
    A[2530] = -0.271397840842283*G0_0_0 - 0.0733296089198858*G0_0_2 - 0.146659217839772*G0_1_0 - 0.0109602974186298*G0_1_2 - 0.0733296089198855*G0_2_0 + 0.121346149991983*G0_2_2;
    A[2531] = -0.0962940416065409*G0_0_2 - 0.124738623002512*G0_1_0 - 0.192588083213083*G0_1_2 - 0.0962940416065417*G0_2_0 - 0.257045070413125*G0_2_2;
    A[2532] = -0.0306627368259308*G0_0_0 - 0.02126819618139*G0_0_1 - 0.0212681961813901*G0_1_0 - 0.0118736555368493*G0_1_1;
    A[2533] = -0.00130479731174179*G0_0_0 - 0.0012395574461548*G0_0_1 - 0.00123955744615495*G0_1_0 - 0.00117431758056788*G0_1_1;
    A[2534] = -0.01500516908503*G0_0_0 - 0.0149399292194426*G0_0_1 - 0.0149399292194427*G0_1_0 - 0.0148746893538554*G0_1_1;
    A[2535] = 0.00939454064454038*G0_0_1 + 0.00939454064454064*G0_1_0 + 0.0187890812890812*G0_1_1;
    A[2536] = -0.111886369481854*G0_0_0 - 0.111886369481854*G0_0_1 - 0.0906181733004638*G0_0_2 - 0.0641307878721066*G0_1_0 - 0.0641307878721065*G0_1_1 - 0.0547362472275661*G0_1_2 - 0.0906181733004636*G0_2_0 - 0.0906181733004636*G0_2_1 - 0.0693499771190732*G0_2_2;
    A[2537] = 0.20941996853455*G0_0_0 + 0.20941996853455*G0_0_1 + 0.173472802596065*G0_0_2 + 0.127087258163646*G0_1_0 + 0.127087258163646*G0_1_1 + 0.0893133759887222*G0_1_2 + 0.173472802596065*G0_2_0 + 0.173472802596065*G0_2_1 + 0.0846813455320385*G0_2_2;
    A[2538] = -0.271397840842283*G0_0_0 - 0.271397840842283*G0_0_1 - 0.198068231922397*G0_0_2 - 0.124738623002511*G0_1_0 - 0.124738623002511*G0_1_1 - 0.0623693115012553*G0_1_2 - 0.198068231922397*G0_2_0 - 0.198068231922397*G0_2_1 - 0.0033924730105277*G0_2_2;
    A[2539] = 0.0962940416065408*G0_0_2 + 0.124738623002512*G0_1_0 + 0.124738623002512*G0_1_1 + 0.02844458139597*G0_1_2 + 0.0962940416065416*G0_2_0 + 0.0962940416065416*G0_2_1 - 0.0644569872000428*G0_2_2;
    A[2540] = -0.0306627368259306*G0_0_0 - 0.00939454064454026*G0_0_1 - 0.0306627368259305*G0_0_2 - 0.0093945406445405*G0_1_0 - 0.00939454064454042*G0_1_2 - 0.0306627368259309*G0_2_0 - 0.00939454064454032*G0_2_1 - 0.0306627368259308*G0_2_2;
    A[2541] = -0.00130479731174203*G0_0_0 - 6.52398655870408e-05*G0_0_1 - 0.00130479731174197*G0_0_2 - 6.52398655873351e-05*G0_1_0 - 6.52398655873587e-05*G0_1_2 - 0.00130479731174219*G0_2_0 - 6.52398655870615e-05*G0_2_1 - 0.00130479731174222*G0_2_2;
    A[2542] = -0.01500516908503*G0_0_0 - 6.52398655874789e-05*G0_0_1 - 0.01500516908503*G0_0_2 - 6.52398655870218e-05*G0_1_0 - 6.52398655870355e-05*G0_1_2 - 0.01500516908503*G0_2_0 - 6.52398655874903e-05*G0_2_1 - 0.0150051690850299*G0_2_2;
    A[2543] = -0.00939454064454037*G0_0_1 - 0.00939454064454068*G0_1_0 - 0.00939454064454063*G0_1_2 - 0.00939454064454018*G0_2_1;
    A[2544] = -0.0187890812890809*G0_0_0 - 0.0400572774704712*G0_0_1 - 0.0400572774704711*G0_0_2 - 0.00939454064454052*G0_1_0 - 0.0187890812890811*G0_1_1 - 0.0187890812890811*G0_1_2 - 0.0400572774704713*G0_2_0 - 0.0613254736518619*G0_2_1 - 0.0613254736518619*G0_2_2;
    A[2545] = -0.000130479731174399*G0_0_0 + 0.0136351319077009*G0_0_1 + 0.0136351319077009*G0_0_2 - 6.52398655871399e-05*G0_1_0 + 0.0148746893538556*G0_1_1 + 0.0148746893538555*G0_1_2 + 0.0136351319077011*G0_2_0 - 0.0254435475789643*G0_2_1 - 0.0254435475789644*G0_2_2;
    A[2546] = -0.000130479731174063*G0_0_0 - 0.0137656116388751*G0_0_1 - 0.013765611638875*G0_0_2 - 6.5239865586931e-05*G0_1_0 + 0.00117431758056777*G0_1_1 + 0.00117431758056779*G0_1_2 - 0.0137656116388753*G0_2_0 - 0.0528442911255406*G0_2_1 - 0.0528442911255406*G0_2_2;
    A[2547] = -0.0187890812890814*G0_0_0 + 0.0212681961813901*G0_0_1 + 0.02126819618139*G0_0_2 - 0.00939454064454091*G0_1_0 + 0.0118736555368493*G0_1_1 + 0.0118736555368492*G0_1_2 + 0.0212681961813901*G0_2_0;
    A[2548] = 0.0521918924696698*G0_0_0 - 0.000652398655870719*G0_0_1 - 0.0378391220405101*G0_0_2 - 0.000652398655870685*G0_1_0 - 0.0534966897814118*G0_1_1 - 0.05284429112554*G0_1_2 - 0.0378391220405096*G0_2_0 - 0.0528442911255406*G0_2_1;
    A[2549] = 0.0260959462348357*G0_0_0 + 0.0198981590040618*G0_0_1 - 0.0120693751336108*G0_0_2 + 0.0198981590040627*G0_1_0 + 0.0137003717732889*G0_1_1 - 0.0127217737894817*G0_1_2 - 0.0120693751336111*G0_2_0 - 0.0127217737894817*G0_2_1;
    A[2550] = -0.0528442911255403*G0_0_1 - 0.01500516908503*G0_0_2 - 0.0528442911255413*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0300103381700596*G0_1_2 - 0.0150051690850294*G0_2_0 - 0.0300103381700596*G0_2_1;
    A[2551] = -0.185933616923199*G0_0_0 - 0.0812236326559237*G0_0_1 + 0.00913358118219185*G0_0_2 - 0.0812236326559233*G0_1_0 + 0.0234863516113524*G0_1_1 + 0.0459941052388967*G0_1_2 + 0.00913358118219261*G0_2_0 + 0.0459941052388983*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[2552] = 0.104709984267275*G0_0_1 + 0.0368605240567045*G0_0_2 + 0.104709984267275*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0737210481134081*G0_1_2 + 0.0368605240567042*G0_2_0 + 0.0737210481134079*G0_2_1 - 0.0228339529554814*G0_2_2;
    A[2553] = -0.271397840842283*G0_0_1 - 0.135698920421142*G0_0_2 - 0.271397840842285*G0_1_0 - 0.542795681684568*G0_1_1 - 0.271397840842283*G0_1_2 - 0.135698920421143*G0_2_0 - 0.271397840842285*G0_2_1 + 0.0521918924696701*G0_2_2;
    A[2554] = 0.0521918924696694*G0_0_0 + 0.0528442911255391*G0_0_1 + 0.0900310145101795*G0_0_2 + 0.0528442911255407*G0_1_0 + 0.0378391220405106*G0_1_2 + 0.0900310145101798*G0_2_0 + 0.0378391220405092*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2555] = 0.0260959462348358*G0_0_0 + 0.00619778723077453*G0_0_1 + 0.0381653213684468*G0_0_2 + 0.00619778723077315*G0_1_0 + 0.00554538857490277*G0_1_2 + 0.0381653213684464*G0_2_0 + 0.00554538857490355*G0_2_1 + 0.0502346965020578*G0_2_2;
    A[2556] = 0.0528442911255403*G0_0_1 + 0.0150051690850298*G0_0_2 + 0.0528442911255411*G0_1_0 + 0.0378391220405111*G0_1_2 + 0.0150051690850294*G0_2_0 + 0.0378391220405098*G0_2_1 + 0.0300103381700599*G0_2_2;
    A[2557] = -0.185933616923198*G0_0_0 - 0.104709984267274*G0_0_1 - 0.19506719810539*G0_0_2 - 0.104709984267275*G0_1_0 - 0.067849460210571*G0_1_2 - 0.195067198105391*G0_2_0 - 0.0678494602105696*G0_2_1 - 0.292274597830152*G0_2_2;
    A[2558] = -0.104709984267275*G0_0_1 - 0.0368605240567045*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105713*G0_1_2 - 0.036860524056704*G0_2_0 - 0.0678494602105707*G0_2_1 - 0.0965550010688895*G0_2_2;
    A[2559] = 0.271397840842283*G0_0_1 + 0.135698920421141*G0_0_2 + 0.271397840842284*G0_1_0 + 0.135698920421142*G0_1_2 + 0.135698920421143*G0_2_0 + 0.13569892042114*G0_2_1 + 0.323589733311954*G0_2_2;
    A[2560] = 0.10568858225108*G0_0_0 + 0.10503618359521*G0_0_1 + 0.12787013655069*G0_0_2 + 0.0528442911255403*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0756782440810216*G0_1_2 + 0.12787013655069*G0_2_0 + 0.180062029020361*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2561] = 0.0123955744615466*G0_0_0 + 0.00619778723077278*G0_0_1 + 0.0061977872307731*G0_0_2 + 0.0061977872307727*G0_1_0 - 0.0137003717732888*G0_1_1 - 0.0264221455627704*G0_1_2 + 0.00619778723077316*G0_2_0 + 0.0714376528178605*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2562] = 0.105688582251082*G0_0_0 + 0.000652398655871061*G0_0_1 - 0.0221815542996092*G0_0_2 + 0.0528442911255415*G0_1_0 + 0.0534966897814118*G0_1_1 + 0.000652398655871851*G0_1_2 - 0.022181554299609*G0_2_0 + 0.0528442911255408*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2563] = -0.20941996853455*G0_0_0 - 0.290643601190473*G0_0_1 - 0.262916658315961*G0_0_2 - 0.104709984267274*G0_1_0 - 0.20941996853455*G0_1_1 - 0.135698920421142*G0_1_2 - 0.262916658315961*G0_2_0 - 0.390134396210781*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2564] = -0.209419968534551*G0_0_0 + 0.0812236326559235*G0_0_1 + 0.0534966897814114*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0234863516113523*G0_1_1 + 0.0225077536275443*G0_1_2 + 0.0534966897814105*G0_2_0 - 0.0459941052388981*G0_2_1 - 0.26813584756293*G0_2_2;
    A[2565] = 0.542795681684566*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684567*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842284*G0_2_1 + 0.647179466623908*G0_2_2;
    A[2566] = 0.0300103381700596*G0_0_0 + 0.0150051690850295*G0_0_1 - 0.00782878387045056*G0_0_2 + 0.0150051690850296*G0_1_0 + 0.0300103381700596*G0_1_2 - 0.00782878387045099*G0_2_0 + 0.0300103381700596*G0_2_1 - 0.045667905910962*G0_2_2;
    A[2567] = 0.00130479731174162*G0_0_0 + 0.000652398655870716*G0_0_1 + 0.000652398655870525*G0_0_2 + 0.000652398655870843*G0_1_0 + 0.0127217737894821*G0_1_2 + 0.000652398655870679*G0_2_0 + 0.0127217737894816*G0_2_1 - 0.0880738185425682*G0_2_2;
    A[2568] = 0.0300103381700602*G0_0_0 + 0.0150051690850299*G0_0_1 + 0.0378391220405102*G0_0_2 + 0.01500516908503*G0_1_0 + 0.0528442911255399*G0_1_2 + 0.0378391220405112*G0_2_0 + 0.0528442911255407*G0_2_1;
    A[2569] = 0.00130479731174226*G0_0_0 + 0.000652398655871164*G0_0_1 - 0.010764577821869*G0_0_2 + 0.000652398655871432*G0_1_0 + 0.00130479731174185*G0_1_2 - 0.0107645778218688*G0_2_0 + 0.0013047973117423*G0_2_1 - 0.0228339529554805*G0_2_2;
    A[2570] = 0.00130479731174161*G0_0_0 + 0.000652398655870933*G0_0_1 + 0.0120693751336106*G0_0_2 + 0.000652398655870831*G0_1_0 + 0.0127217737894817*G0_1_2 + 0.0120693751336111*G0_2_0 + 0.0127217737894825*G0_2_1;
    A[2571] = 0.03001033817006*G0_0_0 + 0.01500516908503*G0_0_1 + 0.0150051690850301*G0_0_2 + 0.0150051690850297*G0_1_0 + 0.0300103381700597*G0_1_2 + 0.0150051690850297*G0_2_0 + 0.0300103381700597*G0_2_1;
    A[2572] = -0.0521918924696693*G0_0_0 - 0.0260959462348337*G0_0_1 - 0.0424059126316061*G0_0_2 - 0.0260959462348345*G0_1_0 - 0.0750258454251494*G0_1_2 - 0.0424059126316054*G0_2_0 - 0.0750258454251493*G0_2_1 + 0.143527704291594*G0_2_2;
    A[2573] = -0.0521918924696702*G0_0_0 - 0.0260959462348349*G0_0_1 - 0.00978597983806236*G0_0_2 - 0.0260959462348355*G0_1_0 - 0.0587158790283786*G0_1_2 - 0.00978597983806497*G0_2_0 - 0.0587158790283788*G0_2_1 + 0.176147637085135*G0_2_2;
    A[2574] = -0.0521918924696716*G0_0_0 - 0.0260959462348363*G0_0_1 - 0.0260959462348355*G0_0_2 - 0.0260959462348356*G0_1_0 - 0.0750258454251502*G0_1_2 - 0.0260959462348352*G0_2_0 - 0.0750258454251519*G0_2_1 + 0.0456679059109617*G0_2_2;
    A[2575] = 0.371867233846397*G0_0_0 + 0.185933616923199*G0_0_1 + 0.185933616923199*G0_0_2 + 0.185933616923198*G0_1_0 + 0.316413348097373*G0_1_2 + 0.185933616923198*G0_2_0 + 0.316413348097374*G0_2_1 - 0.150051690850302*G0_2_2;
    A[2576] = 0.0335985307773497*G0_0_0 + 0.0335985307773497*G0_0_1 + 0.0335985307773497*G0_0_2 + 0.0335985307773497*G0_1_0 + 0.0335985307773497*G0_1_1 + 0.0335985307773497*G0_1_2 + 0.0479930547204846*G0_2_0 + 0.0479930547204847*G0_2_1 + 0.0479930547204846*G0_2_2;
    A[2577] = 0.0189195610202555*G0_0_0 + 0.00517743573299125*G0_2_0;
    A[2578] = 0.0189195610202553*G0_1_1 + 0.00517743573299141*G0_2_1;
    A[2579] = -0.0137421252872642*G0_2_2;
    A[2580] = -0.0306627368259307*G0_1_1 - 0.0212681961813904*G0_1_2 - 0.0212681961813905*G0_2_1 - 0.0118736555368496*G0_2_2;
    A[2581] = -0.001304797311742*G0_1_1 - 0.00123955744615454*G0_1_2 - 0.00123955744615466*G0_2_1 - 0.00117431758056786*G0_2_2;
    A[2582] = -0.0150051690850298*G0_1_1 - 0.0149399292194432*G0_1_2 - 0.014939929219443*G0_2_1 - 0.0148746893538559*G0_2_2;
    A[2583] = 0.0093945406445408*G0_1_2 + 0.0093945406445406*G0_2_1 + 0.0187890812890814*G0_2_2;
    A[2584] = -0.030662736825931*G0_0_0 - 0.0212681961813905*G0_0_2 - 0.0212681961813906*G0_2_0 - 0.01187365553685*G0_2_2;
    A[2585] = -0.00130479731174172*G0_0_0 - 0.00123955744615449*G0_0_2 - 0.0012395574461543*G0_2_0 - 0.00117431758056729*G0_2_2;
    A[2586] = -0.01500516908503*G0_0_0 - 0.014939929219443*G0_0_2 - 0.0149399292194431*G0_2_0 - 0.0148746893538561*G0_2_2;
    A[2587] = 0.00939454064454067*G0_0_2 + 0.00939454064454066*G0_2_0 + 0.0187890812890815*G0_2_2;
    A[2588] = -0.0613254736518622*G0_0_0 - 0.0212681961813904*G0_0_1 - 0.0212681961813905*G0_1_0 - 0.042536392362781*G0_2_0 - 0.01187365553685*G0_2_1;
    A[2589] = -0.0254435475789633*G0_0_0 - 0.0390786794866649*G0_0_1 - 0.0390786794866648*G0_1_0 - 0.0528442911255405*G0_1_1 - 0.0403182369328195*G0_2_0 - 0.0540186087061081*G0_2_1;
    A[2590] = -0.0528442911255408*G0_0_0 - 0.039078679486665*G0_0_1 - 0.039078679486665*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.0540186087061082*G0_2_0 - 0.0403182369328196*G0_2_1;
    A[2591] = -0.0212681961813902*G0_0_1 - 0.0212681961813905*G0_1_0 - 0.0613254736518619*G0_1_1 - 0.0118736555368496*G0_2_0 - 0.0425363923627808*G0_2_1;
    A[2592] = -0.128522535206561*G0_0_0 - 0.128522535206562*G0_0_1 - 0.0322284936000208*G0_0_2 - 0.128522535206561*G0_1_0 - 0.128522535206561*G0_1_1 - 0.0322284936000208*G0_1_2 - 0.0322284936000208*G0_2_0 - 0.0322284936000209*G0_2_1 - 0.0606730749959908*G0_2_2;
    A[2593] = 0.0476251018785736*G0_0_0 + 0.0476251018785736*G0_0_1 - 0.0147442096226819*G0_0_2 + 0.0476251018785733*G0_1_0 + 0.0476251018785734*G0_1_1 - 0.0147442096226821*G0_1_2 - 0.0147442096226817*G0_2_0 - 0.0147442096226816*G0_2_1 + 0.0476251018785736*G0_2_2;
    A[2594] = -0.0150051690850298*G0_0_0 - 0.0150051690850298*G0_0_1 + 0.0227687130898937*G0_0_2 - 0.0150051690850297*G0_1_0 - 0.0150051690850298*G0_1_1 + 0.0227687130898938*G0_1_2 + 0.0227687130898933*G0_2_0 + 0.0227687130898933*G0_2_1 - 0.0665446628988292*G0_2_2;
    A[2595] = -0.00939454064454073*G0_0_2 - 0.00939454064454061*G0_1_2 - 0.0093945406445407*G0_2_0 - 0.00939454064454066*G0_2_1 + 0.0453417065830259*G0_2_2;
    A[2596] = -0.257045070413123*G0_0_0 - 0.160751028806583*G0_0_1 - 0.257045070413123*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000417*G0_1_1 - 0.160751028806582*G0_1_2 - 0.0644569872000415*G0_2_0 - 0.0929015685960116*G0_2_1 - 0.0644569872000416*G0_2_2;
    A[2597] = 0.121346149991981*G0_0_0 + 0.194675758911868*G0_0_1 + 0.121346149991981*G0_0_2 + 0.194675758911868*G0_1_0 - 0.00339247301052892*G0_1_1 + 0.194675758911868*G0_1_2 + 0.132306447410612*G0_2_0 + 0.0589768384907262*G0_2_1 + 0.132306447410613*G0_2_2;
    A[2598] = -0.05284429112554*G0_0_0 - 0.0887914570640255*G0_0_1 - 0.0528442911255399*G0_0_2 - 0.0887914570640255*G0_1_0 + 0.0846813455320394*G0_1_1 - 0.0887914570640255*G0_1_2 - 0.0510175748891017*G0_2_0 - 0.00463203045668239*G0_2_1 - 0.0510175748891018*G0_2_2;
    A[2599] = 0.0212681961813903*G0_0_1 + 0.0212681961813904*G0_1_0 - 0.0693499771190736*G0_1_1 + 0.0212681961813905*G0_1_2 + 0.0118736555368495*G0_2_0 - 0.014613729891508*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[2600] = -0.0644569872000417*G0_0_0 - 0.160751028806582*G0_0_1 - 0.160751028806582*G0_0_2 - 0.160751028806582*G0_1_0 - 0.257045070413123*G0_1_1 - 0.257045070413123*G0_1_2 - 0.0929015685960117*G0_2_0 - 0.0644569872000417*G0_2_1 - 0.0644569872000417*G0_2_2;
    A[2601] = -0.00339247301052929*G0_0_0 + 0.194675758911868*G0_0_1 + 0.194675758911868*G0_0_2 + 0.194675758911868*G0_1_0 + 0.121346149991982*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0589768384907265*G0_2_0 + 0.132306447410612*G0_2_1 + 0.132306447410612*G0_2_2;
    A[2602] = 0.0846813455320398*G0_0_0 - 0.0887914570640256*G0_0_1 - 0.0887914570640255*G0_0_2 - 0.0887914570640255*G0_1_0 - 0.0528442911255404*G0_1_1 - 0.0528442911255405*G0_1_2 - 0.00463203045668315*G0_2_0 - 0.0510175748891021*G0_2_1 - 0.0510175748891021*G0_2_2;
    A[2603] = -0.0693499771190741*G0_0_0 + 0.0212681961813904*G0_0_1 + 0.0212681961813904*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.0146137298915075*G0_2_0 + 0.0118736555368501*G0_2_1 + 0.0118736555368501*G0_2_2;
    A[2604] = -0.0456679059109599*G0_0_0 - 0.0378391220405102*G0_0_1 - 0.0756782440810205*G0_0_2 - 0.0378391220405095*G0_1_0 - 0.05284429112554*G0_1_2 - 0.0756782440810205*G0_2_0 - 0.0528442911255406*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2605] = -0.0880738185425672*G0_0_0 - 0.0887262171984376*G0_0_1 - 0.100795592332049*G0_0_2 - 0.088726217198438*G0_1_0 - 0.0880738185425667*G0_1_1 - 0.100795592332049*G0_1_2 - 0.100795592332049*G0_2_0 - 0.100795592332049*G0_2_1 - 0.113517366121531*G0_2_2;
    A[2606] = -0.0378391220405103*G0_0_1 - 0.0528442911255398*G0_0_2 - 0.0378391220405101*G0_1_0 - 0.0456679059109604*G0_1_1 - 0.0756782440810207*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0756782440810207*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2607] = -0.0228339529554807*G0_0_0 - 0.0120693751336112*G0_0_1 - 0.024138750267222*G0_0_2 - 0.0120693751336115*G0_1_0 - 0.0127217737894816*G0_1_2 - 0.0241387502672224*G0_2_0 - 0.0127217737894819*G0_2_1 - 0.0254435475789634*G0_2_2;
    A[2608] = -0.0120693751336112*G0_0_1 - 0.0127217737894821*G0_0_2 - 0.0120693751336111*G0_1_0 - 0.0228339529554806*G0_1_1 - 0.0241387502672221*G0_1_2 - 0.0127217737894821*G0_2_0 - 0.0241387502672223*G0_2_1 - 0.025443547578964*G0_2_2;
    A[2609] = -0.0150051690850298*G0_0_1 - 0.03001033817006*G0_0_2 - 0.0150051690850297*G0_1_0 - 0.0300103381700604*G0_1_2 - 0.0300103381700597*G0_2_0 - 0.0300103381700597*G0_2_1 - 0.0600206763401204*G0_2_2;
    A[2610] = 0.0521918924696691*G0_0_0 + 0.18789081289081*G0_0_1 + 0.323589733311952*G0_0_2 + 0.187890812890811*G0_1_0 + 0.323589733311952*G0_1_1 + 0.187890812890811*G0_1_2 + 0.323589733311953*G0_2_0 + 0.187890812890811*G0_2_1 + 0.0521918924696693*G0_2_2;
    A[2611] = -0.0880738185425675*G0_0_0 - 0.097207399724759*G0_0_1 - 0.134067923781464*G0_0_2 - 0.0972073997247599*G0_1_0 - 0.292274597830151*G0_1_1 - 0.22442513761958*G0_1_2 - 0.134067923781465*G0_2_0 - 0.224425137619581*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2612] = 0.0378391220405098*G0_0_1 + 0.0528442911255402*G0_0_2 + 0.0378391220405102*G0_1_0 + 0.127870136550691*G0_1_1 + 0.09003101451018*G0_1_2 + 0.0528442911255405*G0_2_0 + 0.0900310145101808*G0_2_1 + 0.0521918924696698*G0_2_2;
    A[2613] = -0.0228339529554807*G0_0_0 - 0.0596944770121851*G0_0_1 - 0.0965550010688891*G0_0_2 - 0.0596944770121847*G0_1_0 - 0.0965550010688889*G0_1_1 - 0.0287055408583181*G0_1_2 - 0.0965550010688884*G0_2_0 - 0.0287055408583184*G0_2_1 + 0.0391439193522531*G0_2_2;
    A[2614] = 0.0120693751336119*G0_0_1 + 0.0127217737894825*G0_0_2 + 0.0120693751336111*G0_1_0 + 0.0502346965020576*G0_1_1 + 0.0446893079271553*G0_1_2 + 0.0127217737894821*G0_2_0 + 0.0446893079271548*G0_2_1 + 0.039143919352253*G0_2_2;
    A[2615] = 0.0150051690850295*G0_0_1 + 0.0300103381700598*G0_0_2 + 0.0150051690850296*G0_1_0 + 0.0300103381700594*G0_1_1 - 0.00782878387045105*G0_1_2 + 0.0300103381700594*G0_2_0 - 0.00782878387045057*G0_2_1 - 0.045667905910962*G0_2_2;
    A[2616] = 0.323589733311952*G0_0_0 + 0.187890812890811*G0_0_1 + 0.187890812890811*G0_0_2 + 0.18789081289081*G0_1_0 + 0.0521918924696695*G0_1_1 + 0.323589733311952*G0_1_2 + 0.18789081289081*G0_2_0 + 0.323589733311952*G0_2_1 + 0.0521918924696692*G0_2_2;
    A[2617] = -0.292274597830151*G0_0_0 - 0.0972073997247596*G0_0_1 - 0.22442513761958*G0_0_2 - 0.0972073997247596*G0_1_0 - 0.0880738185425675*G0_1_1 - 0.134067923781464*G0_1_2 - 0.224425137619579*G0_2_0 - 0.134067923781464*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2618] = 0.127870136550691*G0_0_0 + 0.0378391220405104*G0_0_1 + 0.0900310145101803*G0_0_2 + 0.0378391220405105*G0_1_0 + 0.0528442911255405*G0_1_2 + 0.0900310145101804*G0_2_0 + 0.0528442911255405*G0_2_1 + 0.0521918924696693*G0_2_2;
    A[2619] = -0.0965550010688887*G0_0_0 - 0.0596944770121842*G0_0_1 - 0.0287055408583179*G0_0_2 - 0.0596944770121842*G0_1_0 - 0.0228339529554803*G0_1_1 - 0.0965550010688886*G0_1_2 - 0.0287055408583175*G0_2_0 - 0.0965550010688876*G0_2_1 + 0.0391439193522533*G0_2_2;
    A[2620] = 0.0502346965020571*G0_0_0 + 0.0120693751336112*G0_0_1 + 0.0446893079271546*G0_0_2 + 0.012069375133611*G0_1_0 + 0.0127217737894819*G0_1_2 + 0.0446893079271539*G0_2_0 + 0.012721773789482*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[2621] = 0.0300103381700596*G0_0_0 + 0.0150051690850296*G0_0_1 - 0.00782878387045099*G0_0_2 + 0.0150051690850295*G0_1_0 + 0.0300103381700596*G0_1_2 - 0.00782878387045056*G0_2_0 + 0.0300103381700596*G0_2_1 - 0.045667905910962*G0_2_2;
    A[2622] = 0.647179466623905*G0_0_0 + 0.375781625781621*G0_0_1 + 0.375781625781621*G0_0_2 + 0.375781625781621*G0_1_0 + 0.647179466623904*G0_1_1 + 0.375781625781622*G0_1_2 + 0.375781625781621*G0_2_0 + 0.375781625781622*G0_2_1 + 0.647179466623904*G0_2_2;
    A[2623] = -0.584549195660301*G0_0_0 - 0.321632537344339*G0_0_1 - 0.194414799449519*G0_0_2 - 0.321632537344339*G0_1_0 - 0.268135847562928*G0_1_1 - 0.222141742324031*G0_1_2 - 0.448850275239159*G0_2_0 - 0.290643601190473*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2624] = 0.255740273101383*G0_0_0 + 0.12787013655069*G0_0_1 + 0.0756782440810207*G0_0_2 + 0.127870136550691*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255402*G0_1_2 + 0.18006202902036*G0_2_0 + 0.10503618359521*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2625] = -0.268135847562927*G0_0_0 - 0.321632537344339*G0_0_1 - 0.222141742324031*G0_0_2 - 0.321632537344339*G0_1_0 - 0.584549195660301*G0_1_1 - 0.194414799449519*G0_1_2 - 0.290643601190473*G0_2_0 - 0.44885027523916*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2626] = 0.172233245149909*G0_0_0 + 0.166035457919136*G0_0_1 + 0.100795592332049*G0_0_2 + 0.166035457919136*G0_1_0 + 0.172233245149909*G0_1_1 + 0.100795592332048*G0_1_2 + 0.19865539071268*G0_2_0 + 0.19865539071268*G0_2_1 + 0.113517366121531*G0_2_2;
    A[2627] = 0.105688582251081*G0_0_0 + 0.12787013655069*G0_0_1 + 0.0528442911255399*G0_0_2 + 0.12787013655069*G0_1_0 + 0.255740273101381*G0_1_1 + 0.0756782440810205*G0_1_2 + 0.10503618359521*G0_2_0 + 0.18006202902036*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2628] = -0.150051690850299*G0_0_0 - 0.335985307773498*G0_0_1 - 0.466465038947673*G0_0_2 - 0.335985307773498*G0_1_0 - 0.1500516908503*G0_1_1 - 0.466465038947673*G0_1_2 - 0.466465038947672*G0_2_0 - 0.466465038947673*G0_2_1 - 0.782878387045045*G0_2_2;
    A[2629] = 0.14352770429159*G0_0_0 + 0.185933616923197*G0_0_1 + 0.218553549716741*G0_0_2 + 0.185933616923197*G0_1_0 + 0.176147637085134*G0_1_1 + 0.234863516113513*G0_1_2 + 0.21855354971674*G0_2_0 + 0.234863516113512*G0_2_1 + 0.293579395141891*G0_2_2;
    A[2630] = 0.176147637085135*G0_0_0 + 0.185933616923198*G0_0_1 + 0.234863516113513*G0_0_2 + 0.185933616923198*G0_1_0 + 0.143527704291591*G0_1_1 + 0.218553549716741*G0_1_2 + 0.234863516113513*G0_2_0 + 0.218553549716741*G0_2_1 + 0.293579395141892*G0_2_2;
    A[2631] = 0.0456679059109615*G0_0_0 + 0.0717638521457957*G0_0_1 + 0.120693751336112*G0_0_2 + 0.0717638521457962*G0_1_0 + 0.0456679059109611*G0_1_1 + 0.120693751336112*G0_1_2 + 0.120693751336111*G0_2_0 + 0.12069375133611*G0_2_1 + 0.195719596761261*G0_2_2;
    A[2632] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0298472385060923*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.0298472385060923*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.0345719095719091*G0_2_0 - 0.0345719095719091*G0_2_1 - 0.0345719095719091*G0_2_2;
    A[2633] = -0.0298472385060926*G0_0_0 + 0.0047246710658169*G0_2_0;
    A[2634] = 0.0117431758056756*G0_1_1 + 0.00227817610630113*G0_2_1;
    A[2635] = -0.00946499969937462*G0_2_2;
    A[2636] = -0.00913358118219214*G0_1_1 - 0.00583896797004428*G0_1_2 - 0.00583896797004455*G0_2_1 - 0.00254435475789646*G0_2_2;
    A[2637] = -0.0156575677409009*G0_1_1 - 0.013080593050211*G0_1_2 - 0.0130805930502111*G0_2_1 - 0.0105036183595215*G0_2_2;
    A[2638] = -0.000652398655871146*G0_1_1 + 0.00192457603481891*G0_1_2 + 0.00192457603481872*G0_2_1 + 0.00450155072550908*G0_2_2;
    A[2639] = 0.00329461321214803*G0_1_2 + 0.00329461321214797*G0_2_1 + 0.00658922642429572*G0_2_2;
    A[2640] = 0.064261267603281*G0_0_0 + 0.0422754329004326*G0_0_2 + 0.0422754329004328*G0_2_0 + 0.0429604514890973*G0_2_2;
    A[2641] = 0.00880738185425672*G0_0_0 + 0.00649136662591508*G0_0_2 + 0.00649136662591477*G0_2_0 + 0.0035229527417022*G0_2_2;
    A[2642] = -0.000652398655870797*G0_0_0 - 0.00949240044292118*G0_0_2 - 0.00949240044292095*G0_2_0 - 0.018984800885842*G0_2_2;
    A[2643] = 0.00329461321214802*G0_0_2 + 0.00329461321214792*G0_2_0 + 0.0292600797158086*G0_2_2;
    A[2644] = 0.128522535206562*G0_0_0 + 0.0422754329004325*G0_0_1 + 0.0422754329004325*G0_1_0 + 0.0845508658008654*G0_2_0 + 0.0429604514890973*G0_2_1;
    A[2645] = 0.0371867233846388*G0_0_0 + 0.0743408268364854*G0_0_1 + 0.0743408268364855*G0_1_0 + 0.104709984267275*G0_1_1 + 0.0808321934624004*G0_2_0 + 0.108232937008977*G0_2_1;
    A[2646] = -0.00619778723077301*G0_0_0 + 0.0188869410874617*G0_0_1 + 0.0188869410874619*G0_1_0 + 0.0567586830607656*G0_1_1 + 0.00939454064454087*G0_2_0 + 0.0377738821749236*G0_2_1;
    A[2647] = -0.00583896797004442*G0_0_1 - 0.00583896797004437*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.00254435475789649*G0_2_0 - 0.0142875305635722*G0_2_1;
    A[2648] = 0.0642612676032808*G0_0_0 + 0.0642612676032807*G0_0_1 + 0.0219858347028483*G0_0_2 + 0.0642612676032808*G0_1_0 + 0.0642612676032808*G0_1_1 + 0.0219858347028483*G0_1_2 + 0.0219858347028481*G0_2_0 + 0.0219858347028482*G0_2_1 + 0.0226708532915126*G0_2_2;
    A[2649] = 0.00880738185425673*G0_0_0 + 0.00880738185425674*G0_0_1 + 0.00231601522834155*G0_0_2 + 0.00880738185425669*G0_1_0 + 0.00880738185425668*G0_1_1 + 0.00231601522834159*G0_1_2 + 0.0023160152283417*G0_2_0 + 0.00231601522834174*G0_2_1 - 0.00065239865587084*G0_2_2;
    A[2650] = -0.000652398655870699*G0_0_0 - 0.000652398655870676*G0_0_1 + 0.00884000178705029*G0_0_2 - 0.000652398655870477*G0_1_0 - 0.000652398655870473*G0_1_1 + 0.0088400017870504*G0_1_2 + 0.0088400017870506*G0_2_0 + 0.00884000178705058*G0_2_1 - 0.000652398655870882*G0_2_2;
    A[2651] = -0.00329461321214786*G0_0_2 - 0.00329461321214795*G0_1_2 - 0.00329461321214791*G0_2_0 - 0.00329461321214792*G0_2_1 + 0.0226708532915128*G0_2_2;
    A[2652] = 0.128522535206562*G0_0_0 + 0.0862471023061291*G0_0_1 + 0.128522535206562*G0_0_2 + 0.0862471023061291*G0_1_0 + 0.0439716694056965*G0_1_1 + 0.086247102306129*G0_1_2 + 0.0439716694056964*G0_2_0 + 0.0446566879943609*G0_2_1 + 0.0439716694056964*G0_2_2;
    A[2653] = 0.0371867233846402*G0_0_0 - 0.0371541034518462*G0_0_1 + 0.0371867233846402*G0_0_2 - 0.0371541034518464*G0_1_0 - 0.00678494602105686*G0_1_1 - 0.0371541034518463*G0_1_2 - 0.0436454700777613*G0_2_0 - 0.0097533599052695*G0_2_1 - 0.0436454700777613*G0_2_2;
    A[2654] = -0.00619778723077324*G0_0_0 - 0.0250847283182349*G0_0_1 - 0.0061977872307733*G0_0_2 - 0.0250847283182347*G0_1_0 + 0.012787013655069*G0_1_1 - 0.0250847283182348*G0_1_2 - 0.0155923278753138*G0_2_0 + 0.00329461321214824*G0_2_1 - 0.0155923278753138*G0_2_2;
    A[2655] = 0.00583896797004423*G0_0_1 + 0.00583896797004436*G0_1_0 - 0.031869674339292*G0_1_1 + 0.0058389679700444*G0_1_2 + 0.00254435475789645*G0_2_0 - 0.00590420783563162*G0_2_1 + 0.00254435475789647*G0_2_2;
    A[2656] = 0.0920860702761733*G0_0_0 + 0.134361503176606*G0_0_1 + 0.134361503176605*G0_0_2 + 0.134361503176606*G0_1_0 + 0.176636936077038*G0_1_1 + 0.176636936077038*G0_1_2 + 0.128000616281865*G0_2_0 + 0.1273155976932*G0_2_1 + 0.1273155976932*G0_2_2;
    A[2657] = -0.0622388317700808*G0_0_0 - 0.250814663249556*G0_0_1 - 0.250814663249556*G0_0_2 - 0.250814663249556*G0_1_0 - 0.334680510461757*G0_1_1 - 0.334680510461757*G0_1_2 - 0.160783648739376*G0_2_0 - 0.166948816037356*G0_2_1 - 0.166948816037356*G0_2_2;
    A[2658] = -0.0622388317700815*G0_0_0 + 0.188575831479476*G0_0_1 + 0.188575831479475*G0_0_2 + 0.188575831479475*G0_1_0 + 0.104709984267275*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0985448169692954*G0_2_0 + 0.182410664181496*G0_2_1 + 0.182410664181496*G0_2_2;
    A[2659] = 0.092086070276174*G0_0_0 - 0.0422754329004326*G0_0_1 - 0.0422754329004326*G0_0_2 - 0.0422754329004326*G0_1_0 - 0.0359145460056917*G0_2_0 - 0.0429604514890974*G0_2_1 - 0.0429604514890974*G0_2_2;
    A[2660] = 0.0391439193522509*G0_0_0 + 0.0678494602105704*G0_0_1 + 0.135698920421141*G0_0_2 + 0.0678494602105702*G0_1_0 + 0.104709984267275*G0_1_2 + 0.135698920421141*G0_2_0 + 0.104709984267275*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2661] = 0.019571959676126*G0_0_0 + 0.0606730749959899*G0_0_1 + 0.0748627457611815*G0_0_2 + 0.0606730749959911*G0_1_0 + 0.0929668084615988*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0748627457611827*G0_2_0 + 0.121346149991982*G0_2_1 + 0.149725491522364*G0_2_2;
    A[2662] = -0.00554538857490255*G0_0_1 - 0.00619778723077364*G0_0_2 - 0.00554538857490263*G0_1_0 + 0.0391439193522522*G0_1_1 + 0.0264221455627701*G0_1_2 - 0.00619778723077418*G0_2_0 + 0.0264221455627698*G0_2_1 + 0.0137003717732882*G0_2_2;
    A[2663] = -0.00489298991903107*G0_0_0 + 0.0141896707651917*G0_0_1 + 0.028379341530383*G0_0_2 + 0.0141896707651919*G0_1_0 + 0.0283793415303832*G0_1_2 + 0.0283793415303836*G0_2_0 + 0.0283793415303828*G0_2_1 + 0.056758683060766*G0_2_2;
    A[2664] = -0.0102752788299663*G0_0_1 - 0.0181040627004167*G0_0_2 - 0.0102752788299663*G0_1_0 + 0.0195719596761261*G0_1_1 + 0.00146789697570961*G0_1_2 - 0.018104062700417*G0_2_0 + 0.0014678969757096*G0_2_1 - 0.0166361657247074*G0_2_2;
    A[2665] = -0.00065239865587072*G0_0_1 - 0.00130479731174189*G0_0_2 - 0.000652398655871108*G0_1_0 - 0.0127217737894823*G0_1_2 - 0.0013047973117423*G0_2_0 - 0.012721773789482*G0_2_1 - 0.0254435475789643*G0_2_2;
    A[2666] = 0.0391439193522531*G0_0_0 - 0.0287055408583177*G0_0_1 - 0.096555001068889*G0_0_2 - 0.0287055408583184*G0_1_0 - 0.096555001068889*G0_1_1 - 0.0596944770121845*G0_1_2 - 0.0965550010688886*G0_2_0 - 0.0596944770121842*G0_2_1 - 0.0228339529554801*G0_2_2;
    A[2667] = 0.019571959676126*G0_0_0 - 0.0411011153198654*G0_0_1 - 0.0552907860850566*G0_0_2 - 0.0411011153198645*G0_1_0 - 0.00880738185425634*G0_1_1 + 0.005382288910935*G0_1_2 - 0.0552907860850568*G0_2_0 + 0.00538228891093443*G0_2_1 + 0.0195719596761258*G0_2_2;
    A[2668] = 0.00554538857490256*G0_0_1 + 0.0061977872307732*G0_0_2 + 0.00554538857490257*G0_1_0 + 0.0502346965020567*G0_1_1 + 0.0381653213684456*G0_1_2 + 0.00619778723077419*G0_2_0 + 0.0381653213684461*G0_2_1 + 0.0260959462348353*G0_2_2;
    A[2669] = -0.00489298991903137*G0_0_0 - 0.0190826606842232*G0_0_1 - 0.0332723314494144*G0_0_2 - 0.019082660684223*G0_1_0 - 0.0332723314494147*G0_1_1 - 0.019082660684223*G0_1_2 - 0.0332723314494146*G0_2_0 - 0.0190826606842238*G0_2_1 - 0.00489298991903137*G0_2_2;
    A[2670] = 0.0102752788299661*G0_0_1 + 0.0181040627004168*G0_0_2 + 0.0102752788299664*G0_1_0 + 0.0401225173360588*G0_1_1 + 0.0298472385060927*G0_1_2 + 0.018104062700417*G0_2_0 + 0.0298472385060931*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2671] = 0.00065239865587107*G0_0_1 + 0.00130479731174163*G0_0_2 + 0.000652398655871152*G0_1_0 + 0.0013047973117417*G0_1_1 - 0.0107645778218697*G0_1_2 + 0.00130479731174239*G0_2_0 - 0.0107645778218692*G0_2_1 - 0.0228339529554808*G0_2_2;
    A[2672] = -0.292274597830151*G0_0_0 - 0.224425137619579*G0_0_1 - 0.0972073997247596*G0_0_2 - 0.224425137619579*G0_1_0 - 0.156575677409009*G0_1_1 - 0.134067923781464*G0_1_2 - 0.0972073997247595*G0_2_0 - 0.134067923781464*G0_2_1 - 0.0880738185425671*G0_2_2;
    A[2673] = 0.45602666045374*G0_0_0 + 0.228013330226869*G0_0_1 + 0.228013330226869*G0_0_2 + 0.228013330226869*G0_1_0 + 0.0929668084615993*G0_1_1 + 0.363059851992139*G0_1_2 + 0.228013330226869*G0_2_0 + 0.36305985199214*G0_2_1 + 0.0929668084615989*G0_2_2;
    A[2674] = -0.292274597830151*G0_0_0 - 0.0678494602105705*G0_0_1 - 0.195067198105391*G0_0_2 - 0.0678494602105708*G0_1_0 - 0.104709984267275*G0_1_2 - 0.195067198105391*G0_2_0 - 0.104709984267275*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2675] = -0.00880738185425686*G0_0_0 + 0.00538228891093469*G0_0_1 - 0.0411011153198649*G0_0_2 + 0.00538228891093423*G0_1_0 + 0.0195719596761259*G0_1_1 - 0.0552907860850564*G0_1_2 - 0.041101115319865*G0_2_0 - 0.0552907860850559*G0_2_1 + 0.0195719596761264*G0_2_2;
    A[2676] = -0.00880738185425666*G0_0_0 - 0.0141896707651918*G0_0_1 + 0.0322937334656084*G0_0_2 - 0.0141896707651912*G0_1_0 - 0.0283793415303824*G0_1_2 + 0.0322937334656082*G0_2_0 - 0.0283793415303829*G0_2_1 + 0.0929668084615994*G0_2_2;
    A[2677] = 0.00130479731174162*G0_0_0 + 0.000652398655870849*G0_0_1 + 0.000652398655870688*G0_0_2 + 0.000652398655870708*G0_1_0 + 0.0127217737894816*G0_1_2 + 0.000652398655870524*G0_2_0 + 0.0127217737894821*G0_2_1 - 0.0880738185425682*G0_2_2;
    A[2678] = -0.584549195660302*G0_0_0 - 0.321632537344339*G0_0_1 - 0.448850275239159*G0_0_2 - 0.321632537344339*G0_1_0 - 0.268135847562928*G0_1_1 - 0.290643601190473*G0_1_2 - 0.194414799449519*G0_2_0 - 0.222141742324031*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2679] = 0.912053320907479*G0_0_0 + 0.456026660453739*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453739*G0_1_0 + 0.72611970398428*G0_1_1 + 0.45602666045374*G0_1_2 + 0.456026660453738*G0_2_0 + 0.45602666045374*G0_2_1 + 0.72611970398428*G0_2_2;
    A[2680] = -0.584549195660302*G0_0_0 - 0.262916658315961*G0_0_1 - 0.135698920421141*G0_0_2 - 0.262916658315961*G0_1_0 - 0.20941996853455*G0_1_1 - 0.104709984267275*G0_1_2 - 0.390134396210781*G0_2_0 - 0.290643601190473*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2681] = -0.037186723384641*G0_0_0 + 0.186912214907004*G0_0_1 + 0.0376760223765422*G0_0_2 + 0.186912214907005*G0_1_0 - 0.0371867233846399*G0_1_1 + 0.140428810676205*G0_1_2 + 0.140428810676205*G0_2_0 + 0.037676022376542*G0_2_1 + 0.0655660649150228*G0_2_2;
    A[2682] = -0.037186723384638*G0_0_0 - 0.224098938291644*G0_0_1 - 0.0748627457611815*G0_0_2 - 0.224098938291644*G0_1_0 - 0.448197876583288*G0_1_1 - 0.121346149991982*G0_1_2 - 0.177615534060844*G0_2_0 - 0.326851726591307*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2683] = 0.0123955744615463*G0_0_0 + 0.0061977872307734*G0_0_1 + 0.00619778723077362*G0_0_2 + 0.0061977872307728*G0_1_0 + 0.17223324514991*G0_1_1 - 0.0264221455627703*G0_1_2 + 0.00619778723077285*G0_2_0 + 0.0714376528178607*G0_2_1 - 0.0137003717732882*G0_2_2;
    A[2684] = -0.0391439193522547*G0_0_0 + 0.249542485870607*G0_0_1 + 0.445262082631869*G0_0_2 + 0.249542485870609*G0_1_0 + 0.352295274170271*G0_1_1 + 0.249542485870608*G0_1_2 + 0.44526208263187*G0_2_0 + 0.249542485870608*G0_2_1 + 0.146789697570945*G0_2_2;
    A[2685] = -0.0391439193522491*G0_0_0 - 0.288686405222859*G0_0_1 - 0.484406001984121*G0_0_2 - 0.288686405222861*G0_1_0 - 0.185933616923198*G0_1_1 - 0.484406001984123*G0_1_2 - 0.484406001984122*G0_2_0 - 0.484406001984122*G0_2_1 - 0.782878387045046*G0_2_2;
    A[2686] = -0.0391439193522521*G0_0_0 - 0.0195719596761249*G0_0_1 - 0.0195719596761253*G0_0_2 - 0.0195719596761264*G0_1_0 - 0.234863516113514*G0_1_1 - 0.141896707651915*G0_1_2 - 0.0195719596761256*G0_2_0 - 0.141896707651914*G0_2_1 - 0.0489298991903152*G0_2_2;
    A[2687] = 0.00978597983806245*G0_0_0 + 0.00489298991903163*G0_0_1 + 0.00489298991903207*G0_0_2 + 0.00489298991903115*G0_1_0 - 0.0391439193522521*G0_1_1 + 0.0538228891093478*G0_1_2 + 0.00489298991903115*G0_2_0 + 0.0538228891093463*G0_2_1 + 0.146789697570947*G0_2_2;
    A[2688] = 0.0189195610202553*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0189195610202552*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202552*G0_1_2 + 0.013742125287264*G0_2_0 + 0.013742125287264*G0_2_1 + 0.0137421252872641*G0_2_2;
    A[2689] = 0.0335985307773501*G0_0_0 - 0.014394523943135*G0_2_0;
    A[2690] = 0.0189195610202552*G0_1_1 + 0.00517743573299129*G0_2_1;
    A[2691] = -0.0137421252872643*G0_2_2;
    A[2692] = -0.0306627368259309*G0_1_1 - 0.0212681961813906*G0_1_2 - 0.0212681961813906*G0_2_1 - 0.01187365553685*G0_2_2;
    A[2693] = -0.00130479731174162*G0_1_1 - 0.00123955744615429*G0_1_2 - 0.0012395574461548*G0_2_1 - 0.0011743175805676*G0_2_2;
    A[2694] = -0.0150051690850301*G0_1_1 - 0.0149399292194433*G0_1_2 - 0.0149399292194429*G0_2_1 - 0.0148746893538561*G0_2_2;
    A[2695] = 0.00939454064454068*G0_1_2 + 0.00939454064454017*G0_2_1 + 0.0187890812890811*G0_2_2;
    A[2696] = -0.128522535206562*G0_0_0 - 0.0962940416065404*G0_0_2 - 0.0962940416065403*G0_2_0 - 0.124738623002511*G0_2_2;
    A[2697] = 0.0476251018785731*G0_0_0 + 0.0623693115012551*G0_0_2 + 0.0623693115012545*G0_2_0 + 0.124738623002511*G0_2_2;
    A[2698] = -0.0150051690850296*G0_0_0 - 0.0377738821749233*G0_0_2 - 0.0377738821749228*G0_2_0 - 0.127087258163646*G0_2_2;
    A[2699] = 0.00939454064454022*G0_0_2 + 0.00939454064454018*G0_2_0 + 0.0641307878721069*G0_2_2;
    A[2700] = -0.257045070413124*G0_0_0 - 0.0962940416065404*G0_0_1 - 0.0962940416065407*G0_1_0 - 0.192588083213081*G0_2_0 - 0.124738623002511*G0_2_1;
    A[2701] = 0.121346149991983*G0_0_0 - 0.073329608919886*G0_0_1 - 0.073329608919886*G0_1_0 - 0.271397840842283*G0_1_1 - 0.0109602974186305*G0_2_0 - 0.146659217839772*G0_2_1;
    A[2702] = -0.0528442911255407*G0_0_0 + 0.0359471659384853*G0_0_1 + 0.0359471659384851*G0_1_0 + 0.20941996853455*G0_1_1 - 0.00182671623643855*G0_2_0 + 0.0823327103709045*G0_2_1;
    A[2703] = -0.0212681961813908*G0_0_1 - 0.0212681961813907*G0_1_0 - 0.111886369481855*G0_1_1 - 0.0118736555368499*G0_2_0 - 0.0477555816097482*G0_2_1;
    A[2704] = -0.030662736825931*G0_0_0 - 0.0306627368259309*G0_0_1 - 0.00939454064454068*G0_0_2 - 0.0306627368259309*G0_1_0 - 0.0306627368259308*G0_1_1 - 0.00939454064454035*G0_1_2 - 0.00939454064454054*G0_2_0 - 0.0093945406445405*G0_2_1;
    A[2705] = -0.00130479731174154*G0_0_0 - 0.0013047973117416*G0_0_1 - 6.52398655867775e-05*G0_0_2 - 0.00130479731174221*G0_1_0 - 0.00130479731174225*G0_1_1 - 6.52398655873212e-05*G0_1_2 - 6.52398655868723e-05*G0_2_0 - 6.52398655868609e-05*G0_2_1;
    A[2706] = -0.0150051690850306*G0_0_0 - 0.0150051690850306*G0_0_1 - 6.52398655873342e-05*G0_0_2 - 0.01500516908503*G0_1_0 - 0.01500516908503*G0_1_1 - 6.52398655868513e-05*G0_1_2 - 6.52398655874889e-05*G0_2_0 - 6.52398655875528e-05*G0_2_1;
    A[2707] = -0.00939454064454035*G0_0_2 - 0.00939454064454064*G0_1_2 - 0.00939454064454019*G0_2_0 - 0.00939454064454019*G0_2_1;
    A[2708] = -0.061325473651862*G0_0_0 - 0.0400572774704716*G0_0_1 - 0.0613254736518619*G0_0_2 - 0.0400572774704714*G0_1_0 - 0.018789081289081*G0_1_1 - 0.0400572774704714*G0_1_2 - 0.018789081289081*G0_2_0 - 0.00939454064454043*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[2709] = -0.0254435475789641*G0_0_0 + 0.0136351319077016*G0_0_1 - 0.025443547578964*G0_0_2 + 0.0136351319077015*G0_1_0 - 0.000130479731173831*G0_1_1 + 0.0136351319077016*G0_1_2 + 0.0148746893538561*G0_2_0 - 6.52398655869635e-05*G0_2_1 + 0.014874689353856*G0_2_2;
    A[2710] = -0.0528442911255414*G0_0_0 - 0.013765611638876*G0_0_1 - 0.0528442911255413*G0_0_2 - 0.0137656116388758*G0_1_0 - 0.000130479731174805*G0_1_1 - 0.0137656116388758*G0_1_2 + 0.00117431758056714*G0_2_0 - 6.52398655873093e-05*G0_2_1 + 0.00117431758056723*G0_2_2;
    A[2711] = 0.0212681961813908*G0_0_1 + 0.0212681961813907*G0_1_0 - 0.0187890812890807*G0_1_1 + 0.0212681961813907*G0_1_2 + 0.0118736555368499*G0_2_0 - 0.00939454064454048*G0_2_1 + 0.01187365553685*G0_2_2;
    A[2712] = -0.0693499771190736*G0_0_0 - 0.0906181733004638*G0_0_1 - 0.0906181733004638*G0_0_2 - 0.0906181733004637*G0_1_0 - 0.111886369481854*G0_1_1 - 0.111886369481854*G0_1_2 - 0.0547362472275659*G0_2_0 - 0.0641307878721064*G0_2_1 - 0.0641307878721063*G0_2_2;
    A[2713] = 0.084681345532039*G0_0_0 + 0.173472802596064*G0_0_1 + 0.173472802596064*G0_0_2 + 0.173472802596064*G0_1_0 + 0.209419968534549*G0_1_1 + 0.209419968534549*G0_1_2 + 0.089313375988722*G0_2_0 + 0.127087258163645*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2714] = -0.00339247301052835*G0_0_0 - 0.198068231922397*G0_0_1 - 0.198068231922396*G0_0_2 - 0.198068231922397*G0_1_0 - 0.271397840842283*G0_1_1 - 0.271397840842283*G0_1_2 - 0.0623693115012553*G0_2_0 - 0.124738623002511*G0_2_1 - 0.124738623002511*G0_2_2;
    A[2715] = -0.0644569872000424*G0_0_0 + 0.0962940416065405*G0_0_1 + 0.0962940416065404*G0_0_2 + 0.0962940416065406*G0_1_0 + 0.0284445813959702*G0_2_0 + 0.124738623002511*G0_2_1 + 0.124738623002511*G0_2_2;
    A[2716] = 0.0521918924696705*G0_0_0 - 0.135698920421141*G0_0_1 - 0.271397840842282*G0_0_2 - 0.135698920421141*G0_1_0 - 0.271397840842283*G0_1_2 - 0.271397840842283*G0_2_0 - 0.271397840842283*G0_2_1 - 0.542795681684566*G0_2_2;
    A[2717] = -0.0880738185425675*G0_0_0 + 0.00913358118219277*G0_0_1 + 0.045994105238897*G0_0_2 + 0.00913358118219229*G0_1_0 - 0.185933616923198*G0_1_1 - 0.0812236326559234*G0_1_2 + 0.0459941052388973*G0_2_0 - 0.0812236326559236*G0_2_1 + 0.0234863516113518*G0_2_2;
    A[2718] = -0.0378391220405103*G0_0_1 - 0.0528442911255409*G0_0_2 - 0.0378391220405103*G0_1_0 + 0.05219189246967*G0_1_1 - 0.000652398655870831*G0_1_2 - 0.0528442911255407*G0_2_0 - 0.00065239865587032*G0_2_1 - 0.0534966897814115*G0_2_2;
    A[2719] = -0.0228339529554809*G0_0_0 + 0.0368605240567035*G0_0_1 + 0.0737210481134077*G0_0_2 + 0.0368605240567039*G0_1_0 + 0.104709984267275*G0_1_2 + 0.073721048113408*G0_2_0 + 0.104709984267274*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2720] = -0.012069375133611*G0_0_1 - 0.0127217737894817*G0_0_2 - 0.0120693751336111*G0_1_0 + 0.0260959462348345*G0_1_1 + 0.019898159004062*G0_1_2 - 0.012721773789482*G0_2_0 + 0.0198981590040616*G0_2_1 + 0.0137003717732888*G0_2_2;
    A[2721] = -0.0150051690850301*G0_0_1 - 0.0300103381700598*G0_0_2 - 0.0150051690850302*G0_1_0 - 0.0528442911255411*G0_1_2 - 0.0300103381700599*G0_2_0 - 0.05284429112554*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2722] = -0.0456679059109618*G0_0_0 - 0.00782878387045117*G0_0_1 + 0.0300103381700599*G0_0_2 - 0.00782878387045077*G0_1_0 + 0.03001033817006*G0_1_1 + 0.0150051690850294*G0_1_2 + 0.0300103381700603*G0_2_0 + 0.0150051690850298*G0_2_1;
    A[2723] = -0.0880738185425674*G0_0_0 + 0.000652398655871072*G0_0_1 + 0.0127217737894827*G0_0_2 + 0.000652398655871305*G0_1_0 + 0.00130479731174178*G0_1_1 + 0.000652398655870883*G0_1_2 + 0.0127217737894817*G0_2_0 + 0.000652398655870627*G0_2_1;
    A[2724] = 0.0378391220405109*G0_0_1 + 0.0528442911255408*G0_0_2 + 0.0378391220405102*G0_1_0 + 0.0300103381700601*G0_1_1 + 0.0150051690850297*G0_1_2 + 0.0528442911255407*G0_2_0 + 0.0150051690850304*G0_2_1;
    A[2725] = -0.0228339529554809*G0_0_0 - 0.010764577821869*G0_0_1 + 0.00130479731174123*G0_0_2 - 0.0107645778218694*G0_1_0 + 0.0013047973117425*G0_1_1 + 0.000652398655871269*G0_1_2 + 0.00130479731174161*G0_2_0 + 0.000652398655870735*G0_2_1;
    A[2726] = 0.0120693751336108*G0_0_1 + 0.0127217737894814*G0_0_2 + 0.012069375133611*G0_1_0 + 0.00130479731174129*G0_1_1 + 0.000652398655870881*G0_1_2 + 0.012721773789482*G0_2_0 + 0.000652398655870978*G0_2_1;
    A[2727] = 0.0150051690850301*G0_0_1 + 0.0300103381700608*G0_0_2 + 0.0150051690850302*G0_1_0 + 0.0300103381700601*G0_1_1 + 0.01500516908503*G0_1_2 + 0.0300103381700598*G0_2_0 + 0.0150051690850302*G0_2_1;
    A[2728] = 0.127870136550691*G0_0_0 + 0.0900310145101803*G0_0_1 + 0.0378391220405104*G0_0_2 + 0.09003101451018*G0_1_0 + 0.052191892469669*G0_1_1 + 0.0528442911255397*G0_1_2 + 0.0378391220405101*G0_2_0 + 0.0528442911255407*G0_2_1;
    A[2729] = -0.292274597830151*G0_0_0 - 0.195067198105391*G0_0_1 - 0.0678494602105705*G0_0_2 - 0.19506719810539*G0_1_0 - 0.185933616923198*G0_1_1 - 0.104709984267274*G0_1_2 - 0.0678494602105703*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2730] = 0.323589733311953*G0_0_0 + 0.135698920421141*G0_0_1 + 0.135698920421142*G0_0_2 + 0.135698920421141*G0_1_0 + 0.271397840842282*G0_1_2 + 0.135698920421141*G0_2_0 + 0.271397840842283*G0_2_1;
    A[2731] = 0.050234696502057*G0_0_0 + 0.0381653213684453*G0_0_1 + 0.00554538857490209*G0_0_2 + 0.0381653213684467*G0_1_0 + 0.0260959462348348*G0_1_1 + 0.00619778723077363*G0_1_2 + 0.00554538857490261*G0_2_0 + 0.00619778723077356*G0_2_1;
    A[2732] = -0.0965550010688885*G0_0_0 - 0.0368605240567036*G0_0_1 - 0.0678494602105709*G0_0_2 - 0.0368605240567045*G0_1_0 - 0.104709984267276*G0_1_2 - 0.0678494602105701*G0_2_0 - 0.104709984267274*G0_2_1;
    A[2733] = 0.0300103381700602*G0_0_0 + 0.01500516908503*G0_0_1 + 0.0378391220405112*G0_0_2 + 0.0150051690850299*G0_1_0 + 0.0528442911255407*G0_1_2 + 0.0378391220405102*G0_2_0 + 0.0528442911255399*G0_2_1;
    A[2734] = 0.255740273101383*G0_0_0 + 0.127870136550691*G0_0_1 + 0.18006202902036*G0_0_2 + 0.12787013655069*G0_1_0 + 0.105688582251081*G0_1_1 + 0.10503618359521*G0_1_2 + 0.0756782440810207*G0_2_0 + 0.0528442911255402*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2735] = -0.584549195660302*G0_0_0 - 0.262916658315961*G0_0_1 - 0.390134396210781*G0_0_2 - 0.262916658315961*G0_1_0 - 0.20941996853455*G0_1_1 - 0.290643601190473*G0_1_2 - 0.135698920421141*G0_2_0 - 0.104709984267275*G0_2_1 - 0.209419968534551*G0_2_2;
    A[2736] = 0.647179466623906*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684566*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684566*G0_2_2;
    A[2737] = 0.172233245149911*G0_0_0 + 0.00619778723077296*G0_0_1 + 0.0714376528178603*G0_0_2 + 0.00619778723077296*G0_1_0 + 0.0123955744615467*G0_1_1 + 0.00619778723077314*G0_1_2 - 0.0264221455627706*G0_2_0 + 0.00619778723077291*G0_2_1 - 0.0137003717732889*G0_2_2;
    A[2738] = -0.268135847562929*G0_0_0 + 0.053496689781411*G0_0_1 - 0.045994105238897*G0_0_2 + 0.0534966897814115*G0_1_0 - 0.20941996853455*G0_1_1 + 0.0812236326559235*G0_1_2 + 0.022507753627545*G0_2_0 - 0.104709984267276*G0_2_1 - 0.0234863516113517*G0_2_2;
    A[2739] = 0.105688582251082*G0_0_0 - 0.0221815542996088*G0_0_1 + 0.0528442911255409*G0_0_2 - 0.0221815542996094*G0_1_0 + 0.105688582251081*G0_1_1 + 0.000652398655870836*G0_1_2 + 0.000652398655871363*G0_2_0 + 0.0528442911255414*G0_2_1 + 0.0534966897814114*G0_2_2;
    A[2740] = 0.143527704291594*G0_0_0 - 0.0424059126316054*G0_0_1 - 0.0750258454251496*G0_0_2 - 0.042405912631606*G0_1_0 - 0.0521918924696687*G0_1_1 - 0.0260959462348343*G0_1_2 - 0.0750258454251506*G0_2_0 - 0.0260959462348347*G0_2_1;
    A[2741] = -0.150051690850303*G0_0_0 + 0.185933616923197*G0_0_1 + 0.316413348097373*G0_0_2 + 0.185933616923198*G0_1_0 + 0.371867233846396*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097374*G0_2_0 + 0.185933616923198*G0_2_1;
    A[2742] = 0.176147637085135*G0_0_0 - 0.00978597983806451*G0_0_1 - 0.0587158790283788*G0_0_2 - 0.0097859798380637*G0_1_0 - 0.052191892469671*G0_1_1 - 0.026095946234835*G0_1_2 - 0.0587158790283791*G0_2_0 - 0.0260959462348354*G0_2_1;
    A[2743] = 0.0456679059109619*G0_0_0 - 0.0260959462348344*G0_0_1 - 0.075025845425151*G0_0_2 - 0.0260959462348346*G0_1_0 - 0.0521918924696694*G0_1_1 - 0.0260959462348349*G0_1_2 - 0.07502584542515*G0_2_0 - 0.026095946234835*G0_2_1;
    A[2744] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0298472385060923*G0_0_2 - 0.0298472385060922*G0_1_0 - 0.0298472385060923*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.034571909571909*G0_2_0 - 0.0345719095719091*G0_2_1 - 0.0345719095719091*G0_2_2;
    A[2745] = 0.0117431758056757*G0_0_0 + 0.00227817610630111*G0_2_0;
    A[2746] = -0.0298472385060925*G0_1_1 + 0.00472467106581678*G0_2_1;
    A[2747] = -0.00946499969937459*G0_2_2;
    A[2748] = 0.0642612676032807*G0_1_1 + 0.0422754329004321*G0_1_2 + 0.0422754329004322*G0_2_1 + 0.0429604514890965*G0_2_2;
    A[2749] = 0.00880738185425716*G0_1_1 + 0.00649136662591563*G0_1_2 + 0.00649136662591562*G0_2_1 + 0.00352295274170339*G0_2_2;
    A[2750] = -0.000652398655871072*G0_1_1 - 0.00949240044292133*G0_1_2 - 0.00949240044292128*G0_2_1 - 0.0189848008858426*G0_2_2;
    A[2751] = 0.00329461321214781*G0_1_2 + 0.00329461321214806*G0_2_1 + 0.0292600797158089*G0_2_2;
    A[2752] = -0.00913358118219218*G0_0_0 - 0.00583896797004436*G0_0_2 - 0.00583896797004425*G0_2_0 - 0.00254435475789621*G0_2_2;
    A[2753] = -0.015657567740901*G0_0_0 - 0.0130805930502109*G0_0_2 - 0.0130805930502108*G0_2_0 - 0.0105036183595212*G0_2_2;
    A[2754] = -0.000652398655871055*G0_0_0 + 0.00192457603481894*G0_0_2 + 0.00192457603481887*G0_2_0 + 0.00450155072550913*G0_2_2;
    A[2755] = 0.00329461321214776*G0_0_2 + 0.00329461321214801*G0_2_0 + 0.00658922642429581*G0_2_2;
    A[2756] = -0.0435476102793807*G0_0_0 - 0.00583896797004431*G0_0_1 - 0.00583896797004385*G0_1_0 - 0.0142875305635718*G0_2_0 - 0.00254435475789617*G0_2_1;
    A[2757] = 0.0567586830607657*G0_0_0 + 0.018886941087462*G0_0_1 + 0.0188869410874612*G0_1_0 - 0.00619778723077338*G0_1_1 + 0.0377738821749232*G0_2_0 + 0.00939454064454058*G0_2_1;
    A[2758] = 0.104709984267275*G0_0_0 + 0.0743408268364854*G0_0_1 + 0.0743408268364858*G0_1_0 + 0.0371867233846395*G0_1_1 + 0.108232937008978*G0_2_0 + 0.0808321934624009*G0_2_1;
    A[2759] = 0.0422754329004325*G0_0_1 + 0.0422754329004322*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0429604514890965*G0_2_0 + 0.0845508658008646*G0_2_1;
    A[2760] = 0.0642612676032807*G0_0_0 + 0.0642612676032808*G0_0_1 + 0.0219858347028484*G0_0_2 + 0.0642612676032805*G0_1_0 + 0.0642612676032807*G0_1_1 + 0.0219858347028482*G0_1_2 + 0.0219858347028481*G0_2_0 + 0.0219858347028482*G0_2_1 + 0.0226708532915125*G0_2_2;
    A[2761] = 0.00880738185425706*G0_0_0 + 0.00880738185425701*G0_0_1 + 0.0023160152283416*G0_0_2 + 0.00880738185425711*G0_1_0 + 0.00880738185425702*G0_1_1 + 0.00231601522834175*G0_1_2 + 0.00231601522834164*G0_2_0 + 0.00231601522834162*G0_2_1 - 0.000652398655870555*G0_2_2;
    A[2762] = -0.000652398655871078*G0_0_0 - 0.000652398655871091*G0_0_1 + 0.00884000178705038*G0_0_2 - 0.000652398655871143*G0_1_0 - 0.000652398655871126*G0_1_1 + 0.00884000178705022*G0_1_2 + 0.00884000178705014*G0_2_0 + 0.00884000178705014*G0_2_1 - 0.000652398655871094*G0_2_2;
    A[2763] = -0.00329461321214804*G0_0_2 - 0.00329461321214798*G0_1_2 - 0.00329461321214795*G0_2_0 - 0.00329461321214794*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[2764] = 0.176636936077038*G0_0_0 + 0.134361503176606*G0_0_1 + 0.176636936077038*G0_0_2 + 0.134361503176605*G0_1_0 + 0.0920860702761731*G0_1_1 + 0.134361503176605*G0_1_2 + 0.1273155976932*G0_2_0 + 0.128000616281864*G0_2_1 + 0.1273155976932*G0_2_2;
    A[2765] = -0.334680510461757*G0_0_0 - 0.250814663249556*G0_0_1 - 0.334680510461757*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700805*G0_1_1 - 0.250814663249556*G0_1_2 - 0.166948816037356*G0_2_0 - 0.160783648739376*G0_2_1 - 0.166948816037356*G0_2_2;
    A[2766] = 0.104709984267274*G0_0_0 + 0.188575831479475*G0_0_1 + 0.104709984267274*G0_0_2 + 0.188575831479475*G0_1_0 - 0.0622388317700817*G0_1_1 + 0.188575831479475*G0_1_2 + 0.182410664181495*G0_2_0 + 0.0985448169692946*G0_2_1 + 0.182410664181495*G0_2_2;
    A[2767] = -0.0422754329004321*G0_0_1 - 0.0422754329004321*G0_1_0 + 0.0920860702761738*G0_1_1 - 0.0422754329004323*G0_1_2 - 0.0429604514890964*G0_2_0 - 0.0359145460056913*G0_2_1 - 0.0429604514890964*G0_2_2;
    A[2768] = 0.0439716694056965*G0_0_0 + 0.086247102306129*G0_0_1 + 0.086247102306129*G0_0_2 + 0.086247102306129*G0_1_0 + 0.128522535206562*G0_1_1 + 0.128522535206561*G0_1_2 + 0.0446566879943607*G0_2_0 + 0.0439716694056965*G0_2_1 + 0.0439716694056965*G0_2_2;
    A[2769] = -0.00678494602105682*G0_0_0 - 0.0371541034518462*G0_0_1 - 0.0371541034518461*G0_0_2 - 0.0371541034518461*G0_1_0 + 0.0371867233846398*G0_1_1 + 0.0371867233846398*G0_1_2 - 0.00975335990526923*G0_2_0 - 0.0436454700777614*G0_2_1 - 0.0436454700777613*G0_2_2;
    A[2770] = 0.0127870136550691*G0_0_0 - 0.0250847283182349*G0_0_1 - 0.0250847283182349*G0_0_2 - 0.025084728318235*G0_1_0 - 0.00619778723077343*G0_1_1 - 0.00619778723077334*G0_1_2 + 0.00329461321214793*G0_2_0 - 0.0155923278753137*G0_2_1 - 0.0155923278753137*G0_2_2;
    A[2771] = -0.0318696743392922*G0_0_0 + 0.00583896797004436*G0_0_1 + 0.00583896797004432*G0_0_2 + 0.00583896797004445*G0_1_0 - 0.00590420783563147*G0_2_0 + 0.00254435475789624*G0_2_1 + 0.0025443547578962*G0_2_2;
    A[2772] = 0.0391439193522525*G0_0_0 - 0.0055453885749023*G0_0_1 + 0.0264221455627705*G0_0_2 - 0.00554538857490312*G0_1_0 - 0.00619778723077309*G0_1_2 + 0.0264221455627705*G0_2_0 - 0.00619778723077281*G0_2_1 + 0.0137003717732888*G0_2_2;
    A[2773] = 0.0929668084615985*G0_0_0 + 0.0606730749959904*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0606730749959903*G0_1_0 + 0.0195719596761248*G0_1_1 + 0.0748627457611813*G0_1_2 + 0.121346149991982*G0_2_0 + 0.0748627457611818*G0_2_1 + 0.149725491522364*G0_2_2;
    A[2774] = 0.0678494602105704*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105705*G0_1_0 + 0.039143919352252*G0_1_1 + 0.135698920421141*G0_1_2 + 0.104709984267275*G0_2_0 + 0.135698920421142*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2775] = 0.019571959676126*G0_0_0 - 0.0102752788299663*G0_0_1 + 0.00146789697570936*G0_0_2 - 0.0102752788299657*G0_1_0 - 0.0181040627004171*G0_1_2 + 0.00146789697570963*G0_2_0 - 0.0181040627004165*G0_2_1 - 0.016636165724707*G0_2_2;
    A[2776] = 0.0141896707651918*G0_0_1 + 0.0283793415303832*G0_0_2 + 0.014189670765192*G0_1_0 - 0.004892989919031*G0_1_1 + 0.0283793415303836*G0_1_2 + 0.0283793415303838*G0_2_0 + 0.0283793415303837*G0_2_1 + 0.0567586830607672*G0_2_2;
    A[2777] = -0.00065239865587133*G0_0_1 - 0.0127217737894822*G0_0_2 - 0.000652398655871089*G0_1_0 - 0.00130479731174132*G0_1_2 - 0.0127217737894819*G0_2_0 - 0.00130479731174175*G0_2_1 - 0.0254435475789641*G0_2_2;
    A[2778] = -0.156575677409009*G0_0_0 - 0.22442513761958*G0_0_1 - 0.134067923781464*G0_0_2 - 0.22442513761958*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0972073997247592*G0_1_2 - 0.134067923781464*G0_2_0 - 0.0972073997247597*G0_2_1 - 0.0880738185425672*G0_2_2;
    A[2779] = 0.0929668084615991*G0_0_0 + 0.22801333022687*G0_0_1 + 0.363059851992141*G0_0_2 + 0.228013330226869*G0_1_0 + 0.45602666045374*G0_1_1 + 0.22801333022687*G0_1_2 + 0.363059851992139*G0_2_0 + 0.228013330226869*G0_2_1 + 0.0929668084615995*G0_2_2;
    A[2780] = -0.0678494602105704*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105706*G0_1_0 - 0.292274597830151*G0_1_1 - 0.195067198105391*G0_1_2 - 0.104709984267275*G0_2_0 - 0.19506719810539*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2781] = 0.019571959676126*G0_0_0 + 0.00538228891093467*G0_0_1 - 0.0552907860850571*G0_0_2 + 0.00538228891093422*G0_1_0 - 0.00880738185425716*G0_1_1 - 0.0411011153198656*G0_1_2 - 0.0552907860850565*G0_2_0 - 0.0411011153198647*G0_2_1 + 0.0195719596761257*G0_2_2;
    A[2782] = -0.0141896707651918*G0_0_1 - 0.0283793415303837*G0_0_2 - 0.014189670765192*G0_1_0 - 0.00880738185425696*G0_1_1 + 0.032293733465608*G0_1_2 - 0.0283793415303838*G0_2_0 + 0.0322937334656075*G0_2_1 + 0.0929668084615992*G0_2_2;
    A[2783] = 0.000652398655871116*G0_0_1 + 0.0127217737894827*G0_0_2 + 0.000652398655871187*G0_1_0 + 0.00130479731174228*G0_1_1 + 0.000652398655871582*G0_1_2 + 0.0127217737894819*G0_2_0 + 0.000652398655871224*G0_2_1 - 0.0880738185425676*G0_2_2;
    A[2784] = -0.0965550010688888*G0_0_0 - 0.0287055408583183*G0_0_1 - 0.0596944770121845*G0_0_2 - 0.0287055408583179*G0_1_0 + 0.0391439193522523*G0_1_1 - 0.0965550010688887*G0_1_2 - 0.0596944770121843*G0_2_0 - 0.0965550010688891*G0_2_1 - 0.0228339529554803*G0_2_2;
    A[2785] = -0.00880738185425663*G0_0_0 - 0.041101115319865*G0_0_1 + 0.0053822889109346*G0_0_2 - 0.041101115319865*G0_1_0 + 0.0195719596761258*G0_1_1 - 0.0552907860850564*G0_1_2 + 0.00538228891093456*G0_2_0 - 0.0552907860850565*G0_2_1 + 0.0195719596761258*G0_2_2;
    A[2786] = 0.050234696502057*G0_0_0 + 0.00554538857490235*G0_0_1 + 0.0381653213684458*G0_0_2 + 0.00554538857490207*G0_1_0 + 0.0061977872307733*G0_1_2 + 0.0381653213684459*G0_2_0 + 0.00619778723077279*G0_2_1 + 0.0260959462348348*G0_2_2;
    A[2787] = -0.0332723314494155*G0_0_0 - 0.0190826606842239*G0_0_1 - 0.0190826606842237*G0_0_2 - 0.0190826606842238*G0_1_0 - 0.00489298991903199*G0_1_1 - 0.0332723314494151*G0_1_2 - 0.0190826606842236*G0_2_0 - 0.0332723314494153*G0_2_1 - 0.00489298991903181*G0_2_2;
    A[2788] = 0.040122517336059*G0_0_0 + 0.0102752788299662*G0_0_1 + 0.0298472385060928*G0_0_2 + 0.0102752788299666*G0_1_0 + 0.0181040627004166*G0_1_2 + 0.0298472385060925*G0_2_0 + 0.0181040627004164*G0_2_1 + 0.0195719596761264*G0_2_2;
    A[2789] = 0.00130479731174225*G0_0_0 + 0.000652398655871432*G0_0_1 - 0.0107645778218689*G0_0_2 + 0.000652398655871166*G0_1_0 + 0.00130479731174232*G0_1_2 - 0.0107645778218689*G0_2_0 + 0.00130479731174185*G0_2_1 - 0.0228339529554805*G0_2_2;
    A[2790] = -0.268135847562927*G0_0_0 - 0.321632537344339*G0_0_1 - 0.290643601190473*G0_0_2 - 0.321632537344339*G0_1_0 - 0.584549195660301*G0_1_1 - 0.44885027523916*G0_1_2 - 0.222141742324031*G0_2_0 - 0.194414799449519*G0_2_1 - 0.268135847562928*G0_2_2;
    A[2791] = -0.037186723384641*G0_0_0 + 0.186912214907005*G0_0_1 + 0.140428810676205*G0_0_2 + 0.186912214907004*G0_1_0 - 0.0371867233846399*G0_1_1 + 0.0376760223765419*G0_1_2 + 0.0376760223765422*G0_2_0 + 0.140428810676205*G0_2_1 + 0.0655660649150228*G0_2_2;
    A[2792] = 0.172233245149911*G0_0_0 + 0.00619778723077296*G0_0_1 - 0.0264221455627706*G0_0_2 + 0.00619778723077296*G0_1_0 + 0.0123955744615467*G0_1_1 + 0.00619778723077291*G0_1_2 + 0.0714376528178603*G0_2_0 + 0.00619778723077314*G0_2_1 - 0.0137003717732889*G0_2_2;
    A[2793] = 0.726119703984281*G0_0_0 + 0.456026660453738*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453738*G0_1_0 + 0.912053320907479*G0_1_1 + 0.456026660453738*G0_1_2 + 0.456026660453739*G0_2_0 + 0.456026660453738*G0_2_1 + 0.726119703984279*G0_2_2;
    A[2794] = -0.448197876583289*G0_0_0 - 0.224098938291644*G0_0_1 - 0.121346149991981*G0_0_2 - 0.224098938291643*G0_1_0 - 0.0371867233846391*G0_1_1 - 0.0748627457611809*G0_1_2 - 0.326851726591306*G0_2_0 - 0.177615534060845*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2795] = -0.209419968534549*G0_0_0 - 0.262916658315961*G0_0_1 - 0.104709984267275*G0_0_2 - 0.262916658315961*G0_1_0 - 0.584549195660301*G0_1_1 - 0.13569892042114*G0_1_2 - 0.290643601190473*G0_2_0 - 0.39013439621078*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2796] = 0.352295274170271*G0_0_0 + 0.249542485870609*G0_0_1 + 0.249542485870608*G0_0_2 + 0.249542485870606*G0_1_0 - 0.039143919352254*G0_1_1 + 0.445262082631869*G0_1_2 + 0.249542485870607*G0_2_0 + 0.44526208263187*G0_2_1 + 0.146789697570945*G0_2_2;
    A[2797] = -0.234863516113514*G0_0_0 - 0.0195719596761256*G0_0_1 - 0.141896707651915*G0_0_2 - 0.0195719596761238*G0_1_0 - 0.0391439193522508*G0_1_1 - 0.019571959676125*G0_1_2 - 0.141896707651914*G0_2_0 - 0.0195719596761254*G0_2_1 - 0.0489298991903158*G0_2_2;
    A[2798] = -0.185933616923197*G0_0_0 - 0.288686405222861*G0_0_1 - 0.484406001984122*G0_0_2 - 0.288686405222859*G0_1_0 - 0.03914391935225*G0_1_1 - 0.484406001984122*G0_1_2 - 0.484406001984121*G0_2_0 - 0.484406001984122*G0_2_1 - 0.782878387045046*G0_2_2;
    A[2799] = -0.0391439193522521*G0_0_0 + 0.0048929899190319*G0_0_1 + 0.0538228891093473*G0_0_2 + 0.00489298991903139*G0_1_0 + 0.00978597983806272*G0_1_1 + 0.00489298991903184*G0_1_2 + 0.0538228891093468*G0_2_0 + 0.00489298991903134*G0_2_1 + 0.146789697570947*G0_2_2;
    A[2800] = 0.0117431758056755*G0_0_0 + 0.0117431758056755*G0_0_1 + 0.0117431758056755*G0_0_2 + 0.0117431758056757*G0_1_0 + 0.0117431758056757*G0_1_1 + 0.0117431758056757*G0_1_2 + 0.0094649996993745*G0_2_0 + 0.00946499969937451*G0_2_1 + 0.00946499969937449*G0_2_2;
    A[2801] = -0.0298472385060925*G0_0_0 + 0.00472467106581678*G0_2_0;
    A[2802] = -0.0298472385060924*G0_1_1 + 0.00472467106581677*G0_2_1;
    A[2803] = -0.00946499969937472*G0_2_2;
    A[2804] = 0.0642612676032806*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[2805] = 0.00880738185425679*G0_1_1 + 0.00649136662591528*G0_1_2 + 0.00649136662591531*G0_2_1 + 0.00352295274170301*G0_2_2;
    A[2806] = -0.000652398655870778*G0_1_1 - 0.00949240044292123*G0_1_2 - 0.00949240044292116*G0_2_1 - 0.0189848008858426*G0_2_2;
    A[2807] = 0.00329461321214774*G0_1_2 + 0.00329461321214766*G0_2_1 + 0.0292600797158086*G0_2_2;
    A[2808] = 0.0642612676032809*G0_0_0 + 0.0422754329004323*G0_0_2 + 0.0422754329004328*G0_2_0 + 0.042960451489097*G0_2_2;
    A[2809] = 0.00880738185425684*G0_0_0 + 0.00649136662591529*G0_0_2 + 0.00649136662591491*G0_2_0 + 0.00352295274170288*G0_2_2;
    A[2810] = -0.000652398655870884*G0_0_0 - 0.00949240044292121*G0_0_2 - 0.0094924004429206*G0_2_0 - 0.0189848008858423*G0_2_2;
    A[2811] = 0.00329461321214793*G0_0_2 + 0.00329461321214764*G0_2_0 + 0.0292600797158087*G0_2_2;
    A[2812] = 0.176636936077039*G0_0_0 + 0.0422754329004324*G0_0_1 + 0.0422754329004329*G0_1_0 + 0.0493213383838382*G0_2_0 + 0.042960451489097*G0_2_1;
    A[2813] = -0.334680510461757*G0_0_0 - 0.0838658472122006*G0_0_1 - 0.0838658472122006*G0_1_0 + 0.104709984267275*G0_1_1 - 0.167731694424402*G0_2_0 - 0.077700679914221*G0_2_1;
    A[2814] = 0.104709984267275*G0_0_0 - 0.0838658472122005*G0_0_1 - 0.0838658472122003*G0_1_0 - 0.334680510461757*G0_1_1 - 0.0777006799142199*G0_2_0 - 0.1677316944244*G0_2_1;
    A[2815] = 0.0422754329004329*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.176636936077038*G0_1_1 + 0.0429604514890967*G0_2_0 + 0.0493213383838379*G0_2_1;
    A[2816] = -0.00913358118219214*G0_0_0 - 0.00913358118219217*G0_0_1 - 0.00329461321214775*G0_0_2 - 0.00913358118219208*G0_1_0 - 0.00913358118219212*G0_1_1 - 0.00329461321214801*G0_1_2 - 0.00329461321214744*G0_2_0 - 0.00329461321214753*G0_2_1;
    A[2817] = -0.0156575677409011*G0_0_0 - 0.0156575677409011*G0_0_1 - 0.00257697469069011*G0_0_2 - 0.0156575677409005*G0_1_0 - 0.0156575677409005*G0_1_1 - 0.00257697469068978*G0_1_2 - 0.00257697469068991*G0_2_0 - 0.00257697469068987*G0_2_1;
    A[2818] = -0.00065239865587083*G0_0_0 - 0.000652398655870823*G0_0_1 - 0.00257697469068979*G0_0_2 - 0.00065239865587135*G0_1_0 - 0.000652398655871327*G0_1_1 - 0.00257697469069014*G0_1_2 - 0.00257697469069004*G0_2_0 - 0.00257697469069004*G0_2_1;
    A[2819] = -0.00329461321214795*G0_0_2 - 0.00329461321214777*G0_1_2 - 0.00329461321214768*G0_2_0 - 0.00329461321214767*G0_2_1;
    A[2820] = -0.0435476102793802*G0_0_0 - 0.0377086423093358*G0_0_1 - 0.0435476102793802*G0_0_2 - 0.0377086423093362*G0_1_0 - 0.0318696743392919*G0_1_1 - 0.0377086423093362*G0_1_2 - 0.0292600797158081*G0_2_0 - 0.0259654665036605*G0_2_1 - 0.0292600797158081*G0_2_2;
    A[2821] = 0.0567586830607656*G0_0_0 + 0.0378717419733034*G0_0_1 + 0.0567586830607655*G0_0_2 + 0.0378717419733039*G0_1_0 + 0.0127870136550687*G0_1_1 + 0.0378717419733039*G0_1_2 + 0.0189848008858416*G0_2_0 + 0.00949240044292108*G0_2_1 + 0.0189848008858418*G0_2_2;
    A[2822] = 0.104709984267276*G0_0_0 + 0.0303691574307898*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0303691574307892*G0_1_0 - 0.0067849460210567*G0_1_1 + 0.0303691574307892*G0_1_2 - 0.00352295274170217*G0_2_0 + 0.00296841388421238*G0_2_1 - 0.00352295274170222*G0_2_2;
    A[2823] = -0.042275432900433*G0_0_1 - 0.0422754329004325*G0_1_0 + 0.0439716694056965*G0_1_1 - 0.0422754329004325*G0_1_2 - 0.0429604514890967*G0_2_0 - 0.000685018588664247*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[2824] = -0.0318696743392916*G0_0_0 - 0.0377086423093361*G0_0_1 - 0.037708642309336*G0_0_2 - 0.0377086423093364*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.0435476102793806*G0_1_2 - 0.0259654665036603*G0_2_0 - 0.0292600797158081*G0_2_1 - 0.029260079715808*G0_2_2;
    A[2825] = 0.0127870136550686*G0_0_0 + 0.037871741973304*G0_0_1 + 0.0378717419733039*G0_0_2 + 0.0378717419733041*G0_1_0 + 0.0567586830607655*G0_1_1 + 0.0567586830607656*G0_1_2 + 0.00949240044292088*G0_2_0 + 0.0189848008858421*G0_2_1 + 0.0189848008858421*G0_2_2;
    A[2826] = -0.00678494602105695*G0_0_0 + 0.0303691574307889*G0_0_1 + 0.030369157430789*G0_0_2 + 0.030369157430789*G0_1_0 + 0.104709984267275*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0029684138842125*G0_2_0 - 0.00352295274170258*G0_2_1 - 0.00352295274170258*G0_2_2;
    A[2827] = 0.043971669405697*G0_0_0 - 0.0422754329004324*G0_0_1 - 0.0422754329004323*G0_0_2 - 0.0422754329004324*G0_1_0 - 0.000685018588664342*G0_2_0 - 0.042960451489097*G0_2_1 - 0.042960451489097*G0_2_2;
    A[2828] = -0.15657567740901*G0_0_0 + 0.0678494602105704*G0_0_1 - 0.0225077536275455*G0_0_2 + 0.0678494602105706*G0_1_0 + 0.104709984267275*G0_1_2 - 0.0225077536275451*G0_2_0 + 0.104709984267275*G0_2_1 + 0.0234863516113513*G0_2_2;
    A[2829] = 0.0929668084615995*G0_0_0 - 0.13504652176527*G0_0_1 - 0.270093043530541*G0_0_2 - 0.13504652176527*G0_1_0 + 0.0929668084615999*G0_1_1 - 0.27009304353054*G0_1_2 - 0.27009304353054*G0_2_0 - 0.270093043530541*G0_2_1 - 0.540186087061081*G0_2_2;
    A[2830] = 0.0678494602105709*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0678494602105705*G0_1_0 - 0.156575677409009*G0_1_1 - 0.0225077536275451*G0_1_2 + 0.104709984267275*G0_2_0 - 0.0225077536275442*G0_2_1 + 0.0234863516113518*G0_2_2;
    A[2831] = 0.0195719596761261*G0_0_0 + 0.0141896707651917*G0_0_1 + 0.0748627457611826*G0_0_2 + 0.0141896707651917*G0_1_0 + 0.0283793415303826*G0_1_2 + 0.0748627457611828*G0_2_0 + 0.0283793415303837*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2832] = 0.0141896707651913*G0_0_1 + 0.0283793415303828*G0_0_2 + 0.0141896707651913*G0_1_0 + 0.0195719596761259*G0_1_1 + 0.0748627457611825*G0_1_2 + 0.0283793415303828*G0_2_0 + 0.0748627457611816*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2833] = -0.000652398655870878*G0_0_1 - 0.0127217737894819*G0_0_2 - 0.0006523986558709*G0_1_0 - 0.0127217737894814*G0_1_2 - 0.0127217737894824*G0_2_0 - 0.0127217737894817*G0_2_1 - 0.113517366121532*G0_2_2;
    A[2834] = 0.0391439193522528*G0_0_0 + 0.0446893079271549*G0_0_1 + 0.0127217737894817*G0_0_2 + 0.0446893079271548*G0_1_0 + 0.0502346965020568*G0_1_1 + 0.0120693751336113*G0_1_2 + 0.0127217737894819*G0_2_0 + 0.0120693751336103*G0_2_1;
    A[2835] = 0.0929668084615993*G0_0_0 + 0.0322937334656083*G0_0_1 - 0.0283793415303835*G0_0_2 + 0.0322937334656078*G0_1_0 - 0.00880738185425665*G0_1_1 - 0.0141896707651913*G0_1_2 - 0.0283793415303824*G0_2_0 - 0.0141896707651905*G0_2_1;
    A[2836] = -0.0678494602105711*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105705*G0_1_0 - 0.0965550010688887*G0_1_1 - 0.0368605240567042*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0368605240567045*G0_2_1;
    A[2837] = 0.0195719596761259*G0_0_0 + 0.0298472385060923*G0_0_1 + 0.0181040627004168*G0_0_2 + 0.0298472385060924*G0_1_0 + 0.0401225173360584*G0_1_1 + 0.0102752788299657*G0_1_2 + 0.0181040627004169*G0_2_0 + 0.0102752788299662*G0_2_1;
    A[2838] = -0.0141896707651912*G0_0_1 - 0.0283793415303823*G0_0_2 - 0.0141896707651913*G0_1_0 - 0.0332723314494146*G0_1_1 - 0.0141896707651914*G0_1_2 - 0.0283793415303827*G0_2_0 - 0.0141896707651917*G0_2_1;
    A[2839] = 0.000652398655870912*G0_0_1 + 0.0127217737894818*G0_0_2 + 0.000652398655870846*G0_1_0 + 0.00130479731174212*G0_1_1 + 0.0120693751336115*G0_1_2 + 0.0127217737894824*G0_2_0 + 0.0120693751336113*G0_2_1;
    A[2840] = 0.0502346965020569*G0_0_0 + 0.0446893079271547*G0_0_1 + 0.0120693751336108*G0_0_2 + 0.0446893079271543*G0_1_0 + 0.0391439193522526*G0_1_1 + 0.0127217737894823*G0_1_2 + 0.0120693751336104*G0_2_0 + 0.012721773789482*G0_2_1;
    A[2841] = -0.00880738185425664*G0_0_0 + 0.0322937334656087*G0_0_1 - 0.014189670765191*G0_0_2 + 0.0322937334656084*G0_1_0 + 0.0929668084615989*G0_1_1 - 0.0283793415303829*G0_1_2 - 0.0141896707651912*G0_2_0 - 0.0283793415303826*G0_2_1;
    A[2842] = -0.0965550010688892*G0_0_0 - 0.0678494602105707*G0_0_1 - 0.0368605240567043*G0_0_2 - 0.067849460210571*G0_1_0 - 0.104709984267275*G0_1_2 - 0.0368605240567045*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2843] = 0.0401225173360593*G0_0_0 + 0.0298472385060928*G0_0_1 + 0.0102752788299664*G0_0_2 + 0.0298472385060923*G0_1_0 + 0.0195719596761263*G0_1_1 + 0.0181040627004161*G0_1_2 + 0.0102752788299664*G0_2_0 + 0.0181040627004169*G0_2_1;
    A[2844] = -0.0332723314494149*G0_0_0 - 0.0141896707651915*G0_0_1 - 0.0141896707651919*G0_0_2 - 0.0141896707651912*G0_1_0 - 0.0283793415303831*G0_1_2 - 0.0141896707651913*G0_2_0 - 0.0283793415303833*G0_2_1;
    A[2845] = 0.00130479731174161*G0_0_0 + 0.000652398655870829*G0_0_1 + 0.0120693751336111*G0_0_2 + 0.000652398655870934*G0_1_0 + 0.0127217737894825*G0_1_2 + 0.0120693751336106*G0_2_0 + 0.0127217737894817*G0_2_1;
    A[2846] = 0.172233245149909*G0_0_0 + 0.166035457919136*G0_0_1 + 0.19865539071268*G0_0_2 + 0.166035457919136*G0_1_0 + 0.172233245149909*G0_1_1 + 0.19865539071268*G0_1_2 + 0.100795592332049*G0_2_0 + 0.100795592332048*G0_2_1 + 0.113517366121531*G0_2_2;
    A[2847] = -0.0371867233846381*G0_0_0 - 0.224098938291644*G0_0_1 - 0.177615534060844*G0_0_2 - 0.224098938291644*G0_1_0 - 0.448197876583288*G0_1_1 - 0.326851726591307*G0_1_2 - 0.0748627457611816*G0_2_0 - 0.121346149991982*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2848] = -0.268135847562929*G0_0_0 + 0.0534966897814115*G0_0_1 + 0.022507753627545*G0_0_2 + 0.0534966897814111*G0_1_0 - 0.20941996853455*G0_1_1 - 0.104709984267276*G0_1_2 - 0.045994105238897*G0_2_0 + 0.0812236326559235*G0_2_1 - 0.0234863516113517*G0_2_2;
    A[2849] = -0.448197876583289*G0_0_0 - 0.224098938291643*G0_0_1 - 0.326851726591306*G0_0_2 - 0.224098938291644*G0_1_0 - 0.0371867233846391*G0_1_1 - 0.177615534060845*G0_1_2 - 0.121346149991981*G0_2_0 - 0.0748627457611809*G0_2_1 - 0.149725491522364*G0_2_2;
    A[2850] = 0.72611970398428*G0_0_0 + 0.270093043530541*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530541*G0_1_0 + 0.72611970398428*G0_1_1 + 0.27009304353054*G0_1_2 + 0.270093043530541*G0_2_0 + 0.27009304353054*G0_2_1 + 0.540186087061081*G0_2_2;
    A[2851] = -0.209419968534551*G0_0_0 + 0.0534966897814104*G0_0_1 - 0.104709984267276*G0_0_2 + 0.0534966897814112*G0_1_0 - 0.268135847562928*G0_1_1 + 0.0225077536275451*G0_1_2 + 0.081223632655922*G0_2_0 - 0.0459941052388972*G0_2_1 - 0.023486351611352*G0_2_2;
    A[2852] = -0.234863516113515*G0_0_0 - 0.215291556437388*G0_0_1 - 0.0929668084615989*G0_0_2 - 0.215291556437388*G0_1_0 - 0.234863516113515*G0_1_1 - 0.0929668084615983*G0_1_2 - 0.092966808461599*G0_2_0 - 0.0929668084615995*G0_2_1;
    A[2853] = 0.352295274170272*G0_0_0 + 0.102752788299663*G0_0_1 + 0.102752788299663*G0_0_2 + 0.102752788299663*G0_1_0 - 0.185933616923199*G0_1_1 + 0.298472385060925*G0_1_2 + 0.102752788299662*G0_2_0 + 0.298472385060924*G0_2_1;
    A[2854] = -0.185933616923199*G0_0_0 + 0.102752788299662*G0_0_1 + 0.298472385060923*G0_0_2 + 0.102752788299662*G0_1_0 + 0.352295274170271*G0_1_1 + 0.102752788299662*G0_1_2 + 0.298472385060922*G0_2_0 + 0.102752788299661*G0_2_1;
    A[2855] = -0.0391439193522522*G0_0_0 - 0.0440369092712841*G0_0_1 - 0.0929668084615994*G0_0_2 - 0.0440369092712841*G0_1_0 - 0.0391439193522522*G0_1_1 - 0.0929668084616001*G0_1_2 - 0.0929668084615997*G0_2_0 - 0.0929668084615987*G0_2_1;
    A[2856] = 0.0189195610202553*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0189195610202551*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202551*G0_1_2 + 0.013742125287264*G0_2_0 + 0.013742125287264*G0_2_1 + 0.013742125287264*G0_2_2;
    A[2857] = 0.0189195610202554*G0_0_0 + 0.00517743573299133*G0_2_0;
    A[2858] = 0.03359853077735*G0_1_1 - 0.0143945239431348*G0_2_1;
    A[2859] = -0.0137421252872642*G0_2_2;
    A[2860] = -0.128522535206561*G0_1_1 - 0.09629404160654*G0_1_2 - 0.0962940416065403*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2861] = 0.0476251018785731*G0_1_1 + 0.0623693115012548*G0_1_2 + 0.062369311501255*G0_2_1 + 0.12473862300251*G0_2_2;
    A[2862] = -0.0150051690850297*G0_1_1 - 0.0377738821749231*G0_1_2 - 0.0377738821749233*G0_2_1 - 0.127087258163646*G0_2_2;
    A[2863] = 0.00939454064454066*G0_1_2 + 0.00939454064454068*G0_2_1 + 0.0641307878721073*G0_2_2;
    A[2864] = -0.0306627368259311*G0_0_0 - 0.0212681961813903*G0_0_2 - 0.0212681961813906*G0_2_0 - 0.0118736555368499*G0_2_2;
    A[2865] = -0.00130479731174185*G0_0_0 - 0.00123955744615474*G0_0_2 - 0.00123955744615431*G0_2_0 - 0.00117431758056746*G0_2_2;
    A[2866] = -0.0150051690850299*G0_0_0 - 0.014939929219443*G0_0_2 - 0.0149399292194432*G0_2_0 - 0.0148746893538562*G0_2_2;
    A[2867] = 0.00939454064454073*G0_0_2 + 0.00939454064454073*G0_2_0 + 0.0187890812890815*G0_2_2;
    A[2868] = -0.111886369481855*G0_0_0 - 0.0212681961813903*G0_0_1 - 0.0212681961813912*G0_1_0 - 0.0477555816097481*G0_2_0 - 0.0118736555368499*G0_2_1;
    A[2869] = 0.20941996853455*G0_0_0 + 0.0359471659384848*G0_0_1 + 0.0359471659384858*G0_1_0 - 0.0528442911255403*G0_1_1 + 0.0823327103709046*G0_2_0 - 0.00182671623643833*G0_2_1;
    A[2870] = -0.271397840842282*G0_0_0 - 0.0733296089198856*G0_0_1 - 0.0733296089198865*G0_1_0 + 0.121346149991982*G0_1_1 - 0.146659217839772*G0_2_0 - 0.0109602974186309*G0_2_1;
    A[2871] = -0.0962940416065409*G0_0_1 - 0.0962940416065401*G0_1_0 - 0.257045070413123*G0_1_1 - 0.12473862300251*G0_2_0 - 0.192588083213081*G0_2_1;
    A[2872] = -0.0306627368259306*G0_0_0 - 0.0306627368259307*G0_0_1 - 0.00939454064454049*G0_0_2 - 0.0306627368259307*G0_1_0 - 0.0306627368259308*G0_1_1 - 0.00939454064454034*G0_1_2 - 0.00939454064454031*G0_2_0 - 0.00939454064454039*G0_2_1;
    A[2873] = -0.00130479731174183*G0_0_0 - 0.00130479731174181*G0_0_1 - 6.5239865587193e-05*G0_0_2 - 0.00130479731174223*G0_1_0 - 0.00130479731174211*G0_1_1 - 6.52398655872991e-05*G0_1_2 - 6.52398655873965e-05*G0_2_0 - 6.52398655872768e-05*G0_2_1;
    A[2874] = -0.0150051690850301*G0_0_0 - 0.0150051690850301*G0_0_1 - 6.52398655871212e-05*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0150051690850299*G0_1_1 - 6.52398655869993e-05*G0_1_2 - 6.52398655868403e-05*G0_2_0 - 6.5239865586879e-05*G0_2_1;
    A[2875] = -0.00939454064454046*G0_0_2 - 0.0093945406445405*G0_1_2 - 0.00939454064454076*G0_2_0 - 0.00939454064454077*G0_2_1;
    A[2876] = -0.111886369481854*G0_0_0 - 0.090618173300464*G0_0_1 - 0.111886369481854*G0_0_2 - 0.0906181733004634*G0_1_0 - 0.0693499771190732*G0_1_1 - 0.0906181733004634*G0_1_2 - 0.0641307878721063*G0_2_0 - 0.0547362472275659*G0_2_1 - 0.0641307878721063*G0_2_2;
    A[2877] = 0.20941996853455*G0_0_0 + 0.173472802596065*G0_0_1 + 0.20941996853455*G0_0_2 + 0.173472802596064*G0_1_0 + 0.0846813455320386*G0_1_1 + 0.173472802596064*G0_1_2 + 0.127087258163645*G0_2_0 + 0.089313375988722*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2878] = -0.271397840842283*G0_0_0 - 0.198068231922397*G0_0_1 - 0.271397840842283*G0_0_2 - 0.198068231922396*G0_1_0 - 0.00339247301052827*G0_1_1 - 0.198068231922396*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0623693115012551*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2879] = 0.096294041606541*G0_0_1 + 0.0962940416065401*G0_1_0 - 0.0644569872000422*G0_1_1 + 0.0962940416065401*G0_1_2 + 0.12473862300251*G0_2_0 + 0.02844458139597*G0_2_1 + 0.12473862300251*G0_2_2;
    A[2880] = -0.0187890812890811*G0_0_0 - 0.0400572774704714*G0_0_1 - 0.0400572774704714*G0_0_2 - 0.0400572774704712*G0_1_0 - 0.0613254736518617*G0_1_1 - 0.0613254736518617*G0_1_2 - 0.00939454064454035*G0_2_0 - 0.0187890812890809*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[2881] = -0.000130479731174023*G0_0_0 + 0.0136351319077012*G0_0_1 + 0.0136351319077013*G0_0_2 + 0.0136351319077012*G0_1_0 - 0.0254435475789638*G0_1_1 - 0.0254435475789639*G0_1_2 - 6.52398655871253e-05*G0_2_0 + 0.0148746893538558*G0_2_1 + 0.0148746893538558*G0_2_2;
    A[2882] = -0.000130479731174331*G0_0_0 - 0.0137656116388753*G0_0_1 - 0.0137656116388754*G0_0_2 - 0.0137656116388755*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.0528442911255407*G0_1_2 - 6.523986558719e-05*G0_2_0 + 0.00117431758056757*G0_2_1 + 0.00117431758056757*G0_2_2;
    A[2883] = -0.0187890812890812*G0_0_0 + 0.0212681961813903*G0_0_1 + 0.0212681961813903*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.00939454064454062*G0_2_0 + 0.0118736555368499*G0_2_1 + 0.0118736555368499*G0_2_2;
    A[2884] = 0.05219189246967*G0_0_0 - 0.0378391220405107*G0_0_1 - 0.000652398655870996*G0_0_2 - 0.0378391220405099*G0_1_0 - 0.0528442911255412*G0_1_2 - 0.000652398655870217*G0_2_0 - 0.0528442911255403*G0_2_1 - 0.0534966897814113*G0_2_2;
    A[2885] = -0.185933616923197*G0_0_0 + 0.00913358118219349*G0_0_1 - 0.0812236326559223*G0_0_2 + 0.0091335811821922*G0_1_0 - 0.0880738185425669*G0_1_1 + 0.045994105238897*G0_1_2 - 0.0812236326559232*G0_2_0 + 0.045994105238897*G0_2_1 + 0.0234863516113517*G0_2_2;
    A[2886] = -0.135698920421141*G0_0_1 - 0.271397840842283*G0_0_2 - 0.135698920421141*G0_1_0 + 0.0521918924696699*G0_1_1 - 0.271397840842282*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842282*G0_2_1 - 0.542795681684565*G0_2_2;
    A[2887] = 0.0260959462348347*G0_0_0 - 0.0120693751336114*G0_0_1 + 0.0198981590040619*G0_0_2 - 0.0120693751336122*G0_1_0 - 0.0127217737894817*G0_1_2 + 0.0198981590040607*G0_2_0 - 0.0127217737894826*G0_2_1 + 0.0137003717732885*G0_2_2;
    A[2888] = 0.0368605240567032*G0_0_1 + 0.104709984267274*G0_0_2 + 0.0368605240567037*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.0737210481134077*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0737210481134086*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2889] = -0.0150051690850298*G0_0_1 - 0.0528442911255411*G0_0_2 - 0.0150051690850296*G0_1_0 - 0.0300103381700606*G0_1_2 - 0.0528442911255397*G0_2_0 - 0.0300103381700597*G0_2_1 - 0.105688582251082*G0_2_2;
    A[2890] = 0.0521918924696689*G0_0_0 + 0.0900310145101795*G0_0_1 + 0.05284429112554*G0_0_2 + 0.0900310145101796*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0378391220405096*G0_1_2 + 0.05284429112554*G0_2_0 + 0.0378391220405102*G0_2_1;
    A[2891] = -0.185933616923197*G0_0_0 - 0.19506719810539*G0_0_1 - 0.104709984267274*G0_0_2 - 0.195067198105389*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105702*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105704*G0_2_1;
    A[2892] = 0.135698920421141*G0_0_1 + 0.271397840842283*G0_0_2 + 0.135698920421141*G0_1_0 + 0.323589733311952*G0_1_1 + 0.135698920421141*G0_1_2 + 0.271397840842282*G0_2_0 + 0.135698920421141*G0_2_1;
    A[2893] = 0.0260959462348354*G0_0_0 + 0.0381653213684464*G0_0_1 + 0.00619778723077368*G0_0_2 + 0.0381653213684467*G0_1_0 + 0.0502346965020579*G0_1_1 + 0.00554538857490344*G0_1_2 + 0.00619778723077428*G0_2_0 + 0.00554538857490266*G0_2_1;
    A[2894] = -0.0368605240567045*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0368605240567037*G0_1_0 - 0.096555001068889*G0_1_1 - 0.0678494602105711*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105705*G0_2_1;
    A[2895] = 0.0150051690850301*G0_0_1 + 0.0528442911255405*G0_0_2 + 0.0150051690850296*G0_1_0 + 0.0300103381700595*G0_1_1 + 0.03783912204051*G0_1_2 + 0.0528442911255395*G0_2_0 + 0.0378391220405101*G0_2_1;
    A[2896] = 0.0300103381700595*G0_0_0 - 0.00782878387045071*G0_0_1 + 0.0150051690850297*G0_0_2 - 0.00782878387045089*G0_1_0 - 0.0456679059109614*G0_1_1 + 0.0300103381700594*G0_1_2 + 0.0150051690850296*G0_2_0 + 0.0300103381700597*G0_2_1;
    A[2897] = 0.00130479731174185*G0_0_0 + 0.000652398655870668*G0_0_1 + 0.000652398655870706*G0_0_2 + 0.000652398655870903*G0_1_0 - 0.0880738185425673*G0_1_1 + 0.0127217737894822*G0_1_2 + 0.000652398655871051*G0_2_0 + 0.0127217737894821*G0_2_1;
    A[2898] = 0.0300103381700604*G0_0_0 + 0.0378391220405108*G0_0_1 + 0.0150051690850302*G0_0_2 + 0.0378391220405112*G0_1_0 + 0.0528442911255405*G0_1_2 + 0.0150051690850303*G0_2_0 + 0.0528442911255405*G0_2_1;
    A[2899] = 0.00130479731174217*G0_0_0 - 0.0107645778218689*G0_0_1 + 0.000652398655871264*G0_0_2 - 0.0107645778218687*G0_1_0 - 0.0228339529554803*G0_1_1 + 0.00130479731174248*G0_1_2 + 0.000652398655871107*G0_2_0 + 0.00130479731174257*G0_2_1;
    A[2900] = 0.00130479731174162*G0_0_0 + 0.0120693751336113*G0_0_1 + 0.000652398655871027*G0_0_2 + 0.0120693751336107*G0_1_0 + 0.012721773789482*G0_1_2 + 0.000652398655870701*G0_2_0 + 0.0127217737894824*G0_2_1;
    A[2901] = 0.03001033817006*G0_0_0 + 0.0150051690850297*G0_0_1 + 0.0150051690850297*G0_0_2 + 0.01500516908503*G0_1_0 + 0.0300103381700597*G0_1_2 + 0.0150051690850301*G0_2_0 + 0.0300103381700597*G0_2_1;
    A[2902] = 0.105688582251081*G0_0_0 + 0.12787013655069*G0_0_1 + 0.10503618359521*G0_0_2 + 0.12787013655069*G0_1_0 + 0.255740273101382*G0_1_1 + 0.18006202902036*G0_1_2 + 0.0528442911255399*G0_2_0 + 0.0756782440810205*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2903] = 0.0123955744615463*G0_0_0 + 0.0061977872307728*G0_0_1 + 0.00619778723077285*G0_0_2 + 0.00619778723077339*G0_1_0 + 0.17223324514991*G0_1_1 + 0.0714376528178607*G0_1_2 + 0.00619778723077362*G0_2_0 - 0.0264221455627703*G0_2_1 - 0.0137003717732882*G0_2_2;
    A[2904] = 0.105688582251082*G0_0_0 - 0.0221815542996094*G0_0_1 + 0.000652398655871364*G0_0_2 - 0.0221815542996088*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255414*G0_1_2 + 0.0528442911255409*G0_2_0 + 0.000652398655870835*G0_2_1 + 0.0534966897814114*G0_2_2;
    A[2905] = -0.209419968534549*G0_0_0 - 0.262916658315961*G0_0_1 - 0.290643601190473*G0_0_2 - 0.262916658315961*G0_1_0 - 0.584549195660301*G0_1_1 - 0.39013439621078*G0_1_2 - 0.104709984267275*G0_2_0 - 0.13569892042114*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2906] = -0.209419968534551*G0_0_0 + 0.0534966897814112*G0_0_1 + 0.081223632655922*G0_0_2 + 0.0534966897814104*G0_1_0 - 0.268135847562928*G0_1_1 - 0.0459941052388972*G0_1_2 - 0.104709984267276*G0_2_0 + 0.0225077536275451*G0_2_1 - 0.023486351611352*G0_2_2;
    A[2907] = 0.542795681684566*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623904*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684565*G0_2_2;
    A[2908] = -0.0521918924696679*G0_0_0 - 0.0424059126316052*G0_0_1 - 0.0260959462348336*G0_0_2 - 0.0424059126316043*G0_1_0 + 0.143527704291595*G0_1_1 - 0.0750258454251491*G0_1_2 - 0.0260959462348337*G0_2_0 - 0.0750258454251495*G0_2_1;
    A[2909] = -0.052191892469671*G0_0_0 - 0.00978597983806427*G0_0_1 - 0.0260959462348352*G0_0_2 - 0.00978597983806542*G0_1_0 + 0.176147637085134*G0_1_1 - 0.0587158790283794*G0_1_2 - 0.0260959462348359*G0_2_0 - 0.0587158790283793*G0_2_1;
    A[2910] = 0.371867233846395*G0_0_0 + 0.185933616923197*G0_0_1 + 0.185933616923198*G0_0_2 + 0.185933616923197*G0_1_0 - 0.150051690850303*G0_1_1 + 0.316413348097373*G0_1_2 + 0.185933616923198*G0_2_0 + 0.316413348097372*G0_2_1;
    A[2911] = -0.05219189246967*G0_0_0 - 0.0260959462348341*G0_0_1 - 0.0260959462348355*G0_0_2 - 0.0260959462348345*G0_1_0 + 0.0456679059109616*G0_1_1 - 0.0750258454251509*G0_1_2 - 0.026095946234835*G0_2_0 - 0.0750258454251508*G0_2_1;
    A[2912] = -0.0782878387045041*G0_0_0 - 0.0782878387045041*G0_0_1 - 0.0782878387045041*G0_0_2 - 0.0782878387045042*G0_1_0 - 0.0782878387045042*G0_1_1 - 0.0782878387045043*G0_1_2 - 0.0782878387045043*G0_2_0 - 0.0782878387045043*G0_2_1 - 0.0782878387045042*G0_2_2;
    A[2913] = 0.0440369092712839*G0_0_0;
    A[2914] = 0.0440369092712838*G0_1_1;
    A[2915] = 0.0440369092712839*G0_2_2;
    A[2916] = -0.0505608958299935*G0_1_1 - 0.00260959462348446*G0_1_2 - 0.00260959462348341*G0_2_1;
    A[2917] = 0.0391439193522527*G0_1_1 + 0.0319675341376733*G0_1_2 + 0.0319675341376727*G0_2_1 + 0.0260959462348356*G0_2_2;
    A[2918] = 0.0260959462348351*G0_1_1 + 0.0319675341376729*G0_1_2 + 0.0319675341376733*G0_2_1 + 0.0391439193522524*G0_2_2;
    A[2919] = -0.00260959462348378*G0_1_2 - 0.00260959462348388*G0_2_1 - 0.0505608958299931*G0_2_2;
    A[2920] = -0.050560895829993*G0_0_0 - 0.00260959462348371*G0_0_2 - 0.00260959462348333*G0_2_0;
    A[2921] = 0.0391439193522519*G0_0_0 + 0.0319675341376724*G0_0_2 + 0.0319675341376721*G0_2_0 + 0.0260959462348347*G0_2_2;
    A[2922] = 0.0260959462348356*G0_0_0 + 0.0319675341376734*G0_0_2 + 0.031967534137673*G0_2_0 + 0.0391439193522524*G0_2_2;
    A[2923] = -0.00260959462348401*G0_0_2 - 0.00260959462348397*G0_2_0 - 0.0505608958299932*G0_2_2;
    A[2924] = -0.0505608958299929*G0_0_0 - 0.00260959462348379*G0_0_1 - 0.00260959462348373*G0_1_0;
    A[2925] = 0.0391439193522525*G0_0_0 + 0.0319675341376733*G0_0_1 + 0.0319675341376726*G0_1_0 + 0.0260959462348343*G0_1_1;
    A[2926] = 0.0260959462348347*G0_0_0 + 0.0319675341376724*G0_0_1 + 0.0319675341376731*G0_1_0 + 0.039143919352253*G0_1_1;
    A[2927] = -0.00260959462348416*G0_0_1 - 0.00260959462348453*G0_1_0 - 0.0505608958299936*G0_1_1;
    A[2928] = 0.0962288017409532*G0_0_0 + 0.0962288017409534*G0_0_1 + 0.13145832915798*G0_0_2 + 0.0962288017409532*G0_1_0 + 0.0962288017409532*G0_1_1 + 0.13145832915798*G0_1_2 + 0.131458329157981*G0_2_0 + 0.131458329157981*G0_2_1 + 0.166687856575007*G0_2_2;
    A[2929] = -0.15657567740901*G0_0_0 - 0.15657567740901*G0_0_1 - 0.0662184635708924*G0_0_2 - 0.15657567740901*G0_1_0 - 0.15657567740901*G0_1_1 - 0.0662184635708926*G0_1_2 - 0.0662184635708941*G0_2_0 - 0.0662184635708943*G0_2_1 - 0.161794866655976*G0_2_2;
    A[2930] = 0.0260959462348342*G0_0_0 + 0.0260959462348341*G0_0_1 + 0.0104383784939328*G0_0_2 + 0.0260959462348338*G0_1_0 + 0.0260959462348338*G0_1_1 + 0.010438378493933*G0_1_2 + 0.0104383784939335*G0_2_0 + 0.0104383784939335*G0_2_1 + 0.180714427676231*G0_2_2;
    A[2931] = 0.00260959462348424*G0_0_2 + 0.00260959462348416*G0_1_2 + 0.0026095946234841*G0_2_0 + 0.00260959462348406*G0_2_1 - 0.151356488162042*G0_2_2;
    A[2932] = 0.0962288017409534*G0_0_0 + 0.13145832915798*G0_0_1 + 0.0962288017409535*G0_0_2 + 0.131458329157981*G0_1_0 + 0.166687856575007*G0_1_1 + 0.131458329157981*G0_1_2 + 0.0962288017409537*G0_2_0 + 0.13145832915798*G0_2_1 + 0.0962288017409538*G0_2_2;
    A[2933] = -0.156575677409009*G0_0_0 - 0.0662184635708924*G0_0_1 - 0.156575677409009*G0_0_2 - 0.0662184635708931*G0_1_0 - 0.161794866655975*G0_1_1 - 0.0662184635708932*G0_1_2 - 0.156575677409009*G0_2_0 - 0.0662184635708929*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2934] = 0.0260959462348332*G0_0_0 + 0.0104383784939329*G0_0_1 + 0.0260959462348332*G0_0_2 + 0.0104383784939321*G0_1_0 + 0.180714427676231*G0_1_1 + 0.010438378493932*G0_1_2 + 0.0260959462348339*G0_2_0 + 0.010438378493933*G0_2_1 + 0.0260959462348341*G0_2_2;
    A[2935] = 0.00260959462348405*G0_0_1 + 0.00260959462348461*G0_1_0 - 0.151356488162042*G0_1_1 + 0.00260959462348467*G0_1_2 + 0.00260959462348402*G0_2_1;
    A[2936] = 0.166687856575007*G0_0_0 + 0.131458329157981*G0_0_1 + 0.131458329157981*G0_0_2 + 0.13145832915798*G0_1_0 + 0.0962288017409535*G0_1_1 + 0.0962288017409534*G0_1_2 + 0.13145832915798*G0_2_0 + 0.0962288017409537*G0_2_1 + 0.0962288017409537*G0_2_2;
    A[2937] = -0.161794866655975*G0_0_0 - 0.0662184635708932*G0_0_1 - 0.0662184635708932*G0_0_2 - 0.0662184635708925*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.0662184635708929*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2938] = 0.180714427676231*G0_0_0 + 0.0104383784939329*G0_0_1 + 0.010438378493933*G0_0_2 + 0.0104383784939334*G0_1_0 + 0.0260959462348345*G0_1_1 + 0.0260959462348344*G0_1_2 + 0.0104383784939336*G0_2_0 + 0.0260959462348345*G0_2_1 + 0.0260959462348345*G0_2_2;
    A[2939] = -0.151356488162042*G0_0_0 + 0.0026095946234838*G0_0_1 + 0.0026095946234837*G0_0_2 + 0.0026095946234832*G0_1_0 + 0.00260959462348353*G0_2_0;
    A[2940] = 0.195719596761262*G0_0_0 + 0.0750258454251501*G0_0_1 + 0.0750258454251508*G0_0_2 + 0.0750258454251506*G0_1_0 + 0.0260959462348348*G0_1_2 + 0.0750258454251499*G0_2_0 + 0.0260959462348349*G0_2_1;
    A[2941] = 0.146789697570946*G0_0_0 + 0.141896707651915*G0_0_1 + 0.0929668084615993*G0_0_2 + 0.141896707651915*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0929668084615998*G0_1_2 + 0.0929668084615987*G0_2_0 + 0.0929668084615985*G0_2_1;
    A[2942] = 0.07502584542515*G0_0_1 + 0.0260959462348337*G0_0_2 + 0.0750258454251495*G0_1_0 + 0.195719596761262*G0_1_1 + 0.0750258454251492*G0_1_2 + 0.0260959462348353*G0_2_0 + 0.0750258454251505*G0_2_1;
    A[2943] = 0.146789697570946*G0_0_0 + 0.0929668084615992*G0_0_1 + 0.141896707651914*G0_0_2 + 0.0929668084615985*G0_1_0 + 0.0929668084615988*G0_1_2 + 0.141896707651914*G0_2_0 + 0.0929668084615986*G0_2_1 + 0.146789697570945*G0_2_2;
    A[2944] = 0.0929668084615991*G0_0_1 + 0.092966808461599*G0_0_2 + 0.092966808461599*G0_1_0 + 0.146789697570945*G0_1_1 + 0.141896707651914*G0_1_2 + 0.0929668084615993*G0_2_0 + 0.141896707651914*G0_2_1 + 0.146789697570946*G0_2_2;
    A[2945] = 0.026095946234835*G0_0_1 + 0.075025845425151*G0_0_2 + 0.0260959462348347*G0_1_0 + 0.0750258454251503*G0_1_2 + 0.07502584542515*G0_2_0 + 0.0750258454251498*G0_2_1 + 0.195719596761262*G0_2_2;
    A[2946] = -0.782878387045049*G0_0_0 - 0.466465038947674*G0_0_1 - 0.466465038947675*G0_0_2 - 0.466465038947675*G0_1_0 - 0.150051690850301*G0_1_1 - 0.3359853077735*G0_1_2 - 0.466465038947674*G0_2_0 - 0.3359853077735*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2947] = 0.146789697570946*G0_0_0 + 0.44526208263187*G0_0_1 + 0.24954248587061*G0_0_2 + 0.445262082631871*G0_1_0 - 0.0391439193522535*G0_1_1 + 0.249542485870611*G0_1_2 + 0.24954248587061*G0_2_0 + 0.249542485870609*G0_2_1 + 0.352295274170274*G0_2_2;
    A[2948] = -0.075025845425149*G0_0_1 - 0.0260959462348341*G0_0_2 - 0.0750258454251497*G0_1_0 + 0.143527704291594*G0_1_1 - 0.0424059126316058*G0_1_2 - 0.0260959462348354*G0_2_0 - 0.0424059126316066*G0_2_1 - 0.0521918924696699*G0_2_2;
    A[2949] = 0.146789697570945*G0_0_0 + 0.249542485870611*G0_0_1 + 0.445262082631871*G0_0_2 + 0.249542485870609*G0_1_0 + 0.352295274170273*G0_1_1 + 0.249542485870609*G0_1_2 + 0.44526208263187*G0_2_0 + 0.249542485870611*G0_2_1 - 0.0391439193522533*G0_2_2;
    A[2950] = -0.0929668084616007*G0_0_1 - 0.0929668084616005*G0_0_2 - 0.092966808461599*G0_1_0 - 0.234863516113517*G0_1_1 - 0.21529155643739*G0_1_2 - 0.0929668084615993*G0_2_0 - 0.215291556437389*G0_2_1 - 0.234863516113516*G0_2_2;
    A[2951] = -0.0260959462348341*G0_0_1 - 0.0750258454251494*G0_0_2 - 0.0260959462348344*G0_1_0 - 0.0521918924696684*G0_1_1 - 0.0424059126316054*G0_1_2 - 0.0750258454251496*G0_2_0 - 0.0424059126316064*G0_2_1 + 0.143527704291594*G0_2_2;
    A[2952] = -0.150051690850299*G0_0_0 - 0.466465038947674*G0_0_1 - 0.3359853077735*G0_0_2 - 0.466465038947673*G0_1_0 - 0.782878387045047*G0_1_1 - 0.466465038947674*G0_1_2 - 0.335985307773498*G0_2_0 - 0.466465038947672*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2953] = -0.0391439193522546*G0_0_0 + 0.445262082631871*G0_0_1 + 0.249542485870609*G0_0_2 + 0.445262082631869*G0_1_0 + 0.146789697570946*G0_1_1 + 0.249542485870608*G0_1_2 + 0.249542485870607*G0_2_0 + 0.249542485870608*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2954] = 0.143527704291594*G0_0_0 - 0.0750258454251502*G0_0_1 - 0.0424059126316055*G0_0_2 - 0.0750258454251491*G0_1_0 - 0.0260959462348338*G0_1_2 - 0.0424059126316064*G0_2_0 - 0.0260959462348348*G0_2_1 - 0.0521918924696692*G0_2_2;
    A[2955] = 0.352295274170272*G0_0_0 + 0.249542485870608*G0_0_1 + 0.249542485870608*G0_0_2 + 0.249542485870609*G0_1_0 + 0.146789697570944*G0_1_1 + 0.44526208263187*G0_1_2 + 0.249542485870608*G0_2_0 + 0.445262082631868*G0_2_1 - 0.039143919352254*G0_2_2;
    A[2956] = -0.234863516113515*G0_0_0 - 0.0929668084615994*G0_0_1 - 0.215291556437389*G0_0_2 - 0.0929668084616*G0_1_0 - 0.0929668084615997*G0_1_2 - 0.215291556437387*G0_2_0 - 0.092966808461599*G0_2_1 - 0.234863516113514*G0_2_2;
    A[2957] = -0.0521918924696693*G0_0_0 - 0.0260959462348345*G0_0_1 - 0.0424059126316054*G0_0_2 - 0.0260959462348337*G0_1_0 - 0.0750258454251493*G0_1_2 - 0.0424059126316061*G0_2_0 - 0.0750258454251494*G0_2_1 + 0.143527704291594*G0_2_2;
    A[2958] = -0.150051690850299*G0_0_0 - 0.335985307773498*G0_0_1 - 0.466465038947672*G0_0_2 - 0.335985307773498*G0_1_0 - 0.1500516908503*G0_1_1 - 0.466465038947673*G0_1_2 - 0.466465038947673*G0_2_0 - 0.466465038947673*G0_2_1 - 0.782878387045045*G0_2_2;
    A[2959] = -0.0391439193522547*G0_0_0 + 0.249542485870609*G0_0_1 + 0.44526208263187*G0_0_2 + 0.249542485870607*G0_1_0 + 0.352295274170271*G0_1_1 + 0.249542485870608*G0_1_2 + 0.445262082631869*G0_2_0 + 0.249542485870608*G0_2_1 + 0.146789697570945*G0_2_2;
    A[2960] = 0.143527704291594*G0_0_0 - 0.042405912631606*G0_0_1 - 0.0750258454251506*G0_0_2 - 0.0424059126316054*G0_1_0 - 0.0521918924696687*G0_1_1 - 0.0260959462348347*G0_1_2 - 0.0750258454251496*G0_2_0 - 0.0260959462348343*G0_2_1;
    A[2961] = 0.352295274170271*G0_0_0 + 0.249542485870606*G0_0_1 + 0.249542485870607*G0_0_2 + 0.249542485870609*G0_1_0 - 0.039143919352254*G0_1_1 + 0.44526208263187*G0_1_2 + 0.249542485870608*G0_2_0 + 0.445262082631869*G0_2_1 + 0.146789697570945*G0_2_2;
    A[2962] = -0.234863516113515*G0_0_0 - 0.215291556437388*G0_0_1 - 0.092966808461599*G0_0_2 - 0.215291556437388*G0_1_0 - 0.234863516113515*G0_1_1 - 0.0929668084615995*G0_1_2 - 0.0929668084615989*G0_2_0 - 0.0929668084615983*G0_2_1;
    A[2963] = -0.0521918924696679*G0_0_0 - 0.0424059126316043*G0_0_1 - 0.0260959462348337*G0_0_2 - 0.0424059126316052*G0_1_0 + 0.143527704291595*G0_1_1 - 0.0750258454251495*G0_1_2 - 0.0260959462348336*G0_2_0 - 0.0750258454251491*G0_2_1;
    A[2964] = 1.76147637085136*G0_0_0 + 0.97859798380631*G0_0_1 + 0.978597983806308*G0_0_2 + 0.978597983806311*G0_1_0 + 1.76147637085136*G0_1_1 + 0.978597983806309*G0_1_2 + 0.978597983806308*G0_2_0 + 0.978597983806309*G0_2_1 + 1.76147637085135*G0_2_2;
    A[2965] = -1.17431758056757*G0_0_0 - 0.587158790283785*G0_0_1 - 0.587158790283784*G0_0_2 - 0.587158790283787*G0_1_0 - 0.293579395141892*G0_1_1 - 0.342509294332207*G0_1_2 - 0.587158790283784*G0_2_0 - 0.342509294332207*G0_2_1 - 0.293579395141891*G0_2_2;
    A[2966] = -0.293579395141891*G0_0_0 - 0.587158790283786*G0_0_1 - 0.342509294332206*G0_0_2 - 0.587158790283786*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283783*G0_1_2 - 0.342509294332208*G0_2_0 - 0.587158790283787*G0_2_1 - 0.293579395141891*G0_2_2;
    A[2967] = -0.293579395141892*G0_0_0 - 0.342509294332207*G0_0_1 - 0.587158790283787*G0_0_2 - 0.342509294332208*G0_1_0 - 0.29357939514189*G0_1_1 - 0.587158790283787*G0_1_2 - 0.587158790283784*G0_2_0 - 0.587158790283783*G0_2_1 - 1.17431758056757*G0_2_2;
    A[2968] = 0.0440369092712833*G0_0_0 + 0.0440369092712834*G0_0_1 + 0.0440369092712833*G0_0_2 + 0.0440369092712835*G0_1_0 + 0.0440369092712835*G0_1_1 + 0.0440369092712835*G0_1_2 + 0.0440369092712837*G0_2_0 + 0.0440369092712837*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[2969] = -0.0782878387045048*G0_0_0;
    A[2970] = 0.0440369092712838*G0_1_1;
    A[2971] = 0.0440369092712842*G0_2_2;
    A[2972] = -0.0505608958299925*G0_1_1 - 0.00260959462348371*G0_1_2 - 0.00260959462348339*G0_2_1;
    A[2973] = 0.0391439193522524*G0_1_1 + 0.0319675341376731*G0_1_2 + 0.0319675341376735*G0_2_1 + 0.0260959462348356*G0_2_2;
    A[2974] = 0.0260959462348354*G0_1_1 + 0.0319675341376729*G0_1_2 + 0.0319675341376725*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[2975] = -0.00260959462348359*G0_1_2 - 0.00260959462348272*G0_2_1 - 0.0505608958299921*G0_2_2;
    A[2976] = 0.0962288017409539*G0_0_0 - 0.0352295274170271*G0_0_2 - 0.0352295274170277*G0_2_0;
    A[2977] = -0.156575677409009*G0_0_0 - 0.0903572138381152*G0_0_2 - 0.0903572138381139*G0_2_0 - 0.185933616923198*G0_2_2;
    A[2978] = 0.026095946234834*G0_0_0 + 0.0156575677409003*G0_0_2 + 0.0156575677408996*G0_2_0 + 0.185933616923198*G0_2_2;
    A[2979] = -0.00260959462348272*G0_0_2 - 0.00260959462348267*G0_2_0 - 0.156575677409009*G0_2_2;
    A[2980] = 0.0962288017409541*G0_0_0 - 0.035229527417027*G0_0_1 - 0.0352295274170267*G0_1_0;
    A[2981] = -0.156575677409009*G0_0_0 - 0.090357213838116*G0_0_1 - 0.0903572138381162*G0_1_0 - 0.185933616923199*G0_1_1;
    A[2982] = 0.0260959462348351*G0_0_0 + 0.0156575677409009*G0_0_1 + 0.0156575677409017*G0_1_0 + 0.185933616923199*G0_1_1;
    A[2983] = -0.00260959462348272*G0_0_1 - 0.00260959462348372*G0_1_0 - 0.156575677409009*G0_1_1;
    A[2984] = -0.050560895829992*G0_0_0 - 0.0505608958299921*G0_0_1 - 0.0479513012065079*G0_0_2 - 0.0505608958299921*G0_1_0 - 0.0505608958299923*G0_1_1 - 0.0479513012065085*G0_1_2 - 0.0479513012065084*G0_2_0 - 0.0479513012065085*G0_2_1 - 0.0453417065830254*G0_2_2;
    A[2985] = 0.0391439193522526*G0_0_0 + 0.0391439193522527*G0_0_1 + 0.00717638521457842*G0_0_2 + 0.0391439193522526*G0_1_0 + 0.0391439193522528*G0_1_1 + 0.00717638521457927*G0_1_2 + 0.00717638521457891*G0_2_0 + 0.00717638521457894*G0_2_1 + 0.00130479731174167*G0_2_2;
    A[2986] = 0.0260959462348357*G0_0_0 + 0.0260959462348358*G0_0_1 - 0.00587158790283701*G0_0_2 + 0.0260959462348346*G0_1_0 + 0.0260959462348345*G0_1_1 - 0.00587158790283778*G0_1_2 - 0.0058715879028375*G0_2_0 - 0.00587158790283744*G0_2_1 + 0.00130479731174188*G0_2_2;
    A[2987] = 0.00260959462348294*G0_0_2 + 0.00260959462348357*G0_1_2 + 0.00260959462348263*G0_2_0 + 0.00260959462348265*G0_2_1 - 0.045341706583026*G0_2_2;
    A[2988] = -0.0505608958299924*G0_0_0 - 0.0479513012065083*G0_0_1 - 0.0505608958299923*G0_0_2 - 0.0479513012065087*G0_1_0 - 0.0453417065830252*G0_1_1 - 0.0479513012065087*G0_1_2 - 0.0505608958299921*G0_2_0 - 0.047951301206509*G0_2_1 - 0.0505608958299921*G0_2_2;
    A[2989] = 0.0391439193522522*G0_0_0 + 0.00717638521457891*G0_0_1 + 0.0391439193522519*G0_0_2 + 0.00717638521457924*G0_1_0 + 0.00130479731174175*G0_1_1 + 0.00717638521457946*G0_1_2 + 0.039143919352252*G0_2_0 + 0.00717638521457957*G0_2_1 + 0.039143919352252*G0_2_2;
    A[2990] = 0.0260959462348359*G0_0_0 - 0.00587158790283747*G0_0_1 + 0.026095946234836*G0_0_2 - 0.00587158790283753*G0_1_0 + 0.00130479731174141*G0_1_1 - 0.00587158790283765*G0_1_2 + 0.0260959462348353*G0_2_0 - 0.00587158790283788*G0_2_1 + 0.0260959462348353*G0_2_2;
    A[2991] = 0.00260959462348341*G0_0_1 + 0.00260959462348371*G0_1_0 - 0.0453417065830252*G0_1_1 + 0.00260959462348375*G0_1_2 + 0.00260959462348363*G0_2_1;
    A[2992] = -0.151356488162041*G0_0_0 - 0.153966082785525*G0_0_1 - 0.153966082785525*G0_0_2 - 0.153966082785525*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.153966082785526*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2993] = 0.18071442767623*G0_0_0 + 0.170276049182297*G0_0_1 + 0.170276049182297*G0_0_2 + 0.170276049182297*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182297*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923198*G0_2_2;
    A[2994] = -0.161794866655976*G0_0_0 - 0.095576403085082*G0_0_1 - 0.0955764030850819*G0_0_2 - 0.0955764030850823*G0_1_0 - 0.185933616923198*G0_1_1 - 0.185933616923198*G0_1_2 - 0.0955764030850827*G0_2_0 - 0.185933616923198*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2995] = 0.166687856575008*G0_0_0 + 0.0352295274170272*G0_0_1 + 0.0352295274170272*G0_0_2 + 0.0352295274170269*G0_1_0 + 0.0352295274170271*G0_2_0;
    A[2996] = -0.782878387045047*G0_0_0 - 0.316413348097373*G0_0_1 - 0.316413348097374*G0_0_2 - 0.316413348097372*G0_1_0 - 0.185933616923198*G0_1_2 - 0.316413348097371*G0_2_0 - 0.185933616923198*G0_2_1;
    A[2997] = 0.146789697570946*G0_0_0 - 0.298472385060924*G0_0_1 - 0.102752788299662*G0_0_2 - 0.298472385060925*G0_1_0 - 0.782878387045048*G0_1_1 - 0.298472385060924*G0_1_2 - 0.102752788299663*G0_2_0 - 0.298472385060924*G0_2_1;
    A[2998] = 0.0750258454251502*G0_0_1 + 0.0260959462348359*G0_0_2 + 0.0750258454251505*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0587158790283791*G0_1_2 + 0.0260959462348361*G0_2_0 + 0.0587158790283792*G0_2_1;
    A[2999] = 0.146789697570945*G0_0_0 - 0.102752788299662*G0_0_1 - 0.298472385060923*G0_0_2 - 0.102752788299663*G0_1_0 - 0.298472385060924*G0_1_2 - 0.298472385060925*G0_2_0 - 0.298472385060923*G0_2_1 - 0.782878387045045*G0_2_2;
    A[3000] = 0.0929668084615987*G0_0_1 + 0.0929668084615985*G0_0_2 + 0.0929668084615999*G0_1_0 - 0.0489298991903158*G0_1_1 - 0.0293579395141898*G0_1_2 + 0.0929668084615996*G0_2_0 - 0.0293579395141898*G0_2_1 - 0.0489298991903155*G0_2_2;
    A[3001] = 0.0260959462348353*G0_0_1 + 0.0750258454251497*G0_0_2 + 0.026095946234835*G0_1_0 + 0.0587158790283788*G0_1_2 + 0.0750258454251514*G0_2_0 + 0.0587158790283785*G0_2_1 + 0.293579395141893*G0_2_2;
    A[3002] = 0.195719596761262*G0_0_0 + 0.120693751336111*G0_0_1 + 0.120693751336112*G0_0_2 + 0.120693751336111*G0_1_0 + 0.0456679059109606*G0_1_1 + 0.0717638521457957*G0_1_2 + 0.12069375133611*G0_2_0 + 0.0717638521457953*G0_2_1 + 0.0456679059109604*G0_2_2;
    A[3003] = 0.146789697570946*G0_0_0 + 0.00489298991903259*G0_0_1 + 0.0538228891093462*G0_0_2 + 0.00489298991903235*G0_1_0 + 0.00978597983806359*G0_1_1 + 0.00489298991903181*G0_1_2 + 0.0538228891093476*G0_2_0 + 0.00489298991903195*G0_2_1 - 0.0391439193522517*G0_2_2;
    A[3004] = -0.0750258454251517*G0_0_1 - 0.0260959462348352*G0_0_2 - 0.0750258454251504*G0_1_0 + 0.0456679059109607*G0_1_1 - 0.0260959462348355*G0_1_2 - 0.0260959462348361*G0_2_0 - 0.0260959462348352*G0_2_1 - 0.0521918924696707*G0_2_2;
    A[3005] = 0.146789697570947*G0_0_0 + 0.0538228891093459*G0_0_1 + 0.00489298991903177*G0_0_2 + 0.053822889109347*G0_1_0 - 0.0391439193522524*G0_1_1 + 0.00489298991903133*G0_1_2 + 0.00489298991903221*G0_2_0 + 0.0048929899190328*G0_2_1 + 0.00978597983806339*G0_2_2;
    A[3006] = -0.0929668084615986*G0_0_1 - 0.0929668084615985*G0_0_2 - 0.0929668084615999*G0_1_0 - 0.0391439193522526*G0_1_1 - 0.0440369092712846*G0_1_2 - 0.0929668084615996*G0_2_0 - 0.0440369092712852*G0_2_1 - 0.0391439193522529*G0_2_2;
    A[3007] = -0.0260959462348353*G0_0_1 - 0.0750258454251516*G0_0_2 - 0.026095946234835*G0_1_0 - 0.0521918924696699*G0_1_1 - 0.0260959462348346*G0_1_2 - 0.0750258454251514*G0_2_0 - 0.0260959462348352*G0_2_1 + 0.0456679059109609*G0_2_2;
    A[3008] = 0.14352770429159*G0_0_0 + 0.218553549716741*G0_0_1 + 0.185933616923198*G0_0_2 + 0.218553549716741*G0_1_0 + 0.293579395141891*G0_1_1 + 0.234863516113513*G0_1_2 + 0.185933616923197*G0_2_0 + 0.234863516113513*G0_2_1 + 0.176147637085134*G0_2_2;
    A[3009] = -0.0391439193522491*G0_0_0 - 0.484406001984122*G0_0_1 - 0.288686405222861*G0_0_2 - 0.484406001984121*G0_1_0 - 0.782878387045046*G0_1_1 - 0.484406001984122*G0_1_2 - 0.28868640522286*G0_2_0 - 0.484406001984121*G0_2_1 - 0.185933616923198*G0_2_2;
    A[3010] = -0.150051690850303*G0_0_0 + 0.316413348097373*G0_0_1 + 0.185933616923197*G0_0_2 + 0.316413348097372*G0_1_0 + 0.185933616923197*G0_1_2 + 0.185933616923198*G0_2_0 + 0.185933616923198*G0_2_1 + 0.371867233846395*G0_2_2;
    A[3011] = -0.234863516113515*G0_0_0 - 0.141896707651913*G0_0_1 - 0.0195719596761249*G0_0_2 - 0.141896707651915*G0_1_0 - 0.0489298991903158*G0_1_1 - 0.0195719596761257*G0_1_2 - 0.0195719596761256*G0_2_0 - 0.0195719596761263*G0_2_1 - 0.0391439193522519*G0_2_2;
    A[3012] = 0.352295274170271*G0_0_0 + 0.102752788299662*G0_0_1 + 0.102752788299663*G0_0_2 + 0.102752788299663*G0_1_0 + 0.298472385060925*G0_1_2 + 0.102752788299661*G0_2_0 + 0.298472385060923*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3013] = -0.0521918924696702*G0_0_0 - 0.0260959462348355*G0_0_1 - 0.00978597983806499*G0_0_2 - 0.0260959462348349*G0_1_0 - 0.0587158790283789*G0_1_2 - 0.00978597983806236*G0_2_0 - 0.0587158790283786*G0_2_1 + 0.176147637085135*G0_2_2;
    A[3014] = 0.14352770429159*G0_0_0 + 0.185933616923197*G0_0_1 + 0.21855354971674*G0_0_2 + 0.185933616923197*G0_1_0 + 0.176147637085134*G0_1_1 + 0.234863516113512*G0_1_2 + 0.218553549716741*G0_2_0 + 0.234863516113513*G0_2_1 + 0.293579395141891*G0_2_2;
    A[3015] = -0.0391439193522491*G0_0_0 - 0.288686405222861*G0_0_1 - 0.484406001984122*G0_0_2 - 0.288686405222859*G0_1_0 - 0.185933616923198*G0_1_1 - 0.484406001984122*G0_1_2 - 0.484406001984121*G0_2_0 - 0.484406001984123*G0_2_1 - 0.782878387045046*G0_2_2;
    A[3016] = -0.150051690850303*G0_0_0 + 0.185933616923198*G0_0_1 + 0.316413348097374*G0_0_2 + 0.185933616923197*G0_1_0 + 0.371867233846396*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097373*G0_2_0 + 0.185933616923198*G0_2_1;
    A[3017] = -0.234863516113514*G0_0_0 - 0.0195719596761238*G0_0_1 - 0.141896707651914*G0_0_2 - 0.0195719596761256*G0_1_0 - 0.0391439193522508*G0_1_1 - 0.0195719596761254*G0_1_2 - 0.141896707651915*G0_2_0 - 0.019571959676125*G0_2_1 - 0.0489298991903158*G0_2_2;
    A[3018] = 0.352295274170272*G0_0_0 + 0.102752788299663*G0_0_1 + 0.102752788299662*G0_0_2 + 0.102752788299663*G0_1_0 - 0.185933616923199*G0_1_1 + 0.298472385060924*G0_1_2 + 0.102752788299663*G0_2_0 + 0.298472385060925*G0_2_1;
    A[3019] = -0.052191892469671*G0_0_0 - 0.00978597983806541*G0_0_1 - 0.0260959462348359*G0_0_2 - 0.00978597983806427*G0_1_0 + 0.176147637085134*G0_1_1 - 0.0587158790283793*G0_1_2 - 0.0260959462348352*G0_2_0 - 0.0587158790283793*G0_2_1;
    A[3020] = -1.17431758056757*G0_0_0 - 0.587158790283787*G0_0_1 - 0.587158790283784*G0_0_2 - 0.587158790283785*G0_1_0 - 0.293579395141892*G0_1_1 - 0.342509294332207*G0_1_2 - 0.587158790283784*G0_2_0 - 0.342509294332207*G0_2_1 - 0.293579395141891*G0_2_2;
    A[3021] = 1.76147637085136*G0_0_0 + 0.782878387045046*G0_0_1 + 0.782878387045045*G0_0_2 + 0.782878387045046*G0_1_0 + 1.56575677409009*G0_1_1 + 0.782878387045045*G0_1_2 + 0.782878387045045*G0_2_0 + 0.782878387045045*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3022] = -0.293579395141893*G0_0_0 + 0.293579395141895*G0_0_1 + 0.0489298991903155*G0_0_2 + 0.293579395141892*G0_1_0 - 0.293579395141894*G0_1_1 + 0.0489298991903154*G0_1_2 + 0.048929899190315*G0_2_0 + 0.0489298991903143*G0_2_1 + 0.097859798380631*G0_2_2;
    A[3023] = -0.293579395141894*G0_0_0 + 0.0489298991903151*G0_0_1 + 0.293579395141894*G0_0_2 + 0.0489298991903158*G0_1_0 + 0.0978597983806311*G0_1_1 + 0.0489298991903151*G0_1_2 + 0.293579395141892*G0_2_0 + 0.048929899190316*G0_2_1 - 0.293579395141893*G0_2_2;
    A[3024] = 0.0440369092712838*G0_0_0 + 0.0440369092712838*G0_0_1 + 0.0440369092712838*G0_0_2 + 0.0440369092712837*G0_1_0 + 0.0440369092712836*G0_1_1 + 0.0440369092712837*G0_1_2 + 0.0440369092712836*G0_2_0 + 0.0440369092712836*G0_2_1 + 0.0440369092712836*G0_2_2;
    A[3025] = 0.0440369092712838*G0_0_0;
    A[3026] = -0.0782878387045047*G0_1_1;
    A[3027] = 0.0440369092712838*G0_2_2;
    A[3028] = 0.0962288017409544*G0_1_1 - 0.0352295274170263*G0_1_2 - 0.0352295274170267*G0_2_1;
    A[3029] = -0.15657567740901*G0_1_1 - 0.0903572138381162*G0_1_2 - 0.0903572138381162*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3030] = 0.0260959462348345*G0_1_1 + 0.0156575677409006*G0_1_2 + 0.0156575677409015*G0_2_1 + 0.185933616923199*G0_2_2;
    A[3031] = -0.00260959462348322*G0_1_2 - 0.00260959462348415*G0_2_1 - 0.15657567740901*G0_2_2;
    A[3032] = -0.0505608958299919*G0_0_0 - 0.00260959462348331*G0_0_2 - 0.00260959462348306*G0_2_0;
    A[3033] = 0.0391439193522515*G0_0_0 + 0.0319675341376729*G0_0_2 + 0.031967534137672*G0_2_0 + 0.0260959462348349*G0_2_2;
    A[3034] = 0.0260959462348364*G0_0_0 + 0.0319675341376734*G0_0_2 + 0.0319675341376734*G0_2_0 + 0.0391439193522525*G0_2_2;
    A[3035] = -0.0026095946234836*G0_0_2 - 0.00260959462348404*G0_2_0 - 0.050560895829993*G0_2_2;
    A[3036] = -0.156575677409009*G0_0_0 - 0.0026095946234834*G0_0_1 - 0.00260959462348322*G0_1_0;
    A[3037] = 0.185933616923199*G0_0_0 + 0.0156575677409012*G0_0_1 + 0.0156575677409015*G0_1_0 + 0.0260959462348355*G0_1_1;
    A[3038] = -0.185933616923198*G0_0_0 - 0.0903572138381153*G0_0_1 - 0.0903572138381162*G0_1_0 - 0.15657567740901*G0_1_1;
    A[3039] = -0.0352295274170269*G0_0_1 - 0.0352295274170263*G0_1_0 + 0.0962288017409544*G0_1_1;
    A[3040] = -0.050560895829992*G0_0_0 - 0.0505608958299922*G0_0_1 - 0.047951301206509*G0_0_2 - 0.0505608958299922*G0_1_0 - 0.0505608958299921*G0_1_1 - 0.0479513012065087*G0_1_2 - 0.0479513012065089*G0_2_0 - 0.0479513012065089*G0_2_1 - 0.0453417065830251*G0_2_2;
    A[3041] = 0.0391439193522525*G0_0_0 + 0.0391439193522526*G0_0_1 + 0.00717638521457967*G0_0_2 + 0.0391439193522525*G0_1_0 + 0.0391439193522524*G0_1_1 + 0.00717638521457923*G0_1_2 + 0.00717638521457963*G0_2_0 + 0.00717638521457959*G0_2_1 + 0.00130479731174161*G0_2_2;
    A[3042] = 0.0260959462348353*G0_0_0 + 0.0260959462348353*G0_0_1 - 0.00587158790283799*G0_0_2 + 0.0260959462348362*G0_1_0 + 0.0260959462348363*G0_1_1 - 0.0058715879028375*G0_1_2 - 0.005871587902838*G0_2_0 - 0.00587158790283797*G0_2_1 + 0.00130479731174173*G0_2_2;
    A[3043] = 0.00260959462348377*G0_0_2 + 0.00260959462348325*G0_1_2 + 0.00260959462348397*G0_2_0 + 0.00260959462348393*G0_2_1 - 0.0453417065830257*G0_2_2;
    A[3044] = -0.156575677409009*G0_0_0 - 0.153966082785526*G0_0_1 - 0.156575677409009*G0_0_2 - 0.153966082785525*G0_1_0 - 0.151356488162042*G0_1_1 - 0.153966082785525*G0_1_2 - 0.156575677409009*G0_2_0 - 0.153966082785525*G0_2_1 - 0.156575677409009*G0_2_2;
    A[3045] = 0.185933616923197*G0_0_0 + 0.170276049182297*G0_0_1 + 0.185933616923198*G0_0_2 + 0.170276049182297*G0_1_0 + 0.180714427676231*G0_1_1 + 0.170276049182297*G0_1_2 + 0.185933616923198*G0_2_0 + 0.170276049182297*G0_2_1 + 0.185933616923198*G0_2_2;
    A[3046] = -0.185933616923197*G0_0_0 - 0.0955764030850824*G0_0_1 - 0.185933616923197*G0_0_2 - 0.095576403085081*G0_1_0 - 0.161794866655976*G0_1_1 - 0.0955764030850809*G0_1_2 - 0.185933616923198*G0_2_0 - 0.0955764030850822*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3047] = 0.035229527417027*G0_0_1 + 0.0352295274170262*G0_1_0 + 0.166687856575008*G0_1_1 + 0.0352295274170262*G0_1_2 + 0.035229527417027*G0_2_1;
    A[3048] = -0.0453417065830253*G0_0_0 - 0.0479513012065087*G0_0_1 - 0.0479513012065087*G0_0_2 - 0.0479513012065086*G0_1_0 - 0.0505608958299922*G0_1_1 - 0.0505608958299923*G0_1_2 - 0.0479513012065085*G0_2_0 - 0.0505608958299923*G0_2_1 - 0.0505608958299923*G0_2_2;
    A[3049] = 0.0013047973117415*G0_0_0 + 0.00717638521457926*G0_0_1 + 0.0071763852145793*G0_0_2 + 0.00717638521457909*G0_1_0 + 0.0391439193522521*G0_1_1 + 0.0391439193522522*G0_1_2 + 0.00717638521457904*G0_2_0 + 0.0391439193522524*G0_2_1 + 0.0391439193522523*G0_2_2;
    A[3050] = 0.00130479731174182*G0_0_0 - 0.00587158790283729*G0_0_1 - 0.00587158790283742*G0_0_2 - 0.00587158790283766*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.0260959462348354*G0_1_2 - 0.0058715879028377*G0_2_0 + 0.0260959462348351*G0_2_1 + 0.026095946234835*G0_2_2;
    A[3051] = -0.0453417065830257*G0_0_0 + 0.00260959462348329*G0_0_1 + 0.0026095946234834*G0_0_2 + 0.00260959462348382*G0_1_0 + 0.00260959462348354*G0_2_0;
    A[3052] = 0.293579395141893*G0_0_0 + 0.0750258454251508*G0_0_1 + 0.0587158790283793*G0_0_2 + 0.0750258454251499*G0_1_0 + 0.0260959462348355*G0_1_2 + 0.0587158790283781*G0_2_0 + 0.0260959462348348*G0_2_1;
    A[3053] = -0.782878387045047*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060923*G0_0_2 - 0.298472385060924*G0_1_0 + 0.146789697570947*G0_1_1 - 0.102752788299662*G0_1_2 - 0.298472385060924*G0_2_0 - 0.102752788299663*G0_2_1;
    A[3054] = -0.316413348097373*G0_0_1 - 0.185933616923198*G0_0_2 - 0.316413348097372*G0_1_0 - 0.782878387045047*G0_1_1 - 0.316413348097372*G0_1_2 - 0.185933616923198*G0_2_0 - 0.316413348097373*G0_2_1;
    A[3055] = -0.0489298991903137*G0_0_0 + 0.0929668084616005*G0_0_1 - 0.0293579395141897*G0_0_2 + 0.0929668084616004*G0_1_0 + 0.0929668084615991*G0_1_2 - 0.0293579395141888*G0_2_0 + 0.0929668084615992*G0_2_1 - 0.0489298991903164*G0_2_2;
    A[3056] = -0.102752788299663*G0_0_1 - 0.298472385060925*G0_0_2 - 0.102752788299663*G0_1_0 + 0.146789697570947*G0_1_1 - 0.298472385060924*G0_1_2 - 0.298472385060924*G0_2_0 - 0.298472385060924*G0_2_1 - 0.782878387045048*G0_2_2;
    A[3057] = 0.0260959462348355*G0_0_1 + 0.0587158790283796*G0_0_2 + 0.0260959462348351*G0_1_0 + 0.0750258454251505*G0_1_2 + 0.0587158790283782*G0_2_0 + 0.07502584542515*G0_2_1 + 0.293579395141894*G0_2_2;
    A[3058] = 0.293579395141892*G0_0_0 + 0.218553549716742*G0_0_1 + 0.234863516113514*G0_0_2 + 0.218553549716742*G0_1_0 + 0.143527704291591*G0_1_1 + 0.185933616923199*G0_1_2 + 0.234863516113513*G0_2_0 + 0.185933616923198*G0_2_1 + 0.176147637085134*G0_2_2;
    A[3059] = -0.782878387045047*G0_0_0 - 0.484406001984122*G0_0_1 - 0.484406001984123*G0_0_2 - 0.484406001984124*G0_1_0 - 0.0391439193522503*G0_1_1 - 0.288686405222863*G0_1_2 - 0.484406001984122*G0_2_0 - 0.28868640522286*G0_2_1 - 0.185933616923198*G0_2_2;
    A[3060] = 0.316413348097372*G0_0_1 + 0.185933616923197*G0_0_2 + 0.316413348097372*G0_1_0 - 0.150051690850303*G0_1_1 + 0.185933616923198*G0_1_2 + 0.185933616923199*G0_2_0 + 0.185933616923198*G0_2_1 + 0.371867233846398*G0_2_2;
    A[3061] = -0.0489298991903153*G0_0_0 - 0.141896707651916*G0_0_1 - 0.0195719596761257*G0_0_2 - 0.141896707651915*G0_1_0 - 0.234863516113516*G0_1_1 - 0.0195719596761253*G0_1_2 - 0.019571959676126*G0_2_0 - 0.0195719596761256*G0_2_1 - 0.039143919352252*G0_2_2;
    A[3062] = 0.102752788299663*G0_0_1 + 0.298472385060925*G0_0_2 + 0.102752788299662*G0_1_0 + 0.352295274170273*G0_1_1 + 0.102752788299665*G0_1_2 + 0.298472385060924*G0_2_0 + 0.102752788299662*G0_2_1 - 0.185933616923201*G0_2_2;
    A[3063] = -0.0260959462348357*G0_0_1 - 0.0587158790283791*G0_0_2 - 0.0260959462348352*G0_1_0 - 0.0521918924696709*G0_1_1 - 0.00978597983806487*G0_1_2 - 0.0587158790283782*G0_2_0 - 0.00978597983806372*G0_2_1 + 0.176147637085136*G0_2_2;
    A[3064] = 0.0456679059109605*G0_0_0 + 0.120693751336112*G0_0_1 + 0.0717638521457959*G0_0_2 + 0.120693751336111*G0_1_0 + 0.195719596761262*G0_1_1 + 0.120693751336112*G0_1_2 + 0.0717638521457955*G0_2_0 + 0.120693751336111*G0_2_1 + 0.0456679059109607*G0_2_2;
    A[3065] = 0.00978597983806307*G0_0_0 + 0.00489298991903166*G0_0_1 + 0.00489298991903135*G0_0_2 + 0.00489298991903214*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0538228891093465*G0_1_2 + 0.0048929899190319*G0_2_0 + 0.0538228891093469*G0_2_1 - 0.0391439193522525*G0_2_2;
    A[3066] = 0.0456679059109603*G0_0_0 - 0.0750258454251507*G0_0_1 - 0.0260959462348354*G0_0_2 - 0.0750258454251519*G0_1_0 - 0.0260959462348358*G0_1_2 - 0.0260959462348355*G0_2_0 - 0.0260959462348348*G0_2_1 - 0.0521918924696697*G0_2_2;
    A[3067] = -0.0391439193522524*G0_0_0 + 0.0538228891093478*G0_0_1 + 0.00489298991903175*G0_0_2 + 0.0538228891093463*G0_1_0 + 0.146789697570948*G0_1_1 + 0.00489298991903222*G0_1_2 + 0.004892989919032*G0_2_0 + 0.00489298991903156*G0_2_1 + 0.00978597983806346*G0_2_2;
    A[3068] = -0.0391439193522512*G0_0_0 - 0.0929668084616002*G0_0_1 - 0.0440369092712841*G0_0_2 - 0.092966808461598*G0_1_0 - 0.0929668084615985*G0_1_2 - 0.0440369092712832*G0_2_0 - 0.0929668084615991*G0_2_1 - 0.0391439193522525*G0_2_2;
    A[3069] = -0.0521918924696716*G0_0_0 - 0.0260959462348357*G0_0_1 - 0.0260959462348352*G0_0_2 - 0.0260959462348363*G0_1_0 - 0.0750258454251519*G0_1_2 - 0.0260959462348355*G0_2_0 - 0.0750258454251502*G0_2_1 + 0.0456679059109617*G0_2_2;
    A[3070] = 0.176147637085135*G0_0_0 + 0.185933616923198*G0_0_1 + 0.234863516113513*G0_0_2 + 0.185933616923198*G0_1_0 + 0.143527704291591*G0_1_1 + 0.218553549716741*G0_1_2 + 0.234863516113513*G0_2_0 + 0.218553549716741*G0_2_1 + 0.293579395141892*G0_2_2;
    A[3071] = -0.0391439193522521*G0_0_0 - 0.0195719596761264*G0_0_1 - 0.0195719596761256*G0_0_2 - 0.0195719596761249*G0_1_0 - 0.234863516113514*G0_1_1 - 0.141896707651914*G0_1_2 - 0.0195719596761253*G0_2_0 - 0.141896707651915*G0_2_1 - 0.0489298991903152*G0_2_2;
    A[3072] = 0.176147637085135*G0_0_0 - 0.00978597983806369*G0_0_1 - 0.0587158790283791*G0_0_2 - 0.00978597983806449*G0_1_0 - 0.052191892469671*G0_1_1 - 0.0260959462348354*G0_1_2 - 0.0587158790283788*G0_2_0 - 0.026095946234835*G0_2_1;
    A[3073] = -0.185933616923197*G0_0_0 - 0.288686405222859*G0_0_1 - 0.484406001984121*G0_0_2 - 0.288686405222861*G0_1_0 - 0.03914391935225*G0_1_1 - 0.484406001984122*G0_1_2 - 0.484406001984122*G0_2_0 - 0.484406001984122*G0_2_1 - 0.782878387045046*G0_2_2;
    A[3074] = -0.185933616923199*G0_0_0 + 0.102752788299662*G0_0_1 + 0.298472385060922*G0_0_2 + 0.102752788299662*G0_1_0 + 0.352295274170271*G0_1_1 + 0.102752788299661*G0_1_2 + 0.298472385060923*G0_2_0 + 0.102752788299662*G0_2_1;
    A[3075] = 0.371867233846395*G0_0_0 + 0.185933616923197*G0_0_1 + 0.185933616923198*G0_0_2 + 0.185933616923197*G0_1_0 - 0.150051690850303*G0_1_1 + 0.316413348097372*G0_1_2 + 0.185933616923198*G0_2_0 + 0.316413348097373*G0_2_1;
    A[3076] = -0.293579395141891*G0_0_0 - 0.587158790283786*G0_0_1 - 0.342509294332208*G0_0_2 - 0.587158790283786*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283787*G0_1_2 - 0.342509294332206*G0_2_0 - 0.587158790283783*G0_2_1 - 0.293579395141891*G0_2_2;
    A[3077] = -0.293579395141893*G0_0_0 + 0.293579395141892*G0_0_1 + 0.048929899190315*G0_0_2 + 0.293579395141895*G0_1_0 - 0.293579395141894*G0_1_1 + 0.0489298991903144*G0_1_2 + 0.0489298991903155*G0_2_0 + 0.0489298991903154*G0_2_1 + 0.0978597983806309*G0_2_2;
    A[3078] = 1.56575677409009*G0_0_0 + 0.782878387045047*G0_0_1 + 0.782878387045046*G0_0_2 + 0.782878387045047*G0_1_0 + 1.76147637085136*G0_1_1 + 0.782878387045045*G0_1_2 + 0.782878387045046*G0_2_0 + 0.782878387045045*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3079] = 0.0978597983806297*G0_0_0 + 0.0489298991903151*G0_0_1 + 0.0489298991903152*G0_0_2 + 0.0489298991903145*G0_1_0 - 0.293579395141894*G0_1_1 + 0.293579395141895*G0_1_2 + 0.0489298991903148*G0_2_0 + 0.293579395141893*G0_2_1 - 0.293579395141894*G0_2_2;
    A[3080] = 0.0440369092712839*G0_0_0 + 0.0440369092712839*G0_0_1 + 0.0440369092712839*G0_0_2 + 0.0440369092712836*G0_1_0 + 0.0440369092712836*G0_1_1 + 0.0440369092712836*G0_1_2 + 0.0440369092712837*G0_2_0 + 0.0440369092712837*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[3081] = 0.0440369092712841*G0_0_0;
    A[3082] = 0.044036909271284*G0_1_1;
    A[3083] = -0.0782878387045048*G0_2_2;
    A[3084] = -0.156575677409009*G0_1_1 - 0.00260959462348347*G0_1_2 - 0.00260959462348363*G0_2_1;
    A[3085] = 0.185933616923198*G0_1_1 + 0.015657567740901*G0_1_2 + 0.0156575677409007*G0_2_1 + 0.0260959462348342*G0_2_2;
    A[3086] = -0.185933616923198*G0_1_1 - 0.0903572138381158*G0_1_2 - 0.0903572138381162*G0_2_1 - 0.15657567740901*G0_2_2;
    A[3087] = -0.0352295274170272*G0_1_2 - 0.0352295274170269*G0_2_1 + 0.0962288017409542*G0_2_2;
    A[3088] = -0.15657567740901*G0_0_0 - 0.00260959462348362*G0_0_2 - 0.00260959462348363*G0_2_0;
    A[3089] = 0.1859336169232*G0_0_0 + 0.0156575677409015*G0_0_2 + 0.0156575677409014*G0_2_0 + 0.0260959462348344*G0_2_2;
    A[3090] = -0.185933616923199*G0_0_0 - 0.0903572138381162*G0_0_2 - 0.0903572138381163*G0_2_0 - 0.156575677409009*G0_2_2;
    A[3091] = -0.0352295274170268*G0_0_2 - 0.0352295274170269*G0_2_0 + 0.0962288017409539*G0_2_2;
    A[3092] = -0.0505608958299929*G0_0_0 - 0.0026095946234836*G0_0_1 - 0.00260959462348366*G0_1_0;
    A[3093] = 0.0391439193522523*G0_0_0 + 0.0319675341376726*G0_0_1 + 0.0319675341376728*G0_1_0 + 0.0260959462348347*G0_1_1;
    A[3094] = 0.0260959462348351*G0_0_0 + 0.0319675341376731*G0_0_1 + 0.0319675341376725*G0_1_0 + 0.0391439193522522*G0_1_1;
    A[3095] = -0.00260959462348383*G0_0_1 - 0.00260959462348352*G0_1_0 - 0.0505608958299926*G0_1_1;
    A[3096] = -0.156575677409009*G0_0_0 - 0.156575677409009*G0_0_1 - 0.153966082785526*G0_0_2 - 0.156575677409009*G0_1_0 - 0.156575677409009*G0_1_1 - 0.153966082785525*G0_1_2 - 0.153966082785526*G0_2_0 - 0.153966082785526*G0_2_1 - 0.151356488162042*G0_2_2;
    A[3097] = 0.185933616923199*G0_0_0 + 0.185933616923199*G0_0_1 + 0.170276049182297*G0_0_2 + 0.185933616923199*G0_1_0 + 0.185933616923199*G0_1_1 + 0.170276049182296*G0_1_2 + 0.170276049182298*G0_2_0 + 0.170276049182298*G0_2_1 + 0.180714427676231*G0_2_2;
    A[3098] = -0.185933616923198*G0_0_0 - 0.185933616923199*G0_0_1 - 0.0955764030850822*G0_0_2 - 0.185933616923199*G0_1_0 - 0.185933616923199*G0_1_1 - 0.095576403085082*G0_1_2 - 0.0955764030850825*G0_2_0 - 0.0955764030850825*G0_2_1 - 0.161794866655976*G0_2_2;
    A[3099] = 0.0352295274170265*G0_0_2 + 0.0352295274170267*G0_1_2 + 0.0352295274170268*G0_2_0 + 0.0352295274170268*G0_2_1 + 0.166687856575008*G0_2_2;
    A[3100] = -0.0505608958299929*G0_0_0 - 0.0479513012065092*G0_0_1 - 0.050560895829993*G0_0_2 - 0.0479513012065093*G0_1_0 - 0.0453417065830253*G0_1_1 - 0.0479513012065094*G0_1_2 - 0.0505608958299932*G0_2_0 - 0.0479513012065091*G0_2_1 - 0.0505608958299932*G0_2_2;
    A[3101] = 0.0391439193522522*G0_0_0 + 0.00717638521457982*G0_0_1 + 0.0391439193522524*G0_0_2 + 0.00717638521457954*G0_1_0 + 0.00130479731174176*G0_1_1 + 0.00717638521457972*G0_1_2 + 0.0391439193522526*G0_2_0 + 0.00717638521457985*G0_2_1 + 0.0391439193522527*G0_2_2;
    A[3102] = 0.0260959462348349*G0_0_0 - 0.00587158790283796*G0_0_1 + 0.0260959462348348*G0_0_2 - 0.00587158790283846*G0_1_0 + 0.00130479731174156*G0_1_1 - 0.00587158790283853*G0_1_2 + 0.0260959462348352*G0_2_0 - 0.00587158790283787*G0_2_1 + 0.0260959462348351*G0_2_2;
    A[3103] = 0.00260959462348354*G0_0_1 + 0.00260959462348353*G0_1_0 - 0.0453417065830255*G0_1_1 + 0.00260959462348343*G0_1_2 + 0.00260959462348341*G0_2_1;
    A[3104] = -0.0453417065830257*G0_0_0 - 0.0479513012065094*G0_0_1 - 0.0479513012065094*G0_0_2 - 0.0479513012065085*G0_1_0 - 0.0505608958299928*G0_1_1 - 0.0505608958299928*G0_1_2 - 0.0479513012065087*G0_2_0 - 0.0505608958299929*G0_2_1 - 0.0505608958299929*G0_2_2;
    A[3105] = 0.00130479731174176*G0_0_0 + 0.00717638521457958*G0_0_1 + 0.00717638521457959*G0_0_2 + 0.00717638521457913*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0391439193522519*G0_1_2 + 0.00717638521457914*G0_2_0 + 0.0391439193522519*G0_2_1 + 0.0391439193522519*G0_2_2;
    A[3106] = 0.00130479731174197*G0_0_0 - 0.00587158790283782*G0_0_1 - 0.00587158790283779*G0_0_2 - 0.00587158790283764*G0_1_0 + 0.0260959462348348*G0_1_1 + 0.0260959462348348*G0_1_2 - 0.00587158790283756*G0_2_0 + 0.0260959462348351*G0_2_1 + 0.0260959462348352*G0_2_2;
    A[3107] = -0.0453417065830261*G0_0_0 + 0.00260959462348357*G0_0_1 + 0.00260959462348355*G0_0_2 + 0.00260959462348326*G0_1_0 + 0.00260959462348334*G0_2_0;
    A[3108] = 0.293579395141893*G0_0_0 + 0.0587158790283791*G0_0_1 + 0.0750258454251501*G0_0_2 + 0.058715879028379*G0_1_0 + 0.0260959462348352*G0_1_2 + 0.0750258454251499*G0_2_0 + 0.0260959462348348*G0_2_1;
    A[3109] = -0.0489298991903146*G0_0_0 - 0.0293579395141895*G0_0_1 + 0.0929668084615997*G0_0_2 - 0.0293579395141899*G0_1_0 - 0.0489298991903162*G0_1_1 + 0.0929668084615983*G0_1_2 + 0.0929668084616003*G0_2_0 + 0.0929668084615995*G0_2_1;
    A[3110] = 0.0587158790283795*G0_0_1 + 0.026095946234835*G0_0_2 + 0.0587158790283794*G0_1_0 + 0.293579395141893*G0_1_1 + 0.075025845425151*G0_1_2 + 0.0260959462348338*G0_2_0 + 0.0750258454251498*G0_2_1;
    A[3111] = -0.782878387045048*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060924*G0_0_2 - 0.298472385060925*G0_1_0 - 0.102752788299662*G0_1_2 - 0.298472385060924*G0_2_0 - 0.102752788299662*G0_2_1 + 0.146789697570947*G0_2_2;
    A[3112] = -0.298472385060925*G0_0_1 - 0.102752788299662*G0_0_2 - 0.298472385060924*G0_1_0 - 0.782878387045047*G0_1_1 - 0.298472385060924*G0_1_2 - 0.102752788299662*G0_2_0 - 0.298472385060924*G0_2_1 + 0.146789697570947*G0_2_2;
    A[3113] = -0.185933616923198*G0_0_1 - 0.316413348097374*G0_0_2 - 0.185933616923198*G0_1_0 - 0.316413348097374*G0_1_2 - 0.316413348097373*G0_2_0 - 0.316413348097374*G0_2_1 - 0.782878387045049*G0_2_2;
    A[3114] = 0.293579395141892*G0_0_0 + 0.234863516113514*G0_0_1 + 0.218553549716743*G0_0_2 + 0.234863516113513*G0_1_0 + 0.176147637085135*G0_1_1 + 0.185933616923199*G0_1_2 + 0.218553549716743*G0_2_0 + 0.185933616923199*G0_2_1 + 0.143527704291593*G0_2_2;
    A[3115] = -0.0489298991903153*G0_0_0 - 0.0195719596761257*G0_0_1 - 0.141896707651915*G0_0_2 - 0.0195719596761254*G0_1_0 - 0.0391439193522524*G0_1_1 - 0.0195719596761257*G0_1_2 - 0.141896707651916*G0_2_0 - 0.0195719596761262*G0_2_1 - 0.234863516113516*G0_2_2;
    A[3116] = -0.0587158790283785*G0_0_1 - 0.0260959462348349*G0_0_2 - 0.0587158790283794*G0_1_0 + 0.176147637085136*G0_1_1 - 0.0097859798380625*G0_1_2 - 0.0260959462348339*G0_2_0 - 0.00978597983806312*G0_2_1 - 0.0521918924696694*G0_2_2;
    A[3117] = -0.782878387045048*G0_0_0 - 0.484406001984124*G0_0_1 - 0.484406001984124*G0_0_2 - 0.484406001984122*G0_1_0 - 0.185933616923198*G0_1_1 - 0.288686405222861*G0_1_2 - 0.484406001984123*G0_2_0 - 0.288686405222862*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[3118] = 0.298472385060925*G0_0_1 + 0.102752788299662*G0_0_2 + 0.298472385060924*G0_1_0 - 0.185933616923199*G0_1_1 + 0.102752788299661*G0_1_2 + 0.102752788299662*G0_2_0 + 0.102752788299664*G0_2_1 + 0.352295274170273*G0_2_2;
    A[3119] = 0.185933616923198*G0_0_1 + 0.316413348097374*G0_0_2 + 0.185933616923198*G0_1_0 + 0.371867233846397*G0_1_1 + 0.185933616923199*G0_1_2 + 0.316413348097373*G0_2_0 + 0.185933616923198*G0_2_1 - 0.150051690850302*G0_2_2;
    A[3120] = 0.176147637085135*G0_0_0 + 0.234863516113514*G0_0_1 + 0.185933616923199*G0_0_2 + 0.234863516113512*G0_1_0 + 0.293579395141891*G0_1_1 + 0.218553549716742*G0_1_2 + 0.185933616923198*G0_2_0 + 0.218553549716742*G0_2_1 + 0.143527704291593*G0_2_2;
    A[3121] = -0.0391439193522531*G0_0_0 - 0.0195719596761267*G0_0_1 - 0.0195719596761266*G0_0_2 - 0.0195719596761251*G0_1_0 - 0.0489298991903157*G0_1_1 - 0.141896707651914*G0_1_2 - 0.0195719596761256*G0_2_0 - 0.141896707651915*G0_2_1 - 0.234863516113514*G0_2_2;
    A[3122] = 0.176147637085137*G0_0_0 - 0.0587158790283787*G0_0_1 - 0.00978597983806334*G0_0_2 - 0.0587158790283784*G0_1_0 - 0.0260959462348341*G0_1_2 - 0.00978597983806312*G0_2_0 - 0.0260959462348346*G0_2_1 - 0.0521918924696693*G0_2_2;
    A[3123] = -0.185933616923198*G0_0_0 - 0.484406001984122*G0_0_1 - 0.288686405222861*G0_0_2 - 0.484406001984122*G0_1_0 - 0.782878387045046*G0_1_1 - 0.484406001984122*G0_1_2 - 0.288686405222861*G0_2_0 - 0.484406001984122*G0_2_1 - 0.0391439193522514*G0_2_2;
    A[3124] = -0.1859336169232*G0_0_0 + 0.298472385060924*G0_0_1 + 0.102752788299663*G0_0_2 + 0.298472385060924*G0_1_0 + 0.102752788299662*G0_1_2 + 0.102752788299662*G0_2_0 + 0.102752788299663*G0_2_1 + 0.352295274170271*G0_2_2;
    A[3125] = 0.371867233846397*G0_0_0 + 0.185933616923198*G0_0_1 + 0.185933616923198*G0_0_2 + 0.185933616923199*G0_1_0 + 0.316413348097374*G0_1_2 + 0.185933616923199*G0_2_0 + 0.316413348097373*G0_2_1 - 0.150051690850302*G0_2_2;
    A[3126] = 0.0456679059109615*G0_0_0 + 0.0717638521457962*G0_0_1 + 0.120693751336111*G0_0_2 + 0.0717638521457957*G0_1_0 + 0.0456679059109611*G0_1_1 + 0.12069375133611*G0_1_2 + 0.120693751336112*G0_2_0 + 0.120693751336112*G0_2_1 + 0.195719596761261*G0_2_2;
    A[3127] = 0.00978597983806246*G0_0_0 + 0.00489298991903115*G0_0_1 + 0.00489298991903116*G0_0_2 + 0.00489298991903162*G0_1_0 - 0.0391439193522521*G0_1_1 + 0.0538228891093463*G0_1_2 + 0.00489298991903208*G0_2_0 + 0.0538228891093478*G0_2_1 + 0.146789697570947*G0_2_2;
    A[3128] = 0.0456679059109619*G0_0_0 - 0.0260959462348346*G0_0_1 - 0.07502584542515*G0_0_2 - 0.0260959462348344*G0_1_0 - 0.0521918924696693*G0_1_1 - 0.026095946234835*G0_1_2 - 0.075025845425151*G0_2_0 - 0.0260959462348349*G0_2_1;
    A[3129] = -0.0391439193522521*G0_0_0 + 0.0048929899190314*G0_0_1 + 0.0538228891093468*G0_0_2 + 0.00489298991903189*G0_1_0 + 0.00978597983806273*G0_1_1 + 0.00489298991903134*G0_1_2 + 0.0538228891093473*G0_2_0 + 0.00489298991903186*G0_2_1 + 0.146789697570947*G0_2_2;
    A[3130] = -0.0391439193522522*G0_0_0 - 0.0440369092712841*G0_0_1 - 0.0929668084615997*G0_0_2 - 0.0440369092712841*G0_1_0 - 0.0391439193522522*G0_1_1 - 0.0929668084615987*G0_1_2 - 0.0929668084615994*G0_2_0 - 0.0929668084616001*G0_2_1;
    A[3131] = -0.0521918924696699*G0_0_0 - 0.0260959462348345*G0_0_1 - 0.026095946234835*G0_0_2 - 0.0260959462348341*G0_1_0 + 0.0456679059109616*G0_1_1 - 0.0750258454251508*G0_1_2 - 0.0260959462348355*G0_2_0 - 0.0750258454251509*G0_2_1;
    A[3132] = -0.293579395141892*G0_0_0 - 0.342509294332208*G0_0_1 - 0.587158790283784*G0_0_2 - 0.342509294332207*G0_1_0 - 0.29357939514189*G0_1_1 - 0.587158790283783*G0_1_2 - 0.587158790283787*G0_2_0 - 0.587158790283787*G0_2_1 - 1.17431758056757*G0_2_2;
    A[3133] = -0.293579395141894*G0_0_0 + 0.0489298991903157*G0_0_1 + 0.293579395141892*G0_0_2 + 0.0489298991903151*G0_1_0 + 0.0978597983806311*G0_1_1 + 0.048929899190316*G0_1_2 + 0.293579395141894*G0_2_0 + 0.0489298991903151*G0_2_1 - 0.293579395141893*G0_2_2;
    A[3134] = 0.0978597983806298*G0_0_0 + 0.0489298991903145*G0_0_1 + 0.0489298991903148*G0_0_2 + 0.0489298991903151*G0_1_0 - 0.293579395141894*G0_1_1 + 0.293579395141893*G0_1_2 + 0.0489298991903152*G0_2_0 + 0.293579395141895*G0_2_1 - 0.293579395141894*G0_2_2;
    A[3135] = 1.5657567740901*G0_0_0 + 0.782878387045047*G0_0_1 + 0.782878387045048*G0_0_2 + 0.782878387045047*G0_1_0 + 1.5657567740901*G0_1_1 + 0.782878387045046*G0_1_2 + 0.782878387045048*G0_2_0 + 0.782878387045046*G0_2_1 + 1.76147637085136*G0_2_2;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class UFC_Poisson3D_5BilinearForm: public ufc::form
{
public:

  /// Constructor
  UFC_Poisson3D_5BilinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5BilinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "(dXa0/dxb0)(dXa1/dxb0) | ((d/dXa0)vi0)*((d/dXa1)vi1)*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 0;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5BilinearForm_finite_element_0();
      break;
    case 1:
      return new UFC_Poisson3D_5BilinearForm_finite_element_1();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5BilinearForm_dof_map_0();
      break;
    case 1:
      return new UFC_Poisson3D_5BilinearForm_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new UFC_Poisson3D_5BilinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5LinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[3][3] ={{2*d00 / detJ, 2*d10 / detJ, 2*d20 / detJ}, {2*d01 / detJ, 2*d11 / detJ, 2*d21 / detJ}, {2*d02 / detJ, 2*d12 / detJ, 2*d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.61248608016091, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 0, 4.18330013267038, -0.591607978309963, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05987111581756e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 0, -1.67332005306816, -1.18321595661993, 7.74596669241484, 0, 0.346410161513773, 0, 0.282842712474621, 0.199999999999999, 0, 1.01607205572352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.44948974278317, 0, 0, 0, 7.09929573971954, 0, -0.414039335605422, 0, 0, 0, 0, 0, 0, 0, 1.31603755427645e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0, 4.38178046004132, -0.774596669241483, 0, 0, 4.76235235991626, 0, -0.740656079818042, 0.130930734141596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.12132034355964, 0, 0, 0, 0, 0, 7.17137165600635, 0, -1.20615246152436e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.35849019742426e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 0, 1.58113883008419, 2.45967477524977, 0, 0, 0, 0, 5.34522483824849, -1.20948631362952, 0, 0, 0, 0, 0, -1.32866943359025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, 0, 0, 3.83405790253617, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 1.14970665364125e-14, 0, 0, 1.00926211832331e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.80288457473997, 0, 0, 0, -2.10229466468021, 0, -1.71651640581399, 0, 0, 9.81070843517429, 0, 0.305838870920591, 0, 0, 0.264864231681987, 0, 0.216260739578393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 0, 1.77281052085584, -1.25356634105602, 2.39356776939084, 0, -2.78313372537744, 0, -0.524404424085074, 0.370809924354783, 0, 9.40174755792015, -1.14927687399652e-14, 0.840917865872085, -0.301561193523231, 0, 0.350641890608828, 0, 0.0660687472643403, -0.046717659215121, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.3433848874678e-14, 1.29756443747035, 0, 0, 0, 5.01426536422406, 0, -0.383824736789373, 0, 0, 0, 0, 8.20651806648289, 0, 0, -0.631738055305786, 0, 0.0483573714561588, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0, 4.06201920231798, -0.820651806648292, 0, 0, 5.88642506110457, 0, -1.20156148406979, 0.242752078113805, 0, 0, 0, 5.13809303146605, 0, 0, -0.74161984870956, 0, 0.151382517704866, -0.0305838870920676, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 0, -0.792824967172097, 2.24244764232554, 2.14087209644419, 0, 0.095742710775635, -1.39713895489839e-14, -1.87616630392937, -2.15580611373102, 0, 0, 0, 0, 9.44035859331469, 1.15714811919607e-14, 0.422185670932519, 1.95127251818837e-14, 0.590936840285278, 0.522319308773324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.16057691494799, 0, 0, 0, 1.96214168703485, 0, 2.74642624930237, 0, 0, 0, 0, 0, 0, 0, 8.6522315682783, 0, -0.865042958313572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04119910644746e-14, 0, 0, 0, 0, 0, 0, 1.5543528562087e-14, 0, 0, 0, 0},
    {1.32664991614216, 0, 2.07610309995255, 1.46802658041887, 0, 0, 1.31624465810881, 0, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 0, 5.80409338312195, 0, -1.54743588659734, 0.34193825277012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.00508909139073, 0, 0, 0, 0, 0, 4.45964044034302, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0, 0.749149177264393, 2.75458800949268, 0, 0, 0, 0, 3.3240197266047, 2.06838446274243, 0, 0, 0, 0, 0, 0, 0, 0, 6.28180682119209, -1.77676328054752, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964681, 0, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.80288457473997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 0, -2.20543546226846, -1.55947837083931, 8.33749895885385, 0, 1.06532653910462, 0, 0.869835476750494, 0.615066564126913, 0, -2.18326971917505, 0, -0.278967971730994, -1.89076704011891, 0, -0.241593350361257, 0, -0.197260144544842, -0.139483985865489, 11.8490505948789, 0, 0.245326690731332, 0, 0.0313466946954478, 0, 0.219426862868129, 0, 0.0280373360835799, 0.190029237516524, 0, 0.0242810453028227, 0, 0.0198253904711049, 0.0140186680417927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.45525332069938, 0, 0, 0, 5.28531208250978, 0, -1.36277028773849, 0, 0, 2.33666428910958, -1.04179543359539e-14, -3.93352933905728, 0, 0, -1.19859611757307, 0, 0.30904725218263, 0, 0, 0, 11.55277744383, 0, 0.810328731479531, 0, -0.234843597212094, 0, 0.395334573318417, 0, 0, 0.120463442338382, 0, -0.0310604175145514, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331886, 0, 2.99085908983323, -1.2440333788203, 0.95014618758261, 0, -0.169967317119764, 0, -1.1796107330308, 0.490653381462657, 0, 4.9761335152812, 0, -3.2268221135323, -0.21547290184284, 0, 0.0385449644663875, 0, 0.267510569439613, -0.11126972805284, 0, 0, 10.7636648722209, 0, 1.37533064807499, 0, -0.500120235663638, 0, 0.324307824721468, 0.0216558414515664, 0, -0.00387391468765637, 0, -0.0268858238268342, 0.0111830284386732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.772618130456579, 0, 0, 0, 3.3588901228148, 0, -0.304724700110023, 0, 0, 0, 0, 7.32969916135162, 0, 0, -0.761724681099362, 0, 0.0691050664139862, 0, 0, 0, 0, 0, 9.05975063854702, 0, 0, 0, -0.736662483163211, 0, 0, 0.0765562109321137, 0, -0.00694532049720535, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376113, 0, 3.68560354207934, -0.814411017927954, 0, 0, 6.00856531485322, 0, -1.45362164025439, 0.321208037219811, 0, 0, 0, 6.99294104721297, 0, 0, -1.362614533662, 0, 0.329650402328134, -0.0728431359084717, 0, 0, 0, 0, 5.40219154319687, 0, 0, 0, -0.702817019232737, 0, 0, 0.136947913395212, 0, -0.0331311120155176, 0.00732101061700613, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.1542003094028, 0, 0, 0, -0.761811750275056, 0, 5.59815020469135, 0, 0, 2.13307290077016, 0, 0.0664963811608019, 0, 0, -2.36108976914458, 0, -3.33842298506509, 0, 0, 0, 0, 0, 0, 0, 11.5766222110848, 0, 0.360889439280294, 0, 0, 0.555625639776453, 0, 0.595437196138644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 0, 0.963624111659426, 1.36277028773849, 0.867360833110884, 0, -1.00852844760553, -1.31315568597201e-14, 1.71026313764871, -2.01556443707465, 0, 2.04415543160775, -1.06351258615242e-14, 0.182834820066006, 2.68821895760394, 2.34958042241799e-14, -3.12574097035512, 0, -1.01990506872275, 0.904016610326379, 0, 0, 0, 0, 0, 0, 11.0940489487798, 0, 0.992281903807273, -0.632607619454903, 0, 0.735567892896446, 0, 0.181909068193175, -0.147004726489975, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24564920565478e-14, 0, 0, 1.009146310714e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.705300630647536, 0, 0, 0, 1.81702705031798, 0, 1.25178444059442, 0, 0, 0, 0, 1.78428514239953, 0, 0, 5.631527706439, 0, -0.746494073225355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.68367982311023, 0, 0, -1.32524448062076, 0, 0.133143804689787, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29216299118995e-14, 0, 0},
    {1.1517511068998, 0, 2.20794021658195, 0.892142571199758, 0, 0, 2.13307290077015, 0, 3.91870514328394, -1.31949665727936, 0, 0, 0, 1.11713920350151, 0, 0, 6.6110513536048, 0, -2.33689608954558, 0.591817792331142, 0, 0, 0, 0, 0, 0, 0, 0, 6.06294257990893, 0, 0, -1.55575179137407, 0, 0.416806037297055, -0.0962371838080836, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493888, 0, -0.430945803685668, 3.04724700110021, 0.775791113542722, 0, 0.0346944333244404, -1.49568650774796e-14, -1.52970585407784, 0.901387818865989, 0, 0, 0, 0, 4.51561078268939, 0, 0.20194425340049, 0, -1.83730997289732, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, 0, 0.487107866980278, 0, 0.894874292847885, 0.903959543974648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.630840061880567, 0, 0, 0, 0.711024300256716, 0, 2.23926008187654, 0, 0, 0, 0, 0, 0, 0, 4.13862564412659, 0, 2.68954640486934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.98274066131697, 0, -1.30996183150502, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727542, 0, 1.12848100903608, 1.99489143482414, 0, 0, 0.47696960070847, 0, 4.00571020990215, 0.59009684435208, 0, 0, 0, 0, 0, 0, 2.77627448210728, 0, 4.8112068719424, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.69662601613678, 0, -2.34333096254029, 0.591780433634516, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.546323519313506, 0, 0, 0, 0, 0, 2.69341127303542, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.71889089790563, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059451, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199766, 0, 0.40720550896397, 2.53385243517954, 0, 0, 0, 0, 2.00755023262382, 3.29336152870136, 0, 0, 0, 0, 0, 0, 1.24513151875208e-14, 0, 5.00796559370719, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084685, 0, 0, 2.53908227033118, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.73861278752583, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867318, -0.381881307912986, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 3.62284418654736, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.3228756555323, 0, 4.83045891539648, 0.341565025531989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693486, 0, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62018517460197, 0, 0, 3.76497011940334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923788, -0.836660026534075, -0.591607978309962, 3.87298334620742, -0.599999999999998, 0.173205080756888, -0.489897948556638, 0.141421356237313, 0.0999999999999996, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386223, 1.22474487139159, 1.41421356237309, -0.500000000000003, 4.58257569495584, 3.54964786985977, -1.02469507659596, -0.207019667802713, -0.239045721866875, 0.0845154254728531, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000003, -2.84604989415154, 2.19089023002066, -0.387298334620745, 0, 5.49909083394701, 2.38117617995813, 0.481070235442362, -0.370328039909021, 0.0654653670707988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.59807621135333, 0, -1.5811388300842, -1.1180339887499, 0, 0, 6.87386354243376, 0, 0.267261241912429, 0.188982236504611, 0, 0, 0, -1.56646390951001e-14, 0, 0, 1.01970842391297e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.0062305898749, 1.06066017177982, -0.204124145231929, 1.58771324027146, 0, 0, 0, 3.58568582800318, -0.690065559342348, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.77942286340599, 1.36930639376292, 0.790569415042096, 1.22983738762489, 0, 0, 0, 4.62910049886275, 2.67261241912425, -0.604743156814767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.900000000000007, 0, 1.82574185835055, -1.42009389360937, 0, 0, 0, 0, 6.17213399848367, 0.698297248755177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067893, 0, 0, 1.91702895126809, 0, 0, 0, 0, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.10227038425243, 0, 0, 3.3203915431768, 0, 0, 0, 0, 0, 4.48998886412873, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219015, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849178, 0.152919435460295, -0.0529728463363973, -0.512907379155188, 0.132432115840996, -0.0458758306380885, 0.1081303697892, -0.0374574588632162, -0.0264864231681936, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114625, -0.262202212042537, 0.185404962177395, 5.56214886532174, 4.70087377896007, -1.21376039056904, 0.420458932936042, -0.150780596761618, -0.397100103328477, 0.175320945304415, 0.0572172135271331, 0.0330343736321715, -0.0233588296075578, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735171, 1.87287294316099, -0.529728463363976, -4.04586796856344, 2.50713268211204, 0.904683580569682, -0.191912368394685, -0.554003287767448, 0.156695792632001, 0, 6.35674156036771, 4.10325903324145, -1.42141062443803, 0.509731451534329, -0.315869027652891, -0.113979417590034, 0.0241786857280778, 0.0697978535688028, -0.0197418142283115, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709575, 3.51781181986757, 2.03100960115899, -0.410325903324144, 0, -3.39852909359329, 2.94321253055229, -1.04058276941337, -0.600780742034897, 0.121376039056905, 0, 0, 7.41619848709566, 2.56904651573303, 0, 0.428174419288832, -0.370809924354779, 0.131101106021265, 0.0756912588524378, -0.0152919435460309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.71269767715534, 0, 4.69041575982343, 0.94760708295868, 0, 0, -1.69926454679664, 0, -1.38744369255116, -0.280305955290692, 0, 0, 0, 8.8994381845148, 0, 0, 0.214087209644411, 0, 0.17480147469502, 0.0353152308909268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586045, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878174, 3.24961536185438, -0.938083151964683, -1.07790305686552, 0, 0, 0, 0, 4.72017929665735, -0.731247032282672, 0.211092835466259, -1.02353263143832, 0.295468420142642, 0.26115965438666, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577669, 0.580288457473998, 0.670059394260483, 0.947607082958683, 1.26655701279755, 0.981070843517428, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694758, 0, 0, 0, 0, 5.58499146191198, 4.32611578413915, -1.24884205625912, -0.432521479156774, -0.499432784842937, 0.220720193068322, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071078, -1.34846851907591, 1.03805154997627, 0.73401329020944, 0, 1.51986841535706, 0.658122329054406, -3.19105893754051, 2.45647831545196, -0.705653192844341, 0, 0, 0, 0, 0, 6.70198975429437, 2.90204669156098, 1.00508909139074, -0.773717943298673, 0.170969126385064, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 0, -0.749149177264396, 2.11891385345588, 0, 0, 1.89983551919633, 0, -1.77281052085583, -2.03704530421603, 0, 0, 0, 0, 0, 0, 8.37748719286796, 0, 0.558382828550407, 0.493545355707621, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22447624956168e-14},
    {0.741619848709566, 0.502544545695365, -0.096714742912331, 1.77807891440463, 0, 0, 0, 2.22982022017151, -0.429129101453498, 1.33513642962593, 0, 0, 0, 0, 0, 0, 0, 4.21396412211221, -0.81097777341897, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5744562646538, 0.648782218735174, 0.374574588632197, 1.37729400474634, 0, 0, 0, 2.87868552592027, 1.66200986330235, 1.03419223137121, 0, 0, 0, 0, 0, 0, 0, 5.44020428881872, 3.14090341059604, -0.888381640273766, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.663324958071084, 0, 0.865042958313566, -1.59036212878712, 0, 0, 0, 0, 3.83824736789369, -1.19418232635198, 0, 0, 0, 0, 0, 0, 0, 0, 7.25360571842496, 1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.46904157598234, 0, 0, 1.55707732496443, 0, 0, 0, 0, 0, 2.76353810488345, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.90144228736999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.812403840463591, 0, 0, 2.69693703815183, 0, 0, 0, 0, 0, 4.78658840631075, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.02544545695368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552312, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541881, -0.139483985865496, -0.945383520059461, 0.313838968197358, -0.120796675180629, 0.25624844449503, -0.0986300722724195, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963984, 0.0156733473477253, -0.519258730913241, 0.109713431434065, -0.0364215679542342, 0.01401866804179, 0.0950146187582634, -0.0315420030940289, 0.0121405226514124, -0.0257539376818875, 0.00991269523555186, 0.00700933402089779, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706032, -0.705300630647551, 1.61604676382125, 2.64265604125489, -1.44543616748915, -0.681385143869243, -0.196698948117362, 0.278174320132098, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444731, -0.366484958067585, -0.599298058786534, 0.327794111588103, 0.15452362609131, 0.0446071285599885, -0.0630840061880518, 6.54980915752513, 5.77638872191499, -1.22048487632307, 0.405164365739766, -0.155947837083929, -0.117421798606046, -0.453666435153256, 0.197667286659209, -0.0824223552841552, 0.0368331241581584, 0.0602317211691907, -0.0329445477765348, -0.0155302087572761, -0.00448318510329359, 0.00634018117570608, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659432, 0.914174100330065, 1.49542954491661, -0.622016689410148, 0.475073093791304, 2.87032228155655, -0.0849836585598799, -0.360555127546397, -0.589805366515403, 0.245326690731328, -5.15186050535791, 2.48806675764059, 2.8908723349783, -1.61341105676615, -0.107736450921414, -0.650927909109099, 0.019272482233193, 0.0817662172643042, 0.133755284719809, -0.0556348640264175, 0, 7.27756573058348, 5.38183243611043, -1.7866069197544, 0.687665324037496, 0.517781462682516, -0.250060117831819, -0.290543601573986, 0.162153912360733, 0.0108279207257825, 0.0654207163662268, -0.00193695734382766, -0.00821781403613334, -0.0134429119134184, 0.00559151421933731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927964, 0.386309065228293, 1.93297557093283, -0.525700051567136, 4.5772145303823, 1.6794450614074, 1.93925611658549, -0.152362350055012, -0.762375846428075, 0.207338896470047, 0, -4.7313004641042, 3.66484958067581, 0.282120252259006, -1.03801468669568, -0.38086234054968, -0.439781949681085, 0.0345525332069918, 0.172890591018093, -0.0470200420431731, 0, 0, 8.18726144690641, 4.52987531927351, -1.74354982331868, 0, 0.475513588177819, -0.368331241581605, -0.0283541521970749, 0.10432440129236, 0.0382781054660568, 0.0441997489897906, -0.00347266024860305, -0.0173761581875701, 0.00472569203284699, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880556, -3.39131543920102, 1.84280177103967, -0.407205508963983, 0, 5.01084538062905, 3.00428265742661, 1.33755284719805, -0.726810820127196, 0.160604018609904, 0, 0, -3.78504037128335, 3.49647052360648, 0, -1.13635292017211, -0.681307266831003, -0.303328474207914, 0.164825201164063, -0.0364215679542416, 0, 0, 0, 9.3568702250359, 2.70109577159844, 0, 0, 0.380410870542252, -0.351408509616368, 0, 0.11420776562532, 0.0684739566976048, 0.0304856587023835, -0.0165655560077574, 0.00366050530850137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54950975679639, 0, -2.32737334062814, -1.64570147151095, 0, 0, 6.74536878161603, 0, 0.917928424547679, 0.649073413641553, 0, 0, 0, -1.76635217326556, 0, 0, -1.52970585407782, 0, -0.208166599946601, -0.147196014438789, 0, 0, 0, 0, 10.9163485958752, 0, 0, 0, 0.177525072919717, 0, 0, 0.153741222957161, 0, 0.020921530482028, 0.0147937560766434, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412066, 1.5771001547014, -0.546323519313517, 0.772618130456561, 1.47524211088021, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292753, -1.14271762541258, 1.06653645038508, -0.12876968840943, 0.0332481905804026, -0.011517511068999, 4.57223067739883, -1.18054488457228, 0.408952744138947, -1.66921149253253, 0.578231822728847, 0.512529242570405, 0, 0, 0, 0, 0, 5.78831110554239, -0.698859394077356, 0.180444719640146, -0.062507884474852, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069323, -0.10313274764285, -0.0833438459664692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186249, 0.834522960396274, 0.481812055829709, 0.681385143869246, 0.433680416555438, 1.14215318100896, -0.504264223802767, 1.48113132435986, 0.855131568824349, -1.00778221853732, 1.20933866224478, 1.02207771580387, -0.263899331455877, 0.0914174100330005, 1.34410947880197, 3.53988526627811, -1.56287048517756, -0.88326369896241, -0.509952534361379, 0.452008305163194, 0, 0, 0, 0, 0, 6.56332786985937, 5.54702447438989, -1.4322355606878, 0.496140951903638, -0.316303809727451, -0.833026783443143, 0.367783946448223, 0.15753787423404, 0.0909545340965868, -0.0735023632449856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14091633431099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.743452142666481, 0.35265031532376, 1.01801377240995, 0.575875553449897, -1.46610739697708, 0.908513525158993, 0.327831580195603, 0.62589222029721, 1.80679520936143, -0.851731429836555, 0, 1.38210132827975, 0.892142571199764, -0.309047252182628, -4.54391937932164, 2.8157638532195, 1.01605126164418, -0.373247036612687, -1.07747138531276, 0.382016742281294, 0, 0, 0, 0, 0, 0, 7.50094613698214, 4.84183991155512, -1.67726254578565, 1.06930204231198, -0.662622240310384, -0.239103205508895, 0.066571902344891, 0.192176528696449, -0.0621208350291014, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10027752117729e-14, 0, 0},
    {0.575875553449902, 1.91213231759728, 1.10397010829098, 0.44607128559988, 0, -1.23153021346074, 1.06653645038507, 3.39369820402464, 1.95935257164197, -0.659748328639681, 0, 0, 1.6124515496597, 0.558569601750758, 0, -3.81689227863019, 3.3055256768024, -2.02381137955099, -1.16844804477279, 0.295908896165575, 0, 0, 0, 0, 0, 0, 0, 8.75110382647916, 3.03147128995446, 0, 0.898213715542065, -0.777875895687036, 0.360964616749972, 0.208403018648526, -0.0481185919040409, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.32945523855878e-14, 0, 0},
    {-1.32992762321609, 0, 2.54950975679639, -1.03015750727543, 0, 0, -0.615765106730374, 0, 4.52493093869951, 1.52362350055012, 0, 0, 0, 1.93494185959165, 0, 0, -1.9084461393151, 0, -2.69841517273464, -0.683372323427214, 0, 0, 0, 0, 0, 0, 0, 0, 10.501324591775, 0, 0, 0.449106857771034, 0, 0.481286155666633, 0.111125127955289, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.786795792469438, 0.746420027292178, -0.215472901842835, 1.5236235005501, 0.387895556771362, -0.0600925212577327, 0.0173472166622199, 2.64952825989834, -0.764852927038915, 0.45069390943299, 0, 0, 0, 0, 2.25780539134469, -0.349777707186207, 0.100972126700243, 3.18231422231115, -0.918654986448657, -1.37412670680299, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.44603151471411, -0.843695574376338, 0.243553933490139, -1.54996774159981, 0.447437146423943, 0.451979771987323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608038, 0.315420030940283, 0.364215679542343, 1.28769688409429, 0.458964212273842, 0.355512150128361, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137522, 0, 0, 0, 0, 2.67147136598154, 2.06931282206329, -0.597359157427889, 1.34477320243467, 1.5528103408493, -1.16134903279062, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.4438313884647, 4.99137033065849, -1.44088450201539, -0.654980915752514, -0.756306816047561, 0.381992627346313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.51507875363771, -0.732969916135158, 0.56424050451804, 0.997445717412072, 0, 0.550757054728608, 0.238484800354237, -2.60178510148047, 2.00285510495107, 0.295048422176044, 0, 0, 0, 0, 0, 3.20576563917784, 1.38813724105364, -3.12497053047329, 2.4056034359712, -0.899577092626427, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.73259766615765, 3.34831300806839, 1.5220381072759, -1.17166548127015, 0.295890216817259, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48690428533295, 0, -0.40720550896398, 2.87937776724947, 0, 0, 0.688446318410768, 0, -1.44543616748914, 0.851731429836557, 0, 0, 0, 0, 0, 0, 4.00720704897231, 0, -1.73609473915183, -2.59685538292347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.66574708269706, 0, 0.845576726264391, 0.854161481650102, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449896, 0.273161759656754, -0.0525700051567131, 1.63559471386625, 0, 0, 0, 1.34670563651772, -0.259173620587566, 2.12585572561676, 0, 0, 0, 0, 0, 0, 0, 3.35944544895281, -0.646525578093589, 0.914325240961068, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4.72691760029726, -0.909695716322936, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599885, 0.35265031532377, 0.203602754481986, 1.26692621758977, 0, 0, 0, 1.73858950082559, 1.0037751163119, 1.64668076435068, 0, 0, 0, 0, 0, 0, 0, 4.33702542542885, 2.50398279685359, 0.708233286251854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.10242438161534, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.515078753637711, 0, 0.470200420431694, -1.46292038553772, 0, 0, 0, 0, 2.31811933443412, -1.90142316513449, 0, 0, 0, 0, 0, 0, 0, 0, 5.78270056723845, -0.817797356933148, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.13656584215379, 1.31506763029891, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12763531151527e-14},
    {0.364215679542343, 0, 0, 1.26954113516559, 0, 0, 0, 0, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.46962034034308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880561, 0, 0, 2.19890974840547, 0, 0, 0, 0, 0, 4.33630850246743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.00955871244863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.4358503831282, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.912870929175276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.58198889747161, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867317, -0.381881307912987, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 0.724568837309471, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.661437827766144, 0, 1.93218356615859, -0.170782512765995, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693487, 3.54964786985977, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.540061724867317, 0, 3.54964786985977, 1.25499003980111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.90940653956493, 0, 0, 4.43705983732471, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923789, -0.836660026534075, -0.591607978309964, 3.87298334620742, -0.600000000000001, 0.173205080756889, -0.48989794855664, 0.141421356237313, 0.0999999999999997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386222, 1.22474487139158, 1.41421356237309, -0.500000000000002, 0.654653670707977, 3.54964786985977, -1.02469507659596, -0.20701966780271, -0.239045721866875, 0.0845154254728546, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000001, 0.316227766016839, 2.19089023002066, -0.38729833462074, 0, 1.57116880969914, 2.38117617995813, -0.0534522483824853, -0.37032803990902, 0.0654653670707977, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.519615242270665, 0, 1.58113883008419, -0.223606797749979, 0, 0, 2.94594151818589, 0, -0.267261241912424, 0.0377964473009217, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.00623058987491, 1.06066017177982, -0.204124145231932, 1.58771324027146, 4.53557367611073, 0, 0, 3.58568582800318, -0.690065559342349, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.779422863405996, 0.273861278752581, 0.790569415042093, 1.22983738762489, 0, 4.53557367611073, 0, 0.925820099772549, 2.67261241912425, -0.604743156814761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.450000000000004, 0, 0.730296743340221, 0.710046946804697, 0, 0, 4.53557367611072, 0, 2.46885359939347, -0.349148624377589, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067889, -3.1304951684997, 0, 1.91702895126808, 0, 0, 0, 5.29150262212918, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.367423461417479, 0, -3.1304951684997, 1.10679718105894, 0, 0, 0, 0, 5.29150262212918, 1.49666295470958, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.85788383248864, 0, 0, -2.34787137637477, 0, 0, 0, 0, 0, 6.34980314655501, 0, 0, 0, 0, 0, 0, 1.11852851939587e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219016, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849176, 0.152919435460295, -0.0529728463363977, -0.512907379155187, 0.132432115840995, -0.0458758306380897, 0.1081303697892, -0.0374574588632173, -0.0264864231681947, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114629, -0.262202212042535, 0.185404962177393, 0.618016540591305, 4.70087377896007, -1.21376039056904, 0.42045893293604, -0.150780596761618, -0.397100103328475, 0.175320945304413, 0.057217213527133, 0.0330343736321713, -0.0233588296075595, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735172, 1.87287294316099, -0.529728463363977, 0.20229339842817, 2.50713268211204, 0.904683580569681, -0.191912368394685, -0.554003287767448, 0.156695792632003, 0, 1.41260923563727, 4.10325903324145, -1.42141062443803, -0.0254865725767146, -0.315869027652895, -0.113979417590039, 0.0241786857280816, 0.0697978535688034, -0.0197418142283076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709574, 0.167514848565124, 2.03100960115899, -0.410325903324145, 0, 0.849632273398319, 2.94321253055228, -0.0495515604482551, -0.600780742034895, 0.121376039056903, 0, 0, 2.47206616236523, 2.56904651573303, 0, -0.10704360482221, -0.370809924354777, 0.00624290981053448, 0.0756912588524339, -0.015291943546034, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.428174419288845, 0, 1.34011878852098, -0.236901770739674, 0, 0, 2.54889682019496, 0, -0.396412483586046, 0.0700764888226733, 0, 0, 0, 3.95530585978436, 0, 0, -0.321130814466627, 0, 0.0499432784842904, -0.00882880772273737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586047, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878164, 3.24961536185438, -0.938083151964685, -1.07790305686551, 5.52770798392566, 0, 0, 0, 4.72017929665734, -0.731247032282673, 0.211092835466259, -1.02353263143831, 0.295468420142642, 0.261159654386662, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577672, 0.580288457473996, 0.670059394260487, 0.947607082958685, 0.180936716113935, 0.981070843517426, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694756, 0, 5.52770798392567, 0, 0, 0.797855923130281, 4.32611578413915, -1.24884205625911, -0.432521479156781, -0.499432784842937, 0.220720193068326, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071081, 0.149829835452877, 1.03805154997628, 0.734013290209435, 0, 0.434248118673443, 0.658122329054405, 0.354562104171167, 2.45647831545196, -0.705653192844349, 0, 0, 5.52770798392567, 0, 0, 1.91485421551268, 2.90204669156097, -0.111676565710086, -0.773717943298669, 0.170969126385058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.382970843102543, 0, 0.749149177264393, 0.423782770691177, 0, 0, 0.814215222512714, 0, 1.77281052085584, -0.407409060843206, 0, 0, 0, 5.52770798392568, 0, 0, 3.59035165408627, 0, -0.558382828550415, 0.0987090711415228, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709561, 0.502544545695366, -0.096714742912333, 1.77807891440463, -4.23078640106406, 0, 0, 2.2298202201715, -0.429129101453499, 1.33513642962593, 0, 0, 0, 0, 6.21867148191638, 0, 0, 4.21396412211221, -0.810977773418968, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.574456264653794, 0.129756443747034, 0.374574588632199, 1.37729400474634, 0, -4.23078640106406, 0, 0.575737105184058, 1.66200986330234, 1.03419223137121, 0, 0, 0, 0, 0, 6.21867148191637, 0, 1.08804085776374, 3.14090341059604, -0.88838164027376, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.331662479035532, 0, 0.346017183325429, 0.795181064393553, 0, 0, -4.23078640106406, 0, 1.53529894715748, 0.597091163175992, 0, 0, 0, 0, 0, 0, 6.21867148191637, 0, 2.90144228736999, -0.512907379155182, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.469041575982343, 3.81404493622062, 0, 1.55707732496444, 0, 0, 0, -3.76069902316805, 0, 2.76353810488346, 0, 0, 0, 0, 0, 0, 1.02342965294379e-14, 7.10705312219014, 0, 2.90144228737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.270801280154526, 0, 3.81404493622062, 0.89897901271727, 0, 0, 0, 0, -3.76069902316805, 1.59552946877025, 0, 0, 0, 0, 0, 0, 0, 0, 7.10705312219014, 1.67514848565122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.68079590171775, 0, 0, 4.4497190922574, 0, 0, 0, 0, 0, -2.63248931621764, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.2915619758885, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552311, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541882, -0.139483985865495, -0.945383520059461, 0.313838968197358, -0.120796675180628, 0.256248444495029, -0.0986300722724191, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963982, 0.0156733473477249, -0.519258730913241, 0.109713431434065, -0.0364215679542341, 0.0140186680417898, 0.0950146187582634, -0.0315420030940288, 0.0121405226514124, -0.0257539376818875, 0.00991269523555182, 0.00700933402089769, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706025, -0.705300630647557, 1.61604676382126, 2.64265604125489, -1.44543616748915, -0.681385143869247, -0.196698948117359, 0.278174320132096, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444727, -0.366484958067587, -0.599298058786529, 0.327794111588101, 0.154523626091317, 0.0446071285599885, -0.0630840061880538, 0.595437196138649, 5.77638872191499, -1.22048487632307, 0.405164365739765, -0.155947837083928, -0.117421798606047, -0.453666435153255, 0.197667286659209, -0.0824223552841549, 0.0368331241581608, 0.0602317211691907, -0.0329445477765342, -0.0155302087572759, -0.00448318510329279, 0.00634018117570581, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659437, 0.914174100330057, 1.49542954491662, -0.622016689410149, 0.475073093791305, 2.87032228155655, -0.084983658559881, -0.3605551275464, -0.589805366515401, 0.24532669073133, 0.147196014438797, 2.4880667576406, 2.89087233497828, -1.61341105676615, -0.107736450921419, -0.650927909109092, 0.0192724822331939, 0.0817662172643072, 0.133755284719809, -0.0556348640264189, 0, 1.32319376919699, 5.38183243611043, -1.7866069197544, 0.687665324037495, -0.0147937560766431, -0.25006011783182, -0.290543601573986, 0.162153912360733, 0.0108279207257832, 0.0654207163662248, -0.00193695734382736, -0.00821781403613383, -0.0134429119134179, 0.00559151421933732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927959, 0.386309065228289, 1.93297557093283, -0.525700051567137, 0.0803020093049556, 1.6794450614074, 1.9392561165855, -0.152362350055011, -0.762375846428077, 0.207338896470047, 0, 0.567756055692504, 3.66484958067581, 0.282120252259013, -0.0182107839771216, -0.38086234054968, -0.439781949681088, 0.0345525332069925, 0.172890591018093, -0.0470200420431761, 0, 0, 2.23288948551993, 4.52987531927351, -1.74354982331867, 0, -0.0570616305813383, -0.368331241581605, -0.0283541521970753, 0.0018302526542526, 0.0382781054660563, 0.0441997489897914, -0.00347266024860275, -0.0173761581875703, 0.00472569203284603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880563, 0.09974457174121, 1.84280177103967, -0.407205508963979, 0, 0.513932859551699, 3.00428265742661, -0.03933978962347, -0.726810820127196, 0.160604018609906, 0, 0, 1.51401614851335, 3.49647052360648, 0, -0.116549017453554, -0.681307266831002, 0.00892142571199782, 0.164825201164067, -0.036421567954237, 0, 0, 0, 3.40249826364942, 2.70109577159844, 0, 0, -0.152164348216903, -0.351408509616369, 0, 0.0117136169872135, 0.0684739566976059, -0.000896637020657868, -0.0165655560077585, 0.00366050530850301, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, 0, 1.16368667031409, -0.23510021021585, 0, 0, 2.24845626053868, 0, -0.458964212273845, 0.0927247733773657, 0, 0, 0, 3.53270434653115, 0, 0, -0.509901951359277, 0, 0.104083299973309, -0.0210280020626858, 0, 0, 0, 0, 4.96197663448874, 0, 0, 0, -0.355050145839439, 0, 0, 0.0512470743190546, 0, -0.010460765241014, 0.00211339372523455, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412067, 1.5771001547014, -0.546323519313514, 0.772618130456565, 1.4752421108802, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292756, -1.14271762541258, 1.06653645038508, -0.128769688409428, 0.0332481905804048, -0.0115175110689984, 4.57223067739883, -1.18054488457228, 0.408952744138948, -1.66921149253254, 0.578231822728845, 0.512529242570406, 6.52268767805531, 0, 0, 0, 0, 5.7883111055424, -0.698859394077357, 0.180444719640146, -0.0625078844748521, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069321, -0.103132747642851, -0.0833438459664682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186242, 0.834522960396282, 0.481812055829709, 0.681385143869242, 0.433680416555441, 1.14215318100896, -0.504264223802765, 1.48113132435985, 0.855131568824352, -1.00778221853732, 0.134370962471638, 1.02207771580387, -0.263899331455871, 0.0914174100330022, 1.34410947880196, 3.5398852662781, -1.56287048517756, -0.883263698962406, -0.509952534361376, 0.452008305163191, 0, 6.52268767805531, 0, 0, 0, 0.729258652206596, 5.5470244743899, -1.4322355606878, 0.496140951903638, -0.316303809727452, -0.833026783443137, 0.367783946448223, 0.157537874234044, 0.0909545340965877, -0.0735023632449854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01901886515007e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.74345214266648, 0.352650315323769, 1.01801377240995, 0.5758755534499, 0.0733053698488504, 0.908513525158993, 0.327831580195604, 0.62589222029721, 1.80679520936143, -0.851731429836554, 0, 0.307133628506613, 0.892142571199772, -0.309047252182627, 0.227195968966083, 2.8157638532195, 1.01605126164419, -0.37324703661268, -1.07747138531275, 0.382016742281307, 0, 0, 6.52268767805531, 0, 0, 0, 1.66687691932936, 4.84183991155511, -1.67726254578565, -0.0534651021155991, -0.662622240310383, -0.239103205508893, 0.0665719023448925, 0.192176528696449, -0.0621208350290961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449893, 0.0910539198855843, 1.10397010829098, 0.446071285599878, 0, 0.307882553365187, 1.06653645038508, 0.161604676382129, 1.95935257164197, -0.65974832863968, 0, 0, 0.537483849886575, 0.558569601750753, 0, 0.954223069657542, 3.3055256768024, -0.0963719704548154, -1.16844804477279, 0.295908896165572, 0, 0, 0, 6.52268767805531, 0, 0, 0, 2.91703460882639, 3.03147128995447, 0, -0.224553428885516, -0.777875895687036, 0.0171887912738079, 0.208403018648526, -0.0481185919040412, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.332481905804022, 0, 0.728431359084682, 0.257539376818849, 0, 0, 0.923647660095564, 0, 1.292837411057, -0.380905875137523, 0, 0, 0, 0.859974159818509, 0, 0, 2.86266920897263, 0, -0.770975763638467, 0.170843080856795, 0, 0, 0, 0, 6.52268767805531, 0, 0, 0, 4.66725537412222, 0, 0, -0.673660286656546, 0, 0.137510330190468, -0.0277812819888241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.78679579246944, 0.746420027292177, -0.215472901842835, 1.52362350055011, 0.387895556771359, -0.0600925212577337, 0.0173472166622209, 2.64952825989835, -0.764852927038921, 0.450693909432998, -5.28814187068052, 0, 0, 0, 2.2578053913447, -0.349777707186206, 0.100972126700245, 3.18231422231115, -0.91865498644866, -1.37412670680298, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0, 5.44603151471411, -0.843695574376342, 0.243553933490139, -1.54996774159981, 0.447437146423942, 0.451979771987325, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608043, 0.315420030940282, 0.364215679542341, 1.28769688409429, 0.0655663160391197, 0.355512150128356, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137518, 0, -5.28814187068053, 0, 0, 0.381638766568791, 2.0693128220633, -0.597359157427881, 1.34477320243467, 1.5528103408493, -1.16134903279063, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0.920547341209241, 4.99137033065848, -1.4408845020154, -0.654980915752512, -0.756306816047561, 0.381992627346309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637708, 0.0814411017927942, 0.564240504518039, 0.997445717412071, 0, 0.157359158493888, 0.238484800354236, 0.289087233497833, 2.00285510495107, 0.295048422176043, 0, 0, -5.28814187068052, 0, 0, 0.915933039765097, 1.38813724105364, 0.347218947830363, 2.4056034359712, -0.89957709262643, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 2.20931361890218, 3.34831300806839, -0.16911534525288, -1.17166548127015, 0.295890216817257, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.297380857066588, 0, 0.407205508963972, 0.575875553449888, 0, 0, 0.295048422176039, 0, 1.44543616748915, 0.17034628596731, 0, 0, 0, -5.28814187068053, 0, 0, 1.71737444955956, 0, 1.73609473915183, -0.519371076584704, 0, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 4.14246303544159, 0, -0.845576726264386, 0.170832296330017, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449895, 0.27316175965675, -0.0525700051567132, 1.63559471386626, 4.82647810240717, 0, 0, 1.34670563651771, -0.259173620587563, 2.12585572561677, 0, 0, 0, 0, -4.95763300376299, 0, 0, 3.35944544895281, -0.646525578093588, 0.914325240961076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 0, 4.72691760029725, -0.909695716322934, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599889, 0.0705300630647548, 0.203602754481986, 1.26692621758977, 0, 4.82647810240717, 0, 0.34771790016512, 1.00377511631191, 1.64668076435067, 0, 0, 0, 0, 0, -4.95763300376299, 1.29915143264338e-14, 0.867405085085771, 2.50398279685359, 0.708233286251852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873427, 0, 1.22048487632307, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.257539376818862, 0, 0.188080168172671, 0.731460192768861, 0, 0, 4.82647810240716, 0, 0.927247733773645, 0.950711582567244, 0, 0, 0, -1.22220110126485e-14, 0, 0, -4.95763300376297, 0, 2.31308022689538, 0.408898678466573, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 3.25462633686152, -0.657533815149454, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04894165366026e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, -3.97354131880428, 0, 1.26954113516558, 0, 0, 0, 5.11038857901935, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, -4.24939971751113, 0, 3.46962034034307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.210280020626859, 0, -3.97354131880429, 0.732969916135155, 0, 0, 0, 0, 5.11038857901935, 1.44543616748915, 0, 0, 0, 0, 0, 0, 0, 0, -4.24939971751114, 2.00318623748287, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 1.8119501277094, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.2711894277325, 0, 0, -3.4552533206994, 0, 0, 0, 0, 0, 6.13246629482322, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.2499377798012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[56][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0.8, 0.2}, {0, 0.6, 0.4}, {0, 0.4, 0.6}, {0, 0.2, 0.8}, {0.8, 0, 0.2}, {0.6, 0, 0.4}, {0.4, 0, 0.6}, {0.2, 0, 0.8}, {0.8, 0.2, 0}, {0.6, 0.4, 0}, {0.4, 0.6, 0}, {0.2, 0.8, 0}, {0, 0, 0.2}, {0, 0, 0.4}, {0, 0, 0.6}, {0, 0, 0.8}, {0, 0.2, 0}, {0, 0.4, 0}, {0, 0.6, 0}, {0, 0.8, 0}, {0.2, 0, 0}, {0.4, 0, 0}, {0.6, 0, 0}, {0.8, 0, 0}, {0.6, 0.2, 0.2}, {0.4, 0.4, 0.2}, {0.2, 0.6, 0.2}, {0.4, 0.2, 0.4}, {0.2, 0.4, 0.4}, {0.2, 0.2, 0.6}, {0, 0.2, 0.2}, {0, 0.4, 0.2}, {0, 0.6, 0.2}, {0, 0.2, 0.4}, {0, 0.4, 0.4}, {0, 0.2, 0.6}, {0.2, 0, 0.2}, {0.4, 0, 0.2}, {0.6, 0, 0.2}, {0.2, 0, 0.4}, {0.4, 0, 0.4}, {0.2, 0, 0.6}, {0.2, 0.2, 0}, {0.4, 0.2, 0}, {0.6, 0.2, 0}, {0.2, 0.4, 0}, {0.4, 0.4, 0}, {0.2, 0.6, 0}, {0.2, 0.2, 0.2}, {0.4, 0.2, 0.2}, {0.2, 0.4, 0.2}, {0.2, 0.2, 0.4}};
    
    // Components for each dof
    static unsigned int components[56] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class UFC_Poisson3D_5LinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[3][3] ={{2*d00 / detJ, 2*d10 / detJ, 2*d20 / detJ}, {2*d01 / detJ, 2*d11 / detJ, 2*d21 / detJ}, {2*d02 / detJ, 2*d12 / detJ, 2*d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.00945067404923495, 0.00609788120679251, 0.00352061335622802, 0.00248944957812476, 0.00705946233325038, 0.00546823600997145, 0.00315708753234943, 0.00446479600584757, 0.00257775117585286, 0.00182274533665716, -0.00298267418725716, -0.00252081977982931, -0.00195261860521383, -0.00112734487734488, -0.00218309396769448, -0.00169101731601731, -0.000976309302606915, -0.00138070985681772, -0.000797153207506479, -0.000563672438672439, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048031, 0.000830884437218266, 0.00160900079398463, 0.0012463266558274, 0.000719567030240153, 0.00101762145320216, 0.000587524019939405, 0.000415442218609126, -0.00705593524717798, -0.0063823335708908, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363027, -0.00582624677716565, -0.00513826668309062, -0.00434262794914382, -0.00336378514516178, -0.00194208225905522, -0.0045958054359652, -0.00388416451811043, -0.00300866089851429, -0.00173705117965753, -0.00336378514516178, -0.00260557676948629, -0.00150433044925715, -0.00212744452363027, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923496, -0.00609788120679253, 0.00352061335622803, 0.00248944957812477, 0.00705946233325038, -0.00546823600997146, 0.00315708753234943, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521384, -0.00112734487734488, -0.00218309396769448, 0.00169101731601732, -0.000976309302606918, 0.00138070985681772, -0.000797153207506478, -0.000563672438672437, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533348, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218266, 0.00160900079398463, -0.0012463266558274, 0.000719567030240152, -0.00101762145320216, 0.000587524019939404, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811044, 0.00300866089851429, -0.00173705117965753, -0.00336378514516178, 0.00260557676948629, -0.00150433044925715, 0.00212744452363027, -0.00122828066840393, -0.000868525589828763},
    {0.00945067404923494, 0, -0.00704122671245606, 0.00248944957812477, 0, 0, 0.00947126259704827, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.000563672438672442, 0, 0, 0, 0, 0.00464478520766674, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.000415442218609133, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923497, 0, 0, -0.00746834873437428, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0056367243867244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463432, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835493, -0.0001878908128908, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216138, 0, 0.00646276421933343, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.0536094339843699, 0, 0, -0.0189425251940966, 0, 0.0154665070551172, 0.0153110608279201, 0, 0, 0, -0.00450937950937952, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187275, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559315, 0, 0.041270230458372, -0.018239037386404},
    {0.0100234421734311, 0, 0.0409576306293855, -0.0449826285156207, 0, 0, 0.0189425251940965, 0, -0.00515550235170571, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883227, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957585, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.010023442173431, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, -0.0164047080299144, 0.00947126259704827, 0.00446479600584755, -0.00257775117585285, -0.00619733414463431, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141414, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917747, -0.000187890812890807, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.00109915679455669, 0.000928957041533348, -0.00071956703024015, 0.000415442218609124, -0.00482700238195388, 0.00373897996748219, -0.00215870109072046, 0.00559691799261187, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755859, 0.0153110608279201, -0.00298267418725716, 0.00252081977982932, -0.00195261860521384, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443228, -0.00747795993496437, -0.00321800158796927, 0.00249265331165479, -0.00143913406048031, -0.0106850252586227, 0.00616900220936374, 0.00872428659079176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, 0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, 0.00631417506469883, -0.00446479600584757, 0.00257775117585286, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, -0.00263047138047137, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978791, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588456, 0.00740723107127183, 0.0449826285156207, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878478, -0.00531435471670988, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812426, 0.0211783869997512, 0.00109364720199428, -0.00568275755822899, -0.0169662248222208, 0.00567105258687631, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443533, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023304, 0.0108922666482287, -0.00558007245254535, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609132, 0.00643600317593853, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240148, -0.000415442218609144, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438608, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709707, -0.00052111535389726, 0.000902598269554287, 0.00170195561890421, -0.000491312267361569, -0.000868525589828766},
    {0.010023442173431, -0.0249047128495239, 0.0553363732971484, -0.00431340273437458, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052564, -0.0159820572902878, 0.000729098134662858, -0.00298267418725716, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846297, -0.0116492303992304, 0.00976309302606916, 0.00352848074520083, 0.00132858867917746, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565981, -0.00724050357293083, 0.00928957041533347, -0.00219831358911339, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705068, -0.00575653624192124, -0.0045792965394097, -0.000293762009969691, 0.00290809553026391, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.0041106133464725, -0.00303983956440067, 0.00908221989193682, -0.00776832903622088, 0.000459580543596525, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680785, -0.000868525589828761},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196569, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662877, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820998, -0.0124007936507936, 0.008895262534863, 0.00291483191994852, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496157, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992318, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960612, 0.00290809553026393, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726054, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931506, -0.000336378514516176, -0.00286613444643493, 0.00120346435940571, 0.000850977809452109, 0.000982624534723145, -0.000868525589828767},
    {-0.010023442173431, -0.0128297005588456, 0.0448791059024117, -0.00801060507812425, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467806, -0.0175287079957994, 0.00838462854862291, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625302, -0.019200749617936, 0.000613648825252319, 0.0122230158484327, -0.00469727032227033, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240153, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601075, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554286, 0.000425488904726051, 0.0017195929357655, -0.000868525589828761},
    {-0.010023442173431, -0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997511, 0.0164047080299144, 0.00947126259704827, -0.00446479600584755, -0.00257775117585284, -0.00619733414463433, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141415, -0.00848980987436741, -0.00657617845117846, -0.0037967583990269, -0.0023011830946962, -0.00132858867917747, -0.000187890812890811, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992315, 0.00464478520766675, 0.00109915679455668, 0.00092895704153334, 0.000719567030240147, 0.000415442218609119, -0.00482700238195388, -0.00373897996748219, -0.00215870109072046, -0.00559691799261186, -0.00323138210966672, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.0095537814881164, 0.00740032731935591, 0.00427258096992149, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.00694820471863011, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778515, 0.00607967912880134},
    {0.010023442173431, 0.024904712849524, 0.014378742667763, 0.0536094339843699, -0.0141189246665008, -0.0109364720199429, -0.00631417506469884, -0.0133943880175427, -0.00773325352755859, 0.0153110608279201, 0.00298267418725715, 0.00252081977982931, 0.00195261860521384, 0.00112734487734489, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443227, -0.00747795993496436, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936373, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.020635115229186, -0.018239037386404},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, 0.0109364720199429, 0.00631417506469884, 0.00446479600584756, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974695, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620756, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588456, 0.00740723107127181, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878478, -0.00531435471670987, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489396, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440067},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199427, -0.00568275755822898, 0.0169662248222207, 0.00567105258687631, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890813, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254536, -0.00469727032227033, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240154, -0.000415442218609147, 0.00965400476390777, 0.00498530662330958, 0.00215870109072046, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.01143686949355, 0.00737991258440983, -0.00183832217438608, 0.00310733161448835, 0.00481385743762287, 0.00347410235931506, -0.00201827108709707, 0.000521115353897262, 0.000902598269554291, -0.00170195561890421, -0.000491312267361566, -0.000868525589828765},
    {0.010023442173431, 0.024904712849524, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057841, -0.00803663281052563, -0.0159820572902878, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846299, 0.0116492303992304, 0.00976309302606917, -0.00352848074520085, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.00727023882565979, 0.00724050357293083, 0.00928957041533346, 0.00219831358911338, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496158, -0.00685479660705068, -0.00575653624192124, 0.00457929653940971, -0.000293762009969682, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.00411061334647249, -0.00303983956440066, -0.00908221989193681, -0.00776832903622088, -0.000459580543596517, 0.00427258096992148, 0.00270779480866287, 0.000868525589828773, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.00024565613368078, -0.000868525589828764},
    {0.0100234421734312, -0.0354703486038675, -0.0492363006502187, -0.00431340273437457, 0.0141189246665008, 0.00656188321196574, -0.016416855168217, 0.00982255121286463, 0.0149509568199466, 0.000729098134662854, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820993, 0.0124007936507936, 0.00889526253486299, -0.00291483191994851, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496156, -0.00928957041533347, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870547, -0.0028782681209606, 0.00254405363300538, -0.00381890612960614, 0.00290809553026392, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290444, -0.00388416451811042, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516178, 0.00286613444643492, 0.00120346435940572, -0.000850977809452107, 0.000982624534723153, -0.000868525589828765},
    {-0.0100234421734311, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467807, -0.0175287079957994, 0.00838462854862291, 0, 0, -0.00867830491206149, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.00061364882525232, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766672, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601073, -0.00734405024924252, 0.00290809553026392, 0, 0, 0, 0, -0.0257938940364825, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702858, -0.00347410235931505, 0, 0.00208446141558903, -0.00090259826955429, -0.000425488904726052, 0.0017195929357655, -0.000868525589828762},
    {-0.010023442173431, -0.0452812960900435, -0.0113287063442981, -0.00801060507812423, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177146, -0.0142846454190328, -0.014536160727703, -0.00939454064454065, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380011, 0.00245778931055952, 0.00830884437218264, 0.00965400476390776, 0.00650269929073345, 0.00219831358911337, 0.00743165633226678, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921292, 0.00411266813957582, 0.00290809553026393, 0.0352796762358899, 0.0191470007126724, 0.00788016479265674, 0.000951422114639394, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035127, 0.000776832903622087, -0.00180519653910857, -0.00173705117965753, 0.000672757029032357, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0100234421734311, 0.0603750614533914, -0.00610007264692977, -0.00431340273437462, -0.00564756986660027, -0.0218729440398859, 0.00126283501293979, -0.0178591840233903, 0.00103110047034114, 0.000729098134662865, 0.0417574386216002, 0.0168054651988621, -0.000867830491206149, -0.00526094276094275, 0.0145539597846298, -0.000751563251563241, -0.00455611007883227, -0.00061364882525233, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796929, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096062, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957582, 0.00290809553026392, -0.0705593524717798, -0.0127646671417816, 0.00450295131008959, 0.00380568845855755, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.0041106133464725, 0.00347410235931506, -0.000672757029032361, -0.00194208225905522, 0.00367664434877218, 0.00310733161448835, -0.000601732179702872, -0.00173705117965753, 0.00269102811612942, -0.000521115353897268, -0.00150433044925714, -0.000425488904726058, -0.00122828066840393, -0.000868525589828761},
    {0.010023442173431, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660033, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.016805465198862, 0.000867830491206148, -0.00526094276094277, 0.0145539597846298, 0.000751563251563248, -0.00455611007883229, 0.000613648825252327, -0.00372004830169691, -0.00263047138047139, 0.00557374224920012, 0.0196623144844763, -0.0103860554652283, -0.00321800158796929, 0.00650269929073345, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640432, 0.00411266813957582, 0.00290809553026394, 0.0705593524717798, -0.0127646671417816, -0.00450295131008959, 0.00380568845855756, 0.000736968401042376, -0.00212744452363028, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.00067275702903236, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702873, -0.00173705117965753, 0.00269102811612942, 0.000521115353897269, -0.00150433044925714, 0.000425488904726057, -0.00122828066840393, -0.000868525589828769},
    {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812425, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.0083846285486229, 0.00894802256177146, -0.0142846454190327, 0.014536160727703, -0.00939454064454064, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, -0.00245778931055954, 0.00830884437218264, -0.00965400476390776, 0.00650269929073342, -0.00219831358911339, 0.00743165633226678, -0.00863480436288184, 0.00581619106052784, 0.00643600317593851, -0.00747795993496438, 0.00503696921168108, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265674, 0.000951422114639386, 0.00221090520312707, -0.00212744452363026, 0.0174787403314969, -0.00719357335632687, 0.00086852558982877, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622091, 0.00180519653910857, -0.00173705117965753, 0.000672757029032361, 0.00156334606169178, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828767},
    {0.0787556170769579, 0.0679219441350652, -0.00435719474780694, -0.00308100195312473, 0.0705946233325038, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662861, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226678, 0, 0.00166176887443654, -0.0345935170706695, 0.0130864298861876, -0.00359783515120078, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598859, -0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573522, 0.00377344134083695, 0.00217859737390341, -0.021567013671873, 0, 0.0984282481794862, -0.0852413633734345, -0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681771, -0.000797153207506441, 0.00394570707070705, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293083, -0.039259289658563, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880135},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312476, 0, 0.0218729440398859, 0.0757701007763862, 0.0178591840233903, -0.0412440188136458, -0.00729098134662866, 0, 0, 0.0520698294723688, -0.00901875901875904, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575697, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515273, 0.00407048581280863, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723143, 0.00607967912880135},
    {-0.0214788046573521, 0.00377344134083705, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704829, 0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965863, 0.00650872868404608, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196713, -0.0263060558477138, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165479, -0.00431740218144093, -0.0605484764655282, 0.0185070066280911, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937412, 0, -0.0109364720199429, 0.00947126259704828, -0.00892959201169513, 0.123732056440937, -0.060150596109686, 0, 0, -0.0065087286840461, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.00056367243867244, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.018239037386404},
    {0.0787556170769579, 0.0301875307266957, 0.0174287789912279, 0.0585390371093695, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034115, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094274, -0.00455611007883225, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.078755617076958, -0.0679219441350653, -0.00435719474780698, -0.00308100195312474, 0.0705946233325038, 0.0328094160598287, 0.0126283501293977, 0.0267887760350854, 0.03608851646194, -0.0072909813466286, -0.0298267418725716, 0.0151249186789759, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646542, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120077, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, -0.00377344134083699, 0.00217859737390347, -0.021567013671873, 0, -0.0984282481794862, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.00138070985681772, -0.000797153207506455, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.0392592896585629, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.0030274066306456, -0.0164151336477636, -0.0108311792346515, 0.00680782247561686, -0.00393049813889257, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266957, 0.0610007264692975, -0.00308100195312469, 0, -0.0218729440398858, 0.0757701007763862, -0.0178591840233902, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723688, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135752, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723144, 0.00607967912880135},
    {-0.0214788046573521, -0.00377344134083684, -0.0196073763651313, 0.00924300585937422, 0, 0.0109364720199429, 0.00947126259704829, -0.102690308134494, -0.0695992817480272, -0.0601505961096861, 0, -0.00504163955965862, -0.00650872868404613, -0.00450937950937952, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196706, -0.0263060558477138, 0.000563672438672418, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.00249265331165479, -0.00431740218144094, 0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937409, 0, 0.0109364720199429, 0.00947126259704826, 0.00892959201169515, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404615, 0.00676406926406926, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005186, 0.000563672438672451, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769577, -0.0301875307266956, 0.0174287789912279, 0.0585390371093693, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233902, 0.0103110047034114, 0.0838462854862291, 0, 0, 0, 0, 0, -0.00526094276094273, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769579, -0.0377344134083696, -0.0566435317214905, -0.00308100195312474, 0.0423567739995023, 0.0546823600997145, 0.0378850503881931, 0.0446479600584756, 0.00515550235170573, -0.0072909813466286, 0.0417574386216002, -0.00504163955965861, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055953, -0.00519302773261415, 0.00402250198496158, 0.00464478520766673, -0.00219831358911334, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.0047768907440582, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863012, -0.0090822198919368, 0.00703505727761298, 0.00812338442598859, 0.00680782247561685, 0.00786099627778515, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.00435719474780697, -0.021567013671873, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984379, 0, 0.00159430641501296, 0.00394570707070708, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.00955378148811638, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778514, 0.00607967912880134},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.00308100195312471, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170573, -0.00729098134662862, -0.0417574386216002, -0.00504163955965863, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801535, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261415, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967796, 0.00427258096992148, 0.0367664434877216, -0.00776832903622087, -0.00601732179702859, 0.00694820471863013, -0.0090822198919368, -0.00703505727761299, 0.0081233844259886, -0.00680782247561686, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573521, -0.0188672067041848, 0.0065357921217105, 0.0092430058593742, 0.0141189246665008, 0, -0.0031570875323494, -0.111619900146189, -0.05413277469291, -0.0601505961096861, -0.00894802256177148, -0.00252081977982931, 0.00065087286840459, 0.00112734487734485, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672412, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.0074779599349644, 0.00321800158796925, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.0174787403314969, -0.00451299134777144, -0.00781673030845888, 0.0363288795677472, -0.00938007637015065, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041848, 0.00653579212171051, 0.00924300585937413, 0.0141189246665008, 0, -0.00315708753234943, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177146, -0.00252081977982931, -0.000650872868404611, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672444, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144091, -0.00747795993496439, 0.00321800158796926, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845889, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330016, 0, 0.00631417506469884, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.00679184789949393, 0, 0.00151870335961076, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233622, 0, -0.0329013451166065, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0217859737390348, -0.0523770332031202, 0.0423567739995024, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.0034713219648246, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920009, -0.00491557862111902, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097695, -0.000623163327913691, -0.00215870109072046, 0.0071233501724151, 0.00411266813957582, 0.00540074884191872, 0, -0.0510586685671265, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173252, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440983, 0.00367664434877217, -0.00155366580724417, 0.00240692871881143, 0.00347410235931507, 0.00100913554354853, 0.000260557676948634, 0.000902598269554291, -0.000850977809452114, -0.000491312267361575, -0.000868525589828765},
    {-0.0214788046573522, 0, -0.0217859737390348, 0.00308100195312478, -0.127070321998507, 0, 0.0473563129852413, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468975, 0.0509388592462044, 0, -0.00943765659186685, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533348, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072045, 0, 0.00411266813957582, 0.00540074884191873, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259878, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448834, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554287, 0, -0.000491312267361581, -0.000868525589828775},
    {0.0787556170769579, 0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.00921750786359891, 0.00789141414141415, -0.0034713219648246, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802472, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097694, 0.0006231633279137, -0.00215870109072046, -0.0071233501724151, 0.00411266813957582, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173253, 0.0143871467126537, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877216, -0.00155366580724417, -0.00240692871881143, 0.00347410235931506, 0.00100913554354854, -0.000260557676948635, 0.000902598269554284, 0.000850977809452113, -0.000491312267361575, -0.000868525589828761},
    {-0.0214788046573522, -0.0188672067041848, 0.0108929868695174, 0.00308100195312471, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.00894802256177148, -0.0277290175781224, -0.00585785581564149, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.0072702388256598, 0.0506835250105158, 0.0650269929073344, -0.00659494076734014, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287806, -0.0115130724838424, 0.00356167508620755, -0.00205633406978792, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440067, -0.00302740663064558, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554291, 0.000868525589828753, 0.00100913554354853, 0.000781673030845886, 0.00180519653910857, -0.00042548890472605, 0.00024565613368079, -0.000868525589828766},
    {-0.0214788046573522, 0.0188672067041848, 0.0108929868695174, 0.00308100195312472, 0.0141189246665008, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177147, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846297, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565982, -0.0506835250105158, 0.0650269929073343, 0.00659494076734016, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496158, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078957, -0.00427258096992148, -0.000902598269554294, 0.000868525589828762, 0.00100913554354854, -0.000781673030845887, 0.00180519653910857, 0.000425488904726058, 0.00024565613368079, -0.000868525589828761},
    {0.0787556170769579, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330016, 0, 0.0631417506469886, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.0067221860795448, 0, -0.0601250601250601, 0.000970263985641984, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.00287826812096061, 0, -0.00822533627915164, 0.00540074884191873, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.0182390373864041, 0, -0.00388416451811043, 0, -0.00466099742173253, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940572, 0, 0.000982624534723146, -0.00086852558982877},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.00976309302606911, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226674, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905522, -0.00902598269554285, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598857, 0.00340391123780843, 0.00196524906944629, 0.00607967912880134},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177147, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061668, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.036351194128299, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226681, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978424, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554288, -0.0191075629762328, -0.00908221989193679, 0.00234501909253768, -0.00812338442598858, -0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281178, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969697, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.0727023882565981, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985168, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701215, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.0060548132612912, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.10739402328676, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.61248608016091, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 0, 4.18330013267038, -0.591607978309963, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.05987111581756e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.74954541697351, 0, -1.67332005306816, -1.18321595661993, 7.74596669241484, 0, 0.346410161513773, 0, 0.282842712474621, 0.199999999999999, 0, 1.01607205572352e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.44948974278317, 0, 0, 0, 7.09929573971954, 0, -0.414039335605422, 0, 0, 0, 0, 0, 0, 0, 1.31603755427645e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0, 4.38178046004132, -0.774596669241483, 0, 0, 4.76235235991626, 0, -0.740656079818042, 0.130930734141596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.12132034355964, 0, 0, 0, 0, 0, 7.17137165600635, 0, -1.20615246152436e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.35849019742426e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681198, 0, 1.58113883008419, 2.45967477524977, 0, 0, 0, 0, 5.34522483824849, -1.20948631362952, 0, 0, 0, 0, 0, -1.32866943359025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, 0, 0, 3.83405790253617, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 1.14970665364125e-14, 0, 0, 1.00926211832331e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.80288457473997, 0, 0, 0, -2.10229466468021, 0, -1.71651640581399, 0, 0, 9.81070843517429, 0, 0.305838870920591, 0, 0, 0.264864231681987, 0, 0.216260739578393, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239552, 0, 1.77281052085584, -1.25356634105602, 2.39356776939084, 0, -2.78313372537744, 0, -0.524404424085074, 0.370809924354783, 0, 9.40174755792015, -1.14927687399652e-14, 0.840917865872085, -0.301561193523231, 0, 0.350641890608828, 0, 0.0660687472643403, -0.046717659215121, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.3433848874678e-14, 1.29756443747035, 0, 0, 0, 5.01426536422406, 0, -0.383824736789373, 0, 0, 0, 0, 8.20651806648289, 0, 0, -0.631738055305786, 0, 0.0483573714561588, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0, 4.06201920231798, -0.820651806648292, 0, 0, 5.88642506110457, 0, -1.20156148406979, 0.242752078113805, 0, 0, 0, 5.13809303146605, 0, 0, -0.74161984870956, 0, 0.151382517704866, -0.0305838870920676, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 0, -0.792824967172097, 2.24244764232554, 2.14087209644419, 0, 0.095742710775635, -1.39713895489839e-14, -1.87616630392937, -2.15580611373102, 0, 0, 0, 0, 9.44035859331469, 1.15714811919607e-14, 0.422185670932519, 1.95127251818837e-14, 0.590936840285278, 0.522319308773324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.16057691494799, 0, 0, 0, 1.96214168703485, 0, 2.74642624930237, 0, 0, 0, 0, 0, 0, 0, 8.6522315682783, 0, -0.865042958313572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04119910644746e-14, 0, 0, 0, 0, 0, 0, 1.5543528562087e-14, 0, 0, 0, 0},
    {1.32664991614216, 0, 2.07610309995255, 1.46802658041887, 0, 0, 1.31624465810881, 0, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 0, 5.80409338312195, 0, -1.54743588659734, 0.34193825277012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.00508909139073, 0, 0, 0, 0, 0, 4.45964044034302, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0, 0.749149177264393, 2.75458800949268, 0, 0, 0, 0, 3.3240197266047, 2.06838446274243, 0, 0, 0, 0, 0, 0, 0, 0, 6.28180682119209, -1.77676328054752, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964681, 0, 0, 3.11415464992886, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.80288457473997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 0, -2.20543546226846, -1.55947837083931, 8.33749895885385, 0, 1.06532653910462, 0, 0.869835476750494, 0.615066564126913, 0, -2.18326971917505, 0, -0.278967971730994, -1.89076704011891, 0, -0.241593350361257, 0, -0.197260144544842, -0.139483985865489, 11.8490505948789, 0, 0.245326690731332, 0, 0.0313466946954478, 0, 0.219426862868129, 0, 0.0280373360835799, 0.190029237516524, 0, 0.0242810453028227, 0, 0.0198253904711049, 0.0140186680417927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.45525332069938, 0, 0, 0, 5.28531208250978, 0, -1.36277028773849, 0, 0, 2.33666428910958, -1.04179543359539e-14, -3.93352933905728, 0, 0, -1.19859611757307, 0, 0.30904725218263, 0, 0, 0, 11.55277744383, 0, 0.810328731479531, 0, -0.234843597212094, 0, 0.395334573318417, 0, 0, 0.120463442338382, 0, -0.0310604175145514, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331886, 0, 2.99085908983323, -1.2440333788203, 0.95014618758261, 0, -0.169967317119764, 0, -1.1796107330308, 0.490653381462657, 0, 4.9761335152812, 0, -3.2268221135323, -0.21547290184284, 0, 0.0385449644663875, 0, 0.267510569439613, -0.11126972805284, 0, 0, 10.7636648722209, 0, 1.37533064807499, 0, -0.500120235663638, 0, 0.324307824721468, 0.0216558414515664, 0, -0.00387391468765637, 0, -0.0268858238268342, 0.0111830284386732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.772618130456579, 0, 0, 0, 3.3588901228148, 0, -0.304724700110023, 0, 0, 0, 0, 7.32969916135162, 0, 0, -0.761724681099362, 0, 0.0691050664139862, 0, 0, 0, 0, 0, 9.05975063854702, 0, 0, 0, -0.736662483163211, 0, 0, 0.0765562109321137, 0, -0.00694532049720535, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376113, 0, 3.68560354207934, -0.814411017927954, 0, 0, 6.00856531485322, 0, -1.45362164025439, 0.321208037219811, 0, 0, 0, 6.99294104721297, 0, 0, -1.362614533662, 0, 0.329650402328134, -0.0728431359084717, 0, 0, 0, 0, 5.40219154319687, 0, 0, 0, -0.702817019232737, 0, 0, 0.136947913395212, 0, -0.0331311120155176, 0.00732101061700613, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.1542003094028, 0, 0, 0, -0.761811750275056, 0, 5.59815020469135, 0, 0, 2.13307290077016, 0, 0.0664963811608019, 0, 0, -2.36108976914458, 0, -3.33842298506509, 0, 0, 0, 0, 0, 0, 0, 11.5766222110848, 0, 0.360889439280294, 0, 0, 0.555625639776453, 0, 0.595437196138644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 0, 0.963624111659426, 1.36277028773849, 0.867360833110884, 0, -1.00852844760553, -1.31315568597201e-14, 1.71026313764871, -2.01556443707465, 0, 2.04415543160775, -1.06351258615242e-14, 0.182834820066006, 2.68821895760394, 2.34958042241799e-14, -3.12574097035512, 0, -1.01990506872275, 0.904016610326379, 0, 0, 0, 0, 0, 0, 11.0940489487798, 0, 0.992281903807273, -0.632607619454903, 0, 0.735567892896446, 0, 0.181909068193175, -0.147004726489975, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24564920565478e-14, 0, 0, 1.009146310714e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.705300630647536, 0, 0, 0, 1.81702705031798, 0, 1.25178444059442, 0, 0, 0, 0, 1.78428514239953, 0, 0, 5.631527706439, 0, -0.746494073225355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.68367982311023, 0, 0, -1.32524448062076, 0, 0.133143804689787, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.29216299118995e-14, 0, 0},
    {1.1517511068998, 0, 2.20794021658195, 0.892142571199758, 0, 0, 2.13307290077015, 0, 3.91870514328394, -1.31949665727936, 0, 0, 0, 1.11713920350151, 0, 0, 6.6110513536048, 0, -2.33689608954558, 0.591817792331142, 0, 0, 0, 0, 0, 0, 0, 0, 6.06294257990893, 0, 0, -1.55575179137407, 0, 0.416806037297055, -0.0962371838080836, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.57359158493888, 0, -0.430945803685668, 3.04724700110021, 0.775791113542722, 0, 0.0346944333244404, -1.49568650774796e-14, -1.52970585407784, 0.901387818865989, 0, 0, 0, 0, 4.51561078268939, 0, 0.20194425340049, 0, -1.83730997289732, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, 0, 0.487107866980278, 0, 0.894874292847885, 0.903959543974648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.630840061880567, 0, 0, 0, 0.711024300256716, 0, 2.23926008187654, 0, 0, 0, 0, 0, 0, 0, 4.13862564412659, 0, 2.68954640486934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.98274066131697, 0, -1.30996183150502, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727542, 0, 1.12848100903608, 1.99489143482414, 0, 0, 0.47696960070847, 0, 4.00571020990215, 0.59009684435208, 0, 0, 0, 0, 0, 0, 2.77627448210728, 0, 4.8112068719424, -1.79915418525286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.69662601613678, 0, -2.34333096254029, 0.591780433634516, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.546323519313506, 0, 0, 0, 0, 0, 2.69341127303542, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.71889089790563, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059451, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199766, 0, 0.40720550896397, 2.53385243517954, 0, 0, 0, 0, 2.00755023262382, 3.29336152870136, 0, 0, 0, 0, 0, 0, 1.24513151875208e-14, 0, 5.00796559370719, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.728431359084685, 0, 0, 2.53908227033118, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.73861278752583, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867318, -0.381881307912986, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 3.62284418654736, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.3228756555323, 0, 4.83045891539648, 0.341565025531989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693486, 0, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62018517460197, 0, 0, 3.76497011940334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923788, -0.836660026534075, -0.591607978309962, 3.87298334620742, -0.599999999999998, 0.173205080756888, -0.489897948556638, 0.141421356237313, 0.0999999999999996, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386223, 1.22474487139159, 1.41421356237309, -0.500000000000003, 4.58257569495584, 3.54964786985977, -1.02469507659596, -0.207019667802713, -0.239045721866875, 0.0845154254728531, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000003, -2.84604989415154, 2.19089023002066, -0.387298334620745, 0, 5.49909083394701, 2.38117617995813, 0.481070235442362, -0.370328039909021, 0.0654653670707988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.59807621135333, 0, -1.5811388300842, -1.1180339887499, 0, 0, 6.87386354243376, 0, 0.267261241912429, 0.188982236504611, 0, 0, 0, -1.56646390951001e-14, 0, 0, 1.01970842391297e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.0062305898749, 1.06066017177982, -0.204124145231929, 1.58771324027146, 0, 0, 0, 3.58568582800318, -0.690065559342348, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.77942286340599, 1.36930639376292, 0.790569415042096, 1.22983738762489, 0, 0, 0, 4.62910049886275, 2.67261241912425, -0.604743156814767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.900000000000007, 0, 1.82574185835055, -1.42009389360937, 0, 0, 0, 0, 6.17213399848367, 0.698297248755177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067893, 0, 0, 1.91702895126809, 0, 0, 0, 0, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.10227038425243, 0, 0, 3.3203915431768, 0, 0, 0, 0, 0, 4.48998886412873, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219015, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849178, 0.152919435460295, -0.0529728463363973, -0.512907379155188, 0.132432115840996, -0.0458758306380885, 0.1081303697892, -0.0374574588632162, -0.0264864231681936, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114625, -0.262202212042537, 0.185404962177395, 5.56214886532174, 4.70087377896007, -1.21376039056904, 0.420458932936042, -0.150780596761618, -0.397100103328477, 0.175320945304415, 0.0572172135271331, 0.0330343736321715, -0.0233588296075578, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735171, 1.87287294316099, -0.529728463363976, -4.04586796856344, 2.50713268211204, 0.904683580569682, -0.191912368394685, -0.554003287767448, 0.156695792632001, 0, 6.35674156036771, 4.10325903324145, -1.42141062443803, 0.509731451534329, -0.315869027652891, -0.113979417590034, 0.0241786857280778, 0.0697978535688028, -0.0197418142283115, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709575, 3.51781181986757, 2.03100960115899, -0.410325903324144, 0, -3.39852909359329, 2.94321253055229, -1.04058276941337, -0.600780742034897, 0.121376039056905, 0, 0, 7.41619848709566, 2.56904651573303, 0, 0.428174419288832, -0.370809924354779, 0.131101106021265, 0.0756912588524378, -0.0152919435460309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.71269767715534, 0, 4.69041575982343, 0.94760708295868, 0, 0, -1.69926454679664, 0, -1.38744369255116, -0.280305955290692, 0, 0, 0, 8.8994381845148, 0, 0, 0.214087209644411, 0, 0.17480147469502, 0.0353152308909268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586045, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878174, 3.24961536185438, -0.938083151964683, -1.07790305686552, 0, 0, 0, 0, 4.72017929665735, -0.731247032282672, 0.211092835466259, -1.02353263143832, 0.295468420142642, 0.26115965438666, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577669, 0.580288457473998, 0.670059394260483, 0.947607082958683, 1.26655701279755, 0.981070843517428, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694758, 0, 0, 0, 0, 5.58499146191198, 4.32611578413915, -1.24884205625912, -0.432521479156774, -0.499432784842937, 0.220720193068322, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071078, -1.34846851907591, 1.03805154997627, 0.73401329020944, 0, 1.51986841535706, 0.658122329054406, -3.19105893754051, 2.45647831545196, -0.705653192844341, 0, 0, 0, 0, 0, 6.70198975429437, 2.90204669156098, 1.00508909139074, -0.773717943298673, 0.170969126385064, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 0, -0.749149177264396, 2.11891385345588, 0, 0, 1.89983551919633, 0, -1.77281052085583, -2.03704530421603, 0, 0, 0, 0, 0, 0, 8.37748719286796, 0, 0.558382828550407, 0.493545355707621, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.22447624956168e-14},
    {0.741619848709566, 0.502544545695365, -0.096714742912331, 1.77807891440463, 0, 0, 0, 2.22982022017151, -0.429129101453498, 1.33513642962593, 0, 0, 0, 0, 0, 0, 0, 4.21396412211221, -0.81097777341897, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5744562646538, 0.648782218735174, 0.374574588632197, 1.37729400474634, 0, 0, 0, 2.87868552592027, 1.66200986330235, 1.03419223137121, 0, 0, 0, 0, 0, 0, 0, 5.44020428881872, 3.14090341059604, -0.888381640273766, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.663324958071084, 0, 0.865042958313566, -1.59036212878712, 0, 0, 0, 0, 3.83824736789369, -1.19418232635198, 0, 0, 0, 0, 0, 0, 0, 0, 7.25360571842496, 1.02581475831036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.46904157598234, 0, 0, 1.55707732496443, 0, 0, 0, 0, 0, 2.76353810488345, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.90144228736999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.812403840463591, 0, 0, 2.69693703815183, 0, 0, 0, 0, 0, 4.78658840631075, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.02544545695368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552312, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541881, -0.139483985865496, -0.945383520059461, 0.313838968197358, -0.120796675180629, 0.25624844449503, -0.0986300722724195, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963984, 0.0156733473477253, -0.519258730913241, 0.109713431434065, -0.0364215679542342, 0.01401866804179, 0.0950146187582634, -0.0315420030940289, 0.0121405226514124, -0.0257539376818875, 0.00991269523555186, 0.00700933402089779, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706032, -0.705300630647551, 1.61604676382125, 2.64265604125489, -1.44543616748915, -0.681385143869243, -0.196698948117362, 0.278174320132098, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444731, -0.366484958067585, -0.599298058786534, 0.327794111588103, 0.15452362609131, 0.0446071285599885, -0.0630840061880518, 6.54980915752513, 5.77638872191499, -1.22048487632307, 0.405164365739766, -0.155947837083929, -0.117421798606046, -0.453666435153256, 0.197667286659209, -0.0824223552841552, 0.0368331241581584, 0.0602317211691907, -0.0329445477765348, -0.0155302087572761, -0.00448318510329359, 0.00634018117570608, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659432, 0.914174100330065, 1.49542954491661, -0.622016689410148, 0.475073093791304, 2.87032228155655, -0.0849836585598799, -0.360555127546397, -0.589805366515403, 0.245326690731328, -5.15186050535791, 2.48806675764059, 2.8908723349783, -1.61341105676615, -0.107736450921414, -0.650927909109099, 0.019272482233193, 0.0817662172643042, 0.133755284719809, -0.0556348640264175, 0, 7.27756573058348, 5.38183243611043, -1.7866069197544, 0.687665324037496, 0.517781462682516, -0.250060117831819, -0.290543601573986, 0.162153912360733, 0.0108279207257825, 0.0654207163662268, -0.00193695734382766, -0.00821781403613334, -0.0134429119134184, 0.00559151421933731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927964, 0.386309065228293, 1.93297557093283, -0.525700051567136, 4.5772145303823, 1.6794450614074, 1.93925611658549, -0.152362350055012, -0.762375846428075, 0.207338896470047, 0, -4.7313004641042, 3.66484958067581, 0.282120252259006, -1.03801468669568, -0.38086234054968, -0.439781949681085, 0.0345525332069918, 0.172890591018093, -0.0470200420431731, 0, 0, 8.18726144690641, 4.52987531927351, -1.74354982331868, 0, 0.475513588177819, -0.368331241581605, -0.0283541521970749, 0.10432440129236, 0.0382781054660568, 0.0441997489897906, -0.00347266024860305, -0.0173761581875701, 0.00472569203284699, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880556, -3.39131543920102, 1.84280177103967, -0.407205508963983, 0, 5.01084538062905, 3.00428265742661, 1.33755284719805, -0.726810820127196, 0.160604018609904, 0, 0, -3.78504037128335, 3.49647052360648, 0, -1.13635292017211, -0.681307266831003, -0.303328474207914, 0.164825201164063, -0.0364215679542416, 0, 0, 0, 9.3568702250359, 2.70109577159844, 0, 0, 0.380410870542252, -0.351408509616368, 0, 0.11420776562532, 0.0684739566976048, 0.0304856587023835, -0.0165655560077574, 0.00366050530850137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54950975679639, 0, -2.32737334062814, -1.64570147151095, 0, 0, 6.74536878161603, 0, 0.917928424547679, 0.649073413641553, 0, 0, 0, -1.76635217326556, 0, 0, -1.52970585407782, 0, -0.208166599946601, -0.147196014438789, 0, 0, 0, 0, 10.9163485958752, 0, 0, 0, 0.177525072919717, 0, 0, 0.153741222957161, 0, 0.020921530482028, 0.0147937560766434, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412066, 1.5771001547014, -0.546323519313517, 0.772618130456561, 1.47524211088021, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292753, -1.14271762541258, 1.06653645038508, -0.12876968840943, 0.0332481905804026, -0.011517511068999, 4.57223067739883, -1.18054488457228, 0.408952744138947, -1.66921149253253, 0.578231822728847, 0.512529242570405, 0, 0, 0, 0, 0, 5.78831110554239, -0.698859394077356, 0.180444719640146, -0.062507884474852, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069323, -0.10313274764285, -0.0833438459664692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186249, 0.834522960396274, 0.481812055829709, 0.681385143869246, 0.433680416555438, 1.14215318100896, -0.504264223802767, 1.48113132435986, 0.855131568824349, -1.00778221853732, 1.20933866224478, 1.02207771580387, -0.263899331455877, 0.0914174100330005, 1.34410947880197, 3.53988526627811, -1.56287048517756, -0.88326369896241, -0.509952534361379, 0.452008305163194, 0, 0, 0, 0, 0, 6.56332786985937, 5.54702447438989, -1.4322355606878, 0.496140951903638, -0.316303809727451, -0.833026783443143, 0.367783946448223, 0.15753787423404, 0.0909545340965868, -0.0735023632449856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.14091633431099e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.743452142666481, 0.35265031532376, 1.01801377240995, 0.575875553449897, -1.46610739697708, 0.908513525158993, 0.327831580195603, 0.62589222029721, 1.80679520936143, -0.851731429836555, 0, 1.38210132827975, 0.892142571199764, -0.309047252182628, -4.54391937932164, 2.8157638532195, 1.01605126164418, -0.373247036612687, -1.07747138531276, 0.382016742281294, 0, 0, 0, 0, 0, 0, 7.50094613698214, 4.84183991155512, -1.67726254578565, 1.06930204231198, -0.662622240310384, -0.239103205508895, 0.066571902344891, 0.192176528696449, -0.0621208350291014, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.10027752117729e-14, 0, 0},
    {0.575875553449902, 1.91213231759728, 1.10397010829098, 0.44607128559988, 0, -1.23153021346074, 1.06653645038507, 3.39369820402464, 1.95935257164197, -0.659748328639681, 0, 0, 1.6124515496597, 0.558569601750758, 0, -3.81689227863019, 3.3055256768024, -2.02381137955099, -1.16844804477279, 0.295908896165575, 0, 0, 0, 0, 0, 0, 0, 8.75110382647916, 3.03147128995446, 0, 0.898213715542065, -0.777875895687036, 0.360964616749972, 0.208403018648526, -0.0481185919040409, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.32945523855878e-14, 0, 0},
    {-1.32992762321609, 0, 2.54950975679639, -1.03015750727543, 0, 0, -0.615765106730374, 0, 4.52493093869951, 1.52362350055012, 0, 0, 0, 1.93494185959165, 0, 0, -1.9084461393151, 0, -2.69841517273464, -0.683372323427214, 0, 0, 0, 0, 0, 0, 0, 0, 10.501324591775, 0, 0, 0.449106857771034, 0, 0.481286155666633, 0.111125127955289, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.786795792469438, 0.746420027292178, -0.215472901842835, 1.5236235005501, 0.387895556771362, -0.0600925212577327, 0.0173472166622199, 2.64952825989834, -0.764852927038915, 0.45069390943299, 0, 0, 0, 0, 2.25780539134469, -0.349777707186207, 0.100972126700243, 3.18231422231115, -0.918654986448657, -1.37412670680299, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.44603151471411, -0.843695574376338, 0.243553933490139, -1.54996774159981, 0.447437146423943, 0.451979771987323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608038, 0.315420030940283, 0.364215679542343, 1.28769688409429, 0.458964212273842, 0.355512150128361, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137522, 0, 0, 0, 0, 2.67147136598154, 2.06931282206329, -0.597359157427889, 1.34477320243467, 1.5528103408493, -1.16134903279062, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.4438313884647, 4.99137033065849, -1.44088450201539, -0.654980915752514, -0.756306816047561, 0.381992627346313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.51507875363771, -0.732969916135158, 0.56424050451804, 0.997445717412072, 0, 0.550757054728608, 0.238484800354237, -2.60178510148047, 2.00285510495107, 0.295048422176044, 0, 0, 0, 0, 0, 3.20576563917784, 1.38813724105364, -3.12497053047329, 2.4056034359712, -0.899577092626427, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.73259766615765, 3.34831300806839, 1.5220381072759, -1.17166548127015, 0.295890216817259, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48690428533295, 0, -0.40720550896398, 2.87937776724947, 0, 0, 0.688446318410768, 0, -1.44543616748914, 0.851731429836557, 0, 0, 0, 0, 0, 0, 4.00720704897231, 0, -1.73609473915183, -2.59685538292347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.66574708269706, 0, 0.845576726264391, 0.854161481650102, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449896, 0.273161759656754, -0.0525700051567131, 1.63559471386625, 0, 0, 0, 1.34670563651772, -0.259173620587566, 2.12585572561676, 0, 0, 0, 0, 0, 0, 0, 3.35944544895281, -0.646525578093589, 0.914325240961068, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4.72691760029726, -0.909695716322936, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599885, 0.35265031532377, 0.203602754481986, 1.26692621758977, 0, 0, 0, 1.73858950082559, 1.0037751163119, 1.64668076435068, 0, 0, 0, 0, 0, 0, 0, 4.33702542542885, 2.50398279685359, 0.708233286251854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.10242438161534, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.515078753637711, 0, 0.470200420431694, -1.46292038553772, 0, 0, 0, 0, 2.31811933443412, -1.90142316513449, 0, 0, 0, 0, 0, 0, 0, 0, 5.78270056723845, -0.817797356933148, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.13656584215379, 1.31506763029891, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.12763531151527e-14},
    {0.364215679542343, 0, 0, 1.26954113516559, 0, 0, 0, 0, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.46962034034308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880561, 0, 0, 2.19890974840547, 0, 0, 0, 0, 0, 4.33630850246743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.00955871244863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.4358503831282, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883008419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.912870929175276, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.58198889747161, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.4790199457749, 2.80624304008045, -0.540061724867317, -0.381881307912987, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14564392373896, 0.724568837309471, 2.09165006633519, -0.295803989154981, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.661437827766144, 0, 1.93218356615859, -0.170782512765995, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.935414346693487, 3.54964786985977, 0, 2.17370651192841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.540061724867317, 0, 3.54964786985977, 1.25499003980111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.90940653956493, 0, 0, 4.43705983732471, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.37477270848675, 2.89827534923789, -0.836660026534075, -0.591607978309964, 3.87298334620742, -0.600000000000001, 0.173205080756889, -0.48989794855664, 0.141421356237313, 0.0999999999999997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.16189500386222, 1.22474487139158, 1.41421356237309, -0.500000000000002, 0.654653670707977, 3.54964786985977, -1.02469507659596, -0.20701966780271, -0.239045721866875, 0.0845154254728546, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.900000000000001, 0.316227766016839, 2.19089023002066, -0.38729833462074, 0, 1.57116880969914, 2.38117617995813, -0.0534522483824853, -0.37032803990902, 0.0654653670707977, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.519615242270665, 0, 1.58113883008419, -0.223606797749979, 0, 0, 2.94594151818589, 0, -0.267261241912424, 0.0377964473009217, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.00623058987491, 1.06066017177982, -0.204124145231932, 1.58771324027146, 4.53557367611073, 0, 0, 3.58568582800318, -0.690065559342349, -0.780720058358831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.779422863405996, 0.273861278752581, 0.790569415042093, 1.22983738762489, 0, 4.53557367611073, 0, 0.925820099772549, 2.67261241912425, -0.604743156814761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.450000000000004, 0, 0.730296743340221, 0.710046946804697, 0, 0, 4.53557367611072, 0, 2.46885359939347, -0.349148624377589, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.636396103067889, -3.1304951684997, 0, 1.91702895126808, 0, 0, 0, 5.29150262212918, 0, 2.59229627936314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.367423461417479, 0, -3.1304951684997, 1.10679718105894, 0, 0, 0, 0, 5.29150262212918, 1.49666295470958, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.85788383248864, 0, 0, -2.34787137637477, 0, 0, 0, 0, 0, 6.34980314655501, 0, 0, 0, 0, 0, 0, 1.11852851939587e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.28452325786651, 2.90144228736999, -1.00508909139074, -0.710705312219016, 4.07107611256358, -1.0511473323401, 0.364128117170708, -0.858258202906995, 0.297309362689537, 0.210229466468024, 4.90535421758715, -0.592254426849176, 0.152919435460295, -0.0529728463363977, -0.512907379155187, 0.132432115840995, -0.0458758306380897, 0.1081303697892, -0.0374574588632173, -0.0264864231681947, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.13284303119776, 1.53529894715747, 0.886405260427918, -0.626783170528011, 1.19678388469542, 3.15188435701566, -1.39156686268872, -0.454147553114629, -0.262202212042535, 0.185404962177393, 0.618016540591305, 4.70087377896007, -1.21376039056904, 0.42045893293604, -0.150780596761618, -0.397100103328475, 0.175320945304413, 0.057217213527133, 0.0330343736321713, -0.0233588296075595, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.95742710775634, 0.648782218735172, 1.87287294316099, -0.529728463363977, 0.20229339842817, 2.50713268211204, 0.904683580569681, -0.191912368394685, -0.554003287767448, 0.156695792632003, 0, 1.41260923563727, 4.10325903324145, -1.42141062443803, -0.0254865725767146, -0.315869027652895, -0.113979417590039, 0.0241786857280816, 0.0697978535688034, -0.0197418142283076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709574, 0.167514848565124, 2.03100960115899, -0.410325903324145, 0, 0.849632273398319, 2.94321253055228, -0.0495515604482551, -0.600780742034895, 0.121376039056903, 0, 0, 2.47206616236523, 2.56904651573303, 0, -0.10704360482221, -0.370809924354777, 0.00624290981053448, 0.0756912588524339, -0.015291943546034, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.428174419288845, 0, 1.34011878852098, -0.236901770739674, 0, 0, 2.54889682019496, 0, -0.396412483586046, 0.0700764888226733, 0, 0, 0, 3.95530585978436, 0, 0, -0.321130814466627, 0, 0.0499432784842904, -0.00882880772273737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561023804, 1.37321312465119, -0.396412483586047, 1.12122382116277, 1.07043604822209, -0.165831239517766, 0.0478713553878164, 3.24961536185438, -0.938083151964685, -1.07790305686551, 5.52770798392566, 0, 0, 0, 4.72017929665734, -0.731247032282673, 0.211092835466259, -1.02353263143831, 0.295468420142642, 0.261159654386662, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577672, 0.580288457473996, 0.670059394260487, 0.947607082958685, 0.180936716113935, 0.981070843517426, -0.283210757799439, 1.37321312465118, 1.58564993434418, -0.910994354694756, 0, 5.52770798392567, 0, 0, 0.797855923130281, 4.32611578413915, -1.24884205625911, -0.432521479156781, -0.499432784842937, 0.220720193068326, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071081, 0.149829835452877, 1.03805154997628, 0.734013290209435, 0, 0.434248118673443, 0.658122329054405, 0.354562104171167, 2.45647831545196, -0.705653192844349, 0, 0, 5.52770798392567, 0, 0, 1.91485421551268, 2.90204669156097, -0.111676565710086, -0.773717943298669, 0.170969126385058, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.382970843102543, 0, 0.749149177264393, 0.423782770691177, 0, 0, 0.814215222512714, 0, 1.77281052085584, -0.407409060843206, 0, 0, 0, 5.52770798392568, 0, 0, 3.59035165408627, 0, -0.558382828550415, 0.0987090711415228, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.741619848709561, 0.502544545695366, -0.096714742912333, 1.77807891440463, -4.23078640106406, 0, 0, 2.2298202201715, -0.429129101453499, 1.33513642962593, 0, 0, 0, 0, 6.21867148191638, 0, 0, 4.21396412211221, -0.810977773418968, -1.14689576595224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.574456264653794, 0.129756443747034, 0.374574588632199, 1.37729400474634, 0, -4.23078640106406, 0, 0.575737105184058, 1.66200986330234, 1.03419223137121, 0, 0, 0, 0, 0, 6.21867148191637, 0, 1.08804085776374, 3.14090341059604, -0.88838164027376, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.331662479035532, 0, 0.346017183325429, 0.795181064393553, 0, 0, -4.23078640106406, 0, 1.53529894715748, 0.597091163175992, 0, 0, 0, 0, 0, 0, 6.21867148191637, 0, 2.90144228736999, -0.512907379155182, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.469041575982343, 3.81404493622062, 0, 1.55707732496444, 0, 0, 0, -3.76069902316805, 0, 2.76353810488346, 0, 0, 0, 0, 0, 0, 1.02342965294379e-14, 7.10705312219014, 0, 2.90144228737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.270801280154526, 0, 3.81404493622062, 0.89897901271727, 0, 0, 0, 0, -3.76069902316805, 1.59552946877025, 0, 0, 0, 0, 0, 0, 0, 0, 7.10705312219014, 1.67514848565122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.68079590171775, 0, 0, 4.4497190922574, 0, 0, 0, 0, 0, -2.63248931621764, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.2915619758885, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.20796675180627, 2.86494470509734, -1.10271773113423, -0.779739185419657, 4.16874947942693, -1.38389976928554, 0.532663269552311, -1.12994942996831, 0.434917738375247, 0.307533282063459, 5.16655913866601, -1.09163485958752, 0.362390025541882, -0.139483985865495, -0.945383520059461, 0.313838968197358, -0.120796675180628, 0.256248444495029, -0.0986300722724191, -0.0697419929327415, 5.92452529743945, -0.580548910116139, 0.122663345365667, -0.0407205508963982, 0.0156733473477249, -0.519258730913241, 0.109713431434065, -0.0364215679542341, 0.0140186680417898, 0.0950146187582634, -0.0315420030940288, 0.0121405226514124, -0.0257539376818875, 0.00991269523555182, 0.00700933402089769, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.09264703862702, 1.72762666034969, 0.498722858706025, -0.705300630647557, 1.61604676382126, 2.64265604125489, -1.44543616748915, -0.681385143869247, -0.196698948117359, 0.278174320132096, 1.16833214455479, 4.51392403614434, -1.96676466952864, 0.820092080444727, -0.366484958067587, -0.599298058786529, 0.327794111588101, 0.154523626091317, 0.0446071285599885, -0.0630840061880538, 0.595437196138649, 5.77638872191499, -1.22048487632307, 0.405164365739765, -0.155947837083928, -0.117421798606047, -0.453666435153255, 0.197667286659209, -0.0824223552841549, 0.0368331241581608, 0.0602317211691907, -0.0329445477765342, -0.0155302087572759, -0.00448318510329279, 0.00634018117570581, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.963624111659437, 0.914174100330057, 1.49542954491662, -0.622016689410149, 0.475073093791305, 2.87032228155655, -0.084983658559881, -0.3605551275464, -0.589805366515401, 0.24532669073133, 0.147196014438797, 2.4880667576406, 2.89087233497828, -1.61341105676615, -0.107736450921419, -0.650927909109092, 0.0192724822331939, 0.0817662172643072, 0.133755284719809, -0.0556348640264189, 0, 1.32319376919699, 5.38183243611043, -1.7866069197544, 0.687665324037495, -0.0147937560766431, -0.25006011783182, -0.290543601573986, 0.162153912360733, 0.0108279207257832, 0.0654207163662248, -0.00193695734382736, -0.00821781403613383, -0.0134429119134179, 0.00559151421933732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.814411017927959, 0.386309065228289, 1.93297557093283, -0.525700051567137, 0.0803020093049556, 1.6794450614074, 1.9392561165855, -0.152362350055011, -0.762375846428077, 0.207338896470047, 0, 0.567756055692504, 3.66484958067581, 0.282120252259013, -0.0182107839771216, -0.38086234054968, -0.439781949681088, 0.0345525332069925, 0.172890591018093, -0.0470200420431761, 0, 0, 2.23288948551993, 4.52987531927351, -1.74354982331867, 0, -0.0570616305813383, -0.368331241581605, -0.0283541521970753, 0.0018302526542526, 0.0382781054660563, 0.0441997489897914, -0.00347266024860275, -0.0173761581875703, 0.00472569203284603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.630840061880563, 0.09974457174121, 1.84280177103967, -0.407205508963979, 0, 0.513932859551699, 3.00428265742661, -0.03933978962347, -0.726810820127196, 0.160604018609906, 0, 0, 1.51401614851335, 3.49647052360648, 0, -0.116549017453554, -0.681307266831002, 0.00892142571199782, 0.164825201164067, -0.036421567954237, 0, 0, 0, 3.40249826364942, 2.70109577159844, 0, 0, -0.152164348216903, -0.351408509616369, 0, 0.0117136169872135, 0.0684739566976059, -0.000896637020657868, -0.0165655560077585, 0.00366050530850301, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, 0, 1.16368667031409, -0.23510021021585, 0, 0, 2.24845626053868, 0, -0.458964212273845, 0.0927247733773657, 0, 0, 0, 3.53270434653115, 0, 0, -0.509901951359277, 0, 0.104083299973309, -0.0210280020626858, 0, 0, 0, 0, 4.96197663448874, 0, 0, 0, -0.355050145839439, 0, 0, 0.0512470743190546, 0, -0.010460765241014, 0.00211339372523455, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.997445717412067, 1.5771001547014, -0.546323519313514, 0.772618130456565, 1.4752421108802, -0.380905875137528, 0.131949665727938, 2.79907510234567, -0.969628058292756, -1.14271762541258, 1.06653645038508, -0.128769688409428, 0.0332481905804048, -0.0115175110689984, 4.57223067739883, -1.18054488457228, 0.408952744138948, -1.66921149253254, 0.578231822728845, 0.512529242570406, 6.52268767805531, 0, 0, 0, 0, 5.7883111055424, -0.698859394077357, 0.180444719640146, -0.0625078844748521, -1.07596442479003, 0.277812819888226, -0.0962371838080782, 0.297718598069321, -0.103132747642851, -0.0833438459664682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.879664438186242, 0.834522960396282, 0.481812055829709, 0.681385143869242, 0.433680416555441, 1.14215318100896, -0.504264223802765, 1.48113132435985, 0.855131568824352, -1.00778221853732, 0.134370962471638, 1.02207771580387, -0.263899331455871, 0.0914174100330022, 1.34410947880196, 3.5398852662781, -1.56287048517756, -0.883263698962406, -0.509952534361376, 0.452008305163191, 0, 6.52268767805531, 0, 0, 0, 0.729258652206596, 5.5470244743899, -1.4322355606878, 0.496140951903638, -0.316303809727452, -0.833026783443137, 0.367783946448223, 0.157537874234044, 0.0909545340965877, -0.0735023632449854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01901886515007e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.74345214266648, 0.352650315323769, 1.01801377240995, 0.5758755534499, 0.0733053698488504, 0.908513525158993, 0.327831580195604, 0.62589222029721, 1.80679520936143, -0.851731429836554, 0, 0.307133628506613, 0.892142571199772, -0.309047252182627, 0.227195968966083, 2.8157638532195, 1.01605126164419, -0.37324703661268, -1.07747138531275, 0.382016742281307, 0, 0, 6.52268767805531, 0, 0, 0, 1.66687691932936, 4.84183991155511, -1.67726254578565, -0.0534651021155991, -0.662622240310383, -0.239103205508893, 0.0665719023448925, 0.192176528696449, -0.0621208350290961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449893, 0.0910539198855843, 1.10397010829098, 0.446071285599878, 0, 0.307882553365187, 1.06653645038508, 0.161604676382129, 1.95935257164197, -0.65974832863968, 0, 0, 0.537483849886575, 0.558569601750753, 0, 0.954223069657542, 3.3055256768024, -0.0963719704548154, -1.16844804477279, 0.295908896165572, 0, 0, 0, 6.52268767805531, 0, 0, 0, 2.91703460882639, 3.03147128995447, 0, -0.224553428885516, -0.777875895687036, 0.0171887912738079, 0.208403018648526, -0.0481185919040412, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.332481905804022, 0, 0.728431359084682, 0.257539376818849, 0, 0, 0.923647660095564, 0, 1.292837411057, -0.380905875137523, 0, 0, 0, 0.859974159818509, 0, 0, 2.86266920897263, 0, -0.770975763638467, 0.170843080856795, 0, 0, 0, 0, 6.52268767805531, 0, 0, 0, 4.66725537412222, 0, 0, -0.673660286656546, 0, 0.137510330190468, -0.0277812819888241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.78679579246944, 0.746420027292177, -0.215472901842835, 1.52362350055011, 0.387895556771359, -0.0600925212577337, 0.0173472166622209, 2.64952825989835, -0.764852927038921, 0.450693909432998, -5.28814187068052, 0, 0, 0, 2.2578053913447, -0.349777707186206, 0.100972126700245, 3.18231422231115, -0.91865498644866, -1.37412670680298, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0, 5.44603151471411, -0.843695574376342, 0.243553933490139, -1.54996774159981, 0.447437146423942, 0.451979771987325, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.664963811608043, 0.315420030940282, 0.364215679542341, 1.28769688409429, 0.0655663160391197, 0.355512150128356, -0.102627517788397, 1.11963004093826, 1.292837411057, 0.380905875137518, 0, -5.28814187068053, 0, 0, 0.381638766568791, 2.0693128220633, -0.597359157427881, 1.34477320243467, 1.5528103408493, -1.16134903279063, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 0.920547341209241, 4.99137033065848, -1.4408845020154, -0.654980915752512, -0.756306816047561, 0.381992627346309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637708, 0.0814411017927942, 0.564240504518039, 0.997445717412071, 0, 0.157359158493888, 0.238484800354236, 0.289087233497833, 2.00285510495107, 0.295048422176043, 0, 0, -5.28814187068052, 0, 0, 0.915933039765097, 1.38813724105364, 0.347218947830363, 2.4056034359712, -0.89957709262643, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 2.20931361890218, 3.34831300806839, -0.16911534525288, -1.17166548127015, 0.295890216817257, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.297380857066588, 0, 0.407205508963972, 0.575875553449888, 0, 0, 0.295048422176039, 0, 1.44543616748915, 0.17034628596731, 0, 0, 0, -5.28814187068053, 0, 0, 1.71737444955956, 0, 1.73609473915183, -0.519371076584704, 0, 0, 0, 0, 0, 0, 0, 0, 7.17495644586084, 0, 0, 4.14246303544159, 0, -0.845576726264386, 0.170832296330017, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.575875553449895, 0.27316175965675, -0.0525700051567132, 1.63559471386626, 4.82647810240717, 0, 0, 1.34670563651771, -0.259173620587563, 2.12585572561677, 0, 0, 0, 0, -4.95763300376299, 0, 0, 3.35944544895281, -0.646525578093588, 0.914325240961076, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 0, 4.72691760029725, -0.909695716322934, -1.47029030817897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.446071285599889, 0.0705300630647548, 0.203602754481986, 1.26692621758977, 0, 4.82647810240717, 0, 0.34771790016512, 1.00377511631191, 1.64668076435067, 0, 0, 0, 0, 0, -4.95763300376299, 1.29915143264338e-14, 0.867405085085771, 2.50398279685359, 0.708233286251852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873427, 0, 1.22048487632307, 3.52323635943495, -1.13888197553347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.257539376818862, 0, 0.188080168172671, 0.731460192768861, 0, 0, 4.82647810240716, 0, 0.927247733773645, 0.950711582567244, 0, 0, 0, -1.22220110126485e-14, 0, 0, -4.95763300376297, 0, 2.31308022689538, 0.408898678466573, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.97217382873426, 0, 3.25462633686152, -0.657533815149454, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.04894165366026e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.364215679542344, -3.97354131880428, 0, 1.26954113516558, 0, 0, 0, 5.11038857901935, 0, 2.50356888118884, 0, 0, 0, 0, 0, 0, 0, -4.24939971751113, 0, 3.46962034034307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 0, 3.1383896819736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.210280020626859, 0, -3.97354131880429, 0.732969916135155, 0, 0, 0, 0, 5.11038857901935, 1.44543616748915, 0, 0, 0, 0, 0, 0, 0, 0, -4.24939971751114, 2.00318623748287, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.96869555732605, 1.8119501277094, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.2711894277325, 0, 0, -3.4552533206994, 0, 0, 0, 0, 0, 6.13246629482322, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.2499377798012, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[56][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0.8, 0.2}, {0, 0.6, 0.4}, {0, 0.4, 0.6}, {0, 0.2, 0.8}, {0.8, 0, 0.2}, {0.6, 0, 0.4}, {0.4, 0, 0.6}, {0.2, 0, 0.8}, {0.8, 0.2, 0}, {0.6, 0.4, 0}, {0.4, 0.6, 0}, {0.2, 0.8, 0}, {0, 0, 0.2}, {0, 0, 0.4}, {0, 0, 0.6}, {0, 0, 0.8}, {0, 0.2, 0}, {0, 0.4, 0}, {0, 0.6, 0}, {0, 0.8, 0}, {0.2, 0, 0}, {0.4, 0, 0}, {0.6, 0, 0}, {0.8, 0, 0}, {0.6, 0.2, 0.2}, {0.4, 0.4, 0.2}, {0.2, 0.6, 0.2}, {0.4, 0.2, 0.4}, {0.2, 0.4, 0.4}, {0.2, 0.2, 0.6}, {0, 0.2, 0.2}, {0, 0.4, 0.2}, {0, 0.6, 0.2}, {0, 0.2, 0.4}, {0, 0.4, 0.4}, {0, 0.2, 0.6}, {0.2, 0, 0.2}, {0.4, 0, 0.2}, {0.6, 0, 0.2}, {0.2, 0, 0.4}, {0.4, 0, 0.4}, {0.2, 0, 0.6}, {0.2, 0.2, 0}, {0.4, 0.2, 0}, {0.6, 0.2, 0}, {0.2, 0.4, 0}, {0.4, 0.4, 0}, {0.2, 0.6, 0}, {0.2, 0.2, 0.2}, {0.4, 0.2, 0.2}, {0.2, 0.4, 0.2}, {0.2, 0.2, 0.4}};
    
    // Components for each dof
    static unsigned int components[56] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_finite_element_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5LinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class UFC_Poisson3D_5LinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_dof_map_1();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class UFC_Poisson3D_5LinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    
    // Take absolute value of determinant
    detJ = std::abs(detJ);
    
    // Set scale factor
    const double det = detJ;
    
    // Compute coefficients
    const double c0_0_0_0 = w[0][0];
    const double c0_0_0_1 = w[0][1];
    const double c0_0_0_2 = w[0][2];
    const double c0_0_0_3 = w[0][3];
    const double c0_0_0_4 = w[0][4];
    const double c0_0_0_5 = w[0][5];
    const double c0_0_0_6 = w[0][6];
    const double c0_0_0_7 = w[0][7];
    const double c0_0_0_8 = w[0][8];
    const double c0_0_0_9 = w[0][9];
    const double c0_0_0_10 = w[0][10];
    const double c0_0_0_11 = w[0][11];
    const double c0_0_0_12 = w[0][12];
    const double c0_0_0_13 = w[0][13];
    const double c0_0_0_14 = w[0][14];
    const double c0_0_0_15 = w[0][15];
    const double c0_0_0_16 = w[0][16];
    const double c0_0_0_17 = w[0][17];
    const double c0_0_0_18 = w[0][18];
    const double c0_0_0_19 = w[0][19];
    const double c0_0_0_20 = w[0][20];
    const double c0_0_0_21 = w[0][21];
    const double c0_0_0_22 = w[0][22];
    const double c0_0_0_23 = w[0][23];
    const double c0_0_0_24 = w[0][24];
    const double c0_0_0_25 = w[0][25];
    const double c0_0_0_26 = w[0][26];
    const double c0_0_0_27 = w[0][27];
    const double c0_0_0_28 = w[0][28];
    const double c0_0_0_29 = w[0][29];
    const double c0_0_0_30 = w[0][30];
    const double c0_0_0_31 = w[0][31];
    const double c0_0_0_32 = w[0][32];
    const double c0_0_0_33 = w[0][33];
    const double c0_0_0_34 = w[0][34];
    const double c0_0_0_35 = w[0][35];
    const double c0_0_0_36 = w[0][36];
    const double c0_0_0_37 = w[0][37];
    const double c0_0_0_38 = w[0][38];
    const double c0_0_0_39 = w[0][39];
    const double c0_0_0_40 = w[0][40];
    const double c0_0_0_41 = w[0][41];
    const double c0_0_0_42 = w[0][42];
    const double c0_0_0_43 = w[0][43];
    const double c0_0_0_44 = w[0][44];
    const double c0_0_0_45 = w[0][45];
    const double c0_0_0_46 = w[0][46];
    const double c0_0_0_47 = w[0][47];
    const double c0_0_0_48 = w[0][48];
    const double c0_0_0_49 = w[0][49];
    const double c0_0_0_50 = w[0][50];
    const double c0_0_0_51 = w[0][51];
    const double c0_0_0_52 = w[0][52];
    const double c0_0_0_53 = w[0][53];
    const double c0_0_0_54 = w[0][54];
    const double c0_0_0_55 = w[0][55];
    
    // Compute geometry tensors
    const double G0_0 = det*c0_0_0_0;
    const double G0_1 = det*c0_0_0_1;
    const double G0_2 = det*c0_0_0_2;
    const double G0_3 = det*c0_0_0_3;
    const double G0_4 = det*c0_0_0_4;
    const double G0_5 = det*c0_0_0_5;
    const double G0_6 = det*c0_0_0_6;
    const double G0_7 = det*c0_0_0_7;
    const double G0_8 = det*c0_0_0_8;
    const double G0_9 = det*c0_0_0_9;
    const double G0_10 = det*c0_0_0_10;
    const double G0_11 = det*c0_0_0_11;
    const double G0_12 = det*c0_0_0_12;
    const double G0_13 = det*c0_0_0_13;
    const double G0_14 = det*c0_0_0_14;
    const double G0_15 = det*c0_0_0_15;
    const double G0_16 = det*c0_0_0_16;
    const double G0_17 = det*c0_0_0_17;
    const double G0_18 = det*c0_0_0_18;
    const double G0_19 = det*c0_0_0_19;
    const double G0_20 = det*c0_0_0_20;
    const double G0_21 = det*c0_0_0_21;
    const double G0_22 = det*c0_0_0_22;
    const double G0_23 = det*c0_0_0_23;
    const double G0_24 = det*c0_0_0_24;
    const double G0_25 = det*c0_0_0_25;
    const double G0_26 = det*c0_0_0_26;
    const double G0_27 = det*c0_0_0_27;
    const double G0_28 = det*c0_0_0_28;
    const double G0_29 = det*c0_0_0_29;
    const double G0_30 = det*c0_0_0_30;
    const double G0_31 = det*c0_0_0_31;
    const double G0_32 = det*c0_0_0_32;
    const double G0_33 = det*c0_0_0_33;
    const double G0_34 = det*c0_0_0_34;
    const double G0_35 = det*c0_0_0_35;
    const double G0_36 = det*c0_0_0_36;
    const double G0_37 = det*c0_0_0_37;
    const double G0_38 = det*c0_0_0_38;
    const double G0_39 = det*c0_0_0_39;
    const double G0_40 = det*c0_0_0_40;
    const double G0_41 = det*c0_0_0_41;
    const double G0_42 = det*c0_0_0_42;
    const double G0_43 = det*c0_0_0_43;
    const double G0_44 = det*c0_0_0_44;
    const double G0_45 = det*c0_0_0_45;
    const double G0_46 = det*c0_0_0_46;
    const double G0_47 = det*c0_0_0_47;
    const double G0_48 = det*c0_0_0_48;
    const double G0_49 = det*c0_0_0_49;
    const double G0_50 = det*c0_0_0_50;
    const double G0_51 = det*c0_0_0_51;
    const double G0_52 = det*c0_0_0_52;
    const double G0_53 = det*c0_0_0_53;
    const double G0_54 = det*c0_0_0_54;
    const double G0_55 = det*c0_0_0_55;
    
    // Compute element tensor
    A[0] = 8.34957866207909e-05*G0_0 + 9.27851405282009e-06*G0_1 + 9.27851405282009e-06*G0_2 + 9.27851405282016e-06*G0_3 + 4.07498237359375e-06*G0_4 + 1.52962392545733e-05*G0_5 + 1.52962392545736e-05*G0_6 + 4.07498237359362e-06*G0_7 + 4.07498237359382e-06*G0_8 + 1.52962392545733e-05*G0_9 + 1.52962392545735e-05*G0_10 + 4.07498237359362e-06*G0_11 + 4.07498237359374e-06*G0_12 + 1.52962392545733e-05*G0_13 + 1.52962392545734e-05*G0_14 + 4.07498237359374e-06*G0_15 - 6.04020786312482e-05*G0_16 + 6.04020786312482e-05*G0_17 - 4.43831824040179e-05*G0_18 + 1.81868671452013e-05*G0_19 - 6.04020786312484e-05*G0_20 + 6.04020786312484e-05*G0_21 - 4.43831824040178e-05*G0_22 + 1.81868671452013e-05*G0_23 - 6.04020786312486e-05*G0_24 + 6.04020786312488e-05*G0_25 - 4.43831824040183e-05*G0_26 + 1.81868671452015e-05*G0_27 + 0.000173437673437683*G0_28 - 1.77151327324949e-05*G0_29 + 0.000173437673437683*G0_30 - 1.77151327324946e-05*G0_31 - 1.7715132732495e-05*G0_32 + 0.000173437673437683*G0_33 + 0.000149650214754389*G0_34 - 0.00012149670352796*G0_35 + 9.59527869250145e-05*G0_36 - 0.00012149670352796*G0_37 + 3.98465025201153e-05*G0_38 + 9.59527869250141e-05*G0_39 + 0.000149650214754389*G0_40 - 0.00012149670352796*G0_41 + 9.59527869250144e-05*G0_42 - 0.00012149670352796*G0_43 + 3.98465025201156e-05*G0_44 + 9.59527869250142e-05*G0_45 + 0.000149650214754389*G0_46 - 0.00012149670352796*G0_47 + 9.59527869250145e-05*G0_48 - 0.00012149670352796*G0_49 + 3.98465025201153e-05*G0_50 + 9.59527869250143e-05*G0_51 - 0.00028002957690459*G0_52 + 0.000204250963799586*G0_53 + 0.000204250963799586*G0_54 + 0.000204250963799584*G0_55;
    A[1] = 9.27851405282009e-06*G0_0 + 8.34957866207911e-05*G0_1 + 9.27851405282008e-06*G0_2 + 9.27851405282015e-06*G0_3 + 4.07498237359374e-06*G0_4 + 1.52962392545734e-05*G0_5 + 1.52962392545736e-05*G0_6 + 4.07498237359368e-06*G0_7 - 6.04020786312483e-05*G0_8 + 6.04020786312484e-05*G0_9 - 4.43831824040181e-05*G0_10 + 1.81868671452014e-05*G0_11 - 6.04020786312485e-05*G0_12 + 6.04020786312485e-05*G0_13 - 4.43831824040185e-05*G0_14 + 1.81868671452014e-05*G0_15 + 4.07498237359369e-06*G0_16 + 1.52962392545734e-05*G0_17 + 1.52962392545734e-05*G0_18 + 4.07498237359371e-06*G0_19 + 4.07498237359367e-06*G0_20 + 1.52962392545733e-05*G0_21 + 1.52962392545737e-05*G0_22 + 4.07498237359355e-06*G0_23 + 1.81868671452014e-05*G0_24 - 4.43831824040181e-05*G0_25 + 6.04020786312483e-05*G0_26 - 6.04020786312484e-05*G0_27 + 0.000149650214754389*G0_28 - 0.00012149670352796*G0_29 + 9.59527869250143e-05*G0_30 - 0.00012149670352796*G0_31 + 3.98465025201155e-05*G0_32 + 9.59527869250141e-05*G0_33 + 0.000173437673437683*G0_34 - 1.77151327324949e-05*G0_35 + 0.000173437673437683*G0_36 - 1.77151327324949e-05*G0_37 - 1.77151327324951e-05*G0_38 + 0.000173437673437682*G0_39 + 9.59527869250144e-05*G0_40 - 0.00012149670352796*G0_41 + 0.000149650214754389*G0_42 + 3.98465025201156e-05*G0_43 - 0.00012149670352796*G0_44 + 9.59527869250143e-05*G0_45 + 9.59527869250139e-05*G0_46 - 0.00012149670352796*G0_47 + 0.000149650214754389*G0_48 + 3.98465025201156e-05*G0_49 - 0.00012149670352796*G0_50 + 9.59527869250143e-05*G0_51 + 0.000204250963799586*G0_52 - 0.000280029576904591*G0_53 + 0.000204250963799586*G0_54 + 0.000204250963799585*G0_55;
    A[2] = 9.27851405282009e-06*G0_0 + 9.27851405282008e-06*G0_1 + 8.34957866207908e-05*G0_2 + 9.27851405282012e-06*G0_3 - 6.04020786312486e-05*G0_4 + 6.04020786312485e-05*G0_5 - 4.4383182404018e-05*G0_6 + 1.81868671452014e-05*G0_7 + 4.07498237359378e-06*G0_8 + 1.52962392545734e-05*G0_9 + 1.52962392545734e-05*G0_10 + 4.07498237359367e-06*G0_11 + 1.81868671452015e-05*G0_12 - 4.43831824040183e-05*G0_13 + 6.04020786312486e-05*G0_14 - 6.04020786312487e-05*G0_15 + 4.07498237359369e-06*G0_16 + 1.52962392545734e-05*G0_17 + 1.52962392545734e-05*G0_18 + 4.07498237359371e-06*G0_19 + 1.81868671452015e-05*G0_20 - 4.43831824040183e-05*G0_21 + 6.04020786312489e-05*G0_22 - 6.04020786312487e-05*G0_23 + 4.07498237359363e-06*G0_24 + 1.52962392545736e-05*G0_25 + 1.52962392545733e-05*G0_26 + 4.0749823735938e-06*G0_27 + 9.59527869250141e-05*G0_28 - 0.00012149670352796*G0_29 + 0.000149650214754389*G0_30 + 3.98465025201158e-05*G0_31 - 0.00012149670352796*G0_32 + 9.59527869250139e-05*G0_33 + 9.59527869250141e-05*G0_34 - 0.00012149670352796*G0_35 + 0.000149650214754389*G0_36 + 3.98465025201157e-05*G0_37 - 0.00012149670352796*G0_38 + 9.59527869250137e-05*G0_39 + 0.000173437673437683*G0_40 - 1.77151327324949e-05*G0_41 + 0.000173437673437683*G0_42 - 1.77151327324949e-05*G0_43 - 1.7715132732495e-05*G0_44 + 0.000173437673437682*G0_45 + 9.59527869250138e-05*G0_46 + 3.98465025201157e-05*G0_47 + 9.59527869250141e-05*G0_48 - 0.00012149670352796*G0_49 - 0.00012149670352796*G0_50 + 0.000149650214754389*G0_51 + 0.000204250963799586*G0_52 + 0.000204250963799586*G0_53 - 0.000280029576904592*G0_54 + 0.000204250963799585*G0_55;
    A[3] = 9.27851405282016e-06*G0_0 + 9.27851405282015e-06*G0_1 + 9.27851405282012e-06*G0_2 + 8.3495786620791e-05*G0_3 + 1.81868671452013e-05*G0_4 - 4.43831824040177e-05*G0_5 + 6.04020786312483e-05*G0_6 - 6.04020786312482e-05*G0_7 + 1.81868671452013e-05*G0_8 - 4.43831824040179e-05*G0_9 + 6.04020786312482e-05*G0_10 - 6.04020786312482e-05*G0_11 + 4.07498237359369e-06*G0_12 + 1.52962392545734e-05*G0_13 + 1.52962392545733e-05*G0_14 + 4.07498237359371e-06*G0_15 + 1.81868671452013e-05*G0_16 - 4.43831824040178e-05*G0_17 + 6.04020786312482e-05*G0_18 - 6.04020786312481e-05*G0_19 + 4.07498237359364e-06*G0_20 + 1.52962392545734e-05*G0_21 + 1.52962392545736e-05*G0_22 + 4.07498237359351e-06*G0_23 + 4.07498237359357e-06*G0_24 + 1.52962392545736e-05*G0_25 + 1.52962392545732e-05*G0_26 + 4.07498237359373e-06*G0_27 + 9.59527869250145e-05*G0_28 + 3.98465025201157e-05*G0_29 + 9.59527869250144e-05*G0_30 - 0.00012149670352796*G0_31 - 0.00012149670352796*G0_32 + 0.00014965021475439*G0_33 + 9.59527869250144e-05*G0_34 + 3.98465025201155e-05*G0_35 + 9.59527869250144e-05*G0_36 - 0.00012149670352796*G0_37 - 0.00012149670352796*G0_38 + 0.00014965021475439*G0_39 + 9.59527869250145e-05*G0_40 + 3.98465025201155e-05*G0_41 + 9.59527869250145e-05*G0_42 - 0.00012149670352796*G0_43 - 0.00012149670352796*G0_44 + 0.00014965021475439*G0_45 + 0.000173437673437682*G0_46 - 1.7715132732495e-05*G0_47 + 0.000173437673437683*G0_48 - 1.77151327324949e-05*G0_49 - 1.77151327324949e-05*G0_50 + 0.000173437673437683*G0_51 + 0.000204250963799586*G0_52 + 0.000204250963799586*G0_53 + 0.000204250963799586*G0_54 - 0.000280029576904591*G0_55;
    A[4] = 4.07498237359375e-06*G0_0 + 4.07498237359374e-06*G0_1 - 6.04020786312486e-05*G0_2 + 1.81868671452013e-05*G0_3 + 0.000886258481744638*G0_4 - 0.0008586570001501*G0_5 + 0.000659424487549521*G0_6 - 0.000299350614020768*G0_7 + 5.50356814824434e-05*G0_8 - 0.000178405940124699*G0_9 + 9.04994032887597e-05*G0_10 - 0.000144029549411501*G0_11 - 0.000144029549411501*G0_12 + 0.0003573137253693*G0_13 - 0.000456929981669589*G0_14 + 0.000443129240872319*G0_15 + 5.50356814824435e-05*G0_16 - 0.000178405940124699*G0_17 + 9.04994032887597e-05*G0_18 - 0.000144029549411501*G0_19 - 0.000144029549411501*G0_20 + 0.0003573137253693*G0_21 - 0.000456929981669589*G0_22 + 0.00044312924087232*G0_23 + 2.75178407412218e-05*G0_24 - 2.2499389542215e-05*G0_25 - 2.24993895422146e-05*G0_26 + 2.75178407412217e-05*G0_27 - 4.01476095920557e-05*G0_28 - 5.52029631890775e-05*G0_29 + 0.000110405926378155*G0_30 - 9.28413471816297e-05*G0_31 + 0.000308634748738931*G0_32 - 0.000341254681532477*G0_33 - 4.01476095920556e-05*G0_34 - 5.52029631890778e-05*G0_35 + 0.000110405926378156*G0_36 - 9.28413471816299e-05*G0_37 + 0.000308634748738931*G0_38 - 0.000341254681532476*G0_39 - 0.00026681432208054*G0_40 + 0.000135498182373189*G0_41 - 0.00026681432208054*G0_42 - 8.78228959826186e-06*G0_43 - 8.78228959826172e-06*G0_44 - 0.000555375266023443*G0_45 - 2.00738047960272e-05*G0_46 + 2.88560943942901e-05*G0_47 - 2.00738047960277e-05*G0_48 - 2.76014815945387e-05*G0_49 - 2.7601481594539e-05*G0_50 + 5.52029631890779e-05*G0_51 - 0.00030110707194042*G0_52 - 0.00030110707194042*G0_53 + 0.000727675423856017*G0_54 - 0.000777859935846086*G0_55;
    A[5] = 1.52962392545733e-05*G0_0 + 1.52962392545734e-05*G0_1 + 6.04020786312485e-05*G0_2 - 4.43831824040178e-05*G0_3 - 0.0008586570001501*G0_4 + 0.00168419222238675*G0_5 - 0.00130178624102242*G0_6 + 0.000659424487549522*G0_7 - 0.000178405940124699*G0_8 + 0.000347778668091186*G0_9 - 0.000261963152588166*G0_10 + 0.000357313725369299*G0_11 + 9.049940328876e-05*G0_12 - 0.000261963152588167*G0_13 + 0.000328541271828326*G0_14 - 0.000456929981669589*G0_15 - 0.000178405940124699*G0_16 + 0.000347778668091186*G0_17 - 0.000261963152588166*G0_18 + 0.000357313725369299*G0_19 + 9.04994032887599e-05*G0_20 - 0.000261963152588166*G0_21 + 0.000328541271828326*G0_22 - 0.000456929981669589*G0_23 - 2.24993895422147e-05*G0_24 + 1.1207874344449e-05*G0_25 + 1.1207874344449e-05*G0_26 - 2.24993895422147e-05*G0_27 + 0.000127970505574678*G0_28 - 0.000120442828776168*G0_29 - 0.000110405926378155*G0_30 - 7.02583167860979e-05*G0_31 - 4.51660607910627e-05*G0_32 + 0.000491808217502687*G0_33 + 0.000127970505574678*G0_34 - 0.000120442828776168*G0_35 - 0.000110405926378155*G0_36 - 7.02583167860978e-05*G0_37 - 4.5166060791063e-05*G0_38 + 0.000491808217502686*G0_39 + 0.000170627340766238*G0_40 - 0.000122952054375672*G0_41 + 0.000170627340766238*G0_42 - 0.000148044310370707*G0_43 - 0.000148044310370708*G0_44 + 0.000873210508627221*G0_45 - 0.000109569517844987*G0_46 - 2.25830303955312e-05*G0_47 - 0.000109569517844987*G0_48 + 0.000253431785549854*G0_49 + 0.000253431785549855*G0_50 - 0.000623960765743206*G0_51 + 6.2730639987587e-05*G0_52 + 6.2730639987587e-05*G0_53 - 0.000903321215821262*G0_54 + 0.00110405926378154*G0_55;
    A[6] = 1.52962392545736e-05*G0_0 + 1.52962392545736e-05*G0_1 - 4.4383182404018e-05*G0_2 + 6.04020786312483e-05*G0_3 + 0.000659424487549521*G0_4 - 0.00130178624102242*G0_5 + 0.00168419222238675*G0_6 - 0.0008586570001501*G0_7 + 9.04994032887592e-05*G0_8 - 0.000261963152588165*G0_9 + 0.000328541271828325*G0_10 - 0.000456929981669588*G0_11 - 0.0001784059401247*G0_12 + 0.000347778668091187*G0_13 - 0.000261963152588167*G0_14 + 0.0003573137253693*G0_15 + 9.04994032887592e-05*G0_16 - 0.000261963152588166*G0_17 + 0.000328541271828325*G0_18 - 0.000456929981669588*G0_19 - 0.0001784059401247*G0_20 + 0.000347778668091187*G0_21 - 0.000261963152588166*G0_22 + 0.0003573137253693*G0_23 - 2.24993895422152e-05*G0_24 + 1.12078743444492e-05*G0_25 + 1.12078743444491e-05*G0_26 - 2.24993895422152e-05*G0_27 + 0.000127970505574681*G0_28 - 7.02583167860985e-05*G0_29 + 0.000491808217502689*G0_30 - 0.000120442828776169*G0_31 - 4.51660607910637e-05*G0_32 - 0.000110405926378152*G0_33 + 0.00012797050557468*G0_34 - 7.02583167860987e-05*G0_35 + 0.00049180821750269*G0_36 - 0.000120442828776169*G0_37 - 4.51660607910636e-05*G0_38 - 0.000110405926378151*G0_39 - 0.000109569517844985*G0_40 - 2.25830303955324e-05*G0_41 - 0.000109569517844985*G0_42 + 0.000253431785549854*G0_43 + 0.000253431785549855*G0_44 - 0.000623960765743204*G0_45 + 0.000170627340766239*G0_46 - 0.000122952054375673*G0_47 + 0.00017062734076624*G0_48 - 0.000148044310370707*G0_49 - 0.000148044310370708*G0_50 + 0.000873210508627222*G0_51 + 6.27306399875892e-05*G0_52 + 6.27306399875888e-05*G0_53 + 0.00110405926378155*G0_54 - 0.000903321215821259*G0_55;
    A[7] = 4.07498237359363e-06*G0_0 + 4.07498237359369e-06*G0_1 + 1.81868671452014e-05*G0_2 - 6.04020786312482e-05*G0_3 - 0.000299350614020768*G0_4 + 0.000659424487549522*G0_5 - 0.0008586570001501*G0_6 + 0.000886258481744638*G0_7 - 0.000144029549411501*G0_8 + 0.000357313725369299*G0_9 - 0.000456929981669589*G0_10 + 0.000443129240872319*G0_11 + 5.50356814824437e-05*G0_12 - 0.0001784059401247*G0_13 + 9.04994032887602e-05*G0_14 - 0.000144029549411502*G0_15 - 0.000144029549411501*G0_16 + 0.000357313725369299*G0_17 - 0.000456929981669589*G0_18 + 0.000443129240872319*G0_19 + 5.50356814824437e-05*G0_20 - 0.0001784059401247*G0_21 + 9.049940328876e-05*G0_22 - 0.000144029549411501*G0_23 + 2.7517840741222e-05*G0_24 - 2.2499389542215e-05*G0_25 - 2.24993895422146e-05*G0_26 + 2.75178407412218e-05*G0_27 - 4.01476095920565e-05*G0_28 - 9.28413471816299e-05*G0_29 - 0.000341254681532478*G0_30 - 5.52029631890769e-05*G0_31 + 0.000308634748738933*G0_32 + 0.000110405926378154*G0_33 - 4.01476095920563e-05*G0_34 - 9.28413471816295e-05*G0_35 - 0.000341254681532478*G0_36 - 5.52029631890771e-05*G0_37 + 0.000308634748738932*G0_38 + 0.000110405926378153*G0_39 - 2.00738047960282e-05*G0_40 + 2.88560943942907e-05*G0_41 - 2.00738047960282e-05*G0_42 - 2.76014815945388e-05*G0_43 - 2.7601481594539e-05*G0_44 + 5.52029631890769e-05*G0_45 - 0.00026681432208054*G0_46 + 0.00013549818237319*G0_47 - 0.00026681432208054*G0_48 - 8.78228959826228e-06*G0_49 - 8.78228959826198e-06*G0_50 - 0.000555375266023443*G0_51 - 0.000301107071940421*G0_52 - 0.000301107071940421*G0_53 - 0.000777859935846088*G0_54 + 0.000727675423856017*G0_55;
    A[8] = 4.07498237359382e-06*G0_0 - 6.04020786312483e-05*G0_1 + 4.07498237359379e-06*G0_2 + 1.81868671452013e-05*G0_3 + 5.50356814824434e-05*G0_4 - 0.000178405940124699*G0_5 + 9.04994032887592e-05*G0_6 - 0.000144029549411501*G0_7 + 0.000886258481744639*G0_8 - 0.000858657000150101*G0_9 + 0.000659424487549522*G0_10 - 0.000299350614020769*G0_11 + 0.00044312924087232*G0_12 - 0.00045692998166959*G0_13 + 0.000357313725369301*G0_14 - 0.000144029549411502*G0_15 + 5.50356814824432e-05*G0_16 - 0.000178405940124699*G0_17 + 9.04994032887594e-05*G0_18 - 0.000144029549411501*G0_19 + 2.75178407412216e-05*G0_20 - 2.24993895422146e-05*G0_21 - 2.24993895422153e-05*G0_22 + 2.7517840741222e-05*G0_23 - 0.000144029549411502*G0_24 + 0.000357313725369301*G0_25 - 0.000456929981669591*G0_26 + 0.000443129240872321*G0_27 + 0.000110405926378154*G0_28 - 5.52029631890771e-05*G0_29 - 4.01476095920561e-05*G0_30 + 0.000308634748738931*G0_31 - 9.28413471816294e-05*G0_32 - 0.000341254681532476*G0_33 - 0.00026681432208054*G0_34 + 0.000135498182373189*G0_35 - 0.000266814322080541*G0_36 - 8.78228959826171e-06*G0_37 - 8.78228959826103e-06*G0_38 - 0.000555375266023442*G0_39 - 4.01476095920556e-05*G0_40 - 5.52029631890783e-05*G0_41 + 0.000110405926378157*G0_42 - 9.28413471816298e-05*G0_43 + 0.00030863474873893*G0_44 - 0.000341254681532477*G0_45 - 2.00738047960274e-05*G0_46 - 2.76014815945393e-05*G0_47 + 5.52029631890786e-05*G0_48 + 2.88560943942904e-05*G0_49 - 2.76014815945387e-05*G0_50 - 2.00738047960283e-05*G0_51 - 0.000301107071940422*G0_52 + 0.000727675423856019*G0_53 - 0.000301107071940421*G0_54 - 0.000777859935846087*G0_55;
    A[9] = 1.52962392545733e-05*G0_0 + 6.04020786312484e-05*G0_1 + 1.52962392545734e-05*G0_2 - 4.43831824040178e-05*G0_3 - 0.000178405940124699*G0_4 + 0.000347778668091186*G0_5 - 0.000261963152588165*G0_6 + 0.000357313725369299*G0_7 - 0.000858657000150101*G0_8 + 0.00168419222238676*G0_9 - 0.00130178624102242*G0_10 + 0.000659424487549522*G0_11 - 0.000456929981669589*G0_12 + 0.000328541271828327*G0_13 - 0.000261963152588168*G0_14 + 9.04994032887599e-05*G0_15 - 0.000178405940124699*G0_16 + 0.000347778668091186*G0_17 - 0.000261963152588166*G0_18 + 0.000357313725369299*G0_19 - 2.24993895422144e-05*G0_20 + 1.12078743444486e-05*G0_21 + 1.12078743444497e-05*G0_22 - 2.2499389542215e-05*G0_23 + 9.04994032887607e-05*G0_24 - 0.000261963152588168*G0_25 + 0.000328541271828328*G0_26 - 0.000456929981669591*G0_27 - 0.000110405926378154*G0_28 - 0.000120442828776169*G0_29 + 0.000127970505574679*G0_30 - 4.51660607910614e-05*G0_31 - 7.02583167860984e-05*G0_32 + 0.000491808217502687*G0_33 + 0.00017062734076624*G0_34 - 0.000122952054375672*G0_35 + 0.00017062734076624*G0_36 - 0.000148044310370707*G0_37 - 0.000148044310370708*G0_38 + 0.00087321050862722*G0_39 + 0.00012797050557468*G0_40 - 0.000120442828776169*G0_41 - 0.000110405926378156*G0_42 - 7.02583167860981e-05*G0_43 - 4.51660607910616e-05*G0_44 + 0.000491808217502687*G0_45 - 0.000109569517844987*G0_46 + 0.000253431785549855*G0_47 - 0.000623960765743208*G0_48 - 2.25830303955314e-05*G0_49 + 0.000253431785549854*G0_50 - 0.000109569517844986*G0_51 + 6.27306399875891e-05*G0_52 - 0.000903321215821264*G0_53 + 6.27306399875879e-05*G0_54 + 0.00110405926378154*G0_55;
    A[10] = 1.52962392545735e-05*G0_0 - 4.43831824040181e-05*G0_1 + 1.52962392545734e-05*G0_2 + 6.04020786312482e-05*G0_3 + 9.04994032887597e-05*G0_4 - 0.000261963152588166*G0_5 + 0.000328541271828325*G0_6 - 0.000456929981669589*G0_7 + 0.000659424487549522*G0_8 - 0.00130178624102242*G0_9 + 0.00168419222238676*G0_10 - 0.000858657000150101*G0_11 + 0.0003573137253693*G0_12 - 0.000261963152588167*G0_13 + 0.000347778668091187*G0_14 - 0.000178405940124699*G0_15 + 9.04994032887593e-05*G0_16 - 0.000261963152588165*G0_17 + 0.000328541271828325*G0_18 - 0.000456929981669589*G0_19 - 2.24993895422151e-05*G0_20 + 1.12078743444493e-05*G0_21 + 1.12078743444488e-05*G0_22 - 2.24993895422149e-05*G0_23 - 0.0001784059401247*G0_24 + 0.000347778668091188*G0_25 - 0.000261963152588168*G0_26 + 0.000357313725369301*G0_27 + 0.000491808217502687*G0_28 - 7.02583167860971e-05*G0_29 + 0.000127970505574679*G0_30 - 4.51660607910646e-05*G0_31 - 0.000120442828776169*G0_32 - 0.000110405926378153*G0_33 - 0.000109569517844987*G0_34 - 2.25830303955314e-05*G0_35 - 0.000109569517844987*G0_36 + 0.000253431785549855*G0_37 + 0.000253431785549855*G0_38 - 0.000623960765743205*G0_39 + 0.000127970505574679*G0_40 - 7.02583167860984e-05*G0_41 + 0.000491808217502688*G0_42 - 0.000120442828776169*G0_43 - 4.51660607910639e-05*G0_44 - 0.000110405926378154*G0_45 + 0.000170627340766238*G0_46 - 0.000148044310370708*G0_47 + 0.000873210508627222*G0_48 - 0.000122952054375672*G0_49 - 0.000148044310370707*G0_50 + 0.000170627340766238*G0_51 + 6.27306399875865e-05*G0_52 + 0.00110405926378154*G0_53 + 6.27306399875874e-05*G0_54 - 0.000903321215821262*G0_55;
    A[11] = 4.07498237359363e-06*G0_0 + 1.81868671452014e-05*G0_1 + 4.07498237359367e-06*G0_2 - 6.04020786312482e-05*G0_3 - 0.000144029549411501*G0_4 + 0.000357313725369299*G0_5 - 0.000456929981669588*G0_6 + 0.000443129240872319*G0_7 - 0.000299350614020769*G0_8 + 0.000659424487549522*G0_9 - 0.000858657000150101*G0_10 + 0.000886258481744639*G0_11 - 0.000144029549411502*G0_12 + 9.049940328876e-05*G0_13 - 0.0001784059401247*G0_14 + 5.50356814824434e-05*G0_15 - 0.000144029549411501*G0_16 + 0.000357313725369299*G0_17 - 0.000456929981669589*G0_18 + 0.000443129240872319*G0_19 + 2.75178407412219e-05*G0_20 - 2.24993895422149e-05*G0_21 - 2.24993895422149e-05*G0_22 + 2.7517840741222e-05*G0_23 + 5.50356814824439e-05*G0_24 - 0.0001784059401247*G0_25 + 9.04994032887606e-05*G0_26 - 0.000144029549411502*G0_27 - 0.000341254681532477*G0_28 - 9.28413471816304e-05*G0_29 - 4.01476095920566e-05*G0_30 + 0.000308634748738932*G0_31 - 5.52029631890767e-05*G0_32 + 0.000110405926378153*G0_33 - 2.00738047960283e-05*G0_34 + 2.88560943942903e-05*G0_35 - 2.00738047960279e-05*G0_36 - 2.76014815945381e-05*G0_37 - 2.76014815945386e-05*G0_38 + 5.52029631890763e-05*G0_39 - 4.01476095920564e-05*G0_40 - 9.28413471816296e-05*G0_41 - 0.000341254681532478*G0_42 - 5.5202963189077e-05*G0_43 + 0.000308634748738932*G0_44 + 0.000110405926378154*G0_45 - 0.00026681432208054*G0_46 - 8.78228959826148e-06*G0_47 - 0.000555375266023444*G0_48 + 0.00013549818237319*G0_49 - 8.78228959826205e-06*G0_50 - 0.00026681432208054*G0_51 - 0.000301107071940421*G0_52 - 0.000777859935846087*G0_53 - 0.000301107071940421*G0_54 + 0.000727675423856017*G0_55;
    A[12] = 4.07498237359375e-06*G0_0 - 6.04020786312485e-05*G0_1 + 1.81868671452015e-05*G0_2 + 4.0749823735937e-06*G0_3 - 0.000144029549411501*G0_4 + 9.049940328876e-05*G0_5 - 0.0001784059401247*G0_6 + 5.50356814824437e-05*G0_7 + 0.00044312924087232*G0_8 - 0.000456929981669589*G0_9 + 0.0003573137253693*G0_10 - 0.000144029549411502*G0_11 + 0.000886258481744638*G0_12 - 0.000858657000150101*G0_13 + 0.000659424487549523*G0_14 - 0.000299350614020769*G0_15 + 2.75178407412216e-05*G0_16 - 2.24993895422146e-05*G0_17 - 2.24993895422149e-05*G0_18 + 2.75178407412218e-05*G0_19 + 5.50356814824436e-05*G0_20 - 0.000178405940124699*G0_21 + 9.04994032887596e-05*G0_22 - 0.000144029549411501*G0_23 - 0.000144029549411502*G0_24 + 0.000357313725369301*G0_25 - 0.00045692998166959*G0_26 + 0.00044312924087232*G0_27 + 0.000110405926378154*G0_28 + 0.000308634748738932*G0_29 - 0.000341254681532478*G0_30 - 5.5202963189078e-05*G0_31 - 9.28413471816289e-05*G0_32 - 4.01476095920558e-05*G0_33 - 0.00026681432208054*G0_34 - 8.78228959826156e-06*G0_35 - 0.000555375266023444*G0_36 + 0.000135498182373189*G0_37 - 8.78228959826115e-06*G0_38 - 0.000266814322080539*G0_39 - 2.00738047960281e-05*G0_40 - 2.76014815945389e-05*G0_41 + 5.52029631890787e-05*G0_42 + 2.88560943942901e-05*G0_43 - 2.76014815945398e-05*G0_44 - 2.00738047960277e-05*G0_45 - 4.01476095920555e-05*G0_46 - 5.52029631890788e-05*G0_47 + 0.000110405926378157*G0_48 - 9.28413471816291e-05*G0_49 + 0.000308634748738931*G0_50 - 0.000341254681532477*G0_51 - 0.000301107071940421*G0_52 + 0.000727675423856019*G0_53 - 0.000777859935846089*G0_54 - 0.00030110707194042*G0_55;
    A[13] = 1.52962392545733e-05*G0_0 + 6.04020786312485e-05*G0_1 - 4.43831824040183e-05*G0_2 + 1.52962392545734e-05*G0_3 + 0.0003573137253693*G0_4 - 0.000261963152588166*G0_5 + 0.000347778668091187*G0_6 - 0.0001784059401247*G0_7 - 0.00045692998166959*G0_8 + 0.000328541271828327*G0_9 - 0.000261963152588167*G0_10 + 9.049940328876e-05*G0_11 - 0.000858657000150101*G0_12 + 0.00168419222238676*G0_13 - 0.00130178624102242*G0_14 + 0.000659424487549522*G0_15 - 2.24993895422146e-05*G0_16 + 1.12078743444488e-05*G0_17 + 1.12078743444491e-05*G0_18 - 2.24993895422148e-05*G0_19 - 0.000178405940124699*G0_20 + 0.000347778668091186*G0_21 - 0.000261963152588166*G0_22 + 0.000357313725369299*G0_23 + 9.04994032887606e-05*G0_24 - 0.000261963152588168*G0_25 + 0.000328541271828328*G0_26 - 0.00045692998166959*G0_27 - 0.000110405926378154*G0_28 - 4.51660607910634e-05*G0_29 + 0.000491808217502689*G0_30 - 0.000120442828776168*G0_31 - 7.02583167860996e-05*G0_32 + 0.000127970505574679*G0_33 + 0.000170627340766239*G0_34 - 0.000148044310370707*G0_35 + 0.000873210508627222*G0_36 - 0.000122952054375672*G0_37 - 0.000148044310370708*G0_38 + 0.000170627340766239*G0_39 - 0.000109569517844987*G0_40 + 0.000253431785549855*G0_41 - 0.000623960765743207*G0_42 - 2.25830303955313e-05*G0_43 + 0.000253431785549856*G0_44 - 0.000109569517844987*G0_45 + 0.000127970505574678*G0_46 - 0.000120442828776167*G0_47 - 0.000110405926378157*G0_48 - 7.02583167860988e-05*G0_49 - 4.51660607910629e-05*G0_50 + 0.000491808217502688*G0_51 + 6.27306399875881e-05*G0_52 - 0.000903321215821265*G0_53 + 0.00110405926378155*G0_54 + 6.2730639987587e-05*G0_55;
    A[14] = 1.52962392545734e-05*G0_0 - 4.43831824040185e-05*G0_1 + 6.04020786312486e-05*G0_2 + 1.52962392545733e-05*G0_3 - 0.000456929981669589*G0_4 + 0.000328541271828326*G0_5 - 0.000261963152588167*G0_6 + 9.04994032887602e-05*G0_7 + 0.000357313725369301*G0_8 - 0.000261963152588168*G0_9 + 0.000347778668091187*G0_10 - 0.0001784059401247*G0_11 + 0.000659424487549523*G0_12 - 0.00130178624102242*G0_13 + 0.00168419222238676*G0_14 - 0.0008586570001501*G0_15 - 2.24993895422149e-05*G0_16 + 1.12078743444492e-05*G0_17 + 1.12078743444484e-05*G0_18 - 2.24993895422144e-05*G0_19 + 9.04994032887597e-05*G0_20 - 0.000261963152588166*G0_21 + 0.000328541271828326*G0_22 - 0.000456929981669589*G0_23 - 0.0001784059401247*G0_24 + 0.000347778668091189*G0_25 - 0.000261963152588169*G0_26 + 0.000357313725369301*G0_27 + 0.000491808217502686*G0_28 - 4.51660607910605e-05*G0_29 - 0.000110405926378156*G0_30 - 7.0258316786098e-05*G0_31 - 0.000120442828776169*G0_32 + 0.000127970505574679*G0_33 - 0.000109569517844989*G0_34 + 0.000253431785549855*G0_35 - 0.00062396076574321*G0_36 - 2.25830303955305e-05*G0_37 + 0.000253431785549855*G0_38 - 0.000109569517844989*G0_39 + 0.000170627340766238*G0_40 - 0.000148044310370708*G0_41 + 0.000873210508627222*G0_42 - 0.000122952054375672*G0_43 - 0.000148044310370707*G0_44 + 0.000170627340766238*G0_45 + 0.000127970505574677*G0_46 - 7.02583167860988e-05*G0_47 + 0.000491808217502688*G0_48 - 0.000120442828776168*G0_49 - 4.51660607910618e-05*G0_50 - 0.000110405926378157*G0_51 + 6.27306399875837e-05*G0_52 + 0.00110405926378155*G0_53 - 0.000903321215821268*G0_54 + 6.2730639987584e-05*G0_55;
    A[15] = 4.07498237359374e-06*G0_0 + 1.81868671452015e-05*G0_1 - 6.04020786312487e-05*G0_2 + 4.07498237359371e-06*G0_3 + 0.000443129240872319*G0_4 - 0.000456929981669589*G0_5 + 0.0003573137253693*G0_6 - 0.000144029549411502*G0_7 - 0.000144029549411502*G0_8 + 9.04994032887599e-05*G0_9 - 0.000178405940124699*G0_10 + 5.50356814824434e-05*G0_11 - 0.000299350614020769*G0_12 + 0.000659424487549522*G0_13 - 0.0008586570001501*G0_14 + 0.000886258481744638*G0_15 + 2.75178407412217e-05*G0_16 - 2.24993895422147e-05*G0_17 - 2.24993895422149e-05*G0_18 + 2.75178407412218e-05*G0_19 - 0.000144029549411501*G0_20 + 0.000357313725369299*G0_21 - 0.000456929981669589*G0_22 + 0.000443129240872319*G0_23 + 5.50356814824438e-05*G0_24 - 0.0001784059401247*G0_25 + 9.04994032887606e-05*G0_26 - 0.000144029549411502*G0_27 - 0.000341254681532476*G0_28 + 0.000308634748738931*G0_29 + 0.000110405926378156*G0_30 - 9.28413471816302e-05*G0_31 - 5.52029631890783e-05*G0_32 - 4.01476095920553e-05*G0_33 - 2.00738047960283e-05*G0_34 - 2.7601481594539e-05*G0_35 + 5.52029631890779e-05*G0_36 + 2.88560943942902e-05*G0_37 - 2.7601481594539e-05*G0_38 - 2.00738047960274e-05*G0_39 - 0.00026681432208054*G0_40 - 8.78228959826191e-06*G0_41 - 0.000555375266023443*G0_42 + 0.00013549818237319*G0_43 - 8.78228959826159e-06*G0_44 - 0.000266814322080539*G0_45 - 4.01476095920558e-05*G0_46 - 9.28413471816295e-05*G0_47 - 0.000341254681532477*G0_48 - 5.52029631890773e-05*G0_49 + 0.000308634748738932*G0_50 + 0.000110405926378155*G0_51 - 0.000301107071940422*G0_52 - 0.000777859935846086*G0_53 + 0.000727675423856019*G0_54 - 0.000301107071940419*G0_55;
    A[16] = -6.04020786312482e-05*G0_0 + 4.0749823735937e-06*G0_1 + 4.07498237359369e-06*G0_2 + 1.81868671452013e-05*G0_3 + 5.50356814824435e-05*G0_4 - 0.000178405940124699*G0_5 + 9.04994032887592e-05*G0_6 - 0.000144029549411501*G0_7 + 5.50356814824432e-05*G0_8 - 0.000178405940124699*G0_9 + 9.04994032887593e-05*G0_10 - 0.000144029549411501*G0_11 + 2.75178407412216e-05*G0_12 - 2.24993895422146e-05*G0_13 - 2.24993895422149e-05*G0_14 + 2.75178407412217e-05*G0_15 + 0.000886258481744638*G0_16 - 0.0008586570001501*G0_17 + 0.000659424487549522*G0_18 - 0.000299350614020769*G0_19 + 0.00044312924087232*G0_20 - 0.000456929981669589*G0_21 + 0.000357313725369299*G0_22 - 0.000144029549411501*G0_23 + 0.00044312924087232*G0_24 - 0.000456929981669589*G0_25 + 0.0003573137253693*G0_26 - 0.000144029549411501*G0_27 - 0.00026681432208054*G0_28 + 0.00013549818237319*G0_29 - 0.00026681432208054*G0_30 - 8.78228959826269e-06*G0_31 - 8.78228959826098e-06*G0_32 - 0.000555375266023443*G0_33 + 0.000110405926378154*G0_34 - 5.52029631890773e-05*G0_35 - 4.01476095920567e-05*G0_36 + 0.00030863474873893*G0_37 - 9.28413471816287e-05*G0_38 - 0.000341254681532477*G0_39 + 0.000110405926378155*G0_40 - 5.52029631890769e-05*G0_41 - 4.01476095920567e-05*G0_42 + 0.00030863474873893*G0_43 - 9.28413471816296e-05*G0_44 - 0.000341254681532477*G0_45 + 5.52029631890777e-05*G0_46 - 2.76014815945387e-05*G0_47 - 2.00738047960287e-05*G0_48 - 2.76014815945385e-05*G0_49 + 2.88560943942907e-05*G0_50 - 2.00738047960287e-05*G0_51 + 0.000727675423856015*G0_52 - 0.000301107071940422*G0_53 - 0.00030110707194042*G0_54 - 0.000777859935846084*G0_55;
    A[17] = 6.04020786312482e-05*G0_0 + 1.52962392545734e-05*G0_1 + 1.52962392545734e-05*G0_2 - 4.43831824040178e-05*G0_3 - 0.000178405940124699*G0_4 + 0.000347778668091186*G0_5 - 0.000261963152588166*G0_6 + 0.000357313725369299*G0_7 - 0.000178405940124699*G0_8 + 0.000347778668091186*G0_9 - 0.000261963152588165*G0_10 + 0.000357313725369299*G0_11 - 2.24993895422147e-05*G0_12 + 1.12078743444488e-05*G0_13 + 1.12078743444492e-05*G0_14 - 2.24993895422147e-05*G0_15 - 0.0008586570001501*G0_16 + 0.00168419222238675*G0_17 - 0.00130178624102242*G0_18 + 0.000659424487549522*G0_19 - 0.000456929981669589*G0_20 + 0.000328541271828326*G0_21 - 0.000261963152588166*G0_22 + 9.049940328876e-05*G0_23 - 0.000456929981669589*G0_24 + 0.000328541271828326*G0_25 - 0.000261963152588167*G0_26 + 9.04994032887601e-05*G0_27 + 0.000170627340766239*G0_28 - 0.000122952054375672*G0_29 + 0.000170627340766239*G0_30 - 0.000148044310370707*G0_31 - 0.000148044310370708*G0_32 + 0.000873210508627221*G0_33 - 0.000110405926378154*G0_34 - 0.000120442828776168*G0_35 + 0.000127970505574679*G0_36 - 4.51660607910618e-05*G0_37 - 7.02583167860993e-05*G0_38 + 0.000491808217502687*G0_39 - 0.000110405926378155*G0_40 - 0.000120442828776168*G0_41 + 0.00012797050557468*G0_42 - 4.51660607910622e-05*G0_43 - 7.02583167860982e-05*G0_44 + 0.000491808217502687*G0_45 - 0.000623960765743207*G0_46 + 0.000253431785549855*G0_47 - 0.000109569517844986*G0_48 + 0.000253431785549854*G0_49 - 2.25830303955316e-05*G0_50 - 0.000109569517844986*G0_51 - 0.000903321215821262*G0_52 + 6.27306399875888e-05*G0_53 + 6.27306399875876e-05*G0_54 + 0.00110405926378154*G0_55;
    A[18] = -4.43831824040179e-05*G0_0 + 1.52962392545734e-05*G0_1 + 1.52962392545734e-05*G0_2 + 6.04020786312482e-05*G0_3 + 9.04994032887597e-05*G0_4 - 0.000261963152588166*G0_5 + 0.000328541271828325*G0_6 - 0.000456929981669589*G0_7 + 9.04994032887594e-05*G0_8 - 0.000261963152588166*G0_9 + 0.000328541271828325*G0_10 - 0.000456929981669589*G0_11 - 2.24993895422149e-05*G0_12 + 1.12078743444491e-05*G0_13 + 1.12078743444484e-05*G0_14 - 2.24993895422149e-05*G0_15 + 0.000659424487549522*G0_16 - 0.00130178624102242*G0_17 + 0.00168419222238675*G0_18 - 0.0008586570001501*G0_19 + 0.0003573137253693*G0_20 - 0.000261963152588167*G0_21 + 0.000347778668091187*G0_22 - 0.0001784059401247*G0_23 + 0.0003573137253693*G0_24 - 0.000261963152588166*G0_25 + 0.000347778668091186*G0_26 - 0.000178405940124699*G0_27 - 0.000109569517844987*G0_28 - 2.25830303955315e-05*G0_29 - 0.000109569517844987*G0_30 + 0.000253431785549854*G0_31 + 0.000253431785549855*G0_32 - 0.000623960765743206*G0_33 + 0.000491808217502688*G0_34 - 7.02583167860982e-05*G0_35 + 0.000127970505574679*G0_36 - 4.51660607910644e-05*G0_37 - 0.000120442828776168*G0_38 - 0.000110405926378154*G0_39 + 0.000491808217502688*G0_40 - 7.0258316786098e-05*G0_41 + 0.000127970505574679*G0_42 - 4.51660607910637e-05*G0_43 - 0.000120442828776169*G0_44 - 0.000110405926378154*G0_45 + 0.000873210508627221*G0_46 - 0.000148044310370707*G0_47 + 0.000170627340766238*G0_48 - 0.000148044310370707*G0_49 - 0.000122952054375672*G0_50 + 0.000170627340766238*G0_51 + 0.00110405926378154*G0_52 + 6.27306399875871e-05*G0_53 + 6.27306399875881e-05*G0_54 - 0.000903321215821262*G0_55;
    A[19] = 1.81868671452013e-05*G0_0 + 4.07498237359371e-06*G0_1 + 4.07498237359371e-06*G0_2 - 6.04020786312481e-05*G0_3 - 0.000144029549411501*G0_4 + 0.000357313725369299*G0_5 - 0.000456929981669588*G0_6 + 0.000443129240872319*G0_7 - 0.000144029549411501*G0_8 + 0.000357313725369299*G0_9 - 0.000456929981669589*G0_10 + 0.000443129240872319*G0_11 + 2.75178407412218e-05*G0_12 - 2.24993895422148e-05*G0_13 - 2.24993895422144e-05*G0_14 + 2.75178407412218e-05*G0_15 - 0.000299350614020768*G0_16 + 0.000659424487549522*G0_17 - 0.0008586570001501*G0_18 + 0.000886258481744638*G0_19 - 0.000144029549411501*G0_20 + 9.04994032887599e-05*G0_21 - 0.0001784059401247*G0_22 + 5.50356814824436e-05*G0_23 - 0.000144029549411501*G0_24 + 9.04994032887596e-05*G0_25 - 0.000178405940124699*G0_26 + 5.50356814824435e-05*G0_27 - 2.00738047960283e-05*G0_28 + 2.88560943942904e-05*G0_29 - 2.00738047960283e-05*G0_30 - 2.76014815945385e-05*G0_31 - 2.76014815945392e-05*G0_32 + 5.52029631890773e-05*G0_33 - 0.000341254681532477*G0_34 - 9.28413471816296e-05*G0_35 - 4.01476095920561e-05*G0_36 + 0.000308634748738932*G0_37 - 5.52029631890775e-05*G0_38 + 0.000110405926378154*G0_39 - 0.000341254681532477*G0_40 - 9.28413471816298e-05*G0_41 - 4.01476095920558e-05*G0_42 + 0.000308634748738932*G0_43 - 5.52029631890775e-05*G0_44 + 0.000110405926378155*G0_45 - 0.000555375266023443*G0_46 - 8.78228959826212e-06*G0_47 - 0.00026681432208054*G0_48 - 8.78228959826207e-06*G0_49 + 0.00013549818237319*G0_50 - 0.00026681432208054*G0_51 - 0.000777859935846087*G0_52 - 0.000301107071940421*G0_53 - 0.000301107071940422*G0_54 + 0.000727675423856018*G0_55;
    A[20] = -6.04020786312484e-05*G0_0 + 4.07498237359367e-06*G0_1 + 1.81868671452015e-05*G0_2 + 4.07498237359364e-06*G0_3 - 0.000144029549411501*G0_4 + 9.04994032887599e-05*G0_5 - 0.0001784059401247*G0_6 + 5.50356814824437e-05*G0_7 + 2.75178407412216e-05*G0_8 - 2.24993895422144e-05*G0_9 - 2.24993895422151e-05*G0_10 + 2.75178407412219e-05*G0_11 + 5.50356814824436e-05*G0_12 - 0.000178405940124699*G0_13 + 9.04994032887597e-05*G0_14 - 0.000144029549411501*G0_15 + 0.00044312924087232*G0_16 - 0.000456929981669589*G0_17 + 0.0003573137253693*G0_18 - 0.000144029549411501*G0_19 + 0.000886258481744638*G0_20 - 0.000858657000150099*G0_21 + 0.000659424487549522*G0_22 - 0.000299350614020768*G0_23 + 0.000443129240872319*G0_24 - 0.000456929981669589*G0_25 + 0.0003573137253693*G0_26 - 0.000144029549411501*G0_27 - 0.000266814322080541*G0_28 - 8.78228959826186e-06*G0_29 - 0.000555375266023444*G0_30 + 0.000135498182373189*G0_31 - 8.78228959826113e-06*G0_32 - 0.00026681432208054*G0_33 + 0.000110405926378155*G0_34 + 0.000308634748738931*G0_35 - 0.000341254681532478*G0_36 - 5.52029631890783e-05*G0_37 - 9.28413471816289e-05*G0_38 - 4.01476095920561e-05*G0_39 + 5.52029631890777e-05*G0_40 - 2.76014815945385e-05*G0_41 - 2.00738047960287e-05*G0_42 - 2.76014815945394e-05*G0_43 + 2.885609439429e-05*G0_44 - 2.00738047960279e-05*G0_45 + 0.000110405926378154*G0_46 - 5.52029631890772e-05*G0_47 - 4.0147609592057e-05*G0_48 + 0.000308634748738931*G0_49 - 9.28413471816287e-05*G0_50 - 0.000341254681532478*G0_51 + 0.000727675423856016*G0_52 - 0.000301107071940422*G0_53 - 0.000777859935846087*G0_54 - 0.000301107071940419*G0_55;
    A[21] = 6.04020786312484e-05*G0_0 + 1.52962392545733e-05*G0_1 - 4.43831824040183e-05*G0_2 + 1.52962392545734e-05*G0_3 + 0.0003573137253693*G0_4 - 0.000261963152588166*G0_5 + 0.000347778668091187*G0_6 - 0.0001784059401247*G0_7 - 2.24993895422146e-05*G0_8 + 1.12078743444486e-05*G0_9 + 1.12078743444493e-05*G0_10 - 2.24993895422149e-05*G0_11 - 0.000178405940124699*G0_12 + 0.000347778668091186*G0_13 - 0.000261963152588166*G0_14 + 0.000357313725369299*G0_15 - 0.000456929981669589*G0_16 + 0.000328541271828326*G0_17 - 0.000261963152588167*G0_18 + 9.049940328876e-05*G0_19 - 0.000858657000150099*G0_20 + 0.00168419222238675*G0_21 - 0.00130178624102242*G0_22 + 0.000659424487549522*G0_23 - 0.000456929981669589*G0_24 + 0.000328541271828326*G0_25 - 0.000261963152588167*G0_26 + 9.04994032887603e-05*G0_27 + 0.000170627340766239*G0_28 - 0.000148044310370708*G0_29 + 0.000873210508627223*G0_30 - 0.000122952054375671*G0_31 - 0.000148044310370708*G0_32 + 0.000170627340766239*G0_33 - 0.000110405926378156*G0_34 - 4.5166060791064e-05*G0_35 + 0.000491808217502689*G0_36 - 0.000120442828776167*G0_37 - 7.02583167860993e-05*G0_38 + 0.000127970505574679*G0_39 - 0.000623960765743208*G0_40 + 0.000253431785549855*G0_41 - 0.000109569517844986*G0_42 + 0.000253431785549855*G0_43 - 2.25830303955309e-05*G0_44 - 0.000109569517844987*G0_45 - 0.000110405926378156*G0_46 - 0.000120442828776167*G0_47 + 0.000127970505574679*G0_48 - 4.51660607910631e-05*G0_49 - 7.02583167860992e-05*G0_50 + 0.000491808217502688*G0_51 - 0.000903321215821263*G0_52 + 6.27306399875894e-05*G0_53 + 0.00110405926378154*G0_54 + 6.27306399875869e-05*G0_55;
    A[22] = -4.43831824040178e-05*G0_0 + 1.52962392545737e-05*G0_1 + 6.04020786312489e-05*G0_2 + 1.52962392545736e-05*G0_3 - 0.000456929981669589*G0_4 + 0.000328541271828326*G0_5 - 0.000261963152588166*G0_6 + 9.049940328876e-05*G0_7 - 2.24993895422153e-05*G0_8 + 1.12078743444497e-05*G0_9 + 1.12078743444488e-05*G0_10 - 2.24993895422149e-05*G0_11 + 9.04994032887596e-05*G0_12 - 0.000261963152588166*G0_13 + 0.000328541271828326*G0_14 - 0.000456929981669589*G0_15 + 0.000357313725369299*G0_16 - 0.000261963152588166*G0_17 + 0.000347778668091187*G0_18 - 0.0001784059401247*G0_19 + 0.000659424487549522*G0_20 - 0.00130178624102242*G0_21 + 0.00168419222238676*G0_22 - 0.000858657000150101*G0_23 + 0.0003573137253693*G0_24 - 0.000261963152588166*G0_25 + 0.000347778668091187*G0_26 - 0.0001784059401247*G0_27 - 0.000109569517844985*G0_28 + 0.000253431785549854*G0_29 - 0.000623960765743206*G0_30 - 2.25830303955327e-05*G0_31 + 0.000253431785549855*G0_32 - 0.000109569517844985*G0_33 + 0.000491808217502691*G0_34 - 4.51660607910632e-05*G0_35 - 0.000110405926378154*G0_36 - 7.02583167860998e-05*G0_37 - 0.000120442828776168*G0_38 + 0.00012797050557468*G0_39 + 0.000873210508627225*G0_40 - 0.000148044310370708*G0_41 + 0.000170627340766241*G0_42 - 0.000148044310370708*G0_43 - 0.000122952054375673*G0_44 + 0.000170627340766241*G0_45 + 0.00049180821750269*G0_46 - 7.02583167860992e-05*G0_47 + 0.00012797050557468*G0_48 - 4.51660607910647e-05*G0_49 - 0.000120442828776168*G0_50 - 0.000110405926378153*G0_51 + 0.00110405926378155*G0_52 + 6.27306399875883e-05*G0_53 - 0.000903321215821261*G0_54 + 6.27306399875902e-05*G0_55;
    A[23] = 1.81868671452013e-05*G0_0 + 4.07498237359355e-06*G0_1 - 6.04020786312487e-05*G0_2 + 4.0749823735935e-06*G0_3 + 0.00044312924087232*G0_4 - 0.000456929981669589*G0_5 + 0.0003573137253693*G0_6 - 0.000144029549411501*G0_7 + 2.7517840741222e-05*G0_8 - 2.2499389542215e-05*G0_9 - 2.24993895422149e-05*G0_10 + 2.7517840741222e-05*G0_11 - 0.000144029549411501*G0_12 + 0.000357313725369299*G0_13 - 0.000456929981669589*G0_14 + 0.000443129240872319*G0_15 - 0.000144029549411501*G0_16 + 9.049940328876e-05*G0_17 - 0.0001784059401247*G0_18 + 5.50356814824436e-05*G0_19 - 0.000299350614020768*G0_20 + 0.000659424487549522*G0_21 - 0.000858657000150101*G0_22 + 0.000886258481744639*G0_23 - 0.000144029549411501*G0_24 + 9.04994032887596e-05*G0_25 - 0.0001784059401247*G0_26 + 5.50356814824438e-05*G0_27 - 2.00738047960292e-05*G0_28 - 2.76014815945393e-05*G0_29 + 5.52029631890766e-05*G0_30 + 2.8856094394291e-05*G0_31 - 2.7601481594539e-05*G0_32 - 2.00738047960291e-05*G0_33 - 0.000341254681532478*G0_34 + 0.000308634748738931*G0_35 + 0.000110405926378154*G0_36 - 9.28413471816292e-05*G0_37 - 5.52029631890777e-05*G0_38 - 4.01476095920566e-05*G0_39 - 0.000555375266023445*G0_40 - 8.78228959826189e-06*G0_41 - 0.000266814322080541*G0_42 - 8.7822895982613e-06*G0_43 + 0.000135498182373191*G0_44 - 0.000266814322080541*G0_45 - 0.000341254681532478*G0_46 - 9.28413471816293e-05*G0_47 - 4.01476095920571e-05*G0_48 + 0.000308634748738932*G0_49 - 5.52029631890778e-05*G0_50 + 0.000110405926378154*G0_51 - 0.00077785993584609*G0_52 - 0.000301107071940423*G0_53 + 0.000727675423856015*G0_54 - 0.000301107071940422*G0_55;
    A[24] = -6.04020786312486e-05*G0_0 + 1.81868671452015e-05*G0_1 + 4.07498237359363e-06*G0_2 + 4.07498237359357e-06*G0_3 + 2.75178407412218e-05*G0_4 - 2.24993895422147e-05*G0_5 - 2.24993895422152e-05*G0_6 + 2.7517840741222e-05*G0_7 - 0.000144029549411502*G0_8 + 9.04994032887607e-05*G0_9 - 0.0001784059401247*G0_10 + 5.50356814824439e-05*G0_11 - 0.000144029549411502*G0_12 + 9.04994032887606e-05*G0_13 - 0.0001784059401247*G0_14 + 5.50356814824438e-05*G0_15 + 0.00044312924087232*G0_16 - 0.000456929981669589*G0_17 + 0.0003573137253693*G0_18 - 0.000144029549411501*G0_19 + 0.000443129240872319*G0_20 - 0.000456929981669589*G0_21 + 0.0003573137253693*G0_22 - 0.000144029549411501*G0_23 + 0.00088625848174464*G0_24 - 0.000858657000150102*G0_25 + 0.000659424487549524*G0_26 - 0.000299350614020769*G0_27 - 0.000555375266023446*G0_28 - 8.78228959826136e-06*G0_29 - 0.000266814322080541*G0_30 - 8.78228959826182e-06*G0_31 + 0.00013549818237319*G0_32 - 0.00026681432208054*G0_33 + 5.52029631890771e-05*G0_34 - 2.76014815945383e-05*G0_35 - 2.00738047960286e-05*G0_36 - 2.76014815945395e-05*G0_37 + 2.88560943942905e-05*G0_38 - 2.00738047960283e-05*G0_39 + 0.000110405926378154*G0_40 + 0.000308634748738932*G0_41 - 0.00034125468153248*G0_42 - 5.52029631890783e-05*G0_43 - 9.28413471816289e-05*G0_44 - 4.01476095920561e-05*G0_45 + 0.000110405926378154*G0_46 + 0.000308634748738932*G0_47 - 0.000341254681532479*G0_48 - 5.52029631890774e-05*G0_49 - 9.28413471816285e-05*G0_50 - 4.0147609592057e-05*G0_51 + 0.000727675423856016*G0_52 - 0.000777859935846089*G0_53 - 0.000301107071940421*G0_54 - 0.000301107071940419*G0_55;
    A[25] = 6.04020786312488e-05*G0_0 - 4.43831824040181e-05*G0_1 + 1.52962392545736e-05*G0_2 + 1.52962392545736e-05*G0_3 - 2.2499389542215e-05*G0_4 + 1.1207874344449e-05*G0_5 + 1.12078743444492e-05*G0_6 - 2.2499389542215e-05*G0_7 + 0.000357313725369301*G0_8 - 0.000261963152588168*G0_9 + 0.000347778668091188*G0_10 - 0.0001784059401247*G0_11 + 0.000357313725369301*G0_12 - 0.000261963152588168*G0_13 + 0.000347778668091189*G0_14 - 0.0001784059401247*G0_15 - 0.000456929981669589*G0_16 + 0.000328541271828326*G0_17 - 0.000261963152588166*G0_18 + 9.04994032887597e-05*G0_19 - 0.000456929981669589*G0_20 + 0.000328541271828326*G0_21 - 0.000261963152588166*G0_22 + 9.04994032887596e-05*G0_23 - 0.000858657000150102*G0_24 + 0.00168419222238676*G0_25 - 0.00130178624102242*G0_26 + 0.000659424487549524*G0_27 + 0.000873210508627226*G0_28 - 0.000148044310370709*G0_29 + 0.00017062734076624*G0_30 - 0.000148044310370708*G0_31 - 0.000122952054375672*G0_32 + 0.00017062734076624*G0_33 - 0.000623960765743205*G0_34 + 0.000253431785549854*G0_35 - 0.000109569517844986*G0_36 + 0.000253431785549855*G0_37 - 2.25830303955317e-05*G0_38 - 0.000109569517844986*G0_39 - 0.000110405926378153*G0_40 - 4.51660607910655e-05*G0_41 + 0.000491808217502692*G0_42 - 0.000120442828776168*G0_43 - 7.02583167860997e-05*G0_44 + 0.00012797050557468*G0_45 - 0.000110405926378153*G0_46 - 4.51660607910658e-05*G0_47 + 0.000491808217502692*G0_48 - 0.000120442828776168*G0_49 - 7.02583167861003e-05*G0_50 + 0.00012797050557468*G0_51 - 0.000903321215821263*G0_52 + 0.00110405926378155*G0_53 + 6.27306399875874e-05*G0_54 + 6.27306399875868e-05*G0_55;
    A[26] = -4.43831824040183e-05*G0_0 + 6.04020786312483e-05*G0_1 + 1.52962392545733e-05*G0_2 + 1.52962392545732e-05*G0_3 - 2.24993895422146e-05*G0_4 + 1.1207874344449e-05*G0_5 + 1.12078743444491e-05*G0_6 - 2.24993895422146e-05*G0_7 - 0.000456929981669591*G0_8 + 0.000328541271828328*G0_9 - 0.000261963152588168*G0_10 + 9.04994032887606e-05*G0_11 - 0.00045692998166959*G0_12 + 0.000328541271828328*G0_13 - 0.000261963152588169*G0_14 + 9.04994032887606e-05*G0_15 + 0.0003573137253693*G0_16 - 0.000261963152588167*G0_17 + 0.000347778668091186*G0_18 - 0.000178405940124699*G0_19 + 0.0003573137253693*G0_20 - 0.000261963152588167*G0_21 + 0.000347778668091187*G0_22 - 0.0001784059401247*G0_23 + 0.000659424487549524*G0_24 - 0.00130178624102242*G0_25 + 0.00168419222238676*G0_26 - 0.000858657000150102*G0_27 - 0.00062396076574321*G0_28 + 0.000253431785549856*G0_29 - 0.000109569517844988*G0_30 + 0.000253431785549855*G0_31 - 2.25830303955311e-05*G0_32 - 0.000109569517844988*G0_33 + 0.000873210508627221*G0_34 - 0.000148044310370707*G0_35 + 0.000170627340766238*G0_36 - 0.000148044310370708*G0_37 - 0.000122952054375672*G0_38 + 0.000170627340766238*G0_39 + 0.000491808217502687*G0_40 - 4.51660607910606e-05*G0_41 - 0.000110405926378159*G0_42 - 7.02583167860987e-05*G0_43 - 0.000120442828776167*G0_44 + 0.000127970505574678*G0_45 + 0.000491808217502686*G0_46 - 4.51660607910605e-05*G0_47 - 0.000110405926378159*G0_48 - 7.02583167860988e-05*G0_49 - 0.000120442828776166*G0_50 + 0.000127970505574678*G0_51 + 0.00110405926378154*G0_52 - 0.000903321215821266*G0_53 + 6.27306399875886e-05*G0_54 + 6.27306399875875e-05*G0_55;
    A[27] = 1.81868671452015e-05*G0_0 - 6.04020786312484e-05*G0_1 + 4.0749823735938e-06*G0_2 + 4.07498237359373e-06*G0_3 + 2.75178407412217e-05*G0_4 - 2.24993895422147e-05*G0_5 - 2.24993895422152e-05*G0_6 + 2.75178407412218e-05*G0_7 + 0.000443129240872321*G0_8 - 0.000456929981669591*G0_9 + 0.000357313725369301*G0_10 - 0.000144029549411502*G0_11 + 0.00044312924087232*G0_12 - 0.00045692998166959*G0_13 + 0.000357313725369301*G0_14 - 0.000144029549411502*G0_15 - 0.000144029549411501*G0_16 + 9.04994032887601e-05*G0_17 - 0.000178405940124699*G0_18 + 5.50356814824435e-05*G0_19 - 0.000144029549411501*G0_20 + 9.04994032887603e-05*G0_21 - 0.0001784059401247*G0_22 + 5.50356814824438e-05*G0_23 - 0.000299350614020769*G0_24 + 0.000659424487549524*G0_25 - 0.000858657000150102*G0_26 + 0.00088625848174464*G0_27 + 5.52029631890783e-05*G0_28 - 2.76014815945388e-05*G0_29 - 2.00738047960278e-05*G0_30 - 2.76014815945388e-05*G0_31 + 2.88560943942901e-05*G0_32 - 2.00738047960278e-05*G0_33 - 0.000555375266023443*G0_34 - 8.78228959826216e-06*G0_35 - 0.00026681432208054*G0_36 - 8.78228959826152e-06*G0_37 + 0.00013549818237319*G0_38 - 0.000266814322080539*G0_39 - 0.000341254681532476*G0_40 + 0.000308634748738931*G0_41 + 0.000110405926378157*G0_42 - 9.28413471816296e-05*G0_43 - 5.52029631890777e-05*G0_44 - 4.01476095920558e-05*G0_45 - 0.000341254681532476*G0_46 + 0.00030863474873893*G0_47 + 0.000110405926378156*G0_48 - 9.28413471816294e-05*G0_49 - 5.52029631890778e-05*G0_50 - 4.01476095920563e-05*G0_51 - 0.000777859935846088*G0_52 + 0.000727675423856018*G0_53 - 0.000301107071940423*G0_54 - 0.00030110707194042*G0_55;
    A[28] = 0.000173437673437683*G0_0 + 0.000149650214754389*G0_1 + 9.59527869250141e-05*G0_2 + 9.59527869250146e-05*G0_3 - 4.01476095920558e-05*G0_4 + 0.000127970505574678*G0_5 + 0.000127970505574681*G0_6 - 4.01476095920565e-05*G0_7 + 0.000110405926378154*G0_8 - 0.000110405926378154*G0_9 + 0.000491808217502688*G0_10 - 0.000341254681532477*G0_11 + 0.000110405926378154*G0_12 - 0.000110405926378154*G0_13 + 0.000491808217502687*G0_14 - 0.000341254681532476*G0_15 - 0.00026681432208054*G0_16 + 0.000170627340766239*G0_17 - 0.000109569517844987*G0_18 - 2.00738047960283e-05*G0_19 - 0.000266814322080541*G0_20 + 0.000170627340766239*G0_21 - 0.000109569517844985*G0_22 - 2.00738047960292e-05*G0_23 - 0.000555375266023446*G0_24 + 0.000873210508627226*G0_25 - 0.00062396076574321*G0_26 + 5.52029631890783e-05*G0_27 + 0.00455006242043303*G0_28 - 0.00180664243164252*G0_29 + 0.00103714658112812*G0_30 - 0.00180664243164252*G0_31 + 0.000514391247898216*G0_32 + 0.00103714658112812*G0_33 + 0.0006189423145442*G0_34 - 0.000213284175957798*G0_35 + 0.000694219082529305*G0_36 - 0.000213284175957797*G0_37 + 0.000577121887885804*G0_38 + 0.000694219082529303*G0_39 + 0.000694219082529308*G0_40 - 0.00125461279975176*G0_41 + 0.00227503121021652*G0_42 + 6.27306399875871e-05*G0_43 - 0.000903321215821262*G0_44 + 0.000518573290564059*G0_45 + 0.000694219082529304*G0_46 - 0.00125461279975176*G0_47 + 0.00227503121021652*G0_48 + 6.27306399875872e-05*G0_49 - 0.000903321215821263*G0_50 + 0.000518573290564061*G0_51 - 0.000351291583930489*G0_52 + 0.00140516633572197*G0_53 + 0.000777859935846091*G0_54 + 0.00077785993584608*G0_55;
    A[29] = -1.77151327324949e-05*G0_0 - 0.00012149670352796*G0_1 - 0.00012149670352796*G0_2 + 3.98465025201157e-05*G0_3 - 5.52029631890775e-05*G0_4 - 0.000120442828776168*G0_5 - 7.02583167860985e-05*G0_6 - 9.28413471816299e-05*G0_7 - 5.52029631890771e-05*G0_8 - 0.000120442828776169*G0_9 - 7.02583167860972e-05*G0_10 - 9.28413471816304e-05*G0_11 + 0.000308634748738932*G0_12 - 4.51660607910634e-05*G0_13 - 4.51660607910606e-05*G0_14 + 0.000308634748738931*G0_15 + 0.00013549818237319*G0_16 - 0.000122952054375672*G0_17 - 2.25830303955315e-05*G0_18 + 2.88560943942904e-05*G0_19 - 8.78228959826189e-06*G0_20 - 0.000148044310370708*G0_21 + 0.000253431785549854*G0_22 - 2.76014815945393e-05*G0_23 - 8.78228959826138e-06*G0_24 - 0.000148044310370709*G0_25 + 0.000253431785549856*G0_26 - 2.76014815945388e-05*G0_27 - 0.00180664243164252*G0_28 + 0.00526937375895737*G0_29 - 0.00180664243164253*G0_30 - 0.000940959599813819*G0_31 - 0.000940959599813813*G0_32 + 0.00051439124789822*G0_33 - 0.000213284175957798*G0_34 + 0.000225830303955316*G0_35 - 0.00125461279975176*G0_36 + 0.000414022223918077*G0_37 - 0.000639852527873391*G0_38 + 6.27306399875894e-05*G0_39 - 0.0002132841759578*G0_40 + 0.000225830303955318*G0_41 - 0.00125461279975176*G0_42 + 0.000414022223918078*G0_43 - 0.000639852527873395*G0_44 + 6.27306399875875e-05*G0_45 + 0.000577121887885807*G0_46 - 0.000639852527873394*G0_47 - 0.000903321215821263*G0_48 - 0.000639852527873394*G0_49 + 0.00263468687947868*G0_50 - 0.000903321215821265*G0_51 - 0.000639852527873394*G0_52 + 0.000677490911865947*G0_53 + 0.000677490911865944*G0_54 - 0.00120442828776168*G0_55;
    A[30] = 0.000173437673437683*G0_0 + 9.59527869250143e-05*G0_1 + 0.000149650214754389*G0_2 + 9.59527869250144e-05*G0_3 + 0.000110405926378155*G0_4 - 0.000110405926378155*G0_5 + 0.000491808217502689*G0_6 - 0.000341254681532478*G0_7 - 4.01476095920561e-05*G0_8 + 0.000127970505574679*G0_9 + 0.000127970505574679*G0_10 - 4.01476095920566e-05*G0_11 - 0.000341254681532478*G0_12 + 0.000491808217502689*G0_13 - 0.000110405926378156*G0_14 + 0.000110405926378156*G0_15 - 0.00026681432208054*G0_16 + 0.000170627340766239*G0_17 - 0.000109569517844987*G0_18 - 2.00738047960283e-05*G0_19 - 0.000555375266023444*G0_20 + 0.000873210508627223*G0_21 - 0.000623960765743206*G0_22 + 5.52029631890765e-05*G0_23 - 0.000266814322080541*G0_24 + 0.00017062734076624*G0_25 - 0.000109569517844988*G0_26 - 2.00738047960278e-05*G0_27 + 0.00103714658112812*G0_28 - 0.00180664243164253*G0_29 + 0.00455006242043303*G0_30 + 0.000514391247898219*G0_31 - 0.00180664243164253*G0_32 + 0.00103714658112812*G0_33 + 0.000694219082529305*G0_34 - 0.00125461279975176*G0_35 + 0.00227503121021651*G0_36 + 6.27306399875886e-05*G0_37 - 0.000903321215821263*G0_38 + 0.000518573290564056*G0_39 + 0.000618942314544199*G0_40 - 0.0002132841759578*G0_41 + 0.000694219082529305*G0_42 - 0.000213284175957797*G0_43 + 0.000577121887885807*G0_44 + 0.000694219082529303*G0_45 + 0.000694219082529302*G0_46 + 6.27306399875866e-05*G0_47 + 0.000518573290564059*G0_48 - 0.00125461279975176*G0_49 - 0.000903321215821262*G0_50 + 0.00227503121021652*G0_51 - 0.000351291583930492*G0_52 + 0.000777859935846089*G0_53 + 0.00140516633572197*G0_54 + 0.00077785993584608*G0_55;
    A[31] = -1.77151327324946e-05*G0_0 - 0.00012149670352796*G0_1 + 3.98465025201158e-05*G0_2 - 0.00012149670352796*G0_3 - 9.28413471816297e-05*G0_4 - 7.02583167860979e-05*G0_5 - 0.000120442828776169*G0_6 - 5.52029631890769e-05*G0_7 + 0.000308634748738931*G0_8 - 4.51660607910615e-05*G0_9 - 4.51660607910647e-05*G0_10 + 0.000308634748738932*G0_11 - 5.5202963189078e-05*G0_12 - 0.000120442828776169*G0_13 - 7.0258316786098e-05*G0_14 - 9.28413471816303e-05*G0_15 - 8.78228959826264e-06*G0_16 - 0.000148044310370707*G0_17 + 0.000253431785549854*G0_18 - 2.76014815945385e-05*G0_19 + 0.000135498182373189*G0_20 - 0.000122952054375671*G0_21 - 2.25830303955327e-05*G0_22 + 2.8856094394291e-05*G0_23 - 8.78228959826185e-06*G0_24 - 0.000148044310370708*G0_25 + 0.000253431785549855*G0_26 - 2.76014815945388e-05*G0_27 - 0.00180664243164252*G0_28 - 0.000940959599813819*G0_29 + 0.000514391247898219*G0_30 + 0.00526937375895737*G0_31 - 0.000940959599813811*G0_32 - 0.00180664243164253*G0_33 - 0.000213284175957798*G0_34 + 0.000414022223918076*G0_35 + 6.2730639987588e-05*G0_36 + 0.000225830303955314*G0_37 - 0.000639852527873391*G0_38 - 0.00125461279975175*G0_39 + 0.000577121887885806*G0_40 - 0.000639852527873394*G0_41 - 0.000903321215821263*G0_42 - 0.000639852527873395*G0_43 + 0.00263468687947868*G0_44 - 0.000903321215821263*G0_45 - 0.000213284175957798*G0_46 + 0.000225830303955318*G0_47 - 0.00125461279975176*G0_48 + 0.000414022223918077*G0_49 - 0.000639852527873395*G0_50 + 6.27306399875877e-05*G0_51 - 0.0006398525278734*G0_52 + 0.000677490911865948*G0_53 - 0.00120442828776168*G0_54 + 0.000677490911865952*G0_55;
    A[32] = -1.77151327324951e-05*G0_0 + 3.98465025201155e-05*G0_1 - 0.00012149670352796*G0_2 - 0.00012149670352796*G0_3 + 0.000308634748738931*G0_4 - 4.51660607910626e-05*G0_5 - 4.51660607910638e-05*G0_6 + 0.000308634748738933*G0_7 - 9.28413471816294e-05*G0_8 - 7.02583167860984e-05*G0_9 - 0.000120442828776169*G0_10 - 5.52029631890767e-05*G0_11 - 9.28413471816289e-05*G0_12 - 7.02583167860996e-05*G0_13 - 0.000120442828776169*G0_14 - 5.52029631890783e-05*G0_15 - 8.78228959826094e-06*G0_16 - 0.000148044310370708*G0_17 + 0.000253431785549855*G0_18 - 2.76014815945392e-05*G0_19 - 8.78228959826114e-06*G0_20 - 0.000148044310370708*G0_21 + 0.000253431785549855*G0_22 - 2.7601481594539e-05*G0_23 + 0.00013549818237319*G0_24 - 0.000122952054375672*G0_25 - 2.25830303955311e-05*G0_26 + 2.88560943942901e-05*G0_27 + 0.000514391247898216*G0_28 - 0.000940959599813813*G0_29 - 0.00180664243164253*G0_30 - 0.000940959599813811*G0_31 + 0.00526937375895736*G0_32 - 0.00180664243164253*G0_33 + 0.000577121887885805*G0_34 - 0.000639852527873391*G0_35 - 0.000903321215821264*G0_36 - 0.000639852527873393*G0_37 + 0.00263468687947868*G0_38 - 0.000903321215821263*G0_39 - 0.000213284175957798*G0_40 + 0.000414022223918078*G0_41 + 6.27306399875877e-05*G0_42 + 0.000225830303955317*G0_43 - 0.000639852527873394*G0_44 - 0.00125461279975175*G0_45 - 0.000213284175957797*G0_46 + 0.000414022223918079*G0_47 + 6.27306399875877e-05*G0_48 + 0.000225830303955317*G0_49 - 0.000639852527873395*G0_50 - 0.00125461279975175*G0_51 - 0.00063985252787339*G0_52 - 0.00120442828776168*G0_53 + 0.000677490911865942*G0_54 + 0.000677490911865944*G0_55;
    A[33] = 0.000173437673437683*G0_0 + 9.59527869250141e-05*G0_1 + 9.59527869250139e-05*G0_2 + 0.00014965021475439*G0_3 - 0.000341254681532477*G0_4 + 0.000491808217502687*G0_5 - 0.000110405926378152*G0_6 + 0.000110405926378154*G0_7 - 0.000341254681532476*G0_8 + 0.000491808217502687*G0_9 - 0.000110405926378153*G0_10 + 0.000110405926378153*G0_11 - 4.01476095920559e-05*G0_12 + 0.000127970505574679*G0_13 + 0.000127970505574678*G0_14 - 4.01476095920553e-05*G0_15 - 0.000555375266023443*G0_16 + 0.000873210508627221*G0_17 - 0.000623960765743206*G0_18 + 5.52029631890773e-05*G0_19 - 0.00026681432208054*G0_20 + 0.000170627340766239*G0_21 - 0.000109569517844985*G0_22 - 2.00738047960291e-05*G0_23 - 0.00026681432208054*G0_24 + 0.00017062734076624*G0_25 - 0.000109569517844988*G0_26 - 2.00738047960277e-05*G0_27 + 0.00103714658112812*G0_28 + 0.00051439124789822*G0_29 + 0.00103714658112812*G0_30 - 0.00180664243164253*G0_31 - 0.00180664243164253*G0_32 + 0.00455006242043303*G0_33 + 0.000694219082529303*G0_34 + 6.27306399875879e-05*G0_35 + 0.000518573290564059*G0_36 - 0.00125461279975175*G0_37 - 0.000903321215821263*G0_38 + 0.00227503121021651*G0_39 + 0.000694219082529303*G0_40 + 6.27306399875869e-05*G0_41 + 0.000518573290564059*G0_42 - 0.00125461279975175*G0_43 - 0.000903321215821262*G0_44 + 0.00227503121021651*G0_45 + 0.000618942314544195*G0_46 - 0.000213284175957799*G0_47 + 0.000694219082529305*G0_48 - 0.000213284175957798*G0_49 + 0.000577121887885807*G0_50 + 0.000694219082529304*G0_51 - 0.00035129158393049*G0_52 + 0.000777859935846087*G0_53 + 0.000777859935846091*G0_54 + 0.00140516633572196*G0_55;
    A[34] = 0.000149650214754389*G0_0 + 0.000173437673437683*G0_1 + 9.59527869250141e-05*G0_2 + 9.59527869250144e-05*G0_3 - 4.01476095920556e-05*G0_4 + 0.000127970505574678*G0_5 + 0.00012797050557468*G0_6 - 4.01476095920563e-05*G0_7 - 0.00026681432208054*G0_8 + 0.00017062734076624*G0_9 - 0.000109569517844987*G0_10 - 2.00738047960283e-05*G0_11 - 0.00026681432208054*G0_12 + 0.000170627340766239*G0_13 - 0.000109569517844989*G0_14 - 2.00738047960283e-05*G0_15 + 0.000110405926378154*G0_16 - 0.000110405926378154*G0_17 + 0.000491808217502688*G0_18 - 0.000341254681532477*G0_19 + 0.000110405926378155*G0_20 - 0.000110405926378155*G0_21 + 0.000491808217502691*G0_22 - 0.000341254681532478*G0_23 + 5.52029631890771e-05*G0_24 - 0.000623960765743204*G0_25 + 0.000873210508627221*G0_26 - 0.000555375266023443*G0_27 + 0.0006189423145442*G0_28 - 0.000213284175957798*G0_29 + 0.000694219082529305*G0_30 - 0.000213284175957798*G0_31 + 0.000577121887885805*G0_32 + 0.000694219082529303*G0_33 + 0.00455006242043302*G0_34 - 0.00180664243164252*G0_35 + 0.00103714658112812*G0_36 - 0.00180664243164252*G0_37 + 0.000514391247898216*G0_38 + 0.00103714658112811*G0_39 + 0.00227503121021651*G0_40 - 0.00125461279975175*G0_41 + 0.000694219082529305*G0_42 - 0.000903321215821262*G0_43 + 6.2730639987587e-05*G0_44 + 0.000518573290564059*G0_45 + 0.00227503121021651*G0_46 - 0.00125461279975176*G0_47 + 0.000694219082529305*G0_48 - 0.000903321215821263*G0_49 + 6.27306399875867e-05*G0_50 + 0.00051857329056406*G0_51 + 0.00140516633572197*G0_52 - 0.000351291583930489*G0_53 + 0.000777859935846089*G0_54 + 0.000777859935846079*G0_55;
    A[35] = -0.00012149670352796*G0_0 - 1.77151327324949e-05*G0_1 - 0.00012149670352796*G0_2 + 3.98465025201155e-05*G0_3 - 5.52029631890778e-05*G0_4 - 0.000120442828776168*G0_5 - 7.02583167860987e-05*G0_6 - 9.28413471816295e-05*G0_7 + 0.000135498182373189*G0_8 - 0.000122952054375672*G0_9 - 2.25830303955314e-05*G0_10 + 2.88560943942903e-05*G0_11 - 8.78228959826156e-06*G0_12 - 0.000148044310370707*G0_13 + 0.000253431785549855*G0_14 - 2.7601481594539e-05*G0_15 - 5.52029631890773e-05*G0_16 - 0.000120442828776168*G0_17 - 7.02583167860982e-05*G0_18 - 9.28413471816296e-05*G0_19 + 0.000308634748738931*G0_20 - 4.5166060791064e-05*G0_21 - 4.51660607910633e-05*G0_22 + 0.000308634748738931*G0_23 - 2.76014815945383e-05*G0_24 + 0.000253431785549854*G0_25 - 0.000148044310370707*G0_26 - 8.78228959826213e-06*G0_27 - 0.000213284175957798*G0_28 + 0.000225830303955316*G0_29 - 0.00125461279975176*G0_30 + 0.000414022223918076*G0_31 - 0.000639852527873391*G0_32 + 6.27306399875878e-05*G0_33 - 0.00180664243164252*G0_34 + 0.00526937375895736*G0_35 - 0.00180664243164253*G0_36 - 0.000940959599813816*G0_37 - 0.000940959599813811*G0_38 + 0.000514391247898219*G0_39 - 0.00125461279975175*G0_40 + 0.000225830303955316*G0_41 - 0.000213284175957799*G0_42 - 0.000639852527873395*G0_43 + 0.000414022223918079*G0_44 + 6.27306399875872e-05*G0_45 - 0.00090332121582126*G0_46 - 0.000639852527873394*G0_47 + 0.000577121887885807*G0_48 + 0.00263468687947869*G0_49 - 0.000639852527873393*G0_50 - 0.000903321215821265*G0_51 + 0.000677490911865944*G0_52 - 0.000639852527873396*G0_53 + 0.000677490911865947*G0_54 - 0.00120442828776168*G0_55;
    A[36] = 9.59527869250145e-05*G0_0 + 0.000173437673437683*G0_1 + 0.000149650214754389*G0_2 + 9.59527869250144e-05*G0_3 + 0.000110405926378156*G0_4 - 0.000110405926378155*G0_5 + 0.000491808217502689*G0_6 - 0.000341254681532478*G0_7 - 0.000266814322080541*G0_8 + 0.00017062734076624*G0_9 - 0.000109569517844987*G0_10 - 2.00738047960279e-05*G0_11 - 0.000555375266023445*G0_12 + 0.000873210508627222*G0_13 - 0.00062396076574321*G0_14 + 5.52029631890779e-05*G0_15 - 4.01476095920567e-05*G0_16 + 0.000127970505574679*G0_17 + 0.000127970505574679*G0_18 - 4.01476095920561e-05*G0_19 - 0.000341254681532478*G0_20 + 0.000491808217502689*G0_21 - 0.000110405926378154*G0_22 + 0.000110405926378154*G0_23 - 2.00738047960286e-05*G0_24 - 0.000109569517844986*G0_25 + 0.000170627340766238*G0_26 - 0.00026681432208054*G0_27 + 0.000694219082529306*G0_28 - 0.00125461279975176*G0_29 + 0.00227503121021651*G0_30 + 6.27306399875881e-05*G0_31 - 0.000903321215821264*G0_32 + 0.000518573290564059*G0_33 + 0.00103714658112812*G0_34 - 0.00180664243164253*G0_35 + 0.00455006242043303*G0_36 + 0.000514391247898219*G0_37 - 0.00180664243164253*G0_38 + 0.00103714658112812*G0_39 + 0.000694219082529306*G0_40 - 0.000213284175957799*G0_41 + 0.000618942314544199*G0_42 + 0.000577121887885806*G0_43 - 0.000213284175957798*G0_44 + 0.000694219082529304*G0_45 + 0.000518573290564056*G0_46 + 6.2730639987587e-05*G0_47 + 0.000694219082529304*G0_48 - 0.000903321215821265*G0_49 - 0.00125461279975176*G0_50 + 0.00227503121021652*G0_51 + 0.000777859935846089*G0_52 - 0.000351291583930492*G0_53 + 0.00140516633572197*G0_54 + 0.000777859935846083*G0_55;
    A[37] = -0.00012149670352796*G0_0 - 1.77151327324949e-05*G0_1 + 3.98465025201157e-05*G0_2 - 0.00012149670352796*G0_3 - 9.28413471816299e-05*G0_4 - 7.02583167860978e-05*G0_5 - 0.000120442828776169*G0_6 - 5.52029631890771e-05*G0_7 - 8.7822895982617e-06*G0_8 - 0.000148044310370707*G0_9 + 0.000253431785549855*G0_10 - 2.76014815945381e-05*G0_11 + 0.000135498182373189*G0_12 - 0.000122952054375672*G0_13 - 2.25830303955305e-05*G0_14 + 2.88560943942902e-05*G0_15 + 0.00030863474873893*G0_16 - 4.51660607910619e-05*G0_17 - 4.51660607910645e-05*G0_18 + 0.000308634748738932*G0_19 - 5.52029631890782e-05*G0_20 - 0.000120442828776167*G0_21 - 7.02583167860998e-05*G0_22 - 9.28413471816292e-05*G0_23 - 2.76014815945395e-05*G0_24 + 0.000253431785549855*G0_25 - 0.000148044310370708*G0_26 - 8.78228959826148e-06*G0_27 - 0.000213284175957797*G0_28 + 0.000414022223918077*G0_29 + 6.27306399875886e-05*G0_30 + 0.000225830303955314*G0_31 - 0.000639852527873393*G0_32 - 0.00125461279975175*G0_33 - 0.00180664243164252*G0_34 - 0.000940959599813816*G0_35 + 0.000514391247898219*G0_36 + 0.00526937375895736*G0_37 - 0.000940959599813813*G0_38 - 0.00180664243164253*G0_39 - 0.000903321215821262*G0_40 - 0.000639852527873393*G0_41 + 0.000577121887885807*G0_42 + 0.00263468687947868*G0_43 - 0.000639852527873394*G0_44 - 0.000903321215821263*G0_45 - 0.00125461279975175*G0_46 + 0.000225830303955317*G0_47 - 0.000213284175957799*G0_48 - 0.000639852527873395*G0_49 + 0.000414022223918078*G0_50 + 6.27306399875874e-05*G0_51 + 0.00067749091186594*G0_52 - 0.000639852527873392*G0_53 - 0.00120442828776168*G0_54 + 0.000677490911865951*G0_55;
    A[38] = 3.98465025201153e-05*G0_0 - 1.77151327324951e-05*G0_1 - 0.00012149670352796*G0_2 - 0.00012149670352796*G0_3 + 0.000308634748738931*G0_4 - 4.51660607910629e-05*G0_5 - 4.51660607910637e-05*G0_6 + 0.000308634748738932*G0_7 - 8.78228959826104e-06*G0_8 - 0.000148044310370708*G0_9 + 0.000253431785549855*G0_10 - 2.76014815945386e-05*G0_11 - 8.78228959826119e-06*G0_12 - 0.000148044310370708*G0_13 + 0.000253431785549855*G0_14 - 2.7601481594539e-05*G0_15 - 9.28413471816287e-05*G0_16 - 7.02583167860993e-05*G0_17 - 0.000120442828776168*G0_18 - 5.52029631890775e-05*G0_19 - 9.28413471816289e-05*G0_20 - 7.02583167860994e-05*G0_21 - 0.000120442828776168*G0_22 - 5.52029631890777e-05*G0_23 + 2.88560943942905e-05*G0_24 - 2.25830303955316e-05*G0_25 - 0.000122952054375672*G0_26 + 0.00013549818237319*G0_27 + 0.000577121887885804*G0_28 - 0.000639852527873391*G0_29 - 0.000903321215821263*G0_30 - 0.000639852527873391*G0_31 + 0.00263468687947868*G0_32 - 0.000903321215821263*G0_33 + 0.000514391247898216*G0_34 - 0.000940959599813811*G0_35 - 0.00180664243164253*G0_36 - 0.000940959599813812*G0_37 + 0.00526937375895736*G0_38 - 0.00180664243164253*G0_39 + 6.2730639987587e-05*G0_40 + 0.000414022223918079*G0_41 - 0.000213284175957799*G0_42 - 0.000639852527873394*G0_43 + 0.000225830303955317*G0_44 - 0.00125461279975175*G0_45 + 6.27306399875875e-05*G0_46 + 0.000414022223918078*G0_47 - 0.000213284175957799*G0_48 - 0.000639852527873395*G0_49 + 0.000225830303955317*G0_50 - 0.00125461279975175*G0_51 - 0.00120442828776168*G0_52 - 0.000639852527873391*G0_53 + 0.000677490911865941*G0_54 + 0.000677490911865943*G0_55;
    A[39] = 9.59527869250141e-05*G0_0 + 0.000173437673437682*G0_1 + 9.59527869250137e-05*G0_2 + 0.00014965021475439*G0_3 - 0.000341254681532476*G0_4 + 0.000491808217502686*G0_5 - 0.000110405926378152*G0_6 + 0.000110405926378153*G0_7 - 0.000555375266023442*G0_8 + 0.00087321050862722*G0_9 - 0.000623960765743205*G0_10 + 5.52029631890763e-05*G0_11 - 0.000266814322080539*G0_12 + 0.000170627340766239*G0_13 - 0.000109569517844989*G0_14 - 2.00738047960273e-05*G0_15 - 0.000341254681532477*G0_16 + 0.000491808217502687*G0_17 - 0.000110405926378154*G0_18 + 0.000110405926378154*G0_19 - 4.01476095920561e-05*G0_20 + 0.000127970505574679*G0_21 + 0.00012797050557468*G0_22 - 4.01476095920566e-05*G0_23 - 2.00738047960283e-05*G0_24 - 0.000109569517844986*G0_25 + 0.000170627340766238*G0_26 - 0.000266814322080539*G0_27 + 0.000694219082529303*G0_28 + 6.27306399875894e-05*G0_29 + 0.000518573290564056*G0_30 - 0.00125461279975175*G0_31 - 0.000903321215821263*G0_32 + 0.00227503121021651*G0_33 + 0.00103714658112811*G0_34 + 0.000514391247898219*G0_35 + 0.00103714658112811*G0_36 - 0.00180664243164253*G0_37 - 0.00180664243164253*G0_38 + 0.00455006242043302*G0_39 + 0.000518573290564057*G0_40 + 6.27306399875879e-05*G0_41 + 0.000694219082529301*G0_42 - 0.000903321215821263*G0_43 - 0.00125461279975175*G0_44 + 0.00227503121021651*G0_45 + 0.000694219082529301*G0_46 - 0.000213284175957798*G0_47 + 0.000618942314544197*G0_48 + 0.000577121887885807*G0_49 - 0.000213284175957797*G0_50 + 0.000694219082529304*G0_51 + 0.000777859935846084*G0_52 - 0.000351291583930489*G0_53 + 0.000777859935846091*G0_54 + 0.00140516633572195*G0_55;
    A[40] = 0.000149650214754389*G0_0 + 9.59527869250144e-05*G0_1 + 0.000173437673437683*G0_2 + 9.59527869250145e-05*G0_3 - 0.00026681432208054*G0_4 + 0.000170627340766238*G0_5 - 0.000109569517844985*G0_6 - 2.00738047960282e-05*G0_7 - 4.01476095920556e-05*G0_8 + 0.00012797050557468*G0_9 + 0.000127970505574679*G0_10 - 4.01476095920564e-05*G0_11 - 2.00738047960281e-05*G0_12 - 0.000109569517844987*G0_13 + 0.000170627340766238*G0_14 - 0.00026681432208054*G0_15 + 0.000110405926378155*G0_16 - 0.000110405926378155*G0_17 + 0.000491808217502688*G0_18 - 0.000341254681532477*G0_19 + 5.52029631890777e-05*G0_20 - 0.000623960765743208*G0_21 + 0.000873210508627225*G0_22 - 0.000555375266023445*G0_23 + 0.000110405926378154*G0_24 - 0.000110405926378153*G0_25 + 0.000491808217502687*G0_26 - 0.000341254681532476*G0_27 + 0.000694219082529308*G0_28 - 0.0002132841759578*G0_29 + 0.000618942314544199*G0_30 + 0.000577121887885806*G0_31 - 0.000213284175957798*G0_32 + 0.000694219082529303*G0_33 + 0.00227503121021651*G0_34 - 0.00125461279975175*G0_35 + 0.000694219082529306*G0_36 - 0.000903321215821262*G0_37 + 6.2730639987587e-05*G0_38 + 0.000518573290564057*G0_39 + 0.00455006242043303*G0_40 - 0.00180664243164253*G0_41 + 0.00103714658112812*G0_42 - 0.00180664243164253*G0_43 + 0.000514391247898217*G0_44 + 0.00103714658112812*G0_45 + 0.00227503121021651*G0_46 - 0.000903321215821265*G0_47 + 0.00051857329056406*G0_48 - 0.00125461279975175*G0_49 + 6.27306399875865e-05*G0_50 + 0.000694219082529305*G0_51 + 0.00140516633572197*G0_52 + 0.000777859935846086*G0_53 - 0.000351291583930489*G0_54 + 0.000777859935846079*G0_55;
    A[41] = -0.00012149670352796*G0_0 - 0.00012149670352796*G0_1 - 1.77151327324949e-05*G0_2 + 3.98465025201156e-05*G0_3 + 0.000135498182373189*G0_4 - 0.000122952054375672*G0_5 - 2.25830303955324e-05*G0_6 + 2.88560943942907e-05*G0_7 - 5.52029631890782e-05*G0_8 - 0.000120442828776169*G0_9 - 7.02583167860985e-05*G0_10 - 9.28413471816297e-05*G0_11 - 2.76014815945389e-05*G0_12 + 0.000253431785549855*G0_13 - 0.000148044310370708*G0_14 - 8.78228959826187e-06*G0_15 - 5.52029631890769e-05*G0_16 - 0.000120442828776168*G0_17 - 7.0258316786098e-05*G0_18 - 9.28413471816297e-05*G0_19 - 2.76014815945386e-05*G0_20 + 0.000253431785549855*G0_21 - 0.000148044310370708*G0_22 - 8.78228959826185e-06*G0_23 + 0.000308634748738932*G0_24 - 4.51660607910655e-05*G0_25 - 4.51660607910606e-05*G0_26 + 0.000308634748738931*G0_27 - 0.00125461279975176*G0_28 + 0.000225830303955318*G0_29 - 0.0002132841759578*G0_30 - 0.000639852527873394*G0_31 + 0.000414022223918078*G0_32 + 6.27306399875869e-05*G0_33 - 0.00125461279975175*G0_34 + 0.000225830303955316*G0_35 - 0.000213284175957799*G0_36 - 0.000639852527873393*G0_37 + 0.000414022223918079*G0_38 + 6.27306399875878e-05*G0_39 - 0.00180664243164253*G0_40 + 0.00526937375895737*G0_41 - 0.00180664243164253*G0_42 - 0.000940959599813815*G0_43 - 0.000940959599813815*G0_44 + 0.000514391247898219*G0_45 - 0.000903321215821265*G0_46 + 0.00263468687947869*G0_47 - 0.000903321215821265*G0_48 - 0.000639852527873395*G0_49 - 0.000639852527873393*G0_50 + 0.000577121887885806*G0_51 + 0.000677490911865948*G0_52 + 0.000677490911865945*G0_53 - 0.000639852527873397*G0_54 - 0.00120442828776168*G0_55;
    A[42] = 9.59527869250144e-05*G0_0 + 0.000149650214754389*G0_1 + 0.000173437673437683*G0_2 + 9.59527869250145e-05*G0_3 - 0.00026681432208054*G0_4 + 0.000170627340766238*G0_5 - 0.000109569517844985*G0_6 - 2.00738047960282e-05*G0_7 + 0.000110405926378157*G0_8 - 0.000110405926378156*G0_9 + 0.000491808217502688*G0_10 - 0.000341254681532478*G0_11 + 5.52029631890788e-05*G0_12 - 0.000623960765743207*G0_13 + 0.000873210508627222*G0_14 - 0.000555375266023443*G0_15 - 4.01476095920567e-05*G0_16 + 0.00012797050557468*G0_17 + 0.000127970505574679*G0_18 - 4.01476095920558e-05*G0_19 - 2.00738047960287e-05*G0_20 - 0.000109569517844986*G0_21 + 0.000170627340766241*G0_22 - 0.000266814322080541*G0_23 - 0.00034125468153248*G0_24 + 0.000491808217502692*G0_25 - 0.000110405926378159*G0_26 + 0.000110405926378157*G0_27 + 0.00227503121021652*G0_28 - 0.00125461279975176*G0_29 + 0.000694219082529305*G0_30 - 0.000903321215821263*G0_31 + 6.27306399875877e-05*G0_32 + 0.000518573290564059*G0_33 + 0.000694219082529305*G0_34 - 0.000213284175957799*G0_35 + 0.000618942314544199*G0_36 + 0.000577121887885807*G0_37 - 0.000213284175957799*G0_38 + 0.000694219082529301*G0_39 + 0.00103714658112812*G0_40 - 0.00180664243164253*G0_41 + 0.00455006242043304*G0_42 + 0.000514391247898218*G0_43 - 0.00180664243164253*G0_44 + 0.00103714658112812*G0_45 + 0.000518573290564057*G0_46 - 0.000903321215821266*G0_47 + 0.00227503121021652*G0_48 + 6.2730639987587e-05*G0_49 - 0.00125461279975175*G0_50 + 0.000694219082529305*G0_51 + 0.000777859935846089*G0_52 + 0.00140516633572197*G0_53 - 0.000351291583930488*G0_54 + 0.000777859935846082*G0_55;
    A[43] = -0.00012149670352796*G0_0 + 3.98465025201156e-05*G0_1 - 1.77151327324949e-05*G0_2 - 0.00012149670352796*G0_3 - 8.78228959826185e-06*G0_4 - 0.000148044310370707*G0_5 + 0.000253431785549854*G0_6 - 2.76014815945388e-05*G0_7 - 9.28413471816298e-05*G0_8 - 7.02583167860981e-05*G0_9 - 0.000120442828776169*G0_10 - 5.5202963189077e-05*G0_11 + 2.88560943942901e-05*G0_12 - 2.25830303955313e-05*G0_13 - 0.000122952054375672*G0_14 + 0.00013549818237319*G0_15 + 0.00030863474873893*G0_16 - 4.5166060791062e-05*G0_17 - 4.51660607910636e-05*G0_18 + 0.000308634748738932*G0_19 - 2.76014815945394e-05*G0_20 + 0.000253431785549855*G0_21 - 0.000148044310370708*G0_22 - 8.78228959826132e-06*G0_23 - 5.52029631890783e-05*G0_24 - 0.000120442828776168*G0_25 - 7.02583167860987e-05*G0_26 - 9.28413471816296e-05*G0_27 + 6.27306399875872e-05*G0_28 + 0.000414022223918078*G0_29 - 0.000213284175957797*G0_30 - 0.000639852527873395*G0_31 + 0.000225830303955317*G0_32 - 0.00125461279975175*G0_33 - 0.000903321215821262*G0_34 - 0.000639852527873395*G0_35 + 0.000577121887885806*G0_36 + 0.00263468687947868*G0_37 - 0.000639852527873394*G0_38 - 0.000903321215821263*G0_39 - 0.00180664243164253*G0_40 - 0.000940959599813815*G0_41 + 0.000514391247898218*G0_42 + 0.00526937375895736*G0_43 - 0.000940959599813813*G0_44 - 0.00180664243164253*G0_45 - 0.00125461279975175*G0_46 - 0.000639852527873394*G0_47 + 6.27306399875867e-05*G0_48 + 0.000225830303955317*G0_49 + 0.000414022223918078*G0_50 - 0.000213284175957798*G0_51 + 0.00067749091186594*G0_52 - 0.00120442828776168*G0_53 - 0.000639852527873393*G0_54 + 0.00067749091186595*G0_55;
    A[44] = 3.98465025201156e-05*G0_0 - 0.00012149670352796*G0_1 - 1.7715132732495e-05*G0_2 - 0.00012149670352796*G0_3 - 8.78228959826165e-06*G0_4 - 0.000148044310370708*G0_5 + 0.000253431785549855*G0_6 - 2.7601481594539e-05*G0_7 + 0.00030863474873893*G0_8 - 4.51660607910616e-05*G0_9 - 4.51660607910639e-05*G0_10 + 0.000308634748738932*G0_11 - 2.76014815945398e-05*G0_12 + 0.000253431785549856*G0_13 - 0.000148044310370707*G0_14 - 8.78228959826157e-06*G0_15 - 9.28413471816296e-05*G0_16 - 7.02583167860982e-05*G0_17 - 0.000120442828776169*G0_18 - 5.52029631890775e-05*G0_19 + 2.885609439429e-05*G0_20 - 2.25830303955309e-05*G0_21 - 0.000122952054375673*G0_22 + 0.000135498182373191*G0_23 - 9.28413471816289e-05*G0_24 - 7.02583167860997e-05*G0_25 - 0.000120442828776167*G0_26 - 5.52029631890777e-05*G0_27 - 0.000903321215821262*G0_28 - 0.000639852527873396*G0_29 + 0.000577121887885807*G0_30 + 0.00263468687947868*G0_31 - 0.000639852527873394*G0_32 - 0.000903321215821262*G0_33 + 6.2730639987587e-05*G0_34 + 0.000414022223918079*G0_35 - 0.000213284175957798*G0_36 - 0.000639852527873394*G0_37 + 0.000225830303955317*G0_38 - 0.00125461279975175*G0_39 + 0.000514391247898217*G0_40 - 0.000940959599813815*G0_41 - 0.00180664243164253*G0_42 - 0.000940959599813813*G0_43 + 0.00526937375895737*G0_44 - 0.00180664243164253*G0_45 + 6.27306399875875e-05*G0_46 - 0.000639852527873394*G0_47 - 0.00125461279975176*G0_48 + 0.000414022223918079*G0_49 + 0.000225830303955316*G0_50 - 0.000213284175957798*G0_51 - 0.00120442828776168*G0_52 + 0.000677490911865944*G0_53 - 0.000639852527873391*G0_54 + 0.000677490911865946*G0_55;
    A[45] = 9.59527869250142e-05*G0_0 + 9.59527869250143e-05*G0_1 + 0.000173437673437682*G0_2 + 0.00014965021475439*G0_3 - 0.000555375266023443*G0_4 + 0.000873210508627221*G0_5 - 0.000623960765743204*G0_6 + 5.52029631890769e-05*G0_7 - 0.000341254681532477*G0_8 + 0.000491808217502687*G0_9 - 0.000110405926378154*G0_10 + 0.000110405926378154*G0_11 - 2.00738047960277e-05*G0_12 - 0.000109569517844987*G0_13 + 0.000170627340766238*G0_14 - 0.000266814322080539*G0_15 - 0.000341254681532477*G0_16 + 0.000491808217502687*G0_17 - 0.000110405926378154*G0_18 + 0.000110405926378155*G0_19 - 2.00738047960279e-05*G0_20 - 0.000109569517844987*G0_21 + 0.000170627340766241*G0_22 - 0.000266814322080541*G0_23 - 4.01476095920561e-05*G0_24 + 0.00012797050557468*G0_25 + 0.000127970505574678*G0_26 - 4.01476095920558e-05*G0_27 + 0.000518573290564059*G0_28 + 6.27306399875875e-05*G0_29 + 0.000694219082529303*G0_30 - 0.000903321215821263*G0_31 - 0.00125461279975175*G0_32 + 0.00227503121021651*G0_33 + 0.000518573290564059*G0_34 + 6.27306399875872e-05*G0_35 + 0.000694219082529304*G0_36 - 0.000903321215821263*G0_37 - 0.00125461279975175*G0_38 + 0.00227503121021651*G0_39 + 0.00103714658112812*G0_40 + 0.000514391247898219*G0_41 + 0.00103714658112812*G0_42 - 0.00180664243164253*G0_43 - 0.00180664243164253*G0_44 + 0.00455006242043303*G0_45 + 0.0006942190825293*G0_46 + 0.000577121887885807*G0_47 + 0.000694219082529305*G0_48 - 0.000213284175957799*G0_49 - 0.000213284175957798*G0_50 + 0.000618942314544198*G0_51 + 0.00077785993584609*G0_52 + 0.000777859935846087*G0_53 - 0.000351291583930488*G0_54 + 0.00140516633572196*G0_55;
    A[46] = 0.000149650214754389*G0_0 + 9.59527869250139e-05*G0_1 + 9.59527869250137e-05*G0_2 + 0.000173437673437682*G0_3 - 2.00738047960272e-05*G0_4 - 0.000109569517844987*G0_5 + 0.000170627340766239*G0_6 - 0.00026681432208054*G0_7 - 2.00738047960274e-05*G0_8 - 0.000109569517844987*G0_9 + 0.000170627340766238*G0_10 - 0.00026681432208054*G0_11 - 4.01476095920555e-05*G0_12 + 0.000127970505574678*G0_13 + 0.000127970505574677*G0_14 - 4.01476095920558e-05*G0_15 + 5.52029631890777e-05*G0_16 - 0.000623960765743207*G0_17 + 0.000873210508627221*G0_18 - 0.000555375266023443*G0_19 + 0.000110405926378155*G0_20 - 0.000110405926378156*G0_21 + 0.00049180821750269*G0_22 - 0.000341254681532478*G0_23 + 0.000110405926378154*G0_24 - 0.000110405926378154*G0_25 + 0.000491808217502686*G0_26 - 0.000341254681532476*G0_27 + 0.000694219082529304*G0_28 + 0.000577121887885807*G0_29 + 0.000694219082529302*G0_30 - 0.000213284175957798*G0_31 - 0.000213284175957797*G0_32 + 0.000618942314544195*G0_33 + 0.00227503121021651*G0_34 - 0.00090332121582126*G0_35 + 0.000518573290564056*G0_36 - 0.00125461279975175*G0_37 + 6.27306399875875e-05*G0_38 + 0.000694219082529301*G0_39 + 0.00227503121021651*G0_40 - 0.000903321215821265*G0_41 + 0.000518573290564057*G0_42 - 0.00125461279975175*G0_43 + 6.27306399875875e-05*G0_44 + 0.0006942190825293*G0_45 + 0.00455006242043302*G0_46 - 0.00180664243164253*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164252*G0_49 + 0.000514391247898217*G0_50 + 0.00103714658112811*G0_51 + 0.00140516633572197*G0_52 + 0.000777859935846084*G0_53 + 0.000777859935846086*G0_54 - 0.000351291583930504*G0_55;
    A[47] = -0.00012149670352796*G0_0 - 0.00012149670352796*G0_1 + 3.98465025201157e-05*G0_2 - 1.7715132732495e-05*G0_3 + 2.88560943942901e-05*G0_4 - 2.25830303955312e-05*G0_5 - 0.000122952054375673*G0_6 + 0.00013549818237319*G0_7 - 2.76014815945393e-05*G0_8 + 0.000253431785549855*G0_9 - 0.000148044310370708*G0_10 - 8.78228959826149e-06*G0_11 - 5.52029631890787e-05*G0_12 - 0.000120442828776167*G0_13 - 7.02583167860988e-05*G0_14 - 9.28413471816295e-05*G0_15 - 2.76014815945387e-05*G0_16 + 0.000253431785549855*G0_17 - 0.000148044310370707*G0_18 - 8.78228959826212e-06*G0_19 - 5.52029631890772e-05*G0_20 - 0.000120442828776167*G0_21 - 7.02583167860991e-05*G0_22 - 9.28413471816293e-05*G0_23 + 0.000308634748738932*G0_24 - 4.51660607910659e-05*G0_25 - 4.51660607910605e-05*G0_26 + 0.00030863474873893*G0_27 - 0.00125461279975176*G0_28 - 0.000639852527873394*G0_29 + 6.27306399875867e-05*G0_30 + 0.000225830303955318*G0_31 + 0.000414022223918079*G0_32 - 0.000213284175957799*G0_33 - 0.00125461279975176*G0_34 - 0.000639852527873394*G0_35 + 6.27306399875869e-05*G0_36 + 0.000225830303955317*G0_37 + 0.000414022223918079*G0_38 - 0.000213284175957798*G0_39 - 0.000903321215821265*G0_40 + 0.00263468687947869*G0_41 - 0.000903321215821266*G0_42 - 0.000639852527873394*G0_43 - 0.000639852527873394*G0_44 + 0.000577121887885807*G0_45 - 0.00180664243164253*G0_46 + 0.00526937375895737*G0_47 - 0.00180664243164253*G0_48 - 0.000940959599813815*G0_49 - 0.000940959599813815*G0_50 + 0.000514391247898219*G0_51 + 0.000677490911865948*G0_52 + 0.000677490911865945*G0_53 - 0.00120442828776168*G0_54 - 0.00063985252787339*G0_55;
    A[48] = 9.59527869250144e-05*G0_0 + 0.000149650214754389*G0_1 + 9.59527869250141e-05*G0_2 + 0.000173437673437683*G0_3 - 2.00738047960277e-05*G0_4 - 0.000109569517844987*G0_5 + 0.00017062734076624*G0_6 - 0.00026681432208054*G0_7 + 5.52029631890786e-05*G0_8 - 0.000623960765743208*G0_9 + 0.000873210508627222*G0_10 - 0.000555375266023444*G0_11 + 0.000110405926378157*G0_12 - 0.000110405926378156*G0_13 + 0.000491808217502688*G0_14 - 0.000341254681532477*G0_15 - 2.00738047960287e-05*G0_16 - 0.000109569517844986*G0_17 + 0.000170627340766238*G0_18 - 0.00026681432208054*G0_19 - 4.0147609592057e-05*G0_20 + 0.000127970505574679*G0_21 + 0.00012797050557468*G0_22 - 4.01476095920571e-05*G0_23 - 0.000341254681532479*G0_24 + 0.000491808217502692*G0_25 - 0.000110405926378159*G0_26 + 0.000110405926378156*G0_27 + 0.00227503121021652*G0_28 - 0.000903321215821263*G0_29 + 0.000518573290564059*G0_30 - 0.00125461279975176*G0_31 + 6.27306399875877e-05*G0_32 + 0.000694219082529305*G0_33 + 0.000694219082529305*G0_34 + 0.000577121887885807*G0_35 + 0.000694219082529304*G0_36 - 0.000213284175957799*G0_37 - 0.000213284175957799*G0_38 + 0.000618942314544198*G0_39 + 0.00051857329056406*G0_40 - 0.000903321215821265*G0_41 + 0.00227503121021652*G0_42 + 6.27306399875867e-05*G0_43 - 0.00125461279975176*G0_44 + 0.000694219082529305*G0_45 + 0.00103714658112812*G0_46 - 0.00180664243164253*G0_47 + 0.00455006242043303*G0_48 + 0.000514391247898218*G0_49 - 0.00180664243164253*G0_50 + 0.00103714658112812*G0_51 + 0.000777859935846087*G0_52 + 0.00140516633572197*G0_53 + 0.000777859935846091*G0_54 - 0.000351291583930498*G0_55;
    A[49] = -0.00012149670352796*G0_0 + 3.98465025201156e-05*G0_1 - 0.00012149670352796*G0_2 - 1.77151327324949e-05*G0_3 - 2.76014815945387e-05*G0_4 + 0.000253431785549854*G0_5 - 0.000148044310370707*G0_6 - 8.7822895982623e-06*G0_7 + 2.88560943942904e-05*G0_8 - 2.25830303955314e-05*G0_9 - 0.000122952054375672*G0_10 + 0.00013549818237319*G0_11 - 9.28413471816291e-05*G0_12 - 7.02583167860988e-05*G0_13 - 0.000120442828776168*G0_14 - 5.52029631890773e-05*G0_15 - 2.76014815945385e-05*G0_16 + 0.000253431785549854*G0_17 - 0.000148044310370707*G0_18 - 8.78228959826205e-06*G0_19 + 0.000308634748738931*G0_20 - 4.51660607910632e-05*G0_21 - 4.51660607910648e-05*G0_22 + 0.000308634748738932*G0_23 - 5.52029631890774e-05*G0_24 - 0.000120442828776168*G0_25 - 7.02583167860988e-05*G0_26 - 9.28413471816294e-05*G0_27 + 6.27306399875872e-05*G0_28 - 0.000639852527873394*G0_29 - 0.00125461279975176*G0_30 + 0.000414022223918077*G0_31 + 0.000225830303955317*G0_32 - 0.000213284175957798*G0_33 - 0.000903321215821264*G0_34 + 0.00263468687947869*G0_35 - 0.000903321215821265*G0_36 - 0.000639852527873395*G0_37 - 0.000639852527873394*G0_38 + 0.000577121887885807*G0_39 - 0.00125461279975175*G0_40 - 0.000639852527873395*G0_41 + 6.27306399875869e-05*G0_42 + 0.000225830303955317*G0_43 + 0.000414022223918079*G0_44 - 0.000213284175957799*G0_45 - 0.00180664243164252*G0_46 - 0.000940959599813815*G0_47 + 0.000514391247898218*G0_48 + 0.00526937375895737*G0_49 - 0.000940959599813812*G0_50 - 0.00180664243164253*G0_51 + 0.000677490911865944*G0_52 - 0.00120442828776169*G0_53 + 0.000677490911865949*G0_54 - 0.000639852527873393*G0_55;
    A[50] = 3.98465025201153e-05*G0_0 - 0.00012149670352796*G0_1 - 0.00012149670352796*G0_2 - 1.77151327324949e-05*G0_3 - 2.7601481594539e-05*G0_4 + 0.000253431785549855*G0_5 - 0.000148044310370708*G0_6 - 8.78228959826201e-06*G0_7 - 2.76014815945387e-05*G0_8 + 0.000253431785549854*G0_9 - 0.000148044310370707*G0_10 - 8.78228959826208e-06*G0_11 + 0.000308634748738931*G0_12 - 4.51660607910629e-05*G0_13 - 4.51660607910618e-05*G0_14 + 0.000308634748738932*G0_15 + 2.88560943942907e-05*G0_16 - 2.25830303955316e-05*G0_17 - 0.000122952054375672*G0_18 + 0.000135498182373189*G0_19 - 9.28413471816286e-05*G0_20 - 7.02583167860992e-05*G0_21 - 0.000120442828776168*G0_22 - 5.52029631890778e-05*G0_23 - 9.28413471816285e-05*G0_24 - 7.02583167861003e-05*G0_25 - 0.000120442828776166*G0_26 - 5.52029631890779e-05*G0_27 - 0.000903321215821263*G0_28 + 0.00263468687947868*G0_29 - 0.000903321215821262*G0_30 - 0.000639852527873395*G0_31 - 0.000639852527873395*G0_32 + 0.000577121887885807*G0_33 + 6.27306399875867e-05*G0_34 - 0.000639852527873393*G0_35 - 0.00125461279975176*G0_36 + 0.000414022223918078*G0_37 + 0.000225830303955317*G0_38 - 0.000213284175957797*G0_39 + 6.27306399875864e-05*G0_40 - 0.000639852527873393*G0_41 - 0.00125461279975175*G0_42 + 0.000414022223918078*G0_43 + 0.000225830303955316*G0_44 - 0.000213284175957798*G0_45 + 0.000514391247898217*G0_46 - 0.000940959599813815*G0_47 - 0.00180664243164253*G0_48 - 0.000940959599813812*G0_49 + 0.00526937375895736*G0_50 - 0.00180664243164253*G0_51 - 0.00120442828776168*G0_52 + 0.000677490911865948*G0_53 + 0.000677490911865943*G0_54 - 0.000639852527873392*G0_55;
    A[51] = 9.59527869250143e-05*G0_0 + 9.59527869250143e-05*G0_1 + 0.000149650214754389*G0_2 + 0.000173437673437683*G0_3 + 5.52029631890779e-05*G0_4 - 0.000623960765743206*G0_5 + 0.000873210508627222*G0_6 - 0.000555375266023443*G0_7 - 2.00738047960283e-05*G0_8 - 0.000109569517844986*G0_9 + 0.000170627340766238*G0_10 - 0.00026681432208054*G0_11 - 0.000341254681532477*G0_12 + 0.000491808217502688*G0_13 - 0.000110405926378157*G0_14 + 0.000110405926378155*G0_15 - 2.00738047960287e-05*G0_16 - 0.000109569517844986*G0_17 + 0.000170627340766238*G0_18 - 0.00026681432208054*G0_19 - 0.000341254681532478*G0_20 + 0.000491808217502688*G0_21 - 0.000110405926378154*G0_22 + 0.000110405926378154*G0_23 - 4.0147609592057e-05*G0_24 + 0.00012797050557468*G0_25 + 0.000127970505574678*G0_26 - 4.01476095920563e-05*G0_27 + 0.000518573290564061*G0_28 - 0.000903321215821265*G0_29 + 0.00227503121021652*G0_30 + 6.27306399875877e-05*G0_31 - 0.00125461279975175*G0_32 + 0.000694219082529304*G0_33 + 0.00051857329056406*G0_34 - 0.000903321215821265*G0_35 + 0.00227503121021652*G0_36 + 6.27306399875874e-05*G0_37 - 0.00125461279975175*G0_38 + 0.000694219082529304*G0_39 + 0.000694219082529305*G0_40 + 0.000577121887885806*G0_41 + 0.000694219082529305*G0_42 - 0.000213284175957798*G0_43 - 0.000213284175957798*G0_44 + 0.000618942314544198*G0_45 + 0.00103714658112811*G0_46 + 0.000514391247898219*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164253*G0_49 - 0.00180664243164253*G0_50 + 0.00455006242043303*G0_51 + 0.00077785993584609*G0_52 + 0.000777859935846086*G0_53 + 0.00140516633572197*G0_54 - 0.000351291583930495*G0_55;
    A[52] = -0.00028002957690459*G0_0 + 0.000204250963799586*G0_1 + 0.000204250963799586*G0_2 + 0.000204250963799586*G0_3 - 0.00030110707194042*G0_4 + 6.27306399875871e-05*G0_5 + 6.27306399875893e-05*G0_6 - 0.000301107071940421*G0_7 - 0.000301107071940422*G0_8 + 6.27306399875891e-05*G0_9 + 6.27306399875865e-05*G0_10 - 0.000301107071940421*G0_11 - 0.000301107071940421*G0_12 + 6.27306399875881e-05*G0_13 + 6.27306399875837e-05*G0_14 - 0.000301107071940422*G0_15 + 0.000727675423856015*G0_16 - 0.000903321215821262*G0_17 + 0.00110405926378154*G0_18 - 0.000777859935846087*G0_19 + 0.000727675423856016*G0_20 - 0.000903321215821264*G0_21 + 0.00110405926378155*G0_22 - 0.00077785993584609*G0_23 + 0.000727675423856016*G0_24 - 0.000903321215821263*G0_25 + 0.00110405926378154*G0_26 - 0.000777859935846088*G0_27 - 0.000351291583930489*G0_28 - 0.000639852527873394*G0_29 - 0.000351291583930492*G0_30 - 0.0006398525278734*G0_31 - 0.00063985252787339*G0_32 - 0.00035129158393049*G0_33 + 0.00140516633572197*G0_34 + 0.000677490911865944*G0_35 + 0.000777859935846089*G0_36 + 0.00067749091186594*G0_37 - 0.00120442828776168*G0_38 + 0.000777859935846084*G0_39 + 0.00140516633572197*G0_40 + 0.000677490911865948*G0_41 + 0.000777859935846089*G0_42 + 0.00067749091186594*G0_43 - 0.00120442828776169*G0_44 + 0.00077785993584609*G0_45 + 0.00140516633572197*G0_46 + 0.000677490911865947*G0_47 + 0.000777859935846088*G0_48 + 0.000677490911865944*G0_49 - 0.00120442828776168*G0_50 + 0.00077785993584609*G0_51 + 0.0165608889567232*G0_52 - 0.00225830303955316*G0_53 - 0.00225830303955315*G0_54 - 0.00225830303955316*G0_55;
    A[53] = 0.000204250963799586*G0_0 - 0.000280029576904591*G0_1 + 0.000204250963799586*G0_2 + 0.000204250963799586*G0_3 - 0.00030110707194042*G0_4 + 6.27306399875869e-05*G0_5 + 6.27306399875889e-05*G0_6 - 0.000301107071940421*G0_7 + 0.000727675423856019*G0_8 - 0.000903321215821264*G0_9 + 0.00110405926378154*G0_10 - 0.000777859935846087*G0_11 + 0.000727675423856019*G0_12 - 0.000903321215821265*G0_13 + 0.00110405926378155*G0_14 - 0.000777859935846086*G0_15 - 0.000301107071940422*G0_16 + 6.27306399875888e-05*G0_17 + 6.27306399875871e-05*G0_18 - 0.000301107071940421*G0_19 - 0.000301107071940422*G0_20 + 6.27306399875894e-05*G0_21 + 6.27306399875883e-05*G0_22 - 0.000301107071940423*G0_23 - 0.000777859935846089*G0_24 + 0.00110405926378155*G0_25 - 0.000903321215821266*G0_26 + 0.000727675423856019*G0_27 + 0.00140516633572197*G0_28 + 0.000677490911865947*G0_29 + 0.000777859935846089*G0_30 + 0.000677490911865948*G0_31 - 0.00120442828776168*G0_32 + 0.000777859935846087*G0_33 - 0.000351291583930489*G0_34 - 0.000639852527873396*G0_35 - 0.000351291583930492*G0_36 - 0.000639852527873392*G0_37 - 0.000639852527873391*G0_38 - 0.000351291583930489*G0_39 + 0.000777859935846087*G0_40 + 0.000677490911865945*G0_41 + 0.00140516633572197*G0_42 - 0.00120442828776168*G0_43 + 0.000677490911865944*G0_44 + 0.000777859935846087*G0_45 + 0.000777859935846085*G0_46 + 0.000677490911865945*G0_47 + 0.00140516633572197*G0_48 - 0.00120442828776169*G0_49 + 0.000677490911865948*G0_50 + 0.000777859935846086*G0_51 - 0.00225830303955316*G0_52 + 0.0165608889567231*G0_53 - 0.00225830303955316*G0_54 - 0.00225830303955316*G0_55;
    A[54] = 0.000204250963799586*G0_0 + 0.000204250963799586*G0_1 - 0.000280029576904592*G0_2 + 0.000204250963799586*G0_3 + 0.000727675423856017*G0_4 - 0.000903321215821262*G0_5 + 0.00110405926378155*G0_6 - 0.000777859935846088*G0_7 - 0.000301107071940421*G0_8 + 6.27306399875879e-05*G0_9 + 6.27306399875873e-05*G0_10 - 0.000301107071940421*G0_11 - 0.000777859935846089*G0_12 + 0.00110405926378155*G0_13 - 0.000903321215821267*G0_14 + 0.000727675423856019*G0_15 - 0.00030110707194042*G0_16 + 6.27306399875877e-05*G0_17 + 6.27306399875881e-05*G0_18 - 0.000301107071940422*G0_19 - 0.000777859935846087*G0_20 + 0.00110405926378154*G0_21 - 0.000903321215821261*G0_22 + 0.000727675423856015*G0_23 - 0.000301107071940421*G0_24 + 6.27306399875874e-05*G0_25 + 6.27306399875885e-05*G0_26 - 0.000301107071940423*G0_27 + 0.000777859935846091*G0_28 + 0.000677490911865943*G0_29 + 0.00140516633572197*G0_30 - 0.00120442828776168*G0_31 + 0.000677490911865942*G0_32 + 0.000777859935846091*G0_33 + 0.000777859935846089*G0_34 + 0.000677490911865947*G0_35 + 0.00140516633572197*G0_36 - 0.00120442828776168*G0_37 + 0.000677490911865941*G0_38 + 0.000777859935846091*G0_39 - 0.000351291583930489*G0_40 - 0.000639852527873397*G0_41 - 0.000351291583930488*G0_42 - 0.000639852527873393*G0_43 - 0.000639852527873391*G0_44 - 0.000351291583930488*G0_45 + 0.000777859935846086*G0_46 - 0.00120442828776168*G0_47 + 0.000777859935846091*G0_48 + 0.000677490911865949*G0_49 + 0.000677490911865943*G0_50 + 0.00140516633572197*G0_51 - 0.00225830303955315*G0_52 - 0.00225830303955316*G0_53 + 0.0165608889567232*G0_54 - 0.00225830303955317*G0_55;
    A[55] = 0.000204250963799584*G0_0 + 0.000204250963799585*G0_1 + 0.000204250963799585*G0_2 - 0.000280029576904591*G0_3 - 0.000777859935846085*G0_4 + 0.00110405926378154*G0_5 - 0.000903321215821259*G0_6 + 0.000727675423856017*G0_7 - 0.000777859935846087*G0_8 + 0.00110405926378154*G0_9 - 0.000903321215821262*G0_10 + 0.000727675423856017*G0_11 - 0.00030110707194042*G0_12 + 6.2730639987587e-05*G0_13 + 6.2730639987584e-05*G0_14 - 0.000301107071940419*G0_15 - 0.000777859935846084*G0_16 + 0.00110405926378154*G0_17 - 0.000903321215821262*G0_18 + 0.000727675423856018*G0_19 - 0.000301107071940419*G0_20 + 6.27306399875869e-05*G0_21 + 6.27306399875902e-05*G0_22 - 0.000301107071940422*G0_23 - 0.000301107071940419*G0_24 + 6.27306399875867e-05*G0_25 + 6.27306399875875e-05*G0_26 - 0.00030110707194042*G0_27 + 0.00077785993584608*G0_28 - 0.00120442828776168*G0_29 + 0.00077785993584608*G0_30 + 0.000677490911865952*G0_31 + 0.000677490911865944*G0_32 + 0.00140516633572196*G0_33 + 0.000777859935846079*G0_34 - 0.00120442828776168*G0_35 + 0.000777859935846082*G0_36 + 0.000677490911865951*G0_37 + 0.000677490911865943*G0_38 + 0.00140516633572195*G0_39 + 0.000777859935846079*G0_40 - 0.00120442828776168*G0_41 + 0.000777859935846082*G0_42 + 0.00067749091186595*G0_43 + 0.000677490911865946*G0_44 + 0.00140516633572196*G0_45 - 0.000351291583930504*G0_46 - 0.00063985252787339*G0_47 - 0.000351291583930498*G0_48 - 0.000639852527873393*G0_49 - 0.000639852527873392*G0_50 - 0.000351291583930495*G0_51 - 0.00225830303955316*G0_52 - 0.00225830303955316*G0_53 - 0.00225830303955317*G0_54 + 0.0165608889567231*G0_55;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class UFC_Poisson3D_5LinearForm: public ufc::form
{
public:

  /// Constructor
  UFC_Poisson3D_5LinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~UFC_Poisson3D_5LinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w0_a0 | vi0*va0*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 1;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5LinearForm_finite_element_0();
      break;
    case 1:
      return new UFC_Poisson3D_5LinearForm_finite_element_1();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new UFC_Poisson3D_5LinearForm_dof_map_0();
      break;
    case 1:
      return new UFC_Poisson3D_5LinearForm_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new UFC_Poisson3D_5LinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

// DOLFIN wrappers

#include <dolfin/Form.h>

class Poisson3D_5BilinearForm : public dolfin::Form
{
public:

  Poisson3D_5BilinearForm() : dolfin::Form()
  {
    // Do nothing
  }

  /// Return UFC form
  virtual const ufc::form& form() const
  {
    return __form;
  }
  
  /// Return array of coefficients
  virtual const dolfin::Array<dolfin::Function*>& coefficients() const
  {
    return __coefficients;
  }

private:

  // UFC form
  UFC_Poisson3D_5BilinearForm __form;

  /// Array of coefficients
  dolfin::Array<dolfin::Function*> __coefficients;

};

class Poisson3D_5LinearForm : public dolfin::Form
{
public:

  Poisson3D_5LinearForm(dolfin::Function& w0) : dolfin::Form()
  {
    __coefficients.push_back(&w0);
  }

  /// Return UFC form
  virtual const ufc::form& form() const
  {
    return __form;
  }
  
  /// Return array of coefficients
  virtual const dolfin::Array<dolfin::Function*>& coefficients() const
  {
    return __coefficients;
  }

private:

  // UFC form
  UFC_Poisson3D_5LinearForm __form;

  /// Array of coefficients
  dolfin::Array<dolfin::Function*> __coefficients;

};

#endif
